/* DEVELOPMENT HISTORY */
/* Revision 1.116  1999/12/01 16:38:41  pkolbe
 * added empty granule gran input query fix
 *
 * Revision 1.115  1999/09/21 21:21:23  pkolbe
 * added include for TKerrHandle.h so that the user does not have
 * to remember to include this file all the time
 *
 * Revision 1.114  1999/09/13 15:21:53  pkolbe
 * finished backward compatability code changes
 *
 * Revision 1.113  1999/09/08 18:50:50  pkolbe
 * changed metadata element ssmiAdjust to tmiRolloverCoef
 *
 * Revision 1.112  1999/08/31 20:50:01  pkolbe
 * removed tk backward compatability/version checking code changes
 * (for now, anyway)
 *
 * Revision 1.111  1999/08/27 19:19:26  pkolbe
 * added tk backward compatability/version checking code changes
 *
 * Revision 1.110  1998/09/09 21:15:18  slywczak
 * Changed the length of TK_ALGID_LEN from 4 to 7.  4 appeared to be always to
 * small, since the smallest algorithm id is 4 characters (e.g., 2A21) by
 * default.  The change in length really accounted for the realtime products
 * which have an algorithm id of the form 2A21R1.
 *
 * Revision 1.109  1998/07/08 21:23:43  pkolbe
 * created TK_YES, TK_NO which are very similar to TK_TRUE and TK_FALSE;
 * changed granuleHandle->subsetGranule to granuleHandle->realTimeFlag
 *
 * Revision 1.108  1998/07/07 21:28:03  pkolbe
 * added  determineSubsetGranFlg function for real time metadata
 *
 * Revision 1.107  1998/06/26 14:29:03  slywczak
 * Added new real time data types for algorithms 1B11, 2A23, 2A25r1, 2a25r2,
 * and 2A12.  Added new scaling macros for geolocation.  Added a subset flag
 * to the io handle and the FORTRAN equivalent (wrapper handle).
 *
 * Revision 1.106  1998/05/21 17:47:23  slywczak
 * Added a new macro (TK_DO_NOT_READ_DATA) for backwards compatibility.
 *
 * Revision 1.105  1998/04/27 19:10:41  pkolbe
 * added ssmi adjust coefficient metadata element code
 *
 * Revision 1.104  1998/02/11 19:30:10  slywczak
 * Defined the new metadata element TK_SOLAR_GAINS as number 176.  Added Solar
 * Gains to both the FORTRAN and C PS Metadata Structure.
 *
 * Revision 1.103  1998/01/14 14:20:17  slywczak
 * Added prototype for the new routine TKgetAlgorithmID.
 *
 * Revision 1.102  1997/08/01 12:39:27  slywczak
 * Deleted the definitions for Troubled SSPA Units, Troubled LNA Units, and
 * Telemetry Temperature Abnormal.
 *
 * Revision 1.101  1997/06/26 19:26:51  slywczak
 * Added the sgiPatch variable to the structures so that the FORTRAN interface
 * would work again.
 *
 * Revision 1.100  1997/06/20 14:36:51  slywczak
 * A patch has been provided for the SGI FORTRAN compiler, so the temporarily
 * variables (sgiPatch) needed for the old compiler were removed.
 *
 * Revision 1.99  1997/06/16 19:36:57  slywczak
 * Modified the prototype for gIDparse so that the product id would be passed
 * into the function.
 *
 * Revision 1.98  1997/05/29 03:13:46  slywczak
 * Added defines for the coincidence query routine and the ASCII database.  Added
 * the prototype for the routine TKqueryGVCoincidenceInfo, and included new
 * metadata for the Level 1 PR algorithms.  These metadata elements are the
 * following:  Software Version, Database Version, Total Quality Code,
 * Longitude On The Equator, UTC Date On The Equator, UTC Time On The Equator,
 * UTC MilliSeconds On The Equator, Center Scan UTC Date, Center Scan UTC Time,
 * Center Scan UTC MilliSeconds, First Scan Latitude, First Scan Longitude,
 * Last Scan Latitude, Last Scan Longitude, and Number Of Rain Scans.
 *
 * Revision 1.97  1997/05/19 02:50:58  slywczak
 * Deleted the definition of the grid structure.  It will be redefined in
 * the code.
 *
 * Revision 1.96  1997/05/14 19:17:47  slywczak
 * Added the following new metadata elements:  Ellipsoid Name, Temperature
 * Telemetry Abnormal (PR), Number  of Troubled SSPA Units (PR), Number of
 * Troubled LNA Units (PR), the Data Product Version Number, the Toolkit
 * Version Number, the Calibration Coefficient Version Number (PR), and the
 * Min/Max Units.  For each of the elements, the metadata structure was
 * updated and added the #defines.  Also, increased the size of the genDate
 * structure elements (IO_HANDLE).  It was defined too small for the date
 * being inserted.
 *
 * Revision 1.95  1997/04/28 19:05:01  jt
 * Added prototypes for TKreadGauge() and TKreadNetHeader().
 *
 * Revision 1.94  1997/03/10 17:07:04  slywczak
 * Modified all structures that are currently being accessed by FORTRAN.  Since
 * the SGI compiler is buggy and requires that structure must end in a type
 * that is at least 4 bytes, these structure were modified, if needed, to end
 * in a 4-byte value.  If none of the members of the structure had a 4-byte data
 * type, a dummy int32 or integer*4 was added with the variable name of sgiPatch.
 *
 * Revision 1.93  1997/02/27 21:13:57  slywczak
 * Updated the element TK_SZ_ELEMENT.  In a previous version of the toolkit, the
 * metadata were defined to be a constant size (51).  One of the delivered science
 * algorithms used the TK_SZ_ELEMENT as a validation criteria.  Once the sizes
 * of the metadata were changed, this value was also changed and the science
 * algorithm broke.  The element has been readded, but it points to
 * TK_SZ_ELEMENT_51 (i.e., it's size is only 51 characters).
 *
 * Revision 1.92  1997/02/27 20:55:10  slywczak
 * Update the parameter TK_INPUT_FILES.  This parameter was changed between
 * toolkit 3.5.2 and 3.5.3 (4.0).  Originally, it was called TK_INPUT_FILES and
 * then changed to TK_INPUT_ID.  Since this indicates a change for the scientists,
 * it has been changed back to TK_INPUT_FILES.
 *
 * Revision 1.91  1997/02/26 19:48:20  jt
 * Changed function prototypes to correspond with ".c" files.
 *
 * Revision 1.90  1997/02/13 17:55:13  piper
 * Added #define for missing values in I*2 fields, TK_MISS_I2
 *
 * Revision 1.89  1997/02/10 15:50:37  piper
 * Fixed TKseek; removed TK_BOO and TK_BOF and added TK_ABS_ORBIT_OFF
 *
 * Revision 1.88  1997/02/04 20:09:31  slywczak
 * Rearrange the IO_HANDLE and WRAPPER_HANDLE so that the structure members match
 * between FORTRAN and C.  Also, update the structure members so that the data
 * types match between FORTRAN and C (IO_HANDLE, WRAPPER_HANDLE, and ELEMENT)
 *
 * Revision 1.87  1997/02/04 16:47:24  amak
 * Increased the size of TK_FNAME_LEN from 255 to 256.
 *
 * Revision 1.86  1997/01/10 20:17:20  amak
 * Added 'cdate' in GIDTOKENS structure.
 *
 * Revision 1.85  1997/01/07 20:33:13  amak
 * Defined function prototype for gIDparse().
 *
 * Revision 1.84  1997/01/03 22:39:03  amak
 * Added entity datatype to GIDTOKENS structure.
 *
 * Revision 1.83  1996/12/30  21:25:31  amak
 * Defined TK_DEFAULT as default value -9999.9.
 *
 * Revision 1.82  1996/12/24 15:54:34  amak
 * Incorporated walkthrough comments.
 * - Changed '<' and '>' to double quotes around the #include statements.
 * - Took out TK_L1A_11 macro.  Toolkit will not taking care of binary file.
 * - Took out TK_IM_DM_QUERY structure in IO_HANDLE, and placed that structure
 *   in hdf_routines.h.
 * - Fixed typo errors in comment block.
 *
 * Revision 1.81  1996/12/16  19:13:31  pshah
 * Defined a new structure TK_IM_DM_QUERY for IM and DM interface
 *
 * Revision 1.80  1996/12/16  15:36:01  pshah
 * added some entities to the structure GIDTOKENS
 *
 * Revision 1.77  1996/12/13  16:50:47  amak
 * Defined metadata macros for GV - TK_GV_DZCAL, TK_GV_L1C_SCALE, TK_GV_ALPHA,
 * TK_GV_RUNTIME_OPT & TK_ANOMALY_FLAG.
 * Added 4 entities to PS_METADATA structure - dzcal, gvL1Cscale, alpha,
 * runtimeOption and anomalyFlag.
 *
 * Revision 1.76  1996/12/12  14:45:45  amak
 * Took out metadata macros TK_QUAL_CMNT, TK_PR_RAIN_THRESHOLD,
 * TK_VAL_NEW_NAME and TK_NUM_VAL_SENSORS.  The corresponding metadatas
 * are no longer valid.
 * Took out certain metadata fields in CORE_METADATA & PS_METADATA structure.
 * Those are qualityIndCmt, prRainThreshold, valNetName and numValSensors.
 * Renamed inputFiles in PS_METADATA to inputID.
 * Added entity granule_ID in GIDTOKENS structure.  This entity is acted as
 * granuleID to query the TSDIS database through IM utility.
 *
 * Revision 1.75  1996/12/03  19:39:12  pshah
 * made modifications as per TRMM office in gIDparse routine
 *
 * Revision 1.74  1996/11/07  14:10:59  dbazell
 * added new metadata element macros:
 * - TK_KEP_EPOCH_MILLISEC
 * - TK_LON_OF_MAX_LAT
 * Also made corresponding changes to the coreMetadata structure
 *
 * Revision 1.73  1996/11/05  15:15:14  dbazell
 * change dimensions of fields in gidtokens structure
 *
 * Revision 1.72  1996/11/01  20:51:00  amak
 * Added booleanFlag to the ELEMENT structure.
 * Defined macro TK_NOUPDATE to check the status of metadata handle.
 *
 * Revision 1.71  1996/10/18  18:24:30  amak
 * Defined macro TK_APPEND for L1GV file append capability.
 *
 * Revision 1.70  1996/10/17  19:46:09  amak
 * Defined function prototype for TKcopyScanHeader() and TKwriteDBRainInfo().
 * Defined GV site macros.
 *
 * Revision 1.69  1996/09/19  16:41:32  pshah
 * Changed the value of TK_BOO to -1
 *
 * Revision 1.68  1996/09/09  13:11:48  pshah
 * TK_BOO ( Beginning of orbit for TMI overlap)
 * TK_BOO ( End of orbit for TMI overlap)
 * TK_OSS ( Overlap Scan Size for TMI overlap)
 *
 * Revision 1.67  1996/09/09  13:07:43  pshah
 * Defined TK_EOO, TK_BOO, TK_OSS, & TK_L1A_11
 *
 * Revision 1.66  1996/09/03  19:44:15  amak
 * Defined prototypes for TKwriteGague() and TKwriteNetHeader().
 *
 * Revision 1.65  1996/08/26  19:18:43  amak
 * Added variable 'granuleFlag' to IO_HANDLE structure in C, and WRAPPER_HANDLE
 * in Fortran.
 *
 * Revision 1.64  1996/07/09  17:35:25  dbazell
 * Removed C version of WRAPPER_HANDLE since it is not needed with the
 * new wrapper.c.  Redefined cfgSTR in FORTRAN WRAPPER_HANDLE to be of
 * type FPOINTER.  Added a #ifdef PSIZE_64 to #define FPOINTER integer*8,
 * otherwise its #define FPOINTER integer*4.
 *
 * Revision 1.63  1996/06/18  14:23:21  dbazell
 * Delelted metadataID from structures, it is obsolete.
 *
 * Revision 1.62  1996/06/17  18:30:09  dbazell
 * Added GIDTOKENS structure to fortran section
 *
 * Revision 1.61  1996/06/13  20:39:01  amak
 * Took out TK_L2A_52 macro.  L2A52 is not in hdf format.
 *
 * Revision 1.60  1996/06/13  15:57:29  dbazell
 * Added TK_L2A_52 macro back in.
 *
 * Revision 1.59  1996/06/13  14:12:52  dbazell
 * Added some additional Metadata macros:
 * TK_SENSOR_ALGN, TK_SENSOR_ALGN_CHAN_OFFSET, TK_SCAN_PATH_MODEL,
 * TK_SCAN_PATH_PARAM
 *
 * Revision 1.58  1996/06/07  14:53:04  amak
 * Defined more product ID macros for multiple radar sites in Texas and
 * Florida for product 2A-53, 2A-54, 2A-55, 3A-53, 3A-54, and 3A-55.
 * Took out TK_L2A_55 product ID macro.  2A-55 will not handle by toolkit.
 *
 * Revision 1.57  1996/05/29  19:23:56  amak
 * Deleted the Fortran data structure on TMI_EL and VIRS_EL blocks.
 *
 * Revision 1.56  1996/05/28  19:04:50  dbazell
 * Changed enum, used to define metadata elements, to a set of #defines.
 * This works for both Fortran and C, enum doesn't.  Deleted GRANULE_ID
 * structure.
 *
 * Revision 1.55  1996/05/17  12:34:58  dbazell
 * deleted some metadata fields and added epochDate
 *
 * Revision 1.54  1996/05/14 13:06:06  dbazell
 * Moved meanMotion, orbitAdjFlag, attitudeMode, beginSolarBeta,
 * endSolarBeta, scnrAlgnCal, and ephemFilename to coreMetadata
 *
 * Revision 1.53  1996/05/13 17:44:09  dbazell
 * Updated metadata structures.  Changed #defines for metadata elements
 * to enum.
 *
 * Revision 1.52  1996/05/10 14:06:20  amak
 * Added TK_HDF_FAIL macro to check the failure return value from HDF.
 * Changed TK_FAIL's return value from -1 to 1.
 *
 * Revision 1.51  1996/04/10  13:29:43  dbazell
 * Added prototypes for TKreadTopo and TKreadlsm.
 *
 * Revision 1.50  1996/03/22 19:33:31  amak
 * Added macro TK_BOF to indicate the beginning of file.
 * Deleted TK_REL_BYTE_OFF and TK_ABS_BYTE_OFF definitions.
 *
 * Revision 1.49  1996/03/21  20:08:08  amak
 * Added TK_L3A_46 as product ID macro.
 *
 * Revision 1.48  1996/03/15  22:25:16  amak
 * Changed value of TK_FAIL back to -1
 *
 * Revision 1.47  1996/03/13  17:42:03  dbazell
 * Changed value of TK_FAIL to 1 (was -1) to agree with the definition in
 * TS_GLOBAL.h
 *
 * Revision 1.46  1996/03/04 19:40:40  amak
 * Added more macros for Product Specific Metadata.  Splitted METADATA data
 * structure into COR_METADATA and PS_METADATA.  Made corrsponding changes
 * on METADATA_STR structure.
 *
 * Revision 1.45  1996/02/29  14:19:20  dbazell
 * Changed TK_ONEWAYBEAMWIDTH to TK_ONE_WAY_BEAM_WIDTH
 *
 * Revision 1.44  1996/01/25 15:23:48  amak
 * Used floor() instead of ffloor() in marco RND.  Because ffloor() function
 * is not ANSI standard.
 *
 * Revision 1.43  1996/01/25  14:56:28  amak
 * Defined macro RND(x).  This is the macro function for rounding purpose.
 * Added math.h as preprocessor statement to support this macro routine.
 *
 * Revision 1.42  1996/01/19  19:57:13  dbazell
 * Added macros for file name length and algorithm ID metadata element
 * length.
 *
 * Revision 1.41  1995/12/13 19:02:42  thyun
 * No changes.
 *
 * Revision 1.40  1995/12/12  17:02:25  thyun
 * Inserted TK in front of the metadata operation definitions.
 *
 * Revision 1.39  1995/12/12  15:23:05  amak
 * Took out TKcopyHeader() function prototype.
 * Added TKgetNvos(), TKgetNsensor(), TKgetNparm(), TKgetNcell(), TKgetNray(),
 * and TKgetNsweep() function prototype to header file.
 *
 * Revision 1.38  1995/12/04  19:53:36  amak
 * Took out all the Coincidence related prototype, macros, and data structure.
 * Because Coincidence issue is no longer part of the Toolkit requirement.
 *
 * Revision 1.37  1995/12/01  17:25:01  amak
 * Defined macro TK_NUMBER_GV to determine the number of GV sites.
 *
 * Revision 1.36  1995/12/01  15:03:09  thyun
 * Modified metadata structures.
 *
 * Revision 1.35  1995/11/28  15:55:08  thyun
 * Inserted data structure of METADATA_STR, TIME_STR, and DATE_STR.  Modified the
 * type of the parameter, value in TKreadMetadataInt and TKwriteMetadataInt to
 * void type.
 *
 * Revision 1.34  1995/11/22  20:01:18  thyun
 * Inserted currVgroup into IO_HANDLE.
 *
 * Revision 1.33  1995/11/22  18:57:18  amak
 * Added METADATA structure and prototype of TKgetCoincidence().
 *
 * Revision 1.32  1995/11/20  21:03:17  amak
 * Changed the length of string of variable dataFileName in GRANULE_ID from
 * 50 to 256.
 *
 * Revision 1.31  1995/11/15  22:35:35  thyun
 * Defined TK_L1B_GV, TK_L1C_GV, TKreadL1GV, and TKwriteL1GV..
 *
 * Revision 1.30  1995/10/10  19:35:28  thyun
 * Defined TK_EOF.
 *
 * Revision 1.29  1995/10/10  14:21:37  thyun
 * Modified GV part of product defintions to single and multiple defintions.
 * For example, changed #define TK_L2A_53 9 to #define TK_L2A_53S 9 and
 * #define TK_L2A_53M 10.
 *
 * Revision 1.28  1995/08/29  15:01:34  amak
 * Modified the preprocessor statements.
 *
 * Revision 1.27  1995/08/25  18:13:44  thyun
 * Deleted TK_PR_SCAN, TK_TMI_SCAN, and TK_VIRS_SCAN.
 *
 * Revision 1.26  1995/08/21  18:24:25  thyun
 * Deleted the definition TK_READ_WRITE.
 *
 * Revision 1.25  1995/08/17  21:48:04  amak
 * Changed the array size of variable origin in GRIDSTRUCT from 51 to 101.
 *
 * Revision 1.24  1995/08/16  19:34:08  amak
 * Changed TKopen() prototype, and added commits for macros.
 *
 * Revision 1.23  1995/08/14  13:26:39  gsain
 * made changes
 *
 * Revision 1.22  1995/08/10  19:00:38  thyun
 * Modified the data structure IO_HANDLE.
 *
 * Revision 1.21  1995/08/07  15:43:22  amak
 * Added a common NAVIGATION block structure to file.
 *
 * Revision 1.20  1995/08/04  19:59:11  amak
 * Added new data structure based on the new file spec (8/3/95)
 *
 * Revision 1.19  1995/08/03  17:45:59  amak
 * Re-modified the code because CVS couldn't do version control nicely.
 *
 * Revision 1.18  1995/08/01  20:37:00  thyun
 * Changed CFG_NODE to CFG_STR to reflect the changes in template.h.
 *
 * Revision 1.16  1995/07/26  14:17:19  thyun
 * Defined GV Level 2 and Level 3 product identifications.
 *
 * Revision 1.14  1995/07/20  18:28:12  amak
 * Added prototypes TKreadGrid() and TKwriteGrid() for Level 3 API.
 *
 * Revision 1.13  1995/07/18  19:53:37  thyun
 * Deleted TK_WRITE_ONLY, inserted TK_READ_WRITE and TK_NEW_FILE, changed the
 * parameters of TKreadScan and TKwriteScan.
 *
 * Revision 1.12  1995/07/10  15:20:50  amak
 * Added more detailed comments
 *
 * Revision 1.11  1995/07/07  18:43:52  thyun
 * Defined TK_FAIL and change the value of TK_SUCCESS to 0
 *
 * Revision 1.10  1995/07/07  14:06:48  thyun
 * Modified and added comments
 *
 * Revision 1.9  1995/06/30  17:45:34  thyun
 * Added new definitions
 *
 * Revision 1.4  1995/06/13  14:31:08  amak
 * Adding TK_SUCCESS as global constant
 *
 * Revision 1.3  1995/06/08  20:10:14  amak
 * Added log file header.
 * */

/********************************************************************************
*										
*  FILE NAME:           IO.h						
*									
*  DESCRIPTION:         This file contains the header information of read and
*                       write data to and from HDF files.		
*								
*********************************************************************************/

#ifndef __IO_H
#ifndef LANGUAGE_FORTRAN
#define __IO_H 1
#include <math.h>
#include <hdf.h>
#include <time.h>
#include "template.h"
#include "toolkit.h"
#include "TKerrHandle.h"
#endif

#ifdef PSIZE_64
#define FPOINTER  integer*8
#else
#define FPOINTER  integer*4
#endif

/* Defining overlap scan size */
#define TK_OSS		50

/* Define Product ID macros */
#define TK_L1B_01       0
#define TK_L1B_11       1
#define TK_L1B_21       2
#define TK_L1C_21       3
#define TK_L2A_12       4
#define TK_L2A_21       5
#define TK_L2A_23       6
#define TK_L2A_25       7
#define	TK_L2A_56	8
#define TK_L2A_53S      9
#define TK_L2A_53TX     10
#define TK_L2A_53FL	11
#define TK_L2A_54S      12
#define TK_L2A_54TX     13
#define TK_L2A_54FL     14
#define TK_L2A_55S      15
#define TK_L2A_55TX     16
#define TK_L2A_55FL     17
#define TK_L2B_31	18
#define TK_L3A_11       19
#define TK_L3A_25       20
#define TK_L3A_26       21
#define TK_L3A_53S      22
#define TK_L3A_53TX     23
#define TK_L3A_53FL     24
#define TK_L3A_54S      25
#define TK_L3A_54TX     26
#define TK_L3A_54FL     27
#define TK_L3A_55       28
#define TK_L3B_31       29
#define TK_L3B_42       30
#define TK_L3B_43       31
#define TK_L1B_GV	32
#define TK_L1C_GV	33
#define TK_L3A_46	34
#define TK_L2A_57       35
#define TK_L1B_11_R1    36
#define TK_L2A_12_R1    37
#define TK_L2A_23_R1    38
#define TK_L2A_25_R1    39
#define TK_L2A_25_R2    40
#define TK_L2N_25       41
#define TK_L3A_27       42 
#define TK_L3A_12       43 

/*
 * Definition of the number of pixels in the scan of
 * each instrument.  These values directly support the
 * TKcopyScanHeader routine.  They are temporary and will
 * be replaced when the toolkit is integrated with 
 * TS_GLOBALS.h
 */

#define TK_TMI_PIXELS  208
#define TK_VIRS_PIXELS 261
#define TK_PR_PIXELS    49

/*
 * Define the condition for backward compatibility.
 */
#define TK_DO_NOT_READ_DATA -2
#define TK_DO_NOT_WRITE_DATA -3

/*
 * Define User ID for the TSDIS database
 */
#define TK_IM_USER_ID	"toolkit"

/* Defined Names of Core Metadata Parameters. Start numbering at 1. */

#define TK_ORBIT_NUMBER           1

#define TK_BEGIN_DATE             3
#define TK_BEGIN_TIME             4
#define TK_END_DATE               5
#define TK_END_TIME               6
#define TK_GRANULE_ID             7
#define TK_DATA_ID                8
#define TK_FILE_SIZE              9
#define TK_SPAT_COV_TYPE          10
#define TK_EQ_RADIUS              11
#define TK_FLATTENING_RATIO       12
#define TK_ORBIT_MODEL_NAME       13
#define TK_KEP_SEMI_MAJOR_AXIS    14
#define TK_KEP_MEAN_ANOMALY       15
#define TK_KEP_RIGHT_ASCEN_NODE   16
#define TK_KEP_ARG_OF_PERIGEE     17
#define TK_KEP_ECCENTRICITY       18
#define TK_KEP_INCLINATION        19
#define TK_KEP_EPOCH_TIME         20
#define TK_KEP_EPOCH_DATE         21
#define TK_WEST_BOUND_COORD       22
#define TK_EAST_BOUND_COORD       23
#define TK_NORTH_BOUND_COORD      24
#define TK_SOUTH_BOUND_COORD      25
#define TK_CENTER_POINT_LAT       26
#define TK_CENTER_POINT_LON       27
#define TK_RADIUS                 28
#define TK_LATITUDE_RES           29
#define TK_LONGITUDE_RES          30
#define TK_GEO_COORD_UNITS        31
#define TK_TEMPOR_RNG_TYPE        32
#define TK_QA_PARAM_NAME          33
#define TK_QA_PARAM_VALUE         34
#define TK_REPRO_STAT             35
#define TK_BROWSE_NAME            36
#define TK_CONTACT                37
#define TK_NUM_ORBITS             38
#define TK_ORBIT_ADJUST           39
#define TK_ATTITUDE_MODE          40
#define TK_BEGIN_SOLAR_BETA       41
#define TK_END_SOLAR_BETA         42
#define TK_SENSOR_ALGN            43
#define TK_SENSOR_ALGN_CHAN_OFFSET 44
#define TK_SCAN_PATH_MODEL        45
#define TK_SCAN_PATH_PARAM        46
#define TK_EPHEM_FILENAME         47
#define TK_KEP_EPOCH_MILLISEC     48
#define TK_LON_OF_MAX_LAT         49
#define TK_ELLIPSOID_NAME         50

/* Defined Names of Product Specific Metadata Parameters 
 * Start numbering at 100 
 */

#define TK_NUM_DATA_GAP           100
#define TK_ALGORITHM_VERSION      101
#define TK_MISSING_DATA           102
#define TK_PERCENT_BAD_MISS_PIXEL 103
#define TK_MAX_VALID_CHANNEL      104
#define TK_MIN_VALID_CHANNEL      105
#define TK_ORBIT_SIZE             106
#define TK_RADAR_WAVELENGTH       107
#define TK_MIN_REF_THRESHOLD      108
#define TK_ALGORITHM_ID           109
#define TK_DATA_ACCURACY          111
#define TK_DATA_GAP               112
#define TK_INPUT_FILES		  113
#define TK_GEN_DATE_INPUT_FILES   114
#define TK_DATA_CENTER_SRC        115
#define TK_GEN_DATE               116
#define TK_DAY_NIGHT              117
#define TK_FIRST_SCAN_UTC_DATE    118
#define TK_FIRST_SCAN_UTC_TIME    119
#define TK_FIRST_SCAN_UTC_MILLISEC 120
#define TK_FIRST_SCAN_SC_SECS     121
#define TK_FIRST_SCAN_SC_SUBSECS  122
#define TK_LAST_SCAN_UTC_DATE     123
#define TK_LAST_SCAN_UTC_TIME     124
#define TK_LAST_SCAN_UTC_MILLISEC 125
#define TK_LAST_SCAN_SC_SECS      126
#define TK_LAST_SCAN_SC_SUBSECS   127
#define TK_UTCF_SECONDS           128
#define TK_UTCF_SUBSECONDS        129
#define TK_UTCF_FLAG              130
#define TK_LEAP_SECS_FLAG         131
#define TK_RADAR_NAME             133
#define TK_RADAR_CITY             134
#define TK_RADAR_STATE            135
#define TK_RADAR_COUNTRY          136
#define TK_NUM_VOS                137
#define TK_RADAR_ORIGIN_LAT       140
#define TK_RADAR_ORIGIN_LON       141
#define TK_RADAR_ORIGIN_ALT       142
#define TK_RADAR_SPACING_X        143
#define TK_RADAR_SPACING_Y        144
#define TK_RADAR_SPACING_Z        145
#define TK_RADAR_GRID_SIZE_X      146
#define TK_RADAR_GRID_SIZE_Y      147
#define TK_RADAR_GRID_SIZE_Z      148
#define TK_GV_DZCAL		  149
#define TK_GV_L1C_SCALE		  150
#define TK_GV_ALPHA		  151
#define TK_GV_RUNTIME_OPT	  152
#define TK_ANOMALY_FLAG		  153
#define TK_PRODUCT_VERSION        157
#define TK_TOOLKIT_VERSION        158
#define TK_CAL_COEFF_VERSION      159
#define TK_MIN_MAX_UNITS          160
#define TK_SOFTWARE_VERSION       161
#define TK_DATABASE_VERSION       162
#define TK_TOTAL_QUALITY_CODE     163
#define TK_LON_ON_EQUATOR         164
#define TK_UTC_DATE_ON_EQUATOR    165
#define TK_UTC_TIME_ON_EQUATOR    166
#define TK_UTC_MILLISEC_ON_EQUATOR 167
#define TK_CENTER_SCAN_UTC_DATE   168
#define TK_CENTER_SCAN_UTC_TIME   169
#define TK_CENTER_SCAN_UTC_MILLISEC 170
#define TK_FIRST_SCAN_LAT         171
#define TK_FIRST_SCAN_LON         172
#define TK_LAST_SCAN_LAT          173
#define TK_LAST_SCAN_LON          174
#define TK_NUM_OF_RAIN_SCANS      175
#define TK_SOLAR_GAINS            176
#define TK_TMI_ROLLOVER_COEF      177

/* Define instrument names */
#define TK_TMI     1
#define TK_VIRS    2
#define TK_PR      3
#define TK_GV      4
#define TK_VARIOUS 5

/*  Define level names  */
#define TK_L1      6
#define TK_L2      7
#define TK_L3      8

/* Define Modes for TKopen */
#define TK_READ_ONLY	'r'
#define TK_NEW_FILE	'n'
#define TK_APPEND	'a'

/* Define general return status */
#define TK_SUCCESS	0
#define TK_FAIL		1


/* Define error return status for HDF routine */
#define TK_HDF_FAIL	-1

/* TK_EOF indicates the end of file. */
#define TK_EOF	        -2

/* Defining End of the Orbit */
#define TK_EOO		-3


/* Define "TYPE" for TKseek */
#define TK_ABS_SCAN_OFF  1 /*Relative to beginning of file */ 
#define TK_ABS_ORBIT_OFF 2 /*Relative to beginning of orbit */
#define TK_REL_SCAN_OFF	 3 /*Relative to current position */

/* Define boolean flags for Toolkit Utility */

#define TK_TRUE         0
#define TK_FALSE        1

/* Define more boolean flags for Toolkit Utility */
/* These are defined to match C definitions of   */
/* true and false which the above definitions    */
/* dont do.                                      */

#define TK_YES       1
#define TK_NO        0

/*  Used for querying the ASCII files (GV Coincidence) */

#define TK_NO_RECORD    1
#define TK_RECORD_FOUND 0

/* Define macro function for rounding purpose */
#define RND(x)		floor(x + 0.5)
#define GEOLOC_SCALE    100

/* Define maximum file name length, and number of characters needed
 * for specification of Algorithm ID metadata  
 */
#define TK_FNAME_LEN          256
#define TK_ALGID_LEN            7
#define TK_MAX_SZ_ATTRIBUTE 32000
#define TK_SZ_ELEMENT_51       51
#define TK_SZ_ELEMENT_101     101
#define TK_SZ_ELEMENT_301     301	
#define TK_SZ_ELEMENT_1001   1001
#define TK_BUFFER_LENGTH      256	

/*  The following define was added to maintain compatibility with a previous
 *  version of the toolkit.  One of the science algorithms used this define
 *  TK_SZ_ELEMENT.  When the toolkit replaced that define with 
 *  TK_SZ_ELEMENT_51, the science algorithm broke.  Instead of asking the 
 *  algorithm developer to change, this define was readded.  It is IMPORTANT
 *  to note that TK_SZ_ELEMENT is size 51, eventhough some metadata elements
 *  now exceed that size.
 */  
#define TK_SZ_ELEMENT TK_SZ_ELEMENT_51

/**
 **  Define macros for primary GV sites
 **/
#define TK_DARW	0	/* Darwin */
#define TK_MELB	1	/* Melbourne */
#define TK_KWAJ	2	/* Kwajalein */
#define TK_HSTN	3	/* Houston */

#define TK_CORC	4	/* Corpus Christi */
#define TK_FLOR	5	/* Florida */
#define TK_GUAM	6	/* Guam */
#define TK_ISBN 7	/* Bengurion */
#define TK_KEYW 8	/* Keywest */
#define TK_LKCH 9	/* Lake Charles */
#define TK_MIAM	10	/* Miami */
#define TK_NWBR	11	/* New Braunfel */
#define TK_SAOP 12	/* Sao Paulo */
#define TK_TAMP 13	/* Tempa Bay */
#define TK_TEXS 14	/* Texas */
#define TK_THOM 15	/* Omkoi */
#define TK_TWWF 16	/* WFS */

/*
 * Set flag to define no update in metadata header
 */
#define TK_NOUPDATE	-1 

/*
 * Unit to convert no of bytes to MB
 *	2^20 = 1048576
 */
#define TK_BYTE_CONVERT_MB	1048576.0

/**
 ** Defined the default value.
 **/
#define TK_DEFAULT	-9999.9
#define TK_MISS_I2	-9999

/**
 ** Defined the Empty Granule Flag which is used for cross checking
 ** of a Granule Input Query failure in initMetadata
 **/
#define TK_EGG_EMPTY	1863

#ifdef LANGUAGE_FORTRAN
/*********************************/
/* Define structures for FORTRAN */
/*********************************/

        /************************/
        /*  Metadata            */
        /************************/

        STRUCTURE /ELEMENT/
	  INTEGER*2	booleanFlag	/* This flag is used to indicate the status    */
                              		/* of the corresponding metadata.  If metadata */
                              		/* has been updated booleanFlag == TK_TRUE;    */
                              		/* otherwise booleanFlag == TK_FALSE.          */
          FPOINTER	NAME
          FPOINTER	VALUE
        END STRUCTURE

	STRUCTURE /CORE_METADATA/
          RECORD /ELEMENT/	 orbitNumber
          RECORD /ELEMENT/	 beginDate
          RECORD /ELEMENT/	 beginTime
          RECORD /ELEMENT/	 endDate
          RECORD /ELEMENT/       endTime
	  RECORD /ELEMENT/       granuleID
	  RECORD /ELEMENT/       dataID
	  RECORD /ELEMENT/       fileSize
	  RECORD /ELEMENT/       spatCovType
          RECORD /ELEMENT/       ellipsoidName
	  RECORD /ELEMENT/       earthEqRad
	  RECORD /ELEMENT/       flatteningRatio
	  RECORD /ELEMENT/       orbitModelName
	  RECORD /ELEMENT/       kepSemiMajorAxis
	  RECORD /ELEMENT/       kepMeanAnomaly
	  RECORD /ELEMENT/       kepRightAscenNode
	  RECORD /ELEMENT/       kepArgOfPerigee
	  RECORD /ELEMENT/       kepEccentricity
	  RECORD /ELEMENT/       kepInclination
	  RECORD /ELEMENT/       kepEpochTime
	  RECORD /ELEMENT/       kepEpochDate
	  RECORD /ELEMENT/       westBoundCoord
	  RECORD /ELEMENT/       eastBoundCoord
	  RECORD /ELEMENT/       northBoundCoord
	  RECORD /ELEMENT/       southBoundCoord
	  RECORD /ELEMENT/       centerPointLat
          RECORD /ELEMENT/       centerPointLon
	  RECORD /ELEMENT/       radius
	  RECORD /ELEMENT/       latitudeRes
	  RECORD /ELEMENT/       longitudeRes
	  RECORD /ELEMENT/       geoCoordUnits
	  RECORD /ELEMENT/       temporRngType
	  RECORD /ELEMENT/       qaParamName
	  RECORD /ELEMENT/       qaParamValue
	  RECORD /ELEMENT/       reproStat
	  RECORD /ELEMENT/       browseName
	  RECORD /ELEMENT/       contact
          RECORD /ELEMENT/       meanMotion 
          RECORD /ELEMENT/       orbitAdjFlag 
          RECORD /ELEMENT/       attitudeMode 
          RECORD /ELEMENT/       beginSolarBeta 
          RECORD /ELEMENT/       endSolarBeta 
          RECORD /ELEMENT/       sensorAlgn 
          RECORD /ELEMENT/       sensorAlgnChanOff 
          RECORD /ELEMENT/       scanPathModel 
          RECORD /ELEMENT/       scanPathParam 
          RECORD /ELEMENT/       ephemFilename 
          RECORD /ELEMENT/       kepEpochMillisec 
          RECORD /ELEMENT/       lonMaxLat 
	END STRUCTURE

	STRUCTURE /PS_METADATA/ 
          RECORD /ELEMENT/       numDataGap
          RECORD /ELEMENT/       algorithmVersion
          RECORD /ELEMENT/       productVersion
          RECORD /ELEMENT/       toolkitVersion
          RECORD /ELEMENT/       calCoeffVersion
	  RECORD /ELEMENT/	 missingData
          RECORD /ELEMENT/	 percentBadMissPix
          RECORD /ELEMENT/	 maxValidChannel
          RECORD /ELEMENT/	 minValidChannel
          RECORD /ELEMENT/       minMaxUnits
          RECORD /ELEMENT/	 orbitSize
          RECORD /ELEMENT/	 radarWavelength
          RECORD /ELEMENT/	 minRefThreshold
	  RECORD /ELEMENT/	 algorithmID
          RECORD /ELEMENT/       dataAccuracy 
          RECORD /ELEMENT/       dataGap 
          RECORD /ELEMENT/       inputID
	  RECORD /ELEMENT/       dateGenInputFile 
          RECORD /ELEMENT/       dataCenterSrc 
	  RECORD /ELEMENT/	 generationDate
          RECORD /ELEMENT/       dayNight 
          RECORD /ELEMENT/       solarGains
          RECORD /ELEMENT/       tmiRolloverCoef 
          RECORD /ELEMENT/       firstScanUTCdate 
          RECORD /ELEMENT/       firstScanUTCtime 
          RECORD /ELEMENT/       firstScanUTCmsec
          RECORD /ELEMENT/       firstScanSCsec 
          RECORD /ELEMENT/       firstScanSCsubsec 
          RECORD /ELEMENT/       lastScanUTCdate 
          RECORD /ELEMENT/       lastScanUTCtime 
          RECORD /ELEMENT/       lastScanUTCmsec 
          RECORD /ELEMENT/       lastScanSCsec 
          RECORD /ELEMENT/       lastScanSCsubsec 
          RECORD /ELEMENT/       utcfSec 
          RECORD /ELEMENT/       utcfSubsec
	  RECORD /ELEMENT/	 utcfFlag
          RECORD /ELEMENT/       leapSecsFlag 
          RECORD /ELEMENT/       radarName 
          RECORD /ELEMENT/       radarCity 
          RECORD /ELEMENT/       radarState 
          RECORD /ELEMENT/       radarCountry 
          RECORD /ELEMENT/       numVOS 
          RECORD /ELEMENT/       radarOriginLat 
          RECORD /ELEMENT/       radarOriginLon 
          RECORD /ELEMENT/       radarOriginAlt 
          RECORD /ELEMENT/       radarSpacingX 
          RECORD /ELEMENT/       radarSpacingY 
          RECORD /ELEMENT/       radarSpacingZ 
          RECORD /ELEMENT/       radarGridSizeX 
          RECORD /ELEMENT/       radarGridSizeY 
          RECORD /ELEMENT/       radarGridSizeZ 
	  RECORD /ELEMENT/	 dzcal
          RECORD /ELEMENT/       gvL1Cscale
          RECORD /ELEMENT/       alpha
          RECORD /ELEMENT/       runtimeOption
          RECORD /ELEMENT/       anomalyFlag
          RECORD /ELEMENT/       softwareVersion
          RECORD /ELEMENT/       databaseVersion
          RECORD /ELEMENT/       totalQualityCode
          RECORD /ELEMENT/       lonOnEquator
          RECORD /ELEMENT/       UTCDateOnEquator
          RECORD /ELEMENT/       UTCTimeOnEquator
          RECORD /ELEMENT/       millisecOnEquator
          RECORD /ELEMENT/       centerScanUTCDate
          RECORD /ELEMENT/       centerScanUTCTime
          RECORD /ELEMENT/       centerScanUTCMillisec
          RECORD /ELEMENT/       firstScanLat
          RECORD /ELEMENT/       firstScanLon
          RECORD /ELEMENT/       lastScanLat
          RECORD /ELEMENT/       lastScanLon
          RECORD /ELEMENT/       numOfRainScans
	END STRUCTURE


	STRUCTURE /METADATA_STR/
	  RECORD /CORE_METADATA/	coreMetadata
	  RECORD /PS_METADATA/		psMetadata
	END STRUCTURE

        /*********************************************************
        * Token structure holds tokens parsed from the granuleID.*
	* The structure is initialized to zero.                  *
	*********************************************************/

        STRUCTURE /GIDTOKENS/
	  CHARACTER*49		 granule_ID 
	  CHARACTER*49           granuleName 
	  CHARACTER*24		 datatype
	  CHARACTER*24           algID
	  CHARACTER*9            collectDate
	  INTEGER*2		 cdate
	  CHARACTER*8            orbitNumber
          CHARACTER*7            radar
          CHARACTER*7            Version
          INTEGER*4              sgiPatch 
        END STRUCTURE

	/****************************************************************/
	/* WRAPPER_HANDLE is a structure used to send IO_HANDLE 	*/
	/* information to Fortran interface.				*/
	/****************************************************************/

	STRUCTURE /WRAPPER_HANDLE/
	  FPOINTER		  cfgSTR
	  INTEGER*2		  productID
	  INTEGER*4       	  currNumOfScanl
	  INTEGER*4		  nRecords
          CHARACTER*12            fsize
          CHARACTER*35            genDate
          CHARACTER*10            TKversion 
	  CHARACTER*1		  mode
	  INTEGER*2		  granuleFlag
          CHARACTER*256           dataFileName
          INTEGER*2               realTimeFlag
          INTEGER*4               sgiPatch 
	  RECORD /METADATA_STR/	  metadata
          RECORD /GIDTOKENS/      gIDtokens
	END STRUCTURE

	/****************************************************************/
	/* NAVIGATION structure is the common block that used by the	*/
	/* L1 and L2 satellite products.				*/
	/****************************************************************/

      	STRUCTURE /NAVIGATION/
           REAL*4               scPosX
           REAL*4               scPosY
           REAL*4               scPosZ
           REAL*4               scVelX
           REAL*4               scVelY
           REAL*4               scVelZ
           REAL*4               scLat
           REAL*4               scLon
           REAL*4               scAlt
           REAL*4               scAttRoll
           REAL*4               scAttPitch
           REAL*4               scAttYaw
           REAL*4               att1
           REAL*4               att2 
           REAL*4               att3
           REAL*4               att4
           REAL*4               att5
           REAL*4               att6
           REAL*4               att7
           REAL*4               att8
           REAL*4               att9
           REAL*4               greenHourAng
      	END STRUCTURE


	STRUCTURE /DATE_STR/
	  INTEGER*2		tkyear
	  INTEGER*2		tkmonth
	  INTEGER*2		tkday
          INTEGER*4             sgiPatch  
      	END STRUCTURE


	STRUCTURE /TIME_STR/
	  BYTE			tkhour
	  BYTE			tkminute
	  BYTE			tksecond
          INTEGER*4             sgiPatch  
      	END STRUCTURE

#else
/***************************/
/* Define structures for C */
/***************************/

	/************************/
	/*  Metadata		*/
	/************************/

	  
        typedef struct {   /* Metadata element structure */
	  int16	booleanFlag;  /* This flag is used to indicate the status    */
			      /* of the corresponding metadata.  If metadata */
			      /* has been updated booleanFlag == TK_TRUE;    */
			      /* otherwise booleanFlag == TK_FALSE.	     */
	  char  *name;
	  char  *value;
	} ELEMENT;


	typedef struct
	{
	  ELEMENT	orbitNumber;
	  ELEMENT	beginDate;
	  ELEMENT	beginTime;
	  ELEMENT	endDate;
	  ELEMENT	endTime;
	  ELEMENT       granuleID;
	  ELEMENT       dataID;
	  ELEMENT       fileSize;
	  ELEMENT       spatCovType;
	  ELEMENT       ellipsoidName;
	  ELEMENT       earthEqRad;
	  ELEMENT       flatteningRatio;
	  ELEMENT       orbitModelName;
	  ELEMENT       kepSemiMajorAxis;
	  ELEMENT       kepMeanAnomaly;
	  ELEMENT       kepRightAscenNode;
	  ELEMENT       kepArgOfPerigee;
	  ELEMENT       kepEccentricity;
	  ELEMENT       kepInclination;
	  ELEMENT       kepEpochTime;
	  ELEMENT       kepEpochDate;
	  ELEMENT       westBoundCoord;
	  ELEMENT       eastBoundCoord;
	  ELEMENT       northBoundCoord;
	  ELEMENT       southBoundCoord;
	  ELEMENT       centerPointLat;
	  ELEMENT       centerPointLon;
	  ELEMENT       radius;
	  ELEMENT       latitudeRes;
	  ELEMENT       longitudeRes;
	  ELEMENT       geoCoordUnits;
	  ELEMENT       temporRngType;
	  ELEMENT       qaParamName;
	  ELEMENT       qaParamValue;
	  ELEMENT       reproStat;
	  ELEMENT       browseName;
	  ELEMENT       contact;
	  ELEMENT	meanMotion;
	  ELEMENT	orbitAdjFlag;
	  ELEMENT	attitudeMode;
	  ELEMENT	beginSolarBeta;
	  ELEMENT	endSolarBeta;
	  ELEMENT	sensorAlgn;
	  ELEMENT	sensorAlgnChanOff;
	  ELEMENT	scanPathModel;
	  ELEMENT	scanPathParam;
	  ELEMENT	ephemFilename;
	  ELEMENT	kepEpochMillisec;
	  ELEMENT	lonMaxLat;
	} CORE_METADATA;

	typedef struct
	{
	  ELEMENT       numDataGap;
	  ELEMENT       algorithmVersion;
	  ELEMENT       productVersion;
          ELEMENT       toolkitVersion;
	  ELEMENT       calCoeffVersion;
	  ELEMENT	missingData;
	  ELEMENT       percentBadMissPix;
	  ELEMENT	maxValidChannel;
	  ELEMENT	minValidChannel;
	  ELEMENT       minMaxUnits;
	  ELEMENT	orbitSize;
	  ELEMENT	radarWavelength;
	  ELEMENT	minRefThreshold; 
	  ELEMENT	algorithmID;
	  ELEMENT	dataAccuracy;
	  ELEMENT	dataGap;
	  ELEMENT	inputID;
	  ELEMENT	dateGenInputFile;
	  ELEMENT	dataCenterSrc;
	  ELEMENT	generationDate;
	  ELEMENT	dayNight;
	  ELEMENT       solarGains;
	  ELEMENT       tmiRolloverCoef;
	  ELEMENT	firstScanUTCdate;
	  ELEMENT	firstScanUTCtime;
	  ELEMENT	firstScanUTCmsec;
	  ELEMENT	firstScanSCsec;
	  ELEMENT	firstScanSCsubsec;
	  ELEMENT	lastScanUTCdate;
	  ELEMENT	lastScanUTCtime;
	  ELEMENT	lastScanUTCmsec;
	  ELEMENT	lastScanSCsec;
	  ELEMENT	lastScanSCsubsec;
	  ELEMENT	utcfSec;
	  ELEMENT	utcfSubsec;
	  ELEMENT	utcfFlag;
	  ELEMENT	leapSecsFlag;
	  ELEMENT	radarName;
	  ELEMENT	radarCity;
	  ELEMENT	radarState;
	  ELEMENT	radarCountry;
	  ELEMENT	numVOS;
	  ELEMENT	radarOriginLat;
	  ELEMENT	radarOriginLon;
	  ELEMENT	radarOriginAlt;
	  ELEMENT	radarSpacingX;
	  ELEMENT	radarSpacingY;
	  ELEMENT	radarSpacingZ;
	  ELEMENT	radarGridSizeX;
	  ELEMENT	radarGridSizeY;
	  ELEMENT	radarGridSizeZ;
	  ELEMENT	dzcal;
          ELEMENT       gvL1Cscale;
          ELEMENT       alpha;
          ELEMENT       runtimeOption;
          ELEMENT       anomalyFlag;
          ELEMENT       softwareVersion;
          ELEMENT       databaseVersion;
          ELEMENT       totalQualityCode;
          ELEMENT       lonOnEquator;
          ELEMENT       UTCDateOnEquator;
          ELEMENT       UTCTimeOnEquator;
          ELEMENT       millisecOnEquator;
          ELEMENT       centerScanUTCDate;
          ELEMENT       centerScanUTCTime;
          ELEMENT       centerScanUTCMillisec;
          ELEMENT       firstScanLat;
          ELEMENT       firstScanLon;
          ELEMENT       lastScanLat;
          ELEMENT       lastScanLon;
          ELEMENT       numOfRainScans;
	} PS_METADATA;


	typedef struct
	{
	  CORE_METADATA 	coreMetadata;
	  PS_METADATA		psMetadata;
	} WRAPPER_METADATA_STR;


	typedef struct
	{
	  CORE_METADATA		coreMetadata;
	  PS_METADATA		psMetadata;
	} METADATA_STR;


        /*********************************************************
        * Token structure holds tokens parsed from the granuleID.*
	* The structure is initialized to zero.                  *
	*********************************************************/
        typedef struct {
	  char granule_ID[49]; 
	  char granuleName[49]; 
	  char datatype[24];
	  char algID[24];
	  char collectDate[9];
	  int  cdate;
	  char orbitNumber[8];
	  char radar[7];
	  char algVersion[7];
	  int32 sgiPatch;  
	} GIDTOKENS;


	/****************************************************************/
	/* IO_HANDLE is a structure to handle input, output, and HDF 	*/
	/* pointers.  HDF pointers are stored in cfgSTR.		*/
	/****************************************************************/
	typedef struct
	{ 
	  CFG_STR		*cfgSTR;
	  int16			productID;
	  int32           	currNumOfScanl;
	  int32			nRecords;
	  char                  fsize[12];
	  char                  genDate[35];
          char                  TKversion[10]; 
	  char 			mode;
	  int16			granuleFlag;
	  char                  dataFileName[256];
	  int16                 realTimeFlag;
	  int32                 sgiPatch;  
	  METADATA_STR		metadata;
	  GIDTOKENS             gIDtokens;
	} IO_HANDLE; 


        /****************************************************************/
        /* NAVIGATION structure is the common block that used by the    */
        /* L1 and L2 satellite products.                                */
        /****************************************************************/

        typedef struct
        {
          float32               scPosX;
          float32               scPosY;
          float32               scPosZ;
          float32               scVelX;
          float32               scVelY;
          float32               scVelZ;
          float32               scLat;
          float32               scLon;
          float32               scAlt;
          float32               scAttRoll;
          float32               scAttPitch;
          float32               scAttYaw;
          float32               att1;
          float32               att2;
          float32               att3;
          float32               att4;
          float32               att5;
          float32               att6;
          float32               att7;
          float32               att8;
          float32               att9;
          float32               greenHourAng;
        } NAVIGATION;


	typedef struct
	{
	  int16 		tkyear;
	  int16			tkmonth;
	  int16			tkday;
	  int32                 sgiPatch; 
	} DATE_STR;


	typedef struct
	{
	  int8			tkhour;
	  int8			tkminute;
	  int8			tksecond;
	  int32                 sgiPatch;  
	} TIME_STR;



/************************/
/* Function prototypes.	*/
/************************/

	int TKopen(char *granuleID, int dataType, char filemode, 
			IO_HANDLE *sGranuleHandle);
	int TKreadScan(IO_HANDLE *sGranuleHandle, void *sSwathData);
	int TKwriteScan(IO_HANDLE *sGranuleHandle, void *sSwathData);
	int TKreadGrid(IO_HANDLE *sGranuleHandle, void *sGrid);
	int TKwriteGrid(IO_HANDLE *sGranuleHandle, void *sGrid);
	int TKreadL1GV(IO_HANDLE *sGranuleHandle, void *sGV);
	int TKwriteL1GV(IO_HANDLE *sGranuleHandle, void *sGV);
	int TKreadMetadataInt(IO_HANDLE *sGranuleHandle, int parameter, 
			void *value);
	int TKreadMetadataFloat(IO_HANDLE *sGranuleHandle, int parameter, 
			float *value);
	int TKreadMetadataChar(IO_HANDLE *sGranuleHandle, int parameter, 
			char *value);
	int TKwriteMetadataInt(IO_HANDLE *sGranuleHandle, int parameter, 
			void *value);
	int TKwriteMetadataFloat(IO_HANDLE *sGranuleHandle, int parameter, 
			float *value);
	int TKwriteMetadataChar(IO_HANDLE *sGranuleHandle, int parameter, 
			char *value);
	int TKclose(IO_HANDLE *sGranuleHandle);
        short int TKreadTopo(float lat, float lon);
        int TKreadlsm(float lat, float lon);
	int TKseek(IO_HANDLE *sGranuleHandle, int offset, int type);
	int TKendOfFile(IO_HANDLE *sGranuleHandle);
	int32 TKgetNvos(IO_HANDLE *sGranuleHandle);
	int32 TKgetNsensor(IO_HANDLE *sGranuleHandle);
	int32 TKgetNparm(IO_HANDLE *sGranuleHandle, int32 vosNumber); 
	int32 TKgetNcell(IO_HANDLE *sGranuleHandle, int32 vosNumber,  
				int32 parmNumber); 
	int32 TKgetNray(IO_HANDLE *sGranuleHandle, int32 vosNumber); 
	int32 TKgetNsweep(IO_HANDLE *sGranuleHandle, int32 vosNumber); 
	int TKreadHeader(IO_HANDLE *sGranuleHandle, void *sHeader);
	int TKwriteHeader(IO_HANDLE *sGranuleHandle, void *sHeader);
	int TKwriteGauge(IO_HANDLE *sGranuleHandle, void *sGauge_Disdrometer);
        int TKreadGauge(IO_HANDLE *sGranuleHandle, void *sGauge_Disdrometer);
	int TKwriteNetHeader(IO_HANDLE *sGranuleHandle, int nnet, void *sNetDesc);
        int TKreadNetHeader(IO_HANDLE *sGranuleHandle, int nnet, void *sNetDesc);
	int TKcopyScanHeader(IO_HANDLE *sInGranuleHandle, void *sInDataHandle, 
			     IO_HANDLE *sOutGranuleHandle, void *sOutDataHandle); 
	int TKwriteDBRainInfo(int instrID, struct tm *sStart_time,
			      struct tm *sEnd_time, int rain_percent);
	int gIDparse(char *granuleID, int prodId, GIDTOKENS *stoken);

        int TKqueryGVCoincidenceInfo ( int, DATE_STR *, TIME_STR *, DATE_STR *, 
				      TIME_STR *, char *, char *, char *, float * );

        int TKgetAlgorithmID ( char * );
        char* TKgetTKversion ( char * );
        int determineSubsetGranFlg ( int );
        int versionCompare ( char * , char *);

	int TKwriteFileInfo(IO_HANDLE *sGranuleHandle, char *file_name, char *label);
	int TKreadFileInfo(IO_HANDLE *sGranuleHandle, char *label, char *text);


#endif
#endif

