/* DEVELOPMENT HISTORY */
/* Revision 1.45  1999/11/23 20:23:09  slywczak
 * Updated the prototypes for tkreadl1gvparm_, tkreadl1gv1byteparm_, and
 * tkreadl1gvbasesweep.  Added the last parameter which the FORTRAN bindings will
 * use to pass the size of the strings to the wrappers.
 *
 * Revision 1.44  1998/07/17 13:15:27  slywczak
 * Removed the scaling factor for 3A54.
 *
 * Revision 1.43  1998/05/14 18:34:36  slywczak
 * Modified MAX_CELLS to 1500.  Reduced the line length for the FORTRAN code to
 * make sure that all lines are under 72 columns.
 *
 * Revision 1.42  1998/05/07 15:00:34  slywczak
 * Modified the MAX_CELL parameter.  Change this parameter to 1500 (from 1024)
 * at the request of the GV team.
 *
 * Revision 1.41  1997/12/16 15:33:36  slywczak
 * Updated the number of rays, cells, and sweeps for the Level 1 GV.
 *
 * Revision 1.40  1997/12/04 20:38:51  slywczak
 * Increased the number of scans from 17 to 19.
 *
 * Revision 1.39  1997/10/14 21:13:42  okelley
 *         Modifications to the Toolkit to implement new
 *         GV utilities.
 *
 * Revision 1.38  1997/08/12 21:41:36  slywczak
 * Changed the location of the IO.h include to mantain compatibility with
 * FORTRAN and n32.
 *
 * Revision 1.37  1997/08/01 13:01:19  slywczak
 * Changed the maximum number of VOSes from 12 to 20. Modifed the parameter
 * MAX_VOS.
 *
 * Revision 1.36  1997/05/19 02:54:52  slywczak
 * Deleted the grid struct definition for algorithm 3A-46.
 *
 * Revision 1.35  1997/03/07 05:18:47  slywczak
 * Added the line of code to redefine the word template as tsdis_template.
 * The word template is a reserved word in C++ and has a conflict with the TRMM
 * Office development.  They are using C++.
 *
 * Revision 1.34  1997/03/05 20:45:56  slywczak
 * Modified the definition of the PARAMETER structure for GV L1 algorithm.
 * Changed the definition of parmData1byte from *parmData1byte[MAX_SWEEP] to
 * ***parmData1byte, and *parmData2byte[MAX_SWEEP] to ***parmData2byte.
 *
 * Revision 1.33  1996/12/24 15:51:09  amak
 * Incorporated walkthrough comments.
 * - Changed '<' and '>' to double quotes around the #include statements.
 *
 * Revision 1.32  1996/12/03  19:38:30  pshah
 * Made modifications as per TRMM office
 *
 * Revision 1.31  1996/11/01  20:56:48  amak
 * Added definition UPDATE_L1GV_TEMPLATE
 * Modified definition of parmData component of PARAMETER structure.
 * Modified definition of parm component of SENSORS structure.
 * Added filemode parameter to prototype of function openInputVgroupL1GV().
 * Added prototype of updateTemplate().
 *
 * Revision 1.30  1996/10/17  20:16:10  amak
 * Updated 3A46 structure.  Took out entity dailyRainfall and inserted entity
 * ssmiData.
 *
 * Revision 1.29  1996/08/26  19:37:03  amak
 * Defined data structures for L2A56 (Rain Gauge) and L2A57 (Disdrometer) in
 * C and Fortran.
 *
 * Revision 1.28  1996/06/13  13:53:01  amak
 * Took out the RADARGRID_STR block for 2A-53, 2A-54, 2A-55, 3A-53, 3A-54,
 * and 3A-55.
 *
 * Revision 1.27  1996/06/07  14:55:49  amak
 * Redefined the data structure of multiple radar sites for 2A-53, 2A-54,
 * 2A-55, 3A-53, 3A-54, and 3A-55.
 * Took out data structure of 2A-52.  Toolkit will not handle 2A-52 any more.
 *
 * Revision 1.26  1996/04/29  18:16:26  amak
 * Took off LANGUAGE_FORTRAN ifndef preprocessor statement.
 *
 * Revision 1.25  1996/03/21  20:08:47  amak
 * Added structure for L3A46.
 *
 * Revision 1.24  1996/03/07  18:32:01  amak
 * For all single radars of product 2A-53, 2A-54, 2A-55, 3A-53, and 3A-54,
 * the dimensions should be 151x151, instead of 150x150.
 *
 * Revision 1.23  1996/02/08  16:51:22  amak
 * Fixed typo error on function prototype setL1GVtemplate().
 *
 * Revision 1.22  1995/12/13  19:03:29  thyun
 * Moved the declaration of routines which are related to Level 1 GV from
 * IO_INTR_GV.c
 *
 * Revision 1.21  1995/12/12  20:13:44  thyun
 * Changed the type of field convStratFlag to BYTE in L2A_54_SINGLE_RADARGRID
 * and L2A_54_MULT_RADARGRID.
 *
 * Revision 1.20  1995/12/12  15:20:24  amak
 * Defined scaling factors macros.
 * Moved all the macros from L1_GV_ACCESS.h to IO_GV.h
 *
 * Revision 1.19  1995/11/28  20:58:28  thyun
 * Modified the data structures which are related with scaling.
 *
 * Revision 1.18  1995/11/28  15:55:45  thyun
 * Deleted the data structure TIME_STR since it is defined in IO.h.
 *
 * Revision 1.17  1995/11/22  20:02:27  thyun
 * Modified the structure of L1B_1C_GV.
 *
 * Revision 1.16  1995/11/20  17:53:52  amak
 * Added scaling calculation variables to the structures.
 *
 * Revision 1.15  1995/11/15  22:36:35  thyun
 * Defined the data structure, L1B_1C_GV.
 *
 * Revision 1.14  1995/10/10  14:25:41  thyun
 * Modified some of the GV data structures to single and multiple data structures.
 * They are L2A_53, L2A_54, L2A_55, L3A_53, and L3A_54.
 *
 * Revision 1.13  1995/09/12  15:33:20  amak
 * Fixed syntax error for L3A-55 FORTRAN data structure.
 *
 * Revision 1.12  1995/08/25  18:34:16  thyun
 * Inserted comments for data structure defintion.
 *
 * Revision 1.11  1995/08/25  14:15:38  amak
 * Replaced all the INTEGER*1 type to BYTE data type.
 *
 * Revision 1.10  1995/08/21  18:25:11  thyun
 * Changed the names of fields, time, hour, minute, and second to tktmie,
 * tkhour, tkminute,and tksecond.
 *
 * Revision 1.9  1995/08/17  22:01:00  thyun
 * Changed the order of dimention according to file specification.
 *
 * Revision 1.8  1995/08/17  19:19:40  amak
 * Modified the proprocessor statements.
 *
 * Revision 1.7  1995/08/10  19:52:04  thyun
 * Modified the c processor "if" statement.
 *
 * Revision 1.6  1995/08/10  19:25:58  thyun
 * Added developement history.
 *
 * Revision 1.5  1995/08/10  19:15:45  thyun
 * Adding file.
 * */

/*******************************************************************************
*
*
*  FILE NAME:           IO_GV.h
*
*  DESCRIPTION:         This file contains the data structures of the GV RADAR
*                       products 
*
********************************************************************************/

#ifndef __IO_GV_H
#ifndef LANGUAGE_FORTRAN  
#define __IO_GV_H 1
#endif  

/*
 *  Added the following line of code, since the TRMM Office uses C++, 
 *  and template is a reserved word in C++.  This should redefine all
 *  instances of template to be tsdis_template.
 */
#define template tsdis_template


#define MAX_VOS		20		
#define MAX_RAY		400
#define MAX_PARM	4
#define MAX_CELL	1500
#define MAX_SWEEP	25

/*
 * Define maximum number of network 
 * for rain gauge and disdrometer.
 */
#define MAX_NNET	10

/* Define Scaling Factors. */
#define L2A56_MEANRAINRATE	10
#define L2A53_RAINRATE		10
#define L2A55_THREEDREFLECT	100
#define L2A55_VERTICALPROFILE	100
#define L3A53_PENTADRAINFALL	10
#define L3A55_VERTICALPROFILE   100



/*
** The definition of the function TKsetL1GVtemplate.  A user must call this
** routine to access setL1GVtemplate.  Since the user can not know the ID of
** inputfile, -1 is initially set.
*/
#define TKsetL1GVtemplate(A,B,C,D,E,F)  setL1GVtemplate(A,B,C,D,E,F,-1)



/*
** These definitiion is used internally in the program in order to perform
** requested L1 GV action.
*/
#define SET_L1GV_INPUTFILE      0
#define ADD_L1GV_TEMPLATE       1
#define DELETE_L1GV_TEMPLATE    2
#define GET_L1GV_NUM_OF_VOS     3
#define GET_L1GV_NUM_OF_PARM    4
#define GET_L1GV_NCELL          5
#define GET_L1GV_NRAY           6
#define GET_L1GV_NSWEEP         7
#define UPDATE_L1GV_TEMPLATE    8


/*
 * Minimum buffer size which is
 * greater than or equal to the
 * sizeof(l1B_GV.comments[])
 */
#define MINIMAL_BUFFER_SIZE 5002

/*
	===================================
		defines for TKutilGV.c
	===================================
 */
        /* Since IO.h and TS_GLOBAL.h cannot be included in the
        toolkit so that the Toolkit will run without those components. */

#define         TK_ALGORITHM_ID_SIZE      7
#define         TK_MISS_I1              -99

        /* to signal that all parameters are to be read
        from a level 1 GV volume scan */

#define         TK_READ_ALL_PARAM       "READ_ALL_PARAM"

        /* for allocating level 1 parameters */

#define         TK_USE_DEFAULT          -99

        /* to force 2 byte integers into the range of a 1 byte
        integer */

#define ONE_BYTE(a)  ( MIN( MAX( (a) , TK_MISS_I1 ), SCHAR_MAX ) )

        /* to define granule_ID inside the VosSummary.  The TS
        macros cannot be used by the toolkit because the toolkit
        must be able to run without the TS component of TSDIS. */

#define         TK_FNAME                128
#define         TK_PATHLEN              256
#define         MAX_FILE_LEN            ( TK_FNAME + TK_PATHLEN )

#define MAX_PARAM_LENGTH        20

        /* To initialize the number of seconds used to search
        an array of dates */

#define         TWENTY_YEARS    60*60*24*365*20

/*      The toolkit's IO_GV.h uses the value of 5000 for the comment
        length without defining a macro. */

#define  COMMENT_LENGTH         5000

        /* used by readGV_L1B_1C_param */

#define        ray_zero                0
#define        azimuth_index           0
#define        elevation_index         1


#ifdef LANGUAGE_FORTRAN
/*********************************/
/* Define structures for FORTRAN */
/*********************************/

	STRUCTURE	/PARAMETER_DESCRIPTOR/
          CHARACTER*9	parmName
          CHARACTER*41	parmDesc
          CHARACTER*9	parmUnits
          INTEGER*2	interPulsePeriod
          INTEGER*2	transFreq
          REAL*4	receiverBandwidth
          INTEGER*2	pulseWidth
          INTEGER*2	polarTransWave
          INTEGER*2	numOfsamples
          INTEGER*2	parmDataType
          CHARACTER*9	thresholdField
          REAL*4	thresholdValue
          REAL*4	scaleFactor
          REAL*4	offsetFactor
          INTEGER*4	deletedOrMissDataFlag
	END STRUCTURE


	STRUCTURE	/CELL_RANGE_VECTOR/
          INTEGER*4			numOfCells
          REAL*4	distanceToCell(MAX_CELL)
	END STRUCTURE


	STRUCTURE	/PARAMETER/
          RECORD /PARAMETER_DESCRIPTOR/ parmDesc
          RECORD /CELL_RANGE_VECTOR/    cellRangeVector
          BYTE		parmData1byte(MAX_CELL,MAX_RAY,MAX_SWEEP)
          INTEGER*2	parmData2byte(MAX_CELL,MAX_RAY,MAX_SWEEP)
	END STRUCTURE

	STRUCTURE	/RADAR_DESCRIPTOR/
          CHARACTER*9			radarName
          REAL*4			radarConstant
          REAL*4			nomPeakPower
          REAL*4		 	nomNoisePower
          REAL*4			receiverGain
          REAL*4			antennaGain
          REAL*4			radarSystemGain
          REAL*4			horBeamWidth
          REAL*4			verBeamWidth
          INTEGER*2			radarType
          INTEGER*2			scanMode
          REAL*4			nomScanRate
          REAL*4			nomStartAngle
          REAL*4			nomStopAngle
          INTEGER*2			numParmDesc
          INTEGER*2			numDesc
          INTEGER*2			dataComp
          INTEGER*2			udataReductAlg
          REAL*4			dataReductParm1
          REAL*4			dataReductParm2
          REAL*4			radarLon
          REAL*4			radarLat
          REAL*4			radarAlt
          REAL*4			velocity
          REAL*4			range
          INTEGER*2			numTransfreqency
          INTEGER*2			numInterPulsePeriods
          REAL*4			frequency1
          REAL*4			frequency2
          REAL*4			frequency3
          REAL*4			frequency4
          REAL*4			frequency5
          REAL*4			interPulsePeriod1
          REAL*4			interPulsePeriod2
          REAL*4			interPulsePeriod3
          REAL*4			interPulsePeriod4
          REAL*4			interPulsePeriod5 
	END STRUCTURE

	STRUCTURE	/CORRECTION_FACTOR_DESCRIPTOR/
          REAL*4			azimuth
          REAL*4			elevation
          REAL*4			rangeDelay
          REAL*4			radarLon
          REAL*4			radarLat
          REAL*4			radarPressAlt
          REAL*4	       	radarAltAboveGround
          REAL*4	       	radarPlatGroundSpeedEW
          REAL*4	       	radarPlatGroundSpeedNS
          REAL*4	      	radarPlatVerticalVelocity
          REAL*4			radarPlatHeading
          REAL*4			radarPlatRoll
          REAL*4			radarPlatPitch
          REAL*4			radarPlatDrift
          REAL*4			radarRotAngle
          REAL*4			radarTiltAngle
	END STRUCTURE

	STRUCTURE	/SWEEP_INFO/
          CHARACTER*9                   radarName
          INTEGER*4			sweepNum
          INTEGER*4			numRays
          REAL*4			trueStartAngle
          REAL*4			trueStopAngle
          REAL*4			fixedAngle
          INTEGER*4			filterFlag
	END STRUCTURE

	STRUCTURE	/VOLUME_DESCRIPTORS/
          INTEGER*2			verNum
          INTEGER*2			volNum
          INTEGER*4			sizeDataRec
          CHARACTER*21			projectName
          INTEGER*2			year
          INTEGER*2			month
          INTEGER*2			day
          INTEGER*2			hour
          INTEGER*2			minute
          INTEGER*2			second
          CHARACTER*9			flightNum
          CHARACTER*9			facName
          INTEGER*2			genYear
          INTEGER*2			genMonth
          INTEGER*2			genDay
          INTEGER*2			numSensorDesc
	END STRUCTURE

	STRUCTURE	/SENSORS/
          RECORD /RADAR_DESCRIPTOR/             radarDesc
          RECORD /CORRECTION_FACTOR_DESCRIPTOR/ corrFactorDesc
          RECORD /SWEEP_INFO/	sweepInfo(MAX_SWEEP)
          INTEGER*4         rayInfoInteger(7,MAX_RAY,MAX_SWEEP)
          REAL*4	    rayInfoFloat(4, MAX_RAY, MAX_SWEEP)
          REAL*4	    platformInfo(18, MAX_RAY, MAX_SWEEP)
          RECORD /PARAMETER/    parm(MAX_PARM)
	END STRUCTURE


        /****************************************/
        /* Define L1B_1C_GV data structure.     */
        /****************************************/

	STRUCTURE	/L1B_1C_GV/
          CHARACTER*5001		comments
          RECORD /VOLUME_DESCRIPTORS/   volDes
          RECORD /SENSORS/              sensor
	END STRUCTURE


	STRUCTURE	/RADARGRID_STR/
	  CHARACTER*51		referLat
	  CHARACTER*51		referLon
	  CHARACTER*51		referX
	  CHARACTER*51		referY
	  CHARACTER*51		gridSpacingX
	  CHARACTER*51		gridSpacingY
	  CHARACTER*101		orign
	END STRUCTURE

	
	STRUCTURE	/RAIN_PERCENT/
	  INTEGER*2		rainPercent
	END STRUCTURE


        /************************************/
        /* Define L2A-53 single radar site. */
        /************************************/

	STRUCTURE	/L2A_53_SINGLE_RADARGRID/
	  RECORD /TIME_STR/	tktime
	  INTEGER*2		rainRate_scale(151,151)
	  REAL*4		      rainRate(151,151)
	END STRUCTURE


        /*********************************************/
        /*Define L2A-53 multiple radar site in Texas.*/
        /*********************************************/

	STRUCTURE	/L2A_53_MULT_TX_RADARGRID/
	  RECORD /TIME_STR/		tktime
	  INTEGER*2		rainRate_scale(363,285)
	  REAL*4	              rainRate(363,285)
	END STRUCTURE

        /***********************************************/
        /*Define L2A-53 multiple radar site in Florida.*/
        /***********************************************/

        STRUCTURE       /L2A_53_MULT_FL_RADARGRID/
          RECORD /TIME_STR/             tktime
          INTEGER*2             rainRate_scale(257,353)
          REAL*4                      rainRate(257,353)
        END STRUCTURE


        /****************************************/
        /* Define L2A-54 single radar site.	*/
        /****************************************/

	STRUCTURE	/L2A_54_SINGLE_RADARGRID/
	  RECORD /TIME_STR/		tktime
	  BYTE			convStratFlag(151,151)
	END STRUCTURE


        /**********************************************/
        /*Define L2A-54 multiple radar site in Texas. */
        /**********************************************/

	STRUCTURE	/L2A_54_MULT_TX_RADARGRID/
	  RECORD /TIME_STR/		tktime
	  BYTE			convStratFlag(363,285)
	END STRUCTURE


        /************************************************/
        /*Define L2A-54 multiple radar site in Florida. */
        /************************************************/

        STRUCTURE       /L2A_54_MULT_FL_RADARGRID/
          RECORD /TIME_STR/             tktime
          BYTE                  convStratFlag(257,353)
        END STRUCTURE


        /****************************************/
        /* Define L2A-55 single radar site.	*/
        /****************************************/

	STRUCTURE	/L2A_55_SINGLE_RADARGRID/
	  RECORD /TIME_STR/		tktime
	  INTEGER*2		threeDreflect_scale(151,151,13)
	  REAL*4		threeDreflect(151,151,13)
	  INTEGER*2		vertProfile_scale(12,13)
	  REAL*4		vertProfile(12,13)
	  INTEGER*4		cfadData(12,86,13)
	END STRUCTURE


        /**********************************************/
        /*Define L2A-55 multiple radar site in Texas. */
        /**********************************************/

	STRUCTURE	/L2A_55_MULT_TX_RADARGRID/
	  RECORD /TIME_STR/		tktime
	  INTEGER*2		threeDreflect_scale(363,285,13)
	  REAL*4		threeDreflect(363,285,13)
	  INTEGER*2		vertProfile_scale(12,13)
	  REAL*4		vertProfile(12,13)
	  INTEGER*4		cfadData(12,86,13)
	END STRUCTURE

        /************************************************/
        /*Define L2A-55 multiple radar site in Florida. */
        /************************************************/

        STRUCTURE       /L2A_55_MULT_FL_RADARGRID/
          RECORD /TIME_STR/             tktime
          INTEGER*2         threeDreflect_scale(257,353,13)
          REAL*4                  threeDreflect(257,353,13)
          INTEGER*2                vertProfile_scale(12,13)
          REAL*4                         vertProfile(12,13)
          INTEGER*4                      cfadData(12,86,13)
        END STRUCTURE


        /****************************************/
        /* Define L3A-53 single radar site.	*/
        /****************************************/

	STRUCTURE	/L3A_53_SINGLE_RADARGRID/
	  INTEGER*2	     pentadRainfall_scale(151,151)
	  REAL*4	           pentadRainfall(151,151)
	END STRUCTURE


        /**********************************************/
        /*Define L3A-53 multiple radar site in Texas. */
        /**********************************************/

	STRUCTURE	/L3A_53_MULT_TX_RADARGRID/
	  INTEGER*2   pentadRainfall_scale(363,285)
	  REAL*4	    pentadRainfall(363,285)
	END STRUCTURE


        /************************************************/
        /*Define L3A-53 multiple radar site in Florida. */
        /************************************************/

        STRUCTURE       /L3A_53_MULT_FL_RADARGRID/
          INTEGER*2   pentadRainfall_scale(257,353)
          REAL*4            pentadRainfall(257,353)
        END STRUCTURE


        /****************************************/
        /* Define L3A-54 single radar site.	*/
        /****************************************/

	STRUCTURE	/L3A_54_SINGLE_RADARGRID/
	  INTEGER*2	   monthlyRainfall(151,151)
	END STRUCTURE


        /**********************************************/
        /*Define L3A-54 multiple radar site in Texas. */
        /**********************************************/

	STRUCTURE	/L3A_54_MULT_TX_RADARGRID/
	  INTEGER*2	   monthlyRainfall(363,285)
	END STRUCTURE


        /************************************************/
        /*Define L3A-54 multiple radar site in Florida. */
        /************************************************/

        STRUCTURE       /L3A_54_MULT_FL_RADARGRID/
          INTEGER*2       monthlyRainfall(257,353)
        END STRUCTURE


        /****************************************/
        /* Define L3A-55 data structure.        */
        /****************************************/

	STRUCTURE	/L3A_55_RADARGRID/
          INTEGER*2	  vertProfile_scale(12,13)
	  REAL*4		vertProfile(12,13)
          INTEGER*4		cfadData(12,86,13)
	END STRUCTURE


	/****************************************/
	/* Define L3A-46 data structure.	*/
	/****************************************/

	STRUCTURE /L3A_46_PLANETGRID/
	  REAL*4	      ssmiData(360,180,2)
	END STRUCTURE


        /****************************************/
        /* Define L2A-56 data structure.        */
        /****************************************/

	STRUCTURE /NETDESC/
	  CHARACTER*21		networkName
	  INTEGER*2		nValidSensor	
	  BYTE			networkID
	END STRUCTURE

	STRUCTURE /GAUGEDESC/
	  BYTE			networkID
	  CHARACTER*21		gaugeType
	  CHARACTER*21		gaugeName
	  CHARACTER*21		gaugeMakeModel
	  INTEGER*4		gaugeNumber
	  REAL*4		gaugeLat
	  REAL*4		gaugeLong
	  REAL*4		range
	  REAL*4		azimuth
	  REAL*4		xCoordinate
	  REAL*4		yCoordinate
	  REAL*4		resolution
	  REAL*4		elevation
	END STRUCTURE


	STRUCTURE /L2A_56_RAINGAUGE/
	  RECORD /GAUGEDESC/	gaugeDesc
	  BYTE			   hour(1440)
	  BYTE		 	 minute(1440)
	  INTEGER*2  meanRainRate_scale(1440)
	  REAL*4           meanRainRate(1440)
	END STRUCTURE


        /****************************************/
        /* Define L2A-57 data structure.        */
        /****************************************/

	STRUCTURE /DISDRODESC/
	  BYTE			networkID
	  CHARACTER*21		disdroName
	  CHARACTER*21		disdroMakeModel
	  INTEGER*4		disdroNumber
	  REAL*4		disdroLat
	  REAL*4		disdroLong
	  REAL*4		range
	  REAL*4		azimuth
	  REAL*4		resolution
	  REAL*4		elevation
	END STRUCTURE


	STRUCTURE /L2A_57_DISDROMETER/
	  RECORD /DISDRODESC/	disdroDesc
	  BYTE			hour(1440)
	  BYTE			minute(1440)
	  INTEGER*2  nConcentration(20,1440)
	END STRUCTURE

/*
	====================================
	    FORTRAN structure definitions
		for TKutilGV.c
	====================================
*/

/*
----------------------------------------------------
   definitions of FORTRAN 1C51 and 1B51 structures
----------------------------------------------------
*/

/*
 * ---- For details, see the analogous definition for C.
 */
	STRUCTURE  /VosSizeSTRUCT/
            INTEGER*4 	which_scan
            INTEGER*4 	num_param
            INTEGER*4 	num_sweep
            INTEGER*4 	num_ray
            INTEGER*4 	num_cell( MAX_PARM )
            INTEGER*4 	productID
            INTEGER*4 	which_param
	    INTEGER*4 	byte_size
	    INTEGER*4 	param_position
	END STRUCTURE
/*
 * ---- structures for 1 and 2 byte parameters
 *
 *	The Toolkit's IO_GV.h defines FORTRAN array indicies in
 *	the reverse order from C's array indicies.
 */
        STRUCTURE       /PARAMETER_2BYTE/
          RECORD /PARAMETER_DESCRIPTOR/ parmDesc
          RECORD /CELL_RANGE_VECTOR/    cellRangeVector
          INTEGER*2  data( MAX_CELL , MAX_RAY , MAX_SWEEP )
        END STRUCTURE

        STRUCTURE       /PARAMETER_1BYTE/
          RECORD /PARAMETER_DESCRIPTOR/ parmDesc
          RECORD /CELL_RANGE_VECTOR/    cellRangeVector
          BYTE       data( MAX_CELL , MAX_RAY , MAX_SWEEP )
        END STRUCTURE
/*
 * ----- SENSOR structures for 1 and 2 byte parameters
 */
        STRUCTURE       /SENSOR_2BYTE/
          RECORD /RADAR_DESCRIPTOR/    radarDesc
          RECORD /CORRECTION_FACTOR_DESCRIPTOR/ corrFactorDesc
          RECORD /SWEEP_INFO/       sweepInfo(MAX_SWEEP)
          INTEGER*4  rayInfoInteger(7,MAX_RAY,MAX_SWEEP)
          REAL*4     rayInfoFloat(4, MAX_RAY, MAX_SWEEP)
          REAL*4     platformInfo(18, MAX_RAY, MAX_SWEEP)
          RECORD /PARAMETER_2BYTE/     parm
        END STRUCTURE

        STRUCTURE       /SENSOR_1BYTE/
          RECORD /RADAR_DESCRIPTOR/    radarDesc
          RECORD /CORRECTION_FACTOR_DESCRIPTOR/ corrFactorDesc
          RECORD /SWEEP_INFO/       sweepInfo(MAX_SWEEP)
          INTEGER*4  rayInfoInteger(7,MAX_RAY,MAX_SWEEP)
          REAL*4     rayInfoFloat(4, MAX_RAY, MAX_SWEEP)
          REAL*4     platformInfo(18, MAX_RAY, MAX_SWEEP)
          RECORD /PARAMETER_1BYTE/  parm
        END STRUCTURE
/*
 * ----- volume scan structures for 1 and 2 byte parameters
 */
        STRUCTURE       /L1_GV_2BYTE/
          CHARACTER*5001              comments
          RECORD /VOLUME_DESCRIPTORS/ volDes
          RECORD /SENSOR_2BYTE/       sensor
        END STRUCTURE

        STRUCTURE       /L1_GV_1BYTE/
          CHARACTER*5001              comments
          RECORD /VOLUME_DESCRIPTORS/ volDes
          RECORD /SENSOR_1BYTE/       sensor
        END STRUCTURE
/*
 * ----- date time structure
 *
 *	It is useful to have the date and the time in the same structure
 *	if you are reading more than one granule.  FORTRAN does not
 *	have a struct tm structure.
 */
	STRUCTURE /DATE_TIME_STR/
          INTEGER*2             tkyear
          INTEGER*2             tkmonth
          INTEGER*2             tkday
          BYTE                  tkhour
          BYTE                  tkminute
          BYTE                  tksecond
	END STRUCTURE

	STRUCTURE /DATE_TIME_ARRAY_STR/
          INTEGER*2             tkyear(   MAX_VOS )
          INTEGER*2             tkmonth(  MAX_VOS )
          INTEGER*2             tkday(    MAX_VOS )
          BYTE                  tkhour(   MAX_VOS )
          BYTE                  tkminute( MAX_VOS )
          BYTE                  tksecond( MAX_VOS )
	  BYTE			num_date
	END STRUCTURE

/*
 * ---- the VosSummary structure
 */

	/* MAX_FILE_LEN = ( TK_FNAME + TK_PATHLEN ) = 128 + 256 = 384 */
	/* MAX_PARAM_LEN = 20 */
	/* macros are not allowed inside FORTRAN structure definitions. */

	STRUCTURE /VosSummarySTRUCT/
	  BYTE			   scans_in_granule
	  BYTE			   which_scan
	  CHARACTER*384		   granule_ID
	  INTEGER*4	 	   productID 
	  BYTE			   num_param 
	  BYTE			   num_sweep 
	  INTEGER*2		   num_ray
	  INTEGER*2		   num_cell( MAX_PARM )
	  INTEGER*2		   start_year
	  BYTE			   start_month
	  BYTE			   start_day
	  BYTE			   start_hour
	  BYTE			   start_minute
	  BYTE			   start_second
	  CHARACTER*20		   param_name( MAX_PARM )
	END STRUCTURE


#else
#include "IO.h"
/***************************/
/* Define structures for C */
/***************************/

	typedef struct
	{
	  char				parmName[9];
	  char				parmDesc[41];
	  char				parmUnits[9];
	  int16				interPulsePeriod;
	  int16				transFreq;
	  float32			receiverBandwidth;
	  int16				pulseWidth;
	  int16				polarTransWave;
	  int16				numOfsamples;
	  int16				parmDataType;
	  char				thresholdField[9];
	  float32			thresholdValue;
	  float32			scaleFactor;
	  float32			offsetFactor;
	  int32				deletedOrMissDataFlag;
	} PARAMETER_DESCRIPTOR;


	typedef struct
	{
 	  int32				numOfCells;
	  float32			distanceToCell[MAX_CELL];
	} CELL_RANGE_VECTOR;


        typedef struct
        {
	  PARAMETER_DESCRIPTOR		parmDesc;
	  CELL_RANGE_VECTOR		cellRangeVector;
          /*
          ** COMPARISON: To access an individual cell value:
          ** static 3D array:  value = parmData?byte[sweep][ray][cell];
          ** calloc'ed array:  value = *(parmData?byte[sweep] + ray*ncell + cell)
          */
          int8                          ***parmData1byte;
          int16                         ***parmData2byte;
        } PARAMETER;

	
	typedef struct
	{
	  char 				radarName[9];
	  float32			radarConstant;
	  float32			nomPeakPower;
	  float32			nomNoisePower;
	  float32			receiverGain;
	  float32			antennaGain;
	  float32			radarSystemGain;
	  float32			horBeamWidth;
	  float32			verBeamWidth;		
	  int16				radarType;
	  int16				scanMode;
	  float32			nomScanRate;
	  float32			nomStartAngle;
	  float32			nomStopAngle;
	  int16				numParmDesc;
	  int16				numDesc;
	  int16				dataComp;
	  int16				dataReductAlg;
	  float32			dataReductParm1;
	  float32			dataReductParm2;
	  float32			radarLon;
	  float32			radarLat;
	  float32			radarAlt;
	  float32			velocity;
	  float32			range;
	  int16				numTransfreqency;
	  int16				numInterPulsePeriods;
	  float32			frequency1;
	  float32			frequency2;
	  float32			frequency3;
	  float32			frequency4;
	  float32			frequency5;
	  float32			interPulsePeriod1;
	  float32			interPulsePeriod2;
	  float32			interPulsePeriod3;
	  float32			interPulsePeriod4;
	  float32			interPulsePeriod5; 
	} RADAR_DESCRIPTOR;

	
	typedef struct
	{
	  float32			azimuth;
	  float32			elevation;
	  float32			rangeDelay;
	  float32			radarLon;
	  float32			radarLat;
	  float32			radarPressAlt;
	  float32			radarAltAboveGround;
	  float32			radarPlatGroundSpeedEW;
	  float32			radarPlatGroundSpeedNS;
	  float32			radarPlatVerticalVelocity;
	  float32			radarPlatHeading;
	  float32			radarPlatRoll;
	  float32			radarPlatPitch;	
	  float32			radarPlatDrift;
	  float32			radarRotAngle;
	  float32			radarTiltAngle;
	} CORRECTION_FACTOR_DESCRIPTOR;


	typedef struct
	{
	  char				radarName[9];
	  int32				sweepNum;
	  int32				numRays;
	  float32			trueStartAngle;
	  float32			trueStopAngle;
	  float32			fixedAngle;
	  int32				filterFlag;
	} SWEEP_INFO;


        typedef struct 
        {
	  RADAR_DESCRIPTOR		radarDesc;
	  CORRECTION_FACTOR_DESCRIPTOR  corrFactorDesc;
	  SWEEP_INFO			sweepInfo[MAX_SWEEP];
	  int32				rayInfoInteger[MAX_SWEEP][MAX_RAY][7];
	  float32			rayInfoFloat[MAX_SWEEP][MAX_RAY][4];
	  float32			platformInfo[MAX_SWEEP][MAX_RAY][18]; 
	  PARAMETER			*parm[MAX_PARM];
        } SENSORS;


	typedef struct
	{
	  int16				verNum;
	  int16				volNum;
	  int32				sizeDataRec;
	  char				projectName[21];
	  int16				year;
	  int16				month;
	  int16				day;
	  int16				hour;
	  int16				minute;
	  int16				second;
	  char				flightNum[9]; 
	  char				facName[9];
	  int16			 	genYear;
	  int16				genMonth;
	  int16				genDay;
	  int16				numSensorDesc;	
	} VOLUME_DESCRIPTORS;


        typedef struct
        {
	  char				comments[5001];
	  VOLUME_DESCRIPTORS		volDes;
	  SENSORS			sensor;
        } L1B_1C_GV;

	typedef struct
	{
  	  char				referLat[51];
  	  char				referLon[51];
  	  char				referX[51];
  	  char				referY[51];
  	  char				gridSpacingX[51];
  	  char				gridSpacingY[51];
  	  char				orign[101];
	} RADARGRID_STR;


	typedef struct
	{
  	  int16				rainPercent;
	} RAIN_PERCENT;



        /****************************************/
        /* Define L2A-53 single radar site.	*/
        /****************************************/

	typedef struct
	{
	  TIME_STR			tktime;
	  int16				rainRate_scale[151][151];
	  float32			rainRate[151][151];
	} L2A_53_SINGLE_RADARGRID;


        /**********************************************/
        /*Define L2A-53 multiple radar site in Texas. */
        /**********************************************/

	typedef struct
	{
	  TIME_STR			tktime;
	  int16				rainRate_scale[285][363];
	  float32			rainRate[285][363];
	} L2A_53_MULT_TX_RADARGRID;


        /************************************************/
        /*Define L2A-53 multiple radar site in Florida. */
        /************************************************/

        typedef struct
        {
          TIME_STR                      tktime;
          int16                         rainRate_scale[353][257];
          float32                       rainRate[353][257];
        } L2A_53_MULT_FL_RADARGRID;



        /****************************************/
        /* Define L2A-54 singe data structure.  */
        /****************************************/

	typedef struct
	{
	  TIME_STR			tktime;
	  int8				convStratFlag[151][151];
	  } L2A_54_SINGLE_RADARGRID;


        /**********************************************/
        /*Define L2A-54 multiple radar site in Texas. */
        /**********************************************/

	typedef struct
	{
	  TIME_STR			tktime;
	  int8				convStratFlag[285][363];
	  } L2A_54_MULT_TX_RADARGRID;

        /************************************************/
        /*Define L2A-54 multiple radar site in Florida. */
        /************************************************/

        typedef struct
        {
          TIME_STR                      tktime;
          int8                          convStratFlag[353][257];
          } L2A_54_MULT_FL_RADARGRID;


        /****************************************/
        /* Define L2A-55 single radar site.	*/
        /****************************************/

	typedef struct
	{
	  TIME_STR			tktime;
	  int16				threeDreflect_scale[13][151][151];
	  float32			threeDreflect[13][151][151];
	  int16				vertProfile_scale[13][12];
	  float32			vertProfile[13][12];
	  int32				cfadData[13][86][12];
	} L2A_55_SINGLE_RADARGRID;


        /**********************************************/
        /*Define L2A-55 multiple radar site in Texas. */
        /**********************************************/

	typedef struct
	{
	  TIME_STR			tktime;
	  int16				threeDreflect_scale[13][285][363];
	  float32			threeDreflect[13][285][363];
	  int16				vertProfile_scale[13][12];
	  float32			vertProfile[13][12];
	  int32				cfadData[13][86][12];
	} L2A_55_MULT_TX_RADARGRID;

        /************************************************/
        /*Define L2A-55 multiple radar site in Florida. */
        /************************************************/

        typedef struct
        {
          TIME_STR                      tktime;
          int16                         threeDreflect_scale[13][353][257];
          float32                       threeDreflect[13][353][257];
          int16                         vertProfile_scale[13][12];
          float32                       vertProfile[13][12];
          int32                         cfadData[13][86][12];
        } L2A_55_MULT_FL_RADARGRID;


        /****************************************/
        /* Define L3A-53 single radar site.	*/
        /****************************************/

	typedef struct
	{
	  int16				pentadRainfall_scale[151][151];
	  float32			pentadRainfall[151][151];
	} L3A_53_SINGLE_RADARGRID;


        /**********************************************/
        /*Define L3A-53 multiple radar site in Texas. */
        /**********************************************/

	typedef struct
	{
	  int16				pentadRainfall_scale[285][363];
	  float32			pentadRainfall[285][363];
	} L3A_53_MULT_TX_RADARGRID;

        /************************************************/
        /*Define L3A-53 multiple radar site in Florida. */
        /************************************************/

        typedef struct
        {
          int16                         pentadRainfall_scale[353][257];
          float32                       pentadRainfall[353][257];
        } L3A_53_MULT_FL_RADARGRID;


        /****************************************/
        /* Define L3A-54 single radar site.	*/
        /****************************************/

	typedef struct
	{
	  int16				monthlyRainfall[151][151];
	} L3A_54_SINGLE_RADARGRID;


        /**********************************************/
        /*Define L3A-54 multiple radar site in Texas. */
        /**********************************************/

	typedef struct
	{
	  int16				monthlyRainfall[285][363];
	} L3A_54_MULT_TX_RADARGRID;



        /************************************************/
        /*Define L3A-54 multiple radar site in Florida. */
        /************************************************/

        typedef struct
        {
          int16                         monthlyRainfall[353][257];
        } L3A_54_MULT_FL_RADARGRID;


        /****************************************/
        /* Define L3A-55 data structure.        */
        /****************************************/

	typedef struct
	{
	  int16				vertProfile_scale[13][12];
	  float32			vertProfile[13][12];
	  int32				cfadData[13][86][12];
	} L3A_55_RADARGRID;


        /****************************************/
        /* Define L3A-46 data structure.        */
        /****************************************/

        typedef struct
        {
	  float32		ssmiData[2][180][360];
	} L3A_46_PLANETGRID;


	typedef struct
	{
	  char			networkName[21];
	  int16			nValidSensor;
	  int8			networkID;
	} NETDESC;



	typedef struct
	{
	  int8			networkID;
	  char			gaugeType[21];
	  char			gaugeName[21];
	  char			gaugeMakeModel[21];
	  int32			gaugeNumber;
	  float32		gaugeLat;
	  float32		gaugeLong;
	  float32		range;
	  float32		azimuth;
	  float32		xCoordinate;
	  float32		yCoordinate;
	  float32		resolution;
	  float32		elevation;
	} GAUGEDESC;


	typedef struct
	{
	  GAUGEDESC		gaugeDesc;
	  int8			hour[1440];
	  int8			minute[1440];
	  int16			meanRainRate_scale[1440];
	  float32		meanRainRate[1440];
	} L2A_56_RAINGAUGE;


	typedef struct
	{
	  int8			networkID;
	  char			disdroName[21];
	  char			disdroMakeModel[21];
	  int32			disdroNumber;
	  float32		disdroLat;
	  float32		disdroLong;
	  float32		range;
	  float32		azimuth;
	  float32		resolution;
	  float32		elevation;
	} DISDRODESC;


        typedef struct
        {
          DISDRODESC            disdroDesc;
          int8                  hour[1440];
          int8                  minute[1440];
          int16                 nConcentration[1440][20];
        } L2A_57_DISDROMETER;


/*
** A structure to hold L1 GV information.  This structure is used to
** build a link list of L1 GV data.
*/
typedef struct L1_GV_TEMPLATE_NODE
{
  int32 inputFile;
  char  filename[TK_FNAME_LEN];
  int32 numOfVOS;
  int32 numOfParm[MAX_VOS];
  int32 nsweep[MAX_VOS];
  int32 nray[MAX_VOS];
  int32 ncell[MAX_VOS][MAX_PARM];
  struct L1_GV_TEMPLATE_NODE *next;
} L1_GV_TEMPLATE;


/*
	====================================
	    C structure definitions
		for TKutilGV.c
	====================================
*/

/*
--------------------------------------------
	typedef of structures
--------------------------------------------
*/

/*
 * ---- the VosSizeSTRUCT structure
 *
 *      This VosSizeSTRUCT structure differs from the one used in GV browse
 *      in that the num_cell field is an array here.  This enhancement
 *      allows the GV CSI program to allocated 1B51 structures as
 *      well as 1C51 structures.
 */
        typedef struct
        {
/*	  The which_scan element is filled with the user selected value
	  in the driver functions TKreadL1GV_driver, TKfreeL1GV_driver,
	  and TK_copy_1c51_to_FORTRAN.

	  The which_scan element is used when determining the size of
	  the volume scan before the volume scan is allocated.  This is
	  done by the function TKreadL1GVsize. */

            int32 which_scan;  /* 0...MAX_VOS-1  */

/*	  The following four elements define the size of a volume scan.
	  The elements are filled by TKreadL1GVsize and
	  used by many functions.  The elements are particularly important
	  in the allocation and deallocation of a volume scan. */
	
            int32 num_param;
            int32 num_sweep;
            int32 num_ray;
            int32 num_cell[ MAX_PARM ] ;

/*	  The productID element is copied from the granule handle in
	  the function TKreadL1GVsize.

	  The productID element is used by the functions that allocate
	  and deallocate a volume scan.  productID helps those functions
	  determine if a parameter is 1 byte or 2 bytes. 

	  The which_param element is assigned and accessed only within
	  the functions that allocate and deallocate a volume scan.

	  When reading the entire volume scan, productID and which_param 
	  pin points whether a parameter is 1 byte or 2 bytes.

	  Product ID is defined in IO.h */

            int32 productID ;
            int32 which_param ;

/*	  The byte_size and param_position elements are only used
	  to allocate a volume scan.  They are not used to deallocate
	  a volume scan.

	  When reading only one parameter of a volume scan, the
	  default method of determining the parameters 1/2 byte size
	  does not work.  When allocating a volume scan with one parameter,
	  the byte_size is determined from the user supplied 
	  param_name.  Only param_names which contain the string
	  "QCM" for quality control mask, are assumed to be 1 byte.

	  The param_position is used to overide the default index
	  that increments the num_cell array in the VosSizeSTRUCT structure. */

	    int32 byte_size ;
	    int32 param_position ;

        } VosSizeSTRUCT;
/*
 * ---- the VosSummary structure
 */
	typedef struct
	{
	  /* high level information */

	  int8			scans_in_granule ;
	  int8			which_scan ;
	  char			granule_ID[ MAX_FILE_LEN ] ;
	  int32 		productID ;
	
	  /* dimensions of the volume scan */

	  int8			num_param ;
	  int8			num_sweep ;
	  int16			num_ray ;
	  int16			num_cell[ MAX_PARM ] ;

	  /* start/stop date/time of volume scan */

	  int16			start_year ;	/* in 4 digits, like 1997 */
	  int8			start_month ;	/* 1 to 12 */
	  int8			start_day ;	/* 1 to 31 */
	  int8			start_hour ;	/* 0 to 23 */
	  int8			start_minute ;  /* 0 to 59 */
	  int8			start_second ;  /* 0 to 59 */

	  /* the names of the parameters in the volume scan */

	  char			param_name[ MAX_PARM ][ MAX_PARAM_LENGTH ] ;

	} VosSummarySTRUCT ;
/*
 * ---- The C definition of the FORTRAN 1c51 and 1b51 volume scans.
 *	Note that the C version of 1c51 and 1b51 volume scans is
 * 	different from the FORTRAN 1c51 and 1b51 volume scans.
 */

	/* <<<< parameter structures >>>> */

        typedef struct 
	{
          PARAMETER_DESCRIPTOR 	parmDesc ;
          CELL_RANGE_VECTOR    	cellRangeVector ;
          int16                 data[ MAX_SWEEP ][ MAX_RAY ][ MAX_CELL ] ;
        } PARAMETER_2BYTE ;

        typedef struct 
	{
          PARAMETER_DESCRIPTOR 	parmDesc ;
          CELL_RANGE_VECTOR    	cellRangeVector ;
          int8                  data[ MAX_SWEEP ][ MAX_RAY ][ MAX_CELL ] ;
        } PARAMETER_1BYTE ;

	/* <<<< sensor structures >>>> */

        typedef struct
        {
          RADAR_DESCRIPTOR              radarDesc;
          CORRECTION_FACTOR_DESCRIPTOR  corrFactorDesc;
          SWEEP_INFO                    sweepInfo[MAX_SWEEP];
          int32                         rayInfoInteger[MAX_SWEEP][MAX_RAY][7];
          float32                       rayInfoFloat[MAX_SWEEP][MAX_RAY][4];
          float32                       platformInfo[MAX_SWEEP][MAX_RAY][18];
          PARAMETER_2BYTE               parm ;
        } SENSOR_2BYTE ;

        typedef struct
        {
          RADAR_DESCRIPTOR              radarDesc;
          CORRECTION_FACTOR_DESCRIPTOR  corrFactorDesc;
          SWEEP_INFO                    sweepInfo[MAX_SWEEP];
          int32                         rayInfoInteger[MAX_SWEEP][MAX_RAY][7];
          float32                       rayInfoFloat[MAX_SWEEP][MAX_RAY][4];
          float32                       platformInfo[MAX_SWEEP][MAX_RAY][18];
          PARAMETER_1BYTE               parm ;
        } SENSOR_1BYTE ;

	/* <<<< volume scan structures >>>> */

        typedef struct
        {
          char                          comments[5001];
          VOLUME_DESCRIPTORS            volDes;
          SENSOR_2BYTE                  sensor;
        } L1_GV_2BYTE;

        typedef struct
        {
          char                          comments[5001];
          VOLUME_DESCRIPTORS            volDes;
          SENSOR_1BYTE                  sensor;
        } L1_GV_1BYTE;
/*
 * ----- date time structure
 *
 *      It is useful to have the date and the time in the same structure
 *      if you are reading more than one granule.  FORTRAN does not
 *      have a struct tm structure.
 */
        typedef struct
        {
          int16                 tkyear ;
          int16                 tkmonth ;
          int16                 tkday ;

          int8                  tkhour ;
          int8                  tkminute ;
          int8                  tksecond ;

	} DATE_TIME_STR ;

        typedef struct
        {
          int16                 tkyear[   MAX_VOS ] ;
          int16                 tkmonth[  MAX_VOS ] ;
          int16                 tkday[    MAX_VOS ] ;

          int8                  tkhour[   MAX_VOS ] ;
          int8                  tkminute[ MAX_VOS ] ;
          int8                  tksecond[ MAX_VOS ] ;

	  int8			num_date ;
	} DATE_TIME_ARRAY_STR ;

/*********************************/
/* Prototypes of L1 GV programs. */
/*********************************/

int32 setL1GVtemplate(int32 nvos, int32 nparm[MAX_VOS], int32 ncell[MAX_VOS][MAX_PARM],
                      int32 nray[MAX_VOS], int32 nsweep[MAX_VOS], char *fileName,
                        int32 inputFile);
int32 manageL1GVtemplate(int32 opType, int32 inputFile, char *filename,
                         L1_GV_TEMPLATE *template, int32 orderOfVOS, int32 orderOfParm);
int32 openInputVgroupL1GV(int32 inputFile, char filemode, int32 sdstartID, 
				VGROUP_LINK *vGroup, int dataType);
void makeL1GVconfigFile(int32 inputFile, char *cfgFileName, int productID);

L1_GV_TEMPLATE *findTemplateNode(int32 inputFile, L1_GV_TEMPLATE *template);
int32 setInputFile(L1_GV_TEMPLATE *template, int32 inputFile, char *filename);
int32 getNvos(L1_GV_TEMPLATE *template, int32 inputFile);
int32 getNparm(L1_GV_TEMPLATE *template, int32 inputFile, int32 vosNumber);
int32 getNcell(int32 inputFile,L1_GV_TEMPLATE *template,int32 vosNumber,
               int32 parmNumber);
int32 getNray(int32 inputFile, L1_GV_TEMPLATE *template, int32 vosNumber);
int32 getNsweep(int32 inputFile, L1_GV_TEMPLATE *template, int32 vosNumber);
int32 addTemplate(L1_GV_TEMPLATE **templateList, L1_GV_TEMPLATE *template);
int32 deleteTemplate(L1_GV_TEMPLATE **templateList, int32 inputFile);
int32 manageL1GVtemplate(int32 opType, int32 inputFile, char *filename,
                         L1_GV_TEMPLATE *template, int32 vosNumber, int32 parmNumber);
int32 findSubVgroups(int32 inputFile, int32 vgroupID, int32 *subVgroups);
void printComment(FILE *fileptr, int order);
void printVolumeDescriptor(FILE *fileptr, int order);
void printRadarDesc(FILE *fileptr, int order);
void printCorrFactorDesc(FILE *fileptr, int order);
void printSweepInfo(FILE *fileptr, int order);
void printRayInfoInteger(FILE *fileptr, int order, int32 nray, int32 nsweep);
void printRayInfoFloat(FILE *fileptr, int order, int32 nray, int32 nsweep);
void printPlatformInfo(FILE *fileptr, int order, int32 nray, int32 nsweep);
void printParmDesc(FILE *fileptr, int order1, int order2);
void printCellRangeVector(FILE *fileptr, int order1, int order2, int32 ncell);
void printParmData(FILE *fileptr, int order1, int order2, int32 ncell,
                   int32 nray, int32 nsweep);
VGROUP_LINK *getCurrVgroup(VGROUP_LINK *vgroupList, int32 vgroupPos);
int32 updateTemplate(L1_GV_TEMPLATE **templateList, int32 inputFile, char *filename);
void bufManage(uint8 **databuf, int *bufsize, int new_bufsize);

/*
	====================================
	    C function prototypes
		for TKutilGV.c
	====================================
*/

int     TK_read_scan_size(
                IO_HANDLE               * handle ,
                VosSizeSTRUCT                 *current_VosSize ) ;

int TK_alloc_a_param(
        PARAMETER               **parm,
        VosSizeSTRUCT                 *vs ) ;

int     TK_alloc_a_2byte_param(
                int16                   **** parmData2byte,
                VosSizeSTRUCT                   * vs  ) ;

int     TK_alloc_a_1byte_param(
                int8                    **** parmData1byte,
                VosSizeSTRUCT                   * vs  ) ;

/*
        ----------------------------------
        function prototypes for DEallocation
        ----------------------------------
*/

void TK_dealloc_a_2byte_param(  int16  *** parmData2byte ) ;

void TK_dealloc_a_1byte_param(  int8   *** parmData1byte ) ;


/*
--------------------------------------------
        prototypes for C functions that 
	interact with FORTRAN
--------------------------------------------
*/

int	TK_read_1c51_1b51_in_C( IO_HANDLE * handle,
                L1B_1C_GV              ** volume_scan ,
                VosSizeSTRUCT                 * VosSize,
		char			algorithm_ID[ TK_ALGORITHM_ID_SIZE ] ,
                char                    param_name[ ],
                int                     read_only_bottom_sweep ) ;

int	TK_read_1c51_1b51_summary( IO_HANDLE * handle,
		VosSummarySTRUCT	*VosSummary ) ;

int     TKreadL1GVsize(
                IO_HANDLE               * handle ,
                VosSizeSTRUCT                 * VosSize ) ;

int     TK_copy_1byte_to_FORTRAN(
                L1B_1C_GV               * volume_scan ,
                L1_GV_1BYTE             * data,
		VosSizeSTRUCT		* VosSize ) ;

int     TK_copy_2byte_to_FORTRAN(
                L1B_1C_GV               * volume_scan ,
                L1_GV_2BYTE             * data,
		VosSizeSTRUCT		* VosSize ) ;

void	TK_copy_dates_to_FORTRAN(
		DATE_TIME_ARRAY_STR * date_array ,
		struct tm       date[ MAX_VOS ] ,
		int		num_date ) ;

void     TK_copy_2byte_metadata_to_FORTRAN(
                L1B_1C_GV               * C_vos ,
                L1_GV_2BYTE             * FORTRAN_vos ) ;

int     TKallocateL1GV(
            VosSizeSTRUCT *vs, L1B_1C_GV **vos ) ;

/*
	internal Toolkit use only
*/

void print_in_C(   	 L1B_1C_GV     * vos_in_c   ) ;
void print_in_FOR( 	 L1_GV_2BYTE   * vos_in_for ) ;
void print_in_FOR_1byte( L1_GV_1BYTE   * vos_in_for ) ;

/*
------------------------------------------------
	APIs for reading and freeing a volume scan in C
------------------------------------------------
*/

int TKreadL1GVbyVosNum(IO_HANDLE * handle, L1B_1C_GV ** vos, int VosNum,
  VosSizeSTRUCT *VosSize ) ;

int TKreadL1GVparam( IO_HANDLE * handle ,
  L1B_1C_GV ** vos , int VosNum , char param_name[ 20 ],
  VosSizeSTRUCT *VosSize ) ;

int TKreadL1GVbaseSweep( IO_HANDLE * handle ,
  L1B_1C_GV ** vos , int VosNum , char param_name[ 20 ],
  VosSizeSTRUCT *VosSize ) ;

int TKreadL1GVsummary( IO_HANDLE * handle ,
  int VosNum, VosSummarySTRUCT *VosSummary ) ;

void TKprintL1GVsummary( VosSummarySTRUCT *VosSummary ) ;

void TKfreeL1GV(      L1B_1C_GV ** vos ) ;

int TKreadL1GVdate( IO_HANDLE *sGranuleHandle, struct tm date[ MAX_VOS ] ,
	int * num_date ) ;

int TKpickL1GVvosNum(IO_HANDLE * handle, struct tm target_date, int *VosNum ,
	int * min_diff_in_seconds ) ;

int TKreadL1GVparamPos( 
	IO_HANDLE *sGranuleHandle, int VosNum,
        char target_param_name[ ], int *param_position ) ;

int TKreadL1GVparamName( 
	IO_HANDLE *sGranuleHandle, int VosNum,
        char param_name[ MAX_PARM ][ MAX_PARAM_LENGTH ], int * num_param ) ;
/*
------------------------------------------------
        wrapper functions for reading and a volume scan in FORTRAN
------------------------------------------------
*/

int  tkreadl1gvparam_(      IO_HANDLE * handle , L1_GV_2BYTE *vos_in_FOR ,
	int *VosNum , char param_name[ ], VosSizeSTRUCT * VosSize, int n ) ;

int  tkreadl1gv1byteparam_( IO_HANDLE * handle , L1_GV_1BYTE *vos_in_FOR ,
	int *VosNum , char param_name[ ], VosSizeSTRUCT * VosSize, int n ) ;

int  tkreadl1gvbasesweep_( IO_HANDLE * handle , L1_GV_2BYTE *vos_in_FOR ,
	int *VosNum , char param_name[ ], VosSizeSTRUCT * VosSize, int n ) ;

int  tkpickl1gvvosnum_( IO_HANDLE * handle, 
	DATE_TIME_STR *target_date,
	int *VosNum, int * diff_in_seconds ) ;

int  tkreadl1gvdate_( IO_HANDLE *sGranuleHandle, 
	DATE_TIME_ARRAY_STR 	*date_array, int * num_date ) ;

int  tkreadl1gvsummary_( IO_HANDLE * handle ,
  int * VosNum, VosSummarySTRUCT *VosSummary ) ;

int tkprintl1gvsummary_( VosSummarySTRUCT *VosSummary ) ;

/*
------------------------------------------------
        high level utilities for reading a volume scan in FORTRAN
------------------------------------------------
*/

int TKreadL1GV_1byte_in_FORTRAN( IO_HANDLE * handle ,
  L1_GV_1BYTE *vos , int VosNum , char param_name[ 20 ],
  int read_only_bottom_sweep, VosSizeSTRUCT *VosSize ) ;

int TKreadL1GV_2byte_in_FORTRAN( IO_HANDLE * handle ,
  L1_GV_2BYTE *vos , int VosNum , char param_name[ 20 ],
  int read_only_bottom_sweep, VosSizeSTRUCT *VosSize ) ;

int TKreadL1GVdate_in_FORTRAN( IO_HANDLE *sGranuleHandle,
        DATE_TIME_ARRAY_STR * date_array, int * num_date ) ;

/*
------------------------------------------------
	high level utilities for reading a volume scan in C
------------------------------------------------
*/

int TKreadL1GV_error_check( IO_HANDLE * handle ,
  L1B_1C_GV ** vos , int VosNum , char param_name[ 20 ],
  int read_only_bottom_sweep, VosSizeSTRUCT *VosSize ) ;

int TKreadL1GV_driver( IO_HANDLE * handle ,
  L1B_1C_GV ** vos , int VosNum , char param_name[ 20 ],
  int read_only_bottom_sweep, VosSizeSTRUCT *VosSize ) ;

int TKresetL1GV( IO_HANDLE *sGranuleHandle, int VosNum ) ;

/*
------------------------------------------------
	low level functions to read a single parameter from a volume scan 
------------------------------------------------
*/

void  copyGV_L1_vosDes(        L1B_1C_GV *sL1b1c , uint8 *ptr ) ;
void  copyGV_L1_radarDes(      L1B_1C_GV *sL1b1c , uint8 *ptr ) ;
void  copyGV_L1_corrFactorDes( L1B_1C_GV *sL1b1c , uint8 *ptr ) ;
void  copyGV_L1_sweepInfo(     L1B_1C_GV *sL1b1c , uint8 *ptr, int32 i ) ;
void  copyGV_L1_paramDesc(     L1B_1C_GV *sL1b1c , uint8 *ptr, int32 i ) ;


void readGV_L1B_1C_metadata(IO_HANDLE *sGranuleHandle, 
	L1B_1C_GV *sL1b1c, VGROUP_LINK *currVgroup ,
          uint8 **DBL_databuf , int * DBL_bufsize ,
	int32 num_cell[ MAX_PARM ], 
	int32 * ptr_num_ray, int32 * ptr_num_sweep ) ;

void readGV_L1B_1C_param(IO_HANDLE *sGranuleHandle, 
	L1B_1C_GV *sL1b1c, int productID,
          uint8 **DBL_databuf , int * DBL_bufsize ,
	VDATA_LINK    *tempVdata , SDS_LINK      *tempSDS,
	int32 num_cell[ MAX_PARM ], int32 * ptr_num_ray, int32 * ptr_num_sweep,
	int32 param_position_in_file, uint8 *ptr ,
	int read_only_bottom_sweep ) ;

int readGV_L1B_1C_subset(IO_HANDLE *sGranuleHandle, 
	L1B_1C_GV *sL1b1c, int productID,
   char target_param_name[ ], int read_only_bottom_sweep ) ;

#endif
#endif
