/* DEVELOPMENT HISTORY */
/* Revision 1.67  1999/09/22 19:50:15  pkolbe
 * changed the order of the 3b31 structure elements
 *
 * Revision 1.66  1999/09/14 20:53:27  pkolbe
 * changed both #define L2A25_NEARSURFRAIN and L2A25_NEARSURFZ from 10 to 100
 * (these are used for 2A25 Real Time 1)
 *
 * Revision 1.65  1999/09/13 15:21:54  pkolbe
 * finished backward compatability code changes
 *
 * Revision 1.64  1999/08/31 21:34:00  pkolbe
 * further 3b31 struct changes
 *
 * Revision 1.63  1999/08/24 15:38:39  pkolbe
 * further changes of the 3b31 structure
 *
 * Revision 1.62  1999/08/16 20:26:20  pkolbe
 * added elements (no subtractions, yet) to the 3b31 structure
 *
 * Revision 1.61  1999/08/16 17:20:32  pkolbe
 * added pia2a25Mean, pia2a25Dev, pia2a25H to 3a25 grid 1 structure
 *
 * Revision 1.60  1999/08/13 17:08:10  pkolbe
 * changed scaling factors for
 * L2A25_RAIN  L2A25_CORRECTZFACTOR  L2B31_SIGMARRSURF
 * from 10 to 100
 *
 * Revision 1.59  1998/09/01 13:43:34  pkolbe
 * added 3A25 data elements
 *
 * Revision 1.58  1998/08/05 17:25:51  pkolbe
 * changed name "nearSurfHeight" to "nearSurfBin" in 2a25 real time 1;
 * also changed the size from 2 bytes to 1 byte
 *
 * Revision 1.57  1998/06/26 20:38:57  pkolbe
 * added fortran 2a23 2a25 real time structures; added nearSurfHeight 2a25 real time 1 variable
 *
 * Revision 1.56  1998/06/26 14:36:54  slywczak
 * Added new real time structures for L2A23 (L2A_23_R1_SWATHDATA) and the two
 * L2A25 products (L2A_25_R1_SWATHDATA and L2A_25_R2_SWATHDATA).  Added scaling
 * macro for near surface rain and near surface reflectivity.
 *
 * Revision 1.55  1998/05/21 17:56:24  pkolbe
 * 3A25 20->30 size changes and 3A26 srt-2A25 name changes
 *
 * Revision 1.54  1998/04/27 13:30:10  slywczak
 * Modified structure L3A_26_GRID.  Added parameter rainMeanTH and changed the
 * data type of srtpDf from float32 to int32.
 *
 * Revision 1.53  1998/04/23 15:54:55  slywczak
 * Modified the structures for the L1B and L1C products.  Added the following
 * fields:  binEllipsoid, binClutterFreeBottom, binDIDHmean, binDIDHtop,
 * and binDIDHbottom.  Modified the structure definition for the L2A23 product
 * and added the following fields:  BBintensity, and spare.  Modified the
 * structures for 2A25 and added the following fields:  nearSurfRain, nearSurfZ,
 * pia2a25, errorRain, and errorZ.  Made the changes in both the C and FORTRAN
 * structures.
 *
 * Revision 1.52  1998/04/22 13:44:58  pkolbe
 * additions/modifications to planetary grid 1 and 2 structures
 *
 * Revision 1.51  1997/09/08 13:30:44  slywczak
 * Changed the dimension of stratRainPix2 and convRainPix2 from 3 to 4.
 *
 * Revision 1.50  1997/08/12 21:41:59  slywczak
 * Changed the location of the IO.h include to mantain compatibility with
 * FORTRAN and n32.
 *
 * Revision 1.49  1997/06/20 20:19:39  slywczak
 * Corrected syntax error in defining bbPixNum2.
 *
 * Revision 1.48  1997/06/20 20:14:10  slywczak
 * Modified the structure definitions for L3A25.  Added the bright band pixel
 * number definition for low and high resolution.
 *
 * Revision 1.47  1997/05/19 02:52:59  slywczak
 * Deleted the definition of grid struct for algorithms 3A-25, 3A-26, and
 * 3B-31.
 *
 * Revision 1.46  1997/03/14 20:20:38  slywczak
 * Updated the L3A_25_GRID structure.  Changed the dimension of the element
 * zmGrandH from 20 to 30 so that the C and FORTRAN structures matched.
 *
 * Revision 1.45  1997/03/10 16:31:16  slywczak
 * Modified the structure L2B_31_SWATHDATA (C and FORTRAN).  Changed the dimension
 * of the structure elements TMI-PIA and Sigma-TMI-PIA from 104 to 49.  Make a
 * corresponding change in the configuration file.
 * Cv: ----------------------------------------------------------------------
 *
 * Revision 1.44  1997/03/07 04:06:21  slywczak
 * Modified the L3A_26_GRID structure definition (both C & FORTRAN).  Changed
 * the datatypes of the following elements:  reliabOrderFit, reliabHBfit,
 * and reliabSRTfit from int16 to float32.
 *
 * Revision 1.43  1997/01/07 23:02:23  amak
 * 'rainCount' in L3A_25_GRID structure must be a 3 dimensional array which
 * is 16x72x4.
 *
 * Revision 1.42  1996/10/17  19:52:12  amak
 * Redefined structures for 1B21(1C21), 2A21, and 3A26.
 * For 1B21/1C21:
 *   - Inserted entities landOceanFlag and surfWarnFlag after osBinStart
 * For 2A21:
 *   - Renamed reliab to reliabFlag
 *   - Inserted reliabFactor before incAngle
 * For 3A26:
 *   - Redefined the whole structure for 3A26
 *
 * Revision 1.41  1996/08/13  18:57:45  amak
 * Modified L2A25 data structure.
 * - added 2-byte integer attenParmNode[49][5]
 * - changed dimension of parameter attenParmAlpha from 49x10 to 49x5
 * - changed dimension of parameter attenParmBeta from 49x10 to 49
 * - added 2-byte integer ZRParmNode[49][5]
 * - changed dimension of parameter ZRParmA from 49x10 to 49x5
 * - changed dimension of parameter ZRParmB from 49x10 to 49x5
 * - added 4-byte float zmmax[49]
 * - changed dimension of parameter rangeBinNum from 49x3 to 49x6
 * - changed dimension of parameter zeta from 49 to 49x2
 * - renamed zeta_m to zeta_mn, and changed dimension from 49 to 49x2
 * - renamed zeta_s to zeta_sd, and changed dimension from 49 to 49x2
 * - changed dimension of parameter xi from 49 to 49x2
 * - added 2-byte integer thickThPIZ[49]
 * - changed dimension of parameter nubfCorrectFactor from 49 to 49x2
 * - changed qualityFlag from 1-byte integer to 2-byte integer
 *
 * Revision 1.40  1996/06/25  15:50:55  amak
 * Fixed typo error.
 *
 * Revision 1.39  1996/06/25  15:15:39  amak
 * Changed array dimension of R-hat and Sigma-R-hat in 2B31 from 60x49 to
 * 80x49.
 *
 * Revision 1.38  1996/06/21  15:04:01  amak
 * Corrected array dimension for some fields in L3A25.
 *
 * Revision 1.37  1996/06/20  19:37:05  amak
 * Modified L2B_31_SWATHDATA for L2B31.
 *   - deleted fields 'sHat' and 'sigmaShat'
 *   - added fields 'rrSurf' and 'sigmaRRsurf'
 *   - defined macros L2B31_RRSURF and L2B31_SIGMARRSURF equal to 10 as
 *     scaling factor for 'rrSurf' and 'sigmaRRsurf'
 *
 * Revision 1.36  1996/06/18  15:48:44  amak
 * Changed array dimension of stormHtZmCCoef in L3A_25_PLANETGRID1 from
 * 72x16x3 to 72x16.
 *
 * Revision 1.35  1996/06/14  21:56:10  amak
 * Re-structure L2A25 header.
 *
 * Revision 1.34  1996/06/13  20:40:41  amak
 * Changed the number of records of fcifIOchar from 240 to 16 for L1B21 and
 * L1C21 header.
 *
 * Revision 1.33  1996/06/13  13:56:59  amak
 * Defined L1B21, L1C21, and L2A25 header structure.
 *
 * Revision 1.32  1996/06/07  14:56:48  amak
 * Added 'prStatus1' and 'prStatus2' fields to Scan Status block.
 *
 * Revision 1.31  1996/05/21  17:52:19  amak
 * Incorporate changes for L1B-21 and L3A-25 data structures.
 *
 * Revision 1.30  1996/05/10  13:58:17  amak
 * Defined table size for L1B21, L1C21, and L2A25 headers.
 *
 * Revision 1.29  1996/03/21  20:10:46  amak
 * Added structure for L2B31.
 *
 * Revision 1.28  1996/03/15  22:25:56  amak
 * Added entity sysNoiseWarnFlag to L1B_21_SWATHDATA structure.
 *
 * Revision 1.27  1996/03/15  19:34:59  amak
 * Added four more variables to structure POWERS (ttlPower, temp, tdaPower,
 * and logAmpInNoise).
 *
 * Revision 1.26  1996/03/07  18:40:19  amak
 * Modified data structure of product 3A-25, 3A-26, and 3B-31.
 *   3A-25
 *   * The member called ttlPix2 was defined as 160x3 and should be 160x720.
 *   * The radar structure L3A_25_RADAR for 3A-25 had been deleted.
 *   3A-26
 *   * In the fractArea element, the dimensions were 72x16x110x10x4 and
 *     should be 72x16x110x4x3.
 *   * Eliminated the structure L3A_26_RADAR from the L3A_26_GRID structure.
 *   3B-31
 *   * The structure was completely changed.
 *
 * Revision 1.25  1996/02/08  16:50:02  amak
 * Modifed code based on the change of L2A-25 data structure.
 * - ncell1 changed from 60 to 80
 * - ncell2 changed from 5 to 10
 * - Deleted Clutter Flag; stored in header identical to 1B-21
 * - Reliability changed from 2 bytes to 1 byte integer
 * - Attenuation Parameter Beta increased to every ncell2, and moved below
 *   Attenuation Parameter Alpha
 * - Doubled Average Rain Rate
 * - Added NUBF Correction Factor
 * - Added Quality Flag
 *
 * Revision 1.24  1996/01/25  14:58:06  amak
 * Changed variable name fractOrbitNum to fractOrbitN in PR_SCAN_STATUS structure.
 *
 * Revision 1.23  1996/01/18  16:01:40  yprasad
 * Make changes in scale factors.  The scale factor has changed from 10 to 100
 * for objects systemNoise, normalSample, osSurf, and osRain for 1B21 and 1C21.
 *
 * Revision 1.22  1995/12/19  17:34:08  amak
 * Defined L1C-21 data structure.  The structure of L1C-21 has the same
 * format at L1B-21.
 *
 * Revision 1.21  1995/12/12  15:11:23  amak
 * Defined scaling factor macros.
 *
 * Revision 1.20  1995/11/20  17:53:53  amak
 * Added scaling calculation variables to the structures.
 *
 * Revision 1.19  1995/11/14  21:29:13  amak
 * Uncommented four fields - minEchoFlag, binStormHeight, scLocalZenith, and
 * binEllipsoid, and changed the array dimension of variable binSurfPeak in
 * L1B_21_SWATHDATA.
 *
 * Revision 1.18  1995/10/10  14:58:39  thyun
 * Commented out the four fields (minEchoFlag, binStormHeight, scLocalZenith,
 * binEllipsoid) of L1B_21_SWATHDATA.
 *
 * Revision 1.17  1995/09/12  15:19:04  amak
 * Added 4 more fields in L1B-21.
 *
 * Revision 1.16  1995/08/25  14:15:40  amak
 * Replaced all the INTEGER*1 type to BYTE data type.
 *
 * Revision 1.15  1995/08/21  18:28:08  thyun
 * Changed the two same field name gvPathAttenH to gvPathAttenH1 and gvPathAttenH2
 * in L3A_25_RADAR.
 *
 * Revision 1.14  1995/08/10  20:14:06  thyun
 * Modified the dimension size of the field ttlPix2 according to file spec.
 *
 * Revision 1.13  1995/08/08  20:15:26  amak
 * Modified L3A-25 and L3A_26 data structures.
 *
 * Revision 1.12  1995/08/07  15:41:57  amak
 * Moved the PR_NAVIGATION block structure to IO.h.
 *
 * Revision 1.11  1995/08/04  19:59:12  amak
 * Added new data structure based on the new file spec (8/3/95)
 *
 * Revision 1.10  1995/08/03  17:46:01  amak
 * Re-modified the code because CVS couldn't do version control nicely.
 *
 * Revision 1.9  1995/07/27  13:36:07  amak
 * Added underscores to the include file names in the preprocessor statements.
 *
 * Revision 1.8  1995/07/19  20:08:58  amak
 * Added Planetary Grid into Level 3 data structures.
 *
 * Revision 1.7  1995/07/08  14:47:26  amak
 * Took out Level 3 data structure due to the change of the API for
 * Level 3 satellite products.
 *
 * Revision 1.6  1995/07/05  18:56:49  amak
 * Corrected syntax errors
 *
 * Revision 1.5  1995/06/30  18:35:56  amak
 * Added new data structures.
 *
 * Revision 1.4  1995/06/27  13:51:20  amak
 * Changed data structures based on the new file spec.
 *
 * Revision 1.3  1995/06/26  21:14:27  amak
 * Structure Changed in File Spec -- SwathData
 *
 * Revision 1.2  1995/06/08  20:10:18  amak
 * Added log file header.
 * */

/******************************************************************************
*
*  FILE NAME:		IO_PR.h
*
*  DESCRIPTION:		This file contains the data structures of the PR
*			products.
*
*******************************************************************************/



#ifndef __IO_PR_H
#ifndef LANGUAGE_FORTRAN
#define __IO_PR_H 1
#endif

/* Define Scaling Factors. */
#define L1B21_SYSTEMNOISE	100
#define L1B21_NORMALSAMPLE	100
#define L1B21_OSSURF		100
#define L1B21_OSRAIN		100
#define L2A21_SIGMAZERO	       	100
#define L2A21_PATHATTEN		100
#define L2A21_INCANGLE		10
#define L2A25_RAIN		100
#define L2A25_CORRECTZFACTOR	100
#define L2A25_ATTENPARMALPHA	10000000
#define L2A25_ZRPARMA		100000
#define L2A25_ZRPARMB		10000
#define L2A25_RAINAVE		100
#define L2A25_LIQWATERSUM	1000
#define L2A25_ATTENPARMBETA	10000
#define L2A25_WEIGHTW		1000
#define L2A25_NEARSURFRAIN      100
#define L2A25_E_SURFRAIN      100
#define L2A25_NEARSURFZ         100
#define L2B31_DHAT		100
#define L2B31_SIGMADHAT		100
#define L2B31_EPSILON		10
#define L2B31_SIGMAEPSILON	10
#define L2B31_RHAT		10
#define L2B31_SIGMARHAT		10
#define L2B31_PIA		10
#define L2B31_SIGMAPIA		10
#define L2B31_TMIPIA		10
#define L2B31_SIGMATMIPIA	10
#define L2B31_RRSURF		10
#define L2B31_SIGMARRSURF	100

/**
 **	Define Table Size for Header 
 **/
#define RAYHDR_TBL_SIZE		49
#define CLUTFLAG_TBL_SIZE	49


#ifdef LANGUAGE_FORTRAN
/*********************************/
/* Define structures for FORTRAN */
/*********************************/

        /****************************************/
        /* Define PR Scan Status Block		*/
        /****************************************/

      STRUCTURE /PR_SCAN_STATUS/
	BYTE		missing
        BYTE		validity
        BYTE		qac
        BYTE		geoQuality
        BYTE		dataQuality
        BYTE		scOrient
        BYTE		acsMode
        BYTE		yawUpdateS
        BYTE		prMode
	BYTE		prStatus1
	BYTE		prStatus2
        REAL*4          fractOrbitN
      END STRUCTURE


        /****************************************/
        /* Define PR L1B & L1C Header		*/
        /****************************************/

      STRUCTURE /PR_CAL_COEF/
	REAL*4		transCoef
	REAL*4		receptCoef
	REAL*4		fcifIOchar(16)
      END STRUCTURE

      STRUCTURE /RAY_HEADER/
	INTEGER*2	rayStart
	INTEGER*2	raySize
	REAL*4		angle
	REAL*4		startBinDist
	REAL*4		rainThres1
	REAL*4		rainThres2
	REAL*4		transAntenna
	REAL*4		recvAntenna
	REAL*4		onewayAlongTrack
	REAL*4		onewayCrossTrack
	REAL*4		eqvWavelength
	REAL*4		radarConst
	REAL*4		prIntrDelay
	REAL*4		rangeBinSize
	REAL*4		logAveOffset
	BYTE		mainlobeEdge
	BYTE		sidelobeRange(3)
      END STRUCTURE


      STRUCTURE /L1B21_L1C21_HEADER/
	RECORD /PR_CAL_COEF/	prCalCoef
	RECORD /RAY_HEADER/	rayHdr(RAYHDR_TBL_SIZE)
      END STRUCTURE

        /****************************************/
        /* Define L1B-21 data structure.        */
        /****************************************/

      STRUCTURE /POWERS/
	INTEGER*2       radarTransPower
	REAL*4          transPulseWidth
      END STRUCTURE


      STRUCTURE /L1B_21_SWATHDATA/
        REAL*8                  scanTime
        REAL*4                  geolocation(2,49)
        RECORD /PR_SCAN_STATUS/ scanStatus
        RECORD /NAVIGATION/  	navigate
        RECORD /POWERS/         power
        INTEGER*2               systemNoise_scale(49)
	REAL*4			systemNoise(49)
	BYTE			sysNoiseWarnFlag(49)
        BYTE                    minEchoFlag(49)
        INTEGER*2               binStormHeight(2,49)
        INTEGER*2               binEllipsoid(49)
        INTEGER*2               binClutterFreeBottom(2,49)
        INTEGER*2               binDIDHmean(49)
        INTEGER*2               binDIDHtop(2,49)
        INTEGER*2               binDIDHbottom(2,49)
        REAL*4                  scLocalZenith(49)
	REAL*4			scRange(49)
	INTEGER*2               osBinStart(2,29)
	INTEGER*2		landOceanFlag(49)
        INTEGER*2		surfWarnFlag(49)
        INTEGER*2               binSurfPeak(49)
        INTEGER*2               normalSample_scale(140,49)
	REAL*4			normalSample(140,49)
        INTEGER*2               osSurf_scale(5,29)
	REAL*4			osSurf(5,29)
        INTEGER*2               osRain_scale(28,11)
	REAL*4			osRain(28,11)
      END STRUCTURE


        /****************************************/
        /* Define L1C-21 data structure.        */
        /* The L1C-21 has the same format as    */
        /* L1B-21.                              */
        /****************************************/


      STRUCTURE /L1C_21_SWATHDATA/
        REAL*8                  scanTime
        REAL*4                  geolocation(2,49)
        RECORD /PR_SCAN_STATUS/ scanStatus
        RECORD /NAVIGATION/     navigate
        RECORD /POWERS/         power
        INTEGER*2               systemNoise_scale(49)
        REAL*4                  systemNoise(49)
	BYTE			sysNoiseWarnFlag(49)
        BYTE                    minEchoFlag(49)
        INTEGER*2               binStormHeight(2,49)
        INTEGER*2               binEllipsoid(49)
        INTEGER*2               binClutterFreeBottom(2,49)
        INTEGER*2               binDIDHmean(49)
        INTEGER*2               binDIDHtop(2,49)
        INTEGER*2               binDIDHbottom(2,49)
        REAL*4                  scLocalZenith(49)
	REAL*4			scRange(49)
	INTEGER*2               osBinStart(2,29)
        INTEGER*2		landOceanFlag(49)
        INTEGER*2		surfWarnFlag(49)
        INTEGER*2               binSurfPeak(49)
        INTEGER*2               normalSample_scale(140,49)
        REAL*4                  normalSample(140,49)
        INTEGER*2               osSurf_scale(5,29)
        REAL*4                  osSurf(5,29)
        INTEGER*2               osRain_scale(28,11)
        REAL*4                  osRain(28,11)
      END STRUCTURE



        /****************************************/
        /* Define L2A-21 data structure.        */
        /****************************************/


      STRUCTURE /L2A_21_SWATHDATA/
	REAL*8			scanTime
	REAL*4			geolocation(2,49)
        RECORD /PR_SCAN_STATUS/	scanStatus
        RECORD /NAVIGATION/ 	navigate
	INTEGER*2		sigmaZero_scale(49)
	REAL*4			sigmaZero(49)
	INTEGER*2		pathAtten_scale(49)
	REAL*4			pathAtten(49)
	INTEGER*2		reliabFlag(49)
        REAL*4			reliabFactor(49)
	INTEGER*2		incAngle_scale(49)
	INTEGER*2		rainFlag(49)
	REAL*4			incAngle(49)
      END STRUCTURE


        /****************************************/
        /* Define L2A-23 data structure.        */
        /****************************************/


      STRUCTURE /L2A_23_SWATHDATA/
	REAL*8			scanTime
	REAL*4			geolocation(2,49)
	RECORD /PR_SCAN_STATUS/	scanStatus
	RECORD /NAVIGATION/	navigate
	BYTE			rainFlag(49)
	BYTE			old_type_rainType(49)
	INTEGER*2		rainType(49)
	BYTE			warmRain(49)
	BYTE			shallowRain(49)
	BYTE			status(49)
	INTEGER*2		rangeBinNum(49)
	INTEGER*2		binBBpeak(49)
  	INTEGER*2		HBB(49)
        REAL*4                  BBintensity(49)
	INTEGER*2		freezH(49)
  	INTEGER*2		stormH(49)
        REAL*4                  old_type_spare(49)
        INTEGER*2               spare(49)
	INTEGER*2		BBboundary(2,49)
	INTEGER*2		BBwidth(49)
	BYTE			BBstatus(49)
      END STRUCTURE


      STRUCTURE /L2A_23_R1_SWATHDATA/
        REAL*8                  scanTime
        INTEGER*2               geolocation_scale(2,49)
        REAL*4                  geolocation(2,49)
	BYTE			old_type_rainType(49)
	INTEGER*2		rainType(49)
        INTEGER*2               freezH(49)
        INTEGER*2               stormH(49)
      END STRUCTURE


        /****************************************/
        /* Define L2A-25 data structure.        */
        /****************************************/

      STRUCTURE /CFLAGS/
	BYTE			mainlobeEdge
	BYTE			sidelobeRange(3)
      END STRUCTURE

      STRUCTURE /CLUTTER_FLAGS/
	RECORD /CFLAGS/		clutFlag(CLUTFLAG_TBL_SIZE)
      END STRUCTURE


      STRUCTURE /L2A_25_SWATHDATA/
	REAL*8			scanTime
	REAL*4			geolocation(2,49)
	RECORD /PR_SCAN_STATUS/	scanStatus
	RECORD /NAVIGATION/	navigate
	REAL*4			scLocalZenith(49)
        INTEGER*2		rain_scale(80,49)
	REAL*4			rain(80,49)
        BYTE			reliab(80,49)
	INTEGER*2		correctZFactor_scale(80,49)
	REAL*4			correctZFactor(80,49)
	INTEGER*2		attenParmNode(5,49)
	INTEGER*2		attenParmAlpha_scale(5,49)
        REAL*4			attenParmAlpha(5,49)
	INTEGER*2		attenParmBeta_scale(49)
	REAL*4			attenParmBeta(49)
	INTEGER*2		ZRParmNode(5,49)
	INTEGER*2		parmNode(5,49)
	REAL*4			precipWaterParmA(5,49)
	REAL*4			precipWaterParmB(5,49)
	INTEGER*2		ZRParmA_scale(5,49)
	REAL*4			ZRParmA(5,49)
	INTEGER*2		ZRParmB_scale(5,49)
	REAL*4			ZRParmB(5,49)
	REAL*4			zmmax(49)
	INTEGER*2		rainFlag(49)
	INTEGER*2		rangeBinNum(7,49)
	INTEGER*2		rainAve_scale(2,49)
	REAL*4			rainAve(2,49)
	INTEGER*2		precipWaterSum_scale(49)
	REAL*4			precipWaterSum(49)
	INTEGER*2		weightW_scale(49)
	REAL*4			weightW(49)
	REAL*4			epsilon_0(49)
	INTEGER*2		method(49)
	REAL*4			epsilon(49)
	REAL*4			zeta(2,49)
	REAL*4			zeta_mn(2,49)
	REAL*4			zeta_sd(2,49)
	REAL*4			xi(2,49)
	REAL*4			sigmaZero(49)
	REAL*4			freezH(49)
	INTEGER*2		thickThPIZ(49)
	REAL*4			nubfCorrectFactor(2,49)
	INTEGER*2		qualityFlag(49)
        REAL*4                  nearSurfRain(49)
        REAL*4                  nearSurfZ(49)
        REAL*4                  pia2a25(49)
        REAL*4                  e_SurfRain(49)
        REAL*4                  pia(3,49)
        REAL*4                  errorRain(49)
        REAL*4                  errorZ(49)
	REAL*4			spare(2,49)
	INTEGER*2		rainType(49)
      END STRUCTURE

/*2N begin*/

      STRUCTURE /L2N_25_SWATHDATA/
        REAL*8                  scanTime
        REAL*4                  geolocation(2,1)
        RECORD /PR_SCAN_STATUS/ scanStatus
        RECORD /NAVIGATION/     navigate
	REAL*4			scLocalZenith(1)
        INTEGER*2               rain_scale(80,1)
        REAL*4                  rain(80,1)
        BYTE                    reliab(80,1)
        INTEGER*2               correctZFactor_scale(80,1)
        REAL*4                  correctZFactor(80,1)
        INTEGER*2               attenParmNode(5,1)
        INTEGER*2               attenParmAlpha_scale(5,1)
        REAL*4                  attenParmAlpha(5,1)
        INTEGER*2               attenParmBeta_scale(1)
        REAL*4                  attenParmBeta(1)
        INTEGER*2               ZRParmNode(5,1)
        INTEGER*2               parmNode(5,1)
        REAL*4                  precipWaterParmA(5,1)
        REAL*4                  precipWaterParmB(5,1)
        INTEGER*2               ZRParmA_scale(5,1)
        REAL*4                  ZRParmA(5,1)
        INTEGER*2               ZRParmB_scale(5,1)
        REAL*4                  ZRParmB(5,1)
        REAL*4                  zmmax(1)
        INTEGER*2               rainFlag(1)
        INTEGER*2               rangeBinNum(7,1)
        INTEGER*2               rainAve_scale(2,1)
        REAL*4                  rainAve(2,1)
        INTEGER*2               precipWaterSum_scale(1)
        REAL*4                  precipWaterSum(1)
        INTEGER*2               weightW_scale(1)
        REAL*4                  weightW(1)
        REAL*4                  epsilon_0(1)
        INTEGER*2               method(1)
        REAL*4                  epsilon(1)
        REAL*4                  zeta(2,1)
        REAL*4                  zeta_mn(2,1)
        REAL*4                  zeta_sd(2,1)
        REAL*4                  xi(2,1)
        REAL*4                  sigmaZero(1)
        REAL*4                  freezH(1)
        INTEGER*2               thickThPIZ(1)
        REAL*4                  nubfCorrectFactor(2,1)
        INTEGER*2               qualityFlag(1)
        REAL*4                  nearSurfRain(1)
        REAL*4                  nearSurfZ(1)
        REAL*4                  pia2a25(1)
        REAL*4                  e_SurfRain(1)
        REAL*4                  pia(3,1)
        REAL*4                  errorRain(1)
        REAL*4                  errorZ(1)
        REAL*4                  spare(2,1)
        INTEGER*2               rainType(1)
      END STRUCTURE

/*2N end*/

      STRUCTURE /L2A_25_R1_SWATHDATA/
        REAL*8                  scanTime
        INTEGER*2               geolocation_scale(2,49)
        REAL*4                  geolocation(2,49)
        INTEGER*2               nearSurfRain_scale(49)
        REAL*4                  nearSurfRain(49)
        INTEGER*2               e_SurfRain_scale(49)
        REAL*4                  e_SurfRain(49)
        INTEGER*2               nearSurfZ_scale(49)
        REAL*4                  nearSurfZ(49)
        INTEGER                 nearSurfBin(49)
      END STRUCTURE


      STRUCTURE /L2A_25_R2_SWATHDATA/
        REAL*8                  scanTime
        INTEGER*2               geolocation_scale(2,49)
        REAL*4                  geolocation(2,49)
        INTEGER*2               rain_scale(20,49)
        REAL*4                  rain(20,49)
      END STRUCTURE


	/****************************************/
	/* Define L2B-31 data structure.	*/
	/****************************************/

      STRUCTURE /L2B_31_SWATHDATA/
        REAL*8			scanTime
        REAL*4			geolocation(2,49)
        RECORD /PR_SCAN_STATUS/	scanStatus
        RECORD /NAVIGATION/     navigate
	INTEGER*2		dHat_scale(49)
        REAL*4			dHat(49)
        INTEGER*2		sigmaDHat_scale(49)
	REAL*4			sigmaDHat(49)
        INTEGER*2		epsilon_scale(49)
        REAL*4			epsilon(49)
        INTEGER*2		sigmaEpsilon_scale(49)
        REAL*4			sigmaEpsilon(49)
        INTEGER*2		rHat_scale(80,49)
        REAL*4			rHat(80,49)
        INTEGER*2		sigmaRHat_scale(80,49)
        REAL*4			sigmaRHat(80,49)
        INTEGER*2		pia_scale(49)
        REAL*4			pia(49)
        INTEGER*2		sigmaPIA_scale(49)
        REAL*4			sigmaPIA(49)
        INTEGER*2		tmiPIA_scale(49)
        REAL*4			tmiPIA(49)
        INTEGER*2		sigmaTMIpia_scale(49)
        REAL*4			sigmaTMIpia(49)
	INTEGER*2		rrSurf_scale(49)
	REAL*4			rrSurf(49)
	INTEGER*2		sigmaRRsurf_scale(49)
	REAL*4			sigmaRRsurf(49)
	REAL*4			latentHeatHH(13,49)
	REAL*4			spare(4,49)
      END STRUCTURE

        /****************************************/
        /* Define L3A-25 data structure.        */
        /****************************************/

      STRUCTURE /L3A_25_PLANETGRID1/
	  REAL*4	rainMean1(16,72,6)
	  REAL*4	rainDev1(16,72,6)
	  REAL*4	convRainMean1(16,72,6)
	  REAL*4	convRainDev1(16,72,6)
	  REAL*4	stratRainMean1(16,72,6)
	  REAL*4	stratRainDev1(16,72,6)

          REAL*4       shallowRainMean1(16,72)
          REAL*4       shallowRainDev1(16,72)
          REAL*4       shallowIsoRainMean1(16,72)
          REAL*4       shallowIsoRainDev1(16,72)

	  REAL*4	zmMean1(16,72,6)
	  REAL*4	zmDev1(16,72,6)
          REAL*4	convZmMean1(16,72,6)
          REAL*4	convZmDev1(16,72,6)
          REAL*4	stratZmMean1(16,72,6)
          REAL*4	stratZmDev1(16,72,6)
	  REAL*4	ztMean1(16,72,6)
	  REAL*4	ztDev1(16,72,6)
          REAL*4	convZtMean1(16,72,6)
          REAL*4	convZtDev1(16,72,6)
          REAL*4	stratZtMean1(16,72,6)
          REAL*4	stratZtDev1(16,72,6)

          REAL*4	piaSrtMean(16,72,5)
          REAL*4	piaSrtDev(16,72,5)
          REAL*4	piaHbMean(16,72,5)
          REAL*4	piaHbDev(16,72,5)
          REAL*4	pia0Mean(16,72,5)
          REAL*4	pia0Dev(16,72,5)
          REAL*4	pia2a25Mean(16,72,5)
          REAL*4	pia2a25Dev(16,72,5)

          REAL*4	piaSrtssMean(16,72,5)
          REAL*4	piaSrtssDev(16,72,5)
          REAL*4	piaHbssMean(16,72,5)
          REAL*4	piaHbssDev(16,72,5)
          REAL*4	pia0ssMean(16,72,5)
          REAL*4	pia0ssDev(16,72,5)
          REAL*4	pia2a25ssMean(16,72,5)
          REAL*4	pia2a25ssDev(16,72,5)

          REAL*4	stormHtMean(16,72,3)
          REAL*4	stormHtDev(16,72,3)

          REAL*4	xiMean(16,72)
          REAL*4	xiDev(16,72)
          REAL*4	nubfCorFacMean(16,72)
          REAL*4	nubfCorFacDev(16,72)
          REAL*4	bbHtMean(16,72)
          REAL*4	bbHtDev(16,72)

          REAL*4       epsilon0StratMean1(16,72)
          REAL*4       epsilon0StratDev1(16,72)
          REAL*4       epsilon0ConvMean1(16,72)
          REAL*4       epsilon0ConvDev1(16,72)
          REAL*4       epsilonStratMean1(16,72)
          REAL*4       epsilonStratDev1(16,72)
          REAL*4       epsilonConvMean1(16,72)
          REAL*4       epsilonConvDev1(16,72)

          REAL*4	epsilonMean1(16,72)
          REAL*4	epsilonDev1(16,72)
          REAL*4	surfRainMean1(16,72)
          REAL*4	surfRainDev1(16,72)
          REAL*4	e_surfRainMean1(16,72)
          REAL*4	e_surfRainDev1(16,72)
          REAL*4	bbZmaxMean1(16,72)
          REAL*4	bbZmaxDev1(16,72)

          REAL*4       bbNadirHtMean1(16,72)
          REAL*4       bbNadirHtDev1(16,72)
          REAL*4       bbNadirWidthMean1(16,72)
          REAL*4       bbNadirWidthDev1(16,72)
          REAL*4       bbNadirZmaxMean1(16,72)
          REAL*4       bbNadirZmaxDev1(16,72)

          REAL*4	bbwidthMean1(16,72)
          REAL*4	bbwidthDev1(16,72)
          REAL*4	sdepthMean1(16,72)
          REAL*4	sdepthDev1(16,72)
          REAL*4	surfRainAllMean1(16,72)
          REAL*4	surfRainAllDev1(16,72)

          REAL*4       surfRainConvMean1(16,72)
          REAL*4       surfRainConvDev1(16,72)
          REAL*4       surfRainStratMean1(16,72)
          REAL*4       surfRainStratDev1(16,72)

          REAL*4       e_surfRainConvMean1(16,72)
          REAL*4       e_surfRainConvDev1(16,72)
          REAL*4       e_surfRainStratMean1(16,72)
          REAL*4       e_surfRainStratDev1(16,72)

          REAL*4       rzStratA1(16,72,2)
          REAL*4       rzStratB1(16,72,2)
          REAL*4       rzConvA1(16,72,2)
          REAL*4       rzConvB1(16,72,2)
          REAL*4       rzA1(16,72,2)
          REAL*4       rzB1(16,72,2)

	  INTEGER*4		ttlPix1(16,72)
          INTEGER*4         bbPixNum1(16,72)
          INTEGER*4         bbNadirPix1(16,72)
          INTEGER*4         wrainPix1(16,72)
          INTEGER*4         surfRainPix1(16,72)
          INTEGER*4         surfRainStratPix1(16,72)
          INTEGER*4         surfRainConvPix1(16,72)
          INTEGER*4         e_surfRainPix1(16,72)
          INTEGER*4         e_surfRainStratPix1(16,72)
          INTEGER*4         e_surfRainConvPix1(16,72)
          INTEGER*4         epsilonPix1(16,72)
          INTEGER*4         epsilon0StratPix1(16,72)
          INTEGER*4         epsilon0ConvPix1(16,72)
          INTEGER*4         epsilonStratPix1(16,72)
          INTEGER*4         epsilonConvPix1(16,72)
          INTEGER*4         surfRainAllPix1(16,72)
	  INTEGER*4		rainPix1(16,72,6)
	  INTEGER*4		convRainPix1(16,72,6)
	  INTEGER*4		stratRainPix1(16,72,6)
          INTEGER*2		ttlAnglePix1(16,72,4)
          INTEGER*2		rainAnglePix1(16,72,4)
          INTEGER*4         rzStratPix1(16,72,2)
          INTEGER*4         rzConvPix1(16,72,2)
          INTEGER*4         rzPix1(16,72,2)
          INTEGER*4         shallowRainPix1(16,72)
          INTEGER*4         shallowIsoRainPix1(16,72)
	  INTEGER*2		stormHH(16,72,30)
	  INTEGER*2		convStormHH(16,72,30)
	  INTEGER*2		stratStormHH(16,72,30)
	  INTEGER*2		BBHH(16,72,30)

          INTEGER*2         bbNadirWidthH(16,72,30)
          INTEGER*2         bbNadirHH(16,72,30)
          INTEGER*2         bbNadirZmaxH(16,72,30)

	  INTEGER*2		snowIceLH(16,72,30)
	  INTEGER*2		bbZmaxH(16,72,30)
	  INTEGER*2		epsilonH(16,72,30)

          INTEGER*2         epsilon0StratH(16,72,30)
          INTEGER*2         epsilon0ConvH(16,72,30)
          INTEGER*2         epsilonStratH(16,72,30)
          INTEGER*2         epsilonConvH(16,72,30)

	  INTEGER*2		surfRainH(16,72,30)
	  INTEGER*2		e_surfRainH(16,72,30)
	  INTEGER*2		surfRainAllH(16,72,30)

          INTEGER*2         surfRainConvH(16,72,30)
          INTEGER*2         surfRainStratH(16,72,30)
          INTEGER*2         e_surfRainConvH(16,72,30)
          INTEGER*2         e_surfRainStratH(16,72,30)

	  INTEGER*2		zmH(16,72,30,6)
	  INTEGER*2		convZmH(16,72,30,6)
	  INTEGER*2		stratZmH(16,72,30,6)
	  INTEGER*2		ztH(16,72,30,6)
	  INTEGER*2		convZtH(16,72,30,6)
	  INTEGER*2		stratZtH(16,72,30,6)
	  INTEGER*2		rainH(16,72,30,6)
	  INTEGER*2		convRainH(16,72,30,6)
	  INTEGER*2		stratRainH(16,72,30,6)

          INTEGER*2         shallowRainH(16,72,30)
          INTEGER*2         shallowIsoRainH(16,72,30)

	  INTEGER*2		piaSrtH(16,72,30,5)
	  INTEGER*2		piaHbH(16,72,30,5)
	  INTEGER*2		pia0H(16,72,30,5)
	  INTEGER*2		pia2a25H(16,72,30,5)

	  INTEGER*2		piaSrtssH(16,72,30,5)
	  INTEGER*2		piaHbssH(16,72,30,5)
	  INTEGER*2		pia0ssH(16,72,30,5)
	  INTEGER*2		pia2a25ssH(16,72,30,5)

	  INTEGER*2		zmGradH(16,72,30,3)
	  INTEGER*2		xiH(16,72,30)
          INTEGER*2		nubfH(16,72,30)
          INTEGER*2		zpzmH(16,72,30)
	  REAL*4	rainCCoef(16,72,3)
	  REAL*4	convRainCCoef(16,72,3) 
	  REAL*4	stratRainCCoef(16,72,3)
	  REAL*4	stormHtZmCCoef(16,72)
	  REAL*4	piaCCoef(16,72,5,6)
	  REAL*4	xiZmCCoef(16,72)

	  REAL*4	piaSrtPix(16,72,5)
	  REAL*4	piaHbPix(16,72,5)
	  REAL*4	pia0Pix(16,72,5)
	  REAL*4	pia2a25Pix(16,72,5)

	  REAL*4	piaSrtssPix(16,72,5)
	  REAL*4	piaHbssPix(16,72,5)
	  REAL*4	pia0ssPix(16,72,5)
	  REAL*4	pia2a25ssPix(16,72,5)

	  REAL*4	rainCCoefPix(16,72,3)
	  REAL*4	stratCCoefPix(16,72,3)
	  REAL*4	convCCoefPix(16,72,3)

	  REAL*4	piaCCoefPix(16,72,5)

	END STRUCTURE	





      STRUCTURE /L3A_25_PLANETGRID2/
	  REAL*4	rainMean2(148,720,4)
	  REAL*4	rainDev2(148,720,4)
	  REAL*4	convRainMean2(148,720,4)
	  REAL*4	convRainDev2(148,720,4)
	  REAL*4	stratRainMean2(148,720,4)
	  REAL*4	stratRainDev2(148,720,4)

          REAL*4       shallowRainMean2(148,720)
          REAL*4       shallowRainDev2(148,720)
          REAL*4       shallowIsoRainMean2(148,720)
          REAL*4       shallowIsoRainDev2(148,720)

          REAL*4	zmMean2(148,720,4)
          REAL*4	convZmMean2(148,720,4)
          REAL*4	stratZmMean2(148,720,4)
          REAL*4	ztMean2(148,720,4)
          REAL*4	convZtMean2(148,720,4)
          REAL*4	stratZtMean2(148,720,4)
          REAL*4	stormHeightMean(148,720,3)
          REAL*4	stormHeightDev2(148,720,3)
          REAL*4	bbHeightMean(148,720)

          REAL*4       epsilon0StratMean2(148,720)
          REAL*4       epsilon0StratDev2(148,720)
          REAL*4       epsilon0ConvMean2(148,720)
          REAL*4       epsilon0ConvDev2(148,720)
          REAL*4       epsilonStratMean2(148,720)
          REAL*4       epsilonStratDev2(148,720)
          REAL*4       epsilonConvMean2(148,720)
          REAL*4       epsilonConvDev2(148,720)

          REAL*4	surfRainMean2(148,720)
          REAL*4	surfRainDev2(148,720)
          REAL*4	e_surfRainMean2(148,720)
          REAL*4	e_surfRainDev2(148,720)
          REAL*4	bbZmaxMean2(148,720)
          REAL*4	bbZmaxDev2(148,720)
          REAL*4	sdepthMean2(148,720)
          REAL*4	sdepthDev2(148,720)
          REAL*4	bbHeightDev2(148,720)
          REAL*4	surfRainAllMean2(148,720)
          REAL*4	surfRainAllDev2(148,720)

          REAL*4       surfRainConvMean2(148,720)
          REAL*4       surfRainConvDev2(148,720)
          REAL*4       surfRainStratMean2(148,720)
          REAL*4       surfRainStratDev2(148,720)

          REAL*4       e_surfRainConvMean2(148,720)
          REAL*4       e_surfRainConvDev2(148,720)
          REAL*4       e_surfRainStratMean2(148,720)
          REAL*4       e_surfRainStratDev2(148,720)

          REAL*4       rzStratA2(148,720,2)
          REAL*4       rzStratB2(148,720,2)
          REAL*4       rzConvA2(148,720,2)
          REAL*4       rzConvB2(148,720,2)
          REAL*4       rzA2(148,720,2)
          REAL*4       rzB2(148,720,2)

	  INTEGER*4		ttlPix2(148,720)
	  INTEGER*4         bbPixNum2(148,720)
	  INTEGER*4         wrainPix2(148,720)
	  INTEGER*4         surfRainPix2(148,720)
          INTEGER*4         surfRainConvPix2(148,720)
          INTEGER*4         surfRainStratPix2(148,720)
	  INTEGER*4         e_surfRainPix2(148,720)
          INTEGER*4         e_surfRainConvPix2(148,720)
          INTEGER*4         e_surfRainStratPix2(148,720)
	  INTEGER*4         surfRainAllPix2(148,720)
	  INTEGER*4		rainPix2(148,720,4)
	  INTEGER*4		convRainPix2(148,720,4)
	  INTEGER*4		stratRainPix2(148,720,4)
          INTEGER*4         shallowRainPix2(148,720)
          INTEGER*4         shallowIsoRainPix2(148,720)
          INTEGER*4         epsilon0StratPix2(148,720)
          INTEGER*4         epsilon0ConvPix2(148,720)
          INTEGER*4         epsilonStratPix2(148,720)
          INTEGER*4         epsilonConvPix2(148,720)
          INTEGER*4         rzStratPix2(148,720,2)
          INTEGER*4         rzConvPix2(148,720,2)
          INTEGER*4         rzPix2(148,720,2)
	END STRUCTURE




      STRUCTURE /L3A_25_GRID/
	RECORD /L3A_25_PLANETGRID1/	grid1
	RECORD /L3A_25_PLANETGRID2/	grid2
      END STRUCTURE



	/****************************************/
	/* Define L3A-26 data structure.	*/
	/****************************************/


      STRUCTURE /L3A_26_GRID/
        INTEGER*4		ttlCount(16,72)
        INTEGER*4		rainCount(16,72,4)
        INTEGER*4		zeroOrderpDf(16,72,25,4,6)
        INTEGER*4		hbpDf(16,72,25,4,6)
        INTEGER*4		pDf2A25(16,72,25,4,6)
        REAL*4			zeroOrderFit(16,72,4,3,6)
        REAL*4			hbFit(16,72,4,3,6)
        REAL*4			fit2A25(16,72,4,3,6)
        REAL*4		        reliabOrderFit(16,72,4,6)
        REAL*4		        reliabHBfit(16,72,4,6)
        REAL*4	      	        reliab2A25fit(16,72,4,6)
        REAL*4                  rainMeanTH(16,72,4)
      END STRUCTURE

      STRUCTURE /L3A_27_GRID/
        REAL*4                  LatentHeating(148,720,19)
      END STRUCTURE

	/****************************************/
        /* Define L3B-31 data structure.        */
        /****************************************/

      STRUCTURE /L3B_31_GRID/
        REAL*4                  sfcrainTMI(16,72) 
        REAL*4                  convectRain(16,72) 
        REAL*4                  cloudWater(16,72,14)
        REAL*4                  rainWaterTMI(16,72,14) 
        REAL*4                  cloudIce(16,72,14)
        REAL*4                  graupel(16,72,14)
        REAL*4                  latentHeat(16,72,14) 
        REAL*4                  sfcrainCOMB(16,72) 
        REAL*4                  rainWaterCOMB(16,72,14) 
        REAL*4                  sfcrainTMIoverlap(16,72) 
        REAL*4                  convectRainoverlap(16,72) 
        REAL*4                  sfcrainCOMBoverlap(16,72) 
        REAL*4                  surfAdjRatio(16,72)
        REAL*4                  surfAdjRatioverlap(16,72) 
        REAL*4                  surfRainfall(16,72) /*tk 4.9.1 and below*/
        REAL*4                  rainWater(16,72,14) /*tk 4.9.1 and below*/
        REAL*4                  profAdjRatio(16,72,14) /*tk 4.9.1 and below*/
      END STRUCTURE

#else
#include "IO.h"
/***************************/
/* Define structures for C */
/***************************/

      typedef struct
      {
	int8		missing;
        int8            validity;
        int8            qac;
        int8	        geoQuality;
        int8            dataQuality;
        int8            scOrient;
        int8            acsMode;
        int8            yawUpdateS;
        int8            prMode;
	int8		prStatus1;
	int8		prStatus2;
        float32         fractOrbitN;
      } PR_SCAN_STATUS;


        /****************************************/
        /* Define PR L1B & L1C Header           */
        /****************************************/

      typedef struct
      {
	float32		transCoef;
	float32		receptCoef;
	float32		fcifIOchar[16];
      } PR_CAL_COEF;

      typedef struct
      {
        int16		rayStart;
        int16		raySize;
        float32		angle;
        float32		startBinDist;
        float32         rainThres1;
        float32		rainThres2;
        float32		transAntenna;
        float32		recvAntenna;
        float32		onewayAlongTrack;
        float32		onewayCrossTrack;
        float32		eqvWavelength;
        float32		radarConst;
        float32		prIntrDelay;
        float32		rangeBinSize;
        float32		logAveOffset;
        int8		mainlobeEdge;
        int8		sidelobeRange[3];
      } RAY_HEADER;


      typedef struct
      {
	PR_CAL_COEF	prCalCoef;
        RAY_HEADER	rayHdr[RAYHDR_TBL_SIZE];
      } L1B21_L1C21_HEADER;



        /****************************************/
        /* Define L1B-21 data structure.        */
        /****************************************/


      typedef struct
      {
        int16		radarTransPower;
        float32		transPulseWidth;
      } POWERS;


      typedef struct
      {
	float64		scanTime;
	float32		geolocation[49][2];
	PR_SCAN_STATUS	scanStatus;
	NAVIGATION	navigate;
	POWERS		power;
	int16		systemNoise_scale[49];
	float32		systemNoise[49];
	int8		sysNoiseWarnFlag[49];
        int8            minEchoFlag[49];
        int16           binStormHeight[49][2];
	int16           binEllipsoid[49];
	int16           binClutterFreeBottom[49][2];
	int16           binDIDHmean[49];
	int16           binDIDHtop[49][2];
	int16           binDIDHbottom[49][2];
        float32         scLocalZenith[49];
	float32		scRange[49];
	int16           osBinStart[29][2];
        int16           landOceanFlag[49];
        int16           surfWarnFlag[49];
	int16		binSurfPeak[49];
	int16		normalSample_scale[49][140];
	float32		normalSample[49][140];
	int16		osSurf_scale[29][5];
	float32		osSurf[29][5];
	int16		osRain_scale[11][28];
	float32		osRain[11][28];
      } L1B_21_SWATHDATA;


        /****************************************/
        /* Define L1C-21 data structure.        */
	/* The L1C-21 has the same format as 	*/
	/* L1B-21.				*/
        /****************************************/

      typedef struct
      {
        float64         scanTime;
        float32         geolocation[49][2];
        PR_SCAN_STATUS  scanStatus;
        NAVIGATION      navigate;
        POWERS          power;
        int16           systemNoise_scale[49];
        float32         systemNoise[49];
	int8		sysNoiseWarnFlag[49];
        int8            minEchoFlag[49];
        int16           binStormHeight[49][2];
	int16           binEllipsoid[49];
	int16           binClutterFreeBottom[49][2];
	int16           binDIDHmean[49];
	int16           binDIDHtop[49][2];
	int16           binDIDHbottom[49][2];
        float32         scLocalZenith[49];
	float32		scRange[49];
	int16           osBinStart[29][2];
	int16		landOceanFlag[49];
	int16		surfWarnFlag[49];
        int16           binSurfPeak[49];
        int16           normalSample_scale[49][140];
        float32         normalSample[49][140];
        int16           osSurf_scale[29][5];
        float32         osSurf[29][5];
        int16           osRain_scale[11][28];
        float32         osRain[11][28];
      } L1C_21_SWATHDATA;


        /****************************************/
        /* Define L2A-21 data structure.        */
        /****************************************/


      typedef struct
      {
	float64		scanTime;
	float32		geolocation[49][2];
        PR_SCAN_STATUS  scanStatus;
        NAVIGATION   	navigate;
	int16		sigmaZero_scale[49];
	float32		sigmaZero[49];
	int16		pathAtten_scale[49];
	float32		pathAtten[49];
	int16		reliabFlag[49];
	float32		reliabFactor[49];
	int16		incAngle_scale[49];
	int16		rainFlag[49];
	float32		incAngle[49];
      } L2A_21_SWATHDATA;


        /****************************************/
        /* Define L2A-23 data structure.        */
        /****************************************/

      typedef struct
      {
	float64		scanTime;
	float32		geolocation[49][2];
	PR_SCAN_STATUS	scanStatus;
	NAVIGATION	navigate;
	int8		rainFlag[49];
	int8		old_type_rainType[49];
	int16		rainType[49];
	int8		warmRain[49];
	int8		shallowRain[49];
	int8		status[49];
	int16		rangeBinNum[49];
	int16		binBBpeak[49];
  	int16		HBB[49];
	float32         BBintensity[49];
	int16		freezH[49];
	int16		stormH[49];
	float32         old_type_spare[49];
	int16           spare[49];
  	int16		BBboundary[49][2];
  	int16		BBwidth[49];
  	int8		BBstatus[49];
      } L2A_23_SWATHDATA;

      typedef struct
      {
	float64		scanTime;
	int16           geolocation_scale[49][2];
	float32		geolocation[49][2];
	int8		old_type_rainType[49];
	int16		rainType[49];
	int16		freezH[49];
	int16		stormH[49];
      } L2A_23_R1_SWATHDATA;


        /****************************************/
        /* Define L2A-25 data structure.        */
        /****************************************/

      typedef struct
      {
	int8		mainlobeEdge;
	int8		sidelobeRange[3];	
      } CFLAGS;

      typedef struct
      {
	CFLAGS		clutFlag[CLUTFLAG_TBL_SIZE];
      } CLUTTER_FLAGS;

      typedef struct
      {
	float64		scanTime;
	float32		geolocation[49][2];
	PR_SCAN_STATUS	scanStatus;
	NAVIGATION	navigate;
	float32		scLocalZenith[49];
        int16           rain_scale[49][80];
	float32		rain[49][80];
        int8		reliab[49][80];
	int16		correctZFactor_scale[49][80];
	float32		correctZFactor[49][80];
	int16		attenParmNode[49][5];
	int16		attenParmAlpha_scale[49][5];
	float32		attenParmAlpha[49][5];
	int16		attenParmBeta_scale[49];
	float32		attenParmBeta[49];
	int16		ZRParmNode[49][5];
	int16		parmNode[49][5];
	float32		precipWaterParmA[49][5];
	float32		precipWaterParmB[49][5];
	int16		ZRParmA_scale[49][5];
	float32		ZRParmA[49][5];
	int16		ZRParmB_scale[49][5];
	float32		ZRParmB[49][5];
	float32		zmmax[49];
	int16		rainFlag[49];
	int16		rangeBinNum[49][7];
	int16		rainAve_scale[49][2];
	float32		rainAve[49][2];
	int16		precipWaterSum_scale[49];
	float32		precipWaterSum[49];
	int16		weightW_scale[49];
	float32		weightW[49];
	float32		epsilon_0[49];
	int16		method[49];
	float32		epsilon[49];
	float32		zeta[49][2];
	float32		zeta_mn[49][2];
	float32		zeta_sd[49][2];
	float32		xi[49][2];
	float32		sigmaZero[49];
	float32		freezH[49];
	int16		thickThPIZ[49];
	float32		nubfCorrectFactor[49][2];
	int16		qualityFlag[49];	
	float32         nearSurfRain[49];
	float32         nearSurfZ[49];
	float32         pia2a25[49];
	float32         e_SurfRain[49];
	float32         pia[49][3];
	float32         errorRain[49];
	float32         errorZ[49];
	float32		spare[49][2];
        int16           rainType[49];
      } L2A_25_SWATHDATA;


/*2N begin*/

      typedef struct
      {
        float64         scanTime;
        float32         geolocation[1][2];
        PR_SCAN_STATUS  scanStatus;
        NAVIGATION      navigate;
	float32		scLocalZenith[1];
        int16           rain_scale[1][80];
        float32         rain[1][80];
        int8            reliab[1][80];
        int16           correctZFactor_scale[1][80];
        float32         correctZFactor[1][80];
        int16           attenParmNode[1][5];
        int16           attenParmAlpha_scale[1][5];
        float32         attenParmAlpha[1][5];
        int16           attenParmBeta_scale[1];
        float32         attenParmBeta[1];
        int16           ZRParmNode[1][5];
        int16           parmNode[1][5];
        float32         precipWaterParmA[1][5];
        float32         precipWaterParmB[1][5];
        int16           ZRParmA_scale[1][5];
        float32         ZRParmA[1][5];
        int16           ZRParmB_scale[1][5];
        float32         ZRParmB[1][5];
        float32         zmmax[1];
        int16           rainFlag[1];
        int16           rangeBinNum[1][7];
        int16           rainAve_scale[1][2];
        float32         rainAve[1][2];
        int16           precipWaterSum_scale[1];
        float32         precipWaterSum[1];
        int16           weightW_scale[1];
        float32         weightW[1];
        float32         epsilon_0[1];
        int16           method[1];
        float32         epsilon[1];
        float32         zeta[1][2];
        float32         zeta_mn[1][2];
        float32         zeta_sd[1][2];
        float32         xi[1][2];
        float32         sigmaZero[1];
        float32         freezH[1];
        int16           thickThPIZ[1];
        float32         nubfCorrectFactor[1][2];
        int16           qualityFlag[1];
        float32         nearSurfRain[1];
        float32         nearSurfZ[1];
        float32         pia2a25[1];
        float32         e_SurfRain[1];
        float32         pia[1][3];
        float32         errorRain[1];
        float32         errorZ[1];
        float32         spare[1][2];
        int16           rainType[1];
      } L2N_25_SWATHDATA;

/*2N end*/

      typedef struct
      {
	float64		scanTime;
	int16           geolocation_scale[49][2];
	float32		geolocation[49][2];
	int16           nearSurfRain_scale[49];
	float32         nearSurfRain[49];
	int16           e_SurfRain_scale[49];
	float32         e_SurfRain[49];
	int16           nearSurfZ_scale[49];
	float32         nearSurfZ[49];
        int8            nearSurfBin[49];
      } L2A_25_R1_SWATHDATA;


      typedef struct
      {
	float64		scanTime;
	int16           geolocation_scale[49][2];
	float32		geolocation[49][2];
        int16           rain_scale[49][20];
	float32		rain[49][20];
      } L2A_25_R2_SWATHDATA;


      typedef struct
      {
	float64		scanTime;
	float32		geolocation[49][2];
	PR_SCAN_STATUS	scanStatus;
	NAVIGATION	navigate;
	int16		dHat_scale[49];
	float32		dHat[49];
	int16		sigmaDHat_scale[49];
	float32		sigmaDHat[49];
	int16		epsilon_scale[49];
	float32		epsilon[49];
	int16		sigmaEpsilon_scale[49];
	float32		sigmaEpsilon[49];
	int16		rHat_scale[49][80];
	float32		rHat[49][80];
	int16		sigmaRHat_scale[49][80];
	float32		sigmaRHat[49][80];
	int16		pia_scale[49];
	float32		pia[49];
	int16		sigmaPIA_scale[49];
	float32		sigmaPIA[49];
	int16		tmiPIA_scale[49];
	float32		tmiPIA[49];
	int16		sigmaTMIpia_scale[49];
	float32		sigmaTMIpia[49];
	int16		rrSurf_scale[49];
	float32		rrSurf[49];
	int16		sigmaRRsurf_scale[49];
	float32		sigmaRRsurf[49];
	float32		latentHeatHH[49][13];
	float32		spare[49][4];
      } L2B_31_SWATHDATA;


        /****************************************/
        /* Define L3A-25 data structure.        */
        /****************************************/

 
	typedef struct
        {
	  float32	rainMean1[6][72][16];
	  float32	rainDev1[6][72][16];
	  float32	convRainMean1[6][72][16];
	  float32	convRainDev1[6][72][16];
	  float32	stratRainMean1[6][72][16];
	  float32	stratRainDev1[6][72][16];

          float32       shallowRainMean1[72][16];
          float32       shallowRainDev1[72][16];
          float32       shallowIsoRainMean1[72][16];
          float32       shallowIsoRainDev1[72][16];

	  float32	zmMean1[6][72][16];
	  float32	zmDev1[6][72][16];
          float32	convZmMean1[6][72][16];
          float32	convZmDev1[6][72][16];
          float32	stratZmMean1[6][72][16];
          float32	stratZmDev1[6][72][16];
	  float32	ztMean1[6][72][16];
	  float32	ztDev1[6][72][16];
          float32	convZtMean1[6][72][16];
          float32	convZtDev1[6][72][16];
          float32	stratZtMean1[6][72][16];
          float32	stratZtDev1[6][72][16];

          float32	piaSrtMean[5][72][16];
          float32	piaSrtDev[5][72][16];
          float32	piaHbMean[5][72][16];
          float32	piaHbDev[5][72][16];
          float32	pia0Mean[5][72][16];
          float32	pia0Dev[5][72][16];
          float32	pia2a25Mean[5][72][16];
          float32	pia2a25Dev[5][72][16];

          float32	piaSrtssMean[5][72][16];
          float32	piaSrtssDev[5][72][16];
          float32	piaHbssMean[5][72][16];
          float32	piaHbssDev[5][72][16];
          float32	pia0ssMean[5][72][16];
          float32	pia0ssDev[5][72][16];
          float32	pia2a25ssMean[5][72][16];
          float32	pia2a25ssDev[5][72][16];

          float32	stormHtMean[3][72][16];
          float32	stormHtDev[3][72][16];

          float32	xiMean[72][16];
          float32	xiDev[72][16];
          float32	nubfCorFacMean[72][16];
          float32	nubfCorFacDev[72][16];
          float32	bbHtMean[72][16];
          float32	bbHtDev[72][16];

          float32       epsilon0StratMean1[72][16];
          float32       epsilon0StratDev1[72][16];
          float32       epsilon0ConvMean1[72][16];
          float32       epsilon0ConvDev1[72][16];
          float32       epsilonStratMean1[72][16];
          float32       epsilonStratDev1[72][16];
          float32       epsilonConvMean1[72][16];
          float32       epsilonConvDev1[72][16];

          float32	epsilonMean1[72][16];
          float32	epsilonDev1[72][16];
          float32	surfRainMean1[72][16];
          float32	surfRainDev1[72][16];
          float32	e_surfRainMean1[72][16];
          float32	e_surfRainDev1[72][16];
          float32	bbZmaxMean1[72][16];
          float32	bbZmaxDev1[72][16];

          float32       bbNadirHtMean1[72][16];
          float32       bbNadirHtDev1[72][16];
          float32       bbNadirWidthMean1[72][16];
          float32       bbNadirWidthDev1[72][16];
          float32       bbNadirZmaxMean1[72][16];
          float32       bbNadirZmaxDev1[72][16];

          float32	bbwidthMean1[72][16];
          float32	bbwidthDev1[72][16];
          float32	sdepthMean1[72][16];
          float32	sdepthDev1[72][16];
          float32	surfRainAllMean1[72][16];
          float32	surfRainAllDev1[72][16];

          float32       surfRainConvMean1[72][16];
          float32       surfRainConvDev1[72][16];
          float32       surfRainStratMean1[72][16];
          float32       surfRainStratDev1[72][16];

          float32       e_surfRainConvMean1[72][16];
          float32       e_surfRainConvDev1[72][16];
          float32       e_surfRainStratMean1[72][16];
          float32       e_surfRainStratDev1[72][16];

          float32       rzStratA1[2][72][16];
          float32       rzStratB1[2][72][16];
          float32       rzConvA1[2][72][16];
          float32       rzConvB1[2][72][16];
          float32       rzA1[2][72][16];
          float32       rzB1[2][72][16];

	  int32		ttlPix1[72][16];
          int32         bbPixNum1[72][16];
          int32         bbNadirPix1[72][16];
          int32         wrainPix1[72][16];
          int32         surfRainPix1[72][16];
          int32         surfRainStratPix1[72][16];
          int32         surfRainConvPix1[72][16];
          int32         e_surfRainPix1[72][16];
          int32         e_surfRainStratPix1[72][16];
          int32         e_surfRainConvPix1[72][16];
          int32         epsilonPix1[72][16];
          int32         epsilon0StratPix1[72][16];
          int32         epsilon0ConvPix1[72][16];
          int32         epsilonStratPix1[72][16];
          int32         epsilonConvPix1[72][16];
          int32         surfRainAllPix1[72][16];
	  int32		rainPix1[6][72][16];
	  int32		convRainPix1[6][72][16];
	  int32		stratRainPix1[6][72][16];
          int16		ttlAnglePix1[4][72][16];
          int16		rainAnglePix1[4][72][16];
          int32         rzStratPix1[2][72][16];
          int32         rzConvPix1[2][72][16];
          int32         rzPix1[2][72][16];
          int32         shallowRainPix1[72][16];
          int32         shallowIsoRainPix1[72][16];
	  int16		stormHH[30][72][16];
	  int16		convStormHH[30][72][16];
	  int16		stratStormHH[30][72][16];
	  int16		BBHH[30][72][16];

          int16         bbNadirWidthH[30][72][16];
          int16         bbNadirHH[30][72][16];
          int16         bbNadirZmaxH[30][72][16];

	  int16		snowIceLH[30][72][16];
	  int16		bbZmaxH[30][72][16];
	  int16		epsilonH[30][72][16];

          int16         epsilon0StratH[30][72][16];
          int16         epsilon0ConvH[30][72][16];
          int16         epsilonStratH[30][72][16];
          int16         epsilonConvH[30][72][16];

	  int16		surfRainH[30][72][16];
	  int16		e_surfRainH[30][72][16];
	  int16		surfRainAllH[30][72][16];

          int16         surfRainConvH[30][72][16];
          int16         surfRainStratH[30][72][16];
          int16         e_surfRainConvH[30][72][16];
          int16         e_surfRainStratH[30][72][16];

	  int16		zmH[6][30][72][16];
	  int16		convZmH[6][30][72][16];
	  int16		stratZmH[6][30][72][16];
	  int16		ztH[6][30][72][16];
	  int16		convZtH[6][30][72][16];
	  int16		stratZtH[6][30][72][16];
	  int16		rainH[6][30][72][16];
	  int16		convRainH[6][30][72][16];
	  int16		stratRainH[6][30][72][16];

          int16         shallowRainH[30][72][16];
          int16         shallowIsoRainH[30][72][16];

	  int16		piaSrtH[5][30][72][16];
	  int16		piaHbH[5][30][72][16];
	  int16		pia0H[5][30][72][16];
	  int16		pia2a25H[5][30][72][16];

	  int16		piaSrtssH[5][30][72][16];
	  int16		piaHbssH[5][30][72][16];
	  int16		pia0ssH[5][30][72][16];
	  int16		pia2a25ssH[5][30][72][16];

	  int16		zmGradH[3][30][72][16];
	  int16		xiH[30][72][16];
          int16		nubfH[30][72][16];
          int16		zpzmH[30][72][16];
	  float32	rainCCoef[3][72][16];
	  float32	convRainCCoef[3][72][16]; 
	  float32	stratRainCCoef[3][72][16];
	  float32	stormHtZmCCoef[72][16];
	  float32	piaCCoef[6][5][72][16];
	  float32	xiZmCCoef[72][16];

	  float32	piaSrtPix[5][72][16];
	  float32	piaHbPix[5][72][16];
	  float32	pia0Pix[5][72][16];
	  float32	pia2a25Pix[5][72][16];
         
	  float32	piaSrtssPix[5][72][16];
	  float32	piaHbssPix[5][72][16];
	  float32	pia0ssPix[5][72][16];
	  float32	pia2a25ssPix[5][72][16];
         
	  float32	rainCCoefPix[3][72][16];
	  float32	stratCCoefPix[3][72][16];
	  float32	convCCoefPix[3][72][16];

	  float32	piaCCoefPix[5][72][16];

	} L3A_25_PLANETGRID1;



	typedef struct
	{
	  float32	rainMean2[4][720][148];
	  float32	rainDev2[4][720][148];
	  float32	convRainMean2[4][720][148];
	  float32	convRainDev2[4][720][148];
	  float32	stratRainMean2[4][720][148];
	  float32	stratRainDev2[4][720][148];

          float32       shallowRainMean2[720][148];
          float32       shallowRainDev2[720][148];
          float32       shallowIsoRainMean2[720][148];
          float32       shallowIsoRainDev2[720][148];

          float32	zmMean2[4][720][148];
          float32	convZmMean2[4][720][148];
          float32	stratZmMean2[4][720][148];
          float32	ztMean2[4][720][148];
          float32	convZtMean2[4][720][148];
          float32	stratZtMean2[4][720][148];
          float32	stormHeightMean[3][720][148];
          float32	stormHeightDev2[3][720][148];
          float32	bbHeightMean[720][148];

          float32       epsilon0StratMean2[720][148];
          float32       epsilon0StratDev2[720][148];
          float32       epsilon0ConvMean2[720][148];
          float32       epsilon0ConvDev2[720][148];
          float32       epsilonStratMean2[720][148];
          float32       epsilonStratDev2[720][148];
          float32       epsilonConvMean2[720][148];
          float32       epsilonConvDev2[720][148];

          float32	surfRainMean2[720][148];
          float32	surfRainDev2[720][148];
          float32	e_surfRainMean2[720][148];
          float32	e_surfRainDev2[720][148];
          float32	bbZmaxMean2[720][148];
          float32	bbZmaxDev2[720][148];
          float32	sdepthMean2[720][148];
          float32	sdepthDev2[720][148];
          float32	bbHeightDev2[720][148];
          float32	surfRainAllMean2[720][148];
          float32	surfRainAllDev2[720][148];

          float32       surfRainConvMean2[720][148];
          float32       surfRainConvDev2[720][148];
          float32       surfRainStratMean2[720][148];
          float32       surfRainStratDev2[720][148];

          float32       e_surfRainConvMean2[720][148];
          float32       e_surfRainConvDev2[720][148];
          float32       e_surfRainStratMean2[720][148];
          float32       e_surfRainStratDev2[720][148];

          float32       rzStratA2[2][720][148];
          float32       rzStratB2[2][720][148];
          float32       rzConvA2[2][720][148];
          float32       rzConvB2[2][720][148];
          float32       rzA2[2][720][148];
          float32       rzB2[2][720][148];

	  int32		ttlPix2[720][148];
	  int32         bbPixNum2[720][148];
	  int32         wrainPix2[720][148];
	  int32         surfRainPix2[720][148];
          int32         surfRainConvPix2[720][148];
          int32         surfRainStratPix2[720][148];
	  int32         e_surfRainPix2[720][148];
          int32         e_surfRainConvPix2[720][148];
          int32         e_surfRainStratPix2[720][148];
	  int32         surfRainAllPix2[720][148];
	  int32		rainPix2[4][720][148];
	  int32		convRainPix2[4][720][148];
	  int32		stratRainPix2[4][720][148];
          int32         shallowRainPix2[720][148];
          int32         shallowIsoRainPix2[720][148];
          int32         epsilon0StratPix2[720][148];
          int32         epsilon0ConvPix2[720][148];
          int32         epsilonStratPix2[720][148];
          int32         epsilonConvPix2[720][148];
          int32         rzStratPix2[2][720][148];
          int32         rzConvPix2[2][720][148];
          int32         rzPix2[2][720][148];
        } L3A_25_PLANETGRID2;



	typedef struct
	{
	  L3A_25_PLANETGRID1	grid1;
	  L3A_25_PLANETGRID2	grid2;
	} L3A_25_GRID;


        /****************************************/
        /* Define L3A-26 data structure.        */
        /****************************************/


      	typedef struct
      	{
	  int32		ttlCount[72][16];
	  int32		rainCount[4][72][16];
	  int32		zeroOrderpDf[6][4][25][72][16];
	  int32		hbpDf[6][4][25][72][16];
	  int32	        pDf2A25[6][4][25][72][16];
	  float32	zeroOrderFit[6][3][4][72][16];
	  float32	hbFit[6][3][4][72][16];
	  float32	fit2A25[6][3][4][72][16];
	  float32	reliabOrderFit[6][4][72][16];
	  float32	reliabHBfit[6][4][72][16];
	  float32	reliab2A25fit[6][4][72][16];
	  float32       rainMeanTH[4][72][16];
        } L3A_26_GRID;


      	typedef struct
      	{
	  float32       LatentHeating[19][720][148];
        } L3A_27_GRID;

        /****************************************/
        /* Define L3B-31 data structure.        */
        /****************************************/

        typedef struct
        {
          float32       sfcrainTMI[72][16]; 
          float32       convectRain[72][16]; 
          float32       cloudWater[14][72][16];
          float32       rainWaterTMI[14][72][16]; 
          float32       cloudIce[14][72][16];
          float32       graupel[14][72][16];
          float32       latentHeat[14][72][16]; 
          float32       sfcrainCOMB[72][16]; 
          float32       rainWaterCOMB[14][72][16]; 
          float32       sfcrainTMIoverlap[72][16]; 
          float32       convectRainoverlap[72][16]; 
          float32       sfcrainCOMBoverlap[72][16]; 
          float32       surfAdjRatio[72][16];
          float32       surfAdjRatioverlap[72][16]; 
          float32       surfRainfall[72][16];  /*tk 4.9.1 and below*/
          float32       rainWater[14][72][16]; /*tk 4.9.1 and below*/
          float32       profAdjRatio[14][72][16]; /*tk 4.9.1 and below*/
        } L3B_31_GRID;

#endif

#endif

