/* DEVELOPMENT HISTORY */
/* Revision 1.42  1999/11/01 20:48:39  pkolbe
 * Changed the structure for 3A11.  Changed the element chiSqFit from a
 * short integer (int16) to an integer (int32).
 *
 * Revision 1.41  1999/11/01 14:48:51  slywczak
 * Changed the structure for 3A11.  Changed the element noOfSamples from a
 * short integer (int16) to an integer (int32).
 *
 * Revision 1.40  1999/08/13 17:11:28  pkolbe
 * added convective rain to the 2a12 structure
 *
 * Revision 1.39  1998/08/06 13:52:25  slywczak
 * Modifed the realtime structure of 2A21.  Changed the number of levels for the
 * elements cldWater and precipIce from 14 to 1.
 *
 * Revision 1.38  1998/06/26 20:40:01  pkolbe
 * added 1b11 and 2a12 real time fortran structures
 *
 * Revision 1.37  1998/06/26 14:30:58  slywczak
 * Added new real time structures for L1B11 (L1B_11_R1_SWATHDATA) and L2A12
 * (L2A_12_R1_SWATHDATA).  Added new macros for scaling factors for Satellite
 * Local Zenith Angle (L1B11) and Surface Rain (L2A12).
 *
 * Revision 1.36  1998/01/14 14:23:09  slywczak
 * Added scaled value entries for Hot Load Temperatures, Top Radiator
 * Temperatures, and the 85 GHz Temperature into the L1B 11 structures.  The
 * variables are hotTemp1_scale, hotTemp2_scale, hotTemp3_scale, temp85Ghz_scale,
 * topRadTemp_scale.
 *
 * Revision 1.35  1997/10/01 03:45:30  slywczak
 * Update the year element of the SCAN_TIME structure from a 1-byte value to a
 * 2-byte value.
 *
 * Revision 1.34  1997/09/29 22:14:30  slywczak
 * Changed the definition of the scan time year back to a 1-byte integer from a
 * 2-byte integer.  Writing data as a 1-byte and trying to read it as a 2-byte
 * caused problems.   This will be coordinated when possible and changed later.
 *
 * Revision 1.33  1997/09/18 12:55:30  slywczak
 * Changed the structure definition for the scan time structures that are used
 * in the L1B and L2A products.  Changed the definition of the year from a 1-byte
 * integer to a 2-byte integer.
 *
 * Revision 1.32  1997/08/12 21:42:16  slywczak
 * Changed the location of the IO.h include to mantain compatibility with
 * FORTRAN and n32.
 *
 * Revision 1.31  1997/06/26 19:27:06  slywczak
 * Added the sgiPatch variable to the structures so that the FORTRAN interface
 * would work again.
 *
 * Revision 1.30  1997/06/20 14:37:06  slywczak
 * A patch has been provided for the SGI FORTRAN compiler, so the temporarily
 * variables (sgiPatch) needed for the old compiler were removed.
 *
 * Revision 1.29  1997/05/19 02:51:49  slywczak
 * Deleted the definition of grid struct for algorithm 3A-11.
 *
 * Revision 1.28  1997/03/10 17:12:19  slywczak
 * Modified the definitions of L1B-11, L2A-12, and L3A-11 so that each of the
 * structures end in a 4-byte data type.  This is the current workaround to solve
 * the bug in the SGI FORTRAN compiler.  If a structure did not have a 4-byte
 * data type, an additional int32 or integer*4 data type was added and labeled
 * as sgiPatch.
 *
 * Revision 1.27  1996/12/24 15:51:09  amak
 * Incorporated walkthrough comments.
 * - Changed '<' and '>' to double quotes around the #include statements.
 *
 * Revision 1.26  1996/08/26  19:39:09  amak
 * Fixed the bug in the preprocessor statements for Fortran interface.
 *
 * Revision 1.25  1996/06/07  15:03:11  amak
 * Changed datatype of geoQuality in Scan Status block from 4 bytes integer
 * to 1 byte integer.
 *
 * Revision 1.24  1996/05/31  14:57:48  amak
 * Corrected the data type of entities, calCoefA and calCoefB, in Calibration
 * block from int32 to float32.  Corrected the data type of satLocZenAngle in
 * L1B_11_SWATHDATA from int32 to float32.
 *
 * Revision 1.23  1996/05/21  17:53:31  amak
 * Fixed typo error of entity acsMode in TMI Scan Status block.
 *
 * Revision 1.22  1996/04/26  21:07:26  amak
 * Added one more field (missing, 1 byte) to scan status block.
 * In L2A-12 product, the scaling factors should be corrected to the
 * folowing:
 * 	Cloud Water     1000
 * 	Precip Water	1000
 * 	Cloud Ice       1000
 * 	Precip Ice      1000
 * 	Latent Heating  10
 *
 * Revision 1.21  1996/03/07  18:45:57  amak
 * New definition for parameter scanTime in product 1B-11 and 2A-12.
 * Five elements of 2A-12 (cldWater, precipWater, cldIce, precipIce, and
 * latentHeat) should have scaling factors of 100.  The corresponding
 * macros had been defined.
 *
 * Revision 1.20  1996/01/25  15:02:28  amak
 * Changed variable name fractOrbitNum to fractOrbitN in TMI_SCAN_STATUS structure.
 *
 * Revision 1.19  1995/12/12  15:11:24  amak
 * Defined scaling factor macros.
 *
 * Revision 1.18  1995/11/20  17:53:54  amak
 * Added scaling calculation variables to the structures.
 *
 * Revision 1.17  1995/08/25  14:15:39  amak
 * Replaced all the INTEGER*1 type to BYTE data type.
 *
 * Revision 1.16  1995/08/09  13:21:36  gsain
 * make corrections
 *
 * Revision 1.15  1995/08/08  17:54:52  gsain
 * made changes
 *
 * Revision 1.14  1995/08/07  17:08:17  gsain
 * made changes
 *
 * Revision 1.13  1995/08/07  15:42:12  amak
 * Moved the TMI_NAVIGATION block structure to IO.h.
 *
 * Revision 1.12  1995/08/04  21:02:34  amak
 * Changed data structure for L2A-12 based on the new file spec (8/3/95).
 *
 * Revision 1.11  1995/08/04  19:59:12  amak
 * Added new data structure based on the new file spec (8/3/95)
 *
 * Revision 1.10  1995/08/03  17:46:01  amak
 * Re-modified the code because CVS couldn't do version control nicely.
 *
 * Revision 1.9  1995/07/27  13:36:38  amak
 * Added underscores to the include file names in the preprocessor statements.
 *
 * Revision 1.8  1995/07/20  18:28:59  amak
 * Added data structures for Level 3 satellite products.
 *
 * Revision 1.7  1995/07/19  20:09:08  amak
 * Added Planetary Grid into Level 3 data structures.
 *
 * Revision 1.6  1995/07/18  14:37:55  amak
 * Renamed QAC to qac in scanStatus block.
 *
 * Revision 1.5  1995/07/08  14:48:57  amak
 * Took out Level 3 data structure due to the change of the API for
 * Level 3 satellite products.
 *
 * Revision 1.4  1995/07/06  20:23:22  amak
 * Made minor change
 *
 * Revision 1.3  1995/06/29  18:07:07  gsain
 * added swath structures
 *
 * Revision 1.2  1995/06/08  20:10:18  amak
 * Added log file header.
 * */

/*******************************************************************************
*
*  FILE NAME:           IO_TMI.h
*
*  DESCRIPTION:         This file contains the data structures of the TMI
*                       products.
*
*******************************************************************************/


#ifndef __IO_TMI_H
#ifndef LANGUAGE_FORTRAN 
#define __IO_TMI_H 1
#endif 

/* Define Scaling Factors. */
#define L1B11_LOWRESCH		100
#define L1B11_CALIBRATION	100
#define L1B11_HIGHRESCH		100
#define L1B11_SATLOCZENANGLE    100
#define L2A12_CLOUDWATER	1000
#define L2A12_PRECIPWATER	1000
#define L2A12_CLOUDICE		1000
#define L2A12_PRECIPICE		1000
#define L2A12_LATENTHEAT	10
#define L2A12_SURFACERAIN       10
#define L2A12_CONVECTRAIN       10
#define L3A11_MONTHRAIN		10
#define L3A11_FREEZLEVEL	100
#define L3A11_T0		10
#define L3A11_R0		100
#define L3A11_SIGMAR		100
#define L3A11_PROBRAIN		1000


#ifdef LANGUAGE_FORTRAN
/****************************************************************
*	Structure definition for FORTRAN 			*
*****************************************************************/

      STRUCTURE /TMI_SCAN_STATUS/
	BYTE		missing
	BYTE		validity	
	BYTE		qac
	BYTE		geoQuality
	BYTE		ch1
	BYTE		ch2
	BYTE		ch3
	BYTE		ch4
	BYTE		ch5
	BYTE		ch6
	BYTE		ch7
	BYTE		ch8
	BYTE		ch9
	BYTE		scOrient
	BYTE		acsMode
	BYTE		yawUpStat
	BYTE		tmiIsStatus
	REAL*4		fractOrbitN
      END STRUCTURE


      STRUCTURE /CALIBRATION/
	INTEGER*2	hotTemp1_scale
        REAL*4          hotTemp1
	INTEGER*2	hotTemp2_scale
        REAL*4          hotTemp2
	INTEGER*2	hotTemp3_scale
        REAL*4          hotTemp3
	INTEGER*2	posBridgeVolt	
	INTEGER*2	nearZeroVolt
	INTEGER*2	temp85Ghz_scale
        REAL*4          temp85Ghz
	INTEGER*2	topRadTemp_scale
        REAL*4          topRadTemp
	BYTE		autoCont1
	BYTE		autoCont2
	BYTE		autoCont3
	BYTE		autoCont4
	BYTE		autoCont5
	BYTE		autoCont6
	BYTE		autoCont7
	BYTE		autoCont8
	BYTE		autoCont9
	REAL*4		calCoef1A
	REAL*4		calCoef2A
	REAL*4		calCoef3A
	REAL*4		calCoef4A
	REAL*4		calCoef5A
	REAL*4		calCoef6A
	REAL*4		calCoef7A
	REAL*4		calCoef8A
	REAL*4		calCoef9A
	REAL*4		calCoef1B
	REAL*4		calCoef2B
	REAL*4		calCoef3B
	REAL*4		calCoef4B
	REAL*4		calCoef5B
	REAL*4		calCoef6B
	REAL*4		calCoef7B
	REAL*4		calCoef8B
	REAL*4		calCoef9B
      END STRUCTURE 

      STRUCTURE /SCAN_TIME/
        INTEGER*2 	year
	BYTE		month
	BYTE		dayOfMonth
	BYTE		hour
	BYTE		minute
	BYTE		second
	INTEGER*2	dayOfYear
        INTEGER*4       sgiPatch  
      END STRUCTURE


      STRUCTURE /L1B_11_SWATHDATA/
	RECORD /SCAN_TIME/		scanTime
	REAL*4				geolocation(2,208)
	RECORD /TMI_SCAN_STATUS/	scanStatus
	RECORD /NAVIGATION/		navigate
	RECORD /CALIBRATION/		calib
	INTEGER*2			calCounts(16,2,9)
	REAL*4				satLocZenAngle(12)
	INTEGER*2			lowResCh_scale(7,104) 
	REAL*4				lowResCh(7,104)
	INTEGER*2			highResCh_scale(2,208)
	REAL*4				highResCh(2,208)
      END STRUCTURE


      STRUCTURE /L1B_11_R1_SWATHDATA/
        RECORD /SCAN_TIME/              scanTime
        INTEGER*2                       geolocation_scale(2,208)
        REAL*4                          geolocation(2,208)
        INTEGER*2                       satLocZenAngle_scale(12)
        REAL*4                          satLocZenAngle(12)
        INTEGER*2                       lowResCh_scale(7,104)
        REAL*4                          lowResCh(7,104)
        INTEGER*2                       highResCh_scale(2,208)
        REAL*4                          highResCh(2,208)
      END STRUCTURE


      STRUCTURE /L2A_12_SWATHDATA/
	 RECORD /SCAN_TIME/		scanTime
	 REAL*4				geolocation(2,208)
	 RECORD /TMI_SCAN_STATUS/	scanStatus
	 RECORD /NAVIGATION/		navigate
	 BYTE				dataFlag(208)
	 BYTE				rainFlag(208)
	 BYTE				surfaceFlag(208)
	 REAL*4				surfaceRain(208)
	 REAL*4				convectRain(208)
	 REAL*4				confidence(208)
         INTEGER*2			cldWater_scale(14,208)
         REAL*4				cldWater(14,208)
         INTEGER*2			precipWater_scale(14,208)
         REAL*4				precipWater(14,208)
         INTEGER*2			cldIce_scale(14,208)
         REAL*4				cldIce(14,208)
         INTEGER*2			precipIce_scale(14,208)
         REAL*4				precipIce(14,208)
         INTEGER*2			latentHeat_scale(14,208)
         REAL*4				latentHeat(14,208)
      END STRUCTURE


      STRUCTURE /L2A_12_R1_SWATHDATA/
         RECORD /SCAN_TIME/             scanTime
         INTEGER*2                      geolocation_scale(2,208)
         REAL*4                         geolocation(2,208)
         BYTE                           rainFlag(208)
         INTEGER*2                      surfaceRain_scale(208)
         INTEGER*2                      convectRain_scale(208)
         REAL*4                         surfaceRain(208)
         REAL*4                         convectRain(208)
         INTEGER*2                      cldWater_scale(208)
         REAL*4                         cldWater(208)
         INTEGER*2                      precipIce_scale(208)
         REAL*4                         precipIce(208)
      END STRUCTURE


      STRUCTURE /L3A_11_PLANETGRID/
	  INTEGER*2			monthRain_scale(16,72) 
	  REAL*4			monthRain(16,72)
	  INTEGER*4			noOfSamples(16,72)
	  INTEGER*4			chiSqFit(16,72)
	  INTEGER*2			freezLevel_scale(16,72)
	  REAL*4			freezLevel(16,72)
	  INTEGER*2			T0_scale(16,72)
	  REAL*4			T0(16,72)
	  INTEGER*2			r0_scale(16,72)
	  REAL*4			r0(16,72)
	  INTEGER*2			sigmaR_scale(16,72)
	  REAL*4			sigmaR(16,72)
	  INTEGER*2			qInd1(16,72)
	  INTEGER*2			qInd2(16,72)
	  INTEGER*2			qInd3(16,72)
	  INTEGER*2			spare(16,72)
	  INTEGER*2			probRain_scale(16,72)
	  REAL*4			probRain(16,72)
      END STRUCTURE


      STRUCTURE /L3A_12_GRID/
          REAL*4      surfaceRain (160,720)
          REAL*4      convectRain (160,720)
          REAL*4      stratiformRain (160,720)
          REAL*4        cldWater (160,720,14)
          REAL*4     precipWater (160,720,14)
          REAL*4        cldIce (160,720,14)
          REAL*4        precipIce (160,720,14)
          REAL*4        latentHeat (160,720,14)
          INTEGER*4      npixTotal (160,720)
          INTEGER*4    npixRain (160,720)
          INTEGER*4    npixConvectRain (160,720)
          INTEGER*4    npixStratiformRain (160,720)
          INTEGER*4    npixUnclassifiable (160,720)
          INTEGER*4    npixAmbiguous (160,720)
      END STRUCTURE


#else
#include "IO.h"
/****************************************************************
*	Structure definition for C	 			*
*****************************************************************/

	typedef struct 
	{
	  int8		missing;
	  int8		validity; 
	  int8		qac;
	  int8		geoQuality;
	  int8		ch1;
	  int8		ch2;
	  int8		ch3;
	  int8		ch4;
	  int8		ch5;
	  int8		ch6;
	  int8		ch7;
	  int8		ch8;
	  int8		ch9;
	  int8		scOrient;
	  int8		acsMode;
	  int8		yawUpStat;
	  int8		tmiIsStatus;
	  float32	fractOrbitN;
	} TMI_SCAN_STATUS;


	typedef struct
	{
	  int16		hotTemp1_scale;
	  float32	hotTemp1;  
	  int16		hotTemp2_scale;
	  float32       hotTemp2; 
	  int16		hotTemp3_scale;
	  float32	hotTemp3;  
	  int16		posBridgeVolt;
	  int16		nearZeroVolt;
	  int16		temp85Ghz_scale;
	  float32	temp85Ghz;  
	  int16		topRadTemp_scale;
	  float32	topRadTemp;  
	  int8		autoCont1;
	  int8		autoCont2;
	  int8		autoCont3;
	  int8		autoCont4;
	  int8		autoCont5;
	  int8		autoCont6;
	  int8		autoCont7;
	  int8		autoCont8;
	  int8		autoCont9;
	  float32	calCoef1A;
	  float32	calCoef2A;
	  float32	calCoef3A;
	  float32	calCoef4A;
	  float32	calCoef5A;
	  float32	calCoef6A;
	  float32	calCoef7A;
	  float32	calCoef8A;
	  float32	calCoef9A;
	  float32	calCoef1B;
	  float32	calCoef2B;
	  float32	calCoef3B;
	  float32	calCoef4B;
	  float32	calCoef5B;
	  float32	calCoef6B;
	  float32	calCoef7B;
	  float32	calCoef8B;
	  float32	calCoef9B;
	} CALIBRATION;


	typedef struct
	{
	  int16		year;
	  int8		month;
	  int8		dayOfMonth;
	  int8		hour;
	  int8		minute;
	  int8		second;
	  int16		dayOfYear;
	  int32         sgiPatch;  
	} SCAN_TIME;


	typedef struct
	{	
	  SCAN_TIME		scanTime;
	  float32		geolocation[208][2];
	  TMI_SCAN_STATUS	scanStatus;   
	  NAVIGATION		navigate;  
	  CALIBRATION		calib; 
	  int16			calCounts[9][2][16];
	  float32		satLocZenAngle[12];
	  int16			lowResCh_scale[104][7];
	  float32		lowResCh[104][7];
	  int16			highResCh_scale[208][2];
	  float32		highResCh[208][2];
	} L1B_11_SWATHDATA;

	typedef struct
	{	
	  SCAN_TIME		scanTime;
	  int16                 geolocation_scale[208][2];
	  float32		geolocation[208][2];
	  int16                 satLocZenAngle_scale[12];
	  float32		satLocZenAngle[12];
	  int16			lowResCh_scale[104][7];
	  float32		lowResCh[104][7];
	  int16			highResCh_scale[208][2];
	  float32		highResCh[208][2];
	} L1B_11_R1_SWATHDATA;
	 
 
	typedef struct
	{  
  	  SCAN_TIME		scanTime;
	  float32		geolocation[208][2];
          TMI_SCAN_STATUS	scanStatus;
 	  NAVIGATION		navigate;
	  int8			dataFlag[208];
	  int8			rainFlag[208];
	  int8			surfaceFlag[208];
	  float32		surfaceRain[208];
	  float32		convectRain[208];
	  float32		confidence[208];
	  int16			cldWater_scale[208][14];
	  float32		cldWater[208][14];
	  int16			precipWater_scale[208][14];
	  float32		precipWater[208][14];
	  int16			cldIce_scale[208][14];
	  float32		cldIce[208][14];
	  int16			precipIce_scale[208][14];
	  float32		precipIce[208][14];
	  int16			latentHeat_scale[208][14];
	  float32		latentHeat[208][14];
	} L2A_12_SWATHDATA;
 
	typedef struct
	{  
  	  SCAN_TIME		scanTime;
	  int16                 geolocation_scale[208][2];
	  float32		geolocation[208][2];
	  int8			rainFlag[208];
	  int16                 surfaceRain_scale[208];
	  int16                 convectRain_scale[208];
	  float32		surfaceRain[208];
	  float32		convectRain[208];
	  int16			cldWater_scale[208];
	  float32		cldWater[208];
	  int16			precipIce_scale[208];
	  float32		precipIce[208];
	} L2A_12_R1_SWATHDATA;

	typedef struct
	{
	  int16			monthRain_scale[72][16];
	  float32		monthRain[72][16];
	  int32			noOfSamples[72][16];
	  int32			chiSqFit[72][16];
	  int16			freezLevel_scale[72][16];
	  float32		freezLevel[72][16];
	  int16			T0_scale[72][16];
	  float32		T0[72][16];
	  int16			r0_scale[72][16];
	  float32		r0[72][16];
	  int16			sigmaR_scale[72][16];
	  float32		sigmaR[72][16];
	  int16			qInd1[72][16];
	  int16			qInd2[72][16];
	  int16			qInd3[72][16];
	  int16			spare[72][16]; 
	  int16			probRain_scale[72][16];
	  float32		probRain[72][16];
	} L3A_11_PLANETGRID;


typedef struct {

   float32        surfaceRain[720][160];
   float32        convectRain[720][160];
   float32        stratiformRain[720][160];

   float32        cldWater[14][720][160];
   float32        precipWater[14][720][160];
   float32        cldIce[14][720][160];
   float32        precipIce[14][720][160];
   float32        latentHeat[14][720][160];

   int32         npixTotal[720][160];

   int32         npixRain[720][160];
   int32         npixConvectRain[720][160];
   int32         npixStratiformRain[720][160];
   int32         npixUnclassifiable[720][160];
   int32         npixAmbiguous[720][160];


}L3A_12_GRID;


#endif

#endif
