/* DEVELOPMENT HISTORY */
/* Revision 1.20  1998/02/11 19:31:01  slywczak
 * Added the Solar Calibration structure and made this structure a sub-structure
 * of L1B_01_SWATHDATA.
 *
 * Revision 1.19  1997/10/30 21:54:26  slywczak
 * Modified the structures L1B_01_SWATHDATA.  Added the element tempCounts.
 *
 * Revision 1.18  1997/08/12 21:42:34  slywczak
 * Changed the location of the IO.h include to mantain compatibility with
 * FORTRAN and n32.
 *
 * Revision 1.17  1997/05/19 02:53:54  slywczak
 * Deleted the definition of grid struct for algorithms 3B-42 and 3B-43.
 *
 * Revision 1.16  1996/08/26 19:39:10  amak
 * Fixed the bug in the preprocessor statements for Fortran interface.
 *
 * Revision 1.15  1996/06/07  14:59:35  amak
 * Added scaling field 'localDirection_scale' for L1B_01_SWATHDATA.
 * Defined L1B01_LOCALDIR as scaling factor.
 *
 * Revision 1.14  1996/04/26  21:08:17  amak
 * Added one more field (missing, 1 byte) to scan status block.
 *
 * Revision 1.13  1996/03/15  16:41:29  amak
 * Added two more variables, virsMode and virsAbnCon, to VIRS_SCAN_STATUS
 * structure.
 *
 * Revision 1.12  1995/12/12  15:11:23  amak
 * Defined scaling factor macros.
 *
 * Revision 1.11  1995/11/20  17:53:55  amak
 * Added scaling calculation variables to the structures.
 *
 * Revision 1.10  1995/08/29  15:01:36  amak
 * Modified the preprocessor statements.
 *
 * Revision 1.9  1995/08/25  14:15:39  amak
 * Replaced all the INTEGER*1 type to BYTE data type.
 *
 * Revision 1.8  1995/08/07  15:42:25  amak
 * Moved the VIRS_NAVIGATION block structure to IO.h.
 *
 * Revision 1.7  1995/07/27  13:36:49  amak
 * Added underscores to the include file names in the preprocessor statements.
 *
 * Revision 1.6  1995/07/20  18:28:59  amak
 * Added data structures for Level 3 satellite products.
 *
 * Revision 1.5  1995/07/19  20:10:01  amak
 * Added Planetary Grid into Level 3 data structures.
 *
 * Revision 1.4  1995/07/08  14:50:50  amak
 * Took out Level 3 data structure due to the change of API for the
 * Level 3 satellite products.
 *
 * Revision 1.3  1995/06/30  17:50:46  thyun
 * Added Swathdata structure
 *
 * Revision 1.2  1995/06/08  20:10:19  amak
 * Added log file header.
 * */

/*******************************************************************************
*
*  FILE NAME:           IO_VIRS.h
*
*  DESCRIPTION:         This file contains the data structures of the VIRS 
*                       products.
*
*******************************************************************************/


#ifndef __IO_VIRS_H
#ifndef LANGUAGE_FORTRAN 
#define __IO_VIRS_H 1
#endif 


/* Define Scaling Factors. */
#define L1B01_LOCALDIR	100
#define L1B01_CHANNELS1	500
#define L1B01_CHANNELS2 1000
#define L1B01_CHANNELS3 100000
#define L1B01_CHANNELS4 10000
#define L1B01_CHANNELS5 10000


#ifdef LANGUAGE_FORTRAN

/*********************************/
/* Define structures for FORTRAN */
/*********************************/

      STRUCTURE /VIRS_SCAN_STATUS/
	 BYTE		missing
         BYTE		validity
         BYTE		qac
         BYTE		geoQuality
         BYTE		ch1Quality
         BYTE		ch2Quality
         BYTE		ch3Quality
         BYTE		ch4Quality
         BYTE		ch5Quality
         BYTE		scOrient
         BYTE		acsMode
         BYTE		yawUpdateS
         BYTE		virsInstS
	 BYTE		virsMode
	 BYTE		virsAbnCon
         REAL*4         fractOrbitN
      END STRUCTURE

      STRUCTURE /SOLAR_CALIBRATION/
         REAL*8         sunVecX
         REAL*8         sunVecY
         REAL*8         sunVecZ
         REAL*8         sunMag
      END STRUCTURE

      STRUCTURE /L1B_01_SWATHDATA/
         REAL*8                 	scanTime
         REAL*4                 	geolocation(2,261)
         RECORD /VIRS_SCAN_STATUS/   	scanStatus
         RECORD /NAVIGATION/    	navigate
         RECORD /SOLAR_CALIBRATION/     solarCal
         INTEGER*2              	calCounts(5,2,3)
         INTEGER*2                      tempCounts(6)
         INTEGER*2              	localDirection_scale(2,2,27)
	 REAL*4				localDirection(2,2,27)
         INTEGER*2              	channels_scale(5,261)
	 REAL*4				channels(5,261)
      END STRUCTURE


      STRUCTURE /L3B_42_PLANETGRID/ 
	 REAL*4				precipitate(80,360)
	 REAL*4				relError(80,360)
	 REAL*4				precipitation(400,1440)
	 REAL*4				relativeError(400,1440)
      END STRUCTURE


      STRUCTURE /L3B_43_PLANETGRID/
         REAL*4                 	precipitate(80,360)
         REAL*4                 	relError(80,360)
	 REAL*4				precipitation(400,1440)
	 REAL*4				relativeError(400,1440)
      END STRUCTURE

#else
#include "IO.h"
/***************************/
/* Define structures for C */
/***************************/

        typedef struct
        {
	  int8		missing;
          int8          validity;
          int8          qac;
          int8		geoQuality;
          int8          ch1Quality;
          int8          ch2Quality;
          int8          ch3Quality;
          int8          ch4Quality;
          int8          ch5Quality;
          int8          scOrient;
          int8          acsMode;
          int8          yawUpdateS;
          int8          virsInstS;
	  int8		virsMode;
	  int8		virsAbnCon;
          float32       fractOrbitN;
        } VIRS_SCAN_STATUS;


        typedef struct
        {
	  float64       sunVecX;
	  float64       sunVecY;
	  float64       sunVecZ;
	  float64       sunMag;
	} SOLAR_CALIBRATION;

	typedef struct
        {
          float64       	scanTime;
          float32       	geolocation[261][2];
          VIRS_SCAN_STATUS   	scanStatus;
          NAVIGATION    	navigate;
	  SOLAR_CALIBRATION     solarCal;
          int16         	calCounts[3][2][5];
	  int16                 tempCounts[6];
          int16         	localDirection_scale[27][2][2];
	  float32		localDirection[27][2][2];
          int16         	channels_scale[261][5];
	  float32		channels[261][5];
        } L1B_01_SWATHDATA;


	typedef struct
	{
	  float32	precipitate[360][80];
	  float32	relError[360][80];
	  float32	precipitation[1440][400];
	  float32	relativeError[1440][400];
	} L3B_42_PLANETGRID;


	typedef struct
	{
          float32       precipitate[360][80];
	  float32	relError[360][80];
	  float32	precipitation[1440][400];
	  float32	relativeError[1440][400];
	} L3B_43_PLANETGRID;

#endif

#endif
