/* The following is a set of constant values and conversion factors
 * that the TSDIS Toolkit is considering supplying to Algorithm
 * Developers.  Please provide feedback on whether these values are:
 * 1) Useful, i.e., will anyone use them?
 * 2) Correct, i.e., are these the values that people use?
 * 3) Complete, i.e., are there other things that are needed?
 * Please be as specific as possible, supply new values if any of
 * these conflict with yours.  Some of these macros are implemented
 * in C.  We will provide equivalent ones for FORTRAN.
 */

/* Related to the planet Earth. */
#define R_EQUATOR           	6378.137    /* Radius of the Earth equator in km */
#define EARTHROT		7.292115855E-5 /* Earth rotation rate rad/sec */ 
#define ECCENTRICITY 		0.01677	/* Earth's orbital eccentricity */

/* Miscellaneous constants. */
#define E		2.718281828  /* The constant e */
#define PI              3.141592654  /* The constant pi */
#define PI_BY_TWO 	1.570796327  /* pi divided by 2 */
#define RAD2DEG         57.29577951  /* 180 deg / pi radians */
#define DEG2RAD         0.017453292  /* pi radians / 180 deg */
#define TWOPI           6.283185307  /* pi times 2 */

/* Useful macros. */
#ifndef LANGUAGE_FORTRAN

#define log10(x)   		((float)(log((double)(x))/log((double)(10.0))))
#define MIN(a,b)        	((a)<=(b) ? (a):(b))  /* min function */
#define MAX(a,b)     		((a) > (b) ? (a):(b)) /* max function */

#endif
