/* Revision 1.1  1997/03/05 20:38:37  slywczak
 * Initial add of the header file for the TKcopyScanHeader function.  This file
 * defines the temporary working structures that are used for the routine, and
 * the corresponding function prototypes.
 * */
/********************************************************************************
*										
*  FILE NAME:           TKcopyScanHeadr.h						
*									
*  DESCRIPTION:         This file contains the header information for the copy
*                       scan header routines.  The temporary working structures
*                       are defined in this file.
*								
*********************************************************************************/
#ifndef _TKCOPYHEADR_H_
#ifndef LANGUAGE_FORTRAN
#define _TKCOPYHEADR_H_
#include "IO.h"
#include "IO_TMI.h"
#include "IO_VIRS.h"
#include "IO_PR.h"
#include "TKerrHandle.h"
#include "TS_TK_15.h"
#endif

/*
 *  Defines for the copy direction.  It tells the routine whether
 *  to copy from the input to the temporary structure, or from the
 *  temporary structure to the output.  These defines are used in 
 *  the copy scan auxillary functions (see TKcopyHeadr.c).
 */
#define COPY_FROM_INPUT 1
#define COPY_TO_OUTPUT  2

/*
 *  TMI structure for the scan header.  This structure will hold the
 *  scan time, geolocation, scan status, and the navigation blocks.
 *  When the copy direction is set to COPY_FROM_INPUT, data is copied
 *  to these temporary working structures and when the copy direction
 *  is set to COPY_TO_OUTPUT, the data is copied back to the output 
 *  buffer.
 */
typedef struct
{
  SCAN_TIME scanTime;
  TMI_SCAN_STATUS scanStatus;
  float32 geolocation[TK_TMI_PIXELS][2];
  NAVIGATION navigate;
} TMI_COPY_HEADER;

/*
 *  VIRS structure for the scan header.  This structure will hold the
 *  scan time, geolocation, scan status, and the navigation blocks.
 *  When the copy direction is set to COPY_FROM_INPUT, data is copied
 *  to these temporary working structures and when the copy direction
 *  is set to COPY_TO_OUTPUT, the data is copied back to the output 
 *  buffer.
 */
typedef struct
{
  float64 scanTime;
  VIRS_SCAN_STATUS scanStatus;
  float32 geolocation[TK_VIRS_PIXELS][2];
  NAVIGATION navigate;
} VIRS_COPY_HEADER;

/*
 *  PR structure for the scan header.  This structure will hold the
 *  scan time, geolocation, scan status, and the navigation blocks.
 *  When the copy direction is set to COPY_FROM_INPUT, data is copied
 *  to these temporary working structures and when the copy direction
 *  is set to COPY_TO_OUTPUT, the data is copied back to the output 
 *  buffer.
 */
typedef struct
{
  float64 scanTime;
  PR_SCAN_STATUS scanStatus;
  float32 geolocation[TK_PR_PIXELS][2];
  NAVIGATION navigate;
} PR_COPY_HEADER;

/*
 *  Function prototypes for the TKcopyScanHeader routine and the 
 *  auxillary functions.
 */
void copy1B01ScanHeader ( L1B_01_SWATHDATA *, VIRS_COPY_HEADER *, int, int );
void copy1B11ScanHeader ( L1B_11_SWATHDATA *, TMI_COPY_HEADER *, int, int );
void copy2A12ScanHeader ( L2A_12_SWATHDATA *, TMI_COPY_HEADER *, int, int );
void copy1B21ScanHeader ( L1B_21_SWATHDATA *, PR_COPY_HEADER *, int, int );
void copy1C21Scanheader ( L1C_21_SWATHDATA *, PR_COPY_HEADER *, int, int );
void copy2A21ScanHeader ( L2A_21_SWATHDATA *, PR_COPY_HEADER *, int, int );
void copy2A23ScanHeader ( L2A_23_SWATHDATA *, PR_COPY_HEADER *, int, int );
void copy2A25ScanHeader ( L2A_25_SWATHDATA *, PR_COPY_HEADER *, int, int );
void copy2B31ScanHeader ( L2B_31_SWATHDATA *, PR_COPY_HEADER *, int, int );
int TKcopyScanHeader ( IO_HANDLE *, void *, IO_HANDLE *, void * );

#endif
