/* DEVELOPMENT HISTORY */
/* Revision 1.12  1999/10/26 17:45:48  pkolbe
 * made a new macro called TKreportErrorParm
 *
 * Revision 1.11  1998/05/14 18:36:36  slywczak
 * Added the macro TKreportWarningParm to report a static warning message and
 * supply a user defined string.  Modified the reportWarn prototype to include
 * the user defined message.
 *
 * Revision 1.10  1997/12/16 17:00:14  slywczak
 * Corrected spelling mistake on the include line.
 *
 * Revision 1.9  1997/12/16 15:34:00  slywczak
 * Added include files for sys/types.h and unistd.h.
 *
 * Revision 1.8  1997/12/11 21:47:56  piper
 * Modified sendtrap prototype due to calling sequence change.
 *
 * Revision 1.7  1997/05/30 13:10:57  piper
 * Added prototype for sendtrap call added to TKerrHandle.c.
 *
 * Revision 1.6  1997/02/04 16:46:37  amak
 * Increased the size of TK_MAX_SIZE_COMMAND from 300 to 1000.
 *
 * Revision 1.5  1996/12/20 20:34:40  amak
 * Defined TK_MAX_SIZE_COMMAND macro as 300.
 *
 * Revision 1.4  1996/10/07  18:10:10  amak
 * Redefined function prototype getMessage().  Added one more parameter
 * (mnemonicString) to the function call.
 *
 * Revision 1.3  1996/06/06  18:25:35  amak
 * Took out Log_File parameter from getMessage prototype declaration.
 *
 * Revision 1.2  1996/05/21  17:49:29  amak
 * Fixed syntax error.
 *
 * Revision 1.1  1996/04/19  15:44:23  amak
 * Added this file.  This file is now contained all the error handling calls'
 * header for Toolkit.
 * */

/********************************************************************************
*
*  FILE NAME:           TKerrHandle.h                                            
*                                                                       
*  DESCRIPTION:         
*	This file contains the header information for the Toolkit error 
*	handling routine. 
*                                                               
*********************************************************************************/

#ifndef __TKERRHANDLE_H
#define __TKERRHANDLE_H 1

#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <sys/types.h>
#include <unistd.h>

/**
 **	Define macros for TKreportWarning and TKreportError calls.
 **/
#define TKreportWarning(A)	reportWarn(A, NULL, __LINE__, __FILE__)
#define TKreportWarningParm(A,B) reportWarn ( A, B, __LINE__, __FILE__ )
#define TKreportError(A)	reportException(A, NULL, __LINE__, __FILE__)
#define TKreportErrorParm(A,B) reportException ( A, B, __LINE__, __FILE__ )

/**
 **	Define the maximum size of the buffer and the arrays.
 **/
#define TK_MAX_INSTR_SIZE	 11
#define TK_MAX_LABEL_SIZE	 32
#define TK_MAX_MNEMONIC_SIZE	 50
#define TK_MAX_MSG_SIZE		241
#define TK_MAX_MSGBUF_SIZE	481
#define TK_MAX_SIZE_COMMAND	1000

/**
 **	Define the file prefix of all the ASCII message file.
 **/
#define	TK_FILE_PREFIX	"TS"


typedef struct
{
  int	seed;
  char	instr[TK_MAX_INSTR_SIZE];
  char	label[TK_MAX_LABEL_SIZE];
} FILEINFO;


typedef struct
{
  int	code;
  char	mnemonic[TK_MAX_MNEMONIC_SIZE];
  char	msg[TK_MAX_MSG_SIZE];
  char	action[TK_MAX_MNEMONIC_SIZE];
} MSGDATA;


typedef struct
{
  FILEINFO	fileinfo;
  MSGDATA	msgdata;
} TK_MSGINFO;


/**
 **	Define function prototypes
 **/
char getMsgFile(int msgNumber, char **filePath);
void extractMsgInfo(TK_MSGINFO *msgInfo, char *line);
char getMessage(int msgCode, char **filePath, char *msgString, char *mnemonicString);
void reportWarn(int warnNumber, char *message, int linenumber, char *filename);
void reportException(int exceptionNumber, char *message, int linenumber, char *filename);
int sendtrap(pid_t gpid, int priority, char *message);

#endif
