# DEVELOPMENT HISTORY
# Revision 1.9  1999/11/23 20:24:32  slywczak
# Added message to warning the user when the string sent for the parameter type
# is longer than the maximum size.
#
# Revision 1.8  1998/05/21 17:48:38  slywczak
# Added new error message for not finding metadata fields, not being able to
# open SDSs, and not being able to open VDatas.
#
# Revision 1.7  1998/05/14 18:37:16  slywczak
# Added the error message W_TK_NO_METADATA_FIELD.
#
# Revision 1.6  1997/10/14 21:13:43  okelley
#         Modifications to the Toolkit to implement new
#         GV utilities.
#
# Revision 1.5  1997/10/14 19:26:31  slywczak
# Added the error mnemonic W_TK_NOSTARTSTOPTIMES.
#
# Revision 1.4  1997/09/03 20:35:03  okelley
#  Two error messages added for reading level 1 GV volume scans.
#
#   E_TK_MEM_ALLOC        Could not allocate memory.
#   E_TK_L1_GV            Could not read a level 1 GV volume scan.
#
# Revision 1.3  1997/05/29 03:05:22  slywczak
# Added error messages for the ASCII database and the GV Coincidence query.  The
# messages added were the following:  W_TK_FAILOPENDTM, W_TK_FAILOPENGM,
# W_TK_FAILOPENGI, W_TK_ASCII_DB_FAIL, W_TK_NO_RECORD, and W_TK_COIN_FILE_OPEN.
#
# Revision 1.2  1997/05/15 13:21:21  slywczak
# Added mnemonic for E_TK_SETCALINFO which will be issued when the scaling
# factors could not be added to an SDS.
#
# Revision 1.1  1997/05/07 15:19:25  jt
# Added more error messages.
#
# Revision 1.20  1997/01/10 20:30:51  amak
# Added new mnemonics.
#
# Revision 1.19  1997/01/03 22:46:41  amak
# Made W_TK_MDCPY to be E_TK_MDCPY.
#
# Revision 1.18  1996/12/23 16:28:16  amak
# Added mnemonic E_TK_MEMFAILGIDPARSE.
#
# Revision 1.17  1996/12/10  20:53:29  amak
# Modified warning messages for W_TK_FAILIMINSERT and W_TK_FAILIMOPEN.
#
# Revision 1.16  1996/12/06  20:53:09  amak
# Added mnemonics W_TK_INVALIDGVID, W_TK_FAILFMGV, W_TK_FAILSTAT, W_TK_FAILTIME,
# W_TK_FAILGMTIME, W_TK_FAILSTRFTIME.
#
# Revision 1.15  1996/12/06  20:09:31  pshah
# Corrected the mnemonic from E_TK_FAILIMOPEN to W_TK_FAILIMOPEN
#
# Revision 1.14  1996/12/06  20:04:24  pshah
# Added the mnemonic E_TK_FAILIMOPEN for failure to open database for IM
#
# Added the mnemonic W_TK_IMINSERT for error in inserting in to the database
#
# Revision 1.13  1996/10/17  18:03:25  pshah
# Added the mnemonic W_TK_SCANWARNING for use in TMI instruments
# which has pre and post orbit overlaps
#
# Revision 1.12  1996/10/16  19:49:49  amak
# Added mnemonic W_TK_INVALIDGVID for function TKwriteDBRainInfo().
#
# Revision 1.11  1996/10/09  17:49:43  amak
# Modified the error messages.
#
# Revision 1.10  1996/10/08  13:35:00  amak
# Made mnemonics E_TK_BADMODECH, E_TK_BADIDCH, E_TK_FAILSDGETINFO, and
# E_TK_MEMALFAILCH become warning messages by changing prefix "E_TK_" to
# "W_TK_".
#
# Revision 1.9  1996/09/03  18:47:08  amak
# Defined mnemonic E_TK_FAILVSWTNETHDR.
#
# Revision 1.8  1996/08/29  23:05:45  amak
# Defined more mnemonics for warning messages - W_TK_BADPIDNETHDR,
# W_TK_BADFMODENETHDR, W_TK_BADPIDWTGAUGE, and W_TK_BADFMODEWTGAUGE.
#
# Revision 1.7  1996/08/26  19:32:58  amak
# Rephrase the warning message for W_TK_NOGRANULEDATA.
#
# Revision 1.6  1996/08/02  20:01:47  amak
# Added macros E_TK_FAILVSELTS and E_TK_NOGRANULEDATA.
#
# Revision 1.5  1996/07/01  19:04:37  amak
# Took out macro E_TK_BADFMODEPO.
#
# Revision 1.4  1996/06/13  13:50:07  amak
# Added new mnemonics (E_TK_MEMALLOCFAIL, E_TK_VWRITEFAIL, and E_TK_VREADFAIL)
# for error messages.
#
# Revision 1.3  1996/05/29  19:40:38  amak
# Made some of the mnemonics from prefix 'E_' to prefix 'W_' as they are
# used as warning messages.
#
# Revision 1.2  1996/05/28  13:32:47  dbazell
# Added several metadata warning messages.
#
# Revision 1.1  1996/04/24 18:06:41  amak
# Added file.
#


# Purpose:      Define error messages for TK
# File          TS_TK.t

%INSTR = TSDIS
%LABEL = TK
%SEED = 15 

E_TK_FAILHDFOPEN      Opening a HDF file failed in TKopen.
E_TK_BADDEFVG         Trouble with defining Vgroup in attachVgroup.
E_TK_FAILVSFINDIN     VSfind failed in openInputVdata.
E_TK_FAILVSATTIN      VSattach failed in openInputVdata.
E_TK_FAILVSSETFLDIN   VSsetfields failed in openInputVdata.
E_TK_FAILSDNAMETOIND  SDnametoindex failed in openInputSdata.
E_TK_FAILSDSELECTIN   SDselect failed in openInputSdata. 
E_TK_FAILVSATTOUT     VSattach failed in openOutputVdataField.
E_TK_FAILDEFOUT       VSfdefine failed in openOutputVdataField.
E_TK_FAILSETFLDOUT    VSsetfields failed in openOutputVdataField.
E_TK_FAILCRTOUT       SDcreate failed in openOutputSdata.
E_TK_FAILWRTVWRT      VSwrite failed in vdataWrite.
E_TK_FAILWRTSWRT      SDwritedata failed in sdsWrite.
E_TK_FAILRDVRD        VSread failed in vdataRead.
E_TK_FAILRDSRD        SDreaddata failed in sdsRead.
E_TK_BADHDFTYPE       Incorrect HDF type in getSizeofType.
E_TK_FAILMEMREQPRL    Memory requrest failed in parse.l.
E_TK_FAILMEMREQPARSE  Memory request failed in parse.
E_TK_FAILVINCLS       Inserting a vdata object failed in attachObjects.
E_TK_FAILSREFCLS      Obtaining SDS reference # failed in attachObjects.
E_TK_FAILSINCLS       Inserting a sds object failed in attachObjects.
E_TK_CONFIGERROR      Error in configuration file.
E_TK_MEMERRVF         Memory allocation error in pushVdataFieldNode.
E_TK_MEMERRSDS        Memory allocation error in pushSDSnode.
E_TK_MEMERRVD         Memory allocation error in pushVdataNode.
E_TK_MEMERRVN         Memory allocation error in pushVgroupNode. 
E_TK_FAILOPPRS        Opening configuration file failed in parse.
E_TK_LSBNDS           Index out of bounds in Land Sea array.
E_TK_NULLPATH         getenv returned a null path.
E_TK_OPENFAIL         fopen failed.
E_TK_SEEKFAIL         fseek failed.
E_TK_READFAIL         fread failed.
E_TK_BADMEMALLCAT     Memory allocation failed in strcatFields.
E_TK_MEMALFAILAVM     Memory allocation failed in allocateVdataMemory.
E_TK_FAILHOPEN        Hopen failed in openHDF.
E_TK_FAILSDSTART      SDstart failed in startSD.
E_TK_BADIDGCFS        Undefined productID in getConfigFilenameAndScanline.
E_TK_FAILSDENDDO      SDendaccess failed in detachObjects.
E_TK_BADTEMPSIF       Invalid template in setInputFile.
E_TK_BADTEMPGNV       Invalid template in getNvos.
E_TK_BADTEMPGNP       Invalid template in getNparm.
E_TK_BADINPUTDT       Invalid inputFile in deleteTemplate.
E_TK_VATTACHFAILOIV   Vattach failed in openInputVgroupL1GV.
E_TK_BADPATHMCF       Invalid configuration file path in makeL1GVconfigFile.
E_TK_OPENFAILEDMCF    Opening configuration file failed in makeL1GVconfigFile.
E_TK_FAILATTACHFSV    Vattach failed in findSubVgroups.
E_TK_FAILSEARCHFTN    Searcing template failed in findTemplateNode.
E_TK_FAILVINDM        Inserting a vdata object failed in detachMetadata().
E_TK_EMPTYVGGV        Empty vgroup lists.
E_TK_FAILVSSK         VSseek failed in resetVgroup().
E_TK_ERRLOGIN         Error login to the database.
E_TK_INVALIDSITE      Invalid GV coincidence site.
E_TK_FAILINCOMP       SDsetcompress failed in openInputSdata().
E_TK_FAILOUTCOMP      SDsetcompress failed in openOutputSdata().
E_TK_FAILVINSTDHDR    Inserting a vdata object failed in detachHeader().
E_TK_MEMALLOCFAIL     Memory allocation failed in allocMem().
E_TK_VWRITEFAIL       VSwrite failed while writing header.
E_TK_VREADFAIL        VSread failed while reading header.
E_TK_FAILVSELTS	      Unable to determine the number of elements in a vdata.
E_TK_FAILVSWTNETHDR   Failed VSwrite in writeNetworkDesc().
E_TK_FAILFILEMODE     Invalid filemode in openInputVgroupL1GV().
E_TK_MEMFAILMETA      Memory allocation failed in finalizeMetadata().
E_TK_FAILPDCKSTAT     Invalid productID in checkAllFiles().
E_TK_MEMERRINIT       Memory allocation failed in initMetadata().
E_TK_MEMFAILPRCALHDR  Memory allocation failed in makePRcalCoefHdr().
E_TK_MEMFAILRAYHDR    Memory allocation failed in makeRayHdr().
E_TK_MEMFAILCLUTHDR   Memory allocation failed in makeClutterHdr().
E_TK_MEMFAILPROHDR    Memory allocation failed in processHeader().
E_TK_MEMFAILNETHDR    Memory allocation failed in makeNetHdr().
E_TK_MEMFAILGAUGE     Memory allocation failed in makeGaugeDesc().
E_TK_MEMFAILDISDRO    Memory allocation failed in makeDisdroDesc().
E_TK_MEMFAILNETWORK   Memory allocation failed in processNetworkHdr().
E_TK_MEMFAIL2A56SDS   Memory allocation failed in make2a56SDS().
E_TK_MEMFAIL2A57SDS   Memory allocation failed in make2a57SDS().
E_TK_MEMFAILGIDPARSE  Memory allocation failed in gIDparse().
E_TK_MDCPY            Malloc failed in mdcpy.
E_TK_MEMFAILWRITEL1GV Memory allocation failed in writeGV_L1B_1C().
E_TK_MEMFAILREADL1GV  Memory allocation failed in readGV_L1B_1C().
E_TK_MEMFAILL1GVTEMP  Memory allocation failed in setL1GVtemplate().
E_TK_FAILFCLOSE	      Unable to close the configuration file.
E_TK_MEMFAILBUFMANAGE Memory allocation failed in bufManage().
E_TK_MEMFAILCREATEPVL Memory allocation failed in create_pvl().
E_TK_MEMFAILMETAPVL Memory allocation failed in createMetadataPVL.

W_TK_NOGRANULEDATA    Invalid granule data in hdf file.
W_TK_BADMODEOPEN      Invalid filemode in TKopen.
W_TK_FAILADGHEL       Adding sGranuleHandle to error list failed.
W_TK_BADPIDRS         Invalid ProductID in TKreadScan.
W_TK_BADREADFMODE     Invalid file mode in TKreadScan.
W_TK_BADPIDWS         Invalid ProductID in TKwriteScan.
W_TK_BADWRITEFMODE    Invalid file mode in TKwriteScan.
W_TK_BADPIDRG         Invalid ProductID in TKreadGrid.
W_TK_READGRIDMODE     Invalid file mode in TKreadGrid. 
W_TK_WRITEGRIDMODE    Invalid file mode in TKwriteGrid.
W_TK_BADPIDWG         Invalid ProductID in TKwriteGrid.
W_TK_BADPIDRGV        Invalid product ID in TKreadL1GV.
W_TK_BADFMODERGV      Invalid file mode in TKreadL1GV.
W_TK_BADPIDWGV        Invalid product ID in TKwriteL1GV.
W_TK_BADFMODEWGV      Invalid file mode in TKwriteL1GV.
W_TK_BADFILEMODERI    Invalid filemode in TKreadMetadataInt.
W_TK_FAILDLGHEL       Deleting sGranuleHandle from error list failed.
W_TK_FAILDTCL         Deleting a L1-GV template failed.
W_TK_BADTYPSK         Undefined type in TKseek.
W_TK_BADMODEEF        Incorrect file mode in TKendOfFile.
W_TK_FAILVSQCEF       VSQueryCount failed in TKendOfFile.
W_TK_FAILSDGIEF       SDgetinfo failed in TKendOfFile.
W_TK_EMPTYHDFEF       No objects exist in the HDF file.
W_TK_FAILENDACCSCLS   SDendaccess failed in TKclose.
W_TK_FAILENDCLS       SDend failed in TKclose.
W_TK_FAILCLS          Hclose failed in TKclose.
W_TK_BADMEMALLCAT     Memory allocation error in strcatFields.
W_TK_BADSDSVWRT       Empty SDS list in vdataWrite.
W_TK_BADSDSSWRT       Empty SDS list in sdsWrite.
W_TK_BADVDATAVRD      Empty Vdata list in vdataRead.
W_TK_BADSDSSRD        Empty SDS list in sdsRead.
W_TK_FAILOPPRS        Opening configuration file failed in parse.
W_TK_BADHDFTYP        Incorrect HDF type in pr.h.
W_TK_BADOFSET         Invalid Offset.
W_TK_FAILVSSK         VSseek failed in TKseek
W_TK_BADOPTYPE        Invalid operation type in manageFiles.
W_TK_MEMERRFILES      Memory allocation error in addFile.
W_TK_BADIOHANDLE      Invalid sGranuleHandle in deleteFile.
W_TK_BADCOMPRESS      Scientific Data Set Compress failed.
W_TK_BADCOMPTYPE      Invalid Compress Type, compression is skiped.
W_TK_EMPTYLIST        File list is empty in deleteFile.
W_TK_PBNDS            Parameter out of bounds.
W_TK_LSBNDS           Index out of bounds in Land Sea array.
W_TK_BADOPMT          Invalid operation type in manageL1GVtemplate.
W_TK_BADPARMRI        Invalid parameter in TKreadMetadataInt. 
W_TK_BADPARMRF        Invalid parameter in TKreadMetadataFloat.
W_TK_BADFILEMODERF    Invalid filemode in TKreadMetadataFloat.
W_TK_BADPARMRC        Invalid parameter in TKreadMetadataChar.
W_TK_BADFILEMODERC    Invalid filemode in TKreadMetadataChar.
W_TK_BADPARMWI        Invalid parameter in TKwriteMetadataInt.
W_TK_BADFILEMODEWI    Invalid filemode in TKwriteMetadataInt.
W_TK_BADVALUESIZEWI   Size of value out of range in TKwriteMetadataInt.
W_TK_BADPARMWF        Invalid parameter in TKwriteMetadataFloat.
W_TK_BADFILEMODEWF    Invalid filemode in TKwriteMetadataFloat.
W_TK_BADVALUESIZEWF   Size of value out of range in TKwriteMetadataFloat.
W_TK_BADPARMWC        Invalid parameter in TKwriteMetadataChar.
W_TK_BADFILEMODEWC    Invalid filemode in TKwriteMetadataChar.
W_TK_BADVALUESIZEWC   Size of value out of range in TKwriteMetadataChar.
W_TK_BADFILEMODEWH    Invalid filemode in TKwriteHeader.
W_TK_BADFILEMODERH    Invalid filemode in TKreadHeader.
W_TK_BADPIDWH	      Invalid product ID in TKwriteHeader.
W_TK_BADPIDRH	      Invalid product ID in TKreadHeader. 
W_TK_FAILPROCESSHDR   Fail to process header information in TKopen.
W_TK_MAXSIZE          Maximum size of a metadata value exceeded.
W_TK_SETATTR          An error occurred during a call to SDsetAttr.
W_TK_ATTRINFO         An error occurred during a call to SDattrInfo.
W_TK_READATTR         An error occurred during a call to SDreadAttr.
W_TK_STRSTR           An error occurred during a call to strstr.
W_TK_READPVL          An error occurred during a call to read_pvl_from_hdffile.
W_TK_WRITEPVL         An error occurred during a call to write_pvl_to_hdffile.
W_TK_BADPIDNETHDR     Invalid product identifier for TKwriteNetHeader.
W_TK_BADPIDRNETHDR    Invalid product identifier for TKreadNetHeader.
W_TK_BADFMODENETHDR   Invalid filemode for TKwriteNetHeader.
W_TK_BADFMODERNETHDR  Invalid filemode for TKreadNetHeader.
W_TK_BADPIDWTGAUGE    Invalid product identifier for TKwriteGauge.
W_TK_BADPIDRDGAUGE    Invalid product identifier for TKreadGauge.
W_TK_BADFMODEWTGAUGE  Invalid filemode for TKwriteGauge.
W_TK_BADFMODERDGAUGE  Invalid filemode for TKreadGauge.
W_TK_BADMODECH        Invalid mode in TKcopyScanHeader.
W_TK_BADIDCH          Invalid product ID in TKcopyScanHeader.
W_TK_FAILSDGETINFO    SDgetinfo failed in TKcopyScanHeader.
W_TK_MEMALFAILCH      Memory allocation failed in TKcopyScanHeader.
W_TK_SCANWARNING      Number of Scans is less than the Preorbit & Postorbit overlap.
W_TK_INVALIDGVID      Invalid GV instrument ID.
W_TK_FAILFMGV	      Unable to do file append with this product.
W_TK_FAILSTAT	      Unable to get file information.
W_TK_FAILTIME	      Unable to get current processing time.
W_TK_FAILGMTIME       Unable to get Greenwich mean time.
W_TK_FAILSTRFTIME     Unable to store date and time in string.
W_TK_FAILIMQUERY      Unable to query the database.
W_TK_FAILIMINSERT     Error in inserting entity to TSDIS database.
W_TK_FAILIMOPEN	      Failed to open TSDIS database. 
W_TK_FAILALLOCREC     Failed to allocate memory using IMallocateRec().
W_TK_FAILIMUPDATE     Unable to do update TSDIS database.
W_TK_FAILDMMAKENAME   Unable to generate a browse name from DM.
W_TK_NORECORDSINDB    Record does not exist in database (or supporting tables).
W_TK_NORECSUPDATE     Record does not exist in the database to update.
E_TK_SETCALINFO	      Could not set the scaling factors for one (or more) SDS(s)
W_TK_FAILOPENDTM      Could not open the ASCII database files (Data Type Metadata).
W_TK_FAILOPENGM	      Could not open the ASCII database files (Granule Metadata). 
W_TK_FAILOPENGI       Could not open the ASCII database files (Granule Input Metadata).
W_TK_ASCII_DB_FAIL    Could not retrieve information from the ASCII database.
W_TK_NO_RECORD        No records retrieved from ASCII Database.
W_TK_COIN_FILE_OPEN   Could not open the GV coincidence file.
E_TK_MEM_ALLOC        Could not allocate memory.
E_TK_L1_GV            Could not read a level 1 GV volume scan.
W_TK_NOSTARTSTOPTIMES Could not get the start/stop times from either the input or output handle.
E_TK_GV_MPSP1         VosSize->param_position is TK_USE_DEFAULT which implies that this is a multi-parameter vos, when expecting a single-parameter vos. Metadata will be copied but not data.
E_TK_GV_MPSP2         VosSize->byte_size is 0 which implies that this is a multi-parameter vos, when expecting a single-parameter vos. Metadata will be copied but not data.
E_TK_GV_1B2B          VosSize->byte_size is 2 which means that the user requested a 2 byte param be loaded in a 1 byte param struct. The data will be copied after being truncated.
E_TK_ALG_ID           Could not read the algorithm_ID.
E_TK_GV_NPARAM        Could not read the number of parameters in a level 1 GV volume scan.
E_TK_GV_NSWEEP        Could not read the number of sweeps in a level 1 GV volume scan.
E_TK_GV_NRAY          Could not read the number of rays in a level 1 GV volume scan.
E_TK_GV_NCELL         Could not read the number of cells in a level 1 GV volume scan.
E_TK_GV_L1_PROD       Encountered an invalid product ID. Expected TK_L1C_GV or TK_L1B_GV.
E_TK_GV_BAD_YR        The user chose a year outside the range of 1980 - 2020, which is invalid.
E_TK_GV_BAD_DATE      Invalid fields in the use supplied date.
E_TK_GV_USE_TKOPEN    Open the granule with TKopen before trying to read data.
E_TK_GV_VOS_NUM       The user suplied a volume scan number that is invalid because it is outside the range of 0 to MAX_VOS -1.
E_TK_GV_NAME_LEN      The user suplied a parameter name with zero length.
E_TK_GV_ALLOC_VS      The user must allocate the VosSizeSTRUCT structure before calling this function.
E_TK_GV_NULL_VOS      The pointer to the volume scan is NULL, when a non-NULL address was expected.
E_TK_GV_V_EMPTY       The granule is empty and contains no volume scans.  You cannot read a volume scan from an empty granule.
E_TK_GV_P_EMPTY       The granule is empty and contains no volume scans.  You cannot determine a parameter's position in an empty granule.
E_TK_GV_D_EMPTY       The granule is empty and contains no volume scans.  You cannot read start date/times of volume scans in an empty granule.
E_TK_GV_B_SWEEP       The variable read_only_bottom_sweep has an invalid value, it must be TS_TRUE or TS_FALSE.
W_TK_GV_V_NOT_NULL    The volume scan pointer is not NULL when it was expected to be.  Perhaps you forgot to call TKfreeL1GV.  This function may cause a memory leak.
E_TK_GV_NO_SUCH_P     The parameter requested by the user does not exist in this volume scan.
W_TK_NO_METADATA_FIELD  This field was not found in either the Core or Product Specific Metadata. 
W_TK_FAIL_OPEN_SDS   Reading an OLD version of TRMM data. SDS does NOT exist in this version of the data product but does exist in a later version. The input file will be opened but do not read from this SDS: 
W_TK_FAIL_OPEN_VDATA   Reading an OLD version of TRMM data. VDATA does NOT exist in this version of the data product but does exist in a later version. The input file will be opened but do not read from this VDATA: 
W_TK_PARAM_LENGTH_EXCEEDED   The length for the parameter value has been exceeded.  The valid values are V, Z, QCZ, QCMZ.
W_TK_SDS_TYPE_CHANGE  Reading an OLD version of TRMM data. SDS has changed type from a previous version. SDS data will be mapped into the current definition of this variable. Refer to the old_type_ data element for original data. Use CAUTION with this field:
W_TK_SDS_DIM_CHANGE  Reading an OLD version of TRMM data. SDS has changed dimensions from a previous version. SDS data will be mapped into the current definition of this variable. Use CAUTION with this field:


