/* DEVELOPMENT HISTORY */
/* Revision 1.29  1999/08/13 17:46:44  pkolbe
 * added scaleFactor to sdsRead (in order to handle dynamic scaling)
 *
 * Revision 1.28  1998/12/11 00:35:31  pkolbe
 * added setSDSdimensionNames
 *
 * Revision 1.27  1998/04/23 16:16:34  slywczak
 * Added prototype for the routine createPlanetaryGrid2Metadata.
 *
 * Revision 1.26  1998/01/14 14:24:26  slywczak
 * Changed the prototypes for readpvl_from_hdffile (for more flexibility), and
 * findAlgorithmTKID.
 *
 * Revision 1.25  1997/09/08 20:35:58  slywczak
 * Added/Modified function prototypes for findAlgorithmDatatype and
 * createBrowseName.
 *
 * Revision 1.24  1997/06/23 22:12:02  slywczak
 * Added prototype for the routine radarType.
 *
 * Revision 1.23  1997/05/29 03:18:34  slywczak
 * Added prototypes for the following routines: setSDSattributes,
 * createSwathRadarMetadata, createRadar3A25Metadata, createSwathRadarVdata,
 * attachSwathRadarVdata, createL3StructureMetadata, and
 * createSwathStructureMetadata.
 *
 * Revision 1.22  1997/05/14 19:26:33  slywczak
 * Added prototypes for the following routines:  removeIMSpaces and
 * correctNULLElements
 *
 * Revision 1.21  1996/12/24 15:50:48  amak
 * Placed STORE_IM_DM_QUERY structure in the file.  This structure is used
 * to call TK_IM_DM_QUERY in IO.h.
 *
 * Revision 1.20  1996/12/12  14:31:40  amak
 * Defined function prototype for mdcpyLong().
 *
 * Revision 1.19  1996/11/01  20:56:10  amak
 * Defined function prototypes for initFlag() and createMetadataPVL().
 *
 * Revision 1.18  1996/09/03  19:47:26  amak
 * Defined prototypes for makeNetHdr(), processNetworkHdr(), detachVdata(),
 * make2a56SDS(), and make2a57SDS().
 *
 * Revision 1.17  1996/08/26  19:20:24  amak
 * Redefined the function prototype processObjects() from void to int32.
 *
 * Defined prototype to new routine detachVdata().
 *
 * Revision 1.16  1996/07/09  17:38:45  dbazell
 * Change prototype of mdcpy (Element,...) to mdcpy(ELEMENT,...)
 *
 * Revision 1.15  1996/06/13  14:15:23  dbazell
 * added instType prototype
 *
 * Revision 1.14  1996/06/13  13:54:37  amak
 * Defined function prototype for allocMem().
 *
 * Revision 1.13  1996/05/17  12:34:58  dbazell
 * deleted some metadata fields and added epochDate
 *
 * Revision 1.12  1996/03/04 19:44:56  amak
 * Defined new function prototypes. (readCoreMetadata, readPSMetadata,
 * writeCoreMetadata, writePSMetadata, makeCoreMetadataFields, and
 * makePSMetadataFields)
 *
 * Revision 1.11  1995/12/01  15:01:40  thyun
 * Removed the space between all the routines.
 *
 * Revision 1.10  1995/11/15  23:33:34  thyun
 * Defined the prototypes of openHDF, startSD, attachVgroup, detachVgroup, processObjects,
 * attachObjects, and detachObjects.
 *
 * Revision 1.9  1995/08/29  15:01:39  amak
 * Modified the preprocessor statements.
 *
 * Revision 1.8  1995/08/25  18:27:59  thyun
 * Removed template.h and inserted IO.h.
 *
 * Revision 1.7  1995/08/03  17:46:04  amak
 * Re-modified the code because CVS couldn't do version control nicely.
 * */

/*******************************************************************************
*
*  FILE NAME:           hdf_routines.h
*
*  DESCRIPTION:         This file contains the header information of all the 
*			functions/routines that call the HDF functions.
*
*******************************************************************************/


#ifndef __HDF_ROUTINES_H
#define __HDF_ROUTINES_H 1

#include "IO.h"

/*
**      Structure that used to store information that
**      query out from TSDIS database.
*/
typedef struct {
	char dataID[TK_SZ_ELEMENT_101];
	char inputFiles[TK_SZ_ELEMENT_301];
	char contact[TK_SZ_ELEMENT_101];
	char datacentsrc[TK_SZ_ELEMENT_101];
	char lonmaxlat[TK_SZ_ELEMENT_51];
	char leapSecsFlag[TK_SZ_ELEMENT_51];
	char gendateinputfile[TK_SZ_ELEMENT_101];
	char browseName[TK_SZ_ELEMENT_101];
	char algorithmID[TK_SZ_ELEMENT_51];
	char algorithmVersion[TK_SZ_ELEMENT_51];
	char orbitNumber[TK_SZ_ELEMENT_51];
} STORE_IM_DM_QUERY;

int32 openInputVdata(int32 hOpenID, char *vDataName, VDATA_FIELD_LINK *vdataList);
int32 openInputSdata(int32 sdStartID, char *sdName);
int32 openOutputVdataField(int32 hOpenID, char *vName, VDATA_FIELD_LINK *vdataFieldLink);
int32 openOutputSdata(int32 sdStartID, char *sdName, int32 type, int32 rank, int32 *dims);
int   vdataWrite(VDATA_LINK **vdataLink, void *data);
int   sdsWrite(SDS_LINK **sdsLink, void *data);
int   vdataRead(VDATA_LINK **vdataLink, void *data);
int   sdsRead(SDS_LINK **sdsLink, int* scaleFactor, void *data);
uint8 *allocateVdataMemory(VDATA_FIELD_LINK **vdataFieldLink);
int32 openHDF(char *filename, int32 access_mode, int16 n_dds);
int32 startSD(char *fileName, int32 access_mode);
void  attachVgroup(int32 outputFile, VGROUP_LINK *vgroup);
void  detachVgroup(VGROUP_LINK *vgroup);
int32 processObjects(VGROUP_LINK *vgroup, char accessMode, int32 hopenID,
			int32 sdstartID, int32 numOfScanlines, int *granFlg);
void  attachObjects(VGROUP_LINK *vgroup);
void  detachObjects(VGROUP_LINK *vgroup);
void  readCoreMetadata(IO_HANDLE *sGranuleHandle);
void  writeCoreMetadata(IO_HANDLE *sGranuleHandle);
void  makeCoreMetadataFields(VDATA_FIELD_LINK **vdataFields);
void  readPSMetadata(IO_HANDLE *sGranuleHandle);
void  writePSMetadata(IO_HANDLE *sGranuleHandle);
void  makePSMetadataFields(VDATA_FIELD_LINK **vdataFields);
void  processMetadata(IO_HANDLE *sGranuleHandle, char filemode);
void  detachMetadata(IO_HANDLE *sGranuleHandle);
void  resetVgroup(VGROUP_LINK *vgroup, int offsetPos);

void initMetadata (IO_HANDLE *sGranuleHandle, char filemode);
void mdcpy(ELEMENT *e , char *name, char *val);
void mdcpyLong(ELEMENT *e , char *name, char *val, int metadataID);
int  writepvl_to_hdffile(IO_HANDLE *whandle,char *fatr1,char *str1);
int  readpvl_from_hdffile( int32, char *fatr1,char *str1);
void create_pvl(char *outputStr, char *parameter, char  *value);
void parse_pvl(char *inputStr, char *parameter, char *value);
int finalizeMetadata(IO_HANDLE *sGranuleHandle);
void metaFree(IO_HANDLE *sGranuleHandle);
int  instType(int productID);
void processHeader(VGROUP_LINK *vgroup, int productID);
void makePRcalCoefHdr(VDATA_FIELD_LINK **vdataFields);
void makeRayHdr(VDATA_FIELD_LINK **vdataFields);
void makeClutterHdr(VDATA_FIELD_LINK **vdataFields);
uint8 *allocMem(VDATA_FIELD_LINK **vdataFieldLink, int nRecords);

void makeNetHdr(VDATA_FIELD_LINK **vdataFields);
void processNetworkHdr(VGROUP_LINK *vgroup, int productID);

void detachVdata(VGROUP_LINK *vgroup);

void make2a56SDS(SDS_LINK **stack);
void make2a57SDS(SDS_LINK **stack);
void setFlag(ELEMENT *metadata, int value);
void createMetadataPVL(IO_HANDLE *sGranuleHandle);

void removeIMSpaces ( char * );
void correctNULLElements ( IO_HANDLE * );

int updateStartStopTimesinGranuleMetadata ( IO_HANDLE *, IO_HANDLE * );

int setSDSdimensionNames ( VGROUP_LINK *vgroup, FILE* infile, char* datatype);
int setSDSattributes ( IO_HANDLE * );
void createSwathRadarMetadata ( IO_HANDLE * );
void createRadar3A25Metadata ( IO_HANDLE * );
void createSwathRadarVdata ( IO_HANDLE *, int32 *, char *, int32 );
void attachSwathRadarVdata ( VGROUP_LINK *, int32, char * );

char *createL3StructureMetadata ( IO_HANDLE * );
char *createPlanetaryGrid2Metadata ( IO_HANDLE * );
char *createSwathStructureMetadata ( IO_HANDLE * );

char *radarType ( int );

void createBrowseName ( IO_HANDLE *, char * );
char *findAlgorithmDataType ( int );
int findAlgorithmTKID ( char *, char * );

void checkTKenvironment ( );

#endif
