/* DEVELOPMENT HISTORY */
/* Revision 1.23  1999/10/25 13:11:17  pkolbe
 * removed some unnecesary debugging comments
 *
 * Revision 1.22  1999/09/13 15:21:54  pkolbe
 * finished backward compatability code changes
 *
 * Revision 1.21  1999/08/31 20:50:33  pkolbe
 * removed tk backward compatability/version checking code changes
 * (for now, anyway)
 *
 * Revision 1.20  1999/08/27 19:23:46  pkolbe
 * added tk backward compatability/version checking code changes
 *
 * Revision 1.19  1999/08/13 17:53:07  pkolbe
 * added scaling factor to SDS_NODE structure (to handle dynamic scaling)
 *
 * Revision 1.18  1996/12/24 15:55:25  amak
 * Incorporated walkthrough comments.
 * - Changed double quotes around the #include statements to '<' and '>' to
 *   identify system file.
 *
 * Revision 1.17  1996/09/03  19:49:43  amak
 * Took out numOfScanline parameter from prototype pushSDSnode().
 *
 * Revision 1.16  1996/06/13  13:55:21  amak
 * Added field 'dim' in VDATA_FIELD_LINK structure.
 *
 * Revision 1.15  1996/03/21  20:13:53  amak
 * Added one more parameter for pushSDSnode() function prototype.
 *
 * Revision 1.14  1995/11/15  23:19:10  thyun
 * Redefined whole structure in order to adapt changes in parsing.
 *
 * Revision 1.13  1995/08/29  15:01:40  amak
 * Modified the preprocessor statements.
 *
 * Revision 1.12  1995/08/25  18:26:30  thyun
 * Modified c preprocess part of defining template.h.
 *
 * Revision 1.11  1995/08/15  20:03:58  thyun
 * Inserted header comments.
 *
 * Revision 1.10  1995/08/10  19:06:45  thyun
 * Inserted fieldSize in the data structure of VDATA_FIELD_NODE.
 *
 * Revision 1.9  1995/08/03  17:46:05  amak
 * Re-modified the code because CVS couldn't do version control nicely.
 *
 * Revision 1.8  1995/08/01  20:35:44  thyun
 * Modfied the structure of CFG_STR as well as its sub-structures.
 *
 * Revision 1.6  1995/07/10  15:20:51  amak
 * Added more detailed comments
 *
 * Revision 1.5  1995/07/07  18:39:41  gsain
 * corrected syntax error
 *
 * Revision 1.4  1995/07/07  14:07:38  thyun
 * Modified and added comments
 *
 * Revision 1.3  1995/06/30  18:42:53  thyun
 * Deleted annotation
 * 
 * Revision 1.2  1995/06/08  20:10:21  amak
 * Added log file header.
 * */

/*******************************************************************************
*
*  FILE NAME:           template.h
*
*  DESCRIPTION:         This file contains the prototypes which are used for
*			parsing.
*
*******************************************************************************/

#ifndef __TEMPLATE_H
#define __TEMPLATE_H 1

#include <hdf.h>

#define MAX_RANK	6
#define MAX_SIZE	50
#define VGROUPTYPE	1
#define VDATATYPE       2
#define SDSTYPE         3
#define SUBVGROUPTYPE	4

/******************************************************************************/
/* VDATA_FIELD_NODE is a structure being used to hold name of vdata field,    */
/* type, and fieldSize. VDATA_FIELD_LINK is a type used to build a link list  */
/* of vdata fields.							      */ 
/******************************************************************************/
typedef struct VDATA_FIELD_NODE
{
  char				name[MAX_SIZE];
  int32 			type;
  int32				fieldSize;
  int32				dim;
  struct VDATA_FIELD_NODE	*rear;
  struct VDATA_FIELD_NODE 	*next;
} VDATA_FIELD_LINK;


/******************************************************************************/
/* VDATA_NODE is a structure being used to hold the vdata information.	      */
/* VDATA_LINK is a type being used to build a link list of vdata.      	      */
/******************************************************************************/
typedef struct VDATA_NODE
{ 
  VDATA_FIELD_LINK 		*vdataFields;
  char 				name[MAX_SIZE];
  int32 			vdataID;
  struct VDATA_NODE		*rear;
  struct VDATA_NODE 		*next;
} VDATA_LINK;


/******************************************************************************/
/* SDS_NODE is a structure being used to hold SDS information.	 	      */
/* SDS_LINK is a type being used to build a link list of SDS.		      */	
/******************************************************************************/
typedef struct SDS_NODE
{ 
  char	 			name[MAX_SIZE];
  int32 			type;
  int32 			rank;
  int32 			dim[MAX_RANK];
  int32 			start[MAX_RANK];
  int32 			edge[MAX_RANK];
  int32 			sdsID;
  int                           scaling_factor;
  struct SDS_NODE 		*rear;
  struct SDS_NODE 		*next;
} SDS_LINK;


/******************************************************************************/
/* VGROUP_ID is a structure being used to hold Vgroup information.	      */
/******************************************************************************/
typedef struct 
{
  char				name[MAX_SIZE];
  int32				vGroupID;
} VGROUP_ID;


/******************************************************************************/
/* VGROUP_NODE is a structure being used to hold HDF information such as vdata*/
/* sds, and vgroup.							      */
/******************************************************************************/
typedef struct VGROUP_NODE
{
  VGROUP_ID			*vGroupID;
  SDS_LINK 			*sds;
  VDATA_LINK 			*vdata;
  struct VGROUP_NODE		*subVgroup;
  struct VGROUP_NODE		*vgroupRear;
  struct VGROUP_NODE		*prevVgroup;
  struct VGROUP_NODE		*next;
} VGROUP_LINK;


/******************************************************************************/
/* CFG_NODE is a structure being used to hold vgroup information as well as   */
/* HDF file pointers such as ID of Hopen and SDstart.			      */
/******************************************************************************/
typedef struct
{
  int32				hOpenID;
  int32				sdStartID;
  VGROUP_LINK			*vGroup;	
} CFG_STR;


void initializeCFG_NODE(CFG_STR **cfgSTR);

void initializeVdataFieldLink(VDATA_FIELD_LINK **node);

void initializeSDSnode(SDS_LINK **node);

void initializeVdataLlink(VDATA_LINK **node);

void initializeVgroupLink(VGROUP_LINK **vGroup);

void initializeVgroupNode(struct VGROUP_NODE **node);

void initializeVgroupID(VGROUP_ID **node);

void pushVdataFieldNode(VDATA_FIELD_LINK **stack, char *name, int32 dataType, 
			int32 fieldSize);

void pushSDSnode(SDS_LINK **stack, char *name, int32 dataType, int range, 
			int32 dim[]);

void pushVdataNode(VDATA_LINK **stack, char *name);

void pushVgroupNode(VGROUP_LINK **stack, VGROUP_LINK *vGroup, 
			VGROUP_LINK *prevStack);

void createVgroup(VGROUP_LINK **stack, char *name);

void pushSubVgroupNode(VGROUP_LINK **stack, char *name);

void deallocateVgroup(VGROUP_LINK **vGroup);

void deallocateVdata(VDATA_LINK **vData);

void deallocateSDS(SDS_LINK **sds);

void deallocateCFG_STR(CFG_STR **cfgSTR);

#endif
