/* DEVELOPMENT HISTORY */
/* Revision 1.33  1999/08/13 17:19:17  pkolbe
 * added dynamic scaling code; also added code for the new 2a12 data
 * element "convective rain"
 *
 * Revision 1.32  1999/01/08 17:37:12  slywczak
 * Modified routine readScan_1B_11_R1 and readScan_2A_12_R1.  When reading and
 * scaling the variables from the HDF files, integer division was being applied
 * instead of floating point division.
 *
 * Revision 1.31  1998/08/06 13:53:53  slywczak
 * Modified routine readScan_2A12_R1 and writeScan_2A12_R1.  Modified code so that
 * only one level is being either read or written for the cldWater and precipIce
 * element instead of 14.
 *
 * Revision 1.30  1998/07/09 15:00:14  slywczak
 * Modified routine setScalingFactors_1B11 and setScalingFactors_2A12.  Added the
 * scaling factors for the real time elements for 1B11 and 2A12.
 *
 * Revision 1.29  1998/06/26 17:29:52  slywczak
 * Added new read/write routines for the new real time products (1B11_R1, and
 * 2A12_R1).
 *
 * Revision 1.28  1998/03/02 15:00:38  slywczak
 * Updated calibration coefficients for TMI.
 *
 * Revision 1.27  1998/02/09 19:52:22  slywczak
 * Removed extraneous printf statement.
 *
 * Revision 1.26  1998/01/14 14:26:29  slywczak
 * Added code to scale the TMI values of Hot Load Temperatures (3), 85 GHz
 * Temperature, and the Top Radiator Temperature.  These values are now scaled
 * by the toolkit.
 *
 * Revision 1.25  1997/05/19 03:41:19  slywczak
 * Modified the following routines:  readGrid_3A_11 and writeGrid_3A_11.
 * Removed references to the gridStruct structure, since this structure will be
 * referenced in the toolkit code.
 *
 * Revision 1.24  1997/05/16 14:10:57  slywczak
 * Modified the function setScalingFactors_1B11.  Added the offset of 100 for
 * the TMI high and low resolution channels.  The offset is written to the HDF
 * file by the routine SDsetcal (along with the scaling factor).
 *
 * Revision 1.23  1997/05/15 15:05:41  slywczak
 * Added the following functions:  setScalingFactors_1B11, setScalingFactors_2A12,
 * and setScalingFactors_3A11.  These function will write the scaling factors
 * to the SDSs.
 *
 * Revision 1.22  1997/02/21 23:14:32  piper
 * Fixed code to properly handle missing values.
 *
 * Revision 1.22  1997/02/19 14:35:15  jt
 * Fixed scaling calculations for reading missing or invalid data.
 *                                             
 * Revision 1.21  1997/02/13 17:51:45  piper
 * Fixed problem with missing values not being written to output files.
 *
 * Revision 1.20  1997/01/02 15:30:37  amak
 * Define TK_DEFAULT for hard-coded value.
 *
 * Revision 1.19  1996/12/24 14:30:35  amak
 * Incorporated walkthrough comments.
 * - Changed '<' and '>' to double quotes around the #include statements.
 *
 * Revision 1.18  1996/11/06  15:28:14  amak
 * Added detailed description to the routine.
 *
 * Revision 1.17  1996/10/24  13:48:49  pshah
 * Added code to take care of missing or invalid data for scaling
 *
 * Revision 1.16  1996/06/07  14:50:32  amak
 * Added prologue.
 *
 * Revision 1.15  1996/05/31  14:51:41  amak
 * Rearraged the calling sequence of calibration block.
 *
 * Revision 1.14  1996/05/21  19:44:52  amak
 * Corrected typo of entity acsMode in TMI ScanStatus block.
 *
 * Revision 1.13  1996/04/19  15:40:47  amak
 * Made modifications to the preprocessor #include statement.  Replaced
 * TKerrorRou.h and TKwarnRou.h to TKerrHandle.h and TS_TK_15.h for new
 * error handling calls.
 *
 * Revision 1.12  1996/03/21  21:59:55  amak
 * Added prologue.
 *
 * Revision 1.11  1996/03/07  21:38:05  amak
 * Defined new data handling of scanTime for product 1B-11 and 2A-12.
 * Added scaling calculations for product 2A-12 in functions readScan_2A_12()
 * and writeScan_2A_12().  The elements included cldWater, precipWater, cldIce,
 * precipIce, and latentHeat.
 *
 * Revision 1.10  1996/01/25  15:06:23  amak
 * Called macro function RND() when doing the roundup from floating point to
 * integer type.
 *
 * Revision 1.9  1995/12/18  22:25:42  amak
 * Added 0.5 to all the write scaling calculations.  This adding is to avoid
 * the round up problems while computing on various platforms.
 *
 * Revision 1.8  1995/12/12  20:11:56  thyun
 * Found a scaling mistake in readGrid_3A_11 and corrected the mistake.
 *
 * Revision 1.7  1995/12/12  14:49:05  amak
 * Used macro factors to do scaling calculations.
 *
 * Revision 1.6  1995/12/05  18:35:35  thyun
 * Modified assignment of tempSDS and tempVdata which are appearing in declaration
 * of all the functions.
 *
 * Revision 1.5  1995/11/30  21:12:43  amak
 * Fixed syntax errors.
 *
 * Revision 1.4  1995/11/28  21:34:40  amak
 * Fixed syntax errors of the scaling calculation.
 *
 * Revision 1.3  1995/11/20  17:52:39  amak
 * Added scaling calculations to the routines.
 *
 * Revision 1.2  1995/10/18  20:52:23  amak
 * Made modifications after some files need to be renamed.
 *
 * Revision 1.1  1995/10/17  19:41:52  amak
 * Renamed file IO_INTERNAL_TMI.c to IO_INTR_TMI.c
 *
 * Revision 1.21  1995/08/21  18:22:00  thyun
 * Corrected sizeof(char8) to sizeof(sPlanetGrid->gridStruct.origin)) in
 * readGrid_3A_11.
 *
 * Revision 1.20  1995/08/14  13:23:28  gsain
 * made changes
 *
 * Revision 1.19  1995/08/08  17:51:02  gsain
 * added changes
 *
 * Revision 1.18  1995/08/07  15:44:43  amak
 * Shortened the variable names in the navigate structure.
 *
 * Revision 1.17  1995/08/07  13:02:49  amak
 * Shorten variable names
 *
 * Revision 1.16  1995/08/04  19:54:00  amak
 * Put the variable name into the parameter instead of hardcoding a size.
 *
 * Revision 1.15  1995/08/03  17:45:37  amak
 * Re-modified the code because CVS couldn't do version control nicely.
 *
 * Revision 1.14  1995/08/01  20:30:40  thyun
 * Updated the structure names such as VDATA_LINK and SDS_LINK as well as
 * variable names.  These changes occurred becase of changes in parsing.
 *
 * Revision 1.11  1995/07/20  14:20:38  amak
 * Added routines readGrid_3A_11() and writeGrid_3A_11() for Level 3 API.
 *
 * Revision 1.10  1995/07/18  14:33:59  amak
 * Renamed the routines from vWrite, vRead, sWrite and sRead to vdataWrite,
 * vdataRead, sdsWrite and sdsRead respectively.  Also renamed object from
 * QAC to qac in the scanStatus block.
 *
 * Revision 1.9  1995/07/08  14:10:04  amak
 * Took out Level 3A-11 prototype due to the change of the API for Level 3
 * satellite products.
 *
 * Revision 1.8  1995/07/06  20:23:51  amak
 * Made minor change
 * */

/***********************************************************************************
*
*  FILE NAME:           IO_INTR_TMI.c
*
*  DESCRIPTION: 
*       This file contains routines that read and TMI headers and TMI data.
*
*  ROUTINES CALLED:
*       Routine Name            Description
*       ------------            -----------
*       writeScan_1B_11         This function is used to write the L1B-11
*                               information in HDF format.
*       readScan_1B_11          This function is used to read the L1B-11
*                               information in HDF format.
*       writeScan_2A_12         This function is used to write the L2A-12
*                               information in HDF format.
*       readScan_2A_12          This function is used to read the L2A-12
*                               information in HDF format.
*       writeGrid_3A_11         This function is used to write the L3A-11
*                               information in HDF format.
*       readGrid_3A_11          This function is used to read the L3A-11
*                               information in HDF format.
*       allocateVdataMemory     This routine is used to allocate memory to vdata.
*       vdataWrite              This routine is used to write the vdata in HDF    
*                               format.
*       sdsWrite                This routine is used to write the scientific data
*                               set in HDF format.
*       sdsRead                 This routine is used to read the scientific data 
*                               set in HDF format.
*       vdataRead               This routine is used to read the vdata in HDF
*                               format.
*
***********************************************************************************/

#include "IO_TMI.h"
#include "IO_INTR_TMI.h"
#include "hdf_routines.h"
#include "TS_TK_15.h"

/***********************************************************************************
*
*  FUNCTION STATEMENT:
*       writeScan_1B_11(IO_HANDLE *sGranuleHandle, L1B_11_SWATHDATA *sSwathData)
*
*  DESCRIPTION:
*       This function is used to write the L1B-11 information in HDF format.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       sGranuleHandle          A structure containing information about the file.
*                               sGranuleHandle is returned by TKopen().
*       sSwathData              A structure containing the complete scanline data.
*                               This structure must be declared to correspond to
*                               the data file being written.  The structures of
*                               sSwathData is manipulated by TKwriteScan() and
*                               other toolkit routines.
*
*  OUTPUT PARAMETERS:           None
*
*  CALLED BY:                   TKwriteScan()
*
*  ROUTINES CALLED:
*       Routine Name            Description
*       ------------            -----------
*       allocateVdataMemory     This routine is used to allocate memory to vdata.
*       vdataWrite              This routine is used to write the vdata in HDF
*                               format.
*       sdsWrite                This routine is used to write the scientific data
*                               set in HDF format.
*
***********************************************************************************/

int writeScan_1B_11(IO_HANDLE *sGranuleHandle, L1B_11_SWATHDATA *sSwathData)
{

  SDS_LINK *tempSDS = sGranuleHandle->cfgSTR->vGroup->subVgroup->sds;
  VDATA_LINK *tempVdata = sGranuleHandle->cfgSTR->vGroup->subVgroup->vdata;
  uint8 *databuf, *ptr;
  int	i, j;

  /* Allocate memory for packing the vdata */
  databuf = allocateVdataMemory(&tempVdata->vdataFields);
  ptr = databuf;
  memcpy (ptr, &sSwathData->scanTime.year, sizeof(sSwathData->scanTime.year));
  ptr += sizeof(sSwathData->scanTime.year);
  memcpy (ptr, &sSwathData->scanTime.month, sizeof(sSwathData->scanTime.month));
  ptr += sizeof(sSwathData->scanTime.month);
  memcpy (ptr, &sSwathData->scanTime.dayOfMonth, sizeof(sSwathData->scanTime.dayOfMonth));
  ptr += sizeof(sSwathData->scanTime.dayOfMonth);
  memcpy (ptr, &sSwathData->scanTime.hour, sizeof(sSwathData->scanTime.hour));
  ptr += sizeof(sSwathData->scanTime.hour);
  memcpy (ptr, &sSwathData->scanTime.minute, sizeof(sSwathData->scanTime.minute));
  ptr += sizeof(sSwathData->scanTime.minute);
  memcpy (ptr, &sSwathData->scanTime.second, sizeof(sSwathData->scanTime.second));
  ptr += sizeof(sSwathData->scanTime.second);
  memcpy (ptr, &sSwathData->scanTime.dayOfYear, sizeof(sSwathData->scanTime.dayOfYear));
  /* Writing packed vdata, scanTime, to an HDF file */
  vdataWrite(&tempVdata, databuf);
  free(databuf);

  sdsWrite(&tempSDS, &sSwathData->geolocation);


  /* Allocate memory for packing the vdata  */
  databuf = allocateVdataMemory(&tempVdata->vdataFields);
  ptr = databuf;
  memcpy(ptr, &sSwathData->scanStatus.missing,
                sizeof(sSwathData->scanStatus.missing));
  ptr += sizeof(sSwathData->scanStatus.missing);
  memcpy(ptr, &sSwathData->scanStatus.validity, 
		sizeof(sSwathData->scanStatus.validity));
  ptr += sizeof(sSwathData->scanStatus.validity);
  memcpy(ptr, &sSwathData->scanStatus.qac, sizeof(sSwathData->scanStatus.qac));
  ptr += sizeof(sSwathData->scanStatus.qac);
  memcpy(ptr, &sSwathData->scanStatus.geoQuality, 
		sizeof(sSwathData->scanStatus.geoQuality));
  ptr += sizeof(sSwathData->scanStatus.geoQuality);
  memcpy(ptr, &sSwathData->scanStatus.ch1, sizeof(sSwathData->scanStatus.ch1));
  ptr += sizeof(sSwathData->scanStatus.ch1);
  memcpy(ptr, &sSwathData->scanStatus.ch2, sizeof(sSwathData->scanStatus.ch2));
  ptr += sizeof(sSwathData->scanStatus.ch2);
  memcpy(ptr, &sSwathData->scanStatus.ch3, sizeof(sSwathData->scanStatus.ch3));
  ptr += sizeof(sSwathData->scanStatus.ch3);
  memcpy(ptr, &sSwathData->scanStatus.ch4, sizeof(sSwathData->scanStatus.ch4));
  ptr += sizeof(sSwathData->scanStatus.ch4);
  memcpy(ptr, &sSwathData->scanStatus.ch5, sizeof(sSwathData->scanStatus.ch5));
  ptr += sizeof(sSwathData->scanStatus.ch5);
  memcpy(ptr, &sSwathData->scanStatus.ch6, sizeof(sSwathData->scanStatus.ch6));
  ptr += sizeof(sSwathData->scanStatus.ch6);
  memcpy(ptr, &sSwathData->scanStatus.ch7, sizeof(sSwathData->scanStatus.ch7));
  ptr += sizeof(sSwathData->scanStatus.ch7);
  memcpy(ptr, &sSwathData->scanStatus.ch8, sizeof(sSwathData->scanStatus.ch8));
  ptr += sizeof(sSwathData->scanStatus.ch8);
  memcpy(ptr, &sSwathData->scanStatus.ch9, sizeof(sSwathData->scanStatus.ch9));
  ptr += sizeof(sSwathData->scanStatus.ch9);
  memcpy(ptr, &sSwathData->scanStatus.scOrient, 
		sizeof(sSwathData->scanStatus.scOrient));
  ptr += sizeof(sSwathData->scanStatus.scOrient);
  memcpy(ptr, &sSwathData->scanStatus.acsMode, 
		sizeof(sSwathData->scanStatus.acsMode));
  ptr += sizeof(sSwathData->scanStatus.acsMode);
  memcpy(ptr, &sSwathData->scanStatus.yawUpStat, 
		sizeof(sSwathData->scanStatus.yawUpStat));
  ptr += sizeof(sSwathData->scanStatus.yawUpStat);
  memcpy(ptr, &sSwathData->scanStatus.tmiIsStatus, 
		sizeof(sSwathData->scanStatus.tmiIsStatus));
  ptr += sizeof(sSwathData->scanStatus.tmiIsStatus);
  memcpy(ptr, &sSwathData->scanStatus.fractOrbitN, 
		sizeof(sSwathData->scanStatus.fractOrbitN));
  /* Writing packed vdata, scanStatus block, to an HDF file */
  vdataWrite (&tempVdata,  databuf);
  free(databuf);


  /* Allocate memory for packing the vdata */
  databuf = allocateVdataMemory(&tempVdata->vdataFields);
  ptr = databuf;
  memcpy(ptr, &sSwathData->navigate.scPosX, sizeof(sSwathData->navigate.scPosX));
  ptr += sizeof(sSwathData->navigate.scPosX);
  memcpy(ptr, &sSwathData->navigate.scPosY, sizeof(sSwathData->navigate.scPosY));
  ptr += sizeof(sSwathData->navigate.scPosY);
  memcpy(ptr, &sSwathData->navigate.scPosZ, sizeof(sSwathData->navigate.scPosZ));
  ptr += sizeof(sSwathData->navigate.scPosZ);
  memcpy(ptr, &sSwathData->navigate.scVelX, sizeof(sSwathData->navigate.scVelX));
  ptr += sizeof(sSwathData->navigate.scVelX);
  memcpy(ptr, &sSwathData->navigate.scVelY, sizeof(sSwathData->navigate.scVelY));
  ptr += sizeof(sSwathData->navigate.scVelY);
  memcpy(ptr, &sSwathData->navigate.scVelZ, sizeof(sSwathData->navigate.scVelZ));
  ptr += sizeof(sSwathData->navigate.scVelZ);
  memcpy(ptr, &sSwathData->navigate.scLat, sizeof(sSwathData->navigate.scLat));
  ptr += sizeof(sSwathData->navigate.scLat);
  memcpy(ptr, &sSwathData->navigate.scLon, sizeof(sSwathData->navigate.scLon));
  ptr += sizeof(sSwathData->navigate.scLon);
  memcpy(ptr, &sSwathData->navigate.scAlt, sizeof(sSwathData->navigate.scAlt));
  ptr += sizeof(sSwathData->navigate.scAlt);
  memcpy(ptr, &sSwathData->navigate.scAttRoll, 
		sizeof(sSwathData->navigate.scAttRoll));
  ptr += sizeof(sSwathData->navigate.scAttRoll);
  memcpy(ptr, &sSwathData->navigate.scAttPitch, 
		sizeof(sSwathData->navigate.scAttPitch));
  ptr += sizeof(sSwathData->navigate.scAttPitch);
  memcpy(ptr, &sSwathData->navigate.scAttYaw, sizeof(sSwathData->navigate.scAttYaw));
  ptr += sizeof(sSwathData->navigate.scAttYaw);
  memcpy(ptr, &sSwathData->navigate.att1, sizeof(sSwathData->navigate.att1));
  ptr += sizeof(sSwathData->navigate.att1);
  memcpy(ptr, &sSwathData->navigate.att2, sizeof(sSwathData->navigate.att2));
  ptr += sizeof(sSwathData->navigate.att2);
  memcpy(ptr, &sSwathData->navigate.att3, sizeof(sSwathData->navigate.att3));
  ptr += sizeof(sSwathData->navigate.att3);
  memcpy(ptr, &sSwathData->navigate.att4, sizeof(sSwathData->navigate.att4));
  ptr += sizeof(sSwathData->navigate.att4);
  memcpy(ptr, &sSwathData->navigate.att5, sizeof(sSwathData->navigate.att5));
  ptr += sizeof(sSwathData->navigate.att5);
  memcpy(ptr, &sSwathData->navigate.att6, sizeof(sSwathData->navigate.att6));
  ptr += sizeof(sSwathData->navigate.att6);
  memcpy(ptr, &sSwathData->navigate.att7, sizeof(sSwathData->navigate.att7));
  ptr += sizeof(sSwathData->navigate.att7);
  memcpy(ptr, &sSwathData->navigate.att8, sizeof(sSwathData->navigate.att8));
  ptr += sizeof(sSwathData->navigate.att8);
  memcpy(ptr, &sSwathData->navigate.att9, sizeof(sSwathData->navigate.att9));
  ptr += sizeof(sSwathData->navigate.att9);
  memcpy(ptr, &sSwathData->navigate.greenHourAng, 
		sizeof(sSwathData->navigate.greenHourAng));
  /* Writing packed vdata, navigate block, to an HDF file */
  vdataWrite (&tempVdata, databuf);
  free (databuf);


  /*
   * Scaling calculations.  For writing,
   *	Stored value = (int16)RND((value - OFFSET) * SCALING_FACTOR)
   * The calculation will be for the Hot Load Temperatures, 85 GHz Temperatures,
   * and the Top Radiator Temperature.
   */

  /*  Hot Load Temperatures (1, 2, and 3)  */
  if (sSwathData->calib.hotTemp1 > TK_DEFAULT)
    sSwathData->calib.hotTemp1_scale = 
      (int16)RND((sSwathData->calib.hotTemp1 - 80) * L1B11_CALIBRATION);
  else
    sSwathData->calib.hotTemp1_scale = TK_MISS_I2;

  if (sSwathData->calib.hotTemp2 > TK_DEFAULT)
    sSwathData->calib.hotTemp2_scale = 
      (int16)RND((sSwathData->calib.hotTemp2 - 80) * L1B11_CALIBRATION);
  else
    sSwathData->calib.hotTemp2_scale = TK_MISS_I2;

  if (sSwathData->calib.hotTemp3 > TK_DEFAULT)
    sSwathData->calib.hotTemp3_scale = 
      (int16)RND((sSwathData->calib.hotTemp3 - 80) * L1B11_CALIBRATION);
  else
    sSwathData->calib.hotTemp3_scale = TK_MISS_I2;

  /*  85 GHz Temperature  */
  if (sSwathData->calib.temp85Ghz > TK_DEFAULT)
    sSwathData->calib.temp85Ghz_scale = 
      (int16)RND((sSwathData->calib.temp85Ghz + 200) * L1B11_CALIBRATION);
  else
    sSwathData->calib.temp85Ghz_scale = TK_MISS_I2;

  /*  Top Radiator Temperature   */
  if (sSwathData->calib.topRadTemp > TK_DEFAULT)
    sSwathData->calib.topRadTemp_scale = 
      (int16)RND((sSwathData->calib.topRadTemp + 200) * L1B11_CALIBRATION);
  else
    sSwathData->calib.topRadTemp_scale = TK_MISS_I2;

  /* Allocate memory for packig the vdata */
  databuf = allocateVdataMemory(&tempVdata->vdataFields);
  ptr = databuf;
  memcpy(ptr, &sSwathData->calib.hotTemp1_scale, sizeof(sSwathData->calib.hotTemp1_scale));
  ptr += sizeof (sSwathData->calib.hotTemp1_scale);
  memcpy(ptr, &sSwathData->calib.hotTemp2_scale, sizeof(sSwathData->calib.hotTemp2_scale));
  ptr += sizeof (sSwathData->calib.hotTemp2_scale);
  memcpy(ptr, &sSwathData->calib.hotTemp3_scale, sizeof(sSwathData->calib.hotTemp3_scale));
  ptr += sizeof (sSwathData->calib.hotTemp3_scale);
  memcpy(ptr, &sSwathData->calib.posBridgeVolt, 
		sizeof(sSwathData->calib.posBridgeVolt));
  ptr += sizeof (sSwathData->calib.posBridgeVolt);
  memcpy(ptr, &sSwathData->calib.nearZeroVolt, 
		sizeof(sSwathData->calib.nearZeroVolt));
  ptr += sizeof (sSwathData->calib.nearZeroVolt);
  memcpy(ptr, &sSwathData->calib.temp85Ghz_scale, sizeof(sSwathData->calib.temp85Ghz_scale));
  ptr += sizeof (sSwathData->calib.temp85Ghz_scale);
  memcpy(ptr, &sSwathData->calib.topRadTemp_scale, sizeof(sSwathData->calib.topRadTemp_scale));
  ptr += sizeof (sSwathData->calib.topRadTemp_scale);
  memcpy(ptr, &sSwathData->calib.autoCont1, sizeof(sSwathData->calib.autoCont1));
  ptr += sizeof (sSwathData->calib.autoCont1);
  memcpy(ptr, &sSwathData->calib.autoCont2, sizeof(sSwathData->calib.autoCont2));
  ptr += sizeof (sSwathData->calib.autoCont2);
  memcpy(ptr, &sSwathData->calib.autoCont3, sizeof(sSwathData->calib.autoCont3));
  ptr += sizeof (sSwathData->calib.autoCont3);
  memcpy(ptr, &sSwathData->calib.autoCont4, sizeof(sSwathData->calib.autoCont4));
  ptr += sizeof (sSwathData->calib.autoCont4);
  memcpy(ptr, &sSwathData->calib.autoCont5, sizeof(sSwathData->calib.autoCont5));
  ptr += sizeof (sSwathData->calib.autoCont5);
  memcpy(ptr, &sSwathData->calib.autoCont6, sizeof(sSwathData->calib.autoCont6));
  ptr += sizeof (sSwathData->calib.autoCont6);
  memcpy(ptr, &sSwathData->calib.autoCont7, sizeof(sSwathData->calib.autoCont7));
  ptr += sizeof (sSwathData->calib.autoCont7);
  memcpy(ptr, &sSwathData->calib.autoCont8, sizeof(sSwathData->calib.autoCont8));
  ptr += sizeof (sSwathData->calib.autoCont8);
  memcpy(ptr, &sSwathData->calib.autoCont9, sizeof(sSwathData->calib.autoCont9));
  ptr += sizeof (sSwathData->calib.autoCont9);
  memcpy(ptr, &sSwathData->calib.calCoef1A, sizeof(sSwathData->calib.calCoef1A));
  ptr += sizeof (sSwathData->calib.calCoef1A);
  memcpy(ptr, &sSwathData->calib.calCoef2A, sizeof(sSwathData->calib.calCoef2A));
  ptr += sizeof (sSwathData->calib.calCoef2A);
  memcpy(ptr, &sSwathData->calib.calCoef3A, sizeof(sSwathData->calib.calCoef3A));
  ptr += sizeof (sSwathData->calib.calCoef3A);
  memcpy(ptr, &sSwathData->calib.calCoef4A, sizeof(sSwathData->calib.calCoef4A));
  ptr += sizeof (sSwathData->calib.calCoef4A);
  memcpy(ptr, &sSwathData->calib.calCoef5A, sizeof(sSwathData->calib.calCoef5A));
  ptr += sizeof (sSwathData->calib.calCoef5A);
  memcpy(ptr, &sSwathData->calib.calCoef6A, sizeof(sSwathData->calib.calCoef6A));
  ptr += sizeof (sSwathData->calib.calCoef6A);
  memcpy(ptr, &sSwathData->calib.calCoef7A, sizeof(sSwathData->calib.calCoef7A));
  ptr += sizeof (sSwathData->calib.calCoef7A);
  memcpy(ptr, &sSwathData->calib.calCoef8A, sizeof(sSwathData->calib.calCoef8A));
  ptr += sizeof (sSwathData->calib.calCoef8A);
  memcpy(ptr, &sSwathData->calib.calCoef9A, sizeof(sSwathData->calib.calCoef9A));
  ptr += sizeof (sSwathData->calib.calCoef9A);

  memcpy(ptr, &sSwathData->calib.calCoef1B, sizeof(sSwathData->calib.calCoef1B));
  ptr += sizeof (sSwathData->calib.calCoef1B);
  memcpy(ptr, &sSwathData->calib.calCoef2B, sizeof(sSwathData->calib.calCoef2B));
  ptr += sizeof (sSwathData->calib.calCoef2B);
  memcpy(ptr, &sSwathData->calib.calCoef3B, sizeof(sSwathData->calib.calCoef3B));
  ptr += sizeof (sSwathData->calib.calCoef3B);
  memcpy(ptr, &sSwathData->calib.calCoef4B, sizeof(sSwathData->calib.calCoef4B));
  ptr += sizeof (sSwathData->calib.calCoef4B);
  memcpy(ptr, &sSwathData->calib.calCoef5B, sizeof(sSwathData->calib.calCoef5B));
  ptr += sizeof (sSwathData->calib.calCoef5B);
  memcpy(ptr, &sSwathData->calib.calCoef6B, sizeof(sSwathData->calib.calCoef6B));
  ptr += sizeof (sSwathData->calib.calCoef6B);
  memcpy(ptr, &sSwathData->calib.calCoef7B, sizeof(sSwathData->calib.calCoef7B));
  ptr += sizeof (sSwathData->calib.calCoef7B);
  memcpy(ptr, &sSwathData->calib.calCoef8B, sizeof(sSwathData->calib.calCoef8B));
  ptr += sizeof (sSwathData->calib.calCoef8B);
  memcpy(ptr, &sSwathData->calib.calCoef9B, sizeof(sSwathData->calib.calCoef9B));
  /* Writing packed vdata, calib block to an HDF file  */
  vdataWrite(&tempVdata,databuf);
  free(databuf);


  /* Write SDS into HDF format */
  sdsWrite(&tempSDS, &sSwathData->calCounts);
  sdsWrite(&tempSDS, &sSwathData->satLocZenAngle);

  /*
   * Scaling calculations.  For writing,
   *	Stored value = (int16)RND((T - 100K) * L1B11_LOWRESCH)
   */
  for (i = 0; i < 104; ++i)
    for (j = 0; j < 7; ++j)
    {
        /* The Following code was added to take care of the Missing or 
           invalid data.  The reason to add this code was not to scale
           the invalid Data */

        if (sSwathData->lowResCh[i][j] > TK_DEFAULT)
	    sSwathData->lowResCh_scale[i][j] = 
		(int16)RND((sSwathData->lowResCh[i][j] - 100) * L1B11_LOWRESCH);
	else
	    sSwathData->lowResCh_scale[i][j] = TK_MISS_I2;
    }
  sdsWrite(&tempSDS, &sSwathData->lowResCh_scale);

  /*
   * Scaling calculations.  For writing, 
   *    Stored value = (int16)RND((T - 100K) * L1B11_HIGHRESCH) 
   */
  for (i = 0; i < 208; ++i)
    for (j = 0; j < 2; ++j)
    {
        /* The Following code was added to take care of the Missing or
           invalid data.  The reason to add this code was not to scale
           the invalid Data */

        if (sSwathData->highResCh[i][j] > TK_DEFAULT)
           sSwathData->highResCh_scale[i][j] = 
	       (int16)RND((sSwathData->highResCh[i][j] - 100) * L1B11_HIGHRESCH);
	else   
	    sSwathData->highResCh_scale[i][j] = TK_MISS_I2;
    }
  sdsWrite(&tempSDS, &sSwathData->highResCh_scale);
  
  return(TK_SUCCESS);
}

int writeScan_1B_11_R1(IO_HANDLE *sGranuleHandle, L1B_11_R1_SWATHDATA *sSwathData)
{

  SDS_LINK *tempSDS = sGranuleHandle->cfgSTR->vGroup->subVgroup->sds;
  VDATA_LINK *tempVdata = sGranuleHandle->cfgSTR->vGroup->subVgroup->vdata;
  uint8 *databuf, *ptr;
  int	i, j;

  /* Allocate memory for packing the vdata */
  databuf = allocateVdataMemory(&tempVdata->vdataFields);
  ptr = databuf;
  memcpy (ptr, &sSwathData->scanTime.year, sizeof(sSwathData->scanTime.year));
  ptr += sizeof(sSwathData->scanTime.year);
  memcpy (ptr, &sSwathData->scanTime.month, sizeof(sSwathData->scanTime.month));
  ptr += sizeof(sSwathData->scanTime.month);
  memcpy (ptr, &sSwathData->scanTime.dayOfMonth, sizeof(sSwathData->scanTime.dayOfMonth));
  ptr += sizeof(sSwathData->scanTime.dayOfMonth);
  memcpy (ptr, &sSwathData->scanTime.hour, sizeof(sSwathData->scanTime.hour));
  ptr += sizeof(sSwathData->scanTime.hour);
  memcpy (ptr, &sSwathData->scanTime.minute, sizeof(sSwathData->scanTime.minute));
  ptr += sizeof(sSwathData->scanTime.minute);
  memcpy (ptr, &sSwathData->scanTime.second, sizeof(sSwathData->scanTime.second));
  ptr += sizeof(sSwathData->scanTime.second);
  memcpy (ptr, &sSwathData->scanTime.dayOfYear, sizeof(sSwathData->scanTime.dayOfYear));
  /* Writing packed vdata, scanTime, to an HDF file */
  vdataWrite(&tempVdata, databuf);
  free(databuf);

  for ( i = 0; i < 208; ++i )
    for ( j = 0; j < 2; ++j )
      {
	if ( sSwathData->geolocation[i][j] > TK_DEFAULT )
	  sSwathData->geolocation_scale[i][j] = 
	    (int16)RND( sSwathData->geolocation[i][j] * GEOLOC_SCALE );
	else
	  sSwathData->geolocation_scale[i][j] = TK_MISS_I2;
      }
  sdsWrite(&tempSDS, &sSwathData->geolocation_scale);

  for ( i = 0; i < 12; ++i )
    {
      if ( sSwathData->satLocZenAngle[i] > TK_DEFAULT )
	sSwathData->satLocZenAngle_scale[i] = 
	  (int16)RND( sSwathData->satLocZenAngle[i] * L1B11_SATLOCZENANGLE );
      else
	sSwathData->satLocZenAngle_scale[i] = TK_MISS_I2;
    }
  sdsWrite(&tempSDS, &sSwathData->satLocZenAngle_scale);

  /*
   * Scaling calculations.  For writing,
   *	Stored value = (int16)RND((T - 100K) * L1B11_LOWRESCH)
   */
  for (i = 0; i < 104; ++i)
    for (j = 0; j < 7; ++j)
    {
        /* The Following code was added to take care of the Missing or 
           invalid data.  The reason to add this code was not to scale
           the invalid Data */

        if (sSwathData->lowResCh[i][j] > TK_DEFAULT)
	    sSwathData->lowResCh_scale[i][j] = 
		(int16)RND((sSwathData->lowResCh[i][j] - 100) * L1B11_LOWRESCH);
	else
	    sSwathData->lowResCh_scale[i][j] = TK_MISS_I2;
    }
  sdsWrite(&tempSDS, &sSwathData->lowResCh_scale);

  /*
   * Scaling calculations.  For writing, 
   *    Stored value = (int16)RND((T - 100K) * L1B11_HIGHRESCH) 
   */
  for (i = 0; i < 208; ++i)
    for (j = 0; j < 2; ++j)
    {
        /* The Following code was added to take care of the Missing or
           invalid data.  The reason to add this code was not to scale
           the invalid Data */

        if (sSwathData->highResCh[i][j] > TK_DEFAULT)
           sSwathData->highResCh_scale[i][j] = 
	       (int16)RND((sSwathData->highResCh[i][j] - 100) * L1B11_HIGHRESCH);
	else   
	    sSwathData->highResCh_scale[i][j] = TK_MISS_I2;
    }
  sdsWrite(&tempSDS, &sSwathData->highResCh_scale);
  
  return(TK_SUCCESS);
}

/***********************************************************************************
*
*  FUNCTION STATEMENT:
*       readScan_1B_11(IO_HANDLE *sGranuleHandle, L1B_11_SWATHDATA *sSwathData)
*
*  DESCRIPTION:
*       This function is used to read the L1B-11 information in HDF format.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       sGranuleHandle          A structure containing information about the file.
*                               sGranuleHandle is returned by TKopen().
*
*  OUTPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       sSwathData              A structure containing the complete scanline data.
*                               This structure must be declared to correspond to
*                               the data file being read.  The structure of
*                               sSwathData is manipulated by TKreadScan() and other
*                               toolkit routines.
*
*  CALLED BY:                   TKreadScan()
*
*  ROUTINES CALLED:
*       Routine Name            Description
*       ------------            -----------
*       allocateVdataMemory     This routine is used to allocate memory to vdata.
*       vdataRead               This routine is used to read the vdata in HDF
*                               format.
*       sdsRead                 This routine is used to read the scientific data
*                               set in HDF format.
*
************************************************************************************/
int readScan_1B_11(IO_HANDLE *sGranuleHandle, L1B_11_SWATHDATA *sSwathData)
{

  SDS_LINK *tempSDS = sGranuleHandle->cfgSTR->vGroup->subVgroup->sds;
  VDATA_LINK *tempVdata = sGranuleHandle->cfgSTR->vGroup->subVgroup->vdata;
  uint8 *databuf, *ptr;
  int 	i, j;
  int scaleFactor;


  /*  Allocate memory for unpacking the vdata  */
  databuf = allocateVdataMemory(&tempVdata->vdataFields);
  ptr = databuf;
  /* Unpack the content of databuf into scanTime */
  vdataRead(&tempVdata, ptr);
  memcpy (&sSwathData->scanTime.year, ptr, sizeof(sSwathData->scanTime.year));
  ptr += sizeof(sSwathData->scanTime.year);
  memcpy (&sSwathData->scanTime.month, ptr, sizeof(sSwathData->scanTime.month));
  ptr += sizeof(sSwathData->scanTime.month);
  memcpy (&sSwathData->scanTime.dayOfMonth, ptr, 
		sizeof(sSwathData->scanTime.dayOfMonth));
  ptr += sizeof(sSwathData->scanTime.dayOfMonth);
  memcpy (&sSwathData->scanTime.hour, ptr, sizeof(sSwathData->scanTime.hour));
  ptr += sizeof(sSwathData->scanTime.hour);
  memcpy (&sSwathData->scanTime.minute, ptr, sizeof(sSwathData->scanTime.minute));
  ptr += sizeof(sSwathData->scanTime.minute);
  memcpy (&sSwathData->scanTime.second, ptr, sizeof(sSwathData->scanTime.second));
  ptr += sizeof(sSwathData->scanTime.second);
  memcpy (&sSwathData->scanTime.dayOfYear, ptr, 
		sizeof(sSwathData->scanTime.dayOfYear));
  free(databuf);

  sdsRead(&tempSDS, &scaleFactor, &sSwathData->geolocation);

  /* Allocate memory for unpacking the vdata */
  databuf = allocateVdataMemory(&tempVdata->vdataFields);
  ptr = databuf;

  /* Unpack the content of databuf into scanStatus bloc */  
  vdataRead(&tempVdata, ptr);
  memcpy(&sSwathData->scanStatus.missing, ptr,
                sizeof(sSwathData->scanStatus.missing));
  ptr += sizeof(sSwathData->scanStatus.missing);
  memcpy(&sSwathData->scanStatus.validity, ptr, 
		sizeof(sSwathData->scanStatus.validity));
  ptr += sizeof(sSwathData->scanStatus.validity);
  memcpy(&sSwathData->scanStatus.qac, ptr, sizeof(sSwathData->scanStatus.qac));
  ptr += sizeof(sSwathData->scanStatus.qac);
  memcpy(&sSwathData->scanStatus.geoQuality, ptr, 
		sizeof(sSwathData->scanStatus.geoQuality));
  ptr += sizeof(sSwathData->scanStatus.geoQuality);
  memcpy(&sSwathData->scanStatus.ch1, ptr, sizeof(sSwathData->scanStatus.ch1));
  ptr += sizeof(sSwathData->scanStatus.ch1);
  memcpy(&sSwathData->scanStatus.ch2, ptr, sizeof(sSwathData->scanStatus.ch2));
  ptr += sizeof(sSwathData->scanStatus.ch2);
  memcpy(&sSwathData->scanStatus.ch3, ptr, sizeof(sSwathData->scanStatus.ch3));
  ptr += sizeof(sSwathData->scanStatus.ch3);
  memcpy(&sSwathData->scanStatus.ch4, ptr, sizeof(sSwathData->scanStatus.ch4));
  ptr += sizeof(sSwathData->scanStatus.ch4);
  memcpy(&sSwathData->scanStatus.ch5, ptr, sizeof(sSwathData->scanStatus.ch5));
  ptr += sizeof(sSwathData->scanStatus.ch5);
  memcpy(&sSwathData->scanStatus.ch6, ptr, sizeof(sSwathData->scanStatus.ch6));
  ptr += sizeof(sSwathData->scanStatus.ch6);
  memcpy(&sSwathData->scanStatus.ch7, ptr, sizeof(sSwathData->scanStatus.ch7));
  ptr += sizeof(sSwathData->scanStatus.ch7);
  memcpy(&sSwathData->scanStatus.ch8, ptr, sizeof(sSwathData->scanStatus.ch8));
  ptr += sizeof(sSwathData->scanStatus.ch8);
  memcpy(&sSwathData->scanStatus.ch9, ptr, sizeof(sSwathData->scanStatus.ch9));
  ptr += sizeof(sSwathData->scanStatus.ch9);
  memcpy(&sSwathData->scanStatus.scOrient, ptr, 
		sizeof(sSwathData->scanStatus.scOrient));
  ptr += sizeof(sSwathData->scanStatus.scOrient);
  memcpy(&sSwathData->scanStatus.acsMode, ptr, 
		sizeof(sSwathData->scanStatus.acsMode));
  ptr += sizeof(sSwathData->scanStatus.acsMode);
  memcpy(&sSwathData->scanStatus.yawUpStat, ptr, 
		sizeof(sSwathData->scanStatus.yawUpStat));
  ptr += sizeof(sSwathData->scanStatus.yawUpStat);
  memcpy(&sSwathData->scanStatus.tmiIsStatus , ptr, 
		sizeof(sSwathData->scanStatus.tmiIsStatus));
  ptr += sizeof(sSwathData->scanStatus.tmiIsStatus);
  memcpy(&sSwathData->scanStatus.fractOrbitN, ptr, 
		sizeof(sSwathData->scanStatus.fractOrbitN));
  free(databuf);


  /* Allocate memory for unpacking the vdata */
  databuf = allocateVdataMemory(&tempVdata->vdataFields);
  ptr = databuf;
  /* Unpack the content of databuf inro navigate block  */
  vdataRead(&tempVdata, ptr);
  memcpy(&sSwathData->navigate.scPosX, ptr, sizeof(sSwathData->navigate.scPosX));
  ptr += sizeof(sSwathData->navigate.scPosX);
  memcpy(&sSwathData->navigate.scPosY, ptr, sizeof(sSwathData->navigate.scPosY));
  ptr += sizeof(sSwathData->navigate.scPosY);
  memcpy(&sSwathData->navigate.scPosZ, ptr, sizeof(sSwathData->navigate.scPosZ));
  ptr += sizeof(sSwathData->navigate.scPosZ);
  memcpy(&sSwathData->navigate.scVelX, ptr, sizeof(sSwathData->navigate.scVelX));
  ptr += sizeof(sSwathData->navigate.scVelX);
  memcpy(&sSwathData->navigate.scVelY, ptr, sizeof(sSwathData->navigate.scVelY));
  ptr += sizeof(sSwathData->navigate.scVelY);
  memcpy(&sSwathData->navigate.scVelZ, ptr, sizeof(sSwathData->navigate.scVelZ));
  ptr += sizeof(sSwathData->navigate.scVelZ);
  memcpy(&sSwathData->navigate.scLat, ptr, sizeof(sSwathData->navigate.scLat));
  ptr += sizeof(sSwathData->navigate.scLat);
  memcpy(&sSwathData->navigate.scLon, ptr, sizeof(sSwathData->navigate.scLon));
  ptr += sizeof(sSwathData->navigate.scLon);
  memcpy(&sSwathData->navigate.scAlt, ptr, sizeof(sSwathData->navigate.scAlt));
  ptr += sizeof(sSwathData->navigate.scAlt);
  memcpy(&sSwathData->navigate.scAttRoll, ptr, 
		sizeof(sSwathData->navigate.scAttRoll));
  ptr += sizeof(sSwathData->navigate.scAttRoll);
  memcpy(&sSwathData->navigate.scAttPitch, ptr, 
		sizeof(sSwathData->navigate.scAttPitch));
  ptr += sizeof(sSwathData->navigate.scAttPitch);
  memcpy(&sSwathData->navigate.scAttYaw, ptr, sizeof(sSwathData->navigate.scAttYaw));
  ptr += sizeof(sSwathData->navigate.scAttYaw);
  memcpy(&sSwathData->navigate.att1, ptr, sizeof(sSwathData->navigate.att1));
  ptr += sizeof(sSwathData->navigate.att1);
  memcpy(&sSwathData->navigate.att2, ptr, sizeof(sSwathData->navigate.att2));
  ptr += sizeof(sSwathData->navigate.att2);
  memcpy(&sSwathData->navigate.att3, ptr, sizeof(sSwathData->navigate.att3));
  ptr += sizeof(sSwathData->navigate.att3);
  memcpy(&sSwathData->navigate.att4, ptr, sizeof(sSwathData->navigate.att4));
  ptr += sizeof(sSwathData->navigate.att4);
  memcpy(&sSwathData->navigate.att5, ptr, sizeof(sSwathData->navigate.att5));
  ptr += sizeof(sSwathData->navigate.att5);
  memcpy(&sSwathData->navigate.att6, ptr, sizeof(sSwathData->navigate.att6));
  ptr += sizeof(sSwathData->navigate.att6);
  memcpy(&sSwathData->navigate.att7, ptr, sizeof(sSwathData->navigate.att7));
  ptr += sizeof(sSwathData->navigate.att7);
  memcpy(&sSwathData->navigate.att8, ptr, sizeof(sSwathData->navigate.att8));
  ptr += sizeof(sSwathData->navigate.att8);
  memcpy(&sSwathData->navigate.att9, ptr, sizeof(sSwathData->navigate.att9));
  ptr += sizeof(sSwathData->navigate.att9);
  memcpy(&sSwathData->navigate.greenHourAng, ptr, 
		sizeof(sSwathData->navigate.greenHourAng));
  free(databuf);


  /* Allocate memory for unpacking the vdata */
  databuf = allocateVdataMemory (&tempVdata->vdataFields);
  ptr = databuf;
  /* Unpack the content fo databuf into calib block */
  vdataRead(&tempVdata, ptr);
  memcpy(&sSwathData->calib.hotTemp1_scale, ptr, sizeof(sSwathData->calib.hotTemp1_scale));
  ptr += sizeof(sSwathData->calib.hotTemp1_scale);
  memcpy(&sSwathData->calib.hotTemp2_scale, ptr, sizeof(sSwathData->calib.hotTemp2_scale));
  ptr += sizeof(sSwathData->calib.hotTemp2_scale);
  memcpy(&sSwathData->calib.hotTemp3_scale, ptr, sizeof(sSwathData->calib.hotTemp3_scale));
  ptr += sizeof(sSwathData->calib.hotTemp3_scale);
  memcpy(&sSwathData->calib.posBridgeVolt, ptr, 
		sizeof(sSwathData->calib.posBridgeVolt));
  ptr += sizeof(sSwathData->calib.posBridgeVolt);
  memcpy(&sSwathData->calib.nearZeroVolt, ptr, 
		sizeof(sSwathData->calib.nearZeroVolt));
  ptr += sizeof(sSwathData->calib.nearZeroVolt);
  memcpy(&sSwathData->calib.temp85Ghz_scale, ptr, sizeof(sSwathData->calib.temp85Ghz_scale));
  ptr += sizeof(sSwathData->calib.temp85Ghz_scale);
  memcpy(&sSwathData->calib.topRadTemp_scale, ptr, sizeof(sSwathData->calib.topRadTemp_scale));
  ptr += sizeof(sSwathData->calib.topRadTemp_scale);
  memcpy(&sSwathData->calib.autoCont1, ptr, sizeof(sSwathData->calib.autoCont1));
  ptr += sizeof(sSwathData->calib.autoCont1);
  memcpy(&sSwathData->calib.autoCont2, ptr, sizeof(sSwathData->calib.autoCont2));
  ptr += sizeof(sSwathData->calib.autoCont2);
  memcpy(&sSwathData->calib.autoCont3, ptr, sizeof(sSwathData->calib.autoCont3));
  ptr += sizeof(sSwathData->calib.autoCont3);
  memcpy(&sSwathData->calib.autoCont4, ptr, sizeof(sSwathData->calib.autoCont4));
  ptr += sizeof(sSwathData->calib.autoCont4);
  memcpy(&sSwathData->calib.autoCont5, ptr, sizeof(sSwathData->calib.autoCont5));
  ptr += sizeof(sSwathData->calib.autoCont5);
  memcpy(&sSwathData->calib.autoCont6, ptr, sizeof(sSwathData->calib.autoCont6));
  ptr += sizeof(sSwathData->calib.autoCont6);
  memcpy(&sSwathData->calib.autoCont7, ptr, sizeof(sSwathData->calib.autoCont7));
  ptr += sizeof(sSwathData->calib.autoCont7);
  memcpy(&sSwathData->calib.autoCont8, ptr, sizeof(sSwathData->calib.autoCont8));
  ptr += sizeof(sSwathData->calib.autoCont8);
  memcpy(&sSwathData->calib.autoCont9, ptr, sizeof(sSwathData->calib.autoCont9));
  ptr += sizeof(sSwathData->calib.autoCont9);
  memcpy(&sSwathData->calib.calCoef1A, ptr, sizeof(sSwathData->calib.calCoef1A));
  ptr += sizeof(sSwathData->calib.calCoef1A);
  memcpy(&sSwathData->calib.calCoef2A, ptr, sizeof(sSwathData->calib.calCoef2A));
  ptr += sizeof(sSwathData->calib.calCoef2A);
  memcpy(&sSwathData->calib.calCoef3A, ptr, sizeof(sSwathData->calib.calCoef3A));
  ptr += sizeof(sSwathData->calib.calCoef3A);
  memcpy(&sSwathData->calib.calCoef4A, ptr, sizeof(sSwathData->calib.calCoef4A));
  ptr += sizeof(sSwathData->calib.calCoef4A);
  memcpy(&sSwathData->calib.calCoef5A, ptr, sizeof(sSwathData->calib.calCoef5A));
  ptr += sizeof(sSwathData->calib.calCoef5A);
  memcpy(&sSwathData->calib.calCoef6A, ptr, sizeof(sSwathData->calib.calCoef6A));
  ptr += sizeof(sSwathData->calib.calCoef6A);
  memcpy(&sSwathData->calib.calCoef7A, ptr, sizeof(sSwathData->calib.calCoef7A));
  ptr += sizeof(sSwathData->calib.calCoef7A);
  memcpy(&sSwathData->calib.calCoef8A, ptr, sizeof(sSwathData->calib.calCoef8A));
  ptr += sizeof(sSwathData->calib.calCoef8A);
  memcpy(&sSwathData->calib.calCoef9A, ptr, sizeof(sSwathData->calib.calCoef9A));
  ptr += sizeof(sSwathData->calib.calCoef9A);

  memcpy(&sSwathData->calib.calCoef1B, ptr, sizeof(sSwathData->calib.calCoef1B));
  ptr += sizeof(sSwathData->calib.calCoef1B);
  memcpy(&sSwathData->calib.calCoef2B, ptr, sizeof(sSwathData->calib.calCoef2B));
  ptr += sizeof(sSwathData->calib.calCoef2B);
  memcpy(&sSwathData->calib.calCoef3B, ptr, sizeof(sSwathData->calib.calCoef3B));
  ptr += sizeof(sSwathData->calib.calCoef3B);
  memcpy(&sSwathData->calib.calCoef4B, ptr, sizeof(sSwathData->calib.calCoef4B));
  ptr += sizeof(sSwathData->calib.calCoef4B);
  memcpy(&sSwathData->calib.calCoef5B, ptr, sizeof(sSwathData->calib.calCoef5B));
  ptr += sizeof(sSwathData->calib.calCoef5B);
  memcpy(&sSwathData->calib.calCoef6B, ptr, sizeof(sSwathData->calib.calCoef6B));
  ptr += sizeof(sSwathData->calib.calCoef6B);
  memcpy(&sSwathData->calib.calCoef7B, ptr, sizeof(sSwathData->calib.calCoef7B));
  ptr += sizeof(sSwathData->calib.calCoef7B);
  memcpy(&sSwathData->calib.calCoef8B, ptr, sizeof(sSwathData->calib.calCoef8B));
  ptr += sizeof(sSwathData->calib.calCoef8B);
  memcpy(&sSwathData->calib.calCoef9B, ptr, sizeof(sSwathData->calib.calCoef9B));
  free(databuf);

  /*
   * Scaling calculation.  For reading,
   *	Stored value = (value / SCALING_FACTOR) + OFFSET
   * These will cover the hot load temperatures (1, 2, and 3), the 85GHz
   * temperatures, and the top radiator temperatures.
   */

  /*  Hot Load Temperature 1  */
  if (sSwathData->calib.hotTemp1_scale != TK_MISS_I2)
    sSwathData->calib.hotTemp1 =                  
      (sSwathData->calib.hotTemp1_scale * 1.0 / L1B11_CALIBRATION) + 80;
  else
    sSwathData->calib.hotTemp1 = TK_DEFAULT;

  /*  Hot Load Temperature 2  */
  if (sSwathData->calib.hotTemp2_scale != TK_MISS_I2)
    sSwathData->calib.hotTemp2 =                  
      (sSwathData->calib.hotTemp2_scale * 1.0 / L1B11_CALIBRATION) + 80;
  else
    sSwathData->calib.hotTemp2 = TK_DEFAULT;

  /*  Hot Load Temperature 3  */
  if (sSwathData->calib.hotTemp3_scale != TK_MISS_I2)
    sSwathData->calib.hotTemp3 =                  
      (sSwathData->calib.hotTemp3_scale * 1.0 / L1B11_CALIBRATION) + 80;
  else
    sSwathData->calib.hotTemp3 = TK_DEFAULT;

  /*  85 GHz Temperatures  */
  if (sSwathData->calib.temp85Ghz_scale != TK_MISS_I2)
    sSwathData->calib.temp85Ghz =                  
      (sSwathData->calib.temp85Ghz_scale * 1.0 / L1B11_CALIBRATION) - 200;
  else
    sSwathData->calib.temp85Ghz = TK_DEFAULT;

  /*  Top Radiator Temperature  */
  if (sSwathData->calib.topRadTemp_scale != TK_MISS_I2)
    sSwathData->calib.topRadTemp =                  
      (sSwathData->calib.topRadTemp_scale * 1.0 / L1B11_CALIBRATION) - 200;
  else
    sSwathData->calib.topRadTemp = TK_DEFAULT;

  /* Read SDS from HDF format */
  sdsRead(&tempSDS, &scaleFactor, &sSwathData->calCounts);
  sdsRead(&tempSDS, &scaleFactor, &sSwathData->satLocZenAngle);

  /*
   * Scaling calculation.  For reading,
   *	Stored value = (T / L1B11_LOWRESCH) + 100K
   */
  sdsRead (&tempSDS, &scaleFactor, &sSwathData->lowResCh_scale);
  for (i = 0; i < 104; ++i)
    for (j = 0; j < 7; ++j)
        {
        if (sSwathData->lowResCh_scale[i][j] != TK_MISS_I2)
	   sSwathData->lowResCh[i][j] =                  
		(sSwathData->lowResCh_scale[i][j] * 1.0 / L1B11_LOWRESCH) + 100;
        else
           sSwathData->lowResCh[i][j] = TK_DEFAULT;
        }

  /*
   * Scaling calculation.  For reading,
   *    Stored value = (T / L1B11_HIGHRESCH) + 100K
   */
  sdsRead (&tempSDS, &scaleFactor, &sSwathData->highResCh_scale);
  for (i = 0; i < 208; ++i)
    for (j = 0; j < 2; ++j)
        {                     
        if (sSwathData->highResCh_scale[i][j] != TK_MISS_I2)
           sSwathData->highResCh[i][j] = 
		(sSwathData->highResCh_scale[i][j] * 1.0 / L1B11_HIGHRESCH) + 100;
        else
           sSwathData->highResCh[i][j] = TK_DEFAULT;
        }
  return(TK_SUCCESS);
} 

int readScan_1B_11_R1(IO_HANDLE *sGranuleHandle, L1B_11_R1_SWATHDATA *sSwathData)
{

  SDS_LINK *tempSDS = sGranuleHandle->cfgSTR->vGroup->subVgroup->sds;
  VDATA_LINK *tempVdata = sGranuleHandle->cfgSTR->vGroup->subVgroup->vdata;
  uint8 *databuf, *ptr;
  int 	i, j;
  int scaleFactor;


  /*  Allocate memory for unpacking the vdata  */
  databuf = allocateVdataMemory(&tempVdata->vdataFields);
  ptr = databuf;
  /* Unpack the content of databuf into scanTime */
  vdataRead(&tempVdata, ptr);
  memcpy (&sSwathData->scanTime.year, ptr, sizeof(sSwathData->scanTime.year));
  ptr += sizeof(sSwathData->scanTime.year);
  memcpy (&sSwathData->scanTime.month, ptr, sizeof(sSwathData->scanTime.month));
  ptr += sizeof(sSwathData->scanTime.month);
  memcpy (&sSwathData->scanTime.dayOfMonth, ptr, 
		sizeof(sSwathData->scanTime.dayOfMonth));
  ptr += sizeof(sSwathData->scanTime.dayOfMonth);
  memcpy (&sSwathData->scanTime.hour, ptr, sizeof(sSwathData->scanTime.hour));
  ptr += sizeof(sSwathData->scanTime.hour);
  memcpy (&sSwathData->scanTime.minute, ptr, sizeof(sSwathData->scanTime.minute));
  ptr += sizeof(sSwathData->scanTime.minute);
  memcpy (&sSwathData->scanTime.second, ptr, sizeof(sSwathData->scanTime.second));
  ptr += sizeof(sSwathData->scanTime.second);
  memcpy (&sSwathData->scanTime.dayOfYear, ptr, 
		sizeof(sSwathData->scanTime.dayOfYear));
  free(databuf);

  sdsRead(&tempSDS, &scaleFactor, &sSwathData->geolocation_scale);
  for ( i = 0; i < 208; ++i )
    for ( j = 0; j < 2; ++j )
      {
        /*if ( sSwathData->geolocation_scale[i][j] != TK_MISS_I2 ) tk5.74*/
        if (( sSwathData->geolocation_scale[i][0] != TK_MISS_I2 )
           || ( sSwathData->geolocation_scale[i][1] != TK_MISS_I2 ))
	  sSwathData->geolocation[i][j] =
	     sSwathData->geolocation_scale[i][j] * 1.0 / scaleFactor;
	else
	  sSwathData->geolocation[i][j] = TK_DEFAULT;
      }
  
  sdsRead(&tempSDS, &scaleFactor, &sSwathData->satLocZenAngle_scale);
  
  for ( i = 0; i < 12; ++i )
    {
      if ( sSwathData->satLocZenAngle_scale[i] != TK_MISS_I2 )
	sSwathData->satLocZenAngle[i] = 
	  ( sSwathData->satLocZenAngle_scale[i] * 1.0/ L1B11_SATLOCZENANGLE );
      else
	sSwathData->satLocZenAngle[i] = TK_DEFAULT;
    }
  
  /*
   * Scaling calculation.  For reading,
   *	Stored value = (T / L1B11_LOWRESCH) + 100K
   */
  sdsRead (&tempSDS, &scaleFactor, &sSwathData->lowResCh_scale);
  for (i = 0; i < 104; ++i)
    for (j = 0; j < 7; ++j)
      {
        if (sSwathData->lowResCh_scale[i][j] != TK_MISS_I2)
	  sSwathData->lowResCh[i][j] =                  
	    (sSwathData->lowResCh_scale[i][j] * 1.0 / L1B11_LOWRESCH) + 100;
        else
	  sSwathData->lowResCh[i][j] = TK_DEFAULT;
      }
  
  /*
   * Scaling calculation.  For reading,
   *    Stored value = (T / L1B11_HIGHRESCH) + 100K
   */
  sdsRead (&tempSDS, &scaleFactor, &sSwathData->highResCh_scale);
  for (i = 0; i < 208; ++i)
    for (j = 0; j < 2; ++j)
        {                     
        if (sSwathData->highResCh_scale[i][j] != TK_MISS_I2)
           sSwathData->highResCh[i][j] = 
		(sSwathData->highResCh_scale[i][j] * 1.0 / L1B11_HIGHRESCH) + 100;
        else
           sSwathData->highResCh[i][j] = TK_DEFAULT;
        }
  return(TK_SUCCESS);
} 


/***********************************************************************************
*
*  FUNCTION STATEMENT:
*       writeScan_2A_12(IO_HANDLE *sGranuleHandle, L2A_12_SWATHDATA *sSwathData)
*
*  DESCRIPTION:
*       This function is used to write the L2A-12 information in HDF format.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       sGranuleHandle          A structure containing information about the file.
*                               sGranuleHandle is returned by TKopen().
*
*       sSwathData              A structure containing the complete scanline data.
*                               This structure must be declared to correspond to
*                               the data file being written.  The structures of
*                               sSwathData is manipulated by TKwriteScan() and
*                               other toolkit routines.
*
*  OUTPUT PARAMETERS:           None
*
*  CALLED BY:                   TKwriteScan()
*
*  ROUTINES CALLED:
*       Routine Name            Description
*       ------------            -----------
*       allocateVdataMemory     This routine is used to allocate memory to vdata.
*       vdataWrite              This routine is used to write the vdata in HDF
*                               format.
*       sdsWrite                This routine is used to write the scientific data
*                               set in HDF format.
*
***********************************************************************************/

int writeScan_2A_12(IO_HANDLE *sGranuleHandle,L2A_12_SWATHDATA *sSwathData)
{
  SDS_LINK *tempSDS = sGranuleHandle->cfgSTR->vGroup->subVgroup->sds;
  VDATA_LINK *tempVdata = sGranuleHandle->cfgSTR->vGroup->subVgroup->vdata;
  uint8 *databuf, *ptr;
  int 	i, j;

  /* Allocate memory for packing the vdata */
  databuf = allocateVdataMemory(&tempVdata->vdataFields);
  ptr = databuf;
  memcpy (ptr, &sSwathData->scanTime.year, sizeof(sSwathData->scanTime.year));
  ptr += sizeof(sSwathData->scanTime.year);
  memcpy (ptr, &sSwathData->scanTime.month, sizeof(sSwathData->scanTime.month));
  ptr += sizeof(sSwathData->scanTime.month);
  memcpy (ptr, &sSwathData->scanTime.dayOfMonth, 
		sizeof(sSwathData->scanTime.dayOfMonth));
  ptr += sizeof(sSwathData->scanTime.dayOfMonth);
  memcpy (ptr, &sSwathData->scanTime.hour, sizeof(sSwathData->scanTime.hour));
  ptr += sizeof(sSwathData->scanTime.hour);
  memcpy (ptr, &sSwathData->scanTime.minute, sizeof(sSwathData->scanTime.minute));
  ptr += sizeof(sSwathData->scanTime.minute);
  memcpy (ptr, &sSwathData->scanTime.second, sizeof(sSwathData->scanTime.second));
  ptr += sizeof(sSwathData->scanTime.second);
  memcpy (ptr, &sSwathData->scanTime.dayOfYear, 
		sizeof(sSwathData->scanTime.dayOfYear));
  /* Writing packed vdata, scanTime, to an HDF file */
  vdataWrite(&tempVdata, databuf);
  free(databuf);


  sdsWrite(&tempSDS, &sSwathData->geolocation);


  /* Allocate memory for packing the vdata  */
  databuf = allocateVdataMemory(&tempVdata->vdataFields);
  ptr = databuf;
  memcpy(ptr, &sSwathData->scanStatus.missing,
                sizeof(sSwathData->scanStatus.missing));
  ptr += sizeof(sSwathData->scanStatus.missing);
  memcpy(ptr, &sSwathData->scanStatus.validity, 
		sizeof(sSwathData->scanStatus.validity));
  ptr += sizeof(sSwathData->scanStatus.validity);
  memcpy(ptr, &sSwathData->scanStatus.qac, 
		sizeof(sSwathData->scanStatus.qac));
  ptr += sizeof(sSwathData->scanStatus.qac);
  memcpy(ptr, &sSwathData->scanStatus.geoQuality, 
		sizeof(sSwathData->scanStatus.geoQuality));
  ptr += sizeof(sSwathData->scanStatus.geoQuality);
  memcpy(ptr, &sSwathData->scanStatus.ch1, sizeof(sSwathData->scanStatus.ch1));
  ptr += sizeof(sSwathData->scanStatus.ch1);
  memcpy(ptr, &sSwathData->scanStatus.ch2, sizeof(sSwathData->scanStatus.ch2));
  ptr += sizeof(sSwathData->scanStatus.ch2);
  memcpy(ptr, &sSwathData->scanStatus.ch3, sizeof(sSwathData->scanStatus.ch3));
  ptr += sizeof(sSwathData->scanStatus.ch3);
  memcpy(ptr, &sSwathData->scanStatus.ch4, sizeof(sSwathData->scanStatus.ch4));
  ptr += sizeof(sSwathData->scanStatus.ch4);
  memcpy(ptr, &sSwathData->scanStatus.ch5, sizeof(sSwathData->scanStatus.ch5));
  ptr += sizeof(sSwathData->scanStatus.ch5);
  memcpy(ptr, &sSwathData->scanStatus.ch6, sizeof(sSwathData->scanStatus.ch6));
  ptr += sizeof(sSwathData->scanStatus.ch6);
  memcpy(ptr, &sSwathData->scanStatus.ch7, sizeof(sSwathData->scanStatus.ch7));
  ptr += sizeof(sSwathData->scanStatus.ch7);
  memcpy(ptr, &sSwathData->scanStatus.ch8, sizeof(sSwathData->scanStatus.ch8));
  ptr += sizeof(sSwathData->scanStatus.ch8);
  memcpy(ptr, &sSwathData->scanStatus.ch9, sizeof(sSwathData->scanStatus.ch9));
  ptr += sizeof(sSwathData->scanStatus.ch9);
  memcpy(ptr, &sSwathData->scanStatus.scOrient, 
		sizeof(sSwathData->scanStatus.scOrient));
  ptr += sizeof(sSwathData->scanStatus.scOrient);
  memcpy(ptr, &sSwathData->scanStatus.acsMode, 
		sizeof(sSwathData->scanStatus.acsMode));
  ptr += sizeof(sSwathData->scanStatus.acsMode);
  memcpy(ptr, &sSwathData->scanStatus.yawUpStat, 
		sizeof(sSwathData->scanStatus.yawUpStat));
  ptr += sizeof(sSwathData->scanStatus.yawUpStat);
  memcpy(ptr, &sSwathData->scanStatus.tmiIsStatus, 
		sizeof(sSwathData->scanStatus.tmiIsStatus));
  ptr += sizeof(sSwathData->scanStatus.tmiIsStatus);
  memcpy(ptr, &sSwathData->scanStatus.fractOrbitN, 
		sizeof(sSwathData->scanStatus.fractOrbitN));
  /* Writing packed vdata, scanStatus block, to an HDF file */
  vdataWrite (&tempVdata,  databuf);
  free(databuf);


  /* Allocate memory for packing the vdata */
  databuf = allocateVdataMemory(&tempVdata->vdataFields);
  ptr = databuf;
  memcpy(ptr, &sSwathData->navigate.scPosX, sizeof(sSwathData->navigate.scPosX));
  ptr += sizeof(sSwathData->navigate.scPosX);
  memcpy(ptr, &sSwathData->navigate.scPosY, sizeof(sSwathData->navigate.scPosY));
  ptr += sizeof(sSwathData->navigate.scPosY);
  memcpy(ptr, &sSwathData->navigate.scPosZ, sizeof(sSwathData->navigate.scPosZ));
  ptr += sizeof(sSwathData->navigate.scPosZ);
  memcpy(ptr, &sSwathData->navigate.scVelX, sizeof(sSwathData->navigate.scVelX));
  ptr += sizeof(sSwathData->navigate.scVelX);
  memcpy(ptr, &sSwathData->navigate.scVelY, sizeof(sSwathData->navigate.scVelY));
  ptr += sizeof(sSwathData->navigate.scVelY);
  memcpy(ptr, &sSwathData->navigate.scVelZ, sizeof(sSwathData->navigate.scVelZ));
  ptr += sizeof(sSwathData->navigate.scVelZ);
  memcpy(ptr, &sSwathData->navigate.scLat, sizeof(sSwathData->navigate.scLat));
  ptr += sizeof(sSwathData->navigate.scLat);
  memcpy(ptr, &sSwathData->navigate.scLon, sizeof(sSwathData->navigate.scLon));
  ptr += sizeof(sSwathData->navigate.scLon);
  memcpy(ptr, &sSwathData->navigate.scAlt, sizeof(sSwathData->navigate.scAlt));
  ptr += sizeof(sSwathData->navigate.scAlt);
  memcpy(ptr, &sSwathData->navigate.scAttRoll, 
		sizeof(sSwathData->navigate.scAttRoll));
  ptr += sizeof(sSwathData->navigate.scAttRoll);
  memcpy(ptr, &sSwathData->navigate.scAttPitch, 
		sizeof(sSwathData->navigate.scAttPitch));
  ptr += sizeof(sSwathData->navigate.scAttPitch);
  memcpy(ptr, &sSwathData->navigate.scAttYaw, sizeof(sSwathData->navigate.scAttYaw));
  ptr += sizeof(sSwathData->navigate.scAttYaw);
  memcpy(ptr, &sSwathData->navigate.att1, sizeof(sSwathData->navigate.att1));
  ptr += sizeof(sSwathData->navigate.att1);
  memcpy(ptr, &sSwathData->navigate.att2, sizeof(sSwathData->navigate.att2));
  ptr += sizeof(sSwathData->navigate.att2);
  memcpy(ptr, &sSwathData->navigate.att3, sizeof(sSwathData->navigate.att3));
  ptr += sizeof(sSwathData->navigate.att3);
  memcpy(ptr, &sSwathData->navigate.att4, sizeof(sSwathData->navigate.att4));
  ptr += sizeof(sSwathData->navigate.att4);
  memcpy(ptr, &sSwathData->navigate.att5, sizeof(sSwathData->navigate.att5));
  ptr += sizeof(sSwathData->navigate.att5);
  memcpy(ptr, &sSwathData->navigate.att6, sizeof(sSwathData->navigate.att6));
  ptr += sizeof(sSwathData->navigate.att6);
  memcpy(ptr, &sSwathData->navigate.att7, sizeof(sSwathData->navigate.att7));
  ptr += sizeof(sSwathData->navigate.att7);
  memcpy(ptr, &sSwathData->navigate.att8, sizeof(sSwathData->navigate.att8));
  ptr += sizeof(sSwathData->navigate.att8);
  memcpy(ptr, &sSwathData->navigate.att9, sizeof(sSwathData->navigate.att9));
  ptr += sizeof(sSwathData->navigate.att9);
  memcpy(ptr, &sSwathData->navigate.greenHourAng, 
		sizeof(sSwathData->navigate.greenHourAng));
  /* Writing packed vdata, navigate block, to an HDF file */
  vdataWrite (&tempVdata, databuf);
  free (databuf);


  /* Write SDS into HDF format */
  sdsWrite(&tempSDS, &sSwathData->dataFlag);
  sdsWrite(&tempSDS, &sSwathData->rainFlag);
  sdsWrite(&tempSDS, &sSwathData->surfaceFlag);
  sdsWrite(&tempSDS, &sSwathData->surfaceRain);
  sdsWrite(&tempSDS, &sSwathData->convectRain);
  sdsWrite(&tempSDS, &sSwathData->confidence);


  /* Write SDS into HDF format */
  /*
   * Scaling calculation.  For writing,
   *	Multiplied the data by L2A12_CLOUDWATER.
   */
  for (i = 0; i < 208; ++i)
    for(j = 0; j < 14; ++j)
    {
        /* The Following code was added to take care of the Missing or
           invalid data.  The reason to add this code was not to scale
           the invalid Data */

        if (sSwathData->cldWater[i][j] > TK_DEFAULT)
	    sSwathData->cldWater_scale[i][j] =
		(int16)RND(sSwathData->cldWater[i][j] * L2A12_CLOUDWATER);
	else
	    sSwathData->cldWater_scale[i][j] = TK_MISS_I2;
    }
  sdsWrite(&tempSDS, &sSwathData->cldWater_scale);

  /*
   * Scaling calculation.  For writing,
   *    Multiplied the data by L2A12_PRECIPWATER.
   */
  for (i = 0; i < 208; ++i)
    for(j = 0; j < 14; ++j)
    {
        /* The Following code was added to take care of the Missing or
           invalid data.  The reason to add this code was not to scale
           the invalid Data */

        if (sSwathData->precipWater[i][j] > TK_DEFAULT)
            sSwathData->precipWater_scale[i][j] =
                (int16)RND(sSwathData->precipWater[i][j] * L2A12_PRECIPWATER);
	else
            sSwathData->precipWater_scale[i][j] = TK_MISS_I2;
    }
  sdsWrite(&tempSDS, &sSwathData->precipWater_scale);

  /*
   * Scaling calculation.  For writing,
   *    Multiplied the data by L2A12_CLOUDICE.
   */
  for (i = 0; i < 208; ++i)
    for(j = 0; j < 14; ++j)
    {
        /* The Following code was added to take care of the Missing or
           invalid data.  The reason to add this code was not to scale
           the invalid Data */

        if (sSwathData->cldIce[i][j] > TK_DEFAULT)
            sSwathData->cldIce_scale[i][j] =
                (int16)RND(sSwathData->cldIce[i][j] * L2A12_CLOUDICE);
	else
            sSwathData->cldIce_scale[i][j] = TK_MISS_I2;
    }
  sdsWrite(&tempSDS, &sSwathData->cldIce_scale);

  /*
   * Scaling calculation.  For writing,
   *    Multiplied the data by L2A12_PRECIPICE.
   */
  for (i = 0; i < 208; ++i)
    for(j = 0; j < 14; ++j)
    {
        /* The Following code was added to take care of the Missing or
           invalid data.  The reason to add this code was not to scale
           the invalid Data */

        if (sSwathData->precipIce[i][j] > TK_DEFAULT)
            sSwathData->precipIce_scale[i][j] =
                (int16)RND(sSwathData->precipIce[i][j] * L2A12_PRECIPICE);
	else
            sSwathData->precipIce_scale[i][j] = TK_MISS_I2;
    }
  sdsWrite(&tempSDS, &sSwathData->precipIce_scale);

  /*
   * Scaling calculation.  For writing,
   *    Multiplied the data by L2A12_LATENTHEAT.
   */
  for (i = 0; i < 208; ++i)
    for(j = 0; j < 14; ++j)
    {
        /* The Following code was added to take care of the Missing or
           invalid data.  The reason to add this code was not to scale
           the invalid Data */

        if (sSwathData->latentHeat[i][j] > TK_DEFAULT)
            sSwathData->latentHeat_scale[i][j] =
                (int16)RND(sSwathData->latentHeat[i][j] * L2A12_LATENTHEAT);
	else
            sSwathData->latentHeat_scale[i][j] = TK_MISS_I2;
    }
  sdsWrite(&tempSDS, &sSwathData->latentHeat_scale);

  return(TK_SUCCESS);
}

int writeScan_2A_12_R1(IO_HANDLE *sGranuleHandle,L2A_12_R1_SWATHDATA *sSwathData)
{
  SDS_LINK *tempSDS = sGranuleHandle->cfgSTR->vGroup->subVgroup->sds;
  VDATA_LINK *tempVdata = sGranuleHandle->cfgSTR->vGroup->subVgroup->vdata;
  uint8 *databuf, *ptr;
  int 	i, j;

  /* Allocate memory for packing the vdata */
  databuf = allocateVdataMemory(&tempVdata->vdataFields);
  ptr = databuf;
  memcpy (ptr, &sSwathData->scanTime.year, sizeof(sSwathData->scanTime.year));
  ptr += sizeof(sSwathData->scanTime.year);
  memcpy (ptr, &sSwathData->scanTime.month, sizeof(sSwathData->scanTime.month));
  ptr += sizeof(sSwathData->scanTime.month);
  memcpy (ptr, &sSwathData->scanTime.dayOfMonth, 
		sizeof(sSwathData->scanTime.dayOfMonth));
  ptr += sizeof(sSwathData->scanTime.dayOfMonth);
  memcpy (ptr, &sSwathData->scanTime.hour, sizeof(sSwathData->scanTime.hour));
  ptr += sizeof(sSwathData->scanTime.hour);
  memcpy (ptr, &sSwathData->scanTime.minute, sizeof(sSwathData->scanTime.minute));
  ptr += sizeof(sSwathData->scanTime.minute);
  memcpy (ptr, &sSwathData->scanTime.second, sizeof(sSwathData->scanTime.second));
  ptr += sizeof(sSwathData->scanTime.second);
  memcpy (ptr, &sSwathData->scanTime.dayOfYear, 
		sizeof(sSwathData->scanTime.dayOfYear));
  /* Writing packed vdata, scanTime, to an HDF file */
  vdataWrite(&tempVdata, databuf);
  free(databuf);

  for ( i = 0; i < 208; ++i )
    for ( j = 0; j < 2; ++j )
      {
	if ( sSwathData->geolocation[i][j] > TK_DEFAULT )
	  sSwathData->geolocation_scale[i][j] = 
	    (int16)RND( sSwathData->geolocation[i][j] * GEOLOC_SCALE );
	else
	  sSwathData->geolocation_scale[i][j] = TK_MISS_I2;
      }
  sdsWrite(&tempSDS, &sSwathData->geolocation_scale);


  sdsWrite(&tempSDS, &sSwathData->rainFlag);

  for ( i = 0; i < 208; ++i )
    {
      if ( sSwathData->surfaceRain[i] > TK_DEFAULT )
	sSwathData->surfaceRain_scale[i] = 
	  (int16)RND( sSwathData->surfaceRain[i] * L2A12_SURFACERAIN );
      else
	sSwathData->surfaceRain_scale[i] = TK_MISS_I2;
    }
  sdsWrite(&tempSDS, &sSwathData->surfaceRain_scale);

  for ( i = 0; i < 208; ++i )
    {
      if ( sSwathData->convectRain[i] > TK_DEFAULT )
	sSwathData->convectRain_scale[i] = 
	  (int16)RND( sSwathData->convectRain[i] * L2A12_CONVECTRAIN );
      else
	sSwathData->convectRain_scale[i] = TK_MISS_I2;
    }
  sdsWrite(&tempSDS, &sSwathData->convectRain_scale);

  /* Write SDS into HDF format */
  /*
   * Scaling calculation.  For writing,
   *	Multiplied the data by L2A12_CLOUDWATER.
   */
  for (i = 0; i < 208; ++i)
    {
        /* The Following code was added to take care of the Missing or
           invalid data.  The reason to add this code was not to scale
           the invalid Data */

        if (sSwathData->cldWater[i] > TK_DEFAULT)
	    sSwathData->cldWater_scale[i] =
		(int16)RND(sSwathData->cldWater[i] * L2A12_CLOUDWATER);
	else
	    sSwathData->cldWater_scale[i] = TK_MISS_I2;
    }
  sdsWrite(&tempSDS, &sSwathData->cldWater_scale);

  /*
   * Scaling calculation.  For writing,
   *    Multiplied the data by L2A12_PRECIPICE.
   */
  for (i = 0; i < 208; ++i)
    {
        /* The Following code was added to take care of the Missing or
           invalid data.  The reason to add this code was not to scale
           the invalid Data */

        if (sSwathData->precipIce[i] > TK_DEFAULT)
            sSwathData->precipIce_scale[i] =
                (int16)RND(sSwathData->precipIce[i] * L2A12_PRECIPICE);
	else
            sSwathData->precipIce_scale[i] = TK_MISS_I2;
    }
  sdsWrite(&tempSDS, &sSwathData->precipIce_scale);

  return(TK_SUCCESS);
}


/***********************************************************************************
*
*  FUNCTION STATEMENT:
*       readScan_2A_12(IO_HANDLE *sGranuleHandle, L2A_12_SWATHDATA *sSwathData)
*
*  DESCRIPTION:
*       This function is used to read the L2A-12 information in HDF format.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       sGranuleHandle          A structure containing information about the file.
*                               sGranuleHandle is returned by TKopen().
*
*  OUTPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       sSwathData              A structure containing the complete scanline data.
*                               This structure must be declared to correspond to
*                               the data file being read.  The structure of
*                               sSwathData is manipulated by TKreadScan() and other
*                               toolkit routines.
*
*  CALLED BY:                   TKreadScan()
*
*  ROUTINES CALLED:
*       Routine Name            Description
*       ------------            -----------
*       allocateVdataMemory     This routine is used to allocate memory to vdata.
*       vdataRead               This routine is used to read the vdata in HDF
*                               format.
*       sdsRead                 This routine is used to read the scientific data
*                               set in HDF format.
*
************************************************************************************/

int readScan_2A_12(IO_HANDLE *sGranuleHandle,L2A_12_SWATHDATA *sSwathData)
{
  SDS_LINK *tempSDS = sGranuleHandle->cfgSTR->vGroup->subVgroup->sds;
  VDATA_LINK *tempVdata = sGranuleHandle->cfgSTR->vGroup->subVgroup->vdata;
  uint8 *databuf, *ptr;
  int	i, j;
  int scaleFactor;


  /*  Allocate memory for unpacking the vdata  */
  databuf = allocateVdataMemory(&tempVdata->vdataFields);
  ptr = databuf;
  /* Unpack the content of databuf into scanTime */
  vdataRead(&tempVdata, ptr);
  memcpy (&sSwathData->scanTime.year, ptr, sizeof(sSwathData->scanTime.year));
  ptr += sizeof(sSwathData->scanTime.year);
  memcpy (&sSwathData->scanTime.month, ptr, sizeof(sSwathData->scanTime.month));
  ptr += sizeof(sSwathData->scanTime.month);
  memcpy (&sSwathData->scanTime.dayOfMonth, ptr, 
		sizeof(sSwathData->scanTime.dayOfMonth));
  ptr += sizeof(sSwathData->scanTime.dayOfMonth);
  memcpy (&sSwathData->scanTime.hour, ptr, sizeof(sSwathData->scanTime.hour));
  ptr += sizeof(sSwathData->scanTime.hour);
  memcpy (&sSwathData->scanTime.minute, ptr, sizeof(sSwathData->scanTime.minute));
  ptr += sizeof(sSwathData->scanTime.minute);
  memcpy (&sSwathData->scanTime.second, ptr, sizeof(sSwathData->scanTime.second));
  ptr += sizeof(sSwathData->scanTime.second);
  memcpy (&sSwathData->scanTime.dayOfYear, ptr, 
		sizeof(sSwathData->scanTime.dayOfYear));
  free(databuf);


  sdsRead(&tempSDS, &scaleFactor, &sSwathData->geolocation);

  /* Allocate memory for unpacking the vdata */
  databuf = allocateVdataMemory(&tempVdata->vdataFields);
  ptr = databuf;

  /* Unpack the content of databuf into scanStatus bloc */  
  vdataRead(&tempVdata, ptr);
  memcpy(&sSwathData->scanStatus.missing, ptr,
                sizeof(sSwathData->scanStatus.missing));
  ptr += sizeof(sSwathData->scanStatus.missing);
  memcpy(&sSwathData->scanStatus.validity, ptr, 
		sizeof(sSwathData->scanStatus.validity));
  ptr += sizeof(sSwathData->scanStatus.validity);
  memcpy(&sSwathData->scanStatus.qac, ptr, 
		sizeof(sSwathData->scanStatus.qac));
  ptr += sizeof(sSwathData->scanStatus.qac);
  memcpy(&sSwathData->scanStatus.geoQuality, ptr, 
		sizeof(sSwathData->scanStatus.geoQuality));
  ptr += sizeof(sSwathData->scanStatus.geoQuality);
  memcpy(&sSwathData->scanStatus.ch1, ptr, sizeof(sSwathData->scanStatus.ch1));
  ptr += sizeof(sSwathData->scanStatus.ch1);
  memcpy(&sSwathData->scanStatus.ch2, ptr, sizeof(sSwathData->scanStatus.ch2));
  ptr += sizeof(sSwathData->scanStatus.ch2);
  memcpy(&sSwathData->scanStatus.ch3, ptr, sizeof(sSwathData->scanStatus.ch3));
  ptr += sizeof(sSwathData->scanStatus.ch3);
  memcpy(&sSwathData->scanStatus.ch4, ptr, sizeof(sSwathData->scanStatus.ch4));
  ptr += sizeof(sSwathData->scanStatus.ch4);
  memcpy(&sSwathData->scanStatus.ch5, ptr, sizeof(sSwathData->scanStatus.ch5));
  ptr += sizeof(sSwathData->scanStatus.ch5);
  memcpy(&sSwathData->scanStatus.ch6, ptr, sizeof(sSwathData->scanStatus.ch6));
  ptr += sizeof(sSwathData->scanStatus.ch6);
  memcpy(&sSwathData->scanStatus.ch7, ptr, sizeof(sSwathData->scanStatus.ch7));
  ptr += sizeof(sSwathData->scanStatus.ch7);
  memcpy(&sSwathData->scanStatus.ch8, ptr, sizeof(sSwathData->scanStatus.ch8));
  ptr += sizeof(sSwathData->scanStatus.ch8);
  memcpy(&sSwathData->scanStatus.ch9, ptr, sizeof(sSwathData->scanStatus.ch9));
  ptr += sizeof(sSwathData->scanStatus.ch9);
  memcpy(&sSwathData->scanStatus.scOrient, ptr, 
		sizeof(sSwathData->scanStatus.scOrient));
  ptr += sizeof(sSwathData->scanStatus.scOrient);
  memcpy(&sSwathData->scanStatus.acsMode, ptr, 
		sizeof(sSwathData->scanStatus.acsMode));
  ptr += sizeof(sSwathData->scanStatus.acsMode);
  memcpy(&sSwathData->scanStatus.yawUpStat, ptr, 
		sizeof(sSwathData->scanStatus.yawUpStat));
  ptr += sizeof(sSwathData->scanStatus.yawUpStat);
  memcpy(&sSwathData->scanStatus.tmiIsStatus , ptr, 
		sizeof(sSwathData->scanStatus.tmiIsStatus));
  ptr += sizeof(sSwathData->scanStatus.tmiIsStatus);
  memcpy(&sSwathData->scanStatus.fractOrbitN, ptr, 
		sizeof(sSwathData->scanStatus.fractOrbitN));
  free(databuf);


  /* Allocate memory for unpacking the vdata */
  databuf = allocateVdataMemory(&tempVdata->vdataFields);
  ptr = databuf;
  /* Unpack the content of databuf inro navigate block  */
  vdataRead(&tempVdata, ptr);
  memcpy(&sSwathData->navigate.scPosX, ptr, sizeof(sSwathData->navigate.scPosX));
  ptr += sizeof(sSwathData->navigate.scPosX);
  memcpy(&sSwathData->navigate.scPosY, ptr, sizeof(sSwathData->navigate.scPosY));
  ptr += sizeof(sSwathData->navigate.scPosY);
  memcpy(&sSwathData->navigate.scPosZ, ptr, sizeof(sSwathData->navigate.scPosZ));
  ptr += sizeof(sSwathData->navigate.scPosZ);
  memcpy(&sSwathData->navigate.scVelX, ptr, sizeof(sSwathData->navigate.scVelX));
  ptr += sizeof(sSwathData->navigate.scVelX);
  memcpy(&sSwathData->navigate.scVelY, ptr, sizeof(sSwathData->navigate.scVelY));
  ptr += sizeof(sSwathData->navigate.scVelY);
  memcpy(&sSwathData->navigate.scVelZ, ptr, sizeof(sSwathData->navigate.scVelZ));
  ptr += sizeof(sSwathData->navigate.scVelZ);
  memcpy(&sSwathData->navigate.scLat, ptr, sizeof(sSwathData->navigate.scLat));
  ptr += sizeof(sSwathData->navigate.scLat);
  memcpy(&sSwathData->navigate.scLon, ptr, sizeof(sSwathData->navigate.scLon));
  ptr += sizeof(sSwathData->navigate.scLon);
  memcpy(&sSwathData->navigate.scAlt, ptr, sizeof(sSwathData->navigate.scAlt));
  ptr += sizeof(sSwathData->navigate.scAlt);
  memcpy(&sSwathData->navigate.scAttRoll, ptr, 
		sizeof(sSwathData->navigate.scAttRoll));
  ptr += sizeof(sSwathData->navigate.scAttRoll);
  memcpy(&sSwathData->navigate.scAttPitch, ptr, 
		sizeof(sSwathData->navigate.scAttPitch));
  ptr += sizeof(sSwathData->navigate.scAttPitch);
  memcpy(&sSwathData->navigate.scAttYaw, ptr, 
		sizeof(sSwathData->navigate.scAttYaw));
  ptr += sizeof(sSwathData->navigate.scAttYaw);
  memcpy(&sSwathData->navigate.att1, ptr, sizeof(sSwathData->navigate.att1));
  ptr += sizeof(sSwathData->navigate.att1);
  memcpy(&sSwathData->navigate.att2, ptr, sizeof(sSwathData->navigate.att2));
  ptr += sizeof(sSwathData->navigate.att2);
  memcpy(&sSwathData->navigate.att3, ptr, sizeof(sSwathData->navigate.att3));
  ptr += sizeof(sSwathData->navigate.att3);
  memcpy(&sSwathData->navigate.att4, ptr, sizeof(sSwathData->navigate.att4));
  ptr += sizeof(sSwathData->navigate.att4);
  memcpy(&sSwathData->navigate.att5, ptr, sizeof(sSwathData->navigate.att5));
  ptr += sizeof(sSwathData->navigate.att5);
  memcpy(&sSwathData->navigate.att6, ptr, sizeof(sSwathData->navigate.att6));
  ptr += sizeof(sSwathData->navigate.att6);
  memcpy(&sSwathData->navigate.att7, ptr, sizeof(sSwathData->navigate.att7));
  ptr += sizeof(sSwathData->navigate.att7);
  memcpy(&sSwathData->navigate.att8, ptr, sizeof(sSwathData->navigate.att8));
  ptr += sizeof(sSwathData->navigate.att8);
  memcpy(&sSwathData->navigate.att9, ptr, sizeof(sSwathData->navigate.att9));
  ptr += sizeof(sSwathData->navigate.att9);
  memcpy(&sSwathData->navigate.greenHourAng, ptr, 
		sizeof(sSwathData->navigate.greenHourAng));
  free(databuf);


  /* Read SDS from HDF format */
  sdsRead(&tempSDS, &scaleFactor, &sSwathData->dataFlag);
  sdsRead(&tempSDS, &scaleFactor, &sSwathData->rainFlag);
  sdsRead(&tempSDS, &scaleFactor, &sSwathData->surfaceFlag);
  sdsRead(&tempSDS, &scaleFactor, &sSwathData->surfaceRain);
  sdsRead(&tempSDS, &scaleFactor, &sSwathData->convectRain);
  sdsRead(&tempSDS, &scaleFactor, &sSwathData->confidence);


  /* Read SDS from HDF format */
  /*
   * Scaling calculation.  For reading,
   *    Divided the data by L2A12_CLOUDWATER.
   */
  sdsRead(&tempSDS, &scaleFactor, &sSwathData->cldWater_scale);
  for (i = 0; i < 208; ++i)
    for (j = 0; j < 14; ++j)
        {
        if (sSwathData->cldWater_scale[i][j] != TK_MISS_I2)
	   sSwathData->cldWater[i][j] =
		sSwathData->cldWater_scale[i][j] * 1.0 / scaleFactor;
        else
           sSwathData->cldWater[i][j] = TK_DEFAULT;
        }
  /*
   * Scaling calculation.  For reading,
   *    Divided the data by L2A12_PRECIPWATER.
   */
  sdsRead(&tempSDS, &scaleFactor, &sSwathData->precipWater_scale);
  for (i = 0; i < 208; ++i)
    for (j = 0; j < 14; ++j)
        {
        if (sSwathData->precipWater_scale[i][j] != TK_MISS_I2)
           sSwathData->precipWater[i][j] =
                sSwathData->precipWater_scale[i][j] * 1.0 / scaleFactor;
        else 
           sSwathData->precipWater[i][j] = TK_DEFAULT;
        }
  /*
   * Scaling calculation.  For reading,
   *    Divided the data by L2A12_CLOUDICE.
   */
  sdsRead(&tempSDS, &scaleFactor, &sSwathData->cldIce_scale);
  for (i = 0; i < 208; ++i)
    for (j = 0; j < 14; ++j)
        {
        if (sSwathData->cldIce_scale[i][j] != TK_MISS_I2)
           sSwathData->cldIce[i][j] =
             sSwathData->cldIce_scale[i][j] * 1.0 / scaleFactor;
        else
           sSwathData->cldIce[i][j] = TK_DEFAULT;
        }
  /*
   * Scaling calculation.  For reading,
   *    Divided the data by L2A12_PRECIPICE.
   */
  sdsRead(&tempSDS, &scaleFactor, &sSwathData->precipIce_scale);
  for (i = 0; i < 208; ++i)
    for (j = 0; j < 14; ++j)
        {
        if (sSwathData->precipIce_scale[i][j] != TK_MISS_I2)
           sSwathData->precipIce[i][j] =
             sSwathData->precipIce_scale[i][j] * 1.0 / scaleFactor;
        else
           sSwathData->precipIce[i][j] = TK_DEFAULT;
        }
  /*
   * Scaling calculation.  For reading,
   *    Divided the data by L2A12_LATENTHEAT.
   */
  sdsRead(&tempSDS, &scaleFactor, &sSwathData->latentHeat_scale);
  for (i = 0; i < 208; ++i)
    for (j = 0; j < 14; ++j)
        {
        if (sSwathData->latentHeat_scale[i][j] != TK_MISS_I2)
           sSwathData->latentHeat[i][j] =
             sSwathData->latentHeat_scale[i][j] * 1.0 / scaleFactor;
        else
           sSwathData->latentHeat[i][j] = TK_DEFAULT;
        }
  return(TK_SUCCESS);
} 


int readScan_2A_12_R1(IO_HANDLE *sGranuleHandle,L2A_12_R1_SWATHDATA *sSwathData)
{
  SDS_LINK *tempSDS = sGranuleHandle->cfgSTR->vGroup->subVgroup->sds;
  VDATA_LINK *tempVdata = sGranuleHandle->cfgSTR->vGroup->subVgroup->vdata;
  uint8 *databuf, *ptr;
  int	i, j;
  int scaleFactor;


  /*  Allocate memory for unpacking the vdata  */
  databuf = allocateVdataMemory(&tempVdata->vdataFields);
  ptr = databuf;
  /* Unpack the content of databuf into scanTime */
  vdataRead(&tempVdata, ptr);
  memcpy (&sSwathData->scanTime.year, ptr, sizeof(sSwathData->scanTime.year));
  ptr += sizeof(sSwathData->scanTime.year);
  memcpy (&sSwathData->scanTime.month, ptr, sizeof(sSwathData->scanTime.month));
  ptr += sizeof(sSwathData->scanTime.month);
  memcpy (&sSwathData->scanTime.dayOfMonth, ptr, 
		sizeof(sSwathData->scanTime.dayOfMonth));
  ptr += sizeof(sSwathData->scanTime.dayOfMonth);
  memcpy (&sSwathData->scanTime.hour, ptr, sizeof(sSwathData->scanTime.hour));
  ptr += sizeof(sSwathData->scanTime.hour);
  memcpy (&sSwathData->scanTime.minute, ptr, sizeof(sSwathData->scanTime.minute));
  ptr += sizeof(sSwathData->scanTime.minute);
  memcpy (&sSwathData->scanTime.second, ptr, sizeof(sSwathData->scanTime.second));
  ptr += sizeof(sSwathData->scanTime.second);
  memcpy (&sSwathData->scanTime.dayOfYear, ptr, 
		sizeof(sSwathData->scanTime.dayOfYear));
  free(databuf);


  sdsRead(&tempSDS, &scaleFactor, &sSwathData->geolocation_scale);
  for ( i = 0; i < 208; ++i )
    for ( j = 0; j < 2; ++j )
      {
        /*if ( sSwathData->geolocation_scale[i][j] != TK_MISS_I2 ) tk5.74*/
        if (( sSwathData->geolocation_scale[i][0] != TK_MISS_I2 )
           || ( sSwathData->geolocation_scale[i][1] != TK_MISS_I2 ))
	  sSwathData->geolocation[i][j] =
	    sSwathData->geolocation_scale[i][j] * 1.0 / scaleFactor;
	else
	  sSwathData->geolocation[i][j] = TK_DEFAULT;
      }
  
  sdsRead(&tempSDS, &scaleFactor, &sSwathData->rainFlag);

  sdsRead(&tempSDS, &scaleFactor, &sSwathData->surfaceRain_scale);

  for ( i = 0; i < 208; ++i )
    {
      if ( sSwathData->surfaceRain_scale[i] != TK_MISS_I2 )
	sSwathData->surfaceRain[i] = 
	  (sSwathData->surfaceRain_scale[i] * 1.0 / L2A12_SURFACERAIN );
      else
	sSwathData->surfaceRain[i] = TK_DEFAULT;
    }

  sdsRead(&tempSDS, &scaleFactor, &sSwathData->convectRain_scale);

  for ( i = 0; i < 208; ++i )
    {
      if ( sSwathData->convectRain_scale[i] != TK_MISS_I2 )
	sSwathData->convectRain[i] = 
	  (sSwathData->convectRain_scale[i] * 1.0 / L2A12_CONVECTRAIN );
      else
	sSwathData->convectRain[i] = TK_DEFAULT;
    }

  /* Read SDS from HDF format */
  /*
   * Scaling calculation.  For reading,
   *    Divided the data by L2A12_CLOUDWATER.
   */
  sdsRead(&tempSDS, &scaleFactor, &sSwathData->cldWater_scale);
  for (i = 0; i < 208; ++i)
    {
      if (sSwathData->cldWater_scale[i] != TK_MISS_I2)
	sSwathData->cldWater[i] =
	  sSwathData->cldWater_scale[i] * 1.0 / scaleFactor;
      else
	sSwathData->cldWater[i] = TK_DEFAULT;
    }
  
  /*
   * Scaling calculation.  For reading,
   *    Divided the data by L2A12_PRECIPICE.
   */
  sdsRead(&tempSDS, &scaleFactor, &sSwathData->precipIce_scale);
  for (i = 0; i < 208; ++i)
    {
      if (sSwathData->precipIce_scale[i] != TK_MISS_I2)
	sSwathData->precipIce[i] =
	  sSwathData->precipIce_scale[i] * 1.0 / scaleFactor;
      else
	sSwathData->precipIce[i] = TK_DEFAULT;
    }
  
  return(TK_SUCCESS);
} 

/***********************************************************************************
*
*  FUNCTION STATEMENT:
*       writeGrid_3A_11(IO_HANDLE *sGranuleHandle,
*                       L3A_11_PLANETGRID *sPlanetGrid)
*
*  DESCRIPTION:
*       This function is used to write the L3A-11 information in HDF format.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       sGranuleHandle          A structure containing information about the file.
*                               sGranuleHandle is returned by TKopen().
*
*       sPlanetGrid		A structure containing the complete grid data.
*				This structure is manipulated by TKwriteGrid()
*				and other toolkit routines.
*
*  OUTPUT PARAMETERS:           None
*
*  CALLED BY:                   TKwriteGrid()
*
*  ROUTINES CALLED:
*	allocateVdataMemory(), sdsWrite(), vdataWrite()
*
***********************************************************************************/

int writeGrid_3A_11(IO_HANDLE *sGranuleHandle, L3A_11_PLANETGRID *sPlanetGrid)
{
  SDS_LINK *tempSDS = sGranuleHandle->cfgSTR->vGroup->subVgroup->sds;
  VDATA_LINK *tempVdata = sGranuleHandle->cfgSTR->vGroup->subVgroup->vdata;
  uint8 *databuf, *ptr;
  int	i, j;

  /* Write sds to a HDF file */
  /*
   * Scaling calculation.  For writing,
   *	Multiplied the data by L3A11_MONTHRAIN.
   */
  for (i = 0; i < 72; ++i)
    for (j = 0; j < 16; ++j)
    {
        /* The Following code was added to take care of the Missing or
           invalid data.  The reason to add this code was not to scale
           the invalid Data */

        if (sPlanetGrid->monthRain[i][j] > TK_DEFAULT)
	    sPlanetGrid->monthRain_scale[i][j] = 
		(int16)RND(sPlanetGrid->monthRain[i][j] * L3A11_MONTHRAIN);
	else
	    sPlanetGrid->monthRain_scale[i][j] = TK_MISS_I2;
    }
  sdsWrite(&tempSDS, &sPlanetGrid->monthRain_scale);

  sdsWrite(&tempSDS, &sPlanetGrid->noOfSamples);
  sdsWrite(&tempSDS, &sPlanetGrid->chiSqFit);

  /*
   * Scaling calculation.  For writing,
   *    Multiplied the data by L3A11_FREEZLEVEL.
   */
  for (i = 0; i < 72; ++i)
    for (j = 0; j < 16; ++j)
    {
        /* The Following code was added to take care of the Missing or
           invalid data.  The reason to add this code was not to scale
           the invalid Data */

        if (sPlanetGrid->freezLevel[i][j] > TK_DEFAULT)
            sPlanetGrid->freezLevel_scale[i][j] = 
		(int16)RND(sPlanetGrid->freezLevel[i][j] * L3A11_FREEZLEVEL);
	else
            sPlanetGrid->freezLevel_scale[i][j] = TK_MISS_I2;
    }
  sdsWrite(&tempSDS, &sPlanetGrid->freezLevel_scale);

  /*
   * Scaling calculation.  For writing,
   *    Multiplied the data by L3A11_T0.
   */
  for (i = 0; i < 72; ++i)
    for (j = 0; j < 16; ++j)
    {
        /* The Following code was added to take care of the Missing or
           invalid data.  The reason to add this code was not to scale
           the invalid Data */

        if (sPlanetGrid->T0[i][j] > TK_DEFAULT)
            sPlanetGrid->T0_scale[i][j] = 
		(int16)RND(sPlanetGrid->T0[i][j] * L3A11_T0);
	else
            sPlanetGrid->T0_scale[i][j] = TK_MISS_I2;
    }
  sdsWrite(&tempSDS, &sPlanetGrid->T0_scale);

  /*
   * Scaling calculation.  For writing,
   *    Multiplied the data by L3A11_R0.
   */
  for (i = 0; i < 72; ++i)
    for (j = 0; j < 16; ++j)
    {
        /* The Following code was added to take care of the Missing or
           invalid data.  The reason to add this code was not to scale
           the invalid Data */

        if (sPlanetGrid->r0[i][j] > TK_DEFAULT)
            sPlanetGrid->r0_scale[i][j] = 
		(int16)RND(sPlanetGrid->r0[i][j] * L3A11_R0);
	else
            sPlanetGrid->r0_scale[i][j] = TK_MISS_I2;
    }
  sdsWrite(&tempSDS, &sPlanetGrid->r0_scale);

  /*
   * Scaling calculation.  For writing,
   *    Multiplied the data by L3A11_SIGMAR.
   */
  for (i = 0; i < 72; ++i)
    for (j = 0; j < 16; ++j)
    {
        /* The Following code was added to take care of the Missing or
           invalid data.  The reason to add this code was not to scale
           the invalid Data */

        if (sPlanetGrid->sigmaR[i][j] > TK_DEFAULT)
            sPlanetGrid->sigmaR_scale[i][j] = 
		(int16)RND(sPlanetGrid->sigmaR[i][j] * L3A11_SIGMAR);
	else
            sPlanetGrid->sigmaR_scale[i][j] = TK_MISS_I2;
    }
  sdsWrite(&tempSDS, &sPlanetGrid->sigmaR_scale);

  /*
   * Scaling calculation.  For writing,
   *    Multiplied the data by L3A11_PROBRAIN.
   */
  for (i = 0; i < 72; ++i)
    for (j = 0; j < 16; ++j)
    {
        /* The Following code was added to take care of the Missing or
           invalid data.  The reason to add this code was not to scale
           the invalid Data */

        if (sPlanetGrid->probRain[i][j] > TK_DEFAULT)
            sPlanetGrid->probRain_scale[i][j] = 
		(int16)RND(sPlanetGrid->probRain[i][j] * L3A11_PROBRAIN);
	else
            sPlanetGrid->probRain_scale[i][j] = TK_MISS_I2;
    }
  sdsWrite(&tempSDS, &sPlanetGrid->probRain_scale);

  sdsWrite(&tempSDS, &sPlanetGrid->qInd1);
  sdsWrite(&tempSDS, &sPlanetGrid->qInd2);
  sdsWrite(&tempSDS, &sPlanetGrid->qInd3);
  sdsWrite(&tempSDS, &sPlanetGrid->spare);
  
  return(TK_SUCCESS);
}

/***********************************************************************************
*
*  FUNCTION STATEMENT:
*       readGrid_3A_11(IO_HANDLE *sGranuleHandle, L3A_11_PLANETGRID *sPlanetGrid)
*
*  DESCRIPTION:
*       This function is used to read the L3A_11 information in HDF format.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       sGranuleHandle          A structure containing information about the file.
*                               sGranuleHandle is returned by TKopen().
*
*  OUTPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       sPlanetGrid 		A structure containing a complete grid data.
*				This structure must be declared to correspond to
*				the data file being read.  The structure of
*				sPlanetGrid is manipulated by TKreadGrid() and
*				other toolkit routines.
*
*  CALLED BY:                   TKreadGrid()
*
*  ROUTINES CALLED:
*	allocateVdataMemory(), sdsRead(), vdataRead()
*
***********************************************************************************/

int readGrid_3A_11(IO_HANDLE *sGranuleHandle, L3A_11_PLANETGRID *sPlanetGrid)
{ 
  SDS_LINK *tempSDS = sGranuleHandle->cfgSTR->vGroup->subVgroup->sds;
  VDATA_LINK *tempVdata = sGranuleHandle->cfgSTR->vGroup->subVgroup->vdata;
  uint8 *databuf, *ptr;
  int	i, j;
  int scaleFactor;


  /* Read sds from a HDF file */
  /*
   * Scaling calculation.  For reading,
   *    Divided the data by L3A11_MONTHRAIN.
   */
  sdsRead(&tempSDS, &scaleFactor, &sPlanetGrid->monthRain_scale);
  for (i = 0; i < 72; ++i)
    for (j = 0; j < 16; ++j)
        {
        if (sPlanetGrid->monthRain_scale[i][j] != TK_MISS_I2)
           sPlanetGrid->monthRain[i][j] = 
		sPlanetGrid->monthRain_scale[i][j] * 1.0 / scaleFactor;
        else
           sPlanetGrid->monthRain[i][j] = TK_DEFAULT;
        }
  sdsRead(&tempSDS, &scaleFactor, &sPlanetGrid->noOfSamples);
  sdsRead(&tempSDS, &scaleFactor, &sPlanetGrid->chiSqFit);

  /*
   * Scaling calculation.  For reading,
   *    Divided the data by L3A11_FREEZLEVEL.
   */
  sdsRead(&tempSDS, &scaleFactor, &sPlanetGrid->freezLevel_scale);
  for (i = 0; i < 72; ++i)
    for (j = 0; j < 16; ++j)
        {
        if (sPlanetGrid->freezLevel_scale[i][j] != TK_MISS_I2)
           sPlanetGrid->freezLevel[i][j] = 
		sPlanetGrid->freezLevel_scale[i][j] * 1.0 / scaleFactor;
        else
           sPlanetGrid->freezLevel[i][j] = TK_DEFAULT;
        }
  /*
   * Scaling calculation.  For reading,
   *    Divided the data by L3A11_T0.
   */
  sdsRead(&tempSDS, &scaleFactor, &sPlanetGrid->T0_scale);
  for (i = 0; i < 72; ++i)
    for (j = 0; j < 16; ++j)
        {
        if (sPlanetGrid->T0_scale[i][j] != TK_MISS_I2)
           sPlanetGrid->T0[i][j] = sPlanetGrid->T0_scale[i][j] * 1.0/ scaleFactor;
        else
           sPlanetGrid->T0[i][j] = TK_DEFAULT;
        }
  /*
   * Scaling calculation.  For reading,
   *    Divided the data by L3A11_R0.
   */
  sdsRead(&tempSDS, &scaleFactor, &sPlanetGrid->r0_scale);
  for (i = 0; i < 72; ++i)
    for (j = 0; j < 16; ++j)
        {
        if (sPlanetGrid->r0_scale[i][j] != TK_MISS_I2)
           sPlanetGrid->r0[i][j] = sPlanetGrid->r0_scale[i][j] * 1.0/ scaleFactor;
        else
           sPlanetGrid->r0[i][j] = TK_DEFAULT;
        }
  /*
   * Scaling calculation.  For reading,
   *    Divided the data by L3A11_SIGMAR.
   */
  sdsRead(&tempSDS, &scaleFactor, &sPlanetGrid->sigmaR_scale);
  for (i = 0; i < 72; ++i)
    for (j = 0; j < 16; ++j)
        {
        if (sPlanetGrid->sigmaR_scale[i][j] != TK_MISS_I2)
           sPlanetGrid->sigmaR[i][j] = 
		sPlanetGrid->sigmaR_scale[i][j] * 1.0 / scaleFactor;
        else
           sPlanetGrid->sigmaR[i][j] = TK_DEFAULT;
        }
  /*
   * Scaling calculation.  For reading,
   *    Divided the data by L3A11_PROBRAIN.
   */
  sdsRead(&tempSDS, &scaleFactor, &sPlanetGrid->probRain_scale);
  for (i = 0; i < 72; ++i)
    for (j = 0; j < 16; ++j)
        {
        if (sPlanetGrid->probRain_scale[i][j] != TK_MISS_I2)
           sPlanetGrid->probRain[i][j] = 
		sPlanetGrid->probRain_scale[i][j] * 1.0 / scaleFactor;
        else
           sPlanetGrid->probRain[i][j] = TK_DEFAULT;
        }
  sdsRead(&tempSDS, &scaleFactor, &sPlanetGrid->qInd1);
  sdsRead(&tempSDS, &scaleFactor, &sPlanetGrid->qInd2);
  sdsRead(&tempSDS, &scaleFactor, &sPlanetGrid->qInd3);
  sdsRead(&tempSDS, &scaleFactor, &sPlanetGrid->spare);

  return(TK_SUCCESS);
}

/***********************************************************************************
*
*  FUNCTION STATEMENT:
*       setScalingFactors_1B11(VGROUP_LINK *vgroup)
*
*  DESCRIPTION:
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       vgroup                  List of vgroups.  The list of SDSs are retrieve from
*                               this node.
*
*  OUTPUT PARAMETERS:  None.
*
*  CALLED BY:                   setSDSattributes().
*
*  ROUTINES CALLED:     
*       Routine Name            Description
*       ------------            -----------
*       SDsetcal                This routine adds the scaling factor and offset 
*                               to the SDS.
*       TKreportWarning         This routine is called for warnings and informational
*                               messages.  Calling this routine will send an error
*                               message to the central log file and return control to 
*                               the calling routine.
*
***********************************************************************************/
int setScalingFactors_1B11 ( VGROUP_LINK *vgroup, int realTimeFlag )
{

  int status = TK_SUCCESS;
  SDS_LINK *sds;

  if ( vgroup != NULL )
    {
      sds = vgroup->sds;
      while ( sds != NULL )
	{
	  if ( ( !strcmp ( sds->name, "geolocation" ) ) && ( realTimeFlag ) )
	    status = SDsetcal ( sds->sdsID, (float64) GEOLOC_SCALE,
			       (float64) 0, (float64) 0, (float64) 0,
			       DFNT_INT16 );
	  else if ( ( !strcmp ( sds->name, "satLocZenAngle" ) ) && ( realTimeFlag ) )
	    status = SDsetcal ( sds->sdsID, (float64) L1B11_SATLOCZENANGLE,
			       (float64) 0, (float64) 0, (float64) 0,
			       DFNT_INT16 );
	  else if ( !strcmp ( sds->name, "lowResCh" ) )
	    status = SDsetcal ( sds->sdsID, (float64) L1B11_LOWRESCH, 
			       (float64) 0, (float64) 100, (float64) 0, 
			       DFNT_INT16 );
	  else if ( !strcmp ( sds->name, "highResCh" ) )
	    status = SDsetcal ( sds->sdsID, (float64) L1B11_HIGHRESCH, 
			       (float64) 0,  (float64) 100, (float64) 0, 
			       DFNT_INT16 );

	  if ( status == TK_HDF_FAIL )
	    TKreportWarning ( E_TK_SETCALINFO );
	  sds = sds->next;
	}
      setScalingFactors_1B11 ( vgroup->subVgroup, realTimeFlag );
      setScalingFactors_1B11 ( vgroup->next, realTimeFlag );
    }
  return status;
}

/***********************************************************************************
*
*  FUNCTION STATEMENT:
*       setScalingFactors_2A12(VGROUP_LINK *vgroup)
*
*  DESCRIPTION:
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       vgroup                  List of vgroups.  The list of SDSs are retrieve from
*                               this node.
*
*  OUTPUT PARAMETERS:  None.
*
*  CALLED BY:                   setSDSattributes().
*
*  ROUTINES CALLED:     
*       Routine Name            Description
*       ------------            -----------
*       SDsetcal                This routine adds the scaling factor and offset 
*                               to the SDS.
*       TKreportWarning         This routine is called for warnings and informational
*                               messages.  Calling this routine will send an error
*                               message to the central log file and return control to 
*                               the calling routine.
*
***********************************************************************************/
int setScalingFactors_2A12 ( VGROUP_LINK *vgroup, int realTimeFlag )
{

  int status = TK_SUCCESS;
  SDS_LINK *sds;

  if ( vgroup != NULL )
    {
      sds = vgroup->sds;
      while ( sds != NULL )
	{
	  if ( ( !strcmp ( sds->name, "geolocation" ) ) && ( realTimeFlag ) )
	    status = SDsetcal ( sds->sdsID, (float64) GEOLOC_SCALE,
			       (float64) 0, (float64) 0, (float64) 0,
			       DFNT_INT16 );
	  else if ( ( !strcmp ( sds->name, "surfaceRain" ) ) && ( realTimeFlag ) )
	    status = SDsetcal ( sds->sdsID, (float64) L2A12_SURFACERAIN,
			       (float64) 0, (float64) 0, (float64) 0,
			       DFNT_INT16 );
	  else if ( ( !strcmp ( sds->name, "convectRain" ) ) && ( realTimeFlag ) )
	    status = SDsetcal ( sds->sdsID, (float64) L2A12_CONVECTRAIN,
			       (float64) 0, (float64) 0, (float64) 0,
			       DFNT_INT16 );
	  else if ( !strcmp ( sds->name, "cldWater" ) )
	    status = SDsetcal ( sds->sdsID, (float64) L2A12_CLOUDWATER, 
			       (float64) 0, (float64) 0, (float64) 0, 
			       DFNT_INT16 );
	  else if ( !strcmp ( sds->name, "precipWater" ) )
	    status = SDsetcal ( sds->sdsID, (float64) L2A12_PRECIPWATER, 
			       (float64) 0,  (float64) 0, (float64) 0, 
			       DFNT_INT16 );
	  else if ( !strcmp ( sds->name, "cldIce" ) )
	    status = SDsetcal ( sds->sdsID, (float64) L2A12_CLOUDICE, 
			       (float64) 0, (float64) 0, (float64) 0, 
			       DFNT_INT16 );
	  else if ( !strcmp ( sds->name, "precipIce" ) )
	    status = SDsetcal ( sds->sdsID, (float64) L2A12_PRECIPICE, 
			       (float64) 0, (float64) 0, (float64) 0, 
			       DFNT_INT16 );
	  else if ( !strcmp ( sds->name, "latentHeat" ) )
	    status = SDsetcal ( sds->sdsID, (float64) L2A12_LATENTHEAT, 
			       (float64) 0, (float64) 0, (float64) 0, 
			       DFNT_INT16 );

	  if ( status == TK_HDF_FAIL )
	    TKreportWarning ( E_TK_SETCALINFO );
	  sds = sds->next;
	}
      setScalingFactors_2A12 ( vgroup->subVgroup, realTimeFlag );
      setScalingFactors_2A12 ( vgroup->next, realTimeFlag );
    }
  return status;
}

/***********************************************************************************
*
*  FUNCTION STATEMENT:
*       setScalingFactors_3A11(VGROUP_LINK *vgroup)
*
*  DESCRIPTION:
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       vgroup                  List of vgroups.  The list of SDSs are retrieve from
*                               this node.
*
*  OUTPUT PARAMETERS:  None.
*
*  CALLED BY:                   setSDSattributes().
*
*  ROUTINES CALLED:     
*       Routine Name            Description
*       ------------            -----------
*       SDsetcal                This routine adds the scaling factor and offset 
*                               to the SDS.
*       TKreportWarning         This routine is called for warnings and informational
*                               messages.  Calling this routine will send an error
*                               message to the central log file and return control to 
*                               the calling routine.
*
***********************************************************************************/
int setScalingFactors_3A11 ( VGROUP_LINK *vgroup )
{

  int status = TK_SUCCESS;
  SDS_LINK *sds;

  if ( vgroup != NULL )
    {
      sds = vgroup->sds;
      while ( sds != NULL )
	{
	  if ( !strcmp ( sds->name, "monthRain" ) )
	    status = SDsetcal ( sds->sdsID, (float64) L3A11_MONTHRAIN, 
			       (float64) 0, (float64) 0, (float64) 0, 
			       DFNT_INT16 );
	  else if ( !strcmp ( sds->name, "freezLevel" ) )
	    status = SDsetcal ( sds->sdsID, (float64) L3A11_FREEZLEVEL, 
			       (float64) 0,  (float64) 0, (float64) 0, 
			       DFNT_INT16 );
	  else if ( !strcmp ( sds->name, "T0" ) )
	    status = SDsetcal ( sds->sdsID, (float64) L3A11_T0, 
			       (float64) 0, (float64) 0, (float64) 0, 
			       DFNT_INT16 );
	  else if ( !strcmp ( sds->name, "r0" ) )
	    status = SDsetcal ( sds->sdsID, (float64) L3A11_R0, 
			       (float64) 0, (float64) 0, (float64) 0, 
			       DFNT_INT16 );
	  else if ( !strcmp ( sds->name, "sigmaR" ) )
	    status = SDsetcal ( sds->sdsID, (float64) L3A11_SIGMAR, 
			       (float64) 0, (float64) 0, (float64) 0, 
			       DFNT_INT16 );
	  else if ( !strcmp ( sds->name, "probRain" ) )
	    status = SDsetcal ( sds->sdsID, (float64) L3A11_PROBRAIN, 
			       (float64) 0, (float64) 0, (float64) 0, 
			       DFNT_INT16 );

	  if ( status == TK_HDF_FAIL )
	    TKreportWarning ( E_TK_SETCALINFO );
	  sds = sds->next;
	}
      setScalingFactors_3A11 ( vgroup->subVgroup );
      setScalingFactors_3A11 ( vgroup->next );
    }
  return status;
}


int writeGrid_3A_12(IO_HANDLE *sGranuleHandle, L3A_12_GRID *sGrid)
{
  SDS_LINK *tempSDS = sGranuleHandle->cfgSTR->vGroup->subVgroup->sds;
  VDATA_LINK *tempVdata = sGranuleHandle->cfgSTR->vGroup->subVgroup->vdata;
  uint8 *databuf, *ptr;

  sdsWrite(&tempSDS, &sGrid->surfaceRain);
  sdsWrite(&tempSDS, &sGrid->convectRain);
  sdsWrite(&tempSDS, &sGrid->stratiformRain);
  sdsWrite(&tempSDS, &sGrid->cldWater);
  sdsWrite(&tempSDS, &sGrid->precipWater);
  sdsWrite(&tempSDS, &sGrid->cldIce);
  sdsWrite(&tempSDS, &sGrid->precipIce);
  sdsWrite(&tempSDS, &sGrid->latentHeat);
  sdsWrite(&tempSDS, &sGrid->npixTotal);
  sdsWrite(&tempSDS, &sGrid->npixRain);
  sdsWrite(&tempSDS, &sGrid->npixConvectRain);
  sdsWrite(&tempSDS, &sGrid->npixStratiformRain);
  sdsWrite(&tempSDS, &sGrid->npixUnclassifiable);
  sdsWrite(&tempSDS, &sGrid->npixAmbiguous);

  return(TK_SUCCESS);
}

int readGrid_3A_12(IO_HANDLE *sGranuleHandle, L3A_12_GRID *sGrid)
{
  SDS_LINK *tempSDS = sGranuleHandle->cfgSTR->vGroup->subVgroup->sds;
  VDATA_LINK *tempVdata = sGranuleHandle->cfgSTR->vGroup->subVgroup->vdata;
  uint8 *databuf, *ptr;
  int scaleFactor;

  sdsRead(&tempSDS, &scaleFactor, &sGrid->surfaceRain);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->convectRain);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->stratiformRain);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->cldWater);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->precipWater);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->cldIce);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->precipIce);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->latentHeat);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->npixTotal);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->npixRain);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->npixConvectRain);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->npixStratiformRain);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->npixUnclassifiable);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->npixAmbiguous);

  return(TK_SUCCESS);
}



