/* DEVELOPMENT HISTORY */
/* Revision 1.31  1999/11/16 16:04:42  pkolbe
 * changed 1b01 localDirection from scaled int to float because
 * real data was being converted to missing
 *
 * Revision 1.30  1999/08/13 17:21:01  pkolbe
 * added dynamic scaling code
 *
 * Revision 1.29  1998/02/11 19:34:01  slywczak
 * Modified routines writeScan_1B_01 and readScan_1B_01.  Provided the ability to
 * read and write the new structure, Solar Calibration.  This structure is written
 * to the files as a Vdata.
 *
 * Revision 1.28  1997/10/30 21:55:10  slywczak
 * Modified routine writeScan_1B_01 and readScan_1B_01.  Added the element
 * tempCounts.
 *
 * Revision 1.27  1997/05/19 03:44:35  slywczak
 * Modified the following routines:  readGrid_3B_42, writeGrid_3B_42,
 * readGrid_3B_43, and writeGrid_3B_43.  Removed references to the gridStruct
 * structure, since this structure will be referenced in the toolkit code.
 *
 * Revision 1.26  1997/05/16 14:17:28  slywczak
 * Added the function setScalingFactors_1B01.  This routine will write the
 * scaling factors to the VIRS 1B01 HDF file.  The scaling factors will be
 * attached to the local direction and channel SDS.
 *
 * Revision 1.25  1997/02/21 23:14:32  piper
 * Fixed code to properly handle missing values.
 *
 * Revision 1.25  1997/02/20 21:20:20  jt
 * Fixed scaling calculations for reading missing or valid data.
 *
 * Revision 1.24  1997/02/13 17:51:45  piper
 * Fixed problem with missing values not being written to output files.
 *
 * Revision 1.23  1997/01/02 15:30:37  amak
 * Define TK_DEFAULT for hard-coded value.
 *
 * Revision 1.22  1996/12/24 14:30:35  amak
 * Incorporated walkthrough comments.
 * - Changed '<' and '>' to double quotes around the #include statements.
 *
 * Revision 1.21  1996/11/06  15:28:15  amak
 * Added detailed description to the routine.
 *
 * Revision 1.20  1996/10/24  13:49:07  pshah
 * Added code to take care of missing or invalid data for scaling
 *
 * Revision 1.19  1996/06/19  18:23:20  amak
 * Added field "missing" to Scan Status block to process.
 *
 * Revision 1.18  1996/06/07  14:43:35  amak
 * Added scaling calculation to field 'localDirection' for L1B01.
 *
 * Revision 1.17  1996/04/19  15:40:47  amak
 * Made modifications to the preprocessor #include statement.  Replaced
 * TKerrorRou.h and TKwarnRou.h to TKerrHandle.h and TS_TK_15.h for new
 * error handling calls.
 *
 * Revision 1.16  1996/03/21  21:59:56  amak
 * Added prologue.
 *
 * Revision 1.15  1996/03/15  16:45:11  amak
 * Modified write_1B_01() and read_1B_01() routines which based on the two
 * newly added variables, virsMode and virsAbnCon, in VIRS_SCAN_STATUS
 * structure.
 *
 * Revision 1.14  1996/01/25  15:06:24  amak
 * Called macro function RND() when doing the roundup from floating point to
 * integer type.
 *
 * Revision 1.13  1995/12/18  22:26:02  amak
 * Added 0.5 to all the write scaling calculations.  This adding is to avoid
 * the round up problems while computing on various platforms.
 *
 * Revision 1.12  1995/12/14  14:47:57  amak
 * Edited more comments.
 *
 * Revision 1.11  1995/12/13  18:41:59  amak
 * Fixed bugs on functions writeScan_1B_01() and readScan_1B_01().  Switched
 * the calling sequence of Navigation block and ScanStatus block.  The
 * sequence of the calling must be agreed to the declaration in the config-
 * uration file - L1B01VIRS.
 *
 * Revision 1.10  1995/12/12  14:49:06  amak
 * Used macro factors to do scaling calculations.
 *
 * Revision 1.9  1995/12/05  18:35:52  thyun
 * Modified assignment of tempSDS and tempVdata which are appearing in declaration
 * of all the functions.
 *
 * Revision 1.8  1995/12/01  14:35:08  thyun
 * Removed the c function "rint".
 *
 * Revision 1.7  1995/11/28  21:19:51  thyun
 * Get rid of the () from rint.
 *
 * Revision 1.6  1995/11/28  20:57:48  thyun
 * Modified the scaling for the product TK_L1B_01.
 *
 * Revision 1.5  1995/11/21  14:20:59  amak
 * Fixed syntax errors.
 *
 * Revision 1.4  1995/11/20  17:52:40  amak
 * Added scaling calculations to the routines.
 *
 * Revision 1.3  1995/11/15  23:37:40  thyun
 * Modified writeScan_1B_01 and readScan_1B_01 to reflect the changes in
 * configuration structure.
 *
 * Revision 1.2  1995/10/18  20:52:23  amak
 * Made modifications after some files need to be renamed.
 *
 * Revision 1.1  1995/10/17  19:42:22  amak
 * Renamed file IO_INTERNAL_VIRS.c to IO_INTR_VIRS.c
 *
 * Revision 1.16  1995/08/10  19:59:54  thyun
 * Modified writeScan_1B_01 and readScan_1B_01 to reflect the changes in
 * the data structure navigate.
 *
 * Revision 1.15  1995/08/10  18:30:42  thyun
 * Modified readGrid_3B_42, writeGrid_3B_42, readGrid_3B_43, and writeGrid_3B_43
 * to resolve some semantic errors.  These errors were found during the unit
 * testing.  Replace the parameter of the routine "size_of" with the variable
 * itself for all the readGrid and writeGrid routines in IO_INTERNAL_VIRS.c.
 *
 * Revision 1.14  1995/08/07  15:44:44  amak
 * Shortened the variable names in the navigate structure.
 *
 * Revision 1.13  1995/08/04  19:54:18  amak
 * This is a test.
 *
 * Revision 1.12  1995/08/03  17:45:38  amak
 * Re-modified the code because CVS couldn't do version control nicely.
 *
 * Revision 1.11  1995/08/01  20:32:08  thyun
 * Updated the structure names such as VDATA_LINK and SDS_LINK as well as
 * variable names.  These changes occurred becase of changes in parsing.
 *
 * Revision 1.8  1995/07/20  14:19:58  amak
 * dded routines readGrid_3B_42(), readGrid_3B_43(), writeGrid3B_42(), and
 * writeGrid_3B_43() for Level 3 API.
 *
 * Revision 1.7  1995/07/18  14:34:08  amak
 * Renamed the routines from vWrite, vRead, sWrite and sRead to vdataWrite,
 * vdataRead, sdsWrite and sdsRead respectively.
 *
 * Revision 1.6  1995/07/08  14:11:30  amak
 * Took out Level 3B-42 and 3B-43 prototypes due to the change of API for
 * the satellite products.
 *
 * Revision 1.5  1995/07/07  21:17:54  amak
 * Added more detailed comments
 *
 * Revision 1.4  1995/06/30  18:30:02  thyun
 * Modified writeScan_1B_01 and readScan_1B_01
 *
 * Revision 1.3  1995/06/13  14:28:40  amak
 * Adding more comments
 *
 * Revision 1.2  1995/06/08  19:38:10  amak
 * Added more comments
 * */

/***********************************************************************************
*
*  FILE NAME:           IO_INTR_VIRS.c
*
*  DESCRIPTION:
*       This file contains routines that read and write VIRS headers 
*	and data.
*
*  ROUTINES CALLED:
*       Routine Name            Description
*       ------------            -----------
*       writeScan_1B_01         This function is used to write the L1B-01
*                               information in HDF format.      
*       readScan_1B_01          This function is used to read the L1B-01
*                               information in HDF format.      
*       writeGrid_3B_42         This function is used to write the L3B-42
*                               information in HDF format.
*       readGrid_3B_42          This function is used to read the L3B-42
*                               information in HDF format.
*       writeGrid_3B_43         This function is used to write the L3B-43
*                               information in HDf format.
*       readGrid_3B_43          This function is used to read the L3B-43
*                               information in HDF format.
*       allocateVdataMemory     This routine is used to allocate memory to vdata.
*       vdataWrite              This routine is used to write the vdata in HDF    
*                               format.
*       sdsWrite		This routine is used to write the scientific data
*                               set in HDF format.
*       sdsRead			This routine is used to read the scientific data 
*                               set in HDF format.
*       vdataRead               This routine is used to read the vdata in HDF
*                               format.
*
**********************************************************************************/

#include "IO_VIRS.h"
#include "IO_INTR_VIRS.h"
#include "hdf_routines.h"
#include "TKerrHandle.h"
#include "TS_TK_15.h"

/***********************************************************************************
*
*  FUNCTION STATEMENT:
*       writeScan_1B_01(IO_HANDLE *sGranuleHandle, 
*                       L1B_01_SWATHDATA *sSwathData) 
*
*  DESCRIPTION:
*       This function is used to write the L1B-01 information in HDF format.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       sGranuleHandle          A structure containing information about the file.
*                               sGranuleHandle is returned by TKopen().
*
*       sSwathData              A structure containing the complete scanline data.
*                               This structure must be declared to correspond to 
*                               the data file being written.  The structure of 
*                               sSwathData is manipulated by TKwriteScan() and 
*				other toolkit routines.
*
*  OUTPUT PARAMETERS:           None
*
*  CALLED BY:                   TKwriteScan()
*
*  ROUTINES CALLED:     
*       Routine Name            Description
*       ------------            -----------
*       allocateVdataMemory     This routine is used to allocate memory to vdata.
*       vdataWrite              This routine is used to write the vdata in HDF
*                               format.
*       sdsWrite                This routine is used to write the scientific data
*                               set in HDF format.      
*
***********************************************************************************/

int writeScan_1B_01(IO_HANDLE *sGranuleHandle, L1B_01_SWATHDATA *sSwathData)
{
  SDS_LINK *tempSDS  = sGranuleHandle->cfgSTR->vGroup->subVgroup->sds;
  VDATA_LINK *tempVdata = sGranuleHandle->cfgSTR->vGroup->subVgroup->vdata;
  uint8 *databuf, *ptr;
  int	i,j,k;

  /* Pack the content of sSwathData->scanTime into databuf. */
  databuf = allocateVdataMemory(&tempVdata->vdataFields);
  ptr = databuf;
  memcpy(ptr, &sSwathData->scanTime, sizeof(sSwathData->scanTime));
  vdataWrite(&tempVdata, databuf);
  free(databuf); 

  /* Writing scientific data set into HDF format */
  sdsWrite(&tempSDS, &sSwathData->geolocation);

  /* Pack the content of sSwathData->scanStatus into databuf */
  databuf = allocateVdataMemory(&tempVdata->vdataFields);
  ptr = databuf;
  memcpy(ptr, &sSwathData->scanStatus.missing, sizeof(sSwathData->scanStatus.missing));
  ptr += sizeof(sSwathData->scanStatus.missing);
  memcpy(ptr, &sSwathData->scanStatus.validity, sizeof(sSwathData->scanStatus.validity));
  ptr += sizeof(sSwathData->scanStatus.validity);
  memcpy(ptr, &sSwathData->scanStatus.qac, sizeof(sSwathData->scanStatus.qac));
  ptr += sizeof(sSwathData->scanStatus.qac);
  memcpy(ptr, &sSwathData->scanStatus.geoQuality, sizeof(sSwathData->scanStatus.geoQuality));
  ptr += sizeof(sSwathData->scanStatus.geoQuality);
  memcpy(ptr, &sSwathData->scanStatus.ch1Quality, sizeof(sSwathData->scanStatus.ch1Quality));
  ptr += sizeof(sSwathData->scanStatus.ch1Quality);
  memcpy(ptr, &sSwathData->scanStatus.ch2Quality, sizeof(sSwathData->scanStatus.ch2Quality));
  ptr += sizeof(sSwathData->scanStatus.ch2Quality);
  memcpy(ptr, &sSwathData->scanStatus.ch3Quality, sizeof(sSwathData->scanStatus.ch3Quality));
  ptr += sizeof(sSwathData->scanStatus.ch3Quality);
  memcpy(ptr, &sSwathData->scanStatus.ch4Quality, sizeof(sSwathData->scanStatus.ch4Quality));
  ptr += sizeof(sSwathData->scanStatus.ch4Quality);
  memcpy(ptr, &sSwathData->scanStatus.ch5Quality, sizeof(sSwathData->scanStatus.ch5Quality));
  ptr += sizeof(sSwathData->scanStatus.ch5Quality);
  memcpy(ptr, &sSwathData->scanStatus.scOrient, sizeof(sSwathData->scanStatus.scOrient));
  ptr += sizeof(sSwathData->scanStatus.scOrient);
  memcpy(ptr, &sSwathData->scanStatus.acsMode, sizeof(sSwathData->scanStatus.acsMode));
  ptr += sizeof(sSwathData->scanStatus.acsMode);
  memcpy(ptr, &sSwathData->scanStatus.yawUpdateS, sizeof(sSwathData->scanStatus.yawUpdateS));
  ptr += sizeof(sSwathData->scanStatus.yawUpdateS);
  memcpy(ptr, &sSwathData->scanStatus.virsInstS, sizeof(sSwathData->scanStatus.virsInstS));
  ptr += sizeof(sSwathData->scanStatus.virsInstS);
  memcpy(ptr, &sSwathData->scanStatus.virsMode, sizeof(sSwathData->scanStatus.virsMode));
  ptr += sizeof(sSwathData->scanStatus.virsMode);
  memcpy(ptr, &sSwathData->scanStatus.virsAbnCon, sizeof(sSwathData->scanStatus.virsAbnCon));
  ptr += sizeof(sSwathData->scanStatus.virsAbnCon);
  memcpy(ptr, &sSwathData->scanStatus.fractOrbitN, 
		sizeof(sSwathData->scanStatus.fractOrbitN));
  vdataWrite(&tempVdata, databuf);
  free(databuf);


  /* Pack the content of sSwathData->navigate into databuf */
  databuf = allocateVdataMemory(&tempVdata->vdataFields);
  ptr = databuf;
  memcpy(ptr, &sSwathData->navigate.scPosX, sizeof(sSwathData->navigate.scPosX));
  ptr += sizeof(sSwathData->navigate.scPosX);
  memcpy(ptr, &sSwathData->navigate.scPosY, sizeof(sSwathData->navigate.scPosY));
  ptr += sizeof(sSwathData->navigate.scPosY);
  memcpy(ptr, &sSwathData->navigate.scPosZ, sizeof(sSwathData->navigate.scPosZ));
  ptr += sizeof(sSwathData->navigate.scPosZ);
  memcpy(ptr, &sSwathData->navigate.scVelX, sizeof(sSwathData->navigate.scVelX));
  ptr += sizeof(sSwathData->navigate.scVelX);
  memcpy(ptr, &sSwathData->navigate.scVelY, sizeof(sSwathData->navigate.scVelY));
  ptr += sizeof(sSwathData->navigate.scVelY);
  memcpy(ptr, &sSwathData->navigate.scVelZ, sizeof(sSwathData->navigate.scVelZ));
  ptr += sizeof(sSwathData->navigate.scVelZ);
  memcpy(ptr, &sSwathData->navigate.scLat, sizeof(sSwathData->navigate.scLat)); 
  ptr += sizeof(sSwathData->navigate.scLat);
  memcpy(ptr, &sSwathData->navigate.scLon, sizeof(sSwathData->navigate.scLon));
  ptr += sizeof(sSwathData->navigate.scLon);
  memcpy(ptr, &sSwathData->navigate.scAlt, sizeof(sSwathData->navigate.scAlt));
  ptr += sizeof(sSwathData->navigate.scAlt);
  memcpy(ptr, &sSwathData->navigate.scAttRoll, sizeof(sSwathData->navigate.scAttRoll));
  ptr += sizeof(sSwathData->navigate.scAttRoll);
  memcpy(ptr, &sSwathData->navigate.scAttPitch, sizeof(sSwathData->navigate.scAttPitch));
  ptr += sizeof(sSwathData->navigate.scAttPitch);
  memcpy(ptr, &sSwathData->navigate.scAttYaw, sizeof(sSwathData->navigate.scAttYaw));
  ptr += sizeof(sSwathData->navigate.scAttYaw);
  memcpy(ptr, &sSwathData->navigate.att1, sizeof(sSwathData->navigate.att1));
  ptr += sizeof(sSwathData->navigate.att1);
  memcpy(ptr, &sSwathData->navigate.att2, sizeof(sSwathData->navigate.att2));
  ptr += sizeof(sSwathData->navigate.att2);
  memcpy(ptr, &sSwathData->navigate.att3, sizeof(sSwathData->navigate.att3));
  ptr += sizeof(sSwathData->navigate.att3);
  memcpy(ptr, &sSwathData->navigate.att4, sizeof(sSwathData->navigate.att4));
  ptr += sizeof(sSwathData->navigate.att4);
  memcpy(ptr, &sSwathData->navigate.att5, sizeof(sSwathData->navigate.att5));
  ptr += sizeof(sSwathData->navigate.att5);
  memcpy(ptr, &sSwathData->navigate.att6, sizeof(sSwathData->navigate.att6));
  ptr += sizeof(sSwathData->navigate.att6);
  memcpy(ptr, &sSwathData->navigate.att7, sizeof(sSwathData->navigate.att7));
  ptr += sizeof(sSwathData->navigate.att7);
  memcpy(ptr, &sSwathData->navigate.att8, sizeof(sSwathData->navigate.att8));
  ptr += sizeof(sSwathData->navigate.att8);
  memcpy(ptr, &sSwathData->navigate.att9, sizeof(sSwathData->navigate.att9));
  ptr += sizeof(sSwathData->navigate.att9);
  memcpy(ptr, &sSwathData->navigate.greenHourAng, sizeof(sSwathData->navigate.greenHourAng));
  vdataWrite(&tempVdata, databuf);
  free(databuf); 


  /* Pack the content of sSwathData->solarCal into databuf */
  databuf = allocateVdataMemory(&tempVdata->vdataFields);
  ptr = databuf;
  memcpy(ptr, &sSwathData->solarCal.sunVecX, sizeof(sSwathData->solarCal.sunVecX));
  ptr += sizeof(sSwathData->solarCal.sunVecX);
  memcpy(ptr, &sSwathData->solarCal.sunVecY, sizeof(sSwathData->solarCal.sunVecY));
  ptr += sizeof(sSwathData->solarCal.sunVecY);
  memcpy(ptr, &sSwathData->solarCal.sunVecZ, sizeof(sSwathData->solarCal.sunVecZ));
  ptr += sizeof(sSwathData->solarCal.sunVecZ);
  memcpy(ptr, &sSwathData->solarCal.sunMag, sizeof(sSwathData->solarCal.sunMag));
  vdataWrite ( &tempVdata, databuf );
  free ( databuf );
 
  /* Writing scientific data set into HDF format */ 
  sdsWrite(&tempSDS, &sSwathData->calCounts);

  /* Writing scientific data set into HDF format */ 
  sdsWrite(&tempSDS, &sSwathData->tempCounts);

  /*
  ** Scaling calculation.  For writing,
  **	The data is multiplied by the scale factor L1B01_LOCALDIR.
  */
/*
  for (i = 0; i < 27; ++i)
    for (j = 0; j < 2; ++j)
      for (k = 0; k < 2; ++k)
      {
*/
        /* The Following code was added to take care of the Missing or 
           invalid data.  The reason to add this code was not to scale
           the invalid Data */

/*
        if (sSwathData->localDirection[i][j][k] > TK_DEFAULT)
	    sSwathData->localDirection_scale[i][j][k] =
		(int16)RND(sSwathData->localDirection[i][j][k] * L1B01_LOCALDIR);
	else
	    sSwathData->localDirection_scale[i][j][k] =
TK_MISS_I2;
      }
  sdsWrite(&tempSDS, &sSwathData->localDirection_scale);
*/
  sdsWrite(&tempSDS, &sSwathData->localDirection);


  /*
   * Scaling calculation.  For writing,
   *	The data is multiplied by the scale factors L1B01_CHANNELS1, L1B01_CHANNELS2,
   *	L1B01_CHANNELS3, L1B01_CHANNELS4, and L1B01_CHANNELS5 for channels 1, 2, 3, 
   *	4, and 5 respectively.
   */
  for (i = 0; i < 261; ++i)
  {
        /* The Following code was added to take care of the Missing or
           invalid data.  The reason to add this code was not to scale
           the invalid Data */

       if (sSwathData->channels[i][0] > TK_DEFAULT)
            sSwathData->channels_scale[i][0] = 
	       (int16)RND(sSwathData->channels[i][0] * L1B01_CHANNELS1);
       else
            sSwathData->channels_scale[i][0] = TK_MISS_I2;
       if (sSwathData->channels[i][1] > TK_DEFAULT)
            sSwathData->channels_scale[i][1] = 
	       (int16)RND(sSwathData->channels[i][1] * L1B01_CHANNELS2);
       else
            sSwathData->channels_scale[i][1] = TK_MISS_I2;
       if (sSwathData->channels[i][2] > TK_DEFAULT)
            sSwathData->channels_scale[i][2] = 
	       (int16)RND(sSwathData->channels[i][2] * L1B01_CHANNELS3);
       else
            sSwathData->channels_scale[i][2] = TK_MISS_I2;
       if (sSwathData->channels[i][3] > TK_DEFAULT)
            sSwathData->channels_scale[i][3] = 
	       (int16)RND(sSwathData->channels[i][3] * L1B01_CHANNELS4);
       else
            sSwathData->channels_scale[i][3] = TK_MISS_I2;
       if (sSwathData->channels[i][4] > TK_DEFAULT)
            sSwathData->channels_scale[i][4] = 
	       (int16)RND(sSwathData->channels[i][4] * L1B01_CHANNELS5);
       else
            sSwathData->channels_scale[i][4] = TK_MISS_I2;
  }
  sdsWrite(&tempSDS, &sSwathData->channels_scale);

  return(TK_SUCCESS);
}


/***********************************************************************************
*
*  FUNCTION STATEMENT:
*       readScan_1B_01(IO_HANDLE *sGranuleHandle, 
*                       L1B_01_SWATHDATA *sSwathData) 
*
*  DESCRIPTION:
*       This function is used to read the L1B-01 information in HDF format.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       sGranuleHandle          A structure containing information about the file.
*                               sGranuleHandle is returned by TKopen().
*
*  OUTPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       sSwathData              A structure containing the complete scanline data.
*                               This structure must be declared to correspond to 
*                               the data file being read.  The structure of 
*                               sSwathData is manipulated by TKreadScan() and 
*				other toolkit routines.
*
*  CALLED BY:                   TKreadScan()
*
*  ROUTINES CALLED:     
*       Routine Name            Description
*       ------------            -----------
*       allocateVdataMemory     This routine is used to allocate memory to vdata.
*       sdsRead                 This routine is used to read the scientific data
*                               set in HDF format.
*       vdataRead               This routine is used to read the vdata in HDF
*                               format.
*
***********************************************************************************/

int readScan_1B_01(IO_HANDLE *sGranuleHandle, L1B_01_SWATHDATA *sSwathData)
{
  SDS_LINK *tempSDS  = sGranuleHandle->cfgSTR->vGroup->subVgroup->sds;
  VDATA_LINK *tempVdata = sGranuleHandle->cfgSTR->vGroup->subVgroup->vdata;
  uint8 *databuf, *ptr;
  int	i,j,k;
  int scaleFactor;

  intn  status;
  char sds_name[500];
  int32 sds_id, rank, data_type, n_attrs;
  int32 dim_sizes[500];

  /* Unpack the content of databuf into sSwathData->scanTime. */
  databuf = allocateVdataMemory(&tempVdata->vdataFields);
  ptr = databuf;
  vdataRead(&tempVdata, ptr);
  memcpy(&sSwathData->scanTime, ptr, sizeof(float64));
  free(databuf);

  /* Reading scientific data set into HDF format */
  sdsRead(&tempSDS, &scaleFactor, &sSwathData->geolocation);

  /* Unpack the content of databuf into sSwathData->scanStatus. */
  databuf = allocateVdataMemory(&tempVdata->vdataFields);
  ptr = databuf;
  vdataRead(&tempVdata, ptr);
  memcpy(&sSwathData->scanStatus.missing, ptr, sizeof(sSwathData->scanStatus.missing));
  ptr += sizeof(sSwathData->scanStatus.missing);
  memcpy(&sSwathData->scanStatus.validity, ptr, sizeof(sSwathData->scanStatus.validity));
  ptr += sizeof(sSwathData->scanStatus.validity);
  memcpy(&sSwathData->scanStatus.qac, ptr, sizeof(sSwathData->scanStatus.qac));
  ptr += sizeof(sSwathData->scanStatus.qac);
  memcpy(&sSwathData->scanStatus.geoQuality, ptr, sizeof(sSwathData->scanStatus.geoQuality));
  ptr += sizeof(sSwathData->scanStatus.geoQuality);
  memcpy(&sSwathData->scanStatus.ch1Quality, ptr, sizeof(sSwathData->scanStatus.ch1Quality));
  ptr += sizeof(sSwathData->scanStatus.ch1Quality);
  memcpy(&sSwathData->scanStatus.ch2Quality, ptr, sizeof(sSwathData->scanStatus.ch2Quality));
  ptr += sizeof(sSwathData->scanStatus.ch2Quality);
  memcpy(&sSwathData->scanStatus.ch3Quality, ptr, sizeof(sSwathData->scanStatus.ch3Quality));
  ptr += sizeof(sSwathData->scanStatus.ch3Quality);
  memcpy(&sSwathData->scanStatus.ch4Quality, ptr, sizeof(sSwathData->scanStatus.ch4Quality));
  ptr += sizeof(sSwathData->scanStatus.ch4Quality);
  memcpy(&sSwathData->scanStatus.ch5Quality, ptr, sizeof(sSwathData->scanStatus.ch5Quality));
  ptr += sizeof(sSwathData->scanStatus.ch5Quality);
  memcpy(&sSwathData->scanStatus.scOrient, ptr, sizeof(sSwathData->scanStatus.scOrient));
  ptr += sizeof(sSwathData->scanStatus.scOrient);
  memcpy(&sSwathData->scanStatus.acsMode, ptr, sizeof(sSwathData->scanStatus.acsMode));
  ptr += sizeof(sSwathData->scanStatus.acsMode);
  memcpy(&sSwathData->scanStatus.yawUpdateS, ptr, sizeof(sSwathData->scanStatus.yawUpdateS));
  ptr += sizeof(sSwathData->scanStatus.yawUpdateS);
  memcpy(&sSwathData->scanStatus.virsInstS, ptr, sizeof(sSwathData->scanStatus.virsInstS));
  ptr += sizeof(sSwathData->scanStatus.virsInstS);
  memcpy(&sSwathData->scanStatus.virsMode, ptr, sizeof(sSwathData->scanStatus.virsMode));
  ptr += sizeof(sSwathData->scanStatus.virsMode);
  memcpy(&sSwathData->scanStatus.virsAbnCon, ptr, sizeof(sSwathData->scanStatus.virsAbnCon));
  ptr += sizeof(sSwathData->scanStatus.virsAbnCon);
  memcpy(&sSwathData->scanStatus.fractOrbitN, ptr, sizeof(sSwathData->scanStatus.fractOrbitN));
  free(databuf);


  /* Unpack the content of databuf into sSwathData->navigate. */
  databuf = allocateVdataMemory(&tempVdata->vdataFields);
  ptr = databuf;
  vdataRead(&tempVdata, ptr);
  memcpy(&sSwathData->navigate.scPosX, ptr, sizeof(sSwathData->navigate.scPosX));
  ptr += sizeof(sSwathData->navigate.scPosX);
  memcpy(&sSwathData->navigate.scPosY, ptr, sizeof(sSwathData->navigate.scPosY));
  ptr += sizeof(sSwathData->navigate.scPosY);
  memcpy(&sSwathData->navigate.scPosZ, ptr, sizeof(sSwathData->navigate.scPosZ));
  ptr += sizeof(sSwathData->navigate.scPosZ);
  memcpy(&sSwathData->navigate.scVelX, ptr, sizeof(sSwathData->navigate.scVelX));
  ptr += sizeof(sSwathData->navigate.scVelX);
  memcpy(&sSwathData->navigate.scVelY, ptr, sizeof(sSwathData->navigate.scVelY));
  ptr += sizeof(sSwathData->navigate.scVelY);
  memcpy(&sSwathData->navigate.scVelZ, ptr, sizeof(sSwathData->navigate.scVelZ));
  ptr += sizeof(sSwathData->navigate.scVelZ);
  memcpy(&sSwathData->navigate.scLat, ptr, sizeof(sSwathData->navigate.scLat));
  ptr += sizeof(sSwathData->navigate.scLat);
  memcpy(&sSwathData->navigate.scLon, ptr, sizeof(sSwathData->navigate.scLon));
  ptr += sizeof(sSwathData->navigate.scLon);
  memcpy(&sSwathData->navigate.scAlt, ptr, sizeof(sSwathData->navigate.scAlt));
  ptr += sizeof(sSwathData->navigate.scAlt);
  memcpy(&sSwathData->navigate.scAttRoll, ptr, sizeof(sSwathData->navigate.scAttRoll));
  ptr += sizeof(sSwathData->navigate.scAttRoll);
  memcpy(&sSwathData->navigate.scAttPitch, ptr, sizeof(sSwathData->navigate.scAttPitch));
  ptr += sizeof(sSwathData->navigate.scAttPitch);
  memcpy(&sSwathData->navigate.scAttYaw, ptr, sizeof(sSwathData->navigate.scAttYaw));
  ptr += sizeof(sSwathData->navigate.scAttYaw);
  memcpy(&sSwathData->navigate.att1, ptr, sizeof(sSwathData->navigate.att1));
  ptr += sizeof(sSwathData->navigate.att1);
  memcpy(&sSwathData->navigate.att2, ptr, sizeof(sSwathData->navigate.att2));
  ptr += sizeof(sSwathData->navigate.att2);
  memcpy(&sSwathData->navigate.att3, ptr, sizeof(sSwathData->navigate.att3));
  ptr += sizeof(sSwathData->navigate.att3);
  memcpy(&sSwathData->navigate.att4, ptr, sizeof(sSwathData->navigate.att4));
  ptr += sizeof(sSwathData->navigate.att4);
  memcpy(&sSwathData->navigate.att5, ptr, sizeof(sSwathData->navigate.att5));
  ptr += sizeof(sSwathData->navigate.att5);
  memcpy(&sSwathData->navigate.att6, ptr, sizeof(sSwathData->navigate.att6));
  ptr += sizeof(sSwathData->navigate.att6);
  memcpy(&sSwathData->navigate.att7, ptr, sizeof(sSwathData->navigate.att7));
  ptr += sizeof(sSwathData->navigate.att7);
  memcpy(&sSwathData->navigate.att8, ptr, sizeof(sSwathData->navigate.att8));
  ptr += sizeof(sSwathData->navigate.att8);
  memcpy(&sSwathData->navigate.att9, ptr, sizeof(sSwathData->navigate.att9));
  ptr += sizeof(sSwathData->navigate.att9);
  memcpy(&sSwathData->navigate.greenHourAng, ptr, sizeof(sSwathData->navigate.greenHourAng));
  free(databuf);

  /* Unpack the content of databuf into sSwathData->solarCal. */
  databuf = allocateVdataMemory(&tempVdata->vdataFields);
  ptr = databuf;
  vdataRead(&tempVdata, ptr);
  memcpy(&sSwathData->solarCal.sunVecX, ptr, sizeof(sSwathData->solarCal.sunVecX));
  ptr += sizeof(sSwathData->solarCal.sunVecX);
  memcpy(&sSwathData->solarCal.sunVecY, ptr, sizeof(sSwathData->solarCal.sunVecY));
  ptr += sizeof(sSwathData->solarCal.sunVecY);
  memcpy(&sSwathData->solarCal.sunVecZ, ptr, sizeof(sSwathData->solarCal.sunVecZ));
  ptr += sizeof(sSwathData->solarCal.sunVecZ);
  memcpy(&sSwathData->solarCal.sunMag, ptr, sizeof(sSwathData->solarCal.sunMag));
  free ( databuf );

  /* Reading scientific data set into HDF format */
  sdsRead(&tempSDS, &scaleFactor, &sSwathData->calCounts);

  /* Reading scientific data set into HDF format */
  sdsRead(&tempSDS, &scaleFactor, &sSwathData->tempCounts);

  /*
  ** Scaling calculation.  For reading,
  **	The data is divided by the scale factor L1B01_LOCALDIR.
  */
/*
  sdsRead(&tempSDS, &scaleFactor, &sSwathData->localDirection_scale);
  for (i = 0; i < 27; ++i)
    for (j = 0; j < 2; ++j)
      for(k = 0; k < 2; ++k)
        {
        if (sSwathData->localDirection_scale[i][j][k] != TK_MISS_I2)
	   sSwathData->localDirection[i][j][k] =
		(sSwathData->localDirection_scale[i][j][k] * 1.0) / scaleFactor;
        else
           sSwathData->localDirection[i][j][k] = TK_DEFAULT;
        }
*/

  status = SDgetinfo(tempSDS->sdsID,sds_name,&rank,dim_sizes,&data_type,&n_attrs);

  if (data_type==DFNT_INT16)
  {
  sdsRead(&tempSDS, &scaleFactor, &sSwathData->localDirection_scale);
  for (i = 0; i < 27; ++i)
    for (j = 0; j < 2; ++j)
      for(k = 0; k < 2; ++k)
        {
        if (sSwathData->localDirection_scale[i][j][k] != TK_MISS_I2)
           sSwathData->localDirection[i][j][k] =
                (sSwathData->localDirection_scale[i][j][k] * 1.0) / scaleFactor;        else
           sSwathData->localDirection[i][j][k] = TK_DEFAULT;
        }

  }
  else
  {
      sdsRead(&tempSDS, &scaleFactor, &sSwathData->localDirection);
   }

  /*
   * Scaling calculation.  For reading,
   *    The data is divided by the scale factors L1B01_CHANNELS1, L1B01_CHANNELS2, 
   *	L1B01_CHANNELS3, L1B01_CHANNELS4, and L1B01_CHANNELS5 for channels 1, 2, 3, 
   *	4, and 5 respectively.
   */
  sdsRead(&tempSDS, &scaleFactor, &sSwathData->channels_scale);
  for (i = 0; i < 261; ++i)
  {
       if (sSwathData->channels_scale[i][0] != TK_MISS_I2)
          sSwathData->channels[i][0] = 
		(sSwathData->channels_scale[i][0]*1.0)/ L1B01_CHANNELS1;
       else
          sSwathData->channels[i][0] = TK_DEFAULT;
       if (sSwathData->channels_scale[i][1] != TK_MISS_I2)
          sSwathData->channels[i][1] = 
		(sSwathData->channels_scale[i][1]*1.0)/ L1B01_CHANNELS2;
       else
          sSwathData->channels[i][1] = TK_DEFAULT;
       if (sSwathData->channels_scale[i][2] != TK_MISS_I2)      
          sSwathData->channels[i][2] = 
		(sSwathData->channels_scale[i][2]*1.0)/ L1B01_CHANNELS3;
       else
          sSwathData->channels[i][2] = TK_DEFAULT;
       if (sSwathData->channels_scale[i][3] != TK_MISS_I2)
          sSwathData->channels[i][3] = 
		(sSwathData->channels_scale[i][3]*1.0)/ L1B01_CHANNELS4;
       else
          sSwathData->channels[i][3] = TK_DEFAULT;
       if (sSwathData->channels_scale[i][4] != TK_MISS_I2)
          sSwathData->channels[i][4] = 
		(sSwathData->channels_scale[i][4]*1.0)/ L1B01_CHANNELS5;
       else
          sSwathData->channels[i][4] = TK_DEFAULT;
  };

  return(TK_SUCCESS);
}


/***********************************************************************************
*
*  FUNCTION STATEMENT:
*       writeGrid_3B_42(IO_HANDLE *sGranuleHandle, L3B_42_PLANETGRID *sPlanetGrid)
*
*  DESCRIPTION:
*       This function is used to write the L3B-42 information in HDF format.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       sGranuleHandle          A structure containing information about the file.
*                               sGranuleHandle is returned by TKopen().
*       sPlanetGrid		A structure contains the complete grid data.
*
*  OUTPUT PARAMETERS:           None
*
*  CALLED BY:                   TKwriteGrid()
*
*  ROUTINES CALLED:     
*	allocateVdataMemory(), vdataWrite(), sdsWrite()	
*
***********************************************************************************/

int writeGrid_3B_42(IO_HANDLE *sGranuleHandle, L3B_42_PLANETGRID *sPlanetGrid)
{
  SDS_LINK *tempSDS = sGranuleHandle->cfgSTR->vGroup->subVgroup->sds;
  VDATA_LINK *tempVdata = sGranuleHandle->cfgSTR->vGroup->subVgroup->vdata;
  uint8 *databuf, *ptr;

  /* Write SDS to a HDF file */
  sdsWrite(&tempSDS, &sPlanetGrid->precipitation);
  sdsWrite(&tempSDS, &sPlanetGrid->relativeError);

  return(TK_SUCCESS);
}


/***********************************************************************************
*
*  FUNCTION STATEMENT:
*       readGrid_3B_42(IO_HANDLE *sGranuleHandle, L3B_42_PLANETGRID *sPlanetGrid)
*
*  DESCRIPTION:
*       This function is used to read the L3B-42 information in HDF format.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       sGranuleHandle          A structure containing information about the file.
*                               sGranuleHandle is returned by TKopen().
*
*  OUTPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       sPlanetGrid		A structure contains the complete grid data.
*
*  CALLED BY:                   TKreadGrid()
*
*  ROUTINES CALLED:     	
*	allocateVdataMemory(), vdataRead(), sdsRead()
*
***********************************************************************************/

int readGrid_3B_42(IO_HANDLE *sGranuleHandle, L3B_42_PLANETGRID *sPlanetGrid)
{
  SDS_LINK *tempSDS = sGranuleHandle->cfgSTR->vGroup->subVgroup->sds;
  VDATA_LINK *tempVdata = sGranuleHandle->cfgSTR->vGroup->subVgroup->vdata;
  uint8 *databuf, *ptr;
  int scaleFactor;

  /* Read SDS from a HDF file */
  if (versionCompare("6.18",sGranuleHandle->TKversion) >= 0) sdsRead(&tempSDS, &scaleFactor, &sPlanetGrid->precipitate); 
  if (versionCompare("6.18",sGranuleHandle->TKversion) >= 0) sdsRead(&tempSDS, &scaleFactor, &sPlanetGrid->relError);
  sdsRead(&tempSDS, &scaleFactor, &sPlanetGrid->precipitation); 
  sdsRead(&tempSDS, &scaleFactor, &sPlanetGrid->relativeError); 

  return(TK_SUCCESS);
}


/***********************************************************************************
*
*  FUNCTION STATEMENT:
*       writeGrid_3B_43(IO_HANDLE *sGranuleHandle, L3B_43_PLANETGRID *sPlanetGrid)
*
*  DESCRIPTION:
*       This function is used to write the L3B_43 information in HDF format.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       sGranuleHandle          A structure containing information about the file.
*                               sGranuleHandle is returned by TKopen().
*       sPlanetGrid		A structure contains the complete grid data.
*
*  OUTPUT PARAMETERS:           None
*
*  CALLED BY:                   TKwriteGrid()
*
*  ROUTINES CALLED:     
*	allocateVdataMemory(), sdsWrite(), vdataWrite()	
*
***********************************************************************************/

int writeGrid_3B_43(IO_HANDLE *sGranuleHandle, L3B_43_PLANETGRID *sPlanetGrid)
{
  SDS_LINK *tempSDS = sGranuleHandle->cfgSTR->vGroup->subVgroup->sds;
  VDATA_LINK *tempVdata = sGranuleHandle->cfgSTR->vGroup->subVgroup->vdata;
  uint8 *databuf, *ptr;

  /* Write SDS to a HDF file */
  sdsWrite(&tempSDS, &sPlanetGrid->precipitation);
  sdsWrite(&tempSDS, &sPlanetGrid->relativeError);

  return(TK_SUCCESS);
}


/***********************************************************************************
*
*  FUNCTION STATEMENT:
*       readScan_3B_43(IO_HANDLE *sGranuleHandle, L3B_43_PLANETGRID *sPlanetGrid)
*
*  DESCRIPTION:
*       This function is used to read the L3B-43 information in HDF format.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       sGranuleHandle          A structure containing information about the file
*                               sGranuleHandle is returned by TKopen().
*
*  OUTPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       sPlanetGrid		A structure contains the complete grid data.
*
*  CALLED BY:                   TKreadGrid()
*
*  ROUTINES CALLED:     
*	allocateVdataMemory(), vdataRead(), sdsRead()	
*
***********************************************************************************/

int readGrid_3B_43(IO_HANDLE *sGranuleHandle, L3B_43_PLANETGRID *sPlanetGrid)
{
  SDS_LINK *tempSDS = sGranuleHandle->cfgSTR->vGroup->subVgroup->sds;
  VDATA_LINK *tempVdata = sGranuleHandle->cfgSTR->vGroup->subVgroup->vdata;
  uint8 *databuf, *ptr;
  int scaleFactor;

  /* Read SDS from a HDF file */
  if (versionCompare("6.18",sGranuleHandle->TKversion) >= 0) sdsRead(&tempSDS, &scaleFactor, &sPlanetGrid->precipitate); 
  if (versionCompare("6.18",sGranuleHandle->TKversion) >= 0) sdsRead(&tempSDS, &scaleFactor, &sPlanetGrid->relError);
  sdsRead(&tempSDS, &scaleFactor, &sPlanetGrid->precipitation); 
  sdsRead(&tempSDS, &scaleFactor, &sPlanetGrid->relativeError); 

  return(TK_SUCCESS);
}


/***********************************************************************************
*
*  FUNCTION STATEMENT:
*       setScalingFactors_1B01(VGROUP_LINK *vgroup)
*
*  DESCRIPTION:
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       vgroup                  List of vgroups.  The list of SDSs are retrieve from
*                               this node.
*
*  OUTPUT PARAMETERS:  None.
*
*  CALLED BY:                   setSDSattributes().
*
*  ROUTINES CALLED:     
*       Routine Name            Desc
*       ------------            -----------
*       create_pvl              This routine creates the Parameter Value string
*                               for the ECS style metadata.
*       SDsetcal                This routine adds the scaling factor and offset 
*                               to the SDS.
*       TKreportWarning         This routine is called for warnings and informational
*                               messages.  Calling this routine will send an error
*                               message to the central log file and return control to 
*                               the calling routine.
*
***********************************************************************************/
int setScalingFactors_1B01 ( VGROUP_LINK *vgroup )
{
  char *scaleFactorChan;
  char scaleFactorBuffer[10];
  int  status = TK_SUCCESS;
  SDS_LINK *sds;
  
  if ( vgroup != NULL )
    {
      sds = vgroup->sds;
      while ( sds != NULL )
	{
	  if ( !strcmp ( sds->name, "channels" ) )
	    {
	      scaleFactorChan = ( char * ) calloc ( TK_MAX_SZ_ATTRIBUTE, 
						   sizeof ( char ) );
	      if ( scaleFactorChan == NULL )
		{
		  TKreportWarning ( E_TK_MEMFAILMETA );
		  return ( E_TK_MEMFAILMETA );
		}
 
	      /*  
	       *  Since the VIRS channel data has 5 scaling factors for one
	       *  SDS, they will be written in a user defined attributes. 
	       *  The format of the attribute will be the same as the ECS
	       *  style metadata.
	       */
	      sprintf ( scaleFactorBuffer, "%d", L1B01_CHANNELS1 );
	      create_pvl ( scaleFactorChan, "CHANNEL_1", scaleFactorBuffer );
	      sprintf ( scaleFactorBuffer, "%d", L1B01_CHANNELS2 );
	      create_pvl ( scaleFactorChan, "CHANNEL_2", scaleFactorBuffer );
	      sprintf ( scaleFactorBuffer, "%d", L1B01_CHANNELS3 );
	      create_pvl ( scaleFactorChan, "CHANNEL_3", scaleFactorBuffer );
	      sprintf ( scaleFactorBuffer, "%d", L1B01_CHANNELS4 );
	      create_pvl ( scaleFactorChan, "CHANNEL_4", scaleFactorBuffer );
	      sprintf ( scaleFactorBuffer, "%d", L1B01_CHANNELS5 );
	      create_pvl ( scaleFactorChan, "CHANNEL_5", scaleFactorBuffer );
	      strcat ( scaleFactorChan, "END;\n" );

	      status = SDsetattr ( sds->sdsID, "VIRSChannelsScalingFactors.0",
				  DFNT_CHAR8, strlen ( scaleFactorChan ),
				  (VOIDP) scaleFactorChan );
	      if ( status != TK_SUCCESS )
		{
		  TKreportWarning ( W_TK_SETATTR );
		  return ( W_TK_SETATTR );
		}
	      free ( scaleFactorChan );
	    }
/*
	  else if ( !strcmp ( sds->name, "localDirection" ) )
	    status = SDsetcal ( sds->sdsID, (float64) L1B01_LOCALDIR, 
			       (float64) 0, (float64) 0, (float64) 0,
			       DFNT_INT16 );
*/

	  if ( status == TK_HDF_FAIL )
	    TKreportWarning ( E_TK_SETCALINFO );
	  sds = sds->next;
	}
      setScalingFactors_1B01 ( vgroup->subVgroup );
      setScalingFactors_1B01 ( vgroup->next );
    }
  return ( status );
}


