/* DEVELOPMENT HISTORY */
/* Revision 1.39  1998/06/26 17:25:45  slywczak
 * Modified functions getConfigFilenameAndScanline and checkAllFiles.  Added the
 * realtime data types with the corresponding configuration filenames to
 * getConfigFilenameandScanline.  Added default metadata copying (retrieving the
 * input filename) to checkAllFiles.
 *
 * Revision 1.38  1997/09/29 22:18:07  slywczak
 * Modified function checkAllFiles.  When searching for a CSI granule, the
 * conditions were not set appropriately for 2A25.  The initial search (for
 * priority) was not happening.  Added code to make this first search before
 * returning the CSI granule handle.
 *
 * Revision 1.37  1997/09/15 20:46:31  slywczak
 * Modified checkAllFiles.  Added code to copy the metadata from the input to
 * the output for products of the same id's.  This will make sure that CSIs have
 * the correct metadata.
 *
 * Revision 1.36  1997/06/18 19:46:13  slywczak
 * Modified the function checkAllFiles.  Previously, if the output file was L1C
 * GV, it was looking for an input file of L1B.  But, since the reprocessing may
 * take L1C as input and output another L1C, the code was changed to look for
 * either a L1B or L1C granule.
 *
 * Revision 1.35  1997/05/14 19:34:14  slywczak
 * Modified the following routines:  manageFiles and checkAllFiles.  Changed the
 * code to return the input handle rather than just the metadata portion of the
 * input handle.
 *
 * Revision 1.34  1997/02/21 20:23:00  piper
 * Fixed if statement from = to ==.
 *
 * Revision 1.33  1997/02/04 16:42:10  amak
 * Modified getConfigFilenameAndScanline().  That allows Level 1 GV config
 * file providing concurrent processing capability.
 *
 * Revision 1.32  1996/12/30 18:18:48  amak
 * Deleted excessive code in deleteAllFiles().
 *
 * Revision 1.31  1996/12/24 14:31:31  amak
 * Incorporated walkthrough comments.
 * - Changed '<' and '>' to double quotes around the #include statements.
 *
 * Revision 1.30  1996/11/06  15:28:15  amak
 * Added detailed description to the routine.
 *
 * Revision 1.29  1996/11/01  20:34:34  amak
 * Added new routine checkAllFiles() to find out copy metadata handle.
 * Added new routine deleteAllLists() and deleteList() to delete the linked
 * list and free memory before exiting the program.
 * Modified manageFiles()
 *   - added one more parameter 'sHandle' to the function
 *   - added two more case statements
 * Modified addFile()
 *   - added one more parameter 'ckList' to the function
 *
 * Revision 1.28  1996/09/03  20:07:37  amak
 * Added case statements TK_L2A_56 and TK_L2A_57 to function 
 * getConfigFilenameAndScanline().
 *
 * Revision 1.27  1996/07/09  17:57:56  dbazell
 * Deleted references to coreMetadataID in deleteAllFiles.c and addFile.c
 *
 * Revision 1.26  1996/06/19  14:26:00  amak
 * Modified routines.  coreMetadataID and psMetadataID are obsolete.
 *
 * Revision 1.25  1996/06/07  14:46:15  amak
 * Added more case statements in routine getConfigFilenameAndScanline() to
 * extract different configuration files for different products.
 *
 * Revision 1.24  1996/04/19  15:40:48  amak
 * Made modifications to the preprocessor #include statement.  Replaced
 * TKerrorRou.h and TKwarnRou.h to TKerrHandle.h and TS_TK_15.h for new
 * error handling calls.
 *
 * Revision 1.23  1996/03/21  22:00:25  amak
 * Added prologue.
 *
 * Revision 1.22  1996/03/04  19:17:00  amak
 * Modified routine addFiles() based on the new changes on metadata data
 * structure.
 *
 * Revision 1.21  1995/12/19  17:37:14  amak
 * Made change on routine getConfigFilenameAndScanline().  The L1C-21 data
 * product has the same format as L1B-21, so the calling of the L1C-21
 * configuration file will be the same as L1B-21.
 *
 * Revision 1.20  1995/12/15  15:17:15  thyun
 * Modified delteAllFiles.
 *
 * Revision 1.19  1995/12/12  15:09:40  amak
 * Fixed typo errors in prologues.
 *
 * Revision 1.18  1995/12/05  18:37:07  thyun
 * Inserted prolog for getConfigFilenameAndScanline.
 *
 * Revision 1.17  1995/11/15  23:39:11  thyun
 * Implemented the procedure getConfigFilenameAndScanline.
 *
 * Revision 1.16  1995/10/18  20:52:25  amak
 * Made modifications after some files need to be renamed.
 *
 * Revision 1.15  1995/10/10  15:27:21  thyun
 * Changed warning status to error status when memory allocation failed in
 * addFile.
 *
 * Revision 1.14  1995/08/15  21:16:28  thyun
 * Added comments for the TKreportWarning and TKreportError routines.
 *
 * Revision 1.13  1995/08/11  11:38:20  amak
 * Added comments for the TKreportWarning and TKreportError routines.
 *
 * Revision 1.12  1995/08/03  18:23:06  thyun
 * Modified addFile to refelct changes in the data structures in template.h
 *
 * Revision 1.11  1995/08/03  17:45:40  amak
 * Re-modified the code because CVS couldn't do version control nicely.
 *
 * Revision 1.10  1995/07/20  21:08:39  thyun
 * Modified addFile by initializing tempNode->pGranuleHandle.cfgSTR.
 *
 * Revision 1.9  1995/07/20  20:23:14  thyun
 * Modified the routine deleteFile to refelct the type change in the structure
 * FILE_PTR.
 *
 * Revision 1.8  1995/07/19  20:59:51  thyun
 * Replaced error messages using new error routines.
 *
 * Revision 1.7  1995/07/19  20:09:01  thyun
 * Changed addFile, deleteAllFiles, and deleteFile to reflect the type change of
 * the field pGranuleHandle in the strucuture FILE_PTR.
 *
 * Revision 1.6  1995/07/18  20:22:42  thyun
 * Corrected type mistakes and grammer mistakes in comments and changed the name
 * of parameter "files" to "fileList", and modified tempPTR1 and tempPTR2 in
 * deleteFile.
 *
 * Revision 1.5  1995/07/07  20:30:51  amak
 * Added detailed comments
 *
 * Revision 1.4  1995/07/07  18:48:09  thyun
 * Modified occurrances of 0 to NULL and -1 to TK_FAIL
 *
 * Revision 1.3  1995/07/07  14:08:32  thyun
 * Modified and added comments
 *
 * Revision 1.2  1995/07/05  17:11:44  thyun
 * Modified comments
 *
 * Revision 1.1  1995/06/30  18:33:41  thyun
 * adding file
 **/


/**********************************************************************************
*
*  FILE NAME:           file_manager.c
*
*  DESCRIPTION:    
*	This program handles the HDF files opened in case of any critical error
*       condition occurs during the execution of Toolkit program.
*
*  ROUTINES CALLED:
*       Routine Name            Description
*       ------------            -----------
*       manageFiles             This procedure performs add and delete files from the
*				list of files depending on the operation type.
*	addFile			This procedure adds a file in the front the list of
*				files.
*	deleteAllFiles		This procedure deletes all the files from the list of
*				files.
*	deleteFile		This procedure deletes a given file from the list of
*				files.
*	getConfigFilenameAndScanline	
*				This function finds the configuration file name
*				using productID and returns the number of
*				scanlines.
*	checkAllFiles		This routine used to check the input/output file
*				handle status for metadata header copy.
*	deleteAllLists		This routine used to delete and free the memory
*				of the whole linked list.
*	deleteList		This routine used to delete the individual list
*				from the linked list.
*	
**********************************************************************************/

#include <sys/types.h>
#include <unistd.h>
#include <mfhdf.h>
#include "file_manager.h"
#include "TKerrHandle.h"
#include "TS_TK_15.h"


/**********************************************************************************
*
*  FUNCTION STATEMENT:
*	manageFiles(IO_HANDLE *sGranuleHandle, IO_HANDLE *sHandle, int opType)
*
*  DESCRIPTION:         
*	This procedure performs add and delete files from the linked list depending
*       on the operation type.
*
*  INPUT PARAMETERS:
*	Parameter Name		Description
*	--------------		-----------
*	sGranuleHandle		A structure handle which contains information about 
*				the file
*	opType			Macro to determine the operation type for the routine
*
*  OUTPUT PARAMETER:
*	Parameter Name		Description
*	--------------		-----------
*	sHandle			The output structure which contains information
*				of the metadata handle.
*
*  CALLED BY:                   TKopen(), TKclose(), reportException(). 
*
*  ROUTINES CALLED:
*       Routine Name            Description
*       ------------            -----------
*	addFile			This procedure adds a file in front of list of files.
*       deleteAllFiles		This procedure deletes all the files from the list of
*				files.
*       deleteFile		This procedure deletes a given file from the list of
*				files.
*       TKreportWarning         This routine reports warning errors to the error
*                               log file and continues processing.
*	checkAllFiles		This routine used to check the input/output file
*				handle status for metadata header copy.
*	deleteAllLists		This routine is used to delete all the linked lists.
*	deleteList		This routine is used to delete the individual linked
*				list.
*
**********************************************************************************/

int manageFiles(IO_HANDLE *sGranuleHandle, IO_HANDLE **sHandle, int opType)
{ 
  static FILE_PTR *fileList = 0;
  static FILE_PTR *ckList = 0;

  switch(opType)
  {
     case ADD_FILE : 
          return(addFile(sGranuleHandle, &fileList, &ckList));
     case DELETE_FILE :
	  return(deleteFile(sGranuleHandle, &fileList)); 
     case DELETE_ALL_FILES :
          deleteAllFiles(&fileList);
	  break;
     case CHECK_FILE_STATUS:
	  return(checkAllFiles(sGranuleHandle, sHandle, &fileList, &ckList));
     case DELETE_ALL_LISTS:
	  if (fileList == NULL)
	  	deleteAllLists(&ckList);
	  break;
     case DELETE_LIST:
	  deleteList(&ckList);
	  break;
     default:
	  /* Invalid operation type in manageFiles */
	  TKreportWarning(W_TK_BADOPTYPE); 
	  return(TK_FAIL);  
  }

  return(TK_SUCCESS);
}



/**********************************************************************************
*
*  FUNCTION STATEMENT:
*       addFile(IO_HANDLE *sGranuleHandle, FILE_PTR **fileList,
*		FILE_PTR **ckList)
*
*  DESCRIPTION:         
*       This procedure adds a file in front of the list of files.
*
*  INPUT PARAMETER:
*	Parameter Name		Description
*	--------------		-----------
*	sGranuleHandle		The structure handle which contains information 
*				about the file.  
*
*  OUTPUT PARAMETER:
*	Parameter Name		Description
*	--------------		-----------
*	fileList		A linked list that contains information about
*				all the files which have been opened.
*	ckList			A linked list that contains information about 
*				all the files which have been opened.
*
*  CALLED BY:                   manageFiles()
*
*  RETURN STATUS:		TK_SUCCESS if success.
*
*  ROUTINES CALLED:	
*       Routine Name            Description
*       ------------            -----------
*       TKreportWarning         This routine reports warning errors to the error
*                               log file and continues processing.
*
**********************************************************************************/
int addFile(IO_HANDLE *sGranuleHandle, FILE_PTR **fileList, FILE_PTR **ckList)
{
  FILE_PTR *tempNode = (FILE_PTR *)malloc(sizeof(FILE_PTR));
  FILE_PTR *checkNode = (FILE_PTR *)malloc(sizeof(FILE_PTR));

  if ((tempNode != NULL) && (checkNode != NULL))
  { 
      tempNode->pGranuleHandle.cfgSTR  = sGranuleHandle->cfgSTR;  

      tempNode->pGranuleHandle.metadata = sGranuleHandle->metadata;
      checkNode->pGranuleHandle.metadata = tempNode->pGranuleHandle.metadata;

      tempNode->pGranuleHandle.productID = sGranuleHandle->productID;
      checkNode->pGranuleHandle.productID = tempNode->pGranuleHandle.productID;

      tempNode->pGranuleHandle.mode = sGranuleHandle->mode;
      checkNode->pGranuleHandle.mode = tempNode->pGranuleHandle.mode;

      strcpy( tempNode->pGranuleHandle.dataFileName,
	     sGranuleHandle->dataFileName );
      strcpy ( checkNode->pGranuleHandle.dataFileName,
	      tempNode->pGranuleHandle.dataFileName );

      tempNode->pGranuleHandle.gIDtokens = sGranuleHandle->gIDtokens;
      checkNode->pGranuleHandle.gIDtokens = tempNode->pGranuleHandle.gIDtokens;
      tempNode->next 	= *fileList;
      checkNode->next	= *ckList;

      *fileList = tempNode;
      *ckList	= checkNode;
  }
  else
  {   
      /* Memory allocation error in addFile */
      TKreportWarning(W_TK_MEMERRFILES); 
      return(TK_FAIL);
  }

  return(TK_SUCCESS);
}
  


/**********************************************************************************
*
*  FUNCTION STATEMENT:
*       deleteAllFiles(FILE_PTR **fileList)
*
*  DESCRIPTION:
*       This procedure deletes all the files from the list of files.
*
*  INPUT PARAMETER:
*	Parameter Name		Description
*	--------------		-----------
*	fileList		A linked list that contains all information of the
*				files which have been opened.
*
*  OUTPUT PARAMETER:		None
*
*  CALLED BY:                   manageFiles()
*
*  RETURN STATUS:		None
*
*  ROUTINES CALLED:		None
*
**********************************************************************************/

void deleteAllFiles(FILE_PTR **fileList)
{ 
  FILE_PTR *tempPTR1 = *fileList;
  FILE_PTR *tempPTR2;
  CFG_STR *tempSTR;

  while (tempPTR1 != NULL)
  {
	 tempPTR2 = tempPTR1;
         tempPTR1 = tempPTR1->next;
	 Hclose(tempPTR2->pGranuleHandle.cfgSTR->hOpenID);
	 tempSTR = tempPTR2->pGranuleHandle.cfgSTR;

	 manageFiles(&tempPTR2->pGranuleHandle, 0, DELETE_FILE);
	 deallocateCFG_STR(&tempSTR);
  }
}


/**********************************************************************************
*
*  FUNCTION STATEMENT:
*       deleteFile(IO_HANDLE *sGranuleHandle, FILE_PTR **fileList)
*
*  DESCRIPTION:
*       This procedure deletes a given file from the list of files.
*
*  INPUT PARAMETER:
*	Parameter Name		Description
*	--------------		-----------
*	sGranuleHandle		A structure which contains information about the
*				file.
*	fileList		A linked list that contains all information of the
*				files which have been opened.
*
*  OUTPUT PARAMETER:		None
*
*  CALLED BY:                   manageFiles(), deleteAllFiles().
*
*  RETURN STATUS:		TK_SUCCESS if success; TK_FAIL if fail.
*
*  ROUTINES CALLED:		
*       Routine Name            Description
*       ------------            -----------
*       TKreportWarning         This routine reports warning errors to the console
*                               and continues processing.
*
**********************************************************************************/
int deleteFile(IO_HANDLE *sGranuleHandle, FILE_PTR **fileList)
{ 
  FILE_PTR *tempPTR1 = *fileList;
  FILE_PTR *tempPTR2 = tempPTR1;
  int sameIoHandle = TK_FALSE;

  if (tempPTR1 != NULL)
  {	
	/************************************************************************/
	/* Search for the file that needs to be deleted.			*/
	/************************************************************************/
       while((tempPTR1 != NULL) && (sameIoHandle == TK_FALSE))
       {    
              if (tempPTR1->pGranuleHandle.cfgSTR->hOpenID == 
				sGranuleHandle->cfgSTR->hOpenID) 
              {   
                   sameIoHandle = TK_TRUE;
              }
              else
              {   
                   tempPTR2 = tempPTR1;
  	           tempPTR1 = tempPTR1->next;
              }
       }
	/************************************************************************/
	/* If the file is found on fileList, delete that file from the list.    */
	/************************************************************************/
       if (sameIoHandle == TK_TRUE)
       {   
	   if (tempPTR2 == tempPTR1)
	   {	
	       *fileList = tempPTR1->next;
	       free(tempPTR1);
	   }
	   else
	   {
               tempPTR2->next = tempPTR1->next;
               free(tempPTR1);
	   }
       }
       else
       {  
	   /* Invalid sGranuleHandle in deleteFile */
           TKreportWarning(W_TK_BADIOHANDLE);	
           return(TK_FAIL);
       }
  }
  else
  {
       /* File list is empty in deleteFile. */
       TKreportWarning(W_TK_EMPTYLIST);	
       return(TK_FAIL);
  }
 
  return(TK_SUCCESS);
}





/*****************************************************************************
*
*  FUNCTION STATEMENT:
*	getConfigFilenameAndScanline(int32 productID, char *fileName)
*
*  DESCRIPTION:
*	This function finds the configuration file name using productID
*	and returns the number of scanlines.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*	productID		The product identifier.
*
*  OUTPUT PARAMETERS:           
*       Parameter Name          Description
*       --------------          -----------
*	fileName		Configuration file name.
*
*  CALLED BY:                   TKopen().
*
*  RETURN VALUES:               Number of scanlines.
*
*  ROUTINES CALLED:
*       Routine Name            Description
*       ------------            -----------
*       TKreportError           This routine reports fatal errors to the error log
*                               file and terminates all processes.
*
******************************************************************************/

int32 getConfigFilenameAndScanline(int32 productID, char *fileName)
{
  pid_t pid;
  char processno[50];

  switch(productID)
  {
  case TK_L1B_01:
    strcpy(fileName, "L1B01VIRS");
    return(SD_UNLIMITED);
    
  case TK_L1B_11:
    strcpy(fileName, "L1B11TMI");
    return(SD_UNLIMITED);
    
  case TK_L1B_21:
    strcpy(fileName, "L1B21PR");
    return(SD_UNLIMITED);
    
  case TK_L1C_21:
    strcpy(fileName, "L1B21PR");
    return(SD_UNLIMITED);
    
  case TK_L2A_12:
    strcpy(fileName, "L2A12TMI");
    return(SD_UNLIMITED);
    
  case TK_L2A_21:
    strcpy(fileName, "L2A21PR");
    return(SD_UNLIMITED);
    
  case TK_L2A_23:
    strcpy(fileName, "L2A23PR");
    return(SD_UNLIMITED);
    
  case TK_L2A_25:
    strcpy(fileName, "L2A25PR");
    return(SD_UNLIMITED);
    
  case TK_L2N_25:
    strcpy(fileName, "L2N25PR");
    return(SD_UNLIMITED);
    
  case TK_L2A_53S:
    strcpy(fileName, "L2A53GVS");
    return(SD_UNLIMITED);
    
  case TK_L2A_53TX:
    strcpy(fileName, "L2A53TXGV");
    return(SD_UNLIMITED);
    
  case TK_L2A_53FL:
    strcpy(fileName, "L2A53FLGV");
    return(SD_UNLIMITED);
    
  case TK_L2A_54S:
    strcpy(fileName, "L2A54GVS");
    return(SD_UNLIMITED);
    
  case TK_L2A_54TX:
    strcpy(fileName, "L2A54TXGV");
    return(SD_UNLIMITED);
    
  case TK_L2A_54FL:
    strcpy(fileName, "L2A54FLGV");
    return(SD_UNLIMITED);
    
  case TK_L2A_55S:
    strcpy(fileName, "L2A55GVS");
    return(SD_UNLIMITED);
    
  case TK_L2A_55TX:
    strcpy(fileName, "L2A55TXGV");
    return(SD_UNLIMITED);
    
  case TK_L2A_55FL:
    strcpy(fileName, "L2A55FLGV");
    return(SD_UNLIMITED);
    
  case TK_L2B_31:
    strcpy(fileName, "L2B31PR");
    return(SD_UNLIMITED);
    
  case TK_L3A_11:
    strcpy(fileName, "L3A11TMI");
    return(1);
    
  case TK_L3A_12:
    strcpy(fileName, "L3A12TMI");
    return(1);

  case TK_L3A_25:
    strcpy(fileName, "L3A25PR");
    return(1);
    
  case TK_L3A_26:
    strcpy(fileName, "L3A26PR");
    return(1);
    
  case TK_L3A_27:
    strcpy(fileName, "L3A27PR");
    return(1);
    
  case TK_L3A_53S:
    strcpy(fileName, "L3A53GVS");
    return(SD_UNLIMITED);
    
  case TK_L3A_53TX:
    strcpy(fileName, "L3A53TXGV");
    return(SD_UNLIMITED);
    
  case TK_L3A_53FL:
    strcpy(fileName, "L3A53FLGV");
    return(SD_UNLIMITED);
    
  case TK_L3A_54S:
    strcpy(fileName, "L3A54GVS");
    return(SD_UNLIMITED);
    
  case TK_L3A_54TX:
    strcpy(fileName, "L3A54TXGV");
    return(SD_UNLIMITED);
    
  case TK_L3A_54FL:
    strcpy(fileName, "L3A54FLGV");
    return(SD_UNLIMITED);
    
  case TK_L3A_55:
    strcpy(fileName, "L3A55GV");
    return(SD_UNLIMITED);
    
  case TK_L3B_31:
    strcpy(fileName, "L3B31PR");
    return(1);
    
  case TK_L3B_42:
    strcpy(fileName, "L3B42VIRS");
    return(1);
    
  case TK_L3B_43:
    strcpy(fileName, "L3B43VIRS");
    return(1);
    
  case TK_L1B_GV:
    pid = getpid();
    sprintf(processno, "%d", pid);
    strcpy(fileName, "L1BGV.");
    strcat(fileName, processno);
    return(SD_UNLIMITED);
    
  case TK_L1C_GV:
    pid = getpid();
    sprintf(processno, "%d", pid);
    strcpy(fileName, "L1CGV.");
    strcat(fileName, processno);
    return(SD_UNLIMITED);
    
  case TK_L3A_46:
    strcpy(fileName, "L3A46GV");
    return(SD_UNLIMITED);
    
  case TK_L2A_56:
    return(SD_UNLIMITED);
    
  case TK_L2A_57:
    return(SD_UNLIMITED);
    
  case TK_L1B_11_R1:
    strcpy ( fileName, "L1B11TMI_R1" );
   return ( SD_UNLIMITED );
    
  case TK_L2A_12_R1:
    strcpy ( fileName, "L2A12TMI_R1" );
   return ( SD_UNLIMITED );
    
  case TK_L2A_23_R1:
    strcpy ( fileName, "L2A23PR_R1" );
   return ( SD_UNLIMITED );
    
  case TK_L2A_25_R1:
    strcpy ( fileName, "L2A25PR_R1" );
   return ( SD_UNLIMITED );
    
  case TK_L2A_25_R2:
    strcpy ( fileName, "L2A25PR_R2" );
   return ( SD_UNLIMITED );

  default :
    /* Undefined productID in getConfigFilenameAndScanline.*/
    TKreportError(E_TK_BADIDGCFS);
  }
}



/**********************************************************************************
*
*  FUNCTION STATEMENT:
*       checkAllFiles(IO_HANDLE *sGranuleHandle, IO_HANDLE *sHandle, 
*			FILE_PTR **fileList, FILE_PTR **ckList)
*
*  DESCRIPTION:
*       This routine used to check the input/output file handle status for
*	metadata header copy.
*
*  INPUT PARAMETERS:
*	Parameter Name		Description
*	--------------		-----------
*	sGranuleHandle		File handle structure which contains information
*				about the file.
*	fileList		Linked list that carries the information of the
*				files which have been opened.
*	ckList			Linked list that carries the information of the
*				files which have been accessed.
*
*  OUTPUT PARAMETERS:
*	Parameter Name		Description
*	--------------		-----------
*	sHandle			Output file handle 
*
*  CALLED BY:                   manageFiles()
*
*  RETURN STATUS:		TK_NOUPDATE for no update in metadata header.
*				TK_SUCCESS for successfully placed sHandle for 
*				output handle.
*
*  ROUTINES CALLED:		None
*
**********************************************************************************/
int checkAllFiles(IO_HANDLE *sGranuleHandle, IO_HANDLE **sHandle, 
			FILE_PTR **fileList, FILE_PTR **ckList)
{
  FILE_PTR *temp1 = *ckList;
  FILE_PTR *temp2 = temp1;
  FILE_PTR *tempPTR1 = temp1;
  FILE_PTR *tempPTR2 = tempPTR1;
  int	priority1 = TK_FALSE;
  int	priority2 = TK_FALSE;
  int	priority3 = TK_FALSE;
  int	priority4 = TK_FALSE;


  if (temp1 != NULL)
  {
    switch(sGranuleHandle->productID)
    {
	case TK_L2A_56:
	case TK_L2A_57:
	case TK_L3A_46:
	  return(TK_NOUPDATE);

	case TK_L1B_01:
	  /** Look for 1B01 file handle for CSIs **/
    	  while (temp1 != NULL)
	    {
	      if ((temp1->pGranuleHandle.productID == TK_L1B_01) &&
		  (temp1->pGranuleHandle.mode == TK_READ_ONLY))
		{
		  *sHandle = &(temp1->pGranuleHandle);
		  return(TK_SUCCESS);
		}
	      else
		{
		  temp2 = temp1;
		  temp1 = temp1->next;
		}
	    }
	  break;	

	case TK_L1B_11:
	  /** Look for 1B11 file handle for CSIs **/
    	  while (temp1 != NULL)
	    {
	      if ((temp1->pGranuleHandle.productID == TK_L1B_11) &&
		  (temp1->pGranuleHandle.mode == TK_READ_ONLY))
		{
		  *sHandle = &(temp1->pGranuleHandle);
		  return(TK_SUCCESS);
		}
	      else
		{
		  temp2 = temp1;
		  temp1 = temp1->next;
		}
	    }
	  break;	
	  
	case TK_L1B_21:
	  /** Look for 1B21 file handle for CSIs **/
    	  while (temp1 != NULL)
	    {
	      if ((temp1->pGranuleHandle.productID == TK_L1B_21) &&
		  (temp1->pGranuleHandle.mode == TK_READ_ONLY))
		{
		  *sHandle = &(temp1->pGranuleHandle);
		  return(TK_SUCCESS);
		}
	      else
		{
		  temp2 = temp1;
		  temp1 = temp1->next;
		}
	    }
	  break;	

	case TK_L1B_GV:
	  /** Look for the 1B51 (for CSIs) file handle  **/
	  while ( temp1 != NULL )
	    {
	      if ( ( temp1->pGranuleHandle.productID == TK_L1B_GV ) &&
		  ( temp1->pGranuleHandle.mode == TK_READ_ONLY ) )
		{
		  *sHandle = &(temp1->pGranuleHandle);
		  return ( TK_SUCCESS );
		}
	      else
		{
		  temp2 = temp1;
		  temp1 = temp1->next;
		}
	    }
	  break;

	case TK_L1C_GV:
	  /** Look for the 1B51 or 1C51 file handle  **/
	  while ( temp1 != NULL )
	    {
	      if ( ( ( temp1->pGranuleHandle.productID == TK_L1B_GV ) ||
		    ( temp1->pGranuleHandle.productID == TK_L1C_GV ) ) &&
		  ( temp1->pGranuleHandle.mode == TK_READ_ONLY ) )
		{
		  *sHandle = &(temp1->pGranuleHandle);
		  return ( TK_SUCCESS );
		}
	      else
		{
		  temp2 = temp1;
		  temp1 = temp1->next;
		}
	    }
	  break;

	case TK_L2A_53S:
	case TK_L2A_53FL:
	case TK_L2A_53TX:
	case TK_L2A_55S:
	case TK_L2A_55FL:
	case TK_L2A_55TX:
	  /** Look for the 2A54 file handle  **/
	  while ( temp1 != NULL )
	    {
	      if ( (( temp1->pGranuleHandle.productID == TK_L2A_55S ) ||
		    ( temp1->pGranuleHandle.productID == TK_L2A_55FL ) ||
		    ( temp1->pGranuleHandle.productID == TK_L2A_55TX ) || 
		    ( temp1->pGranuleHandle.productID == TK_L2A_53S ) ||
		    ( temp1->pGranuleHandle.productID == TK_L2A_53FL ) ||
		    ( temp1->pGranuleHandle.productID == TK_L2A_53TX ) ||
		    ( temp1->pGranuleHandle.productID == TK_L2A_54S ) ||
		    ( temp1->pGranuleHandle.productID == TK_L2A_54FL ) ||
		    ( temp1->pGranuleHandle.productID == TK_L2A_54TX ) ) &&
		  ( temp1->pGranuleHandle.mode == TK_READ_ONLY ) )
		{
		  *sHandle = &(temp1->pGranuleHandle);
		  return ( TK_SUCCESS );
		}
	      else
		{
		  temp2 = temp1;
		  temp1 = temp1->next;
		}
	    }
	  break;
	  
	case TK_L2A_54S:
	case TK_L2A_54FL:
	case TK_L2A_54TX:
	  /** Look for the 1C51 file handle  **/
	  while ( temp1 != NULL )
	    {
	      if ( ( ( temp1->pGranuleHandle.productID == TK_L2A_54S ) ||
		    ( temp1->pGranuleHandle.productID == TK_L2A_54FL ) ||
		    ( temp1->pGranuleHandle.productID == TK_L2A_54TX ) || 
		  ( temp1->pGranuleHandle.productID == TK_L1C_GV ) ) &&
		  ( temp1->pGranuleHandle.mode == TK_READ_ONLY ) )
		{
		  *sHandle = &(temp1->pGranuleHandle);
		  return ( TK_SUCCESS );
		}
	      else
		{
		  temp2 = temp1;
		  temp1 = temp1->next;
		}
	    }
	  break;

	case TK_L3A_53S:
	case TK_L3A_53FL:	
	case TK_L3A_53TX:
	case TK_L3A_54S:
	case TK_L3A_54FL:
	case TK_L3A_54TX:
	  /** Look for the 2A53 file handle  **/
	  while ( temp1 != NULL )
	    {
	      if ( ( ( temp1->pGranuleHandle.productID == TK_L2A_53S ) ||
		    ( temp1->pGranuleHandle.productID == TK_L2A_53FL ) ||
		    ( temp1->pGranuleHandle.productID == TK_L2A_53TX ) 
		    ) &&
		  ( temp1->pGranuleHandle.mode == TK_READ_ONLY ) )
		{
		  *sHandle = &(temp1->pGranuleHandle);
		  return ( TK_SUCCESS );
		}
	      else
		{
		  temp2 = temp1;
		  temp1 = temp1->next;
		}
	    }
	  break;


	case TK_L3A_55:
	  /** Look for the 2A55 file handle  **/
	  while ( temp1 != NULL )
	    {
	      if ( ( ( temp1->pGranuleHandle.productID == TK_L2A_55S ) ||
		    ( temp1->pGranuleHandle.productID == TK_L2A_55FL ) ||
		    ( temp1->pGranuleHandle.productID == TK_L2A_55TX ) ) &&
		  ( temp1->pGranuleHandle.mode == TK_READ_ONLY ) )
		{
		  *sHandle = &(temp1->pGranuleHandle);
		  return ( TK_SUCCESS );
		}
	      else
		{
		  temp2 = temp1;
		  temp1 = temp1->next;
		}
	    }
	  break;

	case TK_L1C_21:
	    /** Look for 1B21 or 1C21 (for CSIs) file handle **/
    	  while (temp1 != NULL)
    	  {
	    if ( ( ( temp1->pGranuleHandle.productID == TK_L1B_21) ||
		  ( temp1->pGranuleHandle.productID == TK_L1C_21) ) &&
		(temp1->pGranuleHandle.mode == TK_READ_ONLY))
	    {
		*sHandle = &(temp1->pGranuleHandle);
		return(TK_SUCCESS);
	    }
	    else
	    {
		temp2 = temp1;
		temp1 = temp1->next;
	    }
	  }
	  break;	
	
	case TK_L2A_12:
	    /** Look for 1B11 file handle **/
	  while(temp1 != NULL)
    	  {
	    if ( ( (temp1->pGranuleHandle.productID == TK_L1B_11) ||
		    ( temp1->pGranuleHandle.productID == TK_L2A_12) ) &&
		(temp1->pGranuleHandle.mode == TK_READ_ONLY))
	    {
		*sHandle = &(temp1->pGranuleHandle);
		return(TK_SUCCESS);
	    }
	    else
	    {
		temp2 = temp1;
		temp1 = temp1->next;
	    }
	  }
	  break;

	  case TK_L2A_21:
	      /** Look for 1B21 file handle **/
    	  while(temp1 != NULL)
	  {
	    if ( ( ( temp1->pGranuleHandle.productID == TK_L1B_21) ||
		  ( temp1->pGranuleHandle.productID == TK_L2A_21 ) )  &&
		(temp1->pGranuleHandle.mode == TK_READ_ONLY))
	    {
		*sHandle = &(temp1->pGranuleHandle);
		return(TK_SUCCESS);
	    }
	    else
	    {
		temp2 = temp1;
		temp1 = temp1->next;
	    }
	  }
	  break;

	case TK_L2A_23:
		/** Look for 1C21 file handle **/
          while (temp1 != NULL)
          {
            if ( ( (temp1->pGranuleHandle.productID == TK_L1C_21) ||
		  ( temp1->pGranuleHandle.productID == TK_L2A_23 ) ) &&
                (temp1->pGranuleHandle.mode == TK_READ_ONLY))
	    {
                *sHandle = &(temp1->pGranuleHandle);
                return(TK_SUCCESS);
	    }
            else
            {
                temp2 = temp1;
                temp1 = temp1->next;
            }
          }
          break;

	case TK_L2A_25:
		/** 
		 ** Look for 1C21, 2A21 and 2A23 file handle.
		 ** If lC21 file handle is found, set priority1 as TK_TRUE;
		 ** if 2A21 file handle is found, set priority2 as TK_TRUE;
		 ** if 2A23 file handle is found, set priority3 as TK_TRUE.
		 ** This function will based on the priority of the
		 ** productID to decide which handle to pass as
		 ** output.
		 **/
          while (temp1 != NULL)
	    {
            if ((temp1->pGranuleHandle.productID == TK_L1C_21) &&
		(temp1->pGranuleHandle.mode == TK_READ_ONLY)) 
	      priority1 = TK_TRUE;
	    
	    if ((temp1->pGranuleHandle.productID == TK_L2A_21) &&
		(temp1->pGranuleHandle.mode == TK_READ_ONLY))
	      priority2 = TK_TRUE;
            
	    if ((temp1->pGranuleHandle.productID == TK_L2A_23) &&
		(temp1->pGranuleHandle.mode == TK_READ_ONLY))
	      priority3 = TK_TRUE;
	    if ((temp1->pGranuleHandle.productID == TK_L2A_25) &&
		(temp1->pGranuleHandle.mode == TK_READ_ONLY))
	      priority4 = TK_TRUE;

	    temp2 = temp1;
	    temp1 = temp1->next;
	  }

	  if ((priority1 == TK_TRUE) || (priority2 == TK_TRUE) ||
	      (priority3 == TK_TRUE) || (priority4 == TK_TRUE ))
	  {
	    if (priority1 == TK_TRUE)
	    {
	      while (tempPTR1 != NULL)
	      {
	    	if ( ( tempPTR1->pGranuleHandle.productID == TK_L1C_21) &&
			(tempPTR1->pGranuleHandle.mode == TK_READ_ONLY))
	    	{
		  *sHandle = &(tempPTR1->pGranuleHandle);
		  return(TK_SUCCESS);
	    	}
		else
		{
		  tempPTR2 = tempPTR1;
		  tempPTR1 = tempPTR1->next;
		}
	      }
	    }
	    
	    if (priority2 == TK_TRUE)
	    { 
	      while (tempPTR1 != NULL)
	      {
		if ( (tempPTR1->pGranuleHandle.productID == TK_L2A_21) && 
			(tempPTR1->pGranuleHandle.mode == TK_READ_ONLY))
	    	{
		  *sHandle = &(tempPTR1->pGranuleHandle);
		  return(TK_SUCCESS);
	    	}
		else
		{
		  tempPTR2 = tempPTR1;
		  tempPTR1 = tempPTR1->next;
		}
	      }
	    }
	   
	    if (priority3 == TK_TRUE)
	    {
	      while (tempPTR1 != NULL)
	      {
		 if ( (tempPTR1->pGranuleHandle.productID == TK_L2A_23 ) &&
			(tempPTR1->pGranuleHandle.mode == TK_READ_ONLY))
		 {
		   *sHandle = &(tempPTR1->pGranuleHandle);
		   return(TK_SUCCESS);
	    	 }
	    	 else
	    	 {
		   tempPTR2 = tempPTR1;
		   tempPTR1 = tempPTR1->next;
	    	 }
	      }
	    }
	   
	    if (priority4 == TK_TRUE)
	    {
	      while (tempPTR1 != NULL)
	      {
		 if ( (tempPTR1->pGranuleHandle.productID == TK_L2A_25 ) &&
			(tempPTR1->pGranuleHandle.mode == TK_READ_ONLY))
		 {
		   *sHandle = &(tempPTR1->pGranuleHandle);
		   return(TK_SUCCESS);
	    	 }
	    	 else
	    	 {
		   tempPTR2 = tempPTR1;
		   tempPTR1 = tempPTR1->next;
	    	 }
	      }
	    }

	  } /* end if */
	  break;

	case TK_L2B_31:
		/** Look for 1C21 file handle **/
          while (temp1 != NULL)
          {
            if ( ( ( temp1->pGranuleHandle.productID == TK_L1C_21 ) ||
		  ( temp1->pGranuleHandle.productID == TK_L2B_31 ) ) &&
                (temp1->pGranuleHandle.mode == TK_READ_ONLY))
	    {
                *sHandle = &(temp1->pGranuleHandle);
                return(TK_SUCCESS);
	    }
            else
            {
                temp2 = temp1;
                temp1 = temp1->next;
            }
          }
          break;

	case TK_L3B_31:
		/** Look for 2B31 file handle **/
          while (temp1 != NULL)
          {
            if ((temp1->pGranuleHandle.productID == TK_L2B_31) &&
                (temp1->pGranuleHandle.mode == TK_READ_ONLY))
            {
                *sHandle = &(temp1->pGranuleHandle);
                return(TK_SUCCESS);
            }

            else
            {
                temp2 = temp1;
                temp1 = temp1->next;
            }
          }
          break;


	case TK_L3A_11:
		/** Look for 1B11 file handle **/
          while (temp1 != NULL)
          {
            if ((temp1->pGranuleHandle.productID == TK_L1B_11) &&
                (temp1->pGranuleHandle.mode == TK_READ_ONLY))
	    {
                *sHandle = &(temp1->pGranuleHandle);
                return(TK_SUCCESS);
	    }
            else
            {
                temp2 = temp1;
                temp1 = temp1->next;
            }
          }
          break;

	case TK_L3A_25:
		/** 
		 ** Look for 2A25, 1C21, 2A23 and 2A21 file handle.
                 ** If 2A25 file handle is found, set priority1 as TK_TRUE;
                 ** if 1C21 file handle is found, set priority2 as TK_TRUE;
                 ** if 2A23 file handle is found, set priority3 as TK_TRUE;
		 ** if 2A21 file handle is found, set priority4 as TK_TRUE.
                 ** This function will based on the priority of the
                 ** productID to decide which handle to pass as
                 ** output.
                 **/
          while (temp1 != NULL) 
          {
            if ((temp1->pGranuleHandle.productID == TK_L2A_25) &&
	                (temp1->pGranuleHandle.mode == TK_READ_ONLY))
                priority1 = TK_TRUE;

            if ((temp1->pGranuleHandle.productID == TK_L1C_21) &&
                        (temp1->pGranuleHandle.mode == TK_READ_ONLY))
                priority2 = TK_TRUE;

            if ((temp1->pGranuleHandle.productID == TK_L2A_23) &&
                        (temp1->pGranuleHandle.mode == TK_READ_ONLY))
                priority3 = TK_TRUE;

	    if ((temp1->pGranuleHandle.productID == TK_L2A_21) &&
                        (temp1->pGranuleHandle.mode == TK_READ_ONLY))
                priority4 = TK_TRUE;

            temp2 = temp1;
            temp1 = temp1->next;
          }

	  if ((priority1 == TK_TRUE) || (priority2 == TK_TRUE) ||
	      (priority3 == TK_TRUE) || (priority4 == TK_TRUE))
          {
	    if (priority1 == TK_TRUE)
	    {
	      while (tempPTR1 != NULL)
	      {
		 if ((tempPTR1->pGranuleHandle.productID == TK_L2A_25) &&
			(tempPTR1->pGranuleHandle.mode == TK_READ_ONLY))
            	 {
                   *sHandle = &(tempPTR1->pGranuleHandle);
                   return(TK_SUCCESS);
            	 }
		 else
		 {
		   tempPTR2 = tempPTR1;
		   tempPTR1 = tempPTR1->next;
		 }
	      }
	    }
           
	    if (priority2 == TK_TRUE)
	    {
	      while (tempPTR1 != NULL)
	      {
		 if ((tempPTR1->pGranuleHandle.productID == TK_L1C_21) &&
			(tempPTR1->pGranuleHandle.mode == TK_READ_ONLY))
            	 {
                   *sHandle = &(tempPTR1->pGranuleHandle);
                   return(TK_SUCCESS);
            	 }
		 else
		 {
		   tempPTR2 = tempPTR1;
		   tempPTR1 = tempPTR1->next;
		 }
	      }
	    }    	
        
            if (priority3 == TK_TRUE)
	    {
	      while (tempPTR1 != NULL)
	      {
		 if ((tempPTR1->pGranuleHandle.productID == TK_L2A_23) &&
			(tempPTR1->pGranuleHandle.mode == TK_READ_ONLY))
            	 {
                   *sHandle = &(tempPTR1->pGranuleHandle);
                   return(TK_SUCCESS);
            	 }
		 else
		 {
		   tempPTR2 = tempPTR1;
		   tempPTR1 = tempPTR1->next;
		 }
	      }
	    }	

            if (priority4 == TK_TRUE)
	    { 
	      while (tempPTR1 != NULL)
	      {
		 if ((tempPTR1->pGranuleHandle.productID == TK_L2A_21) &&
			(tempPTR1->pGranuleHandle.mode == TK_READ_ONLY))
            	 {
                   *sHandle = &(tempPTR1->pGranuleHandle);
                   return(TK_SUCCESS);
            	 }
            	 else
            	 {
                   tempPTR2 = tempPTR1;
                   tempPTR1 = tempPTR1->next;
            	 }
	      }
 	    }	
	    
          } /* end if */
          break;

	case TK_L3A_26:
		/** Look for 2A25 file handle **/
          while (temp1 != NULL)
          {
            if ((temp1->pGranuleHandle.productID == TK_L2A_25) &&
                (temp1->pGranuleHandle.mode == TK_READ_ONLY))
	    {
                *sHandle = &(temp1->pGranuleHandle);
                return(TK_SUCCESS);
	    }
            else
            {
                temp2 = temp1;
                temp1 = temp1->next;
            }
          }
          break;

	case TK_L3B_42:
		/** Look for 2B31 file handle **/
          while (temp1 != NULL)
          {
            if ((temp1->pGranuleHandle.productID == TK_L2B_31) &&
                (temp1->pGranuleHandle.mode == TK_READ_ONLY))
	    {
                *sHandle = &(temp1->pGranuleHandle);
                return(TK_SUCCESS);
	    }
            else
            {
                temp2 = temp1;
                temp1 = temp1->next;
            }
          }
          break;

	case TK_L3B_43:
		/** Look for 3B42 file handle **/
          while (temp1 != NULL) 
          {
            if ((temp1->pGranuleHandle.productID == TK_L3B_42) &&
                (temp1->pGranuleHandle.mode == TK_READ_ONLY))
	    {
                *sHandle = &(temp1->pGranuleHandle);
                return(TK_SUCCESS);
	    }

            else
            {
                temp2 = temp1;
                temp1 = temp1->next;
            }
          }
          break;

	case TK_L1B_11_R1:
	  /** This will find the input 1B11 handle for the 
	      reduced products **/
    	  while (temp1 != NULL)
	    {
	      if ((temp1->pGranuleHandle.productID == TK_L1B_11) &&
		  (temp1->pGranuleHandle.mode == TK_READ_ONLY))
		{
		  *sHandle = &(temp1->pGranuleHandle);
		  return(TK_SUCCESS);
		}
	      else
		{
		  temp2 = temp1;
		  temp1 = temp1->next;
		}
	    }
	  break;	

	case TK_L2A_12_R1:
	  /** This will find the input 2A12 handle for the 
	      reduced products **/
    	  while (temp1 != NULL)
	    {
/*subtle change december 2004 to fix 2a12 realtime metadata p.kolbe */
	      if (((temp1->pGranuleHandle.productID == TK_L1B_11)||(temp1->pGranuleHandle.productID == TK_L2A_12)) &&
		  (temp1->pGranuleHandle.mode == TK_READ_ONLY))
		{
		  *sHandle = &(temp1->pGranuleHandle);
		  return(TK_SUCCESS);
		}
	      else
		{
		  temp2 = temp1;
		  temp1 = temp1->next;
		}
	    }
	  break;	

	case TK_L2A_23_R1:
	  /** This will find the input 2A23 handle for the 
	      reduced products **/
    	  while (temp1 != NULL)
	    {
	      if ((temp1->pGranuleHandle.productID == TK_L2A_23) &&
		  (temp1->pGranuleHandle.mode == TK_READ_ONLY))
		{
		  *sHandle = &(temp1->pGranuleHandle);
		  return(TK_SUCCESS);
		}
	      else
		{
		  temp2 = temp1;
		  temp1 = temp1->next;
		}
	    }
	  break;	
	  

        case TK_L2N_25:
          return(TK_NOUPDATE);
          break;

        case TK_L3A_27:
          return(TK_NOUPDATE);
          break;

        case TK_L3A_12:
          return(TK_NOUPDATE);
          break;
/*
*/

        /*case TK_L2N_25:*/

	case TK_L2A_25_R1:
	case TK_L2A_25_R2:
	  /** This will find the input 2A25 handle for the 
	      reduced products **/
    	  while (temp1 != NULL)
	    {
	      if ((temp1->pGranuleHandle.productID == TK_L2A_25) &&
		  (temp1->pGranuleHandle.mode == TK_READ_ONLY))
		{
		  *sHandle = &(temp1->pGranuleHandle);
		  return(TK_SUCCESS);
		}
	      else
		{
		  temp2 = temp1;
		  temp1 = temp1->next;
		}
	    }
	  break;	

	default:
	  /* Invalid productID in checkAllFiles(). */
	  TKreportError(E_TK_FAILPDCKSTAT);

    } /* end switch */
  } /* end if */

  return(TK_NOUPDATE);
}


/**********************************************************************************
*
*  FUNCTION STATEMENT:
*       deleteAllLists(FILE_PTR **ckList)
*
*  DESCRIPTION:
*	The routine used to delete and free the memory of the whole linked list.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       ckList                  Linked list that carries the information of the
*                               files which have been accessed.
*
*  OUTPUT PARAMETERS:		None
*
*  CALLED BY:                   manageFiles()
*
*  RETURN STATUS:		None
*
*  ROUTINES CALLED:		manageFiles()
*
**********************************************************************************/

void deleteAllLists(FILE_PTR **ckList)
{
  FILE_PTR *tempPTR1 = *ckList;
  FILE_PTR *tempPTR2;

  while (tempPTR1 != NULL)
  {
	tempPTR2 = tempPTR1;
	tempPTR1 = tempPTR1->next;
	manageFiles(&tempPTR2->pGranuleHandle, 0, DELETE_LIST);
  }
}


/**********************************************************************************
*
*  FUNCTION STATEMENT:
*       deleteList(FILE_PTR **ckList)
*
*  DESCRIPTION:
*       The routine used to delete the individual list from the linked list.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       ckList                  Linked list that carries the information of the
*                               files which have been accessed.
*
*  OUTPUT PARAMETERS:           None
*
*  CALLED BY:                   manageFiles()
*
*  RETURN STATUS:               None
*
*  ROUTINES CALLED:             None
*
**********************************************************************************/

void deleteList(FILE_PTR **ckList)
{
  FILE_PTR *temp1 = *ckList;
  FILE_PTR *temp2 = temp1;

  if (temp1 != NULL)
  {
    if (temp2 == temp1)
    {
	*ckList = temp1->next;
	free(temp1);
    }
    else
    {
	temp2->next = temp1->next;
	free(temp1);
    }
  }

}
