%{
/* DEVELOPMENT HISTORY */
/* Revision 1.13  1999/09/13 18:12:42  pkolbe
 * finished backward compatability code changes
 *
 * Revision 1.11  1999/08/27 19:25:21  pkolbe
 * added tk backward compatability/version checking code changes
 *
 * Revision 1.10  1996/12/24 14:30:57  amak
 * Incorporated walkthrough comments.
 * - Changed '<' and '>' to double quotes around the #include statements.
 *
 * Revision 1.9  1996/04/19  15:40:49  amak
 * Made modifications to the preprocessor #include statement.  Replaced
 * TKerrorRou.h and TKwarnRou.h to TKerrHandle.h and TS_TK_15.h for new
 * error handling calls.
 *
 * Revision 1.8  1995/12/15  16:54:03  amak
 * Put the comment block inside the definition section.
 *
 * Revision 1.7  1995/12/15  15:18:23  thyun
 * Repositioned %{ to the top line of parse.l.
 *
 * Revision 1.6  1995/11/15  23:40:43  thyun
 * Modified in order to adapt the dynamic number of vgroups.
 *
 * Revision 1.5  1995/10/18  20:52:26  amak
 * Made modifications after some files need to be renamed.
 *
 * Revision 1.4  1995/10/17  19:59:42  amak
 * File modification.
 *
 * Revision 1.3  1995/08/23  18:30:14  amak
 * Took out the tabs in the preprocessor statements.
 *
 * Revision 1.2  1995/08/10  17:32:31  thyun
 * Corrected return tk_int16_ to tk_int8 at the list 90.
 *
 * Revision 1.1  1995/08/01  20:33:16  thyun
 * Added tokens to refelct changes in pr.y and changed the name 
 * of file to parse.y
 *
 * Revision 1.7  1995/07/19  13:33:15  thyun
 * Modified the error routines which caused syntax errors.
 *
 * Revision 1.6  1995/07/18  14:36:20  amak
 * Corrected typo errors in comment blocks.
 *
 * Revision 1.5  1995/07/10  14:44:18  gsain
 * error Routines
 *
 * Revision 1.4  1995/07/08  14:18:35  amak
 * Added log file header
 * */

/*****************************************************************************
*
*  FILE NAME:
*       parse.l
*
*  DESCRIPTION
*       This program is part of the parsing program written in LEX.  It reads
*       a configuration file, analyzes the contents of the file, and calls
*       pr.y to parse. This program creates lex.yy.c which can be integrated
*       with other C programs.
*       
*  ROUTINES CALLED:
*       Routine Name            Description
*       ------------            -----------
*       parse()			This routine parses the configuration file
*				and then stores the structure of the result of
*				the parsing in outCFGnode.
*
******************************************************************************/

#include "y.tab.h"
#include "template.h"
#include "TKerrHandle.h"
#include "TS_TK_15.h"
%}


stringConst	[a-zA-Z][a-zA-Z0-9_]*
stringVersion   [0-9]+\.[0-9]+(\.[0-9])*

%%

[\t ]+		{
		}

\n		{
		}

begin		{
		   return(begin_);
		}

end		{
		   return(end_);
		}

vdata		{ 
		   return(vdata_);
		}

sds		{ 
		   return(sds_); 
		}

vgroup		{
		   return(vgroup_);
	        }

tk_int16	{ 
		   yylval.dataType = DFNT_INT16;
		   return(tk_int16_);
		}

tk_float32	{ 
		   yylval.dataType = DFNT_FLOAT;
		   return(tk_float32_);
		}

tk_char8	{ 
		   yylval.dataType = DFNT_CHAR8;
		   return(tk_char8_);
		}

tk_int8		{ 
		   yylval.dataType = DFNT_INT8;
		   return(tk_int8_);
		}

tk_int32	{ 
		   yylval.dataType = DFNT_INT32;
		   return(tk_int32_);
		}

tk_float64	{ 
		   yylval.dataType = DFNT_FLOAT64;
		   return(tk_float64_);
		}

sdstype		{ 
		   yylval.dataType = SDSTYPE;
		   return(sdstype_);
		}

vdatatype	{ 
		   yylval.dataType = VDATATYPE;
		   return(vdatatype_);
		}

vgrouptype	{ 
		   yylval.dataType = VGROUPTYPE;
		   return(vgrouptype_);
		}

[0-9]+          { 
		   yylval.number = atoi(yytext);
                   return(integerConst); 
		}

{stringConst}   { 
		   yylval.lexem = (char *)malloc(strlen(yytext) + 1); 
                   if (yylval.lexem == NULL)  
                   {
		       TKreportError(E_TK_FAILMEMREQPRL);
                   } 
                   strcpy(yylval.lexem, yytext); 
                   return(stringConst); 
		}

{stringVersion}   { 
		   yylval.lexem = (char *)malloc(strlen(yytext) + 1); 
                   if (yylval.lexem == NULL)  
                   {
		       TKreportError(E_TK_FAILMEMREQPRL);
                   } 
                   strcpy(yylval.lexem, yytext); 
                   return(stringVersion); 
		}

%%
