%{
/* DEVELOPMENT HISTORY */
/* Revision 1.24  1999/10/02 21:37:11  pkolbe
 * removed debugger code
 *
 * Revision 1.23  1999/09/13 18:12:42  pkolbe
 * finished backward compatability code changes
 *
 * Revision 1.21  1999/08/27 19:25:21  pkolbe
 * added tk backward compatability/version checking code changes
 *
 * Revision 1.20  1997/11/19 14:39:54  mack
 * Changed TSDIS_TK_ENV to TSU_ENV
 *
 * Revision 1.19  1997/10/29 17:51:17  slywczak
 * In the TSDIS Environment, the config file for L1 GV is written to the current
 * directory (scratch disk) instead of TSDISTK.
 *
 * Revision 1.18  1997/01/02 15:32:05  amak
 * Added prologue
 *
 * Revision 1.17  1996/12/24 14:30:58  amak
 * Incorporated walkthrough comments.
 * - Changed '<' and '>' to double quotes around the #include statements.
 *
 * Revision 1.16  1996/12/10  15:26:37  amak
 * Called function rewind in parse() to avoid parsing an empty configuration
 * file for GV-1C51.
 *
 * Revision 1.15  1996/09/03  20:11:10  amak
 * Modified function parse().  It is now able to handle 2A56 and 2A57 products.
 *
 * Revision 1.14  1996/06/07  14:47:11  amak
 * Added stdlib.h as preprocessor statement.
 *
 * Revision 1.13  1996/04/19  15:40:49  amak
 * Made modifications to the preprocessor #include statement.  Replaced
 * TKerrorRou.h and TKwarnRou.h to TKerrHandle.h and TS_TK_15.h for new
 * error handling calls.
 *
 * Revision 1.12  1996/03/21  22:01:03  amak
 * Added one more parameter to function pushSDSnode().
 *
 * Revision 1.11  1996/01/25  15:08:01  amak
 * This is a test.
 *
 * Revision 1.10  1995/11/15  23:40:51  thyun
 * Modified in order to adapt the dynamic number of vgroups.
 *
 * Revision 1.9  1995/10/18  20:52:27  amak
 * Made modifications after some files need to be renamed.
 *
 * Revision 1.8  1995/10/17  19:59:43  amak
 * File modification.
 *
 * Revision 1.7  1995/09/14  20:19:12  thyun
 * Provide a dynamic way of searching configuration file in the function parse.
 *
 * Revision 1.6  1995/08/23  18:30:14  amak
 * Took out the tabs in the preprocessor statements.
 *
 * Revision 1.5  1995/08/22  19:55:05  amak
 * Replaced TKreportWarning() to TKreportError() in parse() because it is a
 * void function.
 *
 * Revision 1.4  1995/08/21  18:15:48  thyun
 * Changed the warning message W_TK_FAILOPPRS to error message E_TK_FAILOPPRS.
 *
 * Revision 1.3  1995/08/14  20:07:35  thyun
 * Modified the function yyerror to echo better message.
 *
 * Revision 1.2  1995/08/10  17:47:29  thyun
 * Added rules to hold the size of vdata field in the VDATA_FIELD_NODE data
 * structure.
 *
 * Revision 1.1  1995/08/01  20:33:58  thyun
 * Added rules to handle vgroups and changed the name of file to parse.y.
 *
 * Revision 1.9  1995/07/18  21:35:57  thyun
 * Modified the description prolog, and changed the parameter name of yyerror.
 *
 * Revision 1.7  1995/07/10  14:44:19  gsain
 * error Routines
 *
 * Revision 1.6  1995/07/10  13:33:28  gsain
 * added error Routine calls
 *
 * Revision 1.5  1995/07/08  14:22:32  amak
 * Added more detailed comments
 *
 * Revision 1.4  1995/07/08  14:19:21  amak
 * Added log file header
 * */

/*****************************************************************************
*
*  FILE NAME:
*       parse.y
*
*  DESCRIPTION
*       This program is a part of a parsing program written in YACC.  It contains
*       rules to parse a configuration file.  It creates y.tab.h and y.tab.c
*       which can be integrated with other C programs.
*       
*  ROUTINES CALLED:
*       Routine Name   Description
*       ------------    -----------
*       TKopen()	This routine is used to open a data product
*                       file prior to read or write the product data
*                       or metadata.
*
******************************************************************************/

#include <hdf.h>
#include <stdlib.h>
#include "IO.h"
#include "template.h"
#include "TKerrHandle.h"
#include "TS_TK_15.h"
	int		count;
	char		fieldName[MAX_SIZE];
	char		versionName[MAX_SIZE];
	int32		dim[MAX_RANK];
	int32		hdfType;
	int32		range;
	int32		dataValue;
	int32		fieldSize;
	VGROUP_LINK	*tempVgroup;
	VGROUP_LINK	*vGroupList;
	CFG_STR		*cfgNode;
        char            verNum[MAX_SIZE];
%}

%union  {
	char *lexem;
	int number;
	int dataType;
	};

%token <lexem> stringConst
%token <lexem> stringVersion
%token begin_
%token end_
%token vdata_
%token sds_
%token vgroup_
%token tk_char8_
%token tk_float32_
%token tk_float64_
%token tk_int8_ 
%token tk_int16_
%token tk_int32_
%token vdatatype_
%token sdstype_
%token vgrouptype_
%token <number> integerConst

%%

  VGROUPS   	 :	begin_ VGROUP VGROUP_CONTENTS end_
			{
			  vGroupList = vGroupList->prevVgroup; 
			}

	    	 |	VGROUPS begin_ VGROUP VGROUP_CONTENTS end_
			{
 			  vGroupList = vGroupList->prevVgroup; 
			}
	    	 ;



  VGROUP    	 :	VGROUPTYP STRING
	    	 ;



  VGROUPTYP 	 :	vgroup_
			{
		  	  hdfType = VGROUPTYPE;
	        	}
	    	 ;



  VGROUP_CONTENTS:	VGROUP_CONTENT

		 |	VGROUP_CONTENTS VGROUP_CONTENT
		 ;




  VGROUP_CONTENT :	VDATATYP STRINGS

		 |	SDSTYP ICONST STRING

		 |	begin_ VGROUP VGROUP_CONTENTS end_
			{
			  vGroupList = vGroupList->prevVgroup; 
			}
		 ;



  VDATATYP  	 :	vdata_
			{
			  hdfType = VDATATYPE;
			}
	    	 ;



  SDSTYP    	 :	sds_
			{
			  hdfType = SDSTYPE;
			  count = 0;
			}
	    	 ;



  STRINGS   	:	ICONST STRING

	    	|	STRINGS ICONST STRING
	    	;



  STRING    	:	SUBSTRING TYPE
			{ 
			  switch(hdfType)
		  	  { 
			    case VDATATYPE :
			      if (yylval.dataType == VDATATYPE)
			      {
			          pushVdataNode(&tempVgroup->vdata, fieldName); 
			      }
	 	       	      else
                                  if (versionCompare(versionName,verNum) >= 0)
			          {
                                      pushVdataFieldNode(&tempVgroup->vdata->rear->vdataFields,fieldName,dataValue,fieldSize); 
			          }
		       	      break;

		    	    case SDSTYPE :
                              if (versionCompare(versionName,verNum) >= 0)
                              {
			          pushSDSnode(&tempVgroup->sds,fieldName,dataValue,range,dim);
                              }
		       	      break;

		   	    case VGROUPTYPE :
			      createVgroup(&tempVgroup, fieldName);
			      pushVgroupNode(&vGroupList->subVgroup, tempVgroup, vGroupList);
			      vGroupList = vGroupList->subVgroup->vgroupRear;
			      break;

		    	    default :
		       	      TKreportWarning(W_TK_BADHDFTYP); 
		       	      break;
		  	  } 
	       		}
	    	;



  SUBSTRING 	:	stringConst
			{
			  strcpy(fieldName, yylval.lexem);
		 	  free(yylval.lexem);
			}
		|	SUBSTRING stringVersion
			{
			  strcpy(versionName, yylval.lexem);
		 	  free(yylval.lexem);
			}
	    	;


  ICONST    	:  	integerConst
			{
			  range = yylval.number;
			  fieldSize = yylval.number;
			}

	    	|  	ICONST integerConst
			{
			  dim[count] = yylval.number;
			  count++;
			}
	    	;



  TYPE		:	tk_int16_
			{
			  dataValue = DFNT_INT16;
			}
 
	    	|	tk_char8_
			{
			  dataValue = DFNT_CHAR;
			}

	    	|	tk_int8_
			{
			  dataValue = DFNT_INT8;
			}

	    	|   	tk_int32_
			{
			  dataValue = DFNT_INT32;
			}

	    	|	tk_float32_
			{
			  dataValue = DFNT_FLOAT32;
			}

	    	|	tk_float64_
			{
			  dataValue = DFNT_DOUBLE;
			}

	    	|	sdstype_
			{
			  dataValue = SDSTYPE;
			}

	    	|	vdatatype_
			{
			  dataValue = VDATATYPE;
			}

		|	vgrouptype_
			{
			  dataValue = VGROUPTYPE;
			}
	    	;
	
%% 


/******************************************************************************
*
*  FUNCTION STATEMENT:
*       yyerror(char *msg)
*
*  DESCRIPTION:
*       Whenever there is an error in parsing, this routine is automatically 
*	generated.  It prints out the error message.
*		
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       msg			The message of error.
*
*  OUTPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       N/A
*
*  CALLED BY:  N/A 
*
*  ROUTINES CALLED:     None
*
******************************************************************************/

yyerror(char * msg)
{
  TKreportError(E_TK_CONFIGERROR);
}


/******************************************************************************
*
*  FUNCTION STATEMENT:
*       yywrap(char * s)
*
*  DESCRIPTION:
*       When the parsing is done by reaching the end of its input file, the
* 	internal LEX & YACC routine (yylex) calls yywrap(), which returns a 
*	value of 0 or 1.  If the value is 1, the program is done and there 
*	is no more input.  If the value is 0, on the other hand, the lexer 
*	assumes that yywrap() has opened another file for it to read, and 
*	continutes to read.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       s                       N/A 
*
*  OUTPUT PARAMETERS:		N/A
*
*  CALLED BY:  N/A
*
*  ROUTINES CALLED:     None
*
******************************************************************************/

yywrap(char * s)
{
  return(1);
}



/*****************************************************************************
*
*  FUNCTION STATEMENT:
*       parse(char *fileName, CFG_NODE **outCFGnode)
*
*  DESCRIPTION:
*       This routine parses the configuration file and then stores the 
*	structure of the result of the parsing in outCFGnode.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       fieldName		The configuration file name.
*
*  OUTPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       outCFGnode		This variable will contain the stsructure of
*				the result of the parsing. 
*
*  CALLED BY:   TKopen()exit
*
*  ROUTINES CALLED:     	None
*
******************************************************************************/

extern FILE *yyin;
void parse(char *fileName, int dataType, CFG_STR **outCFGnode, char* version) 
{ FILE *file;
  char *path;
  char gvpath[3];
  char fullpath[TK_FNAME_LEN];
  int  gvL1Flag;

  strcpy(verNum,version);

  gvL1Flag = ((dataType == TK_L1B_GV) || (dataType == TK_L1C_GV)) ? 1 : 0;

/*
printf(" hey in the initial parse call int datatType: %d  \n", dataType); 
printf(" hey in the initial parse call int version: %s \n", version); 
*/

#ifndef TSU_ENV
  if ( gvL1Flag )
      strcpy ( fullpath, "./" );
  else
    {
      path = getenv( "TSDISTK");
      if (path == NULL) 
	{
	  TKreportError(E_TK_NULLPATH); /* getenv returned a null path */
	}
      strcpy(fullpath, path);
      strcat(fullpath, "/config/");
    }
#else
  path = getenv( "TSDISTK");
  if (path == NULL) 
    {
      TKreportError(E_TK_NULLPATH); /* getenv returned a null path */
    }
  strcpy(fullpath, path);
  strcat(fullpath, "/config/");
#endif

  cfgNode = (CFG_STR *)malloc(sizeof(CFG_STR));

  if (cfgNode == NULL)
  {  
	/* Memory request failed in parse. */
	TKreportError(E_TK_FAILMEMREQPARSE);
  }

  memset(cfgNode, '\0', sizeof(CFG_STR));
  initializeVgroupLink(&cfgNode->vGroup);


  createVgroup(&cfgNode->vGroup, "DATA_GRANULE");
  vGroupList = cfgNode->vGroup;

  switch (dataType)
  { 
    case TK_L2A_56:
    case TK_L2A_57:
	*outCFGnode = cfgNode;
	break;
    default:
    	strcat(fullpath, fileName);

    	file = fopen(fullpath, "r\0");
    	if (file == NULL)
    	{
          TKreportError(E_TK_FAILOPPRS);
    	}

	if (fgetc(file) != EOF)	/* Read first char from config file. */
	{
	  rewind(file);		/* File is not empty.  Rewind before parsing. */ 
    	  yyin = file;
    	  while (!feof(yyin))
	    yyparse();
	}

    	*outCFGnode = cfgNode;
    	fclose(file);
  }
}

