/* DEVELOPMENT HISTORY */
/* Revision 1.30  1999/09/13 18:12:42  pkolbe
 * finished backward compatability code changes
 *
 * Revision 1.28  1999/08/27 19:25:49  pkolbe
 * added tk backward compatability/version checking code changes
 *
 * Revision 1.27  1996/12/24 14:30:57  amak
 * Incorporated walkthrough comments.
 * - Changed '<' and '>' to double quotes around the #include statements.
 *
 * Revision 1.26  1996/09/03  20:14:45  amak
 * Took out redundant parameter numOfScanlines in pushSDSnode() function.
 *
 * Revision 1.25  1996/04/19  15:40:49  amak
 * Made modifications to the preprocessor #include statement.  Replaced
 * TKerrorRou.h and TKwarnRou.h to TKerrHandle.h and TS_TK_15.h for new
 * error handling calls.
 *
 * Revision 1.24  1996/03/21  22:01:46  amak
 * Added prologue.
 *
 * Revision 1.23  1995/12/12  20:42:14  thyun
 * Modified createVgroup to fix memory leak.
 *
 * Revision 1.22  1995/12/05  18:39:22  thyun
 * Inserted comments for createVgroup.
 *
 * Revision 1.21  1995/12/01  14:34:22  thyun
 * Inserted prolog for the new procedure createVgroup.
 *
 * Revision 1.20  1995/11/22  20:14:54  thyun
 * No changes.
 *
 * Revision 1.19  1995/11/15  23:41:33  thyun
 * Modified whole program in order to adapt changes in parsing.
 *
 * Revision 1.18  1995/10/18  20:52:27  amak
 * Made modifications after some files need to be renamed.
 *
 * Revision 1.17  1995/10/10  15:31:17  thyun
 * Inserted a memory checking condition in pushVgroupNode.
 *
 * Revision 1.16  1995/08/29  15:00:52  amak
 * Added more detailed descriptions in comment headers.
 *
 * Revision 1.15  1995/08/16  19:31:26  amak
 * Added new comments.
 *
 * Revision 1.14  1995/08/15  21:15:11  thyun
 * Added comments for the TKreportWarning and TKreportError routines.
 *
 * Revision 1.13  1995/08/15  20:03:28  thyun
 * Inserted header comments for every routines and Changed TKreportWaring to
 * TKreportError.
 *
 * Revision 1.12  1995/08/10  17:57:29  thyun
 * Added an assignment for fieldSize in pushVdataFieldNode.
 *
 * Revision 1.11  1995/08/01  20:38:26  thyun
 * Modified whole program to support vgroup structures.
 *
 * Revision 1.10  1995/07/19  13:42:09  thyun
 * Corrected spelling and grammar errors in comments.
 *
 * Revision 1.9  1995/07/10  14:10:29  gsain
 * error Routines
 *
 * Revision 1.8  1995/07/07  20:35:29  amak
 * Added detailed comments
 *
 * Revision 1.7  1995/07/07  18:48:57  thyun
 * Modified occurrances of 0 to NULL and -1 to TK_FAIL
 *
 * Revision 1.6  1995/07/07  14:14:47  thyun
 * Modified and added comments
 *
 * Revision 1.5  1995/06/30  18:48:20  thyun
 * Deleted annotation
 *
 * Revision 1.4  1995/06/13  14:29:02  amak
 * Adding more comments
 *
 * Revision 1.3  1995/06/08  18:44:39  amak
 * Took out some comment statements
 *
 * Revision 1.2  1995/06/08  18:36:03  amak
 * Added log file.
 * */


/*****************************************************************************
*
*  FILE NAME:
*       template.c
*
*  DESCRIPTION
*       This program provides the data structure, which stores the parsing
*       results, and the routines that manipulate the data structure.
*
*  ROUTINES CALLED:
*       Routine Name             Description
*       ------------             -----------
*	initializeCFG_NODE 	 This routine is used to initialize the
*				 CFG_STR pointer to NULL.
*	initializeVdataFieldLink This routine is used to initialize the
*				 VDATA_FIELD_LINK pointer to NULL.
*	initializeSDSnode	 This routine is used to initialize the
*				 SDS_LINK to NULL.
*	initializeVdataLink	 This routine is used to initialize the
*				 VDATA_LINK to NULL.
*	initializeVgroupLink	 This routine is used to initialize the
*				 VGROUP_LINK pointer to NULL.
*	initializeVgroupNode	 This routine is used to initialize the
*				 VGROUP_NODE structure to NULL.
*	initializeVgroupID	 This routine is used to initialize the
*				 VGROUP_ID structure to NULL.
*       pushVdataFieldNode	 This routine creates node and initializes
*				 its name, dataType, and so on with input.
*				 Then it is pushed onto the top of the stack.
*       pushSDSnode		 This routine creates a SDS_LINK node
*				 and initializes its name, dataType,
*				 range, dim[], and so on with input.  Then 
*				 it is pushed onto the top of the stack.
*	pushVdataNode		 This routine creates a VDATA_LINK node and
*				 initializes its name with input. Then it is
*				 pushed onto the top of the stack.
*	pushVgroupNode		 This routine creates a VGROUP_LINK node and
*				 initializes its name with input.  Then
*				 it pushes onto the top of the stack.
*	deallocateVdata		 This routine releases the momory allocated 
*				 for each element of stack where the element
*				 is a VDATA_LINK node.
*	deallocateSDS		 This routine releases the momory allocated
*				 for each element of stack where the element
*				 is a SDS_LINK node.
*	deallocateVgroup	 This routine releases the memory allocated
*				 for each element of stack where the element
*				 is a VGROUP_LINK node.
*	deallocateCFG_STR	 This routine release the momory allocated
*				 for a structure CFG_STR.
******************************************************************************/

#include <string.h>
#include "template.h"
#include "IO.h"
#include "TKerrHandle.h"
#include "TS_TK_15.h"


/*****************************************************************************
*
*  FUNCTION STATEMENT:
*       initializeCFG_NODE(CFG_STR **cfgSTR)
*
*  DESCRIPTION:
*       This routine is used to initialize the CFG_STR pointer to NULL.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       cfgSTR			The CFG_STR pointer.
*
*  OUTPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       cfgSTR			The null CFG_STR pointer.
*
*  CALLED BY:   		N/A
*
*  RETURN VALUES:               None
*
*  ROUTINES CALLED:     	None
*
******************************************************************************/
void initializeCFG_NODE(CFG_STR **cfgSTR)
{
  *cfgSTR = NULL;
}



/*****************************************************************************
*
*  FUNCTION STATEMENT:
*       initializeVdataFieldLink(VDATA_FIELD_LINK **node)
*
*  DESCRIPTION:
*       This routine is used to initialize the VDATA_FIELD_LINK pointer to NULL.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       node			The VDATA_FIELD_LINK pointer.
*
*  OUTPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       node			The null VDATA_FIELD_LINK pointer.
*
*  CALLED BY:   		pushVdataNode()
*
*  RETURN VALUES:               None
*
*  ROUTINES CALLED:     	None
*
******************************************************************************/

void initializeVdataFieldLink(VDATA_FIELD_LINK **node)
{
  *node = NULL;
}


/*****************************************************************************
*
*  FUNCTION STATEMENT:
*       initializeSDSnode(SDS_LINK **node)
*
*  DESCRIPTION:
*       This routine is used to initialize the SDS_LINK pointer to NULL.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       node                    The SDS_LINK pointer.
*
*  OUTPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       node                    The null SDS_LINK pointer.
*
*  CALLED BY:   		pushVgroupNode()
*
*  RETURN VALUES:               None
*
*  ROUTINES CALLED:     	None
*
******************************************************************************/

void initializeSDSnode(SDS_LINK **node)
{
  *node = NULL;
}



/*****************************************************************************
*
*  FUNCTION STATEMENT:
*       initializeVdataLink(VDATA_LINK **node)
*
*  DESCRIPTION:
*       This routine is used to initialize the VDATA_LINK pointer to NULL.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       node                    The VDATA_LINK pointer.
*
*  OUTPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       node                    The null VDATA_LINK pointer.
*
*  CALLED BY:   		pushVgroupNode()
*
*  RETURN VALUES:               None
*
*  ROUTINES CALLED:     	None
*
******************************************************************************/

void initializeVdataLink(VDATA_LINK **node)
{
  *node = NULL;
}


/*****************************************************************************
*
*  FUNCTION STATEMENT:
*       initializeVgroupLink(VGROUP_LINK **vGroup)
*
*  DESCRIPTION:
*       This routine is used to initialize the VGROUP_LINK pointer to NULL.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       vGroup			The VGROUP_LINK pointer.
*
*  OUTPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       vGroup                  The null VGROUP_LINK pointer.
*
*  CALLED BY:   		parse()
*
*  RETURN VALUES:               None
*
*  ROUTINES CALLED:     	None
*
******************************************************************************/

void initializeVgroupLink(VGROUP_LINK **vGroup)
{
  *vGroup = NULL;
}

/*****************************************************************************
*
*  FUNCTION STATEMENT:
*       initializeVgroupNode(struct VGROUP_NODE **node)
*
*  DESCRIPTION:
*       This routine is used to initialize the VGROUP_NODE structure to NULL.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       node			The VGROUP_NODE structure.
*
*  OUTPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       node                    The null VGROUP_NODE structure.
*
*  CALLED BY:   		None
*
*  RETURN VALUES:               None
*
*  ROUTINES CALLED:     	None
*
******************************************************************************/

void initializeVgroupNode(struct VGROUP_NODE **node)
{
  *node = NULL;
}

/*****************************************************************************
*
*  FUNCTION STATEMENT:
*       initializeVgroupID(VGROUP_ID **node)
*
*  DESCRIPTION:
*       This routine is used to initialize the VGROUP_ID pointer to NULL.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       node                    The VGROUP_ID pointer.
*
*  OUTPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       node                    The null VGROUP_ID pointer.
*
*  CALLED BY:   		None
*
*  RETURN VALUES:               None
*
*  ROUTINES CALLED:     	None
*
******************************************************************************/

void initializeVgroupID(VGROUP_ID **node)
{
  *node = NULL;
}



/*****************************************************************************
*
*  FUNCTION STATEMENT:
*       pushVdataFieldNode(VDATA_FIELD_LINK **stack, char *name, int32 dataType, 
*		  	   int32 fieldSize)
*
*  DESCRIPTION:
*       This routine creates node initializes its name, type, fieldSize, and
*       next pointer with input.  Then it is inserted at the end of the stack.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       stack                   A list of vdata fields.
*       name                    The name of a vdata field.
*       dataType                The type of a vdata field.
*	fieldSize		The size of field.
*
*  OUTPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       stack                   A list of vdata fields with a new vdata field
*                               node inserted on the top of the stack.
*
*  CALLED BY:   		parse()
*
*  RETURN VALUES:               None
*
*  ROUTINES CALLED:     	
*       Routine Name            Description
*       ------------            -----------
*       TKreportError           This routine reports fatal errors to the error log
*                               file and terminates all processes.
*
******************************************************************************/

void pushVdataFieldNode(VDATA_FIELD_LINK **stack, char *name, int32 dataType, int32 fieldSize)
{
  VDATA_FIELD_LINK *tempStack = *stack;
  VDATA_FIELD_LINK *tempNode = (VDATA_FIELD_LINK *)malloc(sizeof(VDATA_FIELD_LINK));

  if (tempNode != NULL)
  {
      /*
      ** Initialize vdata field node.
      */
      memset(tempNode, '\0', sizeof(VDATA_FIELD_LINK)); 
      strcpy(tempNode->name, name);
      tempNode->type = dataType;
      tempNode->fieldSize = fieldSize;
      initializeVdataFieldLink(&tempNode->next);
      initializeVdataFieldLink(&tempNode->rear);

      /*
      ** Insert the vdata field node at the end of stack.
      */
      if (tempStack == NULL)
      {
	  tempStack = tempNode;
      }
      else
      {
	  tempStack->rear->next = tempNode;
      }
      tempStack->rear = tempNode;
      *stack = tempStack;
  }
  else
  {   
      /* Memory allocation error in pushVdataFieldNode. */
      TKreportError(E_TK_MEMERRVF);
  }
}

/*****************************************************************************
*
*  FUNCTION STATEMENT:
*       pushSDSnode(SDS_LINK **stack, char *name, int32 dataType, int range,
*		    int32 dim[])
*
*  DESCRIPTION:
*       This routine creates a SDS_LINK node and initializes it with name, dataType,
*	range, dim[], and so on. And it is inserted at the end of the stack.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       stack                   A list of SDS.
*       name                    The name of SDS.
*       dataType                The type of SDS.
*       range                   The range of SDS.
*       dim                     The array containing the size of each dimension.
*
*  OUTPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       stack                   A list of SDS with a new SDS node inserted on
*                               the top of stack.
*
*  CALLED BY:   		parse()
*
*  RETURN VALUES:               None
*
*  ROUTINES CALLED:     	
*
*       Routine Name            Description
*       ------------            -----------
*       TKreportError           This routine reports fatal errors to the error log
*                               file and terminates all processes.

******************************************************************************/

void pushSDSnode(SDS_LINK **stack, char *name, int32 dataType, int range, 
			int32 dim[])
{ 
  int i;
  SDS_LINK *tempStack = *stack;
  SDS_LINK *tempSNode = (SDS_LINK *)malloc(sizeof(SDS_LINK));

  if (tempSNode != NULL)
  {   
      memset(tempSNode, '\0', sizeof(SDS_LINK)); 
      strcpy(tempSNode->name, name);
      tempSNode->type = dataType;
      tempSNode->rank = range + 1;
      for (i = 1; i < (range+1); ++i)
      {    
	   tempSNode->dim[i] = dim[i-1];
	   tempSNode->start[i] = 0;
	   tempSNode->edge[i] = dim[i-1];
      }
     /*
     ** edge, dim, and start are initialized here for HDF routine
     ** functions such as SDnametoindex, SDcreate, SDwritedata, and
     ** SDreaddata.
     */
      tempSNode->edge[0] = 1;  
      tempSNode->dim[0] = 1;
      tempSNode->start[0] = 0;
      initializeSDSnode(&tempSNode->next);
      initializeSDSnode(&tempSNode->rear);

      /* 
      ** Insert the sds node at the end of stack.
      */
      if (tempStack == NULL)
      {
	  tempStack = tempSNode;
      }
      else
      {
	  tempStack->rear->next = tempSNode;
      }
      tempStack->rear = tempSNode;
      *stack = tempStack;
  }
  else
  { 
      /* Memory allocation error in pushSDSnode. */
      TKreportError(E_TK_MEMERRSDS);
  }
}

/***************************************************************************** * *  FUNCTION STATEMENT: *       pushVdataNode(VDATA_LINK **stack, char *name)
*
*  DESCRIPTION:
*       This routine creates a VDATA_LINK node and initializes its fields with
*	input.  Then it is inserted at the end of the stack.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       stack                   A list of vdata.
*       name                    The name of vdata.
*
*  OUTPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       stack                   A list of vdata with a new vdata inserted on the top
*				of stack.
*
*  CALLED BY:   		parse()
*
*  RETURN VALUES:               None
*
*  ROUTINES CALLED:     	
*       Routine Name            Description
*       ------------            -----------
*       TKreportError           This routine reports fatal errors to the error log
*                               file and terminates all processes.
*
******************************************************************************/

void pushVdataNode(VDATA_LINK **stack, char *name)
{
  VDATA_LINK *tempStack = *stack;
  VDATA_LINK *tempNode = (VDATA_LINK *)malloc(sizeof(VDATA_LINK));

  if (tempNode != NULL)
  {   
      /*
      ** Initialize vdata node. 
      */
      memset(tempNode, '\0', sizeof(VDATA_LINK)); 
      initializeVdataFieldLink(&tempNode->vdataFields);
      strcpy(tempNode->name,name);
      initializeVdataLink(&tempNode->next);
      initializeVdataLink(&tempNode->rear);

      /*
      ** Insert the vdata node at the end of stack.
      */
      if (tempStack == NULL)
      {
          tempStack = tempNode;
      }
      else
      {
	  tempStack->rear->next = tempNode;
      }
      tempStack->rear = tempNode;
      *stack = tempStack;
  }
  else
  { 
      /* Memory allocation error in pushVdataNode. */
      TKreportError(E_TK_MEMERRVD);
  }
}



/*****************************************************************************
*
*  FUNCTION STATEMENT:
*	createVgroup(VGROUP_LINK **stack, char *name)
*
*  DESCRIPTION:
*       This routine creates a vgroup node and adds it into stack.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*	stack			List of vgroups.
*	name			The name of vgroup.
*
*  OUTPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*	stack			List of vgroups which contains a new vgroup 
*				node.
*
*  CALLED BY:                   parse.y
*
*  RETURN VALUES:               None.
*
*  ROUTINES CALLED:
*       Routine Name            Description
*       ------------            -----------
*       TKreportError           This routine reports fatal errors to the error log
*                               file and terminates all processes.
*
******************************************************************************/

void createVgroup(VGROUP_LINK **stack, char *name)
{

  VGROUP_LINK *tempNode = (VGROUP_LINK *)malloc(sizeof(VGROUP_LINK));
 
  if (tempNode != NULL)
  {
      /*
      ** Initialze Vgroup node.
      */
      memset(tempNode, '\0', sizeof(VGROUP_LINK));
      initializeVdataLink(&tempNode->vdata);
      initializeSDSnode(&tempNode->sds);
      initializeVgroupLink(&tempNode->subVgroup);
      initializeVgroupLink(&tempNode->vgroupRear);
      initializeVgroupLink(&tempNode->prevVgroup);
      initializeVgroupLink(&tempNode->next);
      tempNode->vGroupID = (VGROUP_ID *)malloc(sizeof(VGROUP_ID));
      if (tempNode->vGroupID != NULL)
      {
          strcpy(tempNode->vGroupID->name, name);
      }
      else
      {
          /* Memory allocation error in pushVgroupNode */
          TKreportError(E_TK_MEMERRVN);
      }
      *stack = tempNode;
  }
  else
  { 
      /* Memory allocation error in pushSubVgroupNode. */
      TKreportError(E_TK_MEMERRVN);
  }
}


/*****************************************************************************
*
*  FUNCTION STATEMENT:
*	pushVgroupNode(VGROUP_LINK **stack, VGROUP_LINK *vGroup, VGROUP_LINK *prevStack)
*
*  DESCRIPTION:
*       This routine inserts vGroup at the end of the stack.  Also it 
*	initializes its parent vgroup which is prevStack.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       stack                   A list of vgroups
*	vGroup			A vgroup node to be inserted into stack.
*	prevStack		A parent vgroup of vGroup.
*
*  OUTPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       stack                   A list of vgroups with a new vgroup inserted on the
*				end of stack.
*
*  CALLED BY:                   parse()
*
*  RETURN VALUES:               None
*
*  ROUTINES CALLED:             None
*
******************************************************************************/

void pushVgroupNode(VGROUP_LINK **stack, VGROUP_LINK *vGroup, VGROUP_LINK *prevStack)
{
  VGROUP_LINK *tempStack = *stack;

  if (tempStack == NULL)
  {
      tempStack = vGroup;
      tempStack->prevVgroup = prevStack;
  }
  else
  {
      tempStack->vgroupRear->next = vGroup;
      tempStack->vgroupRear->next->prevVgroup = prevStack;
  }

  tempStack->vgroupRear = vGroup;
  *stack = tempStack;
}





/*****************************************************************************
*
*  FUNCTION STATEMENT:
*       deallocateVdata(VDATA_LINK **stack)
*
*  DESCRIPTION:
*       This routine releases the memory allocated for each element of the 
*	stack where the element of a VDATA_LINK node.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       stack                   A list of vdata.
*
*  OUTPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       stack                   A null pointer after releasing the memory.
*
*  CALLED BY:   		deallocateVgroup()
*
*  RETURN VALUES:               None
*
*  ROUTINES CALLED:     None
*
******************************************************************************/

void deallocateVdata(VDATA_LINK **stack)
{
  VDATA_LINK *tempVdata1 = *stack;
  VDATA_LINK *tempVdata2;
  VDATA_FIELD_LINK *vdataField1;
  VDATA_FIELD_LINK *vdataField2;

  *stack = NULL;
  while(tempVdata1 != NULL)
  {
	tempVdata2 = tempVdata1;
	vdataField1 = tempVdata2->vdataFields;
	while(vdataField1 != NULL)
	{     
	      vdataField2 = vdataField1;
	      vdataField1 = vdataField1->next;
	      if (vdataField2 != NULL)
	          free(vdataField2);
	}
	tempVdata1 = tempVdata1->next;
	if (tempVdata2 != NULL)
	    free(tempVdata2);
  }
}



/*****************************************************************************
*
*  FUNCTION STATEMENT:
*       deallocateSDS(SDS_LINK **stack)
*
*  DESCRIPTION:
*       This routine releases the memory allocated for each element of the
*	stack where the elemnt is a SDS_LINK node.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       stack                   A list of sds.
*
*  OUTPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       stack                   A null pointer after releasing the memory.
*
*  CALLED BY:   		deallocateVgroup()
*
*  RETURN VALUES:               None
*
*  ROUTINES CALLED:     	None
*
******************************************************************************/

void deallocateSDS(SDS_LINK **stack)
{
  SDS_LINK *tempSDS1 = *stack;
  SDS_LINK *tempSDS2;

  *stack = NULL;
  while(tempSDS1 != NULL)
  {
        tempSDS2 = tempSDS1;
	tempSDS1 = tempSDS1->next;
	if (tempSDS2 != NULL)
            free(tempSDS2);
  }
}




/*****************************************************************************
*
*  FUNCTION STATEMENT:
*       deallocateVgroup(VGROUP_LINK **stack)
*
*  DESCRIPTION:
*       This routine recursively releases the memory allocated for each element
*	of the stack where the element is a VGROUP_LINK node.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       stack                   A list of vGroups.
*
*  OUTPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       stack                   A null pointer after releasing the memory.
*
*  CALLED BY:                   deallocateCFG_STR()
*
*  RETURN VALUES:               None
*
*  ROUTINES CALLED:             None
*
******************************************************************************/

void deallocateVgroup(VGROUP_LINK **stack)
{
  VGROUP_LINK *tempVgroup = *stack;

  if (tempVgroup != NULL)
  {
      *stack = NULL;
      deallocateVdata(&tempVgroup->vdata);
      deallocateSDS(&tempVgroup->sds);
      free(tempVgroup->vGroupID);
      deallocateVgroup(&tempVgroup->subVgroup);
      deallocateVgroup(&tempVgroup->next);
      free(tempVgroup);
  }
}



/*****************************************************************************
*
*  FUNCTION STATEMENT:
*       deallocateCFG_STR(CFG_STR **cfgSTR)
*
*  DESCRIPTION:
*       This routine releases the memory allocated for a structure CFG_STR.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       cfgSTR			The configuration structure.
*
*  OUTPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       cfgSTR                  A null pointer after releasing the memory.
*
*  CALLED BY:                   TKclose()
*
*  RETURN VALUES:               None
*
*  ROUTINES CALLED:             None
*
******************************************************************************/

void deallocateCFG_STR(CFG_STR **cfgSTR)
{
  CFG_STR *tempCFG_STR = *cfgSTR;
     
  *cfgSTR = NULL;
  
  deallocateVgroup(&tempCFG_STR->vGroup);

  if (tempCFG_STR != NULL)
      free(tempCFG_STR);
}




