/* DEVELOPMENT HISTORY */
/* Revision 1.48  1999/10/12 20:00:37  slywczak
 * Modified printComment, printVolumeDescriptor, printRadarDesc, printSweepInfo,
 * printCorrFactorDesc, printRayInfoInteger, printRayInfoFloat, printPlatformInfo,
 * printParmDesc, printCellRangeVector, printParmData1C_2byte, printParmData1B,
 * printParmData1C_1byte, and makeL1GVconfigFile.  Updated the routines to include
 * the toolkit version numbers in the elements.
 *
 * Revision 1.47  1999/08/13 16:59:40  pkolbe
 * added dynamic scaling code
 *
 * Revision 1.46  1998/07/17 13:19:38  slywczak
 * Removed setScalingFactors_3A54.  The product 3A54 is not supposed to have
 * scaling factors.  The routine that wrote the scaling factors was removed.
 *
 * Revision 1.45  1997/11/19 14:38:33  mack
 * Changed TSDIS_TK_ENV to TSU_ENV
 *
 * Revision 1.44  1997/10/29 17:37:06  slywczak
 * Modified setL1GVTemplate routine.  In the TSDIS environment, the toolkit
 * will write the config file to the current directory (scratch disk) instead
 * of the TSDISTK directory.
 *
 * Revision 1.43  1997/06/25 12:40:08  slywczak
 * Modified the read/write L1B/C GV functions.  Changed the cast from a
 * uint16 to a uint 8.
 *
 * Revision 1.42  1997/05/30 16:22:25  jt
 * *** empty log message ***
 *
 * Revision 1.41  1997/05/30 16:06:02  jt
 * Changed value to be a scaled value in readGauge.                   
 *
 * Revision 1.40  1997/05/30 14:33:12  slywczak
 * Modified routines setScalingFactors_2A55 and setScalingFactors_2A56.  Add the
 * check on the return code from the HDF routine.  The check is against
 * TK_HDF_FAIL.
 *
 * Revision 1.39  1997/05/19 03:45:32  slywczak
 * Modified the following routines:  readGrid_3A_46 and writeGrid_3A_46.
 * Removed references to the gridStruct structure, since this structure will be
 * referenced in the toolkit code.
 *
 * Revision 1.38  1997/05/16 20:28:20  slywczak
 * Modified function writeNetworkDesc.  The loop index sNetDesc was not begin
 * incremented in the for loop.
 *
 * Revision 1.37  1997/05/15 18:47:33  slywczak
 * Added the function setScalingFactors_2A56.  This function will add the scaling
 * factors to the SDSs.
 *
 * Revision 1.36  1997/05/15 18:26:49  slywczak
 * Added the following functions:  setScalingFactors_2A53, setScalingFactors_2A55,
 * setScalingFactors_3A53, setScalingFactors_3A54, and setScalingFactors_3A55.
 * These functions will add the scaling factors to the SDSs.
 *
 * Revision 1.35  1997/05/14 20:05:49  jt
 * Modified variable in 2A56/2A57 read routines.
 *
 * Revision 1.34  1997/04/28 18:59:30  jt
 * Added the routines:  readGauge(), readDisdrometer(), and readNetworkDesc().
 *
 * Revision 1.33  1997/03/05 20:49:21  slywczak
 * Modified function readGV_L1B_1C and writeGV_L1B_1C.  Changed the memcpy
 * in the read and write routines to increase efficiency.  The arrays are now
 * indexed instead of calculating the pointer value.
 *
 * Revision 1.32  1997/02/21 23:14:31  piper
 * Fixed code to properly handle missing values.
 *
 * Revision 1.32  1997/02/20 10:00:50  jt 
 * Fixed scaling calculations for reading missing or invalid data.
 *
 * Revision 1.31  1997/02/13 17:51:43  piper
 * Fixed problem with missing values not being written to output files.
 *
 * Revision 1.30  1997/02/04 16:38:33  amak
 * Modified writeGV_L1B_1C(), readGV_L1B_1C(), makeL1GVconfigFile(), and
 * openInputVgroupL1GV().  The new added is to adopt the fix of bug in
 * structure layout.
 * Splited function printParmData1C to printParmData1C_1byte and
 * printParmData1C_2byte.
 *
 * Revision 1.29  1997/01/02 15:30:35  amak
 * Define TK_DEFAULT for hard-coded value.
 *
 * Revision 1.28  1996/12/24 15:44:13  amak
 * Incorporated walkthrough comments.
 * - Changed '<' and '>' to double quotes around the #include statements.
 * - Checked return status of the malloc.
 *
 * Revision 1.27  1996/12/09  19:19:05  amak
 * Reordered the memcpy statement in writeGV_L1B_1C() for entity
 * sL1b1c->volDes.projectName.
 *
 * Revision 1.26  1996/12/03  22:23:50  amak
 * Fixed memory allocation errors in the previous revision.
 *
 * Revision 1.25  1996/12/03  19:06:17  pshah
 * Allocated memory for the structure sL1b1c->sensor.parm[i], to avoid core dumps
 * also allocated memory to structure parmData1byte & parmData2byte
 * this changes were done for the routine readGV_L1B_1C
 *
 * also made some changes as per the TRMM office
 *
 * Revision 1.23  1996/11/06  15:28:12  amak
 * Added detailed description to the routine.
 *
 * Revision 1.22  1996/11/01  20:45:05  amak
 * Replaced string "sensor.parm[i]." with "sensor.parm[i]->"
 * Modified memcpy statement to copy a ray of data in writeGV_L1B_1C().
 * Modified memcpy statement to copy a ray of data in readGV_L1B_1C().
 * Added function updateTemplate().
 * Added "case UPDATE_L1GV_TEMPLATE:" section to manageL1GVtemplate().
 * Added filemode parameter to function openInputVgroupL1GV().
 * Added "switch(filemode)" section to openInputVgroupL1GV()
 * Changed buffer management scheme in writeGV_L1B_1C() . Instead of
 * allocating a new buffer for each vdata(sds), now allocates a single
 * large buffer which is used for all vdata(sds) packing. Faster this
 * way on a PC.
 *   1. Added function bufManage().
 *   2. Modified writeGV_L1B_1C().
 *   3. Made similiarly modification to readGV_L1B_1C().
 *
 *
 * Revision 1.21  1996/10/24  13:48:30  pshah
 * Added code to take care of missing or invalid data for scaling
 *
 * Revision 1.20  1996/10/17  20:20:50  amak
 * Took out dailyRainfall entity from 3A46 structure.  Added ssmiData to
 * 3A46 structure.  Since then made modification to routines writeGVGrid_3A_46
 * and readGVGrid_3A_46.
 *
 * Revision 1.19  1996/08/28  22:30:51  amak
 * Added new functions writeNetworkDesc(), writeGauge(), and writeDisdrometer()
 * to file.  These routines are used to handle the L2A56 and L2A57 products.
 *
 * Revision 1.18  1996/08/26  19:01:38  amak
 * Fixed bug in routine openInputVgroupL1GV().  When Vattach returns the code,
 * the if statement should check whether that is equal to TK_HDF_FAIL (-1)
 * for failure, instead of TK_FAIL (1).
 *
 * Uncommented the frequency fields in L1B and L1C.  For HDF4.0r1, VSfdefine()
 * (hdf routine) is not allowed to declare more than certain number of vdata
 * fields (~29 fields).  Problems have been fixed for HDF4.0r2.
 *
 * Revision 1.17  1996/06/10  14:35:36  amak
 * Took out RadarGrid Structure from the heading of L2 and L3 GV products.
 *
 * Revision 1.16  1996/06/07  14:42:24  amak
 * Splited the multiple radar site routines for 2A-53, 2A-54, 2A-55, 3A-53,
 * 3A-54, and 3A-55 into 2 separate routines to handle Texas and Florida
 * multiple radar sites.
 * Took out 2A-52 write and read routines completely.  This product won't
 * store in hdf format.
 *
 * Revision 1.15  1996/04/19  15:40:45  amak
 * Made modifications to the preprocessor #include statement.  Replaced
 * TKerrorRou.h and TKwarnRou.h to TKerrHandle.h and TS_TK_15.h for new
 * error handling calls.
 *
 * Revision 1.14  1996/03/21  21:57:39  amak
 * Added routines readGVGrid_3A_46() and writeGVGrid_3A_46() to handle
 * L3A46 algorithm for reading and writing.
 *
 * Revision 1.13  1996/03/07  21:27:28  amak
 * For product 2A-53, 2A-54, 2A-55, 3A-53, and 3A-54, the dimensions had been
 * changed to 151x151, instead of 150x150.
 *
 * Revision 1.12  1996/01/25  15:06:22  amak
 * Called macro function RND() when doing the roundup from floating point to
 * integer type.
 *
 * Revision 1.11  1995/12/18  22:25:25  amak
 * Added 0.5 to all the write scaling calculations.  This adding is to avoid
 * the round up problems while computing on various platforms.
 *
 * Revision 1.10  1995/12/13  19:02:06  thyun
 * Modified openInputVgroupL1GV.
 *
 * Revision 1.9  1995/12/12  20:09:36  thyun
 * Modfied type errors rasied in Code Review meeting.
 *
 * Revision 1.8  1995/12/12  14:50:49  amak
 * Used macro factors to do scaling calculations.
 * Moved some of the functions which used to be located in L1_GV_ACCESS.c
 * to this file.
 *
 * Revision 1.7  1995/12/05  18:31:37  thyun
 * Inserted commets for writeGV_L1B_1C and readGV_L1B_1C.
 *
 * Revision 1.6  1995/12/01  14:33:08  thyun
 * Inserted prolog for writeGV_L1B_1C() and readGV_L1B_1C().
 *
 * Revision 1.5  1995/11/28  20:57:09  thyun
 * Implemented scaling for the procducts TK_L2A_53S, TK_L2A_53M, TK_L2A_55S,
 * TK_L2A_55M, TK_L3A_53S, TK_L3A_53M, TK_L3A_54S, TK_L3A_54M, and TK_L3A_55.
 *
 * Revision 1.4  1995/11/22  20:05:49  thyun
 * Made changes in order to reflect the change in L1B_1C_GV data structure.
 *
 * Revision 1.3  1995/11/15  23:36:06  thyun
 * Implemented writeGV_L1B_1C and readGV_L1B_1C.
 *
 * Revision 1.2  1995/10/18  20:52:21  amak
 * Made modifications after some files need to be renamed.
 *
 * Revision 1.1  1995/10/17  20:11:34  amak
 * File modification
 *
 * Revision 1.6  1995/10/10  15:07:52  thyun
 * Modified the program according to the changes made in definitions of GV data
 * structures in IO_GV.h.
 *
 * Revision 1.5  1995/08/25  18:39:57  thyun
 * Inserted header comment block.
 *
 * Revision 1.4  1995/08/21  18:23:43  thyun
 * Modified the names of fields, time, hour, minute, and second to tktmie,
 * tkhour, tkminute, and tksecond.
 *
 * Revision 1.3  1995/08/17  19:17:50  amak
 * Added IO_GV.h as header file.
 *
 * Revision 1.2  1995/08/10  17:53:17  thyun
 * Implemented the GV RADAR routines.
 *
 * Revision 1.1  1995/08/03  18:24:38  thyun
 * adding file.
 *
*/

/*******************************************************************************
*
*  FILE NAME:           IO_INTR_GV.h
*
*  DESCRIPTION:
*	This file contains routines that read and write GV RADAR information.
*
*  ROUTINES CALLED:
*       Routine Name            Description
*       ------------            -----------
*	readGV_L1B_1C		This function is used to read the L1B_GV or
*				L1C_GV information in HDF format.
*	writeGV_L1B_1C		This function is used to write the L1B_GV or
*				L1C_GV information in HDF format.
*       writeGVGrid_2A_53_single	
*				This function is used to write the single
*				radar site L2A-53 information in HDF format.
*	readGVGrid_2A_53_single
*				This function is used to read the single 
*				radar site L2A_53 information in HDF format.
*	writeGVGrid_2A_53_TX_mult
*				This function is used to write the multiple 
*				radar site for TX L2A_53 information in HDF format.
*	readGVGrid_2A_53_TX_mult
*				This function is used to read the multiple 
*				radar site for TX L2A_53 information in HDF format.
*	writeGVGrid_2A_53_FL_mult
*				This function is used to write the multiple 
*				radar site for FL L2A_53 information in HDF format.
*	readGVGrid_2A_53_FL_mult
*				This function is used to read the multiple 
*                               radar site for FL L2A_53 information in HDF format.
*       writeGVGrid_2A_54_single        
*                               This function is used to write the single
*                               radar site L2A-54 information in HDF format.
*       readGVGrid_2A_54_single
*                               This function is used to read the single 
*                               radar site L2A_54 information in HDF format.
*       writeGVGrid_2A_54_TX_mult
*                               This function is used to write the multiple 
*                               radar site for TX L2A_54 information in HDF format.
*       readGVGrid_2A_54_TX_mult
*                               This function is used to read the multiple 
*                               radar site for TX L2A_54 information in HDF format.
*       writeGVGrid_2A_54_FL_mult
*                               This function is used to write the multiple 
*                               radar site for FL L2A_54 information in HDF format.
*       readGVGrid_2A_54_FL_mult
*                               This function is used to read the multiple 
*                               radar site for FL L2A_54 information in HDF format.
*       writeGVGrid_2A_55_single        
*                               This function is used to write the single
*                               radar site L2A-55 information in HDF format.
*       readGVGrid_2A_55_single
*                               This function is used to read the single 
*                               radar site L2A_55 information in HDF format.
*       writeGVGrid_2A_55_TX_mult
*                               This function is used to write the multiple 
*                               radar site for TX L2A_55 information in HDF format.
*       readGVGrid_2A_55_TX_mult
*                               This function is used to read the multiple 
*                               radar site for TX L2A_55 information in HDF format.
*       writeGVGrid_2A_55_FL_mult
*                               This function is used to write the multiple 
*                               radar site for FL L2A_55 information in HDF format.
*       readGVGrid_2A_55_FL_mult
*                               This function is used to read the multiple 
*                               radar site for FL L2A_55 information in HDF format.
*       writeGVGrid_3A_53_single        
*                               This function is used to write the single
*                               radar site L3A-53 information in HDF format.
*       readGVGrid_3A_53_single
*                               This function is used to read the single 
*                               radar site L3A_53 information in HDF format.
*       writeGVGrid_3A_53_TX_mult
*                               This function is used to write the multiple 
*                               radar site for TX L3A_53 information in HDF format.
*       readGVGrid_3A_53_TX_mult
*                               This function is used to read the multiple 
*                               radar site for TX L3A_53 information in HDF format.
*       writeGVGrid_3A_53_FL_mult
*                               This function is used to write the multiple 
*                               radar site for FL L3A_53 information in HDF format.
*       readGVGrid_3A_53_FL_mult
*                               This function is used to read the multiple 
*                               radar site for FL L3A_53 information in HDF format.
*       writeGVGrid_3A_54_single        
*                               This function is used to write the single
*                               radar site L3A-54 information in HDF format.
*       readGVGrid_3A_54_single
*                               This function is used to read the single 
*                               radar site L3A_54 information in HDF format.
*       writeGVGrid_3A_54_TX_mult
*                               This function is used to write the multiple 
*                               radar site for TX L3A_54 information in HDF format.
*       readGVGrid_3A_54_TX_mult
*                               This function is used to read the multiple 
*                               radar site for TX L3A_54 information in HDF format.
*       writeGVGrid_3A_54_FL_mult
*                               This function is used to write the multiple 
*                               radar site for FL L3A_54 information in HDF format.
*       readGVGrid_3A_54_FL_mult
*                               This function is used to read the multiple 
*                               radar site for FL L3A_54 information in HDF format.
*       writeGVGrid_3A_55 	This function is used to write the
*                               L3A-55 information in HDF format.
*       readGVGrid_3A_55 	This function is used to read the
*                               radar site L3A_55 information in HDF format.
*       writeGVGrid_3A_46       This function is used to write the
*                               L3A-46 information in HDF format.
*       readGVGrid_3A_46        This function is used to read the
*                               radar site L3A_46 information in HDF format.
*	findTemplateNode	This routine finds the location of inputFile from 
*				the L1 GV template list and returns that postion 
*				to the caller.
*	setL1GVtemplate		This routines is used to create a template which 
*				is initialized with input data.  It calls 
*				manageL1GVtemplate in order to append it into 
*				the list of templates.
*	setInputFile		This routines searches a template for a matching 
*				filename and intializes the template's inputFile 
*				with the input inputFile.
*	getNvos			This routines returns the number of volume scans.  
*	getNparm		This routines returns the number of parameters 
*				for the given volume scan number. 
*	getNcell		This routines returns the number of cells for the 
*				given volume scan number and parameter number.
*	getNray			This routines returns the number of rays for the 
*				given volume scan number.
*	getNsweep		This routines returns the number of sweeps for 
*				the given volume scan number.
*	addTemplate		This routine is used to do insertion to the 
*				template.
*	deleteTemplate		This routine is used to delete the template.
*	manageL1GVtemplate	This routines performs a requested action.
*	findSubVgroups		This routine finds the sub-vgroups for the given 
*				vgroupID and returns the number of sub-vgroups.
*	openInputVgroupL1GV	When vGroup is null, this routine searches the 
*				vgroup ids and the sub-vgroup ids.
*	printComment		This routine is used to create a Level 1 GV 
*				configuration file.  It prints comment information.
*	printVolumeDescriptor	This routine is used to create a Level 1 GV 
*				configuration file.  It prints volume descriptor 
*				information.
*	printRadarDesc		This routine is used to create a Level 1 GV 
*				configuration file.  It prints Radar Descriptor 
*				information.
*	printCorrFactorDesc	This routine is used to create a Level 1 GV 
*				configuration file.  It prints Correction Factor 
*				Descriptor information.
*	printSweepInfo		This routine is used to create a Level 1 GV 
*				configuration file.  It prints Sweep Info 
*				information.
*	printRayInfoInteger	This routine is used to create a Level 1 GV 
*				configuration file.  It prints Ray Info_Integer 
*				information.
*	printRayInfoFloat	This routine is used to create a Level 1 GV 
*				configuration file.  It prints Ray Info_Float 
*				information.
*	printPlatformInfo	This routine is used to create a Level 1 GV 
*				configuration file.  It prints Platfor Info 
*				information.
*	printParmDesc		This routine is used to create a Level 1 GV 
*				configuration file.  It prints Parameter Descriptor
*			 	information.  
*	printCellRangeVector	This routine is used to create a Level 1 GV 
*				configuration file.  It prints Cell Range Vector 
*				information.  
*	printParmData1C		This routine is used to create a Level 1C GV 
*				configuration file.  It prints Parameter Data 
*				information.
*	printParmData1B		This routine is used to create a Level 1B GV 
*				configuration file.  It prints Parameter Data 
*				information.
*	makeL1GVconfigFile	This routine is used to create a Level 1 GV 
*				configuration file. 
*	getCurrVgroup		This routine returns the requested vgroup.
*	writeGauge		This function is used to write L2A56 Rain Gauge 
*				information in HDF format.
*       readGauge               This function is used to read L2A56 Rain Gauge
*                               information in HDF format.
*	writeDisdrometer	This function is used to write L2A57 Disdrometer 
*				information in HDF format.
*       readDisdrometer         This function is used to read L2A57 Disdrometer
*                               information in HDF format.
*	writeNetworkDesc	This function is used to write L2A56 or L2A57 
*				Network Descriptor information in HDF format.
*       readNetworkDesc         This function is used to read L2A56 or L2A57
*                               Network Descriptor information in HDF format.
*	updateTemplate		Appends VOS info from a single_VOS_template_node 
*				into a multi_VOS_template_node, then deletes the 
*				single_VOS_template_node.
*	bufManage		Perform buffer management tasks.
*
**********************************************************************************/

#include <stdio.h>
#include <string.h>
#include "IO_GV.h"
#include "IO_INTR_GV.h"
#include "hdf_routines.h"
#include "TKerrHandle.h"
#include "TS_TK_15.h"



/*******************************************************************************
*
*  FUNCTION STATEMENT:
*	writeGV_L1B_1C(IO_HANDLE *sGranuleHandle, L1B_1C_GV *sL1b1c, int productID)
*
*  DESCRIPTION:
*       This function is used to write the TK_L1B_GV or TK_L1C_GV information in
*	HDF format.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       sGranuleHandle          A structure containing information about the file.
*                               sGranuleHandle is returned by TKopen().
*       sL1b1c			A structure containing one volume scan data.
*                               This structure must be declared to correspond to 
*                               the data file being written.
*	productID		Product identifier.
*
*  OUTPUT PARAMETERS:           None
*
*  CALLED BY:                   TKwriteL1GV()
*
*  RETURN VALUES:		TK_SUCCESS if this routine is successful.
*
*  ROUTINES CALLED:     
*       Routine Name            Description
*       ------------            -----------
*	bufManage		Buffer management tasks.  (vdata buffer)
*       vdataWrite              This routine is used to write the vdata in HDF 
*                               format.
*       sdsWrite                This routine is used to write the scientific data
*                               set in HDF format.      
*	getCurrVgroup		This routine returns the requested postion of 
*				vgroup.  vgroupList contains list of vgroups and 
*				vgroupPos is used to take to the position of a 
*				vgroup in VgroupList.
*	TKgetNray		This routines returns the number of rays for a 
*				given volume scan number.
*	TKgetNsweep		This routines returns the number of sweeps for a 
*				given volume scan number.
*	TKgetNparm		This routines returns the number of parameter for 
*				a given volume scan number.
*	TKgetNcell		This routines returns the number of cells for a 
*				given volume scan number and a parameter number.
*
***********************************************************************************/

int writeGV_L1B_1C(IO_HANDLE *sGranuleHandle, L1B_1C_GV *sL1b1c, int productID)
{
  SDS_LINK 	*tempSDS;
  VDATA_LINK 	*tempVdata;
  VDATA_LINK 	*tempVdata1;
  VGROUP_LINK 	*parmVgroup;
  VGROUP_LINK   *currVgroup;
  int32 	nparm, ncell[MAX_PARM], nray, nsweep;
  uint8 	*databuf, *ptr;
  int32 	i, j, k;
  int           max_ncell = 0;  /* Max. number of cells per ray */
  int           bufsize;

  /*
  ** Get the vgroup position of for next volume scan number to write. 
  */
  currVgroup = (VGROUP_LINK *)getCurrVgroup(sGranuleHandle->cfgSTR->vGroup->subVgroup,
			     	 	     sGranuleHandle->currNumOfScanl);

  /*
  ** Get the number of ray, number of sweeps, number of parameters and number of cells
  ** in order to write sL1b1c data in to HDF file.
  */
  if (currVgroup != NULL)
  {
      nray = TKgetNray(sGranuleHandle, (sGranuleHandle->currNumOfScanl - 1));
      nsweep = TKgetNsweep(sGranuleHandle, (sGranuleHandle->currNumOfScanl - 1));
      nparm = TKgetNparm(sGranuleHandle, (sGranuleHandle->currNumOfScanl - 1));
      for (i = 0; i < nparm; ++i)
      {
        ncell[i] = TKgetNcell(sGranuleHandle, (sGranuleHandle->currNumOfScanl - 1), i);
	if (ncell[i] > max_ncell)
                max_ncell = ncell[i];
      }

      /*
      ** allocate memory for the (hopefully) largest required buffer
      ** we will need when packing the VOS parameters
      */
      bufsize = 2 * max_ncell *nray * nsweep;
      if (bufsize < MINIMAL_BUFFER_SIZE)
           bufsize = MINIMAL_BUFFER_SIZE;
      databuf = (uint8 *)calloc(1, bufsize);

      if (databuf == NULL)
	/* Memory allocation failed in writeGV_L1B_1C(). */
	TKreportError(E_TK_MEMFAILWRITEL1GV);

      /*
      **  Write comments into the HDF file.
      */
      tempSDS = currVgroup->sds;
      tempVdata = currVgroup->vdata;

      ptr = databuf ;
      memcpy(ptr, &sL1b1c->comments, sizeof(sL1b1c->comments));
      vdataWrite(&tempVdata, databuf);

      /*
      ** Write volume descriptor into the HDF file.
      */
      bufManage(&databuf, &bufsize, MINIMAL_BUFFER_SIZE);
      ptr = databuf;
      memcpy(ptr, &sL1b1c->volDes.verNum, sizeof(sL1b1c->volDes.verNum));
      ptr += sizeof(sL1b1c->volDes.verNum);
      memcpy(ptr, &sL1b1c->volDes.volNum, sizeof(sL1b1c->volDes.volNum));
      ptr += sizeof(sL1b1c->volDes.volNum);
      memcpy(ptr, &sL1b1c->volDes.sizeDataRec, sizeof(sL1b1c->volDes.sizeDataRec));
      ptr += sizeof(sL1b1c->volDes.sizeDataRec);
      memcpy(ptr, &sL1b1c->volDes.projectName, sizeof(sL1b1c->volDes.projectName));
      ptr += sizeof(sL1b1c->volDes.projectName);
      memcpy(ptr, &sL1b1c->volDes.year, sizeof(sL1b1c->volDes.year));
      ptr += sizeof(sL1b1c->volDes.year);
      memcpy(ptr, &sL1b1c->volDes.month, sizeof(sL1b1c->volDes.month));
      ptr += sizeof(sL1b1c->volDes.month);
      memcpy(ptr, &sL1b1c->volDes.day, sizeof(sL1b1c->volDes.day));
      ptr += sizeof(sL1b1c->volDes.day);
      memcpy(ptr, &sL1b1c->volDes.hour, sizeof(sL1b1c->volDes.hour));
      ptr += sizeof(sL1b1c->volDes.hour);
      memcpy(ptr, &sL1b1c->volDes.minute, sizeof(sL1b1c->volDes.minute));
      ptr += sizeof(sL1b1c->volDes.minute);
      memcpy(ptr, &sL1b1c->volDes.second, sizeof(sL1b1c->volDes.second));
      ptr += sizeof(sL1b1c->volDes.second);
      memcpy(ptr, &sL1b1c->volDes.flightNum, sizeof(sL1b1c->volDes.flightNum));
      ptr += sizeof(sL1b1c->volDes.flightNum);
      memcpy(ptr, &sL1b1c->volDes.facName, sizeof(sL1b1c->volDes.facName));
      ptr += sizeof(sL1b1c->volDes.facName);
      memcpy(ptr, &sL1b1c->volDes.genYear, sizeof(sL1b1c->volDes.genYear));
      ptr += sizeof(sL1b1c->volDes.genYear);
      memcpy(ptr, &sL1b1c->volDes.genMonth, sizeof(sL1b1c->volDes.genMonth));
      ptr += sizeof(sL1b1c->volDes.genMonth);
      memcpy(ptr, &sL1b1c->volDes.genDay, sizeof(sL1b1c->volDes.genDay));
      ptr += sizeof(sL1b1c->volDes.genDay);
      memcpy(ptr, &sL1b1c->volDes.numSensorDesc, sizeof(sL1b1c->volDes.numSensorDesc));
      vdataWrite(&tempVdata, databuf);

      /*
      **  Write Sensor information into the HDF file. 
      */
      tempSDS = currVgroup->subVgroup->sds;
      tempVdata = currVgroup->subVgroup->vdata;

      bufManage(&databuf, &bufsize, MINIMAL_BUFFER_SIZE);
      ptr = databuf;
      memcpy(ptr, &sL1b1c->sensor.radarDesc.radarName, 
	     sizeof(sL1b1c->sensor.radarDesc.radarName));
      ptr += sizeof(sL1b1c->sensor.radarDesc.radarName);
      memcpy(ptr, &sL1b1c->sensor.radarDesc.radarConstant, 
	     sizeof(sL1b1c->sensor.radarDesc.radarConstant));
      ptr += sizeof(sL1b1c->sensor.radarDesc.radarConstant);
      memcpy(ptr, &sL1b1c->sensor.radarDesc.nomPeakPower, 
             sizeof(sL1b1c->sensor.radarDesc.nomPeakPower));
      ptr += sizeof(sL1b1c->sensor.radarDesc.nomPeakPower);
      memcpy(ptr, &sL1b1c->sensor.radarDesc.nomNoisePower, 
             sizeof(sL1b1c->sensor.radarDesc.nomNoisePower));
      ptr += sizeof(sL1b1c->sensor.radarDesc.nomNoisePower);
      memcpy(ptr, &sL1b1c->sensor.radarDesc.receiverGain,
             sizeof(sL1b1c->sensor.radarDesc.receiverGain));
      ptr += sizeof(sL1b1c->sensor.radarDesc.receiverGain);
      memcpy(ptr, &sL1b1c->sensor.radarDesc.antennaGain,
	     sizeof(sL1b1c->sensor.radarDesc.antennaGain));
      ptr += sizeof(sL1b1c->sensor.radarDesc.antennaGain);
      memcpy(ptr, &sL1b1c->sensor.radarDesc.radarSystemGain, 
             sizeof(sL1b1c->sensor.radarDesc.radarSystemGain));
      ptr += sizeof(sL1b1c->sensor.radarDesc.radarSystemGain);
      memcpy(ptr, &sL1b1c->sensor.radarDesc.horBeamWidth, 
             sizeof(sL1b1c->sensor.radarDesc.horBeamWidth));
      ptr += sizeof(sL1b1c->sensor.radarDesc.horBeamWidth);
      memcpy(ptr, &sL1b1c->sensor.radarDesc.verBeamWidth, 
             sizeof(sL1b1c->sensor.radarDesc.verBeamWidth));
      ptr += sizeof(sL1b1c->sensor.radarDesc.verBeamWidth);
      memcpy(ptr, &sL1b1c->sensor.radarDesc.radarType, 
	     sizeof(sL1b1c->sensor.radarDesc.radarType));
      ptr += sizeof(sL1b1c->sensor.radarDesc.radarType);
      memcpy(ptr, &sL1b1c->sensor.radarDesc.scanMode, 
 	      sizeof(sL1b1c->sensor.radarDesc.scanMode));
      ptr += sizeof(sL1b1c->sensor.radarDesc.scanMode);
      memcpy(ptr, &sL1b1c->sensor.radarDesc.nomScanRate, 
             sizeof(sL1b1c->sensor.radarDesc.nomScanRate));
      ptr += sizeof(sL1b1c->sensor.radarDesc.nomScanRate);
      memcpy(ptr, &sL1b1c->sensor.radarDesc.nomStartAngle, 
      sizeof(sL1b1c->sensor.radarDesc.nomStartAngle));
      ptr += sizeof(sL1b1c->sensor.radarDesc.nomStartAngle);
      memcpy(ptr, &sL1b1c->sensor.radarDesc.nomStopAngle, 
             sizeof(sL1b1c->sensor.radarDesc.nomStopAngle));
      ptr += sizeof(sL1b1c->sensor.radarDesc.nomStopAngle);
      memcpy(ptr, &sL1b1c->sensor.radarDesc.numParmDesc, 
             sizeof(sL1b1c->sensor.radarDesc.numParmDesc));
      ptr += sizeof(sL1b1c->sensor.radarDesc.numParmDesc);
      memcpy(ptr, &sL1b1c->sensor.radarDesc.numDesc, 
             sizeof(sL1b1c->sensor.radarDesc.numDesc));
      ptr += sizeof(sL1b1c->sensor.radarDesc.numDesc);
      memcpy(ptr, &sL1b1c->sensor.radarDesc.dataComp, 
             sizeof(sL1b1c->sensor.radarDesc.dataComp));
      ptr += sizeof(sL1b1c->sensor.radarDesc.dataComp);
      memcpy(ptr, &sL1b1c->sensor.radarDesc.dataReductAlg, 
             sizeof(sL1b1c->sensor.radarDesc.dataReductAlg));
      ptr += sizeof(sL1b1c->sensor.radarDesc.dataReductAlg);
      memcpy(ptr, &sL1b1c->sensor.radarDesc.dataReductParm1, 
             sizeof(sL1b1c->sensor.radarDesc.dataReductParm1));
      ptr += sizeof(sL1b1c->sensor.radarDesc.dataReductParm1);
      memcpy(ptr, &sL1b1c->sensor.radarDesc.dataReductParm2, 
             sizeof(sL1b1c->sensor.radarDesc.dataReductParm2));
      ptr += sizeof(sL1b1c->sensor.radarDesc.dataReductParm2);
      memcpy(ptr, &sL1b1c->sensor.radarDesc.radarLon, 
             sizeof(sL1b1c->sensor.radarDesc.radarLon));
      ptr += sizeof(sL1b1c->sensor.radarDesc.radarLon);
      memcpy(ptr, &sL1b1c->sensor.radarDesc.radarLat, 
             sizeof(sL1b1c->sensor.radarDesc.radarLat));
      ptr += sizeof(sL1b1c->sensor.radarDesc.radarLat);
      memcpy(ptr, &sL1b1c->sensor.radarDesc.radarAlt, 
             sizeof(sL1b1c->sensor.radarDesc.radarAlt));
      ptr += sizeof(sL1b1c->sensor.radarDesc.radarAlt);
      memcpy(ptr, &sL1b1c->sensor.radarDesc.velocity, 
	     sizeof(sL1b1c->sensor.radarDesc.velocity));
      ptr += sizeof(sL1b1c->sensor.radarDesc.velocity);
      memcpy(ptr, &sL1b1c->sensor.radarDesc.range, 
             sizeof(sL1b1c->sensor.radarDesc.range));
      ptr += sizeof(sL1b1c->sensor.radarDesc.range);
      memcpy(ptr, &sL1b1c->sensor.radarDesc.numTransfreqency, 
             sizeof(sL1b1c->sensor.radarDesc.numTransfreqency));
      ptr += sizeof(sL1b1c->sensor.radarDesc.numTransfreqency);
      memcpy(ptr, &sL1b1c->sensor.radarDesc.numInterPulsePeriods, 
	     sizeof(sL1b1c->sensor.radarDesc.numInterPulsePeriods));
      ptr += sizeof(sL1b1c->sensor.radarDesc.numInterPulsePeriods);
      memcpy(ptr, &sL1b1c->sensor.radarDesc.frequency1,
            sizeof(sL1b1c->sensor.radarDesc.frequency1));
      ptr += sizeof(sL1b1c->sensor.radarDesc.frequency1);
      memcpy(ptr, &sL1b1c->sensor.radarDesc.frequency2, 
	     sizeof(sL1b1c->sensor.radarDesc.frequency2));
      ptr += sizeof(sL1b1c->sensor.radarDesc.frequency2);
      memcpy(ptr, &sL1b1c->sensor.radarDesc.frequency3, 
	     sizeof(sL1b1c->sensor.radarDesc.frequency3));
      ptr += sizeof(sL1b1c->sensor.radarDesc.frequency3);
      memcpy(ptr, &sL1b1c->sensor.radarDesc.frequency4, 
	     sizeof(sL1b1c->sensor.radarDesc.frequency4));
      ptr += sizeof(sL1b1c->sensor.radarDesc.frequency4);
      memcpy(ptr, &sL1b1c->sensor.radarDesc.frequency5, 
	     sizeof(sL1b1c->sensor.radarDesc.frequency5));
      ptr += sizeof(sL1b1c->sensor.radarDesc.frequency5);
      memcpy(ptr, &sL1b1c->sensor.radarDesc.interPulsePeriod1, 
	     sizeof(sL1b1c->sensor.radarDesc.interPulsePeriod1));
      ptr += sizeof(sL1b1c->sensor.radarDesc.interPulsePeriod1);
      memcpy(ptr, &sL1b1c->sensor.radarDesc.interPulsePeriod2, 
	     sizeof(sL1b1c->sensor.radarDesc.interPulsePeriod2));
      ptr += sizeof(sL1b1c->sensor.radarDesc.interPulsePeriod2);
      memcpy(ptr, &sL1b1c->sensor.radarDesc.interPulsePeriod3, 
	     sizeof(sL1b1c->sensor.radarDesc.interPulsePeriod3));
      ptr += sizeof(sL1b1c->sensor.radarDesc.interPulsePeriod3);
      memcpy(ptr, &sL1b1c->sensor.radarDesc.interPulsePeriod4, 
	     sizeof(sL1b1c->sensor.radarDesc.interPulsePeriod4));
      ptr += sizeof(sL1b1c->sensor.radarDesc.interPulsePeriod4);
      memcpy(ptr, &sL1b1c->sensor.radarDesc.interPulsePeriod5, 
             sizeof(sL1b1c->sensor.radarDesc.interPulsePeriod5)); 
      vdataWrite(&tempVdata, databuf);

      bufManage(&databuf, &bufsize, MINIMAL_BUFFER_SIZE);
      ptr = databuf;
      memcpy(ptr, &sL1b1c->sensor.corrFactorDesc.azimuth,
	     sizeof(sL1b1c->sensor.corrFactorDesc.azimuth));
      ptr += sizeof(sL1b1c->sensor.corrFactorDesc.azimuth);
      memcpy(ptr, &sL1b1c->sensor.corrFactorDesc.elevation,
	     sizeof(sL1b1c->sensor.corrFactorDesc.elevation));
      ptr += sizeof(sL1b1c->sensor.corrFactorDesc.elevation);
      memcpy(ptr, &sL1b1c->sensor.corrFactorDesc.rangeDelay,
	     sizeof(sL1b1c->sensor.corrFactorDesc.rangeDelay));
      ptr += sizeof(sL1b1c->sensor.corrFactorDesc.rangeDelay);
      memcpy(ptr, &sL1b1c->sensor.corrFactorDesc.radarLon,
	     sizeof(sL1b1c->sensor.corrFactorDesc.radarLon));
      ptr += sizeof(sL1b1c->sensor.corrFactorDesc.radarLon);
      memcpy(ptr, &sL1b1c->sensor.corrFactorDesc.radarLat,
	     sizeof(sL1b1c->sensor.corrFactorDesc.radarLat));
      ptr += sizeof(sL1b1c->sensor.corrFactorDesc.radarLat);
      memcpy(ptr, &sL1b1c->sensor.corrFactorDesc.radarPressAlt,
	     sizeof(sL1b1c->sensor.corrFactorDesc.radarPressAlt));
      ptr += sizeof(sL1b1c->sensor.corrFactorDesc.radarPressAlt);
      memcpy(ptr, &sL1b1c->sensor.corrFactorDesc.radarAltAboveGround,
	     sizeof(sL1b1c->sensor.corrFactorDesc.radarAltAboveGround));
      ptr += sizeof(sL1b1c->sensor.corrFactorDesc.radarAltAboveGround);
      memcpy(ptr, &sL1b1c->sensor.corrFactorDesc.radarPlatGroundSpeedEW,
	     sizeof(sL1b1c->sensor.corrFactorDesc.radarPlatGroundSpeedEW));
      ptr += sizeof(sL1b1c->sensor.corrFactorDesc.radarPlatGroundSpeedEW);
      memcpy(ptr, &sL1b1c->sensor.corrFactorDesc.radarPlatGroundSpeedNS,
	     sizeof(sL1b1c->sensor.corrFactorDesc.radarPlatGroundSpeedNS));
      ptr += sizeof(sL1b1c->sensor.corrFactorDesc.radarPlatGroundSpeedNS);
      memcpy(ptr, &sL1b1c->sensor.corrFactorDesc.radarPlatVerticalVelocity,
             sizeof(sL1b1c->sensor.corrFactorDesc.radarPlatVerticalVelocity));
      ptr += sizeof(sL1b1c->sensor.corrFactorDesc.radarPlatVerticalVelocity);
      memcpy(ptr, &sL1b1c->sensor.corrFactorDesc.radarPlatHeading,
             sizeof(sL1b1c->sensor.corrFactorDesc.radarPlatHeading));
      ptr += sizeof(sL1b1c->sensor.corrFactorDesc.radarPlatHeading);
      memcpy(ptr, &sL1b1c->sensor.corrFactorDesc.radarPlatRoll,
	     sizeof(sL1b1c->sensor.corrFactorDesc.radarPlatRoll));
      ptr += sizeof(sL1b1c->sensor.corrFactorDesc.radarPlatRoll);
      memcpy(ptr, &sL1b1c->sensor.corrFactorDesc.radarPlatPitch,
	     sizeof(sL1b1c->sensor.corrFactorDesc.radarPlatPitch));
      ptr += sizeof(sL1b1c->sensor.corrFactorDesc.radarPlatPitch);
      memcpy(ptr, &sL1b1c->sensor.corrFactorDesc.radarPlatDrift,
	     sizeof(sL1b1c->sensor.corrFactorDesc.radarPlatDrift));
      ptr += sizeof(sL1b1c->sensor.corrFactorDesc.radarPlatDrift);
      memcpy(ptr, &sL1b1c->sensor.corrFactorDesc.radarRotAngle,
	     sizeof(sL1b1c->sensor.corrFactorDesc.radarRotAngle));
      ptr += sizeof(sL1b1c->sensor.corrFactorDesc.radarRotAngle);
      memcpy(ptr, &sL1b1c->sensor.corrFactorDesc.radarTiltAngle,
	     sizeof(sL1b1c->sensor.corrFactorDesc.radarTiltAngle));
      vdataWrite(&tempVdata, databuf);
   
   
      bufManage(&databuf, &bufsize, MINIMAL_BUFFER_SIZE);
      tempVdata1 = tempVdata;
      for (i = 0; i < nsweep; ++i)
      {
           tempVdata = tempVdata1;
           ptr = databuf;
           memcpy(ptr, &sL1b1c->sensor.sweepInfo[i].radarName, 
                  sizeof(sL1b1c->sensor.sweepInfo[i].radarName));
           ptr += sizeof(sL1b1c->sensor.sweepInfo[i].radarName);
           memcpy(ptr, &sL1b1c->sensor.sweepInfo[i].sweepNum, 
                  sizeof(sL1b1c->sensor.sweepInfo[i].sweepNum));
           ptr += sizeof(sL1b1c->sensor.sweepInfo[i].sweepNum);
           memcpy(ptr, &sL1b1c->sensor.sweepInfo[i].numRays, 
                  sizeof(sL1b1c->sensor.sweepInfo[i].numRays));
           ptr += sizeof(sL1b1c->sensor.sweepInfo[i].numRays);
           memcpy(ptr, &sL1b1c->sensor.sweepInfo[i].trueStartAngle, 
	          sizeof(sL1b1c->sensor.sweepInfo[i].trueStartAngle));
           ptr += sizeof(sL1b1c->sensor.sweepInfo[i].trueStartAngle);
           memcpy(ptr, &sL1b1c->sensor.sweepInfo[i].trueStopAngle, 
	          sizeof(sL1b1c->sensor.sweepInfo[i].trueStopAngle));
           ptr += sizeof(sL1b1c->sensor.sweepInfo[i].trueStopAngle);
           memcpy(ptr, &sL1b1c->sensor.sweepInfo[i].fixedAngle, 
	          sizeof(sL1b1c->sensor.sweepInfo[i].fixedAngle));
           ptr += sizeof(sL1b1c->sensor.sweepInfo[i].fixedAngle);
           memcpy(ptr, &sL1b1c->sensor.sweepInfo[i].filterFlag, 
	          sizeof(sL1b1c->sensor.sweepInfo[i].filterFlag));
           vdataWrite(&tempVdata, databuf);
      }


      /* 
      ** Allocate memory to pack the ray info integer information.
      ** 4 represents the byte size of one ray info integer.
      */
      bufManage(&databuf, &bufsize, 4 * 7 * nray * nsweep);
      ptr = databuf;
      for (i = 0; i < nsweep; ++i)
      {
           memcpy(ptr, &sL1b1c->sensor.rayInfoInteger[i],(4 * 7 * nray));
           ptr += (4 * 7 * nray);
      }
      sdsWrite(&tempSDS, databuf);

      /*
      ** Allocate memory to pack the ray info float information.
      ** 4 represents the byte size of one ray info float.
      */
      bufManage(&databuf, &bufsize, 4 * 4 * nray * nsweep);
      ptr = databuf;
      for (i = 0; i < nsweep; ++i)
      {
           memcpy(ptr, &sL1b1c->sensor.rayInfoFloat[i], (4 * 4 * nray));
           ptr +=  (4 * 4 * nray);
      }
      sdsWrite(&tempSDS, databuf);


      /*
      ** Allocate memory to pack the platform info information.
      ** 4 represents the byte size of one platform info.
      */
      bufManage(&databuf, &bufsize, 4 * 18 * nray * nsweep);
      ptr = databuf;
      for (i = 0; i < nsweep; ++i)
      {
           memcpy(ptr, &sL1b1c->sensor.platformInfo[i], (4 * 18 * nray));
           ptr += (4 * 18 * nray);
      }
      sdsWrite(&tempSDS, databuf);


      /*
      ** Write parameter information into the HDF file.
      */
      i = 0;
      parmVgroup = currVgroup->subVgroup->subVgroup;
      while (parmVgroup != NULL)
      {
 	     tempSDS = parmVgroup->sds;
	     tempVdata = parmVgroup->vdata;

             bufManage(&databuf, &bufsize, MINIMAL_BUFFER_SIZE);
             ptr = databuf;
	     memcpy(ptr, &sL1b1c->sensor.parm[i]->parmDesc.parmName, 
		    sizeof(sL1b1c->sensor.parm[i]->parmDesc.parmName));
	     ptr += sizeof(sL1b1c->sensor.parm[i]->parmDesc.parmName);
	     memcpy(ptr, &sL1b1c->sensor.parm[i]->parmDesc.parmDesc, 
	 	    sizeof(sL1b1c->sensor.parm[i]->parmDesc.parmDesc));
	     ptr += sizeof(sL1b1c->sensor.parm[i]->parmDesc.parmDesc);
	     memcpy(ptr, &sL1b1c->sensor.parm[i]->parmDesc.parmUnits, 
		    sizeof(sL1b1c->sensor.parm[i]->parmDesc.parmUnits));
	     ptr += sizeof(sL1b1c->sensor.parm[i]->parmDesc.parmUnits);
	     memcpy(ptr, &sL1b1c->sensor.parm[i]->parmDesc.interPulsePeriod, 
		    sizeof(sL1b1c->sensor.parm[i]->parmDesc.interPulsePeriod));
	     ptr += sizeof(sL1b1c->sensor.parm[i]->parmDesc.interPulsePeriod);
	     memcpy(ptr, &sL1b1c->sensor.parm[i]->parmDesc.transFreq, 
		    sizeof(sL1b1c->sensor.parm[i]->parmDesc.transFreq));
	     ptr += sizeof(sL1b1c->sensor.parm[i]->parmDesc.transFreq);
	     memcpy(ptr, &sL1b1c->sensor.parm[i]->parmDesc.receiverBandwidth, 
		    sizeof(sL1b1c->sensor.parm[i]->parmDesc.receiverBandwidth));
	     ptr += sizeof(sL1b1c->sensor.parm[i]->parmDesc.receiverBandwidth);
	     memcpy(ptr, &sL1b1c->sensor.parm[i]->parmDesc.pulseWidth, 
		    sizeof(sL1b1c->sensor.parm[i]->parmDesc.pulseWidth));
	     ptr += sizeof(sL1b1c->sensor.parm[i]->parmDesc.pulseWidth);
	     memcpy(ptr, &sL1b1c->sensor.parm[i]->parmDesc.polarTransWave, 
		    sizeof(sL1b1c->sensor.parm[i]->parmDesc.polarTransWave));
	     ptr += sizeof(sL1b1c->sensor.parm[i]->parmDesc.polarTransWave);
	     memcpy(ptr, &sL1b1c->sensor.parm[i]->parmDesc.numOfsamples, 
		    sizeof(sL1b1c->sensor.parm[i]->parmDesc.numOfsamples));
	     ptr += sizeof(sL1b1c->sensor.parm[i]->parmDesc.numOfsamples);
	     memcpy(ptr, &sL1b1c->sensor.parm[i]->parmDesc.parmDataType, 
		    sizeof(sL1b1c->sensor.parm[i]->parmDesc.parmDataType));
	     ptr += sizeof(sL1b1c->sensor.parm[i]->parmDesc.parmDataType);
	     memcpy(ptr, &sL1b1c->sensor.parm[i]->parmDesc.thresholdField, 
		    sizeof(sL1b1c->sensor.parm[i]->parmDesc.thresholdField));
	     ptr += sizeof(sL1b1c->sensor.parm[i]->parmDesc.thresholdField);
	     memcpy(ptr, &sL1b1c->sensor.parm[i]->parmDesc.thresholdValue, 
		    sizeof(sL1b1c->sensor.parm[i]->parmDesc.thresholdValue));
	     ptr += sizeof(sL1b1c->sensor.parm[i]->parmDesc.thresholdValue);
	     memcpy(ptr, &sL1b1c->sensor.parm[i]->parmDesc.scaleFactor, 
		    sizeof(sL1b1c->sensor.parm[i]->parmDesc.scaleFactor));
	     ptr += sizeof(sL1b1c->sensor.parm[i]->parmDesc.scaleFactor);
	     memcpy(ptr, &sL1b1c->sensor.parm[i]->parmDesc.offsetFactor, 
		    sizeof(sL1b1c->sensor.parm[i]->parmDesc.offsetFactor));
	     ptr += sizeof(sL1b1c->sensor.parm[i]->parmDesc.offsetFactor);
	     memcpy(ptr, &sL1b1c->sensor.parm[i]->parmDesc.deletedOrMissDataFlag, 
		    sizeof(sL1b1c->sensor.parm[i]->parmDesc.deletedOrMissDataFlag));
             vdataWrite(&tempVdata, databuf);

             /*
             ** Pack the cell range vector information into databuf and
             ** write it into the HDF file.
             */
             bufManage(&databuf, &bufsize, MINIMAL_BUFFER_SIZE);
             ptr = databuf;
	     memcpy(ptr, &sL1b1c->sensor.parm[i]->cellRangeVector.numOfCells,
		    sizeof(sL1b1c->sensor.parm[i]->cellRangeVector.numOfCells));
	     ptr += sizeof(sL1b1c->sensor.parm[i]->cellRangeVector.numOfCells);
            
             for (j = 0; j < ncell[i]; ++j)
	     {    memcpy(ptr, &sL1b1c->sensor.parm[i]->cellRangeVector.distanceToCell[j], 
			 sizeof(sL1b1c->sensor.parm[i]->cellRangeVector.distanceToCell[j]));
	          ptr += sizeof(sL1b1c->sensor.parm[i]->cellRangeVector.distanceToCell[j]);
             }
             vdataWrite(&tempVdata, databuf);


             if (productID == TK_L1C_GV)
             {
		if ((i % 2) == 0)
		{ 
                  /*
               	  ** Memory is allocated to pack parameter data.  1 represents the byte 
               	  ** size of one parameter data.
               	  */

                  bufManage(&databuf, &bufsize, 1 * ncell[i] * nray * nsweep);
               	  ptr = databuf;
               	  for (j = 0; j < nsweep; ++j)
               	  {
                    for (k = 0; k < nray; ++k)
                    {
		      memcpy(ptr, (uint8 *)sL1b1c->sensor.parm[i]->parmData1byte[j][k],
			     (1 * ncell[i]));
                      ptr += (1 * ncell[i]);
                    }
               	  }
               	  sdsWrite(&tempSDS, databuf);
		}

		else
		{

                  /*
                  ** Memory is allocated to pack parameter data.  2 represents the byte 
                  ** size of one parameter data.
                  */ 
                  bufManage(&databuf, &bufsize, 2 * ncell[i] * nray * nsweep);
                  ptr = databuf;
                  for (j = 0; j < nsweep; ++j)
                  {
                    for (k = 0; k < nray; ++k)
                    {
		      memcpy(ptr, (uint16 *)sL1b1c->sensor.parm[i]->parmData2byte[j][k],
			     (2 * ncell[i]));
                      ptr += (2 * ncell[i]);
                    }
                  }
                  sdsWrite(&tempSDS, databuf);
		}
             }  


	     if (productID == TK_L1B_GV)
	     {
               /*
	       ** Memory is allocated to pack parameter data.  2 represents the byte size 
               ** of one parameter data.
               */
	       bufManage(&databuf, &bufsize, 2 * ncell[i] * nray * nsweep);
	       ptr = databuf;
	       for (j = 0; j < nsweep; ++j)
               {
                  for (k = 0; k < nray; ++k)
     		  {
		    memcpy(ptr, (uint16 *)sL1b1c->sensor.parm[i]->parmData2byte[j][k],
			   (2 * ncell[i]));
		    ptr += (2 * ncell[i]);
		  }
	       }
	       sdsWrite(&tempSDS, databuf);
	     }

	     parmVgroup = parmVgroup->next;
	     i++;
       }
       free(databuf);
  }
  return (TK_SUCCESS);
}



/***********************************************************************************
*
*  FUNCTION STATEMENT:
*	readGV_L1B_1C(IO_HANDLE *sGranuleHandle, L1B_1C_GV *sL1b1c, int productID)
*
*  DESCRIPTION:
*       This function is used to read the TK_L1B_GV or TK_L1C_GV information in
*       HDF format.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       sGranuleHandle          A structure containing information about the file.
*                               sGranuleHandle is returned by TKopen().
*	productID		Product identifier.
*
*  OUTPUT PARAMETERS:           
*       Parameter Name          Description
*       --------------          -----------
*       sL1b1c                  A structure containing one volume scan data.
*                               This structure must be declared to correspond to
*                               the data file being written.
*
*  CALLED BY:                   TKreadL1GV()
*
*  RETURN VALUES:               TK_SUCCESS if this routine is successful.
*
*  ROUTINES CALLED:
*       Routine Name            Description
*       ------------            -----------
*       bufManage               Buffer management tasks. (vdata buffer)
*       vdataWrite              This routine is used to write the vdata in HDF
*                               format.
*       sdsWrite                This routine is used to write the scientific data
*                               set in HDF format.
*       getCurrVgroup           This routine returns the requested postion of vgroup. 
*                               vgroupList contains list of vgroups and vgroupPos 
*                               is used to take to the position of a vgroup in VgroupList.
*       TKgetNray               This routines returns the number of rays for a given 
*                               volume scan number.
*       TKgetNsweep             This routines returns the number of sweeps for a given 
*                               volume scan number.
*       TKgetNparm              This routines returns the number of parameter for a given
*                               volume scan number.
*       TKgetNcell              This routines returns the number of cells for a given
*                               volume scan number and a parameter number.
*
***********************************************************************************/

int readGV_L1B_1C(IO_HANDLE *sGranuleHandle, L1B_1C_GV *sL1b1c, int productID)
{
  SDS_LINK 	*tempSDS;
  VDATA_LINK 	*tempVdata;
  VDATA_LINK 	*tempVdata1;
  VGROUP_LINK 	*parmVgroup;
  VGROUP_LINK 	*currVgroup;
  int32 	nparm, ncell[MAX_PARM], nray, nsweep;
  uint8 	*databuf, *ptr;
  int32 	i, j, k;
  int           max_ncell = 0;  /* Max. number of cells per ray */
  int           bufsize;
  int scaleFactor;

  /*
  ** Get the vgroup position of for next volume scan number to write.
  */
  currVgroup = (VGROUP_LINK *)getCurrVgroup(sGranuleHandle->cfgSTR->vGroup->subVgroup,
			     		    sGranuleHandle->currNumOfScanl);
  /*
  ** Get the number of ray, number of sweeps, number of parameters and number of cells
  ** in order to read sL1b1c data in to HDF file.
  */
  if (currVgroup != NULL)
  {
      nray = TKgetNray(sGranuleHandle, (sGranuleHandle->currNumOfScanl - 1));
      nsweep = TKgetNsweep(sGranuleHandle, (sGranuleHandle->currNumOfScanl - 1));
      nparm = TKgetNparm(sGranuleHandle, (sGranuleHandle->currNumOfScanl - 1));
      for (i = 0; i < nparm; ++i)
      {
        ncell[i] = TKgetNcell(sGranuleHandle, (sGranuleHandle->currNumOfScanl - 1), i);
	if (ncell[i] > max_ncell)
                max_ncell = ncell[i];
      }

      /*
      ** Allocate memory for the (hopefully) largest required buffer
      ** we will need when packing the VOS components
      */
      bufsize = 2 * max_ncell * nray * nsweep;
      if (bufsize < MINIMAL_BUFFER_SIZE)
          bufsize = MINIMAL_BUFFER_SIZE;
      databuf = (uint8 *)calloc(1, bufsize);

      if (databuf == NULL)
	/* Memory allocation failed in readGV_L1B_1C(). */
	TKreportError(E_TK_MEMFAILREADL1GV);


      /* 
      ** Read comments from the HDF file.
      */
      tempSDS = currVgroup->sds;
      tempVdata = currVgroup->vdata;

      ptr = databuf ;
      vdataRead(&tempVdata, ptr);
      memcpy(&sL1b1c->comments, ptr, sizeof (sL1b1c->comments));


      /*
      ** Read volume descriptor from the HDF file.
      */
      bufManage(&databuf, &bufsize, MINIMAL_BUFFER_SIZE);
      ptr = databuf;
      vdataRead(&tempVdata, ptr);
      memcpy(&sL1b1c->volDes.verNum, ptr, sizeof(sL1b1c->volDes.verNum));
      ptr += sizeof(sL1b1c->volDes.verNum);
      memcpy(&sL1b1c->volDes.volNum, ptr, sizeof(sL1b1c->volDes.volNum));
      ptr += sizeof(sL1b1c->volDes.volNum);
      memcpy(&sL1b1c->volDes.sizeDataRec, ptr, sizeof(sL1b1c->volDes.sizeDataRec));
      ptr += sizeof(sL1b1c->volDes.sizeDataRec);
      memcpy(&sL1b1c->volDes.projectName, ptr, sizeof(sL1b1c->volDes.projectName));
      ptr += sizeof(sL1b1c->volDes.projectName);
      memcpy(&sL1b1c->volDes.year, ptr, sizeof(sL1b1c->volDes.year));
      ptr += sizeof(sL1b1c->volDes.year);
      memcpy(&sL1b1c->volDes.month, ptr, sizeof(sL1b1c->volDes.month));
      ptr += sizeof(sL1b1c->volDes.month);
      memcpy(&sL1b1c->volDes.day, ptr, sizeof(sL1b1c->volDes.day));
      ptr += sizeof(sL1b1c->volDes.day);
      memcpy(&sL1b1c->volDes.hour, ptr, sizeof(sL1b1c->volDes.hour));
      ptr += sizeof(sL1b1c->volDes.hour);
      memcpy(&sL1b1c->volDes.minute, ptr, sizeof(sL1b1c->volDes.minute));
      ptr += sizeof(sL1b1c->volDes.minute);
      memcpy(&sL1b1c->volDes.second, ptr, sizeof(sL1b1c->volDes.second));
      ptr += sizeof(sL1b1c->volDes.second);
      memcpy(&sL1b1c->volDes.flightNum, ptr, sizeof(sL1b1c->volDes.flightNum));
      ptr += sizeof(sL1b1c->volDes.flightNum);
      memcpy(&sL1b1c->volDes.facName, ptr, sizeof(sL1b1c->volDes.facName));
      ptr += sizeof(sL1b1c->volDes.facName);
      memcpy(&sL1b1c->volDes.genYear, ptr, sizeof(sL1b1c->volDes.genYear));
      ptr += sizeof(sL1b1c->volDes.genYear);
      memcpy(&sL1b1c->volDes.genMonth, ptr, sizeof(sL1b1c->volDes.genMonth));
      ptr += sizeof(sL1b1c->volDes.genMonth);
      memcpy(&sL1b1c->volDes.genDay, ptr, sizeof(sL1b1c->volDes.genDay));
      ptr += sizeof(sL1b1c->volDes.genDay);
      memcpy(&sL1b1c->volDes.numSensorDesc,ptr,sizeof(sL1b1c->volDes.numSensorDesc));
      

      /*
      ** Read Sensor information from the HDF file.
      */
      tempSDS = currVgroup->subVgroup->sds;
      tempVdata = currVgroup->subVgroup->vdata;

      bufManage(&databuf, &bufsize, MINIMAL_BUFFER_SIZE);
      ptr = databuf;
      vdataRead(&tempVdata, ptr);
      memcpy(&sL1b1c->sensor.radarDesc.radarName, ptr,
             sizeof(sL1b1c->sensor.radarDesc.radarName));
      ptr += sizeof(sL1b1c->sensor.radarDesc.radarName);
      memcpy(&sL1b1c->sensor.radarDesc.radarConstant, ptr,
             sizeof(sL1b1c->sensor.radarDesc.radarConstant));
      ptr += sizeof(sL1b1c->sensor.radarDesc.radarConstant);
      memcpy(&sL1b1c->sensor.radarDesc.nomPeakPower, ptr,
             sizeof(sL1b1c->sensor.radarDesc.nomPeakPower));
      ptr += sizeof(sL1b1c->sensor.radarDesc.nomPeakPower);
      memcpy(&sL1b1c->sensor.radarDesc.nomNoisePower, ptr,
             sizeof(sL1b1c->sensor.radarDesc.nomNoisePower));
      ptr += sizeof(sL1b1c->sensor.radarDesc.nomNoisePower);
      memcpy(&sL1b1c->sensor.radarDesc.receiverGain, ptr,
             sizeof(sL1b1c->sensor.radarDesc.receiverGain));
      ptr += sizeof(sL1b1c->sensor.radarDesc.receiverGain);
      memcpy(&sL1b1c->sensor.radarDesc.antennaGain, ptr,
             sizeof(sL1b1c->sensor.radarDesc.antennaGain));
      ptr += sizeof(sL1b1c->sensor.radarDesc.antennaGain);
      memcpy(&sL1b1c->sensor.radarDesc.radarSystemGain, ptr,
             sizeof(sL1b1c->sensor.radarDesc.radarSystemGain));
      ptr += sizeof(sL1b1c->sensor.radarDesc.radarSystemGain);
      memcpy(&sL1b1c->sensor.radarDesc.horBeamWidth, ptr,
             sizeof(sL1b1c->sensor.radarDesc.horBeamWidth));
      ptr += sizeof(sL1b1c->sensor.radarDesc.horBeamWidth);
      memcpy(&sL1b1c->sensor.radarDesc.verBeamWidth, ptr,
             sizeof(sL1b1c->sensor.radarDesc.verBeamWidth));
      ptr += sizeof(sL1b1c->sensor.radarDesc.verBeamWidth);
      memcpy(&sL1b1c->sensor.radarDesc.radarType, ptr,
             sizeof(sL1b1c->sensor.radarDesc.radarType));
      ptr += sizeof(sL1b1c->sensor.radarDesc.radarType);
      memcpy(&sL1b1c->sensor.radarDesc.scanMode, ptr,
             sizeof(sL1b1c->sensor.radarDesc.scanMode));
      ptr += sizeof(sL1b1c->sensor.radarDesc.scanMode);
      memcpy(&sL1b1c->sensor.radarDesc.nomScanRate, ptr,
             sizeof(sL1b1c->sensor.radarDesc.nomScanRate));
      ptr += sizeof(sL1b1c->sensor.radarDesc.nomScanRate);
      memcpy(&sL1b1c->sensor.radarDesc.nomStartAngle, ptr,
             sizeof(sL1b1c->sensor.radarDesc.nomStartAngle));
      ptr += sizeof(sL1b1c->sensor.radarDesc.nomStartAngle);
      memcpy(&sL1b1c->sensor.radarDesc.nomStopAngle, ptr,
             sizeof(sL1b1c->sensor.radarDesc.nomStopAngle));
      ptr += sizeof(sL1b1c->sensor.radarDesc.nomStopAngle);
      memcpy(&sL1b1c->sensor.radarDesc.numParmDesc, ptr,
             sizeof(sL1b1c->sensor.radarDesc.numParmDesc));
      ptr += sizeof(sL1b1c->sensor.radarDesc.numParmDesc);
      memcpy(&sL1b1c->sensor.radarDesc.numDesc, ptr,
             sizeof(sL1b1c->sensor.radarDesc.numDesc));
      ptr += sizeof(sL1b1c->sensor.radarDesc.numDesc);
      memcpy(&sL1b1c->sensor.radarDesc.dataComp, ptr, 
             sizeof(sL1b1c->sensor.radarDesc.dataComp));
      ptr += sizeof(sL1b1c->sensor.radarDesc.dataComp);
      memcpy(&sL1b1c->sensor.radarDesc.dataReductAlg, ptr,
             sizeof(sL1b1c->sensor.radarDesc.dataReductAlg));
      ptr += sizeof(sL1b1c->sensor.radarDesc.dataReductAlg);
      memcpy(&sL1b1c->sensor.radarDesc.dataReductParm1, ptr, 
             sizeof(sL1b1c->sensor.radarDesc.dataReductParm1));
      ptr += sizeof(sL1b1c->sensor.radarDesc.dataReductParm1);
      memcpy(&sL1b1c->sensor.radarDesc.dataReductParm2, ptr, 
             sizeof(sL1b1c->sensor.radarDesc.dataReductParm2));
      ptr += sizeof(sL1b1c->sensor.radarDesc.dataReductParm2);
      memcpy(&sL1b1c->sensor.radarDesc.radarLon, ptr, 
             sizeof(sL1b1c->sensor.radarDesc.radarLon));
      ptr += sizeof(sL1b1c->sensor.radarDesc.radarLon);
      memcpy(&sL1b1c->sensor.radarDesc.radarLat, ptr, 
             sizeof(sL1b1c->sensor.radarDesc.radarLat));
      ptr += sizeof(sL1b1c->sensor.radarDesc.radarLat);
      memcpy(&sL1b1c->sensor.radarDesc.radarAlt, ptr, 
             sizeof(sL1b1c->sensor.radarDesc.radarAlt));
      ptr += sizeof(sL1b1c->sensor.radarDesc.radarAlt);
      memcpy(&sL1b1c->sensor.radarDesc.velocity, ptr, 
             sizeof(sL1b1c->sensor.radarDesc.velocity));
      ptr += sizeof(sL1b1c->sensor.radarDesc.velocity);
      memcpy(&sL1b1c->sensor.radarDesc.range, ptr, 
             sizeof(sL1b1c->sensor.radarDesc.range));
      ptr += sizeof(sL1b1c->sensor.radarDesc.range);
      memcpy(&sL1b1c->sensor.radarDesc.numTransfreqency, ptr, 
             sizeof(sL1b1c->sensor.radarDesc.numTransfreqency));
      ptr += sizeof(sL1b1c->sensor.radarDesc.numTransfreqency);
      memcpy(&sL1b1c->sensor.radarDesc.numInterPulsePeriods, ptr, 
             sizeof(sL1b1c->sensor.radarDesc.numInterPulsePeriods));
      ptr += sizeof(sL1b1c->sensor.radarDesc.numInterPulsePeriods);
      memcpy(&sL1b1c->sensor.radarDesc.frequency1, ptr, 
             sizeof(sL1b1c->sensor.radarDesc.frequency1));
      ptr += sizeof(sL1b1c->sensor.radarDesc.frequency1);
      memcpy(&sL1b1c->sensor.radarDesc.frequency2, ptr, 
             sizeof(sL1b1c->sensor.radarDesc.frequency2));
      ptr += sizeof(sL1b1c->sensor.radarDesc.frequency2);
      memcpy(&sL1b1c->sensor.radarDesc.frequency3, ptr, 
             sizeof(sL1b1c->sensor.radarDesc.frequency3));
      ptr += sizeof(sL1b1c->sensor.radarDesc.frequency3);
      memcpy(&sL1b1c->sensor.radarDesc.frequency4, ptr, 
             sizeof(sL1b1c->sensor.radarDesc.frequency4));
      ptr += sizeof(sL1b1c->sensor.radarDesc.frequency4);
      memcpy(&sL1b1c->sensor.radarDesc.frequency5, ptr, 
             sizeof(sL1b1c->sensor.radarDesc.frequency5));
      ptr += sizeof(sL1b1c->sensor.radarDesc.frequency5);
      memcpy(&sL1b1c->sensor.radarDesc.interPulsePeriod1, ptr, 
             sizeof(sL1b1c->sensor.radarDesc.interPulsePeriod1));
      ptr += sizeof(sL1b1c->sensor.radarDesc.interPulsePeriod1);
      memcpy(&sL1b1c->sensor.radarDesc.interPulsePeriod2, ptr, 
             sizeof(sL1b1c->sensor.radarDesc.interPulsePeriod2));
      ptr += sizeof(sL1b1c->sensor.radarDesc.interPulsePeriod2);
      memcpy(&sL1b1c->sensor.radarDesc.interPulsePeriod3, ptr, 
             sizeof(sL1b1c->sensor.radarDesc.interPulsePeriod3));
      ptr += sizeof(sL1b1c->sensor.radarDesc.interPulsePeriod3);
      memcpy(&sL1b1c->sensor.radarDesc.interPulsePeriod4, ptr, 
             sizeof(sL1b1c->sensor.radarDesc.interPulsePeriod4));
      ptr += sizeof(sL1b1c->sensor.radarDesc.interPulsePeriod4);
      memcpy(&sL1b1c->sensor.radarDesc.interPulsePeriod5, ptr,
             sizeof(sL1b1c->sensor.radarDesc.interPulsePeriod5)); 


      bufManage(&databuf, &bufsize, MINIMAL_BUFFER_SIZE);
      ptr = databuf;
      vdataRead(&tempVdata, ptr);
      memcpy(&sL1b1c->sensor.corrFactorDesc.azimuth, ptr,
             sizeof(sL1b1c->sensor.corrFactorDesc.azimuth));
      ptr += sizeof(sL1b1c->sensor.corrFactorDesc.azimuth);
      memcpy(&sL1b1c->sensor.corrFactorDesc.elevation, ptr,
             sizeof(sL1b1c->sensor.corrFactorDesc.elevation));
      ptr += sizeof(sL1b1c->sensor.corrFactorDesc.elevation);
      memcpy(&sL1b1c->sensor.corrFactorDesc.rangeDelay, ptr,
             sizeof(sL1b1c->sensor.corrFactorDesc.rangeDelay));
      ptr += sizeof(sL1b1c->sensor.corrFactorDesc.rangeDelay);
      memcpy(&sL1b1c->sensor.corrFactorDesc.radarLon, ptr,
             sizeof(sL1b1c->sensor.corrFactorDesc.radarLon));
      ptr += sizeof(sL1b1c->sensor.corrFactorDesc.radarLon);
      memcpy(&sL1b1c->sensor.corrFactorDesc.radarLat, ptr,
             sizeof(sL1b1c->sensor.corrFactorDesc.radarLat));
      ptr += sizeof(sL1b1c->sensor.corrFactorDesc.radarLat);
      memcpy(&sL1b1c->sensor.corrFactorDesc.radarPressAlt, ptr,
             sizeof(sL1b1c->sensor.corrFactorDesc.radarPressAlt));
      ptr += sizeof(sL1b1c->sensor.corrFactorDesc.radarPressAlt);
      memcpy(&sL1b1c->sensor.corrFactorDesc.radarAltAboveGround, ptr,
             sizeof(sL1b1c->sensor.corrFactorDesc.radarAltAboveGround));
      ptr += sizeof(sL1b1c->sensor.corrFactorDesc.radarAltAboveGround);
      memcpy(&sL1b1c->sensor.corrFactorDesc.radarPlatGroundSpeedEW, ptr,
             sizeof(sL1b1c->sensor.corrFactorDesc.radarPlatGroundSpeedEW));
      ptr += sizeof(sL1b1c->sensor.corrFactorDesc.radarPlatGroundSpeedEW);
      memcpy(&sL1b1c->sensor.corrFactorDesc.radarPlatGroundSpeedNS, ptr,
             sizeof(sL1b1c->sensor.corrFactorDesc.radarPlatGroundSpeedNS));
      ptr += sizeof(sL1b1c->sensor.corrFactorDesc.radarPlatGroundSpeedNS);
      memcpy(&sL1b1c->sensor.corrFactorDesc.radarPlatVerticalVelocity, ptr,
             sizeof(sL1b1c->sensor.corrFactorDesc.radarPlatVerticalVelocity));
      ptr += sizeof(sL1b1c->sensor.corrFactorDesc.radarPlatVerticalVelocity);
      memcpy(&sL1b1c->sensor.corrFactorDesc.radarPlatHeading, ptr,
             sizeof(sL1b1c->sensor.corrFactorDesc.radarPlatHeading));
      ptr += sizeof(sL1b1c->sensor.corrFactorDesc.radarPlatHeading);
      memcpy(&sL1b1c->sensor.corrFactorDesc.radarPlatRoll, ptr,
             sizeof(sL1b1c->sensor.corrFactorDesc.radarPlatRoll));
      ptr += sizeof(sL1b1c->sensor.corrFactorDesc.radarPlatRoll);
      memcpy(&sL1b1c->sensor.corrFactorDesc.radarPlatPitch, ptr,
             sizeof(sL1b1c->sensor.corrFactorDesc.radarPlatPitch));
      ptr += sizeof(sL1b1c->sensor.corrFactorDesc.radarPlatPitch);
      memcpy(&sL1b1c->sensor.corrFactorDesc.radarPlatDrift, ptr,
             sizeof(sL1b1c->sensor.corrFactorDesc.radarPlatDrift));
      ptr += sizeof(sL1b1c->sensor.corrFactorDesc.radarPlatDrift);
      memcpy(&sL1b1c->sensor.corrFactorDesc.radarRotAngle, ptr,
             sizeof(sL1b1c->sensor.corrFactorDesc.radarRotAngle));
      ptr += sizeof(sL1b1c->sensor.corrFactorDesc.radarRotAngle);
      memcpy(&sL1b1c->sensor.corrFactorDesc.radarTiltAngle, ptr,
             sizeof(sL1b1c->sensor.corrFactorDesc.radarTiltAngle));


      bufManage(&databuf, &bufsize, MINIMAL_BUFFER_SIZE);
      tempVdata1 = tempVdata;
      for (i = 0; i < nsweep; ++i)
      {
	   tempVdata = tempVdata1;
           ptr = databuf;
           vdataRead(&tempVdata, ptr);
           memcpy(&sL1b1c->sensor.sweepInfo[i].radarName, ptr,
                  sizeof(sL1b1c->sensor.sweepInfo[i].radarName));
           ptr += sizeof(sL1b1c->sensor.sweepInfo[i].radarName);
           memcpy(&sL1b1c->sensor.sweepInfo[i].sweepNum, ptr,
                  sizeof(sL1b1c->sensor.sweepInfo[i].sweepNum));
           ptr += sizeof(sL1b1c->sensor.sweepInfo[i].sweepNum);
           memcpy(&sL1b1c->sensor.sweepInfo[i].numRays, ptr,
                  sizeof(sL1b1c->sensor.sweepInfo[i].numRays));
           ptr += sizeof(sL1b1c->sensor.sweepInfo[i].numRays);
           memcpy(&sL1b1c->sensor.sweepInfo[i].trueStartAngle, ptr,
                  sizeof(sL1b1c->sensor.sweepInfo[i].trueStartAngle));
           ptr += sizeof(sL1b1c->sensor.sweepInfo[i].trueStartAngle);
           memcpy(&sL1b1c->sensor.sweepInfo[i].trueStopAngle, ptr,
                  sizeof(sL1b1c->sensor.sweepInfo[i].trueStopAngle));
           ptr += sizeof(sL1b1c->sensor.sweepInfo[i].trueStopAngle);
           memcpy(&sL1b1c->sensor.sweepInfo[i].fixedAngle, ptr,
                  sizeof(sL1b1c->sensor.sweepInfo[i].fixedAngle));
           ptr += sizeof(sL1b1c->sensor.sweepInfo[i].fixedAngle);
           memcpy(&sL1b1c->sensor.sweepInfo[i].filterFlag, ptr,
                  sizeof(sL1b1c->sensor.sweepInfo[i].filterFlag));
      }


      /*
      ** Allocate memory to read packed ray info integer information from 
      ** the HDF file. 4 represents the byte size of one ray info integer.
      */
      bufManage(&databuf, &bufsize, (4 * 7 * nray * nsweep));
      sdsRead(&tempSDS, &scaleFactor, databuf);
      ptr = databuf;
      for (i = 0; i < nsweep; ++i)
      {
           memcpy(&sL1b1c->sensor.rayInfoInteger[i], ptr, (4 * 7 * nray));
           ptr += (4 * 7 * nray);
      }


      /*
      ** Allocate memory to read packed ray info float information from
      ** the HDF file.  4 represents the byte size of one ray info float.
      */
      bufManage(&databuf, &bufsize, (4 * 4 * nray * nsweep));
      sdsRead(&tempSDS, &scaleFactor, databuf);
      ptr = databuf;
      for (i = 0; i < nsweep; ++i)
      {
           memcpy(&sL1b1c->sensor.rayInfoFloat[i], ptr, (4 * 4 * nray));
           ptr +=  (4 * 4 * nray);
      }


      /*
      ** Allocate memory to read packed platform Info information from
      ** the HDF file.  4 represents the byte size of one platform Info.
      */
      bufManage(&databuf, &bufsize, (4 * 18 * nray * nsweep));
      sdsRead(&tempSDS, &scaleFactor, databuf);
      ptr = databuf;
      for (i = 0; i < nsweep; ++i)
      {
           memcpy(&sL1b1c->sensor.platformInfo[i], ptr, (4 * 18 * nray));
           ptr += (4 * 18 * nray);

      }


      /*
      ** Read parameter information from the HDF file.
      */
      i = 0;
      parmVgroup = currVgroup->subVgroup->subVgroup;
      while (parmVgroup != NULL)
      {
             tempSDS = parmVgroup->sds;
             tempVdata = parmVgroup->vdata;
 
             bufManage(&databuf, &bufsize, MINIMAL_BUFFER_SIZE);
             ptr = databuf;
             vdataRead(&tempVdata, ptr);
             memcpy(&sL1b1c->sensor.parm[i]->parmDesc.parmName, ptr,
            	    sizeof(sL1b1c->sensor.parm[i]->parmDesc.parmName));
             ptr += sizeof(sL1b1c->sensor.parm[i]->parmDesc.parmName);
             memcpy(&sL1b1c->sensor.parm[i]->parmDesc.parmDesc, ptr,
                    sizeof(sL1b1c->sensor.parm[i]->parmDesc.parmDesc));
             ptr += sizeof(sL1b1c->sensor.parm[i]->parmDesc.parmDesc);
             memcpy(&sL1b1c->sensor.parm[i]->parmDesc.parmUnits, ptr,
                    sizeof(sL1b1c->sensor.parm[i]->parmDesc.parmUnits));
             ptr += sizeof(sL1b1c->sensor.parm[i]->parmDesc.parmUnits);
             memcpy(&sL1b1c->sensor.parm[i]->parmDesc.interPulsePeriod, ptr,
                    sizeof(sL1b1c->sensor.parm[i]->parmDesc.interPulsePeriod));
             ptr += sizeof(sL1b1c->sensor.parm[i]->parmDesc.interPulsePeriod);
             memcpy(&sL1b1c->sensor.parm[i]->parmDesc.transFreq, ptr,
                    sizeof(sL1b1c->sensor.parm[i]->parmDesc.transFreq));
             ptr += sizeof(sL1b1c->sensor.parm[i]->parmDesc.transFreq);
             memcpy(&sL1b1c->sensor.parm[i]->parmDesc.receiverBandwidth, ptr,
               	    sizeof(sL1b1c->sensor.parm[i]->parmDesc.receiverBandwidth));
             ptr += sizeof(sL1b1c->sensor.parm[i]->parmDesc.receiverBandwidth);
             memcpy(&sL1b1c->sensor.parm[i]->parmDesc.pulseWidth, ptr,
               	    sizeof(sL1b1c->sensor.parm[i]->parmDesc.pulseWidth));
             ptr += sizeof(sL1b1c->sensor.parm[i]->parmDesc.pulseWidth);
             memcpy(&sL1b1c->sensor.parm[i]->parmDesc.polarTransWave, ptr,
               	    sizeof(sL1b1c->sensor.parm[i]->parmDesc.polarTransWave));
             ptr += sizeof(sL1b1c->sensor.parm[i]->parmDesc.polarTransWave);
             memcpy(&sL1b1c->sensor.parm[i]->parmDesc.numOfsamples, ptr,
             	    sizeof(sL1b1c->sensor.parm[i]->parmDesc.numOfsamples));
             ptr += sizeof(sL1b1c->sensor.parm[i]->parmDesc.numOfsamples);
             memcpy(&sL1b1c->sensor.parm[i]->parmDesc.parmDataType, ptr,
               	    sizeof(sL1b1c->sensor.parm[i]->parmDesc.parmDataType));
             ptr += sizeof(sL1b1c->sensor.parm[i]->parmDesc.parmDataType);
             memcpy(&sL1b1c->sensor.parm[i]->parmDesc.thresholdField, ptr,
              	    sizeof(sL1b1c->sensor.parm[i]->parmDesc.thresholdField));
             ptr += sizeof(sL1b1c->sensor.parm[i]->parmDesc.thresholdField);
             memcpy(&sL1b1c->sensor.parm[i]->parmDesc.thresholdValue, ptr,
               	    sizeof(sL1b1c->sensor.parm[i]->parmDesc.thresholdValue));
             ptr += sizeof(sL1b1c->sensor.parm[i]->parmDesc.thresholdValue);
             memcpy(&sL1b1c->sensor.parm[i]->parmDesc.scaleFactor, ptr,
                    sizeof(sL1b1c->sensor.parm[i]->parmDesc.scaleFactor));
             ptr += sizeof(sL1b1c->sensor.parm[i]->parmDesc.scaleFactor);
             memcpy(&sL1b1c->sensor.parm[i]->parmDesc.offsetFactor, ptr,
                    sizeof(sL1b1c->sensor.parm[i]->parmDesc.offsetFactor));
             ptr += sizeof(sL1b1c->sensor.parm[i]->parmDesc.offsetFactor);
             memcpy(&sL1b1c->sensor.parm[i]->parmDesc.deletedOrMissDataFlag, ptr,
                    sizeof(sL1b1c->sensor.parm[i]->parmDesc.deletedOrMissDataFlag));


	     /*
	     ** Read the packed cell range vector information from databuf.
             */ 
             bufManage(&databuf, &bufsize, MINIMAL_BUFFER_SIZE);
             ptr = databuf;
             vdataRead(&tempVdata, ptr);
             memcpy(&sL1b1c->sensor.parm[i]->cellRangeVector.numOfCells, ptr,
                    sizeof(sL1b1c->sensor.parm[i]->cellRangeVector.numOfCells));
             ptr += sizeof(sL1b1c->sensor.parm[i]->cellRangeVector.numOfCells);
	     for (j = 0; j < ncell[i]; ++j)
	     {
                  memcpy(&sL1b1c->sensor.parm[i]->cellRangeVector.distanceToCell[j], ptr,
			 sizeof(sL1b1c->sensor.parm[i]->cellRangeVector.distanceToCell[j]));
	          ptr += sizeof(sL1b1c->sensor.parm[i]->cellRangeVector.distanceToCell[j]);
             }

     

             if (productID == TK_L1C_GV)
             {
		if ((i % 2) == 0)
		{
	           /*
	           **  Check the datatype of parameter from the Parameter Descriptor.
	           **  If it is equal to 1, the datatype of parameter is a 8-bit integer.
	           **  If it is equal to 2, the datatype of parameter is a 16-bit integer.
	           */
                   bufManage(&databuf, &bufsize, (1 * ncell[i] * nray * nsweep));
               	   sdsRead(&tempSDS, &scaleFactor, databuf);
               	   ptr = databuf;
               	   for (j = 0; j < nsweep; ++j)
               	   {
                     for (k = 0; k < nray; ++k)
                     {
		       memcpy ((uint8 *)sL1b1c->sensor.parm[i]->parmData1byte[j][k],
			       ptr, (1 * ncell[i]));
                        ptr += (1 * ncell[i]);
                     }
               	   }
		}

		else
		{
                  bufManage(&databuf, &bufsize, (2 * ncell[i] * nray * nsweep));
               	  sdsRead(&tempSDS, &scaleFactor, databuf);
               	  ptr = databuf;
               	  for (j = 0; j < nsweep; ++j)
               	  {
                    for (k = 0; k < nray; ++k)
                    {
		      /*  Changed R. Slywczak  03/03/97 
                       memcpy((uint8 *)(sL1b1c->sensor.parm[i]->parmData2byte[j] +
				k * ncell[i]), ptr, (2 * ncell[i]));  */
		      memcpy((uint16 *)sL1b1c->sensor.parm[i]->parmData2byte[j][k],
			     ptr, (2 * ncell[i]));
                       ptr += (2 * ncell[i]);
                    }
               	  }
		}

             }
 

	     if (productID == TK_L1B_GV)
	     { 
	       /*
	       ** Allocate memory to read packed parameter data.   After reading the 
               ** data from the HDF file, unpacked them into sL1b1c structure.
               */ 
               bufManage(&databuf, &bufsize, (2 * ncell[i] * nray * nsweep));
	       sdsRead(&tempSDS, &scaleFactor, databuf);
	       ptr = databuf;
               for (j = 0; j < nsweep; ++j)
	       {
	          for (k = 0; k < nray; ++k)
	          {
		    /*  Changed R. Slywczak 03/03/97
	               memcpy((uint8 *)(sL1b1c->sensor.parm[i]->parmData2byte[j] +
				k * ncell[i]), ptr, (2 * ncell[i])); */
		    memcpy((uint16 *)sL1b1c->sensor.parm[i]->parmData2byte[j][k],
			   ptr, (2 * ncell[i]));
	               ptr += (2 * ncell[i]);
	          }
  	       }
	     }

	     parmVgroup = parmVgroup->next;
	     i++;
      }
      free(databuf);
  }

  return (TK_SUCCESS);
}



/***********************************************************************************
*
*  FUNCTION STATEMENT:
*       writeGVGrid_2A_53_single(IO_HANDLE *sGranuleHandle, 
*				   L2A_53_SINGLE_RADARGRID *sRadarGrid);
*
*  DESCRIPTION:
*       This function is used to write the single radar site L2A_53 information in HDF format.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       sGranuleHandle          A structure containing information about the file.
*                               sGranuleHandle is returned by TKopen().
*
*       sRadarGrid		A structure containing the complete scanline data.
*                               This structure must be declared to correspond to 
*                               the data file being written.  The structures of 
*                               sRadarGrid is manipulated by TKwriteGrid() and 
*				other toolkit routines.
*
*  OUTPUT PARAMETERS:           None
*
*  CALLED BY:                   TKwriteGrid()
*
*  ROUTINES CALLED:     
*       Routine Name            Description
*       ------------            -----------
*       allocateVdataMemory     This routine is used to allocate memory to vdata.
*       vdataWrite              This routine is used to write the vdata in HDF 
*                               format.
*       sdsWrite                This routine is used to write the scientific data
*                               set in HDF format.      
*
***********************************************************************************/

int writeGVGrid_2A_53_single(IO_HANDLE *sGranuleHandle, L2A_53_SINGLE_RADARGRID *sRadarGrid)
{
  SDS_LINK *tempSDS  = sGranuleHandle->cfgSTR->vGroup->subVgroup->sds;
  VDATA_LINK *tempVdata = sGranuleHandle->cfgSTR->vGroup->subVgroup->vdata;
  uint8 *databuf, *ptr;
  int i, j;

  databuf = allocateVdataMemory(&tempVdata->vdataFields);
  ptr = databuf;
  memcpy(ptr, &sRadarGrid->tktime.tkhour, sizeof(sRadarGrid->tktime.tkhour));
  ptr += sizeof(sRadarGrid->tktime.tkhour);
  memcpy(ptr, &sRadarGrid->tktime.tkminute, sizeof(sRadarGrid->tktime.tkminute));
  ptr += sizeof(sRadarGrid->tktime.tkminute);
  memcpy(ptr, &sRadarGrid->tktime.tksecond, sizeof(sRadarGrid->tktime.tksecond));
  vdataWrite(&tempVdata, databuf);
  free(databuf);

  for (i = 0; i < 151; ++i)
       for (j = 0; j < 151; ++j)
       {
        /* The Following code was added to take care of the Missing or 
           invalid data.  The reason to add this code was not to scale
           the invalid Data */

         if(sRadarGrid->rainRate[i][j] > TK_DEFAULT)
              sRadarGrid->rainRate_scale[i][j] = 
		(int16)RND(sRadarGrid->rainRate[i][j] * L2A53_RAINRATE);
	 else 
              sRadarGrid->rainRate_scale[i][j] =  TK_MISS_I2;
       }
  sdsWrite(&tempSDS, &sRadarGrid->rainRate_scale);

  return (TK_SUCCESS);
}

/***********************************************************************************
*
*  FUNCTION STATEMENT:
*       readGVGrid_2A_53_single(IO_HANDLE *sGranuleHandle, L2A_53_SINGLE_*sRadarGrid)
*
*  DESCRIPTION:
*       This function is used to read the single radear site L2A_53 information 
*	in HDF format.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       sGranuleHandle          A structure containing information about the file.
*                               sGranuleHandle is returned by TKopen().
*
*  OUTPUT PARAMETERS: 
*       Parameter Name          Description
*       --------------          -----------
*       sRadarGrid		A structure containing the complete scanline data.
*                               This structure must be declared to correspond to 
*                               the data file being written.  The structures of 
*                               sRadarGrid is manipulated by TKreadGrid() and 
*				other toolkit routines.
*
*
*  CALLED BY:                   TKreadGrid()
*
*  ROUTINES CALLED:     
*       Routine Name            Description
*       ------------            -----------
*       allocateVdataMemory     This routine is used to allocate memory to vdata.
*       vdataWrite              This routine is used to write the vdata in HDF 
*                               format.
*       sdsWrite                This routine is used to write the scientific data
*                               set in HDF format.      
*
***********************************************************************************/

int readGVGrid_2A_53_single(IO_HANDLE *sGranuleHandle, L2A_53_SINGLE_RADARGRID *sRadarGrid)
{
  SDS_LINK *tempSDS  = sGranuleHandle->cfgSTR->vGroup->subVgroup->sds;
  VDATA_LINK *tempVdata = sGranuleHandle->cfgSTR->vGroup->subVgroup->vdata;
  uint8 *databuf, *ptr;
  int i, j;
  int scaleFactor;

  databuf = allocateVdataMemory(&tempVdata->vdataFields);
  ptr = databuf;
  vdataRead(&tempVdata, ptr);
  memcpy(&sRadarGrid->tktime.tkhour, ptr, sizeof(sRadarGrid->tktime.tkhour));
  ptr += sizeof(sRadarGrid->tktime.tkhour);
  memcpy(&sRadarGrid->tktime.tkminute, ptr, sizeof(sRadarGrid->tktime.tkminute));
  ptr += sizeof(sRadarGrid->tktime.tkminute);
  memcpy(&sRadarGrid->tktime.tksecond, ptr, sizeof(sRadarGrid->tktime.tksecond));
  free(databuf);

  sdsRead(&tempSDS, &scaleFactor, &sRadarGrid->rainRate_scale);
  for (i = 0; i < 151; ++i)
       for (j = 0; j < 151; ++j)
            {
            if (sRadarGrid->rainRate_scale[i][j] != TK_MISS_I2)
	       sRadarGrid->rainRate[i][j] = 
			(sRadarGrid->rainRate_scale[i][j] * 1.0)/ scaleFactor;
            else
               sRadarGrid->rainRate[i][j] = TK_DEFAULT;
            }
  return (TK_SUCCESS);
}



/***********************************************************************************
*
*  FUNCTION STATEMENT:
*	writeGVGrid_2A_53_TX_mult(IO_HANDLE *sGranuleHandle, 
*				L2A_53_MULT_TX_RADARGRID *sRadarGrid)
*
*  DESCRIPTION:
*       This function is used to write the multiple radar sites L2A_53 information in 
*	HDF format.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       sGranuleHandle          A structure containing information about the file.
*                               sGranuleHandle is returned by TKopen().
*
*       sRadarGrid              A structure containing the complete scanline data.
*                               This structure must be declared to correspond to
*                               the data file being written.  The structures of
*                               sRadarGrid is manipulated by TKwriteGrid() and
*                               other toolkit routines.
*
*  OUTPUT PARAMETERS:           None
*
*  CALLED BY:                   TKwriteGrid()
*
*  ROUTINES CALLED:
*       Routine Name            Description
*       ------------            -----------
*       allocateVdataMemory     This routine is used to allocate memory to vdata.
*       vdataWrite              This routine is used to write the vdata in HDF
*                               format.
*       sdsWrite                This routine is used to write the scientific data
*                               set in HDF format.
*
***********************************************************************************/

int writeGVGrid_2A_53_TX_mult(IO_HANDLE *sGranuleHandle, L2A_53_MULT_TX_RADARGRID *sRadarGrid)
{
  SDS_LINK *tempSDS  = sGranuleHandle->cfgSTR->vGroup->subVgroup->sds;
  VDATA_LINK *tempVdata = sGranuleHandle->cfgSTR->vGroup->subVgroup->vdata;
  uint8 *databuf, *ptr;
  int i, j;

  databuf = allocateVdataMemory(&tempVdata->vdataFields);
  ptr = databuf;
  memcpy(ptr, &sRadarGrid->tktime.tkhour, sizeof(sRadarGrid->tktime.tkhour));
  ptr += sizeof(sRadarGrid->tktime.tkhour);
  memcpy(ptr, &sRadarGrid->tktime.tkminute, sizeof(sRadarGrid->tktime.tkminute));
  ptr += sizeof(sRadarGrid->tktime.tkminute);
  memcpy(ptr, &sRadarGrid->tktime.tksecond, sizeof(sRadarGrid->tktime.tksecond));
  vdataWrite(&tempVdata, databuf);
  free(databuf);

  for (i = 0; i < 285; ++i)
       for (j = 0; j < 363; ++j)
       {
        /* The Following code was added to take care of the Missing or
           invalid data.  The reason to add this code was not to scale
           the invalid Data */

         if(sRadarGrid->rainRate[i][j] > TK_DEFAULT)
              sRadarGrid->rainRate_scale[i][j] = 
		(int16)RND(sRadarGrid->rainRate[i][j] * L2A53_RAINRATE);
	 else
              sRadarGrid->rainRate_scale[i][j] = TK_MISS_I2;

       }
  sdsWrite(&tempSDS, &sRadarGrid->rainRate_scale);

  return (TK_SUCCESS);
}



/***********************************************************************************
*
*  FUNCTION STATEMENT:
*	readGVGrid_2A_53_TX_mult(IO_HANDLE *sGranuleHandle, 
*					L2A_53_MULT_TX_RADARGRID *sRadarGrid)
*
*  DESCRIPTION:
*       This function is used to read the multiple radear sites L2A_53 information 
*	in HDF format.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       sGranuleHandle          A structure containing information about the file.
*                               sGranuleHandle is returned by TKopen().
*
*  OUTPUT PARAMETERS:    
*       Parameter Name          Description
*       --------------          -----------
*       sRadarGrid              A structure containing the complete scanline data.
*                               This structure must be declared to correspond to
*                               the data file being written.  The structures of
*                               sRadarGrid is manipulated by TKreadGrid() and
*                               other toolkit routines.
*
*  CALLED BY:                   TKreadGrid()
*
*  ROUTINES CALLED:
*       Routine Name            Description
*       ------------            -----------
*       allocateVdataMemory     This routine is used to allocate memory to vdata.
*       vdataWrite              This routine is used to write the vdata in HDF
*                               format.
*       sdsWrite                This routine is used to write the scientific data
*                               set in HDF format.
*
***********************************************************************************/

int readGVGrid_2A_53_TX_mult(IO_HANDLE *sGranuleHandle, L2A_53_MULT_TX_RADARGRID *sRadarGrid)
{
  SDS_LINK *tempSDS  = sGranuleHandle->cfgSTR->vGroup->subVgroup->sds;
  VDATA_LINK *tempVdata = sGranuleHandle->cfgSTR->vGroup->subVgroup->vdata;
  uint8 *databuf, *ptr;
  int i, j;
  int scaleFactor;

  databuf = allocateVdataMemory(&tempVdata->vdataFields);
  ptr = databuf;
  vdataRead(&tempVdata, ptr);
  memcpy(&sRadarGrid->tktime.tkhour, ptr, sizeof(sRadarGrid->tktime.tkhour));
  ptr += sizeof(sRadarGrid->tktime.tkhour);
  memcpy(&sRadarGrid->tktime.tkminute, ptr, sizeof(sRadarGrid->tktime.tkminute));
  ptr += sizeof(sRadarGrid->tktime.tkminute);
  memcpy(&sRadarGrid->tktime.tksecond, ptr, sizeof(sRadarGrid->tktime.tksecond));
  free(databuf);

  sdsRead(&tempSDS, &scaleFactor, &sRadarGrid->rainRate_scale);

  for (i = 0; i < 285; ++i)
       for (j = 0; j < 363; ++j)
            {
            if (sRadarGrid->rainRate_scale[i][j] != TK_MISS_I2)
	       sRadarGrid->rainRate[i][j] = (sRadarGrid->rainRate_scale[i][j] * 1.0)/ scaleFactor;
            else
               sRadarGrid->rainRate[i][j] = TK_DEFAULT;
            }
  return(TK_SUCCESS);
}

/***********************************************************************************
*
*  FUNCTION STATEMENT:
*       writeGVGrid_2A_53_FL_mult(IO_HANDLE *sGranuleHandle, 
*                               L2A_53_MULT_FL_RADARGRID *sRadarGrid)
*
*  DESCRIPTION:
*       This function is used to write the multiple radar sites L2A_53 information in 
*       HDF format.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       sGranuleHandle          A structure containing information about the file.
*                               sGranuleHandle is returned by TKopen().
*
*       sRadarGrid              A structure containing the complete scanline data.
*                               This structure must be declared to correspond to
*                               the data file being written.  The structures of
*                               sRadarGrid is manipulated by TKwriteGrid() and
*                               other toolkit routines.
*
*  OUTPUT PARAMETERS:           None
*
*  CALLED BY:                   TKwriteGrid()
*
*  ROUTINES CALLED:
*       Routine Name            Description
*       ------------            -----------
*       allocateVdataMemory     This routine is used to allocate memory to vdata.
*       vdataWrite              This routine is used to write the vdata in HDF
*                               format.
*       sdsWrite                This routine is used to write the scientific data
*                               set in HDF format.
*
***********************************************************************************/

int writeGVGrid_2A_53_FL_mult(IO_HANDLE *sGranuleHandle, L2A_53_MULT_FL_RADARGRID *sRadarGrid)
{
  SDS_LINK *tempSDS  = sGranuleHandle->cfgSTR->vGroup->subVgroup->sds;
  VDATA_LINK *tempVdata = sGranuleHandle->cfgSTR->vGroup->subVgroup->vdata;
  uint8 *databuf, *ptr;
  int i, j;

  databuf = allocateVdataMemory(&tempVdata->vdataFields);
  ptr = databuf;
  memcpy(ptr, &sRadarGrid->tktime.tkhour, sizeof(sRadarGrid->tktime.tkhour));
  ptr += sizeof(sRadarGrid->tktime.tkhour);
  memcpy(ptr, &sRadarGrid->tktime.tkminute, sizeof(sRadarGrid->tktime.tkminute));
  ptr += sizeof(sRadarGrid->tktime.tkminute);
  memcpy(ptr, &sRadarGrid->tktime.tksecond, sizeof(sRadarGrid->tktime.tksecond));
  vdataWrite(&tempVdata, databuf);
  free(databuf);

  for (i = 0; i < 353; ++i)
       for (j = 0; j < 257; ++j)
       {
        /* The Following code was added to take care of the Missing or
           invalid data.  The reason to add this code was not to scale
           the invalid Data */

         if(sRadarGrid->rainRate[i][j] > TK_DEFAULT)
              sRadarGrid->rainRate_scale[i][j] = 
                (int16)RND(sRadarGrid->rainRate[i][j] * L2A53_RAINRATE);
	 else
              sRadarGrid->rainRate_scale[i][j] = TK_MISS_I2;

       }
  sdsWrite(&tempSDS, &sRadarGrid->rainRate_scale);

  return(TK_SUCCESS);
}



/***********************************************************************************
*
*  FUNCTION STATEMENT:
*       readGVGrid_2A_53_FL_mult(IO_HANDLE *sGranuleHandle, 
*                                       L2A_53_MULT_FL_RADARGRID *sRadarGrid)
*
*  DESCRIPTION:
*       This function is used to read the multiple radear sites L2A_53 information 
*       in HDF format.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       sGranuleHandle          A structure containing information about the file.
*                               sGranuleHandle is returned by TKopen().
*
*  OUTPUT PARAMETERS:    
*       Parameter Name          Description
*       --------------          -----------
*       sRadarGrid              A structure containing the complete scanline data.
*                               This structure must be declared to correspond to
*                               the data file being written.  The structures of
*                               sRadarGrid is manipulated by TKreadGrid() and
*                               other toolkit routines.
*
*  CALLED BY:                   TKreadGrid()
*
*  ROUTINES CALLED:
*       Routine Name            Description
*       ------------            -----------
*       allocateVdataMemory     This routine is used to allocate memory to vdata.
*       vdataWrite              This routine is used to write the vdata in HDF
*                               format.
*       sdsWrite                This routine is used to write the scientific data
*                               set in HDF format.
*
***********************************************************************************/

int readGVGrid_2A_53_FL_mult(IO_HANDLE *sGranuleHandle, L2A_53_MULT_FL_RADARGRID *sRadarGrid)
{
  SDS_LINK *tempSDS  = sGranuleHandle->cfgSTR->vGroup->subVgroup->sds;
  VDATA_LINK *tempVdata = sGranuleHandle->cfgSTR->vGroup->subVgroup->vdata;
  uint8 *databuf, *ptr;
  int i, j;
  int scaleFactor;

  databuf = allocateVdataMemory(&tempVdata->vdataFields);
  ptr = databuf;
  vdataRead(&tempVdata, ptr);
  memcpy(&sRadarGrid->tktime.tkhour, ptr, sizeof(sRadarGrid->tktime.tkhour));
  ptr += sizeof(sRadarGrid->tktime.tkhour);
  memcpy(&sRadarGrid->tktime.tkminute, ptr, sizeof(sRadarGrid->tktime.tkminute));
  ptr += sizeof(sRadarGrid->tktime.tkminute);
  memcpy(&sRadarGrid->tktime.tksecond, ptr, sizeof(sRadarGrid->tktime.tksecond));
  free(databuf);

  sdsRead(&tempSDS, &scaleFactor, &sRadarGrid->rainRate_scale);

  for (i = 0; i < 353; ++i)
       for (j = 0; j < 257; ++j)
            {
            if (sRadarGrid->rainRate_scale[i][j] != TK_MISS_I2)
               sRadarGrid->rainRate[i][j] = (sRadarGrid->rainRate_scale[i][j] * 1.0)/ scaleFactor;
            else
               sRadarGrid->rainRate[i][j] = TK_DEFAULT;
            }
  return(TK_SUCCESS);
}


/***********************************************************************************
*
*  FUNCTION STATEMENT:
*	writeGVGrid_2A_54_single(IO_HANDLE *sGranuleHandle, L2A_54_SINGLE_RADARGRID *sRadarGrid)
*
*  DESCRIPTION:
*       This function is used to write the single radar site L2A_54 information in HDF format.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       sGranuleHandle          A structure containing information about the file.
*                               sGranuleHandle is returned by TKopen().
*
*       sRadarGrid		A structure containing the complete scanline data.
*                               This structure must be declared to correspond to 
*                               the data file being written.  The structures of 
*                               sRadarGrid is manipulated by TKwriteGrid() and 
*				other toolkit routines.
*
*  OUTPUT PARAMETERS:           None
*
*  CALLED BY:                   TKwriteGrid()
*
*  ROUTINES CALLED:     
*       Routine Name            Description
*       ------------            -----------
*       allocateVdataMemory     This routine is used to allocate memory to vdata.
*       vdataWrite              This routine is used to write the vdata in HDF 
*                               format.
*       sdsWrite                This routine is used to write the scientific data
*                               set in HDF format.      
*
***********************************************************************************/

int writeGVGrid_2A_54_single(IO_HANDLE *sGranuleHandle, L2A_54_SINGLE_RADARGRID *sRadarGrid)
{
  SDS_LINK *tempSDS  = sGranuleHandle->cfgSTR->vGroup->subVgroup->sds;
  VDATA_LINK *tempVdata = sGranuleHandle->cfgSTR->vGroup->subVgroup->vdata;
  uint8 *databuf, *ptr;

  databuf = allocateVdataMemory(&tempVdata->vdataFields);
  ptr = databuf;
  memcpy(ptr, &sRadarGrid->tktime.tkhour, sizeof(sRadarGrid->tktime.tkhour));
  ptr += sizeof(sRadarGrid->tktime.tkhour);
  memcpy(ptr, &sRadarGrid->tktime.tkminute, sizeof(sRadarGrid->tktime.tkminute));
  ptr += sizeof(sRadarGrid->tktime.tkminute);
  memcpy(ptr, &sRadarGrid->tktime.tksecond, sizeof(sRadarGrid->tktime.tksecond));
  vdataWrite(&tempVdata, databuf);
  free(databuf);

  sdsWrite(&tempSDS, &sRadarGrid->convStratFlag);

  return TK_SUCCESS;
}

/***********************************************************************************
*
*  FUNCTION STATEMENT:
*       readGVGrid_2A_54_single(IO_HANDLE *sGranuleHandle, L2A_54_SINGLE_RADARGRID *sRadarGrid)
*
*  DESCRIPTION:
*       This function is used to read the single radar site L2A_54 information in HDF format.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       sGranuleHandle          A structure containing information about the file.
*                               sGranuleHandle is returned by TKopen().
*
*  OUTPUT PARAMETERS:           
*       Parameter Name          Description
*       --------------          -----------
*       sRadarGrid		A structure containing the complete scanline data.
*                               This structure must be declared to correspond to 
*                               the data file being written.  The structures of 
*                               sRadarData is manipulated by TKreadGrid() and 
*				other toolkit routines.
*
*
*  CALLED BY:                   TKreadGrid()
*
*  ROUTINES CALLED:     
*       Routine Name            Description
*       ------------            -----------
*       allocateVdataMemory     This routine is used to allocate memory to vdata.
*       vdataWrite              This routine is used to write the vdata in HDF 
*                               format.
*       sdsWrite                This routine is used to write the scientific data
*                               set in HDF format.      
*
***********************************************************************************/

int readGVGrid_2A_54_single(IO_HANDLE *sGranuleHandle, L2A_54_SINGLE_RADARGRID *sRadarGrid)
{
  SDS_LINK *tempSDS  = sGranuleHandle->cfgSTR->vGroup->subVgroup->sds;
  VDATA_LINK *tempVdata = sGranuleHandle->cfgSTR->vGroup->subVgroup->vdata;
  uint8 *databuf, *ptr;
  int scaleFactor;

  databuf = allocateVdataMemory(&tempVdata->vdataFields);
  ptr = databuf;
  vdataRead(&tempVdata, ptr);
  memcpy(&sRadarGrid->tktime.tkhour, ptr, sizeof(sRadarGrid->tktime.tkhour));
  ptr += sizeof(sRadarGrid->tktime.tkhour);
  memcpy(&sRadarGrid->tktime.tkminute, ptr, sizeof(sRadarGrid->tktime.tkminute));
  ptr += sizeof(sRadarGrid->tktime.tkminute);
  memcpy(&sRadarGrid->tktime.tksecond, ptr, sizeof(sRadarGrid->tktime.tksecond));
  free(databuf);

  sdsRead(&tempSDS, &scaleFactor, &sRadarGrid->convStratFlag);

  return TK_SUCCESS;
}


/***********************************************************************************
*
*  FUNCTION STATEMENT:
*	writeGVGrid_2A_54_TX_mult(IO_HANDLE *sGranuleHandle, 
*					L2A_54_MULT_TX_RADARGRID *sRadarGrid)
*
*  DESCRIPTION:
*       This function is used to write the multiple radar sites L2A_54 information in HDF format.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       sGranuleHandle          A structure containing information about the file.
*                               sGranuleHandle is returned by TKopen().
*
*       sRadarGrid              A structure containing the complete scanline data.
*                               This structure must be declared to correspond to
*                               the data file being written.  The structures of
*                               sRadarGrid is manipulated by TKwriteGrid() and
*                               other toolkit routines.
*
*  OUTPUT PARAMETERS:           None
*
*  CALLED BY:                   TKwriteGrid()
*
*  ROUTINES CALLED:
*       Routine Name            Description
*       ------------            -----------
*       allocateVdataMemory     This routine is used to allocate memory to vdata.
*       vdataWrite              This routine is used to write the vdata in HDF
*                               format.
*       sdsWrite                This routine is used to write the scientific data
*                               set in HDF format.
*
***********************************************************************************/

int writeGVGrid_2A_54_TX_mult(IO_HANDLE *sGranuleHandle, L2A_54_MULT_TX_RADARGRID *sRadarGrid)
{
  SDS_LINK *tempSDS  = sGranuleHandle->cfgSTR->vGroup->subVgroup->sds;
  VDATA_LINK *tempVdata = sGranuleHandle->cfgSTR->vGroup->subVgroup->vdata;
  uint8 *databuf, *ptr;

  databuf = allocateVdataMemory(&tempVdata->vdataFields);
  ptr = databuf;
  memcpy(ptr, &sRadarGrid->tktime.tkhour, sizeof(sRadarGrid->tktime.tkhour));
  ptr += sizeof(sRadarGrid->tktime.tkhour);
  memcpy(ptr, &sRadarGrid->tktime.tkminute, sizeof(sRadarGrid->tktime.tkminute));
  ptr += sizeof(sRadarGrid->tktime.tkminute);
  memcpy(ptr, &sRadarGrid->tktime.tksecond, sizeof(sRadarGrid->tktime.tksecond));
  vdataWrite(&tempVdata, databuf);
  free(databuf);

  sdsWrite(&tempSDS, &sRadarGrid->convStratFlag);

  return TK_SUCCESS;
}


/***********************************************************************************
*
*  FUNCTION STATEMENT:
*	readGVGrid_2A_54_TX_mult(IO_HANDLE *sGranuleHandle, 
*					L2A_54_MULT_TX_RADARGRID *sRadarGrid)
*
*  DESCRIPTION:
*       This function is used to read the multiple radar sites L2A_54 information in HDF format.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       sGranuleHandle          A structure containing information about the file.
*                               sGranuleHandle is returned by TKopen().
*
*  OUTPUT PARAMETERS:   
*       Parameter Name          Description
*       --------------          -----------
*       sRadarGrid              A structure containing the complete scanline data.
*                               This structure must be declared to correspond to
*                               the data file being written.  The structures of
*                               sRadarGrid is manipulated by TKreadGrid() and
*                               other toolkit routines.
*
*  CALLED BY:                   TKreadGrid()
*
*  ROUTINES CALLED:
*       Routine Name            Description
*       ------------            -----------
*       allocateVdataMemory     This routine is used to allocate memory to vdata.
*       vdataWrite              This routine is used to write the vdata in HDF
*                               format.
*       sdsWrite                This routine is used to write the scientific data
*                               set in HDF format.
*
***********************************************************************************/

int readGVGrid_2A_54_TX_mult(IO_HANDLE *sGranuleHandle, L2A_54_MULT_TX_RADARGRID *sRadarGrid)
{
  SDS_LINK *tempSDS  = sGranuleHandle->cfgSTR->vGroup->subVgroup->sds;
  VDATA_LINK *tempVdata = sGranuleHandle->cfgSTR->vGroup->subVgroup->vdata;
  uint8 *databuf, *ptr;
  int scaleFactor;

  databuf = allocateVdataMemory(&tempVdata->vdataFields);
  ptr = databuf;
  vdataRead(&tempVdata, ptr);
  memcpy(&sRadarGrid->tktime.tkhour, ptr, sizeof(sRadarGrid->tktime.tkhour));
  ptr += sizeof(sRadarGrid->tktime.tkhour);
  memcpy(&sRadarGrid->tktime.tkminute, ptr, sizeof(sRadarGrid->tktime.tkminute));
  ptr += sizeof(sRadarGrid->tktime.tkminute);
  memcpy(&sRadarGrid->tktime.tksecond, ptr, sizeof(sRadarGrid->tktime.tksecond));
  free(databuf);

  sdsRead(&tempSDS, &scaleFactor, &sRadarGrid->convStratFlag);

  return(TK_SUCCESS);
}


/***********************************************************************************
*
*  FUNCTION STATEMENT:
*       writeGVGrid_2A_54_FL_mult(IO_HANDLE *sGranuleHandle, 
*                                       L2A_54_MULT_FL_RADARGRID *sRadarGrid)
*
*  DESCRIPTION:
*       This function is used to write the multiple radar sites L2A_54 information in HDF format.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       sGranuleHandle          A structure containing information about the file.
*                               sGranuleHandle is returned by TKopen().
*
*       sRadarGrid              A structure containing the complete scanline data.
*                               This structure must be declared to correspond to
*                               the data file being written.  The structures of
*                               sRadarGrid is manipulated by TKwriteGrid() and
*                               other toolkit routines.
*
*  OUTPUT PARAMETERS:           None
*
*  CALLED BY:                   TKwriteGrid()
*
*  ROUTINES CALLED:
*       Routine Name            Description
*       ------------            -----------
*       allocateVdataMemory     This routine is used to allocate memory to vdata.
*       vdataWrite              This routine is used to write the vdata in HDF
*                               format.
*       sdsWrite                This routine is used to write the scientific data
*                               set in HDF format.
*
***********************************************************************************/

int writeGVGrid_2A_54_FL_mult(IO_HANDLE *sGranuleHandle, L2A_54_MULT_FL_RADARGRID *sRadarGrid)
{
  SDS_LINK *tempSDS  = sGranuleHandle->cfgSTR->vGroup->subVgroup->sds;
  VDATA_LINK *tempVdata = sGranuleHandle->cfgSTR->vGroup->subVgroup->vdata;
  uint8 *databuf, *ptr;

  databuf = allocateVdataMemory(&tempVdata->vdataFields);
  ptr = databuf;
  memcpy(ptr, &sRadarGrid->tktime.tkhour, sizeof(sRadarGrid->tktime.tkhour));
  ptr += sizeof(sRadarGrid->tktime.tkhour);
  memcpy(ptr, &sRadarGrid->tktime.tkminute, sizeof(sRadarGrid->tktime.tkminute));
  ptr += sizeof(sRadarGrid->tktime.tkminute);
  memcpy(ptr, &sRadarGrid->tktime.tksecond, sizeof(sRadarGrid->tktime.tksecond));
  vdataWrite(&tempVdata, databuf);
  free(databuf);

  sdsWrite(&tempSDS, &sRadarGrid->convStratFlag);

  return(TK_SUCCESS);
}


/***********************************************************************************
*
*  FUNCTION STATEMENT:
*       readGVGrid_2A_54_FL_mult(IO_HANDLE *sGranuleHandle, 
*                                       L2A_54_MULT_FL_RADARGRID *sRadarGrid)
*
*  DESCRIPTION:
*       This function is used to read the multiple radar sites L2A_54 information in HDF format.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       sGranuleHandle          A structure containing information about the file.
*                               sGranuleHandle is returned by TKopen().
*
*  OUTPUT PARAMETERS:   
*       Parameter Name          Description
*       --------------          -----------
*       sRadarGrid              A structure containing the complete scanline data.
*                               This structure must be declared to correspond to
*                               the data file being written.  The structures of
*                               sRadarGrid is manipulated by TKreadGrid() and
*                               other toolkit routines.
*
*  CALLED BY:                   TKreadGrid()
*
*  ROUTINES CALLED:
*       Routine Name            Description
*       ------------            -----------
*       allocateVdataMemory     This routine is used to allocate memory to vdata.
*       vdataWrite              This routine is used to write the vdata in HDF
*                               format.
*       sdsWrite                This routine is used to write the scientific data
*                               set in HDF format.
*
***********************************************************************************/

int readGVGrid_2A_54_FL_mult(IO_HANDLE *sGranuleHandle, L2A_54_MULT_FL_RADARGRID *sRadarGrid)
{
  SDS_LINK *tempSDS  = sGranuleHandle->cfgSTR->vGroup->subVgroup->sds;
  VDATA_LINK *tempVdata = sGranuleHandle->cfgSTR->vGroup->subVgroup->vdata;
  uint8 *databuf, *ptr;
  int scaleFactor;

  databuf = allocateVdataMemory(&tempVdata->vdataFields);
  ptr = databuf;
  vdataRead(&tempVdata, ptr);
  memcpy(&sRadarGrid->tktime.tkhour, ptr, sizeof(sRadarGrid->tktime.tkhour));
  ptr += sizeof(sRadarGrid->tktime.tkhour);
  memcpy(&sRadarGrid->tktime.tkminute, ptr, sizeof(sRadarGrid->tktime.tkminute));
  ptr += sizeof(sRadarGrid->tktime.tkminute);
  memcpy(&sRadarGrid->tktime.tksecond, ptr, sizeof(sRadarGrid->tktime.tksecond));
  free(databuf);

  sdsRead(&tempSDS, &scaleFactor, &sRadarGrid->convStratFlag);

  return(TK_SUCCESS);
}



/***********************************************************************************
*
*  FUNCTION STATEMENT:
*	writeGVGrid_2A_55_single(IO_HANDLE *sGranuleHandle, 
*					L2A_55_SINGLE_RADARGRID *sRadarGrid)
*
*  DESCRIPTION:
*       This function is used to write the single radar site L2A_55 information in 
*	HDF format.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       sGranuleHandle          A structure containing information about the file.
*                               sGranuleHandle is returned by TKopen().
*
*       sRadarGrid		A structure containing the complete scanline data.
*                               This structure must be declared to correspond to 
*                               the data file being written.  The structures of 
*                               sRadarGrid is manipulated by TKwriteGrid() and 
*				other toolkit routines.
*
*  OUTPUT PARAMETERS:           None
*
*  CALLED BY:                   TKwriteGrid()
*
*  ROUTINES CALLED:     
*       Routine Name            Description
*       ------------            -----------
*       allocateVdataMemory     This routine is used to allocate memory to vdata.
*       vdataWrite              This routine is used to write the vdata in HDF 
*                               format.
*       sdsWrite                This routine is used to write the scientific data
*                               set in HDF format.      
*
***********************************************************************************/

int writeGVGrid_2A_55_single(IO_HANDLE *sGranuleHandle, 
				L2A_55_SINGLE_RADARGRID *sRadarGrid)
{
  SDS_LINK *tempSDS  = sGranuleHandle->cfgSTR->vGroup->subVgroup->sds;
  VDATA_LINK *tempVdata = sGranuleHandle->cfgSTR->vGroup->subVgroup->vdata;
  uint8 *databuf, *ptr;
  int i, j, k;


  databuf = allocateVdataMemory(&tempVdata->vdataFields);
  ptr = databuf;
  memcpy(ptr, &sRadarGrid->tktime.tkhour, sizeof(sRadarGrid->tktime.tkhour));
  ptr += sizeof(sRadarGrid->tktime.tkhour);
  memcpy(ptr, &sRadarGrid->tktime.tkminute, sizeof(sRadarGrid->tktime.tkminute));
  ptr += sizeof(sRadarGrid->tktime.tkminute);
  memcpy(ptr, &sRadarGrid->tktime.tksecond, sizeof(sRadarGrid->tktime.tksecond));
  vdataWrite(&tempVdata, databuf);
  free(databuf);

  for (i = 0; i < 13; ++i)
       for (j = 0; j < 151; ++j)
	    for (k = 0; k < 151; ++k)
		{
		if(sRadarGrid->threeDreflect[i][j][k] > TK_DEFAULT)
		sRadarGrid->threeDreflect_scale[i][j][k] = 
		(int16)RND(sRadarGrid->threeDreflect[i][j][k] * L2A55_THREEDREFLECT);
		else
		sRadarGrid->threeDreflect_scale[i][j][k] = TK_MISS_I2;
		}

  sdsWrite(&tempSDS, &sRadarGrid->threeDreflect_scale);

  for (i = 0; i < 13; ++i)
    for (j = 0; j < 12; ++j)
    {
        /* The Following code was added to take care of the Missing or
           invalid data.  The reason to add this code was not to scale
           the invalid Data */

      if(sRadarGrid->vertProfile[i][j] > TK_DEFAULT)
	   sRadarGrid->vertProfile_scale[i][j] =
              (int16)RND(sRadarGrid->vertProfile[i][j] * L2A55_VERTICALPROFILE);
      else
	   sRadarGrid->vertProfile_scale[i][j] = TK_MISS_I2;
    }
  sdsWrite(&tempSDS, &sRadarGrid->vertProfile_scale);

  sdsWrite(&tempSDS, &sRadarGrid->cfadData);

  return(TK_SUCCESS);
}

/***********************************************************************************
*
*  FUNCTION STATEMENT:
*	readGVGrid_2A_55_single(IO_HANDLE *sGranuleHandle, L2A_55_SINGLE_RADARGRID *sRadarGrid)
*
*  DESCRIPTION:
*       This function is used to write the single radar site L2A_55 information in HDF format.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       sGranuleHandle          A structure containing information about the file.
*                               sGranuleHandle is returned by TKopen().
*
*  OUTPUT PARAMETERS:    
*       Parameter Name          Description
*       --------------          -----------
*       sRadarGrid		A structure containing the complete scanline data.
*                               This structure must be declared to correspond to 
*                               the data file being written.  The structures of 
*                               sRadarData is manipulated by TKreadGrid() and 
*				other toolkit routines.
*
*  CALLED BY:                   TKreadGrid()
*
*  ROUTINES CALLED:     
*       Routine Name            Description
*       ------------            -----------
*       allocateVdataMemory     This routine is used to allocate memory to vdata.
*       vdataWrite              This routine is used to write the vdata in HDF 
*                               format.
*       sdsWrite                This routine is used to write the scientific data
*                               set in HDF format.      
*
***********************************************************************************/

int readGVGrid_2A_55_single(IO_HANDLE *sGranuleHandle, L2A_55_SINGLE_RADARGRID *sRadarGrid)
{
  SDS_LINK *tempSDS  = sGranuleHandle->cfgSTR->vGroup->subVgroup->sds;
  VDATA_LINK *tempVdata = sGranuleHandle->cfgSTR->vGroup->subVgroup->vdata;
  uint8 *databuf, *ptr;
  int i, j, k;
  int scaleFactor;

  databuf = allocateVdataMemory(&tempVdata->vdataFields);
  ptr = databuf;
  vdataRead(&tempVdata, ptr);
  memcpy(&sRadarGrid->tktime.tkhour, ptr, sizeof(sRadarGrid->tktime.tkhour));
  ptr += sizeof(sRadarGrid->tktime.tkhour);
  memcpy(&sRadarGrid->tktime.tkminute, ptr, sizeof(sRadarGrid->tktime.tkminute));
  ptr += sizeof(sRadarGrid->tktime.tkminute);
  memcpy(&sRadarGrid->tktime.tksecond, ptr, sizeof(sRadarGrid->tktime.tksecond));
  free(databuf);

  sdsRead(&tempSDS, &scaleFactor, &sRadarGrid->threeDreflect_scale);

  for (i = 0; i < 13; ++i)
       for (j = 0; j < 151; ++j)
	    for (k = 0; k < 151; ++k)
		{
		if (sRadarGrid->threeDreflect_scale[i][j][k] != TK_MISS_I2)
		 sRadarGrid->threeDreflect[i][j][k] = 
		(sRadarGrid->threeDreflect_scale[i][j][k]*1.0)/ scaleFactor;
		else
		sRadarGrid->threeDreflect[i][j][k] = TK_DEFAULT;
		}

  sdsRead(&tempSDS, &scaleFactor, &sRadarGrid->vertProfile_scale);

  for (i = 0; i < 13; ++i)
    for (j = 0; j < 12; ++j)
        {
        if (sRadarGrid->vertProfile_scale[i][j] != TK_MISS_I2)
	   sRadarGrid->vertProfile[i][j] =
		(sRadarGrid->vertProfile_scale[i][j] * 1.0) / scaleFactor;
        else
           sRadarGrid->vertProfile[i][j] = TK_DEFAULT;
        }
  sdsRead(&tempSDS, &scaleFactor, &sRadarGrid->cfadData);

  return(TK_SUCCESS);
}


/***********************************************************************************
*
*  FUNCTION STATEMENT:
*       writeGVGrid_2A_55_TX_mult(IO_HANDLE *sGranuleHandle, 
*					L2A_55_MULT_TX_RADARGRID *sRadarGrid)
*
*  DESCRIPTION:
*       This function is used to write the multiple radar sites L2A_55 information in HDF format.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       sGranuleHandle          A structure containing information about the file.
*                               sGranuleHandle is returned by TKopen().
*
*       sRadarGrid              A structure containing the complete scanline data.
*                               This structure must be declared to correspond to
*                               the data file being written.  The structures of
*                               sRadarGrid is manipulated by TKwriteGrid() and
*                               other toolkit routines.
*
*  OUTPUT PARAMETERS:           None
*
*  CALLED BY:                   TKwriteGrid()
*
*  ROUTINES CALLED:
*       Routine Name            Description
*       ------------            -----------
*       allocateVdataMemory     This routine is used to allocate memory to vdata.
*       vdataWrite              This routine is used to write the vdata in HDF
*                               format.
*       sdsWrite                This routine is used to write the scientific data
*                               set in HDF format.
*
***********************************************************************************/

int writeGVGrid_2A_55_TX_mult(IO_HANDLE *sGranuleHandle, L2A_55_MULT_TX_RADARGRID *sRadarGrid)
{
  SDS_LINK *tempSDS = sGranuleHandle->cfgSTR->vGroup->subVgroup->sds;
  VDATA_LINK *tempVdata = sGranuleHandle->cfgSTR->vGroup->subVgroup->vdata;
  uint8 *databuf, *ptr;
  int i, j, k;

  databuf = allocateVdataMemory(&tempVdata->vdataFields);
  ptr = databuf;
  memcpy(ptr, &sRadarGrid->tktime.tkhour, sizeof(sRadarGrid->tktime.tkhour));
  ptr += sizeof(sRadarGrid->tktime.tkhour);
  memcpy(ptr, &sRadarGrid->tktime.tkminute, sizeof(sRadarGrid->tktime.tkminute));
  ptr += sizeof(sRadarGrid->tktime.tkminute);
  memcpy(ptr, &sRadarGrid->tktime.tksecond, sizeof(sRadarGrid->tktime.tksecond));
  vdataWrite(&tempVdata, databuf);
  free(databuf);
  
  for (i = 0; i < 13; ++i)
       for (j = 0; j < 285; ++j)
	    for (k = 0; k < 363; ++k)
		{
		if(sRadarGrid->threeDreflect[i][j][k] > TK_DEFAULT)				 sRadarGrid->threeDreflect_scale[i][j][k] = 
		(int16)RND(sRadarGrid->threeDreflect[i][j][k] * L2A55_THREEDREFLECT);
		else
		sRadarGrid->threeDreflect_scale[i][j][k] = TK_MISS_I2;
		}

  sdsWrite(&tempSDS, &sRadarGrid->threeDreflect_scale);

  for (i = 0; i < 13; ++i)
    for (j = 0; j < 12; ++j)
    {
        /* The Following code was added to take care of the Missing or
           invalid data.  The reason to add this code was not to scale
           the invalid Data */

      if(sRadarGrid->vertProfile[i][j] > TK_DEFAULT)
	  sRadarGrid->vertProfile_scale[i][j] =
   	     (int16)RND(sRadarGrid->vertProfile[i][j] * L2A55_VERTICALPROFILE);
      else
	  sRadarGrid->vertProfile_scale[i][j] = TK_MISS_I2;
    }

  sdsWrite(&tempSDS, &sRadarGrid->vertProfile_scale);

  sdsWrite(&tempSDS, &sRadarGrid->cfadData);

  return(TK_SUCCESS);
}




/************************************************************************************************
*  FUNCTION STATEMENT:
*	readGVGrid_2A_55_TX_mult(IO_HANDLE *sGranuleHandle, L2A_55_MULT_TX_RADARGRID *sRadarGrid)
*
*  DESCRIPTION:
*       This function is used to write the mult radar sites L2A_55 information in HDF format.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       sGranuleHandle          A structure containing information about the file.
*                               sGranuleHandle is returned by TKopen().
*
*  OUTPUT PARAMETERS: 
*       Parameter Name          Description
*       --------------          -----------
*       sRadarGrid              A structure containing the complete scanline data.
*                               This structure must be declared to correspond to
*                               the data file being written.  The structures of
*                               sRadarData is manipulated by TKreadGrid() and
*                               other toolkit routines.
*
*  CALLED BY:                   TKreadGrid()
*
*  ROUTINES CALLED:
*       Routine Name            Description
*       ------------            -----------
*       allocateVdataMemory     This routine is used to allocate memory to vdata.
*       vdataWrite              This routine is used to write the vdata in HDF
*                               format.
*       sdsWrite                This routine is used to write the scientific data
*                               set in HDF format.
*
***********************************************************************************/

int readGVGrid_2A_55_TX_mult(IO_HANDLE *sGranuleHandle, L2A_55_MULT_TX_RADARGRID *sRadarGrid)
{
  SDS_LINK *tempSDS  = sGranuleHandle->cfgSTR->vGroup->subVgroup->sds;
  VDATA_LINK *tempVdata = sGranuleHandle->cfgSTR->vGroup->subVgroup->vdata;
  uint8 *databuf, *ptr;
  int i, j, k;
  int scaleFactor;

  databuf = allocateVdataMemory(&tempVdata->vdataFields);
  ptr = databuf;
  vdataRead(&tempVdata, ptr);
  memcpy(&sRadarGrid->tktime.tkhour, ptr, sizeof(sRadarGrid->tktime.tkhour));
  ptr += sizeof(sRadarGrid->tktime.tkhour);
  memcpy(&sRadarGrid->tktime.tkminute, ptr, sizeof(sRadarGrid->tktime.tkminute));
  ptr += sizeof(sRadarGrid->tktime.tkminute);
  memcpy(&sRadarGrid->tktime.tksecond, ptr, sizeof(sRadarGrid->tktime.tksecond));
  free(databuf);

  sdsRead(&tempSDS, &scaleFactor, &sRadarGrid->threeDreflect_scale);
  for (i = 0; i < 13; ++i)
       for (j = 0; j < 285; ++j)
	    for (k = 0; k < 363; ++k)
	   {
		if(sRadarGrid->threeDreflect_scale[i][j][k] != TK_MISS_I2)
		 sRadarGrid->threeDreflect[i][j][k] = 
			(sRadarGrid->threeDreflect_scale[i][j][k]*1.0)/ scaleFactor;
		else
		sRadarGrid->threeDreflect[i][j][k] = TK_DEFAULT;
	   }

  sdsRead(&tempSDS, &scaleFactor, &sRadarGrid->vertProfile_scale);
  for (i = 0; i < 13; ++i)
    for (j = 0; j < 12; ++j)
        {
        if (sRadarGrid->vertProfile_scale[i][j] != TK_MISS_I2)
	   sRadarGrid->vertProfile[i][j] =
		(sRadarGrid->vertProfile_scale[i][j] * 1.0) / scaleFactor;
        else
           sRadarGrid->vertProfile[i][j] = TK_DEFAULT;
        }
  sdsRead(&tempSDS, &scaleFactor, &sRadarGrid->cfadData);

  return(TK_SUCCESS);
}


/***********************************************************************************
*
*  FUNCTION STATEMENT:
*       writeGVGrid_2A_55_FL_mult(IO_HANDLE *sGranuleHandle, 
*                                       L2A_55_MULT_FL_RADARGRID *sRadarGrid)
*
*  DESCRIPTION:
*       This function is used to write the multiple radar sites L2A_55 information in HDF format.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       sGranuleHandle          A structure containing information about the file.
*                               sGranuleHandle is returned by TKopen().
*
*       sRadarGrid              A structure containing the complete scanline data.
*                               This structure must be declared to correspond to
*                               the data file being written.  The structures of
*                               sRadarGrid is manipulated by TKwriteGrid() and
*                               other toolkit routines.
*
*  OUTPUT PARAMETERS:           None
*
*  CALLED BY:                   TKwriteGrid()
*
*  ROUTINES CALLED:
*       Routine Name            Description
*       ------------            -----------
*       allocateVdataMemory     This routine is used to allocate memory to vdata.
*       vdataWrite              This routine is used to write the vdata in HDF
*                               format.
*       sdsWrite                This routine is used to write the scientific data
*                               set in HDF format.
*
***********************************************************************************/

int writeGVGrid_2A_55_FL_mult(IO_HANDLE *sGranuleHandle, L2A_55_MULT_FL_RADARGRID *sRadarGrid)
{
  SDS_LINK *tempSDS  = sGranuleHandle->cfgSTR->vGroup->subVgroup->sds;
  VDATA_LINK *tempVdata = sGranuleHandle->cfgSTR->vGroup->subVgroup->vdata;
  uint8 *databuf, *ptr;
  int i, j, k;


  databuf = allocateVdataMemory(&tempVdata->vdataFields);
  ptr = databuf;
  memcpy(ptr, &sRadarGrid->tktime.tkhour, sizeof(sRadarGrid->tktime.tkhour));
  ptr += sizeof(sRadarGrid->tktime.tkhour);
  memcpy(ptr, &sRadarGrid->tktime.tkminute, sizeof(sRadarGrid->tktime.tkminute));
  ptr += sizeof(sRadarGrid->tktime.tkminute);
  memcpy(ptr, &sRadarGrid->tktime.tksecond, sizeof(sRadarGrid->tktime.tksecond));
  vdataWrite(&tempVdata, databuf);
  free(databuf);

  for (i = 0; i < 13; ++i)
       for (j = 0; j < 353; ++j)
            for (k = 0; k < 257; ++k)
		{
		if(sRadarGrid->threeDreflect[i][j][k] > TK_DEFAULT)
                 sRadarGrid->threeDreflect_scale[i][j][k] = 
                        (int16)RND(sRadarGrid->threeDreflect[i][j][k] * L2A55_THREEDREFLECT);
		else
		sRadarGrid->threeDreflect_scale[i][j][k] = TK_MISS_I2;
		}

	  sdsWrite(&tempSDS, &sRadarGrid->threeDreflect_scale);

  for (i = 0; i < 13; ++i)
    for (j = 0; j < 12; ++j)
    {
        /* The Following code was added to take care of the Missing or
           invalid data.  The reason to add this code was not to scale
           the invalid Data */

      if(sRadarGrid->vertProfile[i][j] > TK_DEFAULT)
          sRadarGrid->vertProfile_scale[i][j] =
             (int16)RND(sRadarGrid->vertProfile[i][j] * L2A55_VERTICALPROFILE);
      else
          sRadarGrid->vertProfile_scale[i][j] = TK_MISS_I2;

    }
  sdsWrite(&tempSDS, &sRadarGrid->vertProfile_scale);

  sdsWrite(&tempSDS, &sRadarGrid->cfadData);

  return(TK_SUCCESS);
}



/***********************************************************************************
*
*  FUNCTION STATEMENT:
*       readGVGrid_2A_55_FL_mult(IO_HANDLE *sGranuleHandle, L2A_55_MULT_FL_RADARGRID *sRadarGrid)
*
*  DESCRIPTION:
*       This function is used to read the mult radar sites L2A_55 information in HDF format.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       sGranuleHandle          A structure containing information about the file.
*                               sGranuleHandle is returned by TKopen().
*
*  OUTPUT PARAMETERS: 
*       Parameter Name          Description
*       --------------          -----------
*       sRadarGrid              A structure containing the complete scanline data.
*                               This structure must be declared to correspond to
*                               the data file being written.  The structures of
*                               sRadarData is manipulated by TKreadGrid() and
*                               other toolkit routines.
*
*  CALLED BY:                   TKreadGrid()
*
*  ROUTINES CALLED:
*       Routine Name            Description
*       ------------            -----------
*       allocateVdataMemory     This routine is used to allocate memory to vdata.
*       vdataWrite              This routine is used to write the vdata in HDF
*                               format.
*       sdsWrite                This routine is used to write the scientific data
*                               set in HDF format.
*
***********************************************************************************/

int readGVGrid_2A_55_FL_mult(IO_HANDLE *sGranuleHandle, L2A_55_MULT_FL_RADARGRID *sRadarGrid)
{
  SDS_LINK *tempSDS  = sGranuleHandle->cfgSTR->vGroup->subVgroup->sds;
  VDATA_LINK *tempVdata = sGranuleHandle->cfgSTR->vGroup->subVgroup->vdata;
  uint8 *databuf, *ptr;
  int i, j, k;
  int scaleFactor;


  databuf = allocateVdataMemory(&tempVdata->vdataFields);
  ptr = databuf;
  vdataRead(&tempVdata, ptr);
  memcpy(&sRadarGrid->tktime.tkhour, ptr, sizeof(sRadarGrid->tktime.tkhour));
  ptr += sizeof(sRadarGrid->tktime.tkhour);
  memcpy(&sRadarGrid->tktime.tkminute, ptr, sizeof(sRadarGrid->tktime.tkminute));
  ptr += sizeof(sRadarGrid->tktime.tkminute);
  memcpy(&sRadarGrid->tktime.tksecond, ptr, sizeof(sRadarGrid->tktime.tksecond));
  free(databuf);

  sdsRead(&tempSDS, &scaleFactor, &sRadarGrid->threeDreflect_scale);
  for (i = 0; i < 13; ++i)
       for (j = 0; j < 353; ++j)
            for (k = 0; k < 257; ++k)
		{
		if(sRadarGrid->threeDreflect_scale[i][j][k] !=
TK_MISS_I2)
                 sRadarGrid->threeDreflect[i][j][k] = 
                        (sRadarGrid->threeDreflect_scale[i][j][k]*1.0)/ scaleFactor;
		else
		sRadarGrid->threeDreflect[i][j][k] = TK_DEFAULT;
		}

  sdsRead(&tempSDS, &scaleFactor, &sRadarGrid->vertProfile_scale);
  for (i = 0; i < 13; ++i)
    for (j = 0; j < 12; ++j)
        {
        if (sRadarGrid->vertProfile_scale[i][j] != TK_MISS_I2)
           sRadarGrid->vertProfile[i][j] =
                (sRadarGrid->vertProfile_scale[i][j] * 1.0) / scaleFactor;
        else
           sRadarGrid->vertProfile[i][j] = TK_DEFAULT;
        }
  sdsRead(&tempSDS, &scaleFactor, &sRadarGrid->cfadData);

  return(TK_SUCCESS);
}





/***********************************************************************************
*
*  FUNCTION STATEMENT:
*       writeGVGrid_3A_53_single(IO_HANDLE *sGranuleHandle, L3A_53_SINGLE_RADARGRID *sRadarGrid)
*
*  DESCRIPTION:
*       This function is used to write the single radar site L3A_53 information in HDF format.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       sGranuleHandle          A structure containing information about the file.
*                               sGranuleHandle is returned by TKopen().
*
*       sRadarGrid		A structure containing the complete scanline data.
*                               This structure must be declared to correspond to 
*                               the data file being written.  The structures of 
*                               sRadarGrid is manipulated by TKwriteGrid() and 
*				other toolkit routines.
*
*  OUTPUT PARAMETERS:           None
*
*  CALLED BY:                   TKwriteGrid()
*
*  ROUTINES CALLED:     
*       Routine Name            Description
*       ------------            -----------
*       allocateVdataMemory     This routine is used to allocate memory to vdata.
*       vdataWrite              This routine is used to write the vdata in HDF 
*                               format.
*       sdsWrite                This routine is used to write the scientific data
*                               set in HDF format.      
*
***********************************************************************************/

int writeGVGrid_3A_53_single(IO_HANDLE *sGranuleHandle, L3A_53_SINGLE_RADARGRID *sRadarGrid)
{
  SDS_LINK *tempSDS  = sGranuleHandle->cfgSTR->vGroup->subVgroup->sds;
  VDATA_LINK *tempVdata = sGranuleHandle->cfgSTR->vGroup->subVgroup->vdata;
  uint8 *databuf, *ptr;
  int i, j;

  for (i = 0; i < 151; ++i)
       for (j = 0; j < 151; ++j)
       {
        /* The Following code was added to take care of the Missing or
           invalid data.  The reason to add this code was not to scale
           the invalid Data */

         if (sRadarGrid->pentadRainfall[i][j] > TK_DEFAULT)
	      sRadarGrid->pentadRainfall_scale[i][j] = 
		(int16)RND(sRadarGrid->pentadRainfall[i][j]* L3A53_PENTADRAINFALL);
	 else
	      sRadarGrid->pentadRainfall_scale[i][j] = TK_MISS_I2;
       }
  sdsWrite(&tempSDS, &sRadarGrid->pentadRainfall_scale);

  return TK_SUCCESS;
}

/***********************************************************************************
*
*  FUNCTION STATEMENT:
*       readGVGrid_3A_53_single(IO_HANDLE *sGranuleHandle, 
*					L3A_53_SINGLE_RADARGRID *sRadarGrid)
*
*  DESCRIPTION:
*       This function is used to read the single radar site L3A_53 information in HDF 
*	format.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       sGranuleHandle          A structure containing information about the file.
*                               sGranuleHandle is returned by TKopen().
*
*  OUTPUT PARAMETERS:          
*       Parameter Name          Description
*       --------------          -----------
*       sRadarGrid		A structure containing the complete scanline data.
*                               This structure must be declared to correspond to 
*                               the data file being written.  The structures of 
*                               sRadarData is manipulated by TKreadGrid() and 
*				other toolkit routines.
*
*  CALLED BY:                   TKreadGrid()
*
*  ROUTINES CALLED:     
*       Routine Name            Description
*       ------------            -----------
*       allocateVdataMemory     This routine is used to allocate memory to vdata.
*       vdataWrite              This routine is used to write the vdata in HDF 
*                               format.
*       sdsWrite                This routine is used to write the scientific data
*                               set in HDF format.      
*
***********************************************************************************/

int readGVGrid_3A_53_single(IO_HANDLE *sGranuleHandle, L3A_53_SINGLE_RADARGRID *sRadarGrid)
{
  SDS_LINK *tempSDS  = sGranuleHandle->cfgSTR->vGroup->subVgroup->sds;
  VDATA_LINK *tempVdata = sGranuleHandle->cfgSTR->vGroup->subVgroup->vdata;
  uint8 *databuf, *ptr;
  int i, j;
  int scaleFactor;

  sdsRead(&tempSDS, &scaleFactor, &sRadarGrid->pentadRainfall_scale);
  for (i = 0; i < 151; ++i)
       for (j = 0; j < 151; ++j)
            {
            if (sRadarGrid->pentadRainfall_scale[i][j] != TK_MISS_I2)
               sRadarGrid->pentadRainfall[i][j]=
		(sRadarGrid->pentadRainfall_scale[i][j]*1.0)/ scaleFactor;
            else
               sRadarGrid->pentadRainfall[i][j] = TK_DEFAULT;
            }
  return TK_SUCCESS;
}

/***********************************************************************************
*
*  FUNCTION STATEMENT:
*       writeGVGrid_3A_53_TX_mult(IO_HANDLE *sGranuleHandle, 
*				L3A_53_MULT_TX_RADARGRID *sRadarGrid)
*
*  DESCRIPTION:
*       This function is used to write the multiple radar sites L3A_53 information in HDF 
*	format.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       sGranuleHandle          A structure containing information about the file.
*                               sGranuleHandle is returned by TKopen().
*
*       sRadarGrid		A structure containing the complete scanline data.
*                               This structure must be declared to correspond to 
*                               the data file being written.  The structures of 
*                               sRadarGrid is manipulated by TKwriteGrid() and 
*				other toolkit routines.
*
*  OUTPUT PARAMETERS:           None
*
*  CALLED BY:                   TKwriteGrid()
*
*  ROUTINES CALLED:     
*       Routine Name            Description
*       ------------            -----------
*       allocateVdataMemory     This routine is used to allocate memory to vdata.
*       vdataWrite              This routine is used to write the vdata in HDF 
*                               format.
*       sdsWrite                This routine is used to write the scientific data
*                               set in HDF format.      
*
***********************************************************************************/

int writeGVGrid_3A_53_TX_mult(IO_HANDLE *sGranuleHandle, L3A_53_MULT_TX_RADARGRID *sRadarGrid)
{
  SDS_LINK *tempSDS  = sGranuleHandle->cfgSTR->vGroup->subVgroup->sds;
  VDATA_LINK *tempVdata = sGranuleHandle->cfgSTR->vGroup->subVgroup->vdata;
  uint8 *databuf, *ptr;
  int i, j;


  for (i = 0; i < 285; ++i)
       for (j = 0; j < 363; ++j)
       {
        /* The Following code was added to take care of the Missing or
           invalid data.  The reason to add this code was not to scale
           the invalid Data */

         if (sRadarGrid->pentadRainfall[i][j] > TK_DEFAULT)
	      sRadarGrid->pentadRainfall_scale[i][j] = 
		(int16)RND(sRadarGrid->pentadRainfall[i][j]* L3A53_PENTADRAINFALL);
	 else
	      sRadarGrid->pentadRainfall_scale[i][j] = TK_MISS_I2;
       }
  sdsWrite(&tempSDS, &sRadarGrid->pentadRainfall_scale);

  return(TK_SUCCESS);
}


/***********************************************************************************
*
*  FUNCTION STATEMENT:
*       readGVGrid_3A_53_TX_mult(IO_HANDLE *sGranuleHandle, 
*				L3A_53_MULT_TX_RADARGRID *sRadarGrid)
*
*  DESCRIPTION:
*       This function is used to read the multiple radar sites L3A_53 information in HDF 
*	format.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       sGranuleHandle          A structure containing information about the file.
*                               sGranuleHandle is returned by TKopen().
*
*  OUTPUT PARAMETERS:     
*       Parameter Name          Description
*       --------------          -----------
*       sRadarGrid		A structure containing the complete scanline data.
*                               This structure must be declared to correspond to 
*                               the data file being written.  The structures of 
*                               sRadarData is manipulated by TKreadGrid() and 
*				other toolkit routines.
*
*  CALLED BY:                   TKreadGrid()
*
*  ROUTINES CALLED:     
*       Routine Name            Description
*       ------------            -----------
*       allocateVdataMemory     This routine is used to allocate memory to vdata.
*       vdataWrite              This routine is used to write the vdata in HDF 
*                               format.
*       sdsWrite                This routine is used to write the scientific data
*                               set in HDF format.      
*
***********************************************************************************/

int readGVGrid_3A_53_TX_mult(IO_HANDLE *sGranuleHandle, L3A_53_MULT_TX_RADARGRID *sRadarGrid)
{
  SDS_LINK *tempSDS  = sGranuleHandle->cfgSTR->vGroup->subVgroup->sds;
  VDATA_LINK *tempVdata = sGranuleHandle->cfgSTR->vGroup->subVgroup->vdata;
  uint8 *databuf, *ptr;
  int i, j;
  int scaleFactor;


  sdsRead(&tempSDS, &scaleFactor, &sRadarGrid->pentadRainfall_scale);
  for (i = 0; i < 285; ++i)
       for (j = 0; j < 363; ++j)
            {
            if (sRadarGrid->pentadRainfall_scale[i][j] != TK_MISS_I2)
               sRadarGrid->pentadRainfall[i][j]=
			(sRadarGrid->pentadRainfall_scale[i][j]*1.0)/ scaleFactor;
            else
               sRadarGrid->pentadRainfall[i][j] = TK_DEFAULT;
            }
  return(TK_SUCCESS);
}



/***********************************************************************************
*
*  FUNCTION STATEMENT:
*       writeGVGrid_3A_53_FL_mult(IO_HANDLE *sGranuleHandle, 
*                               L3A_53_MULT_FL_RADARGRID *sRadarGrid)
*
*  DESCRIPTION:
*       This function is used to write the multiple radar sites L3A_53 information in HDF 
*       format.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       sGranuleHandle          A structure containing information about the file.
*                               sGranuleHandle is returned by TKopen().
*
*       sRadarGrid              A structure containing the complete scanline data.
*                               This structure must be declared to correspond to 
*                               the data file being written.  The structures of 
*                               sRadarGrid is manipulated by TKwriteGrid() and 
*                               other toolkit routines.
*
*  OUTPUT PARAMETERS:           None
*
*  CALLED BY:                   TKwriteGrid()
*
*  ROUTINES CALLED:     
*       Routine Name            Description
*       ------------            -----------
*       allocateVdataMemory     This routine is used to allocate memory to vdata.
*       vdataWrite              This routine is used to write the vdata in HDF 
*                               format.
*       sdsWrite                This routine is used to write the scientific data
*                               set in HDF format.      
*
***********************************************************************************/

int writeGVGrid_3A_53_FL_mult(IO_HANDLE *sGranuleHandle, L3A_53_MULT_FL_RADARGRID *sRadarGrid)
{
  SDS_LINK *tempSDS  = sGranuleHandle->cfgSTR->vGroup->subVgroup->sds;
  VDATA_LINK *tempVdata = sGranuleHandle->cfgSTR->vGroup->subVgroup->vdata;
  uint8 *databuf, *ptr;
  int i, j;


  for (i = 0; i < 353; ++i)
       for (j = 0; j < 257; ++j)
       {
        /* The Following code was added to take care of the Missing or
           invalid data.  The reason to add this code was not to scale
           the invalid Data */

         if (sRadarGrid->pentadRainfall[i][j] > TK_DEFAULT)
              sRadarGrid->pentadRainfall_scale[i][j] = 
                 (int16)RND(sRadarGrid->pentadRainfall[i][j]* L3A53_PENTADRAINFALL);
	 else
              sRadarGrid->pentadRainfall_scale[i][j] = TK_MISS_I2;
       }
  sdsWrite(&tempSDS, &sRadarGrid->pentadRainfall_scale);

  return(TK_SUCCESS);
}


/***********************************************************************************
*
*  FUNCTION STATEMENT:
*       readGVGrid_3A_53_FL_mult(IO_HANDLE *sGranuleHandle, 
*                               L3A_53_MULT_FL_RADARGRID *sRadarGrid)
*
*  DESCRIPTION:
*       This function is used to write the multiple radar sites L3A_53 information in HDF 
*       format.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       sGranuleHandle          A structure containing information about the file.
*                               sGranuleHandle is returned by TKopen().
*
*  OUTPUT PARAMETERS:     
*       Parameter Name          Description
*       --------------          -----------
*       sRadarGrid              A structure containing the complete scanline data.
*                               This structure must be declared to correspond to 
*                               the data file being written.  The structures of 
*                               sRadarData is manipulated by TKreadGrid() and 
*                               other toolkit routines.
*
*  CALLED BY:                   TKreadGrid()
*
*  ROUTINES CALLED:     
*       Routine Name            Description
*       ------------            -----------
*       allocateVdataMemory     This routine is used to allocate memory to vdata.
*       vdataWrite              This routine is used to write the vdata in HDF 
*                               format.
*       sdsWrite                This routine is used to write the scientific data
*                               set in HDF format.      
*
***********************************************************************************/

int readGVGrid_3A_53_FL_mult(IO_HANDLE *sGranuleHandle, L3A_53_MULT_FL_RADARGRID *sRadarGrid)
{
  SDS_LINK *tempSDS  = sGranuleHandle->cfgSTR->vGroup->subVgroup->sds;
  VDATA_LINK *tempVdata = sGranuleHandle->cfgSTR->vGroup->subVgroup->vdata;
  uint8 *databuf, *ptr;
  int i, j;
  int scaleFactor;


  sdsRead(&tempSDS, &scaleFactor, &sRadarGrid->pentadRainfall_scale);
  for (i = 0; i < 353; ++i)
       for (j = 0; j < 257; ++j)
            {
            if (sRadarGrid->pentadRainfall_scale[i][j] != TK_MISS_I2)
               sRadarGrid->pentadRainfall[i][j]=
                        (sRadarGrid->pentadRainfall_scale[i][j]*1.0)/ scaleFactor;
            else
               sRadarGrid->pentadRainfall[i][j] = TK_DEFAULT;
            }
  return(TK_SUCCESS);
}



/***********************************************************************************
*
*  FUNCTION STATEMENT:
*       writeGVGrid_3A_54_single(IO_HANDLE *sGranuleHandle, 
*					L3A_54_SINGLE_RADARGRID *sRadarGrid)
*
*  DESCRIPTION:
*       This function is used to write the single radar site L3A_54 information in HDF 
*	format.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       sGranuleHandle          A structure containing information about the file.
*                               sGranuleHandle is returned by TKopen().
*
*       sRadarGrid		A structure containing the complete scanline data.
*                               This structure must be declared to correspond to 
*                               the data file being written.  The structures of 
*                               sRadarGrid is manipulated by TKwriteGrid() and 
*				other toolkit routines.
*
*  OUTPUT PARAMETERS:           None
*
*  CALLED BY:                   TKwriteGrid()
*
*  ROUTINES CALLED:     
*       Routine Name            Description
*       ------------            -----------
*       allocateVdataMemory     This routine is used to allocate memory to vdata.
*       vdataWrite              This routine is used to write the vdata in HDF 
*                               format.
*       sdsWrite                This routine is used to write the scientific data
*                               set in HDF format.      
*
***********************************************************************************/

int writeGVGrid_3A_54_single(IO_HANDLE *sGranuleHandle, 
				L3A_54_SINGLE_RADARGRID *sRadarGrid)
{
  SDS_LINK *tempSDS  = sGranuleHandle->cfgSTR->vGroup->subVgroup->sds;
  VDATA_LINK *tempVdata = sGranuleHandle->cfgSTR->vGroup->subVgroup->vdata;
  uint8 *databuf, *ptr;
  int i, j;


  sdsWrite(&tempSDS, &sRadarGrid->monthlyRainfall);

  return(TK_SUCCESS);
}

/***********************************************************************************
*
*  FUNCTION STATEMENT:
*       readGVGrid_3A_54_single(IO_HANDLE *sGranuleHandle, L3A_54_SINGLE_RADARGRID *sRadarGrid)
*
*  DESCRIPTION:
*       This function is used to read the single radar site L3A_54 information in HDF format.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       sGranuleHandle          A structure containing information about the file.
*                               sGranuleHandle is returned by TKopen().
*
*  OUTPUT PARAMETERS: 
*       Parameter Name          Description
*       --------------          -----------
*       sRadarGrid		A structure containing the complete scanline data.
*                               This structure must be declared to correspond to 
*                               the data file being written.  The structures of 
*                               sRadarGrid is manipulated by TKreadGrid() and 
*				other toolkit routines.
*
*  CALLED BY:                   TKreadGrid()
*
*  ROUTINES CALLED:     
*       Routine Name            Description
*       ------------            -----------
*       allocateVdataMemory     This routine is used to allocate memory to vdata.
*       vdataWrite              This routine is used to write the vdata in HDF 
*                               format.
*       sdsWrite                This routine is used to write the scientific data
*                               set in HDF format.      
*
***********************************************************************************/

int readGVGrid_3A_54_single(IO_HANDLE *sGranuleHandle, L3A_54_SINGLE_RADARGRID *sRadarGrid)
{
  SDS_LINK *tempSDS  = sGranuleHandle->cfgSTR->vGroup->subVgroup->sds;
  VDATA_LINK *tempVdata = sGranuleHandle->cfgSTR->vGroup->subVgroup->vdata;
  uint8 *databuf, *ptr;
  int i, j;
  int scaleFactor;


  sdsRead(&tempSDS, &scaleFactor, &sRadarGrid->monthlyRainfall);
  
  return(TK_SUCCESS);
}


/***********************************************************************************
*
*  FUNCTION STATEMENT:
*       writeGVGrid_3A_54_TX_mult(IO_HANDLE *sGranuleHandle, 
*					L3A_54_MULT_TX_RADARGRID *sRadarGrid)
*
*  DESCRIPTION:
*       This function is used to write the multiple radar sites L3A_54 information in HDF format.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       sGranuleHandle          A structure containing information about the file.
*                               sGranuleHandle is returned by TKopen().
*
*       sRadarGrid		A structure containing the complete scanline data.
*                               This structure must be declared to correspond to 
*                               the data file being written.  The structures of 
*                               sRadarGrid is manipulated by TKwriteGrid() and 
*				other toolkit routines.
*
*  OUTPUT PARAMETERS:           None
*
*  CALLED BY:                   TKwriteGrid()
*
*  ROUTINES CALLED:     
*       Routine Name            Description
*       ------------            -----------
*       allocateVdataMemory     This routine is used to allocate memory to vdata.
*       vdataWrite              This routine is used to write the vdata in HDF 
*                               format.
*       sdsWrite                This routine is used to write the scientific data
*                               set in HDF format.      
*
***********************************************************************************/

int writeGVGrid_3A_54_TX_mult(IO_HANDLE *sGranuleHandle, L3A_54_MULT_TX_RADARGRID *sRadarGrid)
{
  SDS_LINK *tempSDS  = sGranuleHandle->cfgSTR->vGroup->subVgroup->sds;
  VDATA_LINK *tempVdata = sGranuleHandle->cfgSTR->vGroup->subVgroup->vdata;
  uint8 *databuf, *ptr;
  int i, j;


  sdsWrite(&tempSDS, &sRadarGrid->monthlyRainfall);

  return(TK_SUCCESS);
}

/***********************************************************************************
*
*  FUNCTION STATEMENT:
*       readGVGrid_3A_54_TX_mult(IO_HANDLE *sGranuleHandle, L3A_54_MULT_TX_RADARGRID *sRadarGrid)
*
*  DESCRIPTION:
*       This function is used to read the multiple radar sites L3A_54 information in HDF format.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       sGranuleHandle          A structure containing information about the file.
*                               sGranuleHandle is returned by TKopen().
*
*  OUTPUT PARAMETERS:           
*       Parameter Name          Description
*       --------------          -----------
*       sRadarGrid		A structure containing the complete scanline data.
*                               This structure must be declared to correspond to 
*                               the data file being written.  The structures of 
*                               sRadarGrid is manipulated by TKreadGrid() and 
*				other toolkit routines.
*
*  CALLED BY:                   TKreadGrid()
*
*  ROUTINES CALLED:     
*       Routine Name            Description
*       ------------            -----------
*       allocateVdataMemory     This routine is used to allocate memory to vdata.
*       vdataWrite              This routine is used to write the vdata in HDF 
*                               format.
*       sdsWrite                This routine is used to write the scientific data
*                               set in HDF format.      
*
***********************************************************************************/

int readGVGrid_3A_54_TX_mult(IO_HANDLE *sGranuleHandle, L3A_54_MULT_TX_RADARGRID *sRadarGrid)
{
  SDS_LINK *tempSDS  = sGranuleHandle->cfgSTR->vGroup->subVgroup->sds;
  VDATA_LINK *tempVdata = sGranuleHandle->cfgSTR->vGroup->subVgroup->vdata;
  uint8 *databuf, *ptr;
  int i, j;
  int scaleFactor;


  sdsRead(&tempSDS, &scaleFactor, &sRadarGrid->monthlyRainfall);

  return(TK_SUCCESS);
}



/***********************************************************************************
*
*  FUNCTTION STATEMENT:
*       writeGVGrid_3A_54_FL_mult(IO_HANDLE *sGranuleHandle, 
*                                       L3A_54_MULT_FL_RADARGRID *sRadarGrid)
*
*  DESCRIPTION:
*       This function is used to write the multiple radar sites L3A_54 information in HDF format.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       sGranuleHandle          A structure containing information about the file.
*                               sGranuleHandle is returned by TKopen().
*
*       sRadarGrid              A structure containing the complete scanline data.
*                               This structure must be declared to correspond to 
*                               the data file being written.  The structures of 
*                               sRadarGrid is manipulated by TKwriteGrid() and 
*                               other toolkit routines.
*
*  OUTPUT PARAMETERS:           None
*
*  CALLED BY:                   TKwriteGrid()
*
*  ROUTINES CALLED:     
*       Routine Name            Description
*       ------------            -----------
*       allocateVdataMemory     This routine is used to allocate memory to vdata.
*       vdataWrite              This routine is used to write the vdata in HDF 
*                               format.
*       sdsWrite                This routine is used to write the scientific data
*                               set in HDF format.      
*
***********************************************************************************/

int writeGVGrid_3A_54_FL_mult(IO_HANDLE *sGranuleHandle, L3A_54_MULT_FL_RADARGRID *sRadarGrid)
{
  SDS_LINK *tempSDS  = sGranuleHandle->cfgSTR->vGroup->subVgroup->sds;
  VDATA_LINK *tempVdata = sGranuleHandle->cfgSTR->vGroup->subVgroup->vdata;
  uint8 *databuf, *ptr;
  int i, j;


  sdsWrite(&tempSDS, &sRadarGrid->monthlyRainfall);

  return(TK_SUCCESS);
}

/***********************************************************************************
*
*  FUNCTION STATEMENT:
*       readGVGrid_3A_54_FL_mult(IO_HANDLE *sGranuleHandle, 
*				    L3A_54_MULT_FL_RADARGRID *sRadarGrid)
*
*  DESCRIPTION:
*       This function is used to read the multiple radar sites L3A_54 information in HDF format.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       sGranuleHandle          A structure containing information about the file.
*                               sGranuleHandle is returned by TKopen().
*
*  OUTPUT PARAMETERS:           
*       Parameter Name          Description
*       --------------          -----------
*       sRadarGrid              A structure containing the complete scanline data.
*                               This structure must be declared to correspond to 
*                               the data file being written.  The structures of 
*                               sRadarGrid is manipulated by TKreadGrid() and 
*                               other toolkit routines.
*
*  CALLED BY:                   TKreadGrid()
*
*  ROUTINES CALLED:     
*       Routine Name            Description
*       ------------            -----------
*       allocateVdataMemory     This routine is used to allocate memory to vdata.
*       vdataWrite              This routine is used to write the vdata in HDF 
*                               format.
*       sdsWrite                This routine is used to write the scientific data
*                               set in HDF format.      
*
***********************************************************************************/

int readGVGrid_3A_54_FL_mult(IO_HANDLE *sGranuleHandle, L3A_54_MULT_FL_RADARGRID *sRadarGrid)
{
  SDS_LINK *tempSDS  = sGranuleHandle->cfgSTR->vGroup->subVgroup->sds;
  VDATA_LINK *tempVdata = sGranuleHandle->cfgSTR->vGroup->subVgroup->vdata;
  uint8 *databuf, *ptr;
  int i, j;
  int scaleFactor;


  sdsRead(&tempSDS, &scaleFactor, &sRadarGrid->monthlyRainfall);

  return(TK_SUCCESS);
}




/***********************************************************************************
*
*  FUNCTION STATEMENT:
*       writeGVGrid_3A_55(IO_HANDLE *sGranuleHandle, L3A_55_RADARGRID *sRadarGrid)
*
*  DESCRIPTION:
*       This function is used to write the L3A_55 information in HDF format.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       sGranuleHandle          A structure containing information about the file.
*                               sGranuleHandle is returned by TKopen().
*
*       sRadarGrid		A structure containing the complete scanline data.
*                               This structure must be declared to correspond to 
*                               the data file being written.  The structures of 
*                               sRadarGrid is manipulated by TKwriteGrid() and 
*				other toolkit routines.
*
*  OUTPUT PARAMETERS:           None
*
*  CALLED BY:                   TKwriteGrid()
*
*  ROUTINES CALLED:     
*       Routine Name            Description
*       ------------            -----------
*       allocateVdataMemory     This routine is used to allocate memory to vdata.
*       vdataWrite              This routine is used to write the vdata in HDF 
*                               format.
*       sdsWrite                This routine is used to write the scientific data
*                               set in HDF format.      
*
***********************************************************************************/

int writeGVGrid_3A_55(IO_HANDLE *sGranuleHandle, L3A_55_RADARGRID *sRadarGrid)
{
  SDS_LINK *tempSDS  = sGranuleHandle->cfgSTR->vGroup->subVgroup->sds;
  VDATA_LINK *tempVdata = sGranuleHandle->cfgSTR->vGroup->subVgroup->vdata;
  uint8 *databuf, *ptr;
  int i, j;


  for (i = 0; i < 13; ++i)
       for (j = 0; j < 12; ++ j)
       {
        /* The Following code was added to take care of the Missing or
           invalid data.  The reason to add this code was not to scale
           the invalid Data */

         if(sRadarGrid->vertProfile[i][j] > TK_DEFAULT)
	      sRadarGrid->vertProfile_scale[i][j] = 
		(int16)RND(sRadarGrid->vertProfile[i][j] * L3A55_VERTICALPROFILE);
	 else
	      sRadarGrid->vertProfile_scale[i][j] = TK_MISS_I2;
       }
  sdsWrite(&tempSDS, &sRadarGrid->vertProfile_scale);


  sdsWrite(&tempSDS, &sRadarGrid->cfadData);

  return TK_SUCCESS;
}

/***********************************************************************************
*
*  FUNCTION STATEMENT:
*       readGVGrid_3A_55(IO_HANDLE *sGranuleHandle, L3A_55_RADARGRID *sRadarGrid) 
*
*  DESCRIPTION:
*       This function is used to read the L3A_55 information in HDF format.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       sGranuleHandle          A structure containing information about the file.
*                               sGranuleHandle is returned by TKopen().
*
*  OUTPUT PARAMETERS:           
*       Parameter Name          Description
*       --------------          -----------
*       sRadarGrid		A structure containing the complete scanline data.
*                               This structure must be declared to correspond to 
*                               the data file being written.  The structures of 
*                               sRadarGrid is manipulated by TKreadGrid() and 
*				other toolkit routines.
*
*  CALLED BY:                   TKreadGrid()
*
*  ROUTINES CALLED:     
*       Routine Name            Description
*       ------------            -----------
*       allocateVdataMemory     This routine is used to allocate memory to vdata.
*       vdataWrite              This routine is used to write the vdata in HDF 
*                               format.
*       sdsWrite                This routine is used to write the scientific data
*                               set in HDF format.      
*
***********************************************************************************/

int readGVGrid_3A_55(IO_HANDLE *sGranuleHandle, L3A_55_RADARGRID *sRadarGrid)
{
  SDS_LINK *tempSDS  = sGranuleHandle->cfgSTR->vGroup->subVgroup->sds;
  VDATA_LINK *tempVdata = sGranuleHandle->cfgSTR->vGroup->subVgroup->vdata;
  uint8 *databuf, *ptr;
  int	i, j;
  int scaleFactor;

  sdsRead(&tempSDS, &scaleFactor, &sRadarGrid->vertProfile_scale);
  for (i = 0; i < 13; ++i)
       for (j = 0; j < 12; ++ j)
            {
            if (sRadarGrid->vertProfile_scale[i][j] != TK_MISS_I2)
	       sRadarGrid->vertProfile[i][j] = 
			(sRadarGrid->vertProfile_scale[i][j]*1.0)/ scaleFactor; 
            else
               sRadarGrid->vertProfile[i][j] = TK_DEFAULT;
            }
  sdsRead(&tempSDS, &scaleFactor, &sRadarGrid->cfadData);

  return TK_SUCCESS;
}


/***********************************************************************************
*
*  FUNCTION STATEMENT:
*       writeGVGrid_3A_46(IO_HANDLE *sGranuleHandle, L3A_46_PLANETGRID *sGrid) 
*
*  DESCRIPTION:
*       This function is used to write the L3A-46 information in HDF format.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       sGranuleHandle          A structure containing information about the file.
*                               sGranuleHandle is returned by TKopen().
*
*       sGrid			A structure containing the complete scanline data.
*                               This structure must be declared to correspond to 
*                               the data file being written.  The structure of 
*                               sGrid is manipulated by TKwriteGrid() and other 
*                               toolkit routines.
*
*  OUTPUT PARAMETERS:		None
*
*  CALLED BY:                   TKwriteGrid()
*
*  ROUTINES CALLED:     
*       Routine Name            Description
*       ------------            -----------
*       allocateVdataMemory     This routine is used to allocate memory to vdata.
*       vdataWrite              This routine is used to write the vdata in HDF 
*                               format.
*       sdsWrite                This routine is used to write the scientific data
*                               set in HDF format.      
*
***********************************************************************************/
int writeGVGrid_3A_46(IO_HANDLE *sGranuleHandle, L3A_46_PLANETGRID *sGrid)
{
  SDS_LINK *tempSDS = sGranuleHandle->cfgSTR->vGroup->subVgroup->sds;
  VDATA_LINK *tempVdata = sGranuleHandle->cfgSTR->vGroup->subVgroup->vdata;
  uint8 *databuf, *ptr;

  sdsWrite(&tempSDS, &sGrid->ssmiData);

  return(TK_SUCCESS);

}


/***********************************************************************************
*
*  FUNCTION STATEMENT:
*       readGVGrid_3A_46(IO_HANDLE *sGranuleHandle, L3A_46_PLANETGRID *sGrid)
*
*  DESCRIPTION:
*       This function is used to read the L3A_46 information in HDF format.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       sGranuleHandle          A structure containing information about the file.
*                               sGranuleHandle is returned by TKopen().
*
*  OUTPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       sGrid                   A structure containing the complete grid data.
*                               This structure must be declared to correspond to
*                               the data file to be obtained.  The structure of
*                               sGrid is manipulated by TKreadGrid() and
*                               other toolkit routines.
*
*  CALLED BY:                   TKreadGrid()
*
*  ROUTINES CALLED:
*	allocateVdataMemory(), sdsRead(), vdataRead()
*
***********************************************************************************/

int readGVGrid_3A_46(IO_HANDLE *sGranuleHandle, L3A_46_PLANETGRID *sGrid)
{
  SDS_LINK *tempSDS = sGranuleHandle->cfgSTR->vGroup->subVgroup->sds;
  VDATA_LINK *tempVdata = sGranuleHandle->cfgSTR->vGroup->subVgroup->vdata;
  uint8 *databuf, *ptr;
  int scaleFactor;

  /* Read SDS from a HDF file */
  sdsRead(&tempSDS, &scaleFactor, &sGrid->ssmiData);


  return(TK_SUCCESS);
}



/*****************************************************************************
*
*  FUNCTION STATEMENT:
*       findTemplateNode(int32 inputFile, L1_GV_TEMPLATE *template)
*
*  DESCRIPTION:
*       This routine finds the location of inputFile from the L1 GV template
*	list and returns that postion to the caller.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*	inputFile		Identification number of the input file.  This
*				inputFile must exist in the template list.
*				If it does not exist int the list, it will
*				echo error message.
*	template		A list of L1 GV templates.
*
*  OUTPUT PARAMETERS:		None.
*
*  CALLED BY:  			getNvos(), getNparm(), getNcell(),
*				getNray(), and getNsweep().
*
*  RETURN VALUES:               A template which has matching inputFile.
*
*  ROUTINES CALLED:
*       Routine Name            Description
*       ------------            -----------
*       TKreportError		This routine reports fatal errors to the error 
*				log file and terminates all processes.
*
******************************************************************************/

L1_GV_TEMPLATE *findTemplateNode(int32 inputFile, L1_GV_TEMPLATE *template)
{
  L1_GV_TEMPLATE *tempTemplate = template;

  /*
  ** Search for a template node which has matching inputFile field with input
  ** inputFile.
  */ 
  while (tempTemplate != NULL)
  {
         if (tempTemplate->inputFile == inputFile)
         {
             return (tempTemplate);
         } 
         else
	 {
             tempTemplate = tempTemplate->next;
         }
  }

  /*
  ** At this point, tempTemplate is null which means that the searching 
  ** node does not exist on template list.
  */ 
  /* Searcing template failed in findTemplateNode. */
  TKreportError(E_TK_FAILSEARCHFTN);
}



/*****************************************************************************
*
*  FUNCTION STATEMENT:
*	 setL1GVtemplate(int32 nvos, int32 nparm[MAX_VOS], 
*			 int32 ncell[MAX_VOS][MAX_PARM], int32 nray[MAX_VOS],
*			 int32 nsweep[MAX_VOS], char *filename, int32 inputFile)
*
*  DESCRIPTION:
*	This routines is used to create a template which is initialized with input data.
*	It calls manageL1GVtemplate in order to append it into the list of templates.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       nvos                    Number of volume scans.
*       nparm                   Number of parameters for each volume scan.
*       ncell                   Number of cells for each parameters and for each 
*				volume scans.
*       nray                    Number of rays for each volume scans.
*       nsweep                  Number of sweeps for each volume scans.
*       fileName                HDF file name where L1 GV data will be stored.
*	inputFile		Input file ID.
*
*  OUTPUT PARAMETERS:           None.
*
*  CALLED BY:			openInputVgroupL1GV() 
*
*  RETURN VALUES:               TK_SUCCESS if setL1GVtemplate is successful.
*                               TK_FAIL if setL1GVtemplate is failed.
*
*  ROUTINES CALLED:
*       Routine Name            Description
*       ------------            -----------
*	manageL1GVtemplate	This routines performs a requested action.  The 
*				requestion is described by opType.
*
******************************************************************************/

int32 setL1GVtemplate(int32 nvos, int32 nparm[MAX_VOS], 
		      int32 ncell[MAX_VOS][MAX_PARM], int32 nray[MAX_VOS], 
		      int32 nsweep[MAX_VOS], char *filename, int32 inputFile)
{
  L1_GV_TEMPLATE *template = (L1_GV_TEMPLATE *) malloc(sizeof(L1_GV_TEMPLATE));
  int32 i, j;

  if (template == NULL)
	/* Memory allocation failed in setL1GVtemplate(). */
	TKreportError(E_TK_MEMFAILL1GVTEMP);

  /*
  ** Initialize the template  node with input data.
  */
  template->inputFile = inputFile;
  template->numOfVOS  =  nvos;
  strcpy(template->filename, filename);
  template->next = NULL;
  for (i = 0; i < nvos; ++i)
  {
       template->numOfParm[i] = nparm[i];
       template->nray[i] = nray[i];
       template->nsweep[i] = nsweep[i];
       for (j = 0; j < nparm[i]; ++j)
       {
	    template->ncell[i][j] = ncell[i][j];
       }
  }

  /*
  ** Call manageL1GVtemplate in order to add the template into the internal
  ** template lists.
  */
  return (manageL1GVtemplate(ADD_L1GV_TEMPLATE, inputFile, filename, template,
          -1, -1));
}




/*****************************************************************************
*
*  FUNCTION STATEMENT:
*	setInputFile(L1_GV_TEMPLATE *template, int32 inputFile, char *filename)
*
*  DESCRIPTION:
*	This routines searches a template for a matching filename and intializes
*	the template's inputFile with the input inputFile.	
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*	template		The list of Level 1 GV template nodes.
*	inputFile		InputFile ID.
*	filename		Filename which is used to search a L1 GV template
*				node from the list of templates.
*
*  OUTPUT PARAMETERS:           None.
*
*  CALLED BY:                   manageL1GVtemplate().
*
*  RETURN VALUES:               TK_SUCCESS if seInputFile is successful.
*
*  ROUTINES CALLED:
*       Routine Name            Description
*       ------------            -----------
*       TKreportError           This routine reports fatal errors to the error log
*                               file and terminates all processes.
*
******************************************************************************/

int32 setInputFile(L1_GV_TEMPLATE *template, int32 inputFile, char *filename)
{
  L1_GV_TEMPLATE *tempTemplate = template;

  /*
  ** Search for the template node from the list of templates using filename.
  */
  while (tempTemplate != NULL)
  {
         if (strcmp(tempTemplate->filename, filename) == 0)
	 {
	     break;
         }
	 else
	 {
	     tempTemplate = tempTemplate->next;
	 }
  }

  /* 
  ** If the template node is found, then initialize its field inputFile
  ** with input inputFile.
  */
  if (tempTemplate != NULL)
  {
      tempTemplate->inputFile = inputFile;
  }
  else
  {
      /* Invalid template in setInputFile. */
      TKreportError(E_TK_BADTEMPSIF);
  }

  return (TK_SUCCESS);
}



/*****************************************************************************
*
*  FUNCTION STATEMENT:
*        getNvos(L1_GV_TEMPLATE *template, int32 inputFile)
*
*  DESCRIPTION:
*       This routines returns the number of volume scans.  Inputfile is used
*	to search a L1 GV template node from the input template.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*	template		List of Level 1 GV template nodes.
*	inputFile		InputFile which is used to search a matching
*				inputFile field from the list of templates.
*
*  OUTPUT PARAMETERS:           None.
*
*  CALLED BY:                   manageL1GVtemplate()
*
*  RETURN VALUES:               The number of volume scans.
*
*  ROUTINES CALLED:
*       Routine Name            Description
*       ------------            -----------
*       findTemplateNode	This routine finds the location of inputFile 
*				from the L1 GV template list and returns that 
*				postion to the caller.
*
******************************************************************************/

int32 getNvos(L1_GV_TEMPLATE *template, int32 inputFile)
{
  L1_GV_TEMPLATE *tempTemplate;

  /*
  ** Search the template node using inputFile.  This template node contains
  ** the number of volume scans. 
  */ 
  tempTemplate =  findTemplateNode(inputFile, template);

  return (tempTemplate->numOfVOS);
}



/*****************************************************************************
*
*  FUNCTION STATEMENT:
*	getNparm(L1_GV_TEMPLATE *template, int32 inputFile, int32 vosNumber)
*
*  DESCRIPTION:
*       This routines returns the number of parameters for the given volume scan
*	number.  Inputfile is used to search a L1 GV template node from the input
*	template.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       template                List of Level 1 GV template nodes.
*       inputFile               InputFile which is used to search a matching
*                               inputFile field from the list of templates.
*	vosNumber		Volume scan number.
*
*  OUTPUT PARAMETERS:           None.
*
*  CALLED BY:                   manageL1GVtemplate()
*
*  RETURN VALUES:               The number of parameters for the given volume 
*				scan number.
*
*  ROUTINES CALLED:
*       Routine Name            Description
*       ------------            -----------
*       findTemplateNode        This routine finds the location of inputFile 
*                               from the L1 GV template list and returns that 
*                               postion to the caller.
*
******************************************************************************/

int32 getNparm(L1_GV_TEMPLATE *template, int32 inputFile, int32 vosNumber)
{
  L1_GV_TEMPLATE *tempTemplate;

  /*
  ** Search the template node using inputFile.  This template node contains
  ** the number of parameters. 
  */
  tempTemplate =  findTemplateNode(inputFile, template);

  /*
  ** Return the number of parameters for the given volume scan number.
  */
  return (tempTemplate->numOfParm[vosNumber]);
}



/*****************************************************************************
*
*  FUNCTION STATEMENT:
*	getNcell(int32 inputFile,L1_GV_TEMPLATE *template,int32 vosNumber,
*		 int32 parmNumber)
*
*  DESCRIPTION:
*       This routines returns the number of cells for the given volume scan
*       number and parameter number.  Inputfile is used to search a L1 GV template
*	node from the input template.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       template                List of Level 1 GV template nodes.
*       inputFile               InputFile which is used to search a matching
*                               inputFile field from the list of templates.
*       vosNumber              The volume scan number.
*	parmNumber		The parameter number for the given volume scan number.
*
*  OUTPUT PARAMETERS:           None.
*
*  CALLED BY:                   manageL1GVtemplate()
*
*  RETURN VALUES:               The number of cells for the given volume 
*                               scan number and the given parameter number.
*
*  ROUTINES CALLED:
*       Routine Name            Description
*       ------------            -----------
*       findTemplateNode        This routine finds the location of inputFile 
*                               from the L1 GV template list and returns that 
*                               postion to the caller.
*
******************************************************************************/

int32 getNcell(int32 inputFile,L1_GV_TEMPLATE *template,int32 vosNumber,
	       int32 parmNumber)
{
  L1_GV_TEMPLATE *tempTemplate;

  /*
  ** Search the template node using inputFile.  This template node contains
  ** the number of cells. 
  */
  tempTemplate =  findTemplateNode(inputFile, template);

  /*
  ** Return the number of cells for the given volume scan number and parameter 
  ** number.
  */
  return (tempTemplate->ncell[vosNumber][parmNumber]);
}



/*****************************************************************************
*
*  FUNCTION STATEMENT:
*	getNray(int32 inputFile, L1_GV_TEMPLATE *template, int32 vosNumber)
*
*  DESCRIPTION:
*       This routines returns the number of rays for the given volume scan
*       number.  Inputfile is used to search a L1 GV template node from 
*       the input template.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       template                List of Level 1 GV template nodes.
*       inputFile               InputFile which is used to search a matching
*                               inputFile field from the list of templates.
*       vosNumber              The volume scan number.
*
*  OUTPUT PARAMETERS:           None.
*
*  CALLED BY:                   manageL1GVtemplate()
*
*  RETURN VALUES:               The number of rays  for the given volume
*                               scan number.
*
*  ROUTINES CALLED:
*       Routine Name            Description
*       ------------            -----------
*       findTemplateNode        This routine finds the location of inputFile
*                               from the L1 GV template list and returns that
*                               postion to the caller.
*
******************************************************************************/

int32 getNray(int32 inputFile, L1_GV_TEMPLATE *template, int32 vosNumber)
{
  L1_GV_TEMPLATE *tempTemplate;

  /*
  ** Search the template node using inputFile.  This template node contains
  ** the number of rays. 
  */
  tempTemplate =  findTemplateNode(inputFile, template);

  /*
  ** Return the number of rays for the given volume scan number.
  */
  return (tempTemplate->nray[vosNumber]);
}



/*****************************************************************************
*
*  FUNCTION STATEMENT:
*	getNsweep(int32 inputFile, L1_GV_TEMPLATE *template, int32 vosNumber)
*
*  DESCRIPTION:
*       This routines returns the number of sweeps for the given volume scan
*       number.  Inputfile is used to search a L1 GV template node from 
*       the input template.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       template                List of Level 1 GV template nodes.
*       inputFile               InputFile which is used to search a matching
*                               inputFile field from the list of templates.
*       vosNumber              The volume scan number.
*
*  OUTPUT PARAMETERS:           None.
*
*  CALLED BY:                   manageL1GVtemplate()
*
*  RETURN VALUES:               The number of cells  for the given volume
*                               scan number.
*
*  ROUTINES CALLED:
*       Routine Name            Description
*       ------------            -----------
*       findTemplateNode        This routine finds the location of inputFile
*                               from the L1 GV template list and returns that
*                               postion to the caller.
*
******************************************************************************/

int32 getNsweep(int32 inputFile, L1_GV_TEMPLATE *template, int32 vosNumber)
{
  L1_GV_TEMPLATE *tempTemplate;

  /*
  ** Search the template node using inputFile.  This template node contains
  ** the number of sweeps. 
  */
  tempTemplate =  findTemplateNode(inputFile, template);

  /*
  ** Return the number of sweeps for the given volume scan number.
  */
  return (tempTemplate->nsweep[vosNumber]);
}




/*****************************************************************************
*
*  FUNCTION STATEMENT:
*	addTemplate(L1_GV_TEMPLATE **templateList, L1_GV_TEMPLATE *template)
*
*  DESCRIPTION:
*       This routines inserts the template into the templateList.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       templateList		The list of templates.
*	template		The template to be inserted into the templateList.
*
*  OUTPUT PARAMETERS:           None.
*
*  CALLED BY:                   manageL1GVtemplate()
*
*  RETURN VALUES:               TK_SUCCESS if the routines is successful.
*
*  ROUTINES CALLED:		Node
*
******************************************************************************/

int32 addTemplate(L1_GV_TEMPLATE **templateList, L1_GV_TEMPLATE *template)
{
  L1_GV_TEMPLATE *tempList = *templateList; 

  /* 
  ** Insert template into the list of templates.  The template will be inserted
  ** into the beginning of the list.
  */
  template->next = tempList;
  tempList = template;

  *templateList = tempList;

  return (TK_SUCCESS);
}



/*****************************************************************************
*
*  FUNCTION STATEMENT:
*	deleteTemplate(L1_GV_TEMPLATE **templateList, int32 inputFile)
*
*  DESCRIPTION:
*       This routines searches a template which contains matching inputFile field
*	with the input InputFile from the templateList, and deletes it from the 
*	templateList.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       templateList            The list of templates.
*       inputFile		InputFile is used to search a template which has 
*				matching inputFile from the templateList.
*
*  OUTPUT PARAMETERS:           None.
*
*  CALLED BY:                   manageL1GVtemplate()
*
*  RETURN VALUES:               TK_SUCCESS if the routines is successful.
*
*  ROUTINES CALLED:             
*       Routine Name            Description
*       ------------            -----------
*       TKreportError           This routine reports fatal errors to the error log
*                               file and terminates all processes.
*
******************************************************************************/

int32 deleteTemplate(L1_GV_TEMPLATE **templateList, int32 inputFile)
{
  L1_GV_TEMPLATE *tempList1 = *templateList;
  L1_GV_TEMPLATE *tempList2 = *templateList;

  /*
  ** Search for the template which needs to be deleted from the list using
  ** inputFile. IF the template is found, then exit out of the loop.
  */
  while (tempList1 != NULL)
  {
         if (tempList1->inputFile == inputFile)
         {
	     break;
	 }
	 else
	 {
	     tempList2 = tempList1;
	     tempList1 = tempList1->next;
	 }
  }

  /*
  ** IF the searching template is found, then delete from the list.
  */
  if (tempList1 !=  NULL)
  {
      /*
      ** If the template is located at the beginning of the list, then
      ** assign the templateList with the next of the list.
      */
      if (tempList1->inputFile == tempList2->inputFile)
      {
	  *templateList = tempList1->next;
      }
      else
      {
	  /* If the template is not located at the beginning of te list,
	  ** then adjust the list so that the list will not include the
	  ** the template any more.
	  */
	  tempList2->next = tempList1->next;
      }
      /* 
      ** Deallocate the memory for the template.
      */
      free(tempList1);
  }
  /*
  ** If the template does not exist in the list, then error out.
  */
  else
  {
      /* Invalid inputFile in deleteTemplate. */
      TKreportError(E_TK_BADINPUTDT);
  }

  return (TK_SUCCESS);
}



/*****************************************************************************
*
*  FUNCTION STATEMENT:
*	manageL1GVtemplate(int32 opType, int32 inputFile, char *filename,
*			   L1_GV_TEMPLATE *template, int32 vosNumber, int32 parmNumber)
*
*  DESCRIPTION:
*       This routines performs a requested action.  The request is described by
*	opType.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*	opType			Operation type.
*	inputFile		Inputfile.
*	filename		Filename.
*	template		L1 GV template node.
*	vosNumber		The volume scan number.
*	parmNumber		The parameter number for the given volume scan number.
*
*  OUTPUT PARAMETERS:           None.
*
*  CALLED BY:                   setL1GVtemplate(), TKgetNumOf(), TKgetNparm(),
*				TKgetNcell(), TKgetNray(), TKgetNsweep(),
*				makeL1GVconfigFile(), TKopen, and TKclose().
*
*  RETURN VALUES:               TK_SUCCESS if opType is either ADD_L1GV_TEMPLATE,
*				DELETE_L1GV_TEMPLATE, or SET_L1GV_INPUTFILE.
*				Number of volume scans if opType is GET_L1GV_NUM_OF_VOS.
*				Number of parameters if opeType is GET_L1GV_NUM_OF_PARM.
*				Number of cells if opType is GET_L1GV_NCELL.
*				Number of rays if GET_L1GV_NRAY.
*				Number of sweeps if GET_L1GV_NSWEEP.
*
*  ROUTINES CALLED:
*       Routine Name            Description
*       ------------            -----------
*       TKreportError           This routine reports fatal errors to the error log
*                               file and terminates all processes.
*
******************************************************************************/

int32 manageL1GVtemplate(int32 opType, int32 inputFile, char *filename, 
			 L1_GV_TEMPLATE *template, int32 vosNumber, int32 parmNumber)
{
  static L1_GV_TEMPLATE *templateList = NULL;

  switch(opType)
  {
     case ADD_L1GV_TEMPLATE		:
	return (addTemplate(&templateList, template));

     case DELETE_L1GV_TEMPLATE	:
	return (deleteTemplate(&templateList, inputFile));

     case SET_L1GV_INPUTFILE	:
	return (setInputFile(templateList, inputFile, filename));

     case GET_L1GV_NUM_OF_VOS	:
 	return (getNvos(templateList, inputFile));

     case GET_L1GV_NUM_OF_PARM	:
 	return (getNparm(templateList, inputFile, vosNumber));

     case GET_L1GV_NCELL		:
	return (getNcell(inputFile, templateList, vosNumber, parmNumber));

     case GET_L1GV_NRAY		:
	return (getNray(inputFile, templateList, vosNumber));

     case GET_L1GV_NSWEEP	:
	return (getNsweep(inputFile, templateList, vosNumber));

     case UPDATE_L1GV_TEMPLATE:
        return(updateTemplate(&templateList, inputFile, filename));

     default 			:
	  /* Invalid operation type in manageL1GVtemplate. */
	  TKreportWarning(W_TK_BADOPMT);
	  return (TK_FAIL);
  }
}



/*****************************************************************************
*
*  FUNCTION STATEMENT:
*	findSubVgroups(int32 inputFile, int32 vgroupID, int32 *subVgroups)
*
*  DESCRIPTION:
*	This routine finds the sub-vgroups for the given vgroupID and returns
*	the number of sub-vgroups.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       inputFile               Inputfile.
*	vgroupID		The vgroup identification returned by Vattach.
*
*  OUTPUT PARAMETERS:           
*       Parameter Name          Description
*       --------------          -----------
*	subVgroups		Sub-vgroups of the input vgroupID.  The
*				sub-vgroup IDs are stored in one dimensional
*				array.
*
*  CALLED BY:                   openInputVgroupL1GV().
*
*  RETURN VALUES:               The number of sub-vgroups under the vgroupID.
*
*  ROUTINES CALLED:
*       Routine Name            Description
*       ------------            -----------
*       TKreportError           This routine reports fatal errors to the error log
*                               file and terminates all processes.
*	Vgetid			A HDF routine which sequentially searches the HDF file 
*				for Vgroups.
*	Visvg			A HDF routine which testes if a given reference number 
*				within a Vgropu vgroup_id is a Vgroup.
*	Vattach			A HDF routine which attaches to a Vgroup and
*				returns an identifier for that group.
*
******************************************************************************/

int32 findSubVgroups(int32 inputFile, int32 vgroupID, int32 *subVgroups)
{
  int32 vgroupRef = -1;
  int32 numOfOcc = 0;
  int32 id;
  char vgroupName[100];

  /*
  ** This while loop allows to search every subvgroups of the vgroup.  When
  ** vgroupRef is -1, the loop is terminated.
  ** Vgetid searches through the HDF file and returns the reference number
  ** of the next Vgroup following the Vgroup that has the reference number.
  */
  vgroupRef = Vgetid(inputFile, vgroupRef);
  while (vgroupRef != TK_HDF_FAIL)
  {	 
	 /*
	 ** Visvg returns 1 if the reference number refers to a Vgroup within the
	 ** Vgroup, and 0 otherwise.
	 */
         if (Visvg(vgroupID, vgroupRef))
         {
	     /*
	     ** Vattach attaches a Vgroup and returns an indentifier for that group.
	     */
             subVgroups[numOfOcc] = Vattach(inputFile, vgroupRef, "r");
	     if (subVgroups[numOfOcc] == TK_HDF_FAIL)
     	     {
		 /* Vattach failed in findSubVgroups. */
		 TKreportError(E_TK_FAILATTACHFSV);
	     }
             numOfOcc++;
         }

         vgroupRef = Vgetid(inputFile, vgroupRef);
  }

  return (numOfOcc);
}



/*****************************************************************************
*
*  FUNCTION STATEMENT:
*	 openInputVgroupL1GV(int32 inputFile, char filemode, int32 sdstartID, 
*				VGROUP_LINK *vGroup, int productID)
*
*  DESCRIPTION:
*       When vGroup is null, this routine searches the vgroup ids and the 
*	sub-vgroup ids.  When vGroup is not null, it initializes the vGroupID
*	field in vGroups and sub-Vgroups with vgroup ids and sub-vgroup IDs.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       inputFile               Inputfile.
*	sdstartID		The id returned from SDstart.
*	vGroup			The vgroup structure.  
*	productID		Product identifier
*	filemode		File mode of the input file
*
*  OUTPUT PARAMETERS:           
*       Parameter Name          Description
*       --------------          -----------
*	vGroup			The vgroup structure whose vGroupID field is 
*				initialized with new IDs.	
*
*  CALLED BY:                   TKopen().
*
*  RETURN VALUES:               TK_SUCCESS if the routine is successful.
*
*  ROUTINES CALLED:
*       Routine Name            Description
*       ------------            -----------
*       TKreportError           This routine reports fatal errors to the error log
*                               file and terminates all processes.
*	Vlone			A HDF routine which returns an array of reference numbers 
*				for all lone Vgroups that are not linked to any Vgroup
*				in the HDF file.
*	Vattach			A HDF routine which attaches to a Vgroup and
*                               returns an identifier for that group.
*	findSubVgroups		This routine finds the sub-vgroups for the given vgroupID 
*				and returns the number of sub-vgroups.
*	setL1GVtemplate	 	This routines is used to create a template which is 
*				initialized 
*				with input data. It calls manageL1GVtemplate in order to 
*				append it into the list of templates.
*	SDnametoindex		A HDF routine Uses the name of a scientific data set to 
*				determine the index assigned to it.
*	SDselect		A HDF routine gets the sds_id for a specific data set.
*	SDgetinfo		A HDF routine gets the name, rank, dimension sizes,
*				number type, and attribute count for the specified data
*				set.
*
******************************************************************************/

int32 openInputVgroupL1GV(int32 inputFile, char filemode, int32 sdstartID, 
				VGROUP_LINK *vGroup, int productID)
{
  static int32 	vosIDs[MAX_VOS];
  static int32 	sensorIDs[MAX_VOS][1];
  static int32 	parmIDs[MAX_VOS][MAX_PARM];
  static int32 	numOfVOS;
  static int32	numOfparm[MAX_VOS];
  static int32  rootVgroupID;
  	 int32 	refArray[3];
         int32  status;
         int32  nray[MAX_VOS];
         int32  ncell[MAX_VOS][MAX_PARM];
	 int32  nsweep[MAX_VOS];
         int32  sdsID, sdIndex;
	 int32  dim[4];
	 int32  rank[1];
         int32  nt[1];
         int32  nattr[1];
         int    i, j;
	 char   sdsName[50];
	 char   vgroupName[100];
	 char   parmName[20];

  VGROUP_LINK	*tempVgroup = vGroup;
  VGROUP_LINK	*tempVgroup2;


  /*
  ** No-null tempVgroup indicates that this routine is previously called to 
  ** search vgroup and sub-vgroupIDs.  So when tempVgroup is not null, this
  ** routine initializes vGroupID field of Vgroup with the IDs searched
  ** previously.
  */
  if (tempVgroup != NULL)
  {
      /*
      ** Assgin root-vgroupID to the field vGroupID of vGroup.
      */
      vGroup->vGroupID->vGroupID = rootVgroupID; 
      /*
      ** Assign volume scan, sensor, and parameter IDs to the subVgroup.
      */
      tempVgroup = vGroup->subVgroup;
      for (i = 0; i < numOfVOS; i++)
      {
           tempVgroup->vGroupID->vGroupID = vosIDs[i];
	   tempVgroup->subVgroup->vGroupID->vGroupID = sensorIDs[i][0];
	   tempVgroup2 = tempVgroup->subVgroup->subVgroup;
           for (j = 0; j < numOfparm[i]; j++)
           {
                tempVgroup2->vGroupID->vGroupID = parmIDs[i][j];
	        tempVgroup2 = tempVgroup2->next;
           }
	   tempVgroup = tempVgroup->next;
     }
  }
  /*
  ** When tempVgroup is null, search vgroup and sub-vgroup IDs.  And store them
  ** into the static variables.
  */
  else
  {
      /* 
      ** Initialization.
      */ 
      numOfVOS = 0;
      for (i = 0; i < MAX_VOS; ++i)
      {
           numOfparm[i] = 0;
      }

      /* 
      ** Find the root vgroup ID. Vlone returns the reference number of root vgroups. 
      ** Since we have only one root Vgroup which is Data Granule, find the vgroup
      ** identifier of the root Vgroup.  Vattach returns an identifier for that group.
      */
      status = Vlone(inputFile, refArray, 2);

      switch(filemode)
      {
        case TK_READ_ONLY:
                rootVgroupID = Vattach(inputFile, refArray[0], "r");
                break;
        case TK_APPEND:
                rootVgroupID = Vattach(inputFile, refArray[0], "w");
                break;
        default:
                /* Invalid filemode in openInputVgroupL1GV. */
                TKreportError(E_TK_FAILFILEMODE);
      }


      /* 
      ** If Vattach is failed, then error out.
      */
      if (rootVgroupID == TK_HDF_FAIL)
      {
	  /* Vattach failed in openInputVgroupL1GV. */
          TKreportError(E_TK_VATTACHFAILOIV);
      }

      /* 
      ** Find the number of VOS.
      */
      numOfVOS = findSubVgroups(inputFile, rootVgroupID, vosIDs);

      /*
      ** Find the vgroup IDs of sensors. 
      */
      for (i = 0; i < numOfVOS; ++i)
      {
           status = findSubVgroups(inputFile, vosIDs[i], sensorIDs[i]);
      }

      /* 
      ** Find the vgroup IDs of parameters.
      */
      for (i = 0; i < numOfVOS; ++i)
      {
           numOfparm[i] = findSubVgroups(inputFile, sensorIDs[i][0], parmIDs[i]);
      }

      /*
      ** Search for the number of sweeps, rays, and cells. 
      ** SDnametoindex uses the name of a scientific data set to determine the
      ** index assigned to it. SDselect gets the sds_id for a specific data set. 
      ** SDgetinfo gets the name, rank, dimension sizes, number type, and attribute
      ** count for the specified data set.
      */
      for (i = 0; i < numOfVOS; ++i)
      {
	   for (j = 0; j < numOfparm[i]; ++j)
	   {
	     if (productID == TK_L1C_GV)
	     {
		if ((j % 2) == 0)
	            sprintf(parmName, "Parm_Data1byte%d_%d", (i+1), (j+1));
		else
	            sprintf(parmName, "Parm_Data2byte%d_%d", (i+1), (j+1));
	     }
	     if (productID == TK_L1B_GV)
	     {
	        sprintf(parmName, "Parm_Data2byte%d_%d", (i+1), (j+1));
	     }
      	     sdIndex = SDnametoindex(sdstartID,parmName); 
             sdsID = SDselect(sdstartID, sdIndex);
             SDgetinfo(sdsID, sdsName, rank, dim, nt, nattr);
	     nsweep[i] = dim[1];
      	     nray[i]   = dim[2]; 
             ncell[i][j]  = dim[3];
           }
      }
      /*
      ** Set the fields of template with number of volume scans, parameters, cells,
      ** and so on.
      */
      setL1GVtemplate(numOfVOS, numOfparm, ncell, nray, nsweep, "\0", inputFile);
  }

  return (TK_SUCCESS);
}



/*****************************************************************************
*
*  FUNCTION STATEMENT:
*	printComment(FILE *fileptr, int order)
*
*  DESCRIPTION:
*	This routine is used to create a Level 1 GV configuration file.  It
*	prints comment information.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*	order			The volume scan number.
*	fileptr			The file pointer.
*
*  OUTPUT PARAMETERS:		
*       Parameter Name          Description
*       --------------          -----------
*	filePtr			The file pointer after writing comments into
*				the file.
*
*  CALLED BY:                   makeL1GVconfigFile().
*
*  RETURN VALUES:               None.
*
*  ROUTINES CALLED:		None.
*
******************************************************************************/

void printComment(FILE *fileptr, int order)
{
  fprintf(fileptr, "\tvdata 1 Comment%d 		vdatatype \n", order);
  fprintf(fileptr, "\t   5001 commentField\t\t99.9\t tk_char8 \n");
}



/*****************************************************************************
*
*  FUNCTION STATEMENT:
*	printVolumeDescriptor(FILE *fileptr, int order)
*
*  DESCRIPTION:
*       This routine is used to create a Level 1 GV configuration file.  It
*       prints volume descriptor information.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*	order			The volume scan number.
*	fileptr			The file pointer.
*
*  OUTPUT PARAMETERS:           
*       Parameter Name          Description
*       --------------          -----------
*	filePtr			The file pointer after writing comments into
*				the file.
*
*  CALLED BY:                   makeL1GVconfigFile().
*
*  RETURN VALUES:               None.
*
*  ROUTINES CALLED:             None.
*
******************************************************************************/

void printVolumeDescriptor(FILE *fileptr, int order)
{
  fprintf(fileptr, "\n\tvdata 1 Volume_Descriptor%d 	vdatatype \n", order);
  fprintf(fileptr, "\t      1 versionNumber\t\t99.9\t tk_int16 \n");
  fprintf(fileptr, "\t      1 volumeNumber\t\t99.9\t tk_int16 \n");
  fprintf(fileptr, "\t      1 sizeOfDataRecord\t99.9\t tk_int32 \n");
  fprintf(fileptr, "\t     21 projectName\t\t99.9\t tk_char8 \n");
  fprintf(fileptr, "\t      1 year\t\t\t99.9\t tk_int16 \n");
  fprintf(fileptr, "\t      1 month\t\t\t99.9\t tk_int16 \n");
  fprintf(fileptr, "\t      1 day\t\t\t99.9\t tk_int16 \n");
  fprintf(fileptr, "\t      1 hour\t\t\t99.9\t tk_int16 \n");
  fprintf(fileptr, "\t      1 minutes\t\t\t99.9\t tk_int16 \n");
  fprintf(fileptr, "\t      1 second\t\t\t99.9\t tk_int16 \n");
  fprintf(fileptr, "\t      9 flightNumber\t\t99.9\t tk_char8 \n");
  fprintf(fileptr, "\t      9 facilityName\t\t99.9\t tk_char8 \n");
  fprintf(fileptr, "\t      1 genYear\t\t\t99.9\t tk_int16 \n");
  fprintf(fileptr, "\t      1 genMonth\t\t99.9\t tk_int16 \n");
  fprintf(fileptr, "\t      1 genDay\t\t\t99.9\t tk_int16 \n");
  fprintf(fileptr, "\t      1 numSensorDescriptor\t99.9\t tk_int16 \n");

}


/*****************************************************************************
*
*  FUNCTION STATEMENT:
*	printRadarDesc(FILE *fileptr, int order)
*
*  DESCRIPTION:
*       This routine is used to create a Level 1 GV configuration file.  It
*       prints Radar Descriptor information.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       order                   The volume scan number.
*       fileptr                 The file pointer.
*
*  OUTPUT PARAMETERS:           
*       Parameter Name          Description
*       --------------          -----------
*       filePtr                 The file pointer after writing comments into
*                               the file.
*
*  CALLED BY:                   makeL1GVconfigFile().
*
*  RETURN VALUES:               None.
*
*  ROUTINES CALLED:             None.
*
******************************************************************************/

void printRadarDesc(FILE *fileptr, int order)
{
  fprintf(fileptr, "\t\tvdata 1 Radar_Descriptor%d 	vdatatype \n", order);
  fprintf(fileptr, "\t\t9	RadarName\t\t99.9\t tk_char8 \n");
  fprintf(fileptr, "\t\t1	RadarConstant\t\t99.9\t tk_float32 \n");
  fprintf(fileptr, "\t\t1	NominalPeakPower\t99.9\t tk_float32 \n");
  fprintf(fileptr, "\t\t1	NominalNoisePower\t99.9\t tk_float32 \n");
  fprintf(fileptr, "\t\t1	ReceiverGain\t\t99.9\t tk_float32 \n");
  fprintf(fileptr, "\t\t1	AntennaGain\t\t99.9\t tk_float32 \n");
  fprintf(fileptr, "\t\t1	radarSystemGain\t\t99.9\t tk_float32 \n");
  fprintf(fileptr, "\t\t1	HorizontalBeamWith\t99.9\t tk_float32 \n");
  fprintf(fileptr, "\t\t1	VerticalBeamWith\t99.9\t tk_float32 \n");
  fprintf(fileptr, "\t\t1	RadarType\t\t99.9\t tk_int16 \n");
  fprintf(fileptr, "\t\t1	ScanMode\t\t99.9\t tk_int16 \n");
  fprintf(fileptr, "\t\t1	NominalScanRate\t\t99.9\t tk_float32 \n");
  fprintf(fileptr, "\t\t1	NominalStartAngle\t99.9\t tk_float32 \n");
  fprintf(fileptr, "\t\t1	NominalStopAngle\t99.9\t tk_float32 \n");
  fprintf(fileptr, "\t\t1	NumOfParmDesc\t\t99.9\t tk_int16 \n");
  fprintf(fileptr, "\t\t1	NumOfDescriptor\t\t99.9\t tk_int16 \n");
  fprintf(fileptr, "\t\t1	DataCompression\t\t99.9\t tk_int16 \n");
  fprintf(fileptr, "\t\t1	DataReductionAlgorthm\t99.9\t tk_int16 \n");
  fprintf(fileptr, "\t\t1	DataReductSpecificParm1\t99.9\t tk_float32 \n");
  fprintf(fileptr, "\t\t1	DataReductSpecificParm2\t99.9\t tk_float32 \n");
  fprintf(fileptr, "\t\t1	RadarLongitude\t\t99.9\t tk_float32 \n");
  fprintf(fileptr, "\t\t1	RadarLatitude\t\t99.9\t tk_float32 \n");
  fprintf(fileptr, "\t\t1	RadarAltitude\t\t99.9\t tk_float32 \n");
  fprintf(fileptr, "\t\t1	EffectiveUnambVelocity\t99.9\t tk_float32 \n");
  fprintf(fileptr, "\t\t1	EffectiveUnambRange\t99.9\t tk_float32 \n");
  fprintf(fileptr, "\t\t1	NumOfTransFrequencies\t99.9\t tk_int16 \n");
  fprintf(fileptr, "\t\t1	NumOfInterPulsePeriod\t99.9\t tk_int16 \n");
  fprintf(fileptr, "\t\t1	Frequency1\t\t99.9\t tk_float32 \n");
  fprintf(fileptr, "\t\t1	Frequency2\t\t99.9\t tk_float32 \n");
  fprintf(fileptr, "\t\t1	Frequency3\t\t99.9\t tk_float32 \n");
  fprintf(fileptr, "\t\t1	Frequency4\t\t99.9\t tk_float32 \n");
  fprintf(fileptr, "\t\t1	Frequency5\t\t99.9\t tk_float32 \n");
  fprintf(fileptr, "\t\t1	interPulsePeriod1\t99.9\t tk_float32 \n");
  fprintf(fileptr, "\t\t1	interPulsePeriod2\t99.9\t tk_float32 \n");
  fprintf(fileptr, "\t\t1	interPulsePeriod3\t99.9\t tk_float32 \n");
  fprintf(fileptr, "\t\t1	interPulsePeriod4\t99.9\t tk_float32 \n");
  fprintf(fileptr, "\t\t1	interPulsePeriod5\t99.9\t tk_float32 \n"); 
}



/*****************************************************************************
*
*  FUNCTION STATEMENT:
*	printCorrFactorDesc(FILE *fileptr, int order)
*
*  DESCRIPTION:
*       This routine is used to create a Level 1 GV configuration file.  It
*       prints Correction Factor Descriptor information.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       order                   The volume scan number.
*       fileptr                 The file pointer.
*
*  OUTPUT PARAMETERS:           
*       Parameter Name          Description
*       --------------          -----------
*       filePtr                 The file pointer after writing comments into
*                               the file.
*
*  CALLED BY:                   makeL1GVconfigFile().
*
*  RETURN VALUES:               None.
*
*  ROUTINES CALLED:             None.
*
******************************************************************************/

void printCorrFactorDesc(FILE *fileptr, int order)
{
  fprintf(fileptr, "\n\t\tvdata 1 Correction_Factor_Descriptor%d vdatatype \n", order);
  fprintf(fileptr, "\t\t1	Azimuth\t\t\t99.9\t tk_float32 \n");
  fprintf(fileptr, "\t\t1	Elevation\t\t99.9\t tk_float32 \n");
  fprintf(fileptr, "\t\t1	RangeDelay\t\t99.9\t tk_float32 \n");
  fprintf(fileptr, "\t\t1	RadarLongitude\t\t99.9\t tk_float32 \n");
  fprintf(fileptr, "\t\t1	RadarLatitude\t\t99.9\t tk_float32 \n");
  fprintf(fileptr, "\t\t1	RadarPressureAltitude\t99.9\t tk_float32 \n");
  fprintf(fileptr, "\t\t1	RadarAltitude\t\t99.9\t tk_float32 \n");
  fprintf(fileptr, "\t\t1	RadarPGSpeedEW\t\t99.9\t tk_float32 \n");
  fprintf(fileptr, "\t\t1	RadarPGSpeedNS\t\t99.9\t tk_float32 \n");
  fprintf(fileptr, "\t\t1	RadarPVerticalVelocity\t99.9\t tk_float32 \n");
  fprintf(fileptr, "\t\t1	RadarPlatformHeading\t99.9\t tk_float32 \n");
  fprintf(fileptr, "\t\t1	RadarPlatformRoll\t99.9\t tk_float32 \n");
  fprintf(fileptr, "\t\t1	RadarPlatformPitch\t99.9\t tk_float32 \n");
  fprintf(fileptr, "\t\t1	RadarPlatformDrift\t99.9\t tk_float32 \n");
  fprintf(fileptr, "\t\t1	RadarRotationAngle\t99.9\t tk_float32 \n");
  fprintf(fileptr, "\t\t1	RadarTiltAngle\t\t99.9\t tk_float32 \n");
}



/*****************************************************************************
*
*  FUNCTION STATEMENT:
*	printSweepInfo(FILE *fileptr, int order)
*
*  DESCRIPTION:
*       This routine is used to create a Level 1 GV configuration file.  It
*       prints Sweep Info information.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       order                   The volume scan number.
*       fileptr                 The file pointer.
*
*  OUTPUT PARAMETERS:           
*       Parameter Name          Description
*       --------------          -----------
*       filePtr                 The file pointer after writing comments into
*                               the file.
*
*  CALLED BY:                   makeL1GVconfigFile().
*
*  RETURN VALUES:               None.
*
*  ROUTINES CALLED:             None.
*
******************************************************************************/

void printSweepInfo(FILE *fileptr, int order)
{
  fprintf(fileptr, "\n\t\tvdata 1 Sweep_Info%d 		vdatatype \n", order);
  fprintf(fileptr, "\t\t9	RadarName\t\t99.9\t tk_char8 \n");
  fprintf(fileptr, "\t\t1	SweepNumber\t\t99.9\t tk_int32 \n");
  fprintf(fileptr, "\t\t1	numOfRays\t\t99.9\t tk_int32 \n");
  fprintf(fileptr, "\t\t1	TrueStartAngle\t\t99.9\t tk_float32 \n");
  fprintf(fileptr, "\t\t1	TrueStopAngle\t\t99.9\t tk_float32 \n");
  fprintf(fileptr, "\t\t1	FixedAngle\t\t99.9\t tk_float32 \n");
  fprintf(fileptr, "\t\t1	FilterFlag\t\t99.9\t tk_int32 \n");
}



/*****************************************************************************
*
*  FUNCTION STATEMENT:
*	printRayInfoInteger(FILE *fileptr, int order, int32 nray, int32 nsweep)
*
*  DESCRIPTION:
*       This routine is used to create a Level 1 GV configuration file.  It
*       prints Ray Info_Integer information.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       order                   The volume scan number.
*       fileptr                 The file pointer.
*	nsweep			Number of sweeps.
*	nray			Number of rays.
*
*  OUTPUT PARAMETERS:           
*       Parameter Name          Description
*       --------------          -----------
*       filePtr                 The file pointer after writing comments into
*                               the file.
*
*  CALLED BY:                   makeL1GVconfigFile().
*
*  RETURN VALUES:               None.
*
*  ROUTINES CALLED:             None.
*
******************************************************************************/

void printRayInfoInteger(FILE *fileptr, int order, int32 nray, int32 nsweep)
{
  fprintf(fileptr, "\n\t\tsds 3 %d %d 7 Ray_Info_Integer%d\t99.9\t tk_int32 \n", nsweep, nray, order);
}



/*****************************************************************************
*
*  FUNCTION STATEMENT:
*	printRayInfoFloat(FILE *fileptr, int order, int32 nray, int32 nsweep)
*
*  DESCRIPTION:
*       This routine is used to create a Level 1 GV configuration file.  It
*       prints Ray Info_Float information.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       order                   The volume scan number.
*	nsweep			Number of sweeps.
*	nray			Number of rays.
*       fileptr                 The file pointer.
*
*  OUTPUT PARAMETERS:           
*       Parameter Name          Description
*       --------------          -----------
*       filePtr                 The file pointer after writing comments into
*                               the file.
*
*  CALLED BY:                   makeL1GVconfigFile().
*
*  RETURN VALUES:               None.
*
*  ROUTINES CALLED:             None.
*
******************************************************************************/

void printRayInfoFloat(FILE *fileptr, int order, int32 nray, int32 nsweep)
{
  fprintf(fileptr, "\n\t\tsds 3 %d %d 4 Ray_Info_Float%d\t\t99.9\t tk_float32 \n", nsweep, nray, order);
}



/*****************************************************************************
*
*  FUNCTION STATEMENT:
*	printPlatformInfo(FILE *fileptr, int order, int32 nray, int32 nsweep)
*
*  DESCRIPTION:
*       This routine is used to create a Level 1 GV configuration file.  It
*       prints Platfor Info information.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       order                   The volume scan number.
*	nsweep			Number of sweeps.
*	nray			Number of rays.
*       fileptr                 The file pointer.
*
*  OUTPUT PARAMETERS:           
*       Parameter Name          Description
*       --------------          -----------
*       filePtr                 The file pointer after writing comments into
*                               the file.
*
*  CALLED BY:                   makeL1GVconfigFile().
*
*  RETURN VALUES:               None.
*
*  ROUTINES CALLED:             None.
*
******************************************************************************/

void printPlatformInfo(FILE *fileptr, int order, int32 nray, int32 nsweep)
{
  fprintf(fileptr, "\n\t\tsds 3 %d %d 18 Platform_Info%d\t\t99.9\t tk_float32 \n", nsweep, nray, order);
}



/*****************************************************************************
*
*  FUNCTION STATEMENT:
*	printParmDesc(FILE *fileptr, int order1, int order2)
*
*  DESCRIPTION:
*       This routine is used to create a Level 1 GV configuration file.  It
*       prints Parameter Descriptor information.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       order1                  The volume scan number.
*	order2			The parameter number.
*       fileptr                 The file pointer.
*
*  OUTPUT PARAMETERS:           
*       Parameter Name          Description
*       --------------          -----------
*       filePtr                 The file pointer after writing comments into
*                               the file.
*
*  CALLED BY:                   makeL1GVconfigFile().
*
*  RETURN VALUES:               None.
*
*  ROUTINES CALLED:             None.
*
******************************************************************************/

void printParmDesc(FILE *fileptr, int order1, int order2)
{
  fprintf(fileptr, "\t\t\tvdata 1 Parameter_Descriptor%d_%d vdatatype \n", order1, order2);
  fprintf(fileptr, "\t\t\t9	  NameOfParm\t\t99.9\t tk_char8 \n");
  fprintf(fileptr, "\t\t\t41	  DescOfParm\t\t99.9\t tk_char8 \n");
  fprintf(fileptr, "\t\t\t9	  UnitsOfParm\t\t99.9\t tk_char8 \n");
  fprintf(fileptr, "\t\t\t1	  InterPulsePeriodUsed\t99.9\t tk_int16 \n");
  fprintf(fileptr, "\t\t\t1	  TransFrequenciesUsed\t99.9\t tk_int16 \n");
  fprintf(fileptr, "\t\t\t1	  ReceiverBandwidth\t99.9\t tk_float32 \n");
  fprintf(fileptr, "\t\t\t1	  PulseWidth\t\t99.9\t tk_int16 \n");
  fprintf(fileptr, "\t\t\t1	  PolarTransmittedWave\t99.9\t tk_int16 \n");
  fprintf(fileptr, "\t\t\t1	  numOfSamples\t\t99.9\t tk_int16 \n");
  fprintf(fileptr, "\t\t\t1	  parmDataType\t\t99.9\t tk_int16 \n");
  fprintf(fileptr, "\t\t\t9	  ThresholdField\t99.9\t tk_char8 \n");
  fprintf(fileptr, "\t\t\t1	  ThresholdValue\t99.9\t tk_float32 \n");
  fprintf(fileptr, "\t\t\t1	  ScaleFactor\t\t99.9\t tk_float32 \n");
  fprintf(fileptr, "\t\t\t1	  OffsetFactor\t\t99.9\t tk_float32 \n");
  fprintf(fileptr, "\t\t\t1	  DelOrMissingDataFlag\t99.9\t tk_int32 \n");
}



/*****************************************************************************
*
*  FUNCTION STATEMENT:
*	printCellRangeVector(FILE *fileptr, int order1, int order2, int32 ncell)
*
*  DESCRIPTION:
*       This routine is used to create a Level 1 GV configuration file.  It
*       prints Cell Range Vector information.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       order1                  The volume scan number.
*	order2			The parameter number.
*	ncell			The number of cells.
*       fileptr                 The file pointer.
*
*  OUTPUT PARAMETERS:           
*       Parameter Name          Description
*       --------------          -----------
*       filePtr                 The file pointer after writing comments into
*                               the file.
*
*  CALLED BY:                   makeL1GVconfigFile().
*
*  RETURN VALUES:               None.
*
*  ROUTINES CALLED:             None.
*
******************************************************************************/

void printCellRangeVector(FILE *fileptr, int order1, int order2, int32 ncell)
{
  fprintf(fileptr, "\n\t\t\tvdata 1 Cell_Range_Vector%d_%d vdatatype \n", order1, order2);
  fprintf(fileptr, "\t\t\t1	    NumOfCells\t\t99.9\t tk_int32 \n");
  fprintf(fileptr, "\t\t\t%d	    DistanceToCell\t99.9\t tk_float32\n",ncell);
}




/*****************************************************************************
*
*  FUNCTION STATEMENT:
*       printParmData1C_2byte(FILE *fileptr, int order1, int order2, int32 ncell,
*                     int32 nray, int32 nsweep)
*
*  DESCRIPTION:
*       This routine is used to create a Level 1C GV configuration file.  It
*       prints Parameter Data 2 byte information.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       fileptr                 The file pointer.
*       order1                  The volume scan number.
*       order2                  The parameter number.
*       ncell                   The number of cells.
*       nray                    The number of rays.
*       nsweep                  The number of sweeps.
*
*  OUTPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       filePtr                 The file pointer after writing comments into
*                               the file.
*
*  CALLED BY:                   makeL1GVconfigFile().
*
*  RETURN VALUES:               None.
*
*  ROUTINES CALLED:             None.
*
******************************************************************************/

void printParmData1C_2byte(FILE *fileptr, int order1, int order2, int32 ncell,
                   int32 nray, int32 nsweep)
{
  fprintf(fileptr, "\t\t\tsds 3 %d %d %d Parm_Data2byte%d_%d\t99.9\t tk_int16 \n",
                nsweep, nray, ncell, order1, order2);
}


/*****************************************************************************
*
*  FUNCTION STATEMENT:
*       printParmData1C_1byte(FILE *fileptr, int order1, int order2, int32 ncell,
*                     int32 nray, int32 nsweep)
*
*  DESCRIPTION:
*       This routine is used to create a Level 1C GV configuration file.  It
*       prints Parameter Data 1 byte information.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       fileptr                 The file pointer.
*       order1                  The volume scan number.
*       order2                  The parameter number.
*       ncell                   The number of cells.
*       nray                    The number of rays.
*       nsweep                  The number of sweeps.
*
*  OUTPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       filePtr                 The file pointer after writing comments into
*                               the file.
*
*  CALLED BY:                   makeL1GVconfigFile().
*
*  RETURN VALUES:               None.
*
*  ROUTINES CALLED:             None.
*
******************************************************************************/

void printParmData1C_1byte(FILE *fileptr, int order1, int order2, int32 ncell,
                   int32 nray, int32 nsweep)
{
  fprintf(fileptr, "\t\t\tsds 3 %d %d %d Parm_Data1byte%d_%d\t99.9\t tk_int8 \n",
                nsweep, nray, ncell, order1, order2);
}

/*****************************************************************************
*
*  FUNCTION STATEMENT:
*       printParmData1B(FILE *fileptr, int order1, int order2, int32 ncell,
*                     int32 nray, int32 nsweep)
*
*  DESCRIPTION:
*       This routine is used to create a Level 1B GV configuration file.  It
*       prints Parameter Data information.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       fileptr                 The file pointer.
*       order                   The volume scan number.
*       order2                  The parameter number.
*       ncell                   The number of cells.
*       nray                    The number of rays.
*       nsweep                  The number of sweeps.
*
*  OUTPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       filePtr                 The file pointer after writing comments into
*                               the file.
*
*  CALLED BY:                   makeL1GVconfigFile().
*
*  RETURN VALUES:               None.
*
*  ROUTINES CALLED:             None.
*
******************************************************************************/

void printParmData1B(FILE *fileptr, int order1, int order2, int32 ncell,
                   int32 nray, int32 nsweep)
{
  fprintf(fileptr, "\n\t\t\tsds 3 %d %d %d Parm_Data2byte%d_%d\t99.9\t tk_int16 \n", 
		nsweep, nray, ncell, order1, order2);
}



/*****************************************************************************
*
*  FUNCTION STATEMENT:
*	makeL1GVconfigFile(int32 inputFile, char *cfgFileName, int productID)
*
*  DESCRIPTION:
*       This routine is used to create a Level 1 GV configuration file. 
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*	inputFile		HDF file ID returned by Hopen.
*	cfgFileName		The configuration file name.
*	productID		Product identifier
*
*  OUTPUT PARAMETERS:           
*       Parameter Name          Description
*       --------------          -----------
*	cfgFileName		The file containing configuration structure.
*
*  CALLED BY:                   TKopen().
*
*  RETURN VALUES:               None.
*
*  ROUTINES CALLED:             printComment(), printVolumeDescriptor(),
*				printRadarDesc(), printCorrFactorDesc(),
*				printSweepInfo(), printRayInfoInteger(),
*				printRayInfoFloat(), printPlatformInfo(),
*				printParmDesc(), printCellRangeVector(),
*				printParmData(), manageL1GVtemplate(), and
*				TKreportError().
*
******************************************************************************/

void makeL1GVconfigFile(int32 inputFile, char *cfgFileName, int productID)
{
  int32 nvos, nray[MAX_VOS], ncell[MAX_VOS][MAX_PARM], nparm[MAX_VOS], nsweep[MAX_VOS];
  int i, j;
  FILE *fileptr;
  char *path = NULL;
  char fullpath[TK_FNAME_LEN];
  int l1gvFlag;

  l1gvFlag = ((productID == TK_L1B_GV) || (productID == TK_L1C_GV)) ? 1 : 0;

#ifndef TSU_ENV
  if ( l1gvFlag )
    strcpy ( fullpath, "./" );
#else
  /*
  ** Get the path to store the configuration file.  
  */
  path = getenv( "TSDISTK" );
  if (path == NULL)
  {
      /* Invalid configuration file path in makeL1GVconfigFile. */
      TKreportError(E_TK_BADPATHMCF);
  }

  strcpy(fullpath, path);
  strcat(fullpath, "/config/");
#endif

  strcat(fullpath, cfgFileName); 

  /* 
  ** Open the configuration file.
  */
  fileptr = fopen(fullpath, "w");
  if (fileptr == NULL)
  {
      /* Opening configuration file failed in makeL1GVconfigFile. */
      TKreportError(E_TK_OPENFAILEDMCF);
  }

  /* 
  ** Find the number of volume scans. 
  */
  nvos = manageL1GVtemplate(GET_L1GV_NUM_OF_VOS, inputFile, '\0', NULL,  -1, -1);

  /*
  ** Using the volume scan number, find number of cell, number of rays, the number 
  ** sweeps, and the number of parmameters.
  */
  for (i = 0; i < nvos; ++i)
  {
       nray[i] = manageL1GVtemplate(GET_L1GV_NRAY, inputFile, '\0', NULL, i, -1);
       nparm[i] = manageL1GVtemplate(GET_L1GV_NUM_OF_PARM, inputFile, '\0', NULL, i, -1);
       nsweep[i] = manageL1GVtemplate(GET_L1GV_NSWEEP, inputFile, '\0', NULL, i, -1);
       for (j = 0; j < nparm[i]; ++j)
       {
            ncell[i][j] = manageL1GVtemplate(GET_L1GV_NCELL, inputFile, '\0', NULL, i, j);
       }
  }

  /* 
  ** Print the Level 1 GV configration structure. 
  */
  for (i = 1; i <= nvos; ++i) 
  {
       fprintf(fileptr, "begin\n\n");
       fprintf(fileptr, "vgroup VOS%d	vgrouptype\n\n", i);
       printComment(fileptr, i);
       printVolumeDescriptor(fileptr, i);
       fprintf(fileptr, "\n\tbegin\n");
       fprintf(fileptr, "\n\tvgroup Sensor%d 		vgrouptype\n\n", i);
       printRadarDesc(fileptr, i);
       printCorrFactorDesc(fileptr, i);
       printSweepInfo(fileptr, i);
       printRayInfoInteger(fileptr, i, nray[i-1], nsweep[i-1]);
       printRayInfoFloat(fileptr, i, nray[i-1], nsweep[i-1]);
       printPlatformInfo(fileptr, i, nray[i-1], nsweep[i-1]);
       for (j = 1; j <= nparm[i -1]; ++j)
       {
	    fprintf(fileptr, "\n\t\tbegin\n");
            fprintf(fileptr, "\n\t\tvgroup Parameter%d_%d	vgrouptype\n\n", i, j);
	    printParmDesc(fileptr, i, j);
	    printCellRangeVector(fileptr, i, j, ncell[i-1][j-1]);
            if (productID == TK_L1C_GV)
	    {
		if (((j - 1) % 2) == 0)
		  printParmData1C_1byte(fileptr, i, j, ncell[i-1][j-1], nray[i-1],
                                nsweep[i-1]);
		else
		  printParmData1C_2byte(fileptr, i, j, ncell[i-1][j-1], nray[i-1],
                                nsweep[i-1]);
	    }
	    if (productID == TK_L1B_GV)
	    	printParmData1B(fileptr, i, j, ncell[i-1][j-1], nray[i-1], 
				nsweep[i-1]);
	    fprintf(fileptr, "\t\tend\n");
       }
       fprintf(fileptr, "\tend\n");
       fprintf(fileptr, "end\n\n");
  }

  /* 
  ** Close the configuration file. 
  */
  if (fclose(fileptr) != NULL)
	/* Unable to close the configuration file. */
	TKreportError(E_TK_FAILFCLOSE);
}



/*****************************************************************************
*
*  FUNCTION STATEMENT:
*	getCurrVgroup(VGROUP_LINK *vgroupList, int32 vgroupPos)
*
*  DESCRIPTION:
*       This routine returns the requested vgroup. vgroupList 
*	contains list of vgroups and vgroupPos is used to take to the 
*	position of a vgroup in VgroupList.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*	vgroupList		List of vgroups.
*	vgroupPos		Vgroup position in integer value.
*
*  OUTPUT PARAMETERS:           None.
*
*  CALLED BY:                   writeGV_L1B_1C(), and readGV_L1B_1C().
*
*  RETURN VALUES:               A requested vgroup in vgroupList.
*
*  ROUTINES CALLED:             
*       Routine Name            Description
*       ------------            -----------
*       TKreportError           This routine reports fatal errors to the error log
*                               file and terminates all processes.
*
******************************************************************************/

VGROUP_LINK *getCurrVgroup(VGROUP_LINK *vgroupList, int32 vgroupPos)
{
  VGROUP_LINK *tempVgroup = vgroupList;
  int i;

  /*
  ** Increment the template until the list is reached at the desired position.
  */
  for (i = 1; i < vgroupPos; ++i)
  {
       if (tempVgroup != NULL)
       {
	   tempVgroup = tempVgroup->next;
       }
  }

  /*
  ** If the desired position is null, then error out.
  */
  if (tempVgroup == NULL)
  {
      /* Empty vgroup lists.*/
      TKreportError(E_TK_EMPTYVGGV);
  }

  return (tempVgroup); 
}




/***********************************************************************************
*
*  FUNCTION STATEMENT:
*       writeGauge(IO_HANDLE *sGranuleHandle, L2A_56_RAINGAUGE *sGauge)
*
*  DESCRIPTION:
*       This function is used to write L2A56 Rain Gauge information in HDF format.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       sGranuleHandle          A structure containing information about the file
*                               sGranuleHandle is returned by TKopen().
*
*       sGauge			A structure containing the complete scanline data.
*                               This structure must be declared to correspond to 
*                               the data file being written.  The structure of 
*                               sGauge is manipulated by TKwriteGauge() and 
*                               other toolkit routines.
*
*  OUTPUT PARAMETERS:           None
*
*  CALLED BY:                   TKwriteGauge()
*
*  ROUTINES CALLED:     
*       Routine Name            Description
*       ------------            -----------
*       allocMem		This routine is used to allocate memory to vdata.
*       vdataWrite              This routine is used to write the vdata in HDF 
*                               format.
*       sdsWrite                This routine is used to write the scientific data
*                               set in HDF format.      
*
***********************************************************************************/

int writeGauge(IO_HANDLE *sGranuleHandle, L2A_56_RAINGAUGE *sGauge)
{
  VDATA_LINK    *tempVdata = sGranuleHandle->cfgSTR->vGroup->vdata->next;
  SDS_LINK	*tempSDS = sGranuleHandle->cfgSTR->vGroup->sds;
  uint8         *databuf, *ptr;
  int           i,j;


  /* Allocate memory for packing the vdata */
  databuf = allocMem(&tempVdata->vdataFields,1);
  ptr = databuf;
  memcpy(ptr, &sGauge->gaugeDesc.networkID, sizeof(sGauge->gaugeDesc.networkID));
  ptr += sizeof(sGauge->gaugeDesc.networkID);
  memcpy(ptr, &sGauge->gaugeDesc.gaugeType, sizeof(sGauge->gaugeDesc.gaugeType));
  ptr += sizeof(sGauge->gaugeDesc.gaugeType);
  memcpy(ptr, &sGauge->gaugeDesc.gaugeName, sizeof(sGauge->gaugeDesc.gaugeName));
  ptr += sizeof(sGauge->gaugeDesc.gaugeName);
  memcpy(ptr, &sGauge->gaugeDesc.gaugeMakeModel, sizeof(sGauge->gaugeDesc.gaugeMakeModel));
  ptr += sizeof(sGauge->gaugeDesc.gaugeMakeModel);
  memcpy(ptr, &sGauge->gaugeDesc.gaugeNumber, sizeof(sGauge->gaugeDesc.gaugeNumber));
  ptr += sizeof(sGauge->gaugeDesc.gaugeNumber);
  memcpy(ptr, &sGauge->gaugeDesc.gaugeLat, sizeof(sGauge->gaugeDesc.gaugeLat));
  ptr += sizeof(sGauge->gaugeDesc.gaugeLat);
  memcpy(ptr, &sGauge->gaugeDesc.gaugeLong, sizeof(sGauge->gaugeDesc.gaugeLong));
  ptr += sizeof(sGauge->gaugeDesc.gaugeLong);
  memcpy(ptr, &sGauge->gaugeDesc.range, sizeof(sGauge->gaugeDesc.range));
  ptr += sizeof(sGauge->gaugeDesc.range);
  memcpy(ptr, &sGauge->gaugeDesc.azimuth, sizeof(sGauge->gaugeDesc.azimuth));
  ptr += sizeof(sGauge->gaugeDesc.azimuth);
  memcpy(ptr, &sGauge->gaugeDesc.xCoordinate, sizeof(sGauge->gaugeDesc.xCoordinate));
  ptr += sizeof(sGauge->gaugeDesc.xCoordinate);
  memcpy(ptr, &sGauge->gaugeDesc.yCoordinate, sizeof(sGauge->gaugeDesc.yCoordinate));
  ptr += sizeof(sGauge->gaugeDesc.yCoordinate);
  memcpy(ptr, &sGauge->gaugeDesc.resolution, sizeof(sGauge->gaugeDesc.resolution));
  ptr += sizeof(sGauge->gaugeDesc.resolution);
  memcpy(ptr, &sGauge->gaugeDesc.elevation, sizeof(sGauge->gaugeDesc.elevation));

  vdataWrite(&tempVdata, databuf);
  free(databuf);


  sdsWrite(&tempSDS, &sGauge->hour);
  sdsWrite(&tempSDS, &sGauge->minute);

  for (i = 0; i < 1440; ++i)
  {
        /* The Following code was added to take care of the Missing or
           invalid data.  The reason to add this code was not to scale
           the invalid Data */

     if(sGauge->meanRainRate[i] > TK_DEFAULT)
	  sGauge->meanRainRate_scale[i] =
	     (int16)RND(sGauge->meanRainRate[i] * L2A56_MEANRAINRATE);
     else
	  sGauge->meanRainRate_scale[i] = TK_MISS_I2;
  }
  sdsWrite(&tempSDS, &sGauge->meanRainRate_scale);

  return(TK_SUCCESS);

}

/***********************************************************************************
*
*  FUNCTION STATEMENT:
*       readGauge(IO_HANDLE *sGranuleHandle, L2A_56_RAINGAUGE *sGauge)
*
*  DESCRIPTION:
*       This function is used to read L2A56 Rain Gauge information in HDF format.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       sGranuleHandle          A structure containing information about the file
*                               sGranuleHandle is returned by TKopen().
*
*       sGauge                  A structure containing the complete scanline data.
*                               This structure must be declared to correspond to
*                               the data file being written.  The structure of
*                               sGauge is manipulated by TKwriteGauge() and
*                               other toolkit routines.
*
*  OUTPUT PARAMETERS:           None.
*
*  CALLED BY:                   TKreadGauge()
*
*  ROUTINES CALLED:
*       Routine Name            Description
*       ------------            -----------
*       allocMem                This routine is used to allocate memory to vdata.
*       vdataRead               This routine is used to read the vdata in HDF            
*                               format.
*       sdsRead                 This routine is used to read the scientific data
*                               set in HDF format.
*        
************************************************************************************/
int readGauge(IO_HANDLE *sGranuleHandle, L2A_56_RAINGAUGE *sGauge)
{
  VDATA_LINK    *tempVdata = sGranuleHandle->cfgSTR->vGroup->vdata->next;
  SDS_LINK      *tempSDS = sGranuleHandle->cfgSTR->vGroup->sds;
  uint8         *databuf, *ptr;
  int           i,j;
  int scaleFactor;

  /* Allocate memory for unpacking the vdata */
  databuf = allocateVdataMemory(&tempVdata->vdataFields);
  ptr = databuf;
  /* Unpack the content of databuf into gauge.gaugeDesc */
  vdataRead(&tempVdata,ptr);
  memcpy(&sGauge->gaugeDesc.networkID, ptr, sizeof(sGauge->gaugeDesc.networkID));
  ptr += sizeof(sGauge->gaugeDesc.networkID);
  memcpy(&sGauge->gaugeDesc.gaugeType, ptr, sizeof(sGauge->gaugeDesc.gaugeType));
  ptr += sizeof(sGauge->gaugeDesc.gaugeType);
  memcpy(&sGauge->gaugeDesc.gaugeName, ptr, sizeof(sGauge->gaugeDesc.gaugeName));
  ptr += sizeof(sGauge->gaugeDesc.gaugeName);
  memcpy(&sGauge->gaugeDesc.gaugeMakeModel, ptr, sizeof(sGauge->gaugeDesc.gaugeMakeModel));
  ptr += sizeof(sGauge->gaugeDesc.gaugeMakeModel);
  memcpy(&sGauge->gaugeDesc.gaugeNumber, ptr, sizeof(sGauge->gaugeDesc.gaugeNumber));
  ptr += sizeof(sGauge->gaugeDesc.gaugeNumber);
  memcpy(&sGauge->gaugeDesc.gaugeLat, ptr, sizeof(sGauge->gaugeDesc.gaugeLat));
  ptr += sizeof(sGauge->gaugeDesc.gaugeLat);
  memcpy(&sGauge->gaugeDesc.gaugeLong, ptr, sizeof(sGauge->gaugeDesc.gaugeLong));
  ptr += sizeof(sGauge->gaugeDesc.gaugeLong);
  memcpy(&sGauge->gaugeDesc.range, ptr, sizeof(sGauge->gaugeDesc.range));
  ptr += sizeof(sGauge->gaugeDesc.range);
  memcpy(&sGauge->gaugeDesc.azimuth, ptr, sizeof(sGauge->gaugeDesc.azimuth));
  ptr += sizeof(sGauge->gaugeDesc.azimuth);
  memcpy(&sGauge->gaugeDesc.xCoordinate, ptr, sizeof(sGauge->gaugeDesc.xCoordinate));
  ptr += sizeof(sGauge->gaugeDesc.xCoordinate);
  memcpy(&sGauge->gaugeDesc.yCoordinate, ptr, sizeof(sGauge->gaugeDesc.yCoordinate));
  ptr += sizeof(sGauge->gaugeDesc.yCoordinate);
  memcpy(&sGauge->gaugeDesc.resolution, ptr, sizeof(sGauge->gaugeDesc.resolution));
  ptr += sizeof(sGauge->gaugeDesc.resolution);
  memcpy(&sGauge->gaugeDesc.elevation, ptr, sizeof(sGauge->gaugeDesc.elevation));
  free(databuf);  
  
  sdsRead(&tempSDS, &scaleFactor, &sGauge->hour);
  sdsRead(&tempSDS, &scaleFactor, &sGauge->minute);

  sdsRead(&tempSDS, &scaleFactor, &sGauge->meanRainRate_scale);
  for (i = 0; i < 1440; ++i)
  {
     /* The following code was added to take care of the Missing or   
        invalid data.  The reason to add this code was not to scale
        the invalid Data */
     
     if(sGauge->meanRainRate_scale[i]  != TK_MISS_I2)                
          sGauge->meanRainRate[i] =
                   (sGauge->meanRainRate_scale[i] * 1.0)/scaleFactor;  
     else
          sGauge->meanRainRate[i] = TK_DEFAULT;
  }
     
  return(TK_SUCCESS);

}



/***********************************************************************************
*
*  FUNCTION STATEMENT:
*	writeDisdrometer(IO_HANDLE *sGranuleHandle, L2A_57_DISDROMETER *sDisdrometer)
*
*  DESCRIPTION:
*       This function is used to write L2A57 Disdrometer information in HDF format.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       sGranuleHandle          A structure containing information about the file.
*                               sGranuleHandle is returned by TKopen().
*
*       sDisdrometer		A structure containing the complete scanline data.
*                               This structure must be declared to correspond to
*                               the data file being written.  The structure of
*                               sDisdrometer is manipulated by TKwriteGauge() and
*                               other toolkit routines.
*
*  OUTPUT PARAMETERS:           None
*
*  CALLED BY:                   TKwriteGauge()
*
*  ROUTINES CALLED:
*       Routine Name            Description
*       ------------            -----------
*       allocMem                This routine is used to allocate memory to vdata.
*       vdataWrite              This routine is used to write the vdata in HDF
*                               format.
*       sdsWrite                This routine is used to write the scientific data
*                               set in HDF format.
*
***********************************************************************************/

int writeDisdrometer(IO_HANDLE *sGranuleHandle, L2A_57_DISDROMETER *sDisdrometer)
{
  VDATA_LINK    *tempVdata = sGranuleHandle->cfgSTR->vGroup->vdata->next;
  SDS_LINK      *tempSDS = sGranuleHandle->cfgSTR->vGroup->sds;
  uint8         *databuf, *ptr;
  int           i,j;

  /* Allocate memory for packing the vdata */
  databuf = allocateVdataMemory(&tempVdata->vdataFields);
  ptr = databuf;
  memcpy(ptr, &sDisdrometer->disdroDesc.networkID, 
		sizeof(sDisdrometer->disdroDesc.networkID));
  ptr += sizeof(sDisdrometer->disdroDesc.networkID);
  memcpy(ptr, &sDisdrometer->disdroDesc.disdroName, 
		sizeof(sDisdrometer->disdroDesc.disdroName));
  ptr += sizeof(sDisdrometer->disdroDesc.disdroName);
  memcpy(ptr, &sDisdrometer->disdroDesc.disdroMakeModel, 
		sizeof(sDisdrometer->disdroDesc.disdroMakeModel));
  ptr += sizeof(sDisdrometer->disdroDesc.disdroMakeModel);
  memcpy(ptr, &sDisdrometer->disdroDesc.disdroNumber, 
		sizeof(sDisdrometer->disdroDesc.disdroNumber));
  ptr += sizeof(sDisdrometer->disdroDesc.disdroNumber);
  memcpy(ptr, &sDisdrometer->disdroDesc.disdroLat, 
		sizeof(sDisdrometer->disdroDesc.disdroLat));
  ptr += sizeof(sDisdrometer->disdroDesc.disdroLat);
  memcpy(ptr, &sDisdrometer->disdroDesc.disdroLong, 
		sizeof(sDisdrometer->disdroDesc.disdroLong));
  ptr += sizeof(sDisdrometer->disdroDesc.disdroLong);
  memcpy(ptr, &sDisdrometer->disdroDesc.range, 
		sizeof(sDisdrometer->disdroDesc.range));
  ptr += sizeof(sDisdrometer->disdroDesc.range);
  memcpy(ptr, &sDisdrometer->disdroDesc.azimuth, 
		sizeof(sDisdrometer->disdroDesc.azimuth));
  ptr += sizeof(sDisdrometer->disdroDesc.azimuth);
  memcpy(ptr, &sDisdrometer->disdroDesc.resolution,
                sizeof(sDisdrometer->disdroDesc.resolution));
  ptr += sizeof(sDisdrometer->disdroDesc.resolution);
  memcpy(ptr, &sDisdrometer->disdroDesc.elevation,
                sizeof(sDisdrometer->disdroDesc.elevation));


  vdataWrite(&tempVdata, databuf);
  free(databuf);

  sdsWrite(&tempSDS, &sDisdrometer->hour);
  sdsWrite(&tempSDS, &sDisdrometer->minute);
  sdsWrite(&tempSDS, &sDisdrometer->nConcentration);

  return(TK_SUCCESS);
}



/***********************************************************************************
*
*  FUNCTION STATEMENT:
*      readDisdrometer(IO_HANDLE *sGranuleHandle, L2A_57_DISDROMETER *sDisdrometer)
*
*  DESCRIPTION:
*      This function is used to read L2A57 Disdrometer information in HDF format.
*
*  INPUT PARAMETERS:
*      Parameter Name           Description
*      --------------           -----------
*      sGranuleHandle           A structure containing information about the file.
*                               sGranuleHandle is returned by TKopen().
*
*      sDisdrometer             A structure containing the complete scanline data.
*                               This structure must be declared to correspond to
*                               the data file being written.  The structure of
*                               sDisdrometer is manipulated by TKreadGauge() and
*                               other toolkit routines.
*
*  OUTPUT PARAMETERS:           None.
*
*  CALLED BY:                   TKreadGauge()
*
*  ROUTINES CALLED:
*      Routine Name             Description
*      ------------             -----------
*      allocMem                 This routine is used to allocate memory to vdata.
*      vdataRead                This routine is used to read the vdata in HDF
*                               format.
*      sdsRead                  This routine is used to read the scientific data
*                               set in HDF format.
*
**********************************************************************************/

int readDisdrometer(IO_HANDLE *sGranuleHandle, L2A_57_DISDROMETER *sDisdrometer)
{
  VDATA_LINK     *tempVdata = sGranuleHandle->cfgSTR->vGroup->vdata->next;
  SDS_LINK       *tempSDS = sGranuleHandle->cfgSTR->vGroup->sds;
  uint8          *databuf, *ptr;
  int            i,j;
  int scaleFactor;

  /* Allocate memory for unpacking the vdata */
  databuf = allocateVdataMemory(&tempVdata->vdataFields);
  ptr = databuf;
  /* Unpack the content of databuf into disdrometer.disdroDesc */
  vdataRead(&tempVdata, ptr);
  memcpy(&sDisdrometer->disdroDesc.networkID, ptr, 
           sizeof(sDisdrometer->disdroDesc.networkID));
  ptr += sizeof(sDisdrometer->disdroDesc.networkID);
  memcpy(&sDisdrometer->disdroDesc.disdroName, ptr,
           sizeof(sDisdrometer->disdroDesc.disdroName));
  ptr += sizeof(sDisdrometer->disdroDesc.disdroName);
  memcpy(&sDisdrometer->disdroDesc.disdroMakeModel, ptr,
           sizeof(sDisdrometer->disdroDesc.disdroMakeModel));
  ptr += sizeof(sDisdrometer->disdroDesc.disdroMakeModel);
  memcpy(&sDisdrometer->disdroDesc.disdroNumber, ptr,
           sizeof(sDisdrometer->disdroDesc.disdroNumber));
  ptr += sizeof(sDisdrometer->disdroDesc.disdroNumber);
  memcpy(&sDisdrometer->disdroDesc.disdroLat, ptr,
           sizeof(sDisdrometer->disdroDesc.disdroLat));
  ptr += sizeof(sDisdrometer->disdroDesc.disdroLat);
  memcpy(&sDisdrometer->disdroDesc.disdroLong, ptr,
           sizeof(sDisdrometer->disdroDesc.disdroLong));
  ptr += sizeof(sDisdrometer->disdroDesc.disdroLong);
  memcpy(&sDisdrometer->disdroDesc.range, ptr,
           sizeof(sDisdrometer->disdroDesc.range));
  ptr += sizeof(sDisdrometer->disdroDesc.range);
  memcpy(&sDisdrometer->disdroDesc.azimuth, ptr,
           sizeof(sDisdrometer->disdroDesc.azimuth));
  ptr += sizeof(sDisdrometer->disdroDesc.azimuth);
  memcpy(&sDisdrometer->disdroDesc.resolution, ptr,
           sizeof(sDisdrometer->disdroDesc.resolution));
  ptr += sizeof(sDisdrometer->disdroDesc.resolution);
  memcpy(&sDisdrometer->disdroDesc.elevation, ptr,
           sizeof(sDisdrometer->disdroDesc.elevation));
  free(databuf);

  sdsRead(&tempSDS, &scaleFactor, &sDisdrometer->hour);
  sdsRead(&tempSDS, &scaleFactor, &sDisdrometer->minute);
  sdsRead(&tempSDS, &scaleFactor, &sDisdrometer->nConcentration);
 
  return(TK_SUCCESS);
}



/***********************************************************************************
*
*  FUNCTION STATEMENT:
*	writeNetworkDesc(IO_HANDLE *sGranuleHandle, int nnet, NETDESC *sNetDesc)
*
*  DESCRIPTION:
*       This function is used to write L2A56 or L2A57 Network Descriptor information 
*	in HDF format.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       sGranuleHandle          A structure containing information about the file
*                               sGranuleHandle is returned by TKopen().
*	nnet			The number of networks at a radar site.
*       sNetDesc		A structure containing the complete scanline data.
*                               This structure must be declared to correspond to
*                               the data file being written.  The structure of
*                               sNetDesc is manipulated by TKwriteNetHeader() and
*                               other toolkit routines.
*
*  OUTPUT PARAMETERS:           None
*
*  CALLED BY:                   TKwriteNetHeader()
*
*  ROUTINES CALLED:
*       Routine Name            Description
*       ------------            -----------
*       allocMem                This routine is used to allocate memory to vdata.
*       VSwrite                 This routine is used to write the vdata in HDF
*                               format.
***********************************************************************************/

int writeNetworkDesc(IO_HANDLE *sGranuleHandle, int nnet, NETDESC *sNetDesc)
{
  VDATA_LINK    *tempVdata = sGranuleHandle->cfgSTR->vGroup->vdata;            
  uint8         *databuf, *ptr;
  int           i,j;

  /* Allocate memory for packing the vdata */
  databuf = allocMem(&tempVdata->vdataFields, nnet);
  ptr = databuf;

  for (i = 0; i < nnet; ++i)
  {
    memcpy(ptr, &sNetDesc->networkName, sizeof(sNetDesc->networkName));
    ptr += sizeof(sNetDesc->networkName);
    memcpy(ptr, &sNetDesc->nValidSensor, sizeof(sNetDesc->nValidSensor));
    ptr += sizeof(sNetDesc->nValidSensor);
    memcpy(ptr, &sNetDesc->networkID, sizeof(sNetDesc->networkID));
    ptr += sizeof(sNetDesc->networkID);

    sNetDesc++;
  }

  if (VSwrite(tempVdata->vdataID, databuf, nnet, FULL_INTERLACE) == TK_HDF_FAIL)
  {
	/* Failed VSwrite in writeNetworkDesc(). */
	TKreportError(E_TK_FAILVSWTNETHDR);
  } 

  free(databuf);

  return(TK_SUCCESS);
}



/*****************************************************************************
*
*  FUNCTION STATEMENT:
*        readNetworkDesc(IO_HANDLE *sGranuleHandle, int nnet, NETDESC *sNetDesc)
*
*  DESCRIPTION:
*        This function is used to read L2A56 or L2A57 Network Descriptor information
*        in HDF format.
*
*    INPUT PARAMETERS:
*        Parameter Name          Description
*        --------------          -----------
*        sGranuleHandle          A structure containing information about the file
*                                sGranuleHandle is returned by TKopen().
*        nnet                    The number of networks at a radar site.
*        sNetDesc                A structure containing the complete scanline data.
*                                This structure must be declared to correspond to
*                                the data file being written.  The structure of
*                                sNetDesc is manipulated by TKreadNetHeader() and
*                                other toolkit routines.
*
*  OUTPUT PARAMETERS:            None.
*
*  CALLED BY:                    TKreadNetHeader()
*
*  ROUTINES CALLED:
*        Routine Name            Description
*        ------------            -----------
*        allocMem                This routine is used to allocate memory to vdata.
*        VSread                   This routine is used to read the vdata in HDF
*                                format.
*
************************************************************************************/

int readNetworkDesc(IO_HANDLE *sGranuleHandle, int nnet, NETDESC *sNetDesc)
{
   VDATA_LINK    *tempVdata = sGranuleHandle->cfgSTR->vGroup->vdata;
   uint8          *databuf, *ptr;
   int            i,j;


   /* Allocate memory for packing the vdata */
  databuf = allocMem(&tempVdata->vdataFields, nnet);
  ptr = databuf;


  if (VSread(tempVdata->vdataID, databuf, nnet, FULL_INTERLACE) == TK_HDF_FAIL)
  {
        /* Failed VSread in readNetworkDesc() */
        TKreportError(E_TK_VREADFAIL);
  }

  for (i = 0; i < nnet; ++i)
  {
     memcpy(&sNetDesc->networkName, ptr, sizeof(sNetDesc->networkName));
     ptr += sizeof(sNetDesc->networkName);
     memcpy(&sNetDesc->nValidSensor, ptr, sizeof(sNetDesc->nValidSensor));
     ptr += sizeof(sNetDesc->nValidSensor);
     memcpy(&sNetDesc->networkID, ptr, sizeof(sNetDesc->networkID));
     ptr += sizeof(sNetDesc->networkID);

     sNetDesc++;
  }

  free(databuf);

  return(TK_SUCCESS);
}


/*****************************************************************************
*
*  FUNCTION STATEMENT:
*       updateTemplate(L1_GV_TEMPLATE **templateList, int32 inputFile,
*                       char *filename)
*
*  DESCRIPTION:
*       Appends VOS info from a single_VOS_template_node into a
*       multi_VOS_template_node, then deletes the single_VOS_template_node.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       templateList            The list of templates.
*       inputFile               InputFile is used to search a template which
*                               has matched inputFile from the templateList.
*       filename                Name of the HDF file attached to a
*                               template_node.
*
*  OUTPUT PARAMETERS:           None.
*
*  CALLED BY:                   manageL1GVtemplate()
*
*  RETURN VALUES:               TK_SUCCESS if the routine is successful.
*
*  ROUTINES CALLED:
*       Routine Name            Description
*       ------------            -----------
*       TKreportError           This routine reports fatal errors to console
*                               and terminates all processes.
*
******************************************************************************/

int32 updateTemplate(L1_GV_TEMPLATE **templateList, int32 inputFile,
                        char *filename)
{
  /*
  ** tempListU : template to be updated with info from tempListD
  */
  L1_GV_TEMPLATE *tempListU = *templateList;
  /*
  ** tempListD : template to be read and then deleted
  */
  L1_GV_TEMPLATE *tempListD = *templateList;
  L1_GV_TEMPLATE *tempListB = *templateList;
  int32 j, vos_num;

  /*
  ** Search thru the template list for the template node to be updated.
  ** Exit out of the loop when found. This template node was created
  ** by openInputVgroupL1GV() upon opening an HDF file.
  */
  while (tempListU != NULL)
  {
        if (tempListU->inputFile == inputFile)
                break;
        else
                tempListU = tempListU->next;
  }

  /*
  ** Find the single_VOS_template node which needs to be read and deleted.
  ** Exit out of the loop when found. This template node was previously
  ** created by a call to TKsetL1GVtemplate() after reading a VOS.
  */
  while (tempListD != NULL)
  {
        if ((tempListD->inputFile == -1) &&
            (strcmp(tempListD->filename, filename) == 0))
                break;
        else
        {
                /*
                ** Save the node one back for possible later use in
                ** deleting tempListD.
                */
                tempListB = tempListD;
                tempListD = tempListD->next;
        }
  }

  if ((tempListU != NULL) && (tempListD != NULL))
  {
        /*
        ** Add the VOS info from 'tempListD' to 'tempListU'.
        */
        vos_num = tempListU->numOfVOS;
        tempListU->numOfVOS += 1;
        tempListU->numOfParm[vos_num] = tempListD->numOfParm[0];
        tempListU->nsweep[vos_num] = tempListD->nsweep[0];
        tempListU->nray[vos_num] = tempListD->nray[0];
        for (j=0; j<tempListD->numOfParm[0]; j++)
            tempListU->ncell[vos_num][j] = tempListD->ncell[0][j];

        /*
        ** Unlink 'tempListD' from the linked list.
        */
        if (tempListD == *templateList)  /* First node in list? */
            /*
            ** Unlink tempListD by making the 2nd node the 1st in the list.
            ** We know there exists such a 2nd node.
            */
            *templateList = tempListD->next;
        else
            /*
            ** tempListD is somewhere in middle of list.
            */
            tempListB->next = tempListD->next;

        free(tempListD);
  }
  else
  {
        TKreportError(E_TK_BADINPUTDT);
  }

  return (TK_SUCCESS);
}


/************************************************************************************
*  FUNCTION STATEMENT:
*	bufManage(uint8 **databuf, int *bufsize, int new_bufsize)
*
*  DESCRIPTION:
*       Perform buffer management tasks.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       databuf         	Pointer to data buffer.
*       bufsize                 Size of data buffer in bytes.
*       new_bufsize   		Size of buffer required for next vdata(or sds)
*
*  OUTPUT PARAMETERS:           None
*
*  CALLED BY:                   writeGV_L1B_1C, readGV_L1B_1C
*
*  RETURN VALUES:        	databuf , bufsize
*
*  ROUTINES CALLED:		N/A
*
***********************************************************************************/
void bufManage(uint8 **databuf, int *bufsize, int new_bufsize)
{
  /*  If we've found a vdata (or sds) that requires a bigger buffer than we've
   *  previously allocated, free the old buffer and allocate a bigger one.
   *  This situation will never normally arise. The following if statement is
   *  merely a safety factor.
   */
  if (*bufsize < new_bufsize)
  {
     free(*databuf);
     /* Allocate memory for the new (larger) buffer needed. */
     *bufsize = new_bufsize;
     *databuf = (uint8 *) malloc(*bufsize);
     if (databuf == NULL)
	/* Memory allocation failed in bufManage(). */
	TKreportError(E_TK_MEMFAILBUFMANAGE);
  }

  /* 
   * Clear the first portion of the buffer needed for the vdata(sds). 
   */
  memset(*databuf, '\0', new_bufsize);
}

/***********************************************************************************
*
*  FUNCTION STATEMENT:
*       setScalingFactors_2A53(VGROUP_LINK *vgroup)
*
*  DESCRIPTION:
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       vgroup                  List of vgroups.  The list of SDSs are retrieve from
*                               this node.
*
*  OUTPUT PARAMETERS:  None.
*
*  CALLED BY:                   setSDSattributes().
*
*  ROUTINES CALLED:     
*       Routine Name            Description
*       ------------            -----------
*       SDsetcal                This routine adds the scaling factor and offset 
*                               to the SDS.
*       TKreportWarning         This routine is called for warnings and informational
*                               messages.  Calling this routine will send an error
*                               message to the central log file and return control to 
*                               the calling routine.
*
***********************************************************************************/
int setScalingFactors_2A53 ( VGROUP_LINK *vgroup )
{

  int status = TK_SUCCESS;
  SDS_LINK *sds;

  if ( vgroup != NULL )
    {
      sds = vgroup->sds;
      while ( sds != NULL )
	{
	  if ( !strcmp ( sds->name, "rainRate" ) )
	    status = SDsetcal ( sds->sdsID, (float64) L2A53_RAINRATE, 
			       (float64) 0, (float64) 0, (float64) 0, 
			       DFNT_INT16 );

	  if ( status == TK_HDF_FAIL )
	    TKreportWarning ( E_TK_SETCALINFO );
	  sds = sds->next;
	}
      setScalingFactors_2A53 ( vgroup->subVgroup );
      setScalingFactors_2A53 ( vgroup->next );
    }
  return status;
}

/***********************************************************************************
*
*  FUNCTION STATEMENT:
*       setScalingFactors_2A55(VGROUP_LINK *vgroup)
*
*  DESCRIPTION:
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       vgroup                  List of vgroups.  The list of SDSs are retrieve from
*                               this node.
*
*  OUTPUT PARAMETERS:  None.
*
*  CALLED BY:                   setSDSattributes().
*
*  ROUTINES CALLED:     
*       Routine Name            Description
*       ------------            -----------
*       SDsetcal                This routine adds the scaling factor and offset 
*                               to the SDS.
*       TKreportWarning         This routine is called for warnings and informational
*                               messages.  Calling this routine will send an error
*                               message to the central log file and return control to 
*                               the calling routine.
*
***********************************************************************************/
int setScalingFactors_2A55 ( VGROUP_LINK *vgroup )
{

  int status = TK_SUCCESS;
  SDS_LINK *sds;

  if ( vgroup != NULL )
    {
      sds = vgroup->sds;
      while ( sds != NULL )
	{
	  if ( !strcmp ( sds->name, "threeDreflect" ) )
	    status = SDsetcal ( sds->sdsID, (float64) L2A55_THREEDREFLECT, 
			       (float64) 0, (float64) 0, (float64) 0, 
			       DFNT_INT16 );
	  else if ( !strcmp ( sds->name, "vertProfile" ) )
	    status = SDsetcal ( sds->sdsID, (float64) L2A55_VERTICALPROFILE, 
			       (float64) 0,  (float64) 0, (float64) 0, 
			       DFNT_INT16 );

	  if ( status == TK_HDF_FAIL )
	    TKreportWarning ( E_TK_SETCALINFO );
	  sds = sds->next;
	}
      setScalingFactors_2A55 ( vgroup->subVgroup );
      setScalingFactors_2A55 ( vgroup->next );
    }
  return status;
}

/***********************************************************************************
*
*  FUNCTION STATEMENT:
*       setScalingFactors_2A56(VGROUP_LINK *vgroup)
*
*  DESCRIPTION:
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       vgroup                  List of vgroups.  The list of SDSs are retrieve from
*                               this node.
*
*  OUTPUT PARAMETERS:  None.
*
*  CALLED BY:                   setSDSattributes().
*
*  ROUTINES CALLED:     
*       Routine Name            Description
*       ------------            -----------
*       SDsetcal                This routine adds the scaling factor and offset 
*                               to the SDS.
*       TKreportWarning         This routine is called for warnings and informational
*                               messages.  Calling this routine will send an error
*                               message to the central log file and return control to 
*                               the calling routine.
*
***********************************************************************************/
int setScalingFactors_2A56 ( VGROUP_LINK *vgroup )
{

  int status = TK_SUCCESS;
  SDS_LINK *sds;

  if ( vgroup != NULL )
    {
      sds = vgroup->sds;
      while ( sds != NULL )
	{
	  if ( !strcmp ( sds->name, "meanRainRate" ) )
	    status = SDsetcal ( sds->sdsID, (float64) L2A56_MEANRAINRATE, 
			       (float64) 0, (float64) 0, (float64) 0, 
			       DFNT_INT16 );

	  if ( status == TK_HDF_FAIL )
	    TKreportWarning ( E_TK_SETCALINFO );
	  sds = sds->next;
	}
      setScalingFactors_2A56 ( vgroup->subVgroup );
      setScalingFactors_2A56 ( vgroup->next );
    }
  return status;
}

/***********************************************************************************
*
*  FUNCTION STATEMENT:
*       setScalingFactors_3A53(VGROUP_LINK *vgroup)
*
*  DESCRIPTION:
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       vgroup                  List of vgroups.  The list of SDSs are retrieve from
*                               this node.
*
*  OUTPUT PARAMETERS:  None.
*
*  CALLED BY:                   setSDSattributes().
*
*  ROUTINES CALLED:     
*       Routine Name            Description
*       ------------            -----------
*       SDsetcal                This routine adds the scaling factor and offset 
*                               to the SDS.
*       TKreportWarning         This routine is called for warnings and informational
*                               messages.  Calling this routine will send an error
*                               message to the central log file and return control to 
*                               the calling routine.
*
***********************************************************************************/
int setScalingFactors_3A53 ( VGROUP_LINK *vgroup )
{

  int status = TK_SUCCESS;
  SDS_LINK *sds;

  if ( vgroup != NULL )
    {
      sds = vgroup->sds;
      while ( sds != NULL )
	{
	  if ( !strcmp ( sds->name, "pentadRainfall" ) )
	    status = SDsetcal ( sds->sdsID, (float64) L3A53_PENTADRAINFALL, 
			       (float64) 0, (float64) 0, (float64) 0, 
			       DFNT_INT16 );

	  if ( status == TK_HDF_FAIL )
	    TKreportWarning ( E_TK_SETCALINFO );
	  sds = sds->next;
	}
      setScalingFactors_3A53 ( vgroup->subVgroup );
      setScalingFactors_3A53 ( vgroup->next );
    }
  return status;
}

/***********************************************************************************
*
*  FUNCTION STATEMENT:
*       setScalingFactors_3A55(VGROUP_LINK *vgroup)
*
*  DESCRIPTION:
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       vgroup                  List of vgroups.  The list of SDSs are retrieve from
*                               this node.
*
*  OUTPUT PARAMETERS:  None.
*
*  CALLED BY:                   setSDSattributes().
*
*  ROUTINES CALLED:     
*       Routine Name            Description
*       ------------            -----------
*       SDsetcal                This routine adds the scaling factor and offset 
*                               to the SDS.
*       TKreportWarning         This routine is called for warnings and informational
*                               messages.  Calling this routine will send an error
*                               message to the central log file and return control to 
*                               the calling routine.
*
***********************************************************************************/
int setScalingFactors_3A55 ( VGROUP_LINK *vgroup )
{

  int status = TK_SUCCESS;
  SDS_LINK *sds;

  if ( vgroup != NULL )
    {
      sds = vgroup->sds;
      while ( sds != NULL )
	{
	  if ( !strcmp ( sds->name, "vertProfile" ) )
	    status = SDsetcal ( sds->sdsID, (float64) L3A55_VERTICALPROFILE, 
			       (float64) 0, (float64) 0, (float64) 0, 
			       DFNT_INT16 );

	  if ( status == TK_HDF_FAIL )
	    TKreportWarning ( E_TK_SETCALINFO );
	  sds = sds->next;
	}
      setScalingFactors_3A55 ( vgroup->subVgroup );
      setScalingFactors_3A55 ( vgroup->next );
    }
  return status;
}
