/* DEVELOPMENT HISTORY */
/* Revision 1.59  1999/11/12 21:23:43  pkolbe
 * further fix of 2b31 rrSurf scaling
 *
 * Revision 1.58  1999/11/12 21:09:52  pkolbe
 * corrected a problem with 2b31 rrSurf scaling
 *
 * Revision 1.57  1999/09/22 19:50:19  pkolbe
 * changed the order of the 3b31 structure elements
 *
 * Revision 1.56  1999/09/14 21:05:36  pkolbe
 * corrected a 2a25 scaling factor read
 *
 * Revision 1.55  1999/09/13 18:12:41  pkolbe
 * finished backward compatability code changes
 *
 * Revision 1.54  1999/08/31 21:34:06  pkolbe
 *  further 3b31 struct changes
 *
 * Revision 1.53  1999/08/24 15:38:16  pkolbe
 * further changes of the 3b31 structure
 *
 * Revision 1.52  1999/08/16 20:25:49  pkolbe
 * added elements (no subtractions, yet) to the 3b31 structure
 *
 * Revision 1.51  1999/08/16 17:42:01  pkolbe
 * added pia2a25Mean, pia2a25Dev, pia2a25H to 3a25 grid 1 structure
 *
 * Revision 1.50  1999/08/13 17:01:27  pkolbe
 * added dynamic scaling
 *
 * Revision 1.49  1998/09/01 13:43:01  pkolbe
 * added 3A25 data elements
 *
 * Revision 1.48  1998/08/05 17:26:45  pkolbe
 * changed name "nearSurfHeight" to "nearSurfBin" in 2a25 real time 1;
 * also changed the size from 2 bytes to 1 byte
 *
 * Revision 1.47  1998/07/09 14:58:30  slywczak
 * Modified routine setScalingFactors_2A23 and setScalingFactors_2A25.  Added the
 * real time information for both the 2A23 and 2A25 algorithms.  Combined both
 * the normal processing and real time processing into one routine.
 *
 * Revision 1.46  1998/06/26 20:45:23  pkolbe
 * added sdsWrite of real time 1 variable nearSurfHeight
 *
 * Revision 1.45  1998/06/26 17:29:19  slywczak
 * Added new read/write routines for the new real time products (2A23_R1,
 * 2A25_R1, and 2A25_R2).
 *
 * Revision 1.44  1998/05/21 17:56:59  pkolbe
 * 3A25 20->30 size changes and 3A26 srt-2A25 name changes
 *
 * Revision 1.43  1998/04/27 13:31:09  slywczak
 * Modified routine readGrid_3A_26 and writeGrid_3A_26.  Added support to read
 * and write the rainMeanTH element.
 *
 * Revision 1.42  1998/04/23 15:58:00  slywczak
 * Modified the read/write routines for 1B21/1C21 and added the following
 * elements:  binEllipsoid, binClutterFreeBottom, binDIDHmean, binDIDHtop,
 * and binDIDHbottom.  Modified the read/write routines for 2A23 and added the
 * following elements:  BBintensity and spare.  Modified the read/write routines
 * for 2A25 and added the following elements:  nearSurfRain, nearSurfZ, pia2a25,
 * errorRain, and errorZ.
 *
 * Revision 1.41  1998/04/22 13:45:57  pkolbe
 *  additions/modifications to planetary grid 1 and 2 structures
 *
 * Revision 1.40  1997/06/20 20:19:16  slywczak
 * Modified function readGrid_3A_25 and writeGrid_3A_25.  Added variables for
 * the bright band pixel numbers (high and low)
 *
 * Revision 1.39  1997/05/29 20:50:36  slywczak
 * Modified the routine readScan_1B_21.  Change the equals (=) to equivalence
 * (==) on if check for Oversurface Sample.
 *
 * Revision 1.38  1997/05/29 16:45:30  slywczak
 * Modified the following routines:  readScan_1B_21 and writeScan_1B_21.
 * Added the missing rain value (-32700) to the scaling factor operation;  the
 * routines will not scale this value, but only convert it to the appropriate
 * type.
 *
 * Revision 1.37  1997/05/19 03:43:00  slywczak
 * Modified the following routines:  readGrid_3A_25, writeGrid_3A_25,
 * readGrid_3A_26, writeGrid_3A_26, readWrite_3B_31, and writeGrid_3B_31.
 * Removed references to the gridStruct structure, since this structure will be
 * referenced in the toolkit code.
 *
 * Revision 1.36  1997/05/15 14:35:31  slywczak
 * Added the following routines:  setScalingFactors_1B21, setScalingFactors_2A21,
 * setScalingFactors_2A25, and setScalingFactors_2B31.  These routines will write
 * the scaling factors to the HDF files.
 *
 * Revision 1.35  1997/03/14 20:18:32  slywczak
 * Modified function readScan_1B_21.  Included the code which scaled the missing
 * values for the readScan_1B_21 routine.
 *
 * Revision 1.34  1997/03/10 16:29:56  slywczak
 * Modified function writeScan_2B_31 and readScan_2B_31.  Since the dimensions of
 * the structure elements, TMI-PIA and sigma-TMI-PIA changed from 104 to 49, the
 * corresponding scaling code had change in the above routines.  The number of
 * pixels that had to be scaled is now 49.
 *
 * Revision 1.33  1997/02/21 23:14:31  piper
 * Fixed code to properly handle missing values.
 *
 * Revision 1.33  1997/02/20 17:02:35  jt
 * Fixed scaling calculations for reading missing or invalid data.
 *
 * Revision 1.32  1997/02/13 17:51:44  piper
 * Fixed problem with missing values not being written to output files.
 *
 * Revision 1.31  1997/01/02 15:30:36  amak
 * Define TK_DEFAULT for hard-coded value.
 *
 * Revision 1.30  1996/12/24 14:30:34  amak
 * Incorporated walkthrough comments.
 * - Changed '<' and '>' to double quotes around the #include statements.
 *
 * Revision 1.29  1996/11/06  15:28:13  amak
 * Added detailed description to the routine.
 *
 * Revision 1.28  1996/10/24  13:47:58  pshah
 * Added code to take care of missing or invalid data for scaling
 *
 * Revision 1.27  1996/10/17  20:23:52  amak
 * Made modifications to functions writeScan_1B_21, readScan_1B_21,
 * writeScan_2A_21, readScan_2A_21, writeGrid_3A_26, and readGrid_3A_26 since
 * the structures for 1B21(1C21), 2A21, and 3A26 have been redefined.
 *
 * Revision 1.26  1996/08/13  18:58:48  amak
 * Modified L2A25 data structure.
 * - added 2-byte integer attenParmNode[49][5]
 * - changed dimension of parameter attenParmAlpha from 49x10 to 49x5
 * - changed dimension of parameter attenParmBeta from 49x10 to 49
 * - added 2-byte integer ZRParmNode[49][5]
 * - changed dimension of parameter ZRParmA from 49x10 to 49x5
 * - changed dimension of parameter ZRParmB from 49x10 to 49x5
 * - added 4-byte float zmmax[49]
 * - changed dimension of parameter rangeBinNum from 49x3 to 49x6
 * - changed dimension of parameter zeta from 49 to 49x2
 * - renamed zeta_m to zeta_mn, and changed dimension from 49 to 49x2
 * - renamed zeta_s to zeta_sd, and changed dimension from 49 to 49x2
 * - changed dimension of parameter xi from 49 to 49x2
 * - added 2-byte integer thickThPIZ[49]
 * - changed dimension of parameter nubfCorrectFactor from 49 to 49x2
 * - changed qualityFlag from 1-byte integer to 2-byte integer
 *
 * Revision 1.25  1996/07/11  17:36:39  pshah
 * In the routine writeScan_2B_31 the sigmaPIA part was wrongly multipled by 
 * L2B31_EPSILON, rectified it to L2B31_SIGMAPIA
 *
 * Revision 1.24  1996/06/25  15:14:42  amak
 * Changed array dimension of R-hat and Sigma-R-hat in L2B31 from 60x49 to
 * 80x49.
 *
 * Revision 1.23  1996/06/20  19:33:54  amak
 * Modified L2B31 structure:
 *   - deleted S-hat and Sigma-S-hat
 *   - added RR-Surf and Sigma-RR-Surf and do scaling calculation for these
 *     two fields.
 *
 * Revision 1.22  1996/06/14  21:55:37  amak
 * Remodified routines for write/read L2A25 header.
 *
 * Revision 1.21  1996/06/13  20:42:14  amak
 * Fixed syntax errors for writing and reading L1B21, L1C21, and L2A25 headers.
 *
 * Revision 1.20  1996/06/13  14:00:21  amak
 * Modified writeL1B21_L1C21Header(), readL1B21_L1C21Header(), writeL2A25Header(),
 * and readL2A25Header() to fixed syntax errors.
 *
 * Revision 1.19  1996/06/07  14:35:39  amak
 * Added 'prStatus1' and 'prStatus2' fields to Scan Status block.
 *
 * Revision 1.18  1996/05/21  18:03:55  amak
 * Incorporated the change of L1B-21 and L3A-25 data structures into the code.
 *
 * Revision 1.17  1996/05/10  13:46:18  amak
 * Added routines for reading and writing L1B21, L1C21, and L2A25 headers.
 *
 * Revision 1.16  1996/04/19  15:40:46  amak
 * Made modifications to the preprocessor #include statement.  Replaced
 * TKerrorRou.h and TKwarnRou.h to TKerrHandle.h and TS_TK_15.h for new
 * error handling calls.
 *
 * Revision 1.15  1996/03/21  21:59:33  amak
 * Added routines writeScan_2B_31() and readScan_2B_31 to handle L2B31 algorithm
 * for reading and writing.
 *
 * Revision 1.14  1996/03/15  22:24:03  amak
 * Added entity sysNoiseWarnFlag to L1B-21 structure.
 *
 * Revision 1.13  1996/03/15  19:33:43  amak
 * Modified write_1B_21() and read_1B_21() routines which based on the four
 * newly added variables (ttlPower, temp, tdaPower, and logAmpInNoise) in
 * POWERs structure.
 *
 * Revision 1.12  1996/03/07  21:32:48  amak
 * For product 3A-25 and 3A-26, the radar structure had been deleted.  The
 * corresponding changes had been made on functions readGrid_3A_25(),
 * readGrid_3A_26(), write_3A_25(), and write3A_26().
 * Modified readGrid_3B_31() and writeGrid_3B_31() based on the new data
 * structure.
 *
 * Revision 1.11  1996/02/08  16:49:37  amak
 * Modifed code based on the change of L2A-25 data structure.
 * - ncell1 changed from 60 to 80
 * - ncell2 changed from 5 to 10
 * - Deleted Clutter Flag; stored in header identical to 1B-21
 * - Reliability changed from 2 bytes to 1 byte integer
 * - Attenuation Parameter Beta increased to every ncell2, and moved below
 *   Attenuation Parameter Alpha
 * - Doubled Average Rain Rate
 * - Added NUBF Correction Factor
 * - Added Quality Flag
 *
 * Revision 1.10  1996/01/25  15:06:23  amak
 * Called macro function RND() when doing the roundup from floating point to
 * integer type.
 *
 * Revision 1.9  1995/12/18  22:25:52  amak
 * Added 0.5 to all the write scaling calculations.  This adding is to avoid
 * the round up problems while computing on various platforms.
 *
 * Revision 1.8  1995/12/12  14:49:04  amak
 * Used macro factors to do scaling calculations.
 *
 * Revision 1.7  1995/12/05  18:35:02  thyun
 * Modified assignment of tempSDS and tempVdata which are appearing in declaration
 * of all the functions.
 *
 * Revision 1.6  1995/11/30  21:12:42  amak
 * Fixed syntax errors.
 *
 * Revision 1.5  1995/11/28  21:34:39  amak
 * Fixed syntax errors of the scaling calculation.
 *
 * Revision 1.4  1995/11/21  14:20:58  amak
 * Fixed syntax errors.
 *
 * Revision 1.3  1995/11/20  17:52:38  amak
 * Added scaling calculations to the routines.
 *
 * Revision 1.2  1995/10/18  20:52:22  amak
 * Made modifications after some files need to be renamed.
 *
 * Revision 1.1  1995/10/17  19:06:49  mla
 * Renamed IO_INTERNAL_PR.c to IO_INTR_PR.c
 *
 * Revision 1.33  1995/10/10  15:12:37  thyun
 * Commented out the part of writeScan_1B_21 and readScan_1B_21.
 *
 * Revision 1.32  1995/09/12  15:17:29  amak
 * Added 4 more fields in L1B-21.
 *
 * Revision 1.31  1995/08/10  18:13:14  thyun
 * Modified readGrid_3A_25, writeGrid_3A_25, readGrid_3A_26, and writeGrid_3A_26
 * to resolve some semantic errors.  These errors were found during the unit
 * testing.
 *
 * Revision 1.30  1995/08/08  20:14:36  amak
 * Added L3A-25 and L3A-26 coding.
 *
 * Revision 1.29  1995/08/07  15:44:42  amak
 * Shortened the variable names in the navigate structure.
 *
 * Revision 1.28  1995/08/04  19:53:54  amak
 * Put the variable name into the parameter instead of hardcoding a size.
 *
 * Revision 1.27  1995/08/03  17:45:37  amak
 * Re-modified the code because CVS couldn't do version control nicely.
 *
 * Revision 1.26  1995/08/01  20:26:46  thyun
 * Updated the structure names such as VDATA_LINK and SDS_LINK as well as
 * variable names.  These changes occurred becase of changes in parsing.
 *
 * Revision 1.24  1995/07/20  14:18:38  amak
 * Added routines readGrid_3A_25(), readGrid_3A_26(), readGrid_3B_31(), writeGrid3A_25(),
 * writeGrid_3A_26(), and writeGrid_3B_31() for Level 3 API.
 *
 * Revision 1.23  1995/07/19  20:12:16  thyun
 * Changed the parameters of readScan_2B_31 and writeScan_2B_31.
 *
 * Revision 1.22  1995/07/18  14:32:41  amak
 * Renamed the routines from vWrite, vRead, sWrite and sRead to vdataWrite,
 * vdataRead, sdsWrite and sdsRead respectively.
 *
 * Revision 1.21  1995/07/08  14:08:31  amak
 * Took out Level 3A-25, 3A-26, and 3B-31 prototypes for the change
 * of the API for Level 3 satellite products.
 *
 * Revision 1.20  1995/07/07  21:12:24  amak
 * Added more detailed comments
 *
 * Revision 1.19  1995/07/06  17:20:50  amak
 * Made modification for the new structure -- Swathdata 
 * */

/********************************************************************************
*
*  FILE NAME:           IO_INTR_PR.c
*
*  DESCRIPTION:
*       This file contains routines that read and PR headers and PR data.
*
*  ROUTINES CALLED:
*       Routine Name            Description
*       ------------            -----------
*       writeScan_1B_21         This function is used to write the L1B-21 
*                               information in HDF format.
*       readScan_1B_21          This function is used to read the L1B-21
*                               information in HDF format.
*       writeScan_2A_21         This function is used to write the L2A-21
*                               information in HDF format.
*       readScan_2A_21          This function is used to read the L2A-21
*                               information in HDF format.
*       writeScan_2A_23         This function is used to write the L2A-23
*                               information in HDF format.
*       readScan_2A_23          This function is used to read the L2A-23
*                               information in HDF format.
*       writeScan_2A_25         This function is used to write the L2A-25
*                               information in HDF format.
*       readScan_2A_25          This function is used to read the L2A-25
*                               information in HDF format.
*       writeScan_2B_31         This function is used to write the L2B-31
*                               information in HDF format.
*       readScan_2B_31          This function is used to read the L2B-31
*                               information in HDF format.
*       writeGrid_3A_25         This function is used to write the L3A-25
*                               information in HDF format.
*       readGrid_3A_25          This function is used to read the L3A-25
*                               information in HDF format.
*       writeGrid_3A_26         This function is used to write the L3A-26
*                               information in HDF format.
*       readGrid_3A_26          This function is used to read the L3A-26
*                               information in HDF format.
*       writeGrid_3B_31         This function is used to write the L3B-31
*                               information in HDF format.
*       readGrid_3B_31          This function is used to read the L3B-31
*       allocateVdataMemory     This routine is used to allocate memory to vdata.
*       vdataWrite		This routine is used to write the vdata in HDF 
*                               format.
*       sdsWrite		This routine is used to write the scientific data
*                               set in HDF format.      
*       sdsRead			This routine is used to read the scientific data
*                               set in HDF format.      
*       vdataRead		This routine is used to read the vdata in HDF 
*                               format.
*
********************************************************************************/

#include "IO_PR.h"
#include "IO_INTR_PR.h"
#include "hdf_routines.h"
#include "TKerrHandle.h"
#include "TS_TK_15.h"

/***********************************************************************************
*
*  FUNCTION STATEMENT:
*       writeScan_1B_21(IO_HANDLE *sGranuleHandle, L1B_21_SWATHDATA *sSwathData) 
*
*  DESCRIPTION:
*       This function is used to write the L1B-21 information in HDF format.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       sGranuleHandle          A structure containing information about the file.
*                               sGranuleHandle is returned by TKopen().
*       sSwathData              A structure containing the complete scanline data.
*                               This structure must be declared to correspond to 
*                               the data file being written.  The structures of 
*                               sSwathData is manipulated by TKwriteScan() and 
*				other toolkit routines.
*
*  OUTPUT PARAMETERS:           None
*
*  CALLED BY:                   TKwriteScan()
*
*  ROUTINES CALLED:     
*       Routine Name            Description
*       ------------            -----------
*       allocateVdataMemory     This routine is used to allocate memory to vdata.
*       vdataWrite              This routine is used to write the vdata in HDF 
*                               format.
*       sdsWrite                This routine is used to write the scientific data
*                               set in HDF format.      
*
***********************************************************************************/

int writeScan_1B_21(IO_HANDLE *sGranuleHandle, L1B_21_SWATHDATA *sSwathData)
{
  SDS_LINK *tempSDS = sGranuleHandle->cfgSTR->vGroup->subVgroup->sds;
  VDATA_LINK *tempVdata = sGranuleHandle->cfgSTR->vGroup->subVgroup->vdata;
  uint8 *databuf, *ptr;
  int	i, j;

  /* Allocate memory for packing the vdata */
  databuf = allocateVdataMemory(&tempVdata->vdataFields);
  ptr = databuf;
  memcpy(ptr, &sSwathData->scanTime, sizeof(sSwathData->scanTime));
  /* Writing packed vdata, scanTime, to an HDF file */
  vdataWrite(&tempVdata, databuf); 
  free(databuf);

  sdsWrite(&tempSDS, &sSwathData->geolocation);

  /* Allocate memory for packing the vdata */
  databuf = allocateVdataMemory(&tempVdata->vdataFields);
  ptr = databuf;
  memcpy(ptr, &sSwathData->scanStatus.missing,
                sizeof(sSwathData->scanStatus.missing));
  ptr += sizeof(sSwathData->scanStatus.missing);
  memcpy(ptr, &sSwathData->scanStatus.validity, 
		sizeof(sSwathData->scanStatus.validity));
  ptr += sizeof(sSwathData->scanStatus.validity);
  memcpy(ptr, &sSwathData->scanStatus.qac, 
		sizeof(sSwathData->scanStatus.qac));
  ptr += sizeof(sSwathData->scanStatus.qac);
  memcpy(ptr, &sSwathData->scanStatus.geoQuality, 
		sizeof(sSwathData->scanStatus.geoQuality));
  ptr += sizeof(sSwathData->scanStatus.geoQuality);
  memcpy(ptr, &sSwathData->scanStatus.dataQuality, 
		sizeof(sSwathData->scanStatus.dataQuality));
  ptr += sizeof(sSwathData->scanStatus.dataQuality);
  memcpy(ptr, &sSwathData->scanStatus.scOrient, 
		sizeof(sSwathData->scanStatus.scOrient));
  ptr += sizeof(sSwathData->scanStatus.scOrient);
  memcpy(ptr, &sSwathData->scanStatus.acsMode, 
		sizeof(sSwathData->scanStatus.acsMode));
  ptr += sizeof(sSwathData->scanStatus.acsMode);
  memcpy(ptr, &sSwathData->scanStatus.yawUpdateS, 
		sizeof(sSwathData->scanStatus.yawUpdateS));
  ptr += sizeof(sSwathData->scanStatus.yawUpdateS);
  memcpy(ptr, &sSwathData->scanStatus.prMode, 
		sizeof(sSwathData->scanStatus.prMode));
  ptr += sizeof(sSwathData->scanStatus.prMode);
  memcpy(ptr, &sSwathData->scanStatus.prStatus1,
                sizeof(sSwathData->scanStatus.prStatus1));
  ptr += sizeof(sSwathData->scanStatus.prStatus1);
  memcpy(ptr, &sSwathData->scanStatus.prStatus2,
                sizeof(sSwathData->scanStatus.prStatus2));
  ptr += sizeof(sSwathData->scanStatus.prStatus2);
  memcpy(ptr, &sSwathData->scanStatus.fractOrbitN, 
		sizeof(sSwathData->scanStatus.fractOrbitN));
  /* Writing packed vdata, scanStatus block, to an HDF file */
  vdataWrite(&tempVdata, databuf);
  free(databuf);

  /* Allocate memory for packing the vdata */
  databuf = allocateVdataMemory(&tempVdata->vdataFields);
  ptr = databuf;
  memcpy(ptr, &sSwathData->navigate.scPosX, sizeof(sSwathData->navigate.scPosX));
  ptr += sizeof(sSwathData->navigate.scPosX);
  memcpy(ptr, &sSwathData->navigate.scPosY, sizeof(sSwathData->navigate.scPosY));
  ptr += sizeof(sSwathData->navigate.scPosY);
  memcpy(ptr, &sSwathData->navigate.scPosZ, sizeof(sSwathData->navigate.scPosZ));
  ptr += sizeof(sSwathData->navigate.scPosZ);
  memcpy(ptr, &sSwathData->navigate.scVelX, sizeof(sSwathData->navigate.scVelX));
  ptr += sizeof(sSwathData->navigate.scVelX);
  memcpy(ptr, &sSwathData->navigate.scVelY, sizeof(sSwathData->navigate.scVelY));
  ptr += sizeof(sSwathData->navigate.scVelY);
  memcpy(ptr, &sSwathData->navigate.scVelZ, sizeof(sSwathData->navigate.scVelZ));
  ptr += sizeof(sSwathData->navigate.scVelZ);
  memcpy(ptr, &sSwathData->navigate.scLat, sizeof(sSwathData->navigate.scLat));
  ptr += sizeof(sSwathData->navigate.scLat);
  memcpy(ptr, &sSwathData->navigate.scLon, sizeof(sSwathData->navigate.scLon));
  ptr += sizeof(sSwathData->navigate.scLon);
  memcpy(ptr, &sSwathData->navigate.scAlt, sizeof(sSwathData->navigate.scAlt));
  ptr += sizeof(sSwathData->navigate.scAlt);
  memcpy(ptr, &sSwathData->navigate.scAttRoll, 
		sizeof(sSwathData->navigate.scAttRoll));
  ptr += sizeof(sSwathData->navigate.scAttRoll);
  memcpy(ptr, &sSwathData->navigate.scAttPitch, 
		sizeof(sSwathData->navigate.scAttPitch));
  ptr += sizeof(sSwathData->navigate.scAttPitch);
  memcpy(ptr, &sSwathData->navigate.scAttYaw, 
		sizeof(sSwathData->navigate.scAttYaw));
  ptr += sizeof(sSwathData->navigate.scAttYaw);
  memcpy(ptr, &sSwathData->navigate.att1, sizeof(sSwathData->navigate.att1));
  ptr += sizeof(sSwathData->navigate.att1);
  memcpy(ptr, &sSwathData->navigate.att2, sizeof(sSwathData->navigate.att2));
  ptr += sizeof(sSwathData->navigate.att2);
  memcpy(ptr, &sSwathData->navigate.att3, sizeof(sSwathData->navigate.att3));
  ptr += sizeof(sSwathData->navigate.att3);
  memcpy(ptr, &sSwathData->navigate.att4, sizeof(sSwathData->navigate.att4));
  ptr += sizeof(sSwathData->navigate.att4);
  memcpy(ptr, &sSwathData->navigate.att5, sizeof(sSwathData->navigate.att5));
  ptr += sizeof(sSwathData->navigate.att5);
  memcpy(ptr, &sSwathData->navigate.att6, sizeof(sSwathData->navigate.att6));
  ptr += sizeof(sSwathData->navigate.att6);
  memcpy(ptr, &sSwathData->navigate.att7, sizeof(sSwathData->navigate.att7));
  ptr += sizeof(sSwathData->navigate.att7);
  memcpy(ptr, &sSwathData->navigate.att8, sizeof(sSwathData->navigate.att8));
  ptr += sizeof(sSwathData->navigate.att8);
  memcpy(ptr, &sSwathData->navigate.att9, sizeof(sSwathData->navigate.att9));
  ptr += sizeof(sSwathData->navigate.att9);
  memcpy(ptr, &sSwathData->navigate.greenHourAng, 
		sizeof(sSwathData->navigate.greenHourAng));
  /* Writing packed vdata, navigation block, to an HDF file */
  vdataWrite(&tempVdata, databuf);
  free(databuf);
  
  

  /* Allocate memory for packing the vdata */
  databuf = allocateVdataMemory(&tempVdata->vdataFields);
  ptr = databuf;
  memcpy(ptr, &sSwathData->power.radarTransPower, 
		sizeof(sSwathData->power.radarTransPower));
  ptr += sizeof(sSwathData->power.radarTransPower);
  memcpy(ptr, &sSwathData->power.transPulseWidth, 
		sizeof(sSwathData->power.transPulseWidth));
  /* Writing packed vdata, power block, to an HDF file */
  vdataWrite(&tempVdata, databuf);
  free(databuf);

  /* Writing scientific data set into HDF format */
  /*
   * Scaling calculation.  For writing,
   *	Multiplied the data by L1B21_SYSTEMNOISE.
   */
  for (i = 0; i < 49; ++i)
  {
        /* The Following code was added to take care of the Missing or 
           invalid data.. The reason to add this code was not to scale
           the invalid Data */

        if (sSwathData->systemNoise[i] > -32734)
	     sSwathData->systemNoise_scale[i] = 
		(int16)RND(sSwathData->systemNoise[i] * L1B21_SYSTEMNOISE);
	else
	    sSwathData->systemNoise_scale[i] = -32734; 
  }
  sdsWrite(&tempSDS, &sSwathData->systemNoise_scale);

  sdsWrite(&tempSDS, &sSwathData->sysNoiseWarnFlag);
  sdsWrite(&tempSDS, &sSwathData->minEchoFlag);
  sdsWrite(&tempSDS, &sSwathData->binStormHeight);
  sdsWrite(&tempSDS, &sSwathData->binEllipsoid);
  sdsWrite(&tempSDS, &sSwathData->binClutterFreeBottom);
  sdsWrite(&tempSDS, &sSwathData->binDIDHmean);
  sdsWrite(&tempSDS, &sSwathData->binDIDHtop);
  sdsWrite(&tempSDS, &sSwathData->binDIDHbottom);
  sdsWrite(&tempSDS, &sSwathData->scLocalZenith);
  sdsWrite(&tempSDS, &sSwathData->scRange);
  sdsWrite(&tempSDS, &sSwathData->osBinStart);
  sdsWrite(&tempSDS, &sSwathData->landOceanFlag);
  sdsWrite(&tempSDS, &sSwathData->surfWarnFlag);
  sdsWrite(&tempSDS, &sSwathData->binSurfPeak);

  /*
   * Scaling calculation.  For writing,
   *    Multiplied the data by L1B21_NORMALSAMPLE.
   */
  for (i = 0; i < 49; ++i)
    for (j = 0; j < 140; ++j)
    {
        /* The Following code was added to take care of the Missing or
           invalid data.. The reason to add this code was not to scale
           the invalid Data */

	if(sSwathData->normalSample[i][j] > -32700)
             sSwathData->normalSample_scale[i][j] = 
		(int16)RND(sSwathData->normalSample[i][j] * L1B21_NORMALSAMPLE);
    	else if (sSwathData->normalSample[i][j] > -32734)
		sSwathData->normalSample_scale[i][j] = -32700;
	else if ( sSwathData->normalSample[i][j] > -32767 )
	        sSwathData->normalSample_scale[i][j] = -32734;
	    else
		sSwathData->normalSample_scale[i][j] = -32767;
}
  sdsWrite(&tempSDS, &sSwathData->normalSample_scale);

  /*
   * Scaling calculation.  For writing,
   *    Multiplied the data by L1B21_OSSURF.
   */
  for (i = 0; i < 29; ++i)
    for (j = 0; j < 5; ++j)
    {
        /* The Following code was added to take care of the Missing or
           invalid data.. The reason to add this code was not to scale
           the invalid Data */

        if (sSwathData->osSurf[i][j] > -32700)
             sSwathData->osSurf_scale[i][j] = 
		(int16)RND(sSwathData->osSurf[i][j] * L1B21_OSSURF);
	else if ( sSwathData->osSurf[i][j] > -32734 )
	     sSwathData->osSurf_scale[i][j] = -32700;
        else
             sSwathData->osSurf_scale[i][j] = -32734; 
    }
  sdsWrite(&tempSDS, &sSwathData->osSurf_scale);

  /*
   * Scaling calculation.  For writing,
   *    Multiplied the data by L1B21_OSRAIN.
   */
  for (i = 0; i < 11; ++i)
    for (j = 0; j < 28; ++j)
    {
        /* The Following code was added to take care of the Missing or
           invalid data.. The reason to add this code was not to scale
           the invalid Data */

        if (sSwathData->osRain[i][j] > -32700)
             sSwathData->osRain_scale[i][j] = 
		(int16)RND(sSwathData->osRain[i][j] * L1B21_OSRAIN);
        else if ( sSwathData->osRain[i][j] > -32734 )
	     sSwathData->osRain_scale[i][j] = -32700;
    	else
             sSwathData->osRain_scale[i][j] = -32734; 
}
  sdsWrite(&tempSDS, &sSwathData->osRain_scale);

  return(TK_SUCCESS);
}

/***********************************************************************************
*
*  FUNCTION STATEMENT:
*       readScan_1B_21(IO_HANDLE *sGranuleHandle, L1B_21_SWATHDATA *sSwathData)
*
*  DESCRIPTION:
*       This function is used to read the L1B-21 information in HDF format.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       sGranuleHandle          A structure containing information about the file.
*                               sGranuleHandle is returned by TKopen().
*
*  OUTPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       sSwathData		A structure containing the complete scanline data.
*                               This structure must be declared to correspond to 
*                               the data file being read.  The structure of 
*				sSwathData is manipulated by TKreadScan() and other 
*				toolkit	routines.
*
*  CALLED BY:                   TKreadScan()
*
*  ROUTINES CALLED:     
*       Routine Name            Description
*       ------------            -----------
*       allocateVdataMemory     This routine is used to allocate memory to vdata.
*       vdataRead               This routine is used to read the vdata in HDF 
*                               format.
*       sdsRead                 This routine is used to read the scientific data
*                               set in HDF format.      
*
***********************************************************************************/

int readScan_1B_21(IO_HANDLE *sGranuleHandle, L1B_21_SWATHDATA *sSwathData)
{
  SDS_LINK *tempSDS = sGranuleHandle->cfgSTR->vGroup->subVgroup->sds;
  VDATA_LINK *tempVdata = sGranuleHandle->cfgSTR->vGroup->subVgroup->vdata;
  uint8 *databuf, *ptr;
  int 	i, j;
  int scaleFactor;

  /* Allocate memory for unpacking the vdata */
  databuf = allocateVdataMemory(&tempVdata->vdataFields);
  ptr = databuf;

  /* Unpack the content of databuf into scanTime */
  vdataRead(&tempVdata, ptr);
  memcpy(&sSwathData->scanTime, ptr, sizeof(sSwathData->scanTime));
  free(databuf);

  sdsRead(&tempSDS, &scaleFactor, &sSwathData->geolocation);


  /* Allocate memory for unpacking the vdata */
  databuf = allocateVdataMemory(&tempVdata->vdataFields);
  ptr = databuf;
  /* Unpack the content of databuf into scanStatus block */
  vdataRead(&tempVdata, ptr);
  memcpy(&sSwathData->scanStatus.missing, ptr,
                sizeof(sSwathData->scanStatus.missing));
  ptr += sizeof(sSwathData->scanStatus.missing);
  memcpy(&sSwathData->scanStatus.validity, ptr, 
		sizeof(sSwathData->scanStatus.validity));
  ptr += sizeof(sSwathData->scanStatus.validity);
  memcpy(&sSwathData->scanStatus.qac, ptr, 
		sizeof(sSwathData->scanStatus.qac));
  ptr += sizeof(sSwathData->scanStatus.qac);
  memcpy(&sSwathData->scanStatus.geoQuality, ptr, 
		sizeof(sSwathData->scanStatus.geoQuality));
  ptr += sizeof(sSwathData->scanStatus.geoQuality);
  memcpy(&sSwathData->scanStatus.dataQuality, ptr, 
		sizeof(sSwathData->scanStatus.dataQuality));
  ptr += sizeof(sSwathData->scanStatus.dataQuality);
  memcpy(&sSwathData->scanStatus.scOrient, ptr, 
		sizeof(sSwathData->scanStatus.scOrient));
  ptr += sizeof(sSwathData->scanStatus.scOrient);
  memcpy(&sSwathData->scanStatus.acsMode, ptr, 
		sizeof(sSwathData->scanStatus.acsMode));
  ptr += sizeof(sSwathData->scanStatus.acsMode);
  memcpy(&sSwathData->scanStatus.yawUpdateS, ptr, 
		sizeof(sSwathData->scanStatus.yawUpdateS));
  ptr += sizeof(sSwathData->scanStatus.yawUpdateS);
  memcpy(&sSwathData->scanStatus.prMode, ptr, 
		sizeof(sSwathData->scanStatus.prMode));
  ptr += sizeof(sSwathData->scanStatus.prMode);
  memcpy(&sSwathData->scanStatus.prStatus1, ptr,
                sizeof(sSwathData->scanStatus.prStatus1));
  ptr += sizeof(sSwathData->scanStatus.prStatus1);
  memcpy(&sSwathData->scanStatus.prStatus2, ptr,
                sizeof(sSwathData->scanStatus.prStatus2));
  ptr += sizeof(sSwathData->scanStatus.prStatus2);
  memcpy(&sSwathData->scanStatus.fractOrbitN, ptr, 
		sizeof(sSwathData->scanStatus.fractOrbitN));
  free(databuf);


  /* Allocate memory for unpacking the vdata */
  databuf = allocateVdataMemory(&tempVdata->vdataFields);
  ptr = databuf;
  /* Unpack the content of databuf into navigation block */
  vdataRead(&tempVdata, ptr);
  memcpy(&sSwathData->navigate.scPosX, ptr, sizeof(sSwathData->navigate.scPosX));
  ptr += sizeof(sSwathData->navigate.scPosX);
  memcpy(&sSwathData->navigate.scPosY, ptr, sizeof(sSwathData->navigate.scPosY));
  ptr += sizeof(sSwathData->navigate.scPosY);
  memcpy(&sSwathData->navigate.scPosZ, ptr, sizeof(sSwathData->navigate.scPosZ));
  ptr += sizeof(sSwathData->navigate.scPosZ);
  memcpy(&sSwathData->navigate.scVelX, ptr, sizeof(sSwathData->navigate.scVelX));
  ptr += sizeof(sSwathData->navigate.scVelX);
  memcpy(&sSwathData->navigate.scVelY, ptr, sizeof(sSwathData->navigate.scVelY));
  ptr += sizeof(sSwathData->navigate.scVelY);
  memcpy(&sSwathData->navigate.scVelZ, ptr, sizeof(sSwathData->navigate.scVelZ));
  ptr += sizeof(sSwathData->navigate.scVelZ);
  memcpy(&sSwathData->navigate.scLat, ptr, sizeof(sSwathData->navigate.scLat));
  ptr += sizeof(sSwathData->navigate.scLat);
  memcpy(&sSwathData->navigate.scLon, ptr, sizeof(sSwathData->navigate.scLon));
  ptr += sizeof(sSwathData->navigate.scLon);
  memcpy(&sSwathData->navigate.scAlt, ptr, sizeof(sSwathData->navigate.scAlt));
  ptr += sizeof(sSwathData->navigate.scAlt);
  memcpy(&sSwathData->navigate.scAttRoll, ptr, 
		sizeof(sSwathData->navigate.scAttRoll));
  ptr += sizeof(sSwathData->navigate.scAttRoll);
  memcpy(&sSwathData->navigate.scAttPitch, ptr, 
		sizeof(sSwathData->navigate.scAttPitch));
  ptr += sizeof(sSwathData->navigate.scAttPitch);
  memcpy(&sSwathData->navigate.scAttYaw, ptr, 
		sizeof(sSwathData->navigate.scAttYaw));
  ptr += sizeof(sSwathData->navigate.scAttYaw);
  memcpy(&sSwathData->navigate.att1, ptr, sizeof(sSwathData->navigate.att1));
  ptr += sizeof(sSwathData->navigate.att1);
  memcpy(&sSwathData->navigate.att2, ptr, sizeof(sSwathData->navigate.att2));
  ptr += sizeof(sSwathData->navigate.att2);
  memcpy(&sSwathData->navigate.att3, ptr, sizeof(sSwathData->navigate.att3));
  ptr += sizeof(sSwathData->navigate.att3);
  memcpy(&sSwathData->navigate.att4, ptr, sizeof(sSwathData->navigate.att4));
  ptr += sizeof(sSwathData->navigate.att4);
  memcpy(&sSwathData->navigate.att5, ptr, sizeof(sSwathData->navigate.att5));
  ptr += sizeof(sSwathData->navigate.att5);
  memcpy(&sSwathData->navigate.att6, ptr, sizeof(sSwathData->navigate.att6));
  ptr += sizeof(sSwathData->navigate.att6);
  memcpy(&sSwathData->navigate.att7, ptr, sizeof(sSwathData->navigate.att7));
  ptr += sizeof(sSwathData->navigate.att7);
  memcpy(&sSwathData->navigate.att8, ptr, sizeof(sSwathData->navigate.att8));
  ptr += sizeof(sSwathData->navigate.att8);
  memcpy(&sSwathData->navigate.att9, ptr, sizeof(sSwathData->navigate.att9));
  ptr += sizeof(sSwathData->navigate.att9);
  memcpy(&sSwathData->navigate.greenHourAng, ptr, 
		sizeof(sSwathData->navigate.greenHourAng));
  free(databuf);


  /* Allocate memory for unpacking the vdata */
  databuf = allocateVdataMemory(&tempVdata->vdataFields);
  ptr = databuf;
  /* Unpack the content of databuf into power block */
  vdataRead(&tempVdata, ptr);
  memcpy(&sSwathData->power.radarTransPower, ptr, 
		sizeof(sSwathData->power.radarTransPower));
  ptr += sizeof(sSwathData->power.radarTransPower);
  memcpy(&sSwathData->power.transPulseWidth, ptr, 
		sizeof(sSwathData->power.transPulseWidth));
  free(databuf);

  /* Reading scientific data set into HDF format */
  /*
   * Scaling calculation.  For reading,
   *    Divided the data by L1B21_SYSTEMNOISE.
   */
  sdsRead(&tempSDS, &scaleFactor, &sSwathData->systemNoise_scale);
  for (i = 0; i < 49; ++i)
    if (sSwathData->systemNoise_scale[i] == -32734)
      sSwathData->systemNoise[i] = -32734.0;
    else
      sSwathData->systemNoise[i] = 
	sSwathData->systemNoise_scale[i] * 1.0 / scaleFactor;
  sdsRead(&tempSDS, &scaleFactor, &sSwathData->sysNoiseWarnFlag);
  sdsRead(&tempSDS, &scaleFactor, &sSwathData->minEchoFlag);
  sdsRead(&tempSDS, &scaleFactor, &sSwathData->binStormHeight);
  sdsRead(&tempSDS, &scaleFactor, &sSwathData->binEllipsoid);
  sdsRead(&tempSDS, &scaleFactor, &sSwathData->binClutterFreeBottom);
  sdsRead(&tempSDS, &scaleFactor, &sSwathData->binDIDHmean);
  sdsRead(&tempSDS, &scaleFactor, &sSwathData->binDIDHtop);
  sdsRead(&tempSDS, &scaleFactor, &sSwathData->binDIDHbottom);
  sdsRead(&tempSDS, &scaleFactor, &sSwathData->scLocalZenith);
  sdsRead(&tempSDS, &scaleFactor, &sSwathData->scRange);
  sdsRead(&tempSDS, &scaleFactor, &sSwathData->osBinStart);
  sdsRead(&tempSDS, &scaleFactor, &sSwathData->landOceanFlag);
  sdsRead(&tempSDS, &scaleFactor, &sSwathData->surfWarnFlag);
  sdsRead(&tempSDS, &scaleFactor, &sSwathData->binSurfPeak);

  /*
   * Scaling calculation.  For reading,
   *    Divided the data by L1B21_NORMALSAMPLE.
   */
  sdsRead(&tempSDS, &scaleFactor, &sSwathData->normalSample_scale);
  for (i = 0; i < 49; ++i)
    for (j = 0; j < 140; ++j)
      if ( sSwathData->normalSample_scale[i][j] == -32734 )
	sSwathData->normalSample[i][j] = -32734.0;
      else if ( sSwathData->normalSample_scale[i][j] == -32767 )
	sSwathData->normalSample[i][j] = -32767.0;
      else if ( sSwathData->normalSample_scale[i][j] == -32700 )
	sSwathData->normalSample[i][j] = -32700.0;
      else
        sSwathData->normalSample[i][j] = 
	  sSwathData->normalSample_scale[i][j] * 1.0 / scaleFactor;

  /*
   * Scaling calculation.  For reading,
   *    Divided the data by L1B21_OSSURF.
   */
  sdsRead(&tempSDS, &scaleFactor, &sSwathData->osSurf_scale);
  for (i = 0; i < 29; ++i)
    for (j = 0; j < 5; ++j)
      if (sSwathData->osSurf_scale[i][j] == -32734)
	sSwathData->osSurf[i][j] = -32734.0;
      else if ( sSwathData->osSurf_scale[i][j] == -32700 )
	sSwathData->osSurf[i][j] = -32700.0;
      else
        sSwathData->osSurf[i][j] = 
	  sSwathData->osSurf_scale[i][j] * 1.0 / scaleFactor;

  /*
   * Scaling calculation.  For reading,
   *    Divided the data by L1B21_OSRAIN.
   */
  sdsRead(&tempSDS, &scaleFactor, &sSwathData->osRain_scale);
  for (i = 0; i < 11; ++i)
    for (j = 0; j < 28; ++j)
      if (sSwathData->osRain_scale[i][j] == -32734)
	sSwathData->osRain[i][j] = -32734.0;
      else if ( sSwathData->osRain_scale[i][j] == -32700 )
	sSwathData->osRain[i][j] = -32700.0;
      else
        sSwathData->osRain[i][j] = 
	  sSwathData->osRain_scale[i][j] * 1.0 / scaleFactor;

  return(TK_SUCCESS);
}

/***********************************************************************************
*
*  FUNCTION STATEMENT:
*       writeScan_2A_21(IO_HANDLE *sGranuleHandle, L2A_21_SWATHDATA *sSwathData) 
*
*  DESCRIPTION:
*       This function is used to write the L2A_21 information in HDF format.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       sGranuleHandle          A structure containing information about the file.
*                               sGranuleHandle is returned by TKopen().
*
*       sSwathData              A structure containing the complete scanline data.
*                               This structure must be declared to correspond to 
*                               the data file being written.  The structure of 
*                               sSwathData is manipulated by TKwriteScan() and other 
*				toolkit routines.
*
*  OUTPUT PARAMETERS:           None
*
*  CALLED BY:                   TKwriteScan()
*
*  ROUTINES CALLED:     
*       Routine Name            Description
*       ------------            -----------
*       allocateVdataMemory     This routine is used to allocate memory to vdata.
*       vdataWrite              This routine is used to write the vdata in HDF 
*                               format.
*       sdsWrite                This routine is used to write the scientific data
*                               set in HDF format.      
*
***********************************************************************************/

int writeScan_2A_21(IO_HANDLE *sGranuleHandle, L2A_21_SWATHDATA *sSwathData)
{
  SDS_LINK *tempSDS = sGranuleHandle->cfgSTR->vGroup->subVgroup->sds;
  VDATA_LINK *tempVdata = sGranuleHandle->cfgSTR->vGroup->subVgroup->vdata;
  uint8 *databuf, *ptr;
  int	i;

  /* Allocate memory for packing the vdata */
  databuf = allocateVdataMemory(&tempVdata->vdataFields);
  ptr = databuf;
  memcpy(ptr, &sSwathData->scanTime, sizeof(sSwathData->scanTime));
  /* Writing packed vdata, scanTime, to an HDF file */
  vdataWrite(&tempVdata, databuf); 
  free(databuf);

  sdsWrite(&tempSDS, &sSwathData->geolocation);


  /* Allocate memory for packing the vdata */
  databuf = allocateVdataMemory(&tempVdata->vdataFields);
  ptr = databuf;
  memcpy(ptr, &sSwathData->scanStatus.missing,
                sizeof(sSwathData->scanStatus.missing));
  ptr += sizeof(sSwathData->scanStatus.missing);
  memcpy(ptr, &sSwathData->scanStatus.validity, 
		sizeof(sSwathData->scanStatus.validity));
  ptr += sizeof(sSwathData->scanStatus.validity);
  memcpy(ptr, &sSwathData->scanStatus.qac, 
		sizeof(sSwathData->scanStatus.qac));
  ptr += sizeof(sSwathData->scanStatus.qac);
  memcpy(ptr, &sSwathData->scanStatus.geoQuality, 
		sizeof(sSwathData->scanStatus.geoQuality));
  ptr += sizeof(sSwathData->scanStatus.geoQuality);
  memcpy(ptr, &sSwathData->scanStatus.dataQuality, 
		sizeof(sSwathData->scanStatus.dataQuality));
  ptr += sizeof(sSwathData->scanStatus.dataQuality);
  memcpy(ptr, &sSwathData->scanStatus.scOrient, 
		sizeof(sSwathData->scanStatus.scOrient));
  ptr += sizeof(sSwathData->scanStatus.scOrient);
  memcpy(ptr, &sSwathData->scanStatus.acsMode, 
		sizeof(sSwathData->scanStatus.acsMode));
  ptr += sizeof(sSwathData->scanStatus.acsMode);
  memcpy(ptr, &sSwathData->scanStatus.yawUpdateS, 
		sizeof(sSwathData->scanStatus.yawUpdateS));
  ptr += sizeof(sSwathData->scanStatus.yawUpdateS);
  memcpy(ptr, &sSwathData->scanStatus.prMode, 
		sizeof(sSwathData->scanStatus.prMode));
  ptr += sizeof(sSwathData->scanStatus.prMode);
  memcpy(ptr, &sSwathData->scanStatus.prStatus1,
                sizeof(sSwathData->scanStatus.prStatus1));
  ptr += sizeof(sSwathData->scanStatus.prStatus1);
  memcpy(ptr, &sSwathData->scanStatus.prStatus2,
                sizeof(sSwathData->scanStatus.prStatus2));
  ptr += sizeof(sSwathData->scanStatus.prStatus2);
  memcpy(ptr, &sSwathData->scanStatus.fractOrbitN, 
		sizeof(sSwathData->scanStatus.fractOrbitN));
  /* Writing packed vdata, scanStatus block, to an HDF file */
  vdataWrite(&tempVdata, databuf);
  free(databuf);


  /* Allocate memory for packing the vdata */
  databuf = allocateVdataMemory(&tempVdata->vdataFields);
  ptr = databuf;
  memcpy(ptr, &sSwathData->navigate.scPosX, sizeof(sSwathData->navigate.scPosX));
  ptr += sizeof(sSwathData->navigate.scPosX);
  memcpy(ptr, &sSwathData->navigate.scPosY, sizeof(sSwathData->navigate.scPosY));
  ptr += sizeof(sSwathData->navigate.scPosY);
  memcpy(ptr, &sSwathData->navigate.scPosZ, sizeof(sSwathData->navigate.scPosZ));
  ptr += sizeof(sSwathData->navigate.scPosZ);
  memcpy(ptr, &sSwathData->navigate.scVelX, sizeof(sSwathData->navigate.scVelX));
  ptr += sizeof(sSwathData->navigate.scVelX);
  memcpy(ptr, &sSwathData->navigate.scVelY, sizeof(sSwathData->navigate.scVelY));
  ptr += sizeof(sSwathData->navigate.scVelY);
  memcpy(ptr, &sSwathData->navigate.scVelZ, sizeof(sSwathData->navigate.scVelZ));
  ptr += sizeof(sSwathData->navigate.scVelZ);
  memcpy(ptr, &sSwathData->navigate.scLat, sizeof(sSwathData->navigate.scLat));
  ptr += sizeof(sSwathData->navigate.scLat);
  memcpy(ptr, &sSwathData->navigate.scLon, sizeof(sSwathData->navigate.scLon));
  ptr += sizeof(sSwathData->navigate.scLon);
  memcpy(ptr, &sSwathData->navigate.scAlt, sizeof(sSwathData->navigate.scAlt));
  ptr += sizeof(sSwathData->navigate.scAlt);
  memcpy(ptr, &sSwathData->navigate.scAttRoll, 
		sizeof(sSwathData->navigate.scAttRoll));
  ptr += sizeof(sSwathData->navigate.scAttRoll);
  memcpy(ptr, &sSwathData->navigate.scAttPitch, 
		sizeof(sSwathData->navigate.scAttPitch));
  ptr += sizeof(sSwathData->navigate.scAttPitch);
  memcpy(ptr, &sSwathData->navigate.scAttYaw, 
		sizeof(sSwathData->navigate.scAttYaw));
  ptr += sizeof(sSwathData->navigate.scAttYaw);
  memcpy(ptr, &sSwathData->navigate.att1, sizeof(sSwathData->navigate.att1));
  ptr += sizeof(sSwathData->navigate.att1);
  memcpy(ptr, &sSwathData->navigate.att2, sizeof(sSwathData->navigate.att2));
  ptr += sizeof(sSwathData->navigate.att2);
  memcpy(ptr, &sSwathData->navigate.att3, sizeof(sSwathData->navigate.att3));
  ptr += sizeof(sSwathData->navigate.att3);
  memcpy(ptr, &sSwathData->navigate.att4, sizeof(sSwathData->navigate.att4));
  ptr += sizeof(sSwathData->navigate.att4);
  memcpy(ptr, &sSwathData->navigate.att5, sizeof(sSwathData->navigate.att5));
  ptr += sizeof(sSwathData->navigate.att5);
  memcpy(ptr, &sSwathData->navigate.att6, sizeof(sSwathData->navigate.att6));
  ptr += sizeof(sSwathData->navigate.att6);
  memcpy(ptr, &sSwathData->navigate.att7, sizeof(sSwathData->navigate.att7));
  ptr += sizeof(sSwathData->navigate.att7);
  memcpy(ptr, &sSwathData->navigate.att8, sizeof(sSwathData->navigate.att8));
  ptr += sizeof(sSwathData->navigate.att8);
  memcpy(ptr, &sSwathData->navigate.att9, sizeof(sSwathData->navigate.att9));
  ptr += sizeof(sSwathData->navigate.att9);
  memcpy(ptr, &sSwathData->navigate.greenHourAng, 
		sizeof(sSwathData->navigate.greenHourAng));
  /* Writing packed vdata, navigation block, to an HDF file */
  vdataWrite(&tempVdata, databuf);
  free(databuf);
  

  /* Write SDS into HDF format */
  /*
   * Scaling calculation.  For writing,
   *	Multiplied the data by L2A21_SIGMAZERO.
   */
  for (i = 0; i < 49; ++i)
  {
        /* The Following code was added to take care of the Missing or
           invalid data.  The reason to add this code was not to scale
           the invalid Data */

     if(sSwathData->sigmaZero[i] > TK_DEFAULT)
	 sSwathData->sigmaZero_scale[i] = 
		(int16)RND(sSwathData->sigmaZero[i] * L2A21_SIGMAZERO);
     else
	 sSwathData->sigmaZero_scale[i] = TK_MISS_I2;
  }
  sdsWrite(&tempSDS, &sSwathData->sigmaZero_scale);

  /*
   * Scaling calculation.  For writing,
   *    Multiplied the data by L2A21_PATHATTEN.
   */
  for (i = 0; i < 49; ++i)
  {
        /* The Following code was added to take care of the Missing or
           invalid data.  The reason to add this code was not to scale
           the invalid Data */

     if(sSwathData->pathAtten[i] > TK_DEFAULT)
         sSwathData->pathAtten_scale[i] = 
		(int16)RND(sSwathData->pathAtten[i] * L2A21_PATHATTEN);
     else
         sSwathData->pathAtten_scale[i] = TK_MISS_I2;
  }
  sdsWrite(&tempSDS, &sSwathData->pathAtten_scale);

  sdsWrite(&tempSDS, &sSwathData->reliabFlag);
  sdsWrite(&tempSDS, &sSwathData->reliabFactor);

  /*
   * Scaling calculation.  For writing,
   *    Multiplied the data by L2A21_INCANGLE.
   */
  for (i = 0; i < 49; ++i)
  {
        /* The Following code was added to take care of the Missing or
           invalid data.  The reason to add this code was not to scale
           the invalid Data */

     if(sSwathData->incAngle[i] > TK_DEFAULT)
         sSwathData->incAngle_scale[i] = 
		(int16)RND(sSwathData->incAngle[i] * L2A21_INCANGLE);
     else
         sSwathData->incAngle_scale[i] = TK_MISS_I2;
  }
  sdsWrite(&tempSDS, &sSwathData->incAngle_scale);

  sdsWrite(&tempSDS, &sSwathData->rainFlag);

  return(TK_SUCCESS);
}

/***********************************************************************************
*
*  FUNCTION STATEMENT:
*       readScan_2A_21(IO_HANDLE *sGranuleHandle, L2A_21_SWATHDATA *sSwathData) 
*
*  DESCRIPTION:
*       This function is used to read the L2A_21 information in HDF format.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       sGranuleHandle          A structure containing information about the file.
*                               sGranuleHandle is returned by TKopen().
*
*  OUTPUT PARAMETERS:     
*       Parameter Name          Description
*       --------------          -----------
*       sSwathData              A structure containing the complete scanline data.
*                               This structure must be declared to correspond to 
*                               the data file being read.  The structure of 
*                               sSwathData is manipulated by TKreadScan() and other 
*				toolkit routines.
*
*  CALLED BY:                   TKreadScan()
*
*  ROUTINES CALLED:     
*       Routine Name            Description
*       ------------            -----------
*       allocateVdataMemory     This routine is used to allocate memory to vdata.
*       vdataRead               This routine is used to read the vdata in HDF 
*                               format.
*       sdsRead                 This routine is used to read the scientific data
*                               set in HDF format.      
*
***********************************************************************************/

int readScan_2A_21(IO_HANDLE *sGranuleHandle, L2A_21_SWATHDATA *sSwathData)
{
  SDS_LINK *tempSDS = sGranuleHandle->cfgSTR->vGroup->subVgroup->sds;
  VDATA_LINK *tempVdata = sGranuleHandle->cfgSTR->vGroup->subVgroup->vdata;
  uint8 *databuf, *ptr;
  int	i;
  int scaleFactor;

  /* Allocate memory for unpacking the vdata */
  databuf = allocateVdataMemory(&tempVdata->vdataFields);
  ptr = databuf;

  /* Unpack the content of databuf into scanTime */
  vdataRead(&tempVdata, ptr);
  memcpy(&sSwathData->scanTime, ptr, sizeof(sSwathData->scanTime));
  free(databuf);

  sdsRead(&tempSDS, &scaleFactor, &sSwathData->geolocation);

  /* Allocate memory for unpacking the vdata */
  databuf = allocateVdataMemory(&tempVdata->vdataFields);
  ptr = databuf;
  /* Unpack the content of databuf into scanStatus block */
  vdataRead(&tempVdata, ptr);
  memcpy(&sSwathData->scanStatus.missing, ptr,
                sizeof(sSwathData->scanStatus.missing));
  ptr += sizeof(sSwathData->scanStatus.missing);
  memcpy(&sSwathData->scanStatus.validity, ptr, 
		sizeof(sSwathData->scanStatus.validity));
  ptr += sizeof(sSwathData->scanStatus.validity);
  memcpy(&sSwathData->scanStatus.qac, ptr, 
		sizeof(sSwathData->scanStatus.qac));
  ptr += sizeof(sSwathData->scanStatus.qac);
  memcpy(&sSwathData->scanStatus.geoQuality, ptr, 
		sizeof(sSwathData->scanStatus.geoQuality));
  ptr += sizeof(sSwathData->scanStatus.geoQuality);
  memcpy(&sSwathData->scanStatus.dataQuality, ptr, 
		sizeof(sSwathData->scanStatus.dataQuality));
  ptr += sizeof(sSwathData->scanStatus.dataQuality);
  memcpy(&sSwathData->scanStatus.scOrient, ptr, 
		sizeof(sSwathData->scanStatus.scOrient));
  ptr += sizeof(sSwathData->scanStatus.scOrient);
  memcpy(&sSwathData->scanStatus.acsMode, ptr, 
		sizeof(sSwathData->scanStatus.acsMode));
  ptr += sizeof(sSwathData->scanStatus.acsMode);
  memcpy(&sSwathData->scanStatus.yawUpdateS, ptr, 
		sizeof(sSwathData->scanStatus.yawUpdateS));
  ptr += sizeof(sSwathData->scanStatus.yawUpdateS);
  memcpy(&sSwathData->scanStatus.prMode, ptr, 
		sizeof(sSwathData->scanStatus.prMode));
  ptr += sizeof(sSwathData->scanStatus.prMode);
  memcpy(&sSwathData->scanStatus.prStatus1, ptr,
                sizeof(sSwathData->scanStatus.prStatus1));
  ptr += sizeof(sSwathData->scanStatus.prStatus1);
  memcpy(&sSwathData->scanStatus.prStatus2, ptr,
                sizeof(sSwathData->scanStatus.prStatus2));
  ptr += sizeof(sSwathData->scanStatus.prStatus2);
  memcpy(&sSwathData->scanStatus.fractOrbitN, ptr, 
		sizeof(sSwathData->scanStatus.fractOrbitN));
  free(databuf);


  /* Allocate memory for unpacking the vdata */
  databuf = allocateVdataMemory(&tempVdata->vdataFields);
  ptr = databuf;
  /* Unpack the content of databuf into navigation block */
  vdataRead(&tempVdata, ptr);
  memcpy(&sSwathData->navigate.scPosX, ptr, sizeof(sSwathData->navigate.scPosX));
  ptr += sizeof(sSwathData->navigate.scPosX);
  memcpy(&sSwathData->navigate.scPosY, ptr, sizeof(sSwathData->navigate.scPosY));
  ptr += sizeof(sSwathData->navigate.scPosY);
  memcpy(&sSwathData->navigate.scPosZ, ptr, sizeof(sSwathData->navigate.scPosZ));
  ptr += sizeof(sSwathData->navigate.scPosZ);
  memcpy(&sSwathData->navigate.scVelX, ptr, sizeof(sSwathData->navigate.scVelX));
  ptr += sizeof(sSwathData->navigate.scVelX);
  memcpy(&sSwathData->navigate.scVelY, ptr, sizeof(sSwathData->navigate.scVelY));
  ptr += sizeof(sSwathData->navigate.scVelY);
  memcpy(&sSwathData->navigate.scVelZ, ptr, sizeof(sSwathData->navigate.scVelZ));
  ptr += sizeof(sSwathData->navigate.scVelZ);
  memcpy(&sSwathData->navigate.scLat, ptr, sizeof(sSwathData->navigate.scLat));
  ptr += sizeof(sSwathData->navigate.scLat);
  memcpy(&sSwathData->navigate.scLon, ptr, sizeof(sSwathData->navigate.scLon));
  ptr += sizeof(sSwathData->navigate.scLon);
  memcpy(&sSwathData->navigate.scAlt, ptr, sizeof(sSwathData->navigate.scAlt));
  ptr += sizeof(sSwathData->navigate.scAlt);
  memcpy(&sSwathData->navigate.scAttRoll, ptr, 
		sizeof(sSwathData->navigate.scAttRoll));
  ptr += sizeof(sSwathData->navigate.scAttRoll);
  memcpy(&sSwathData->navigate.scAttPitch, ptr, 
		sizeof(sSwathData->navigate.scAttPitch));
  ptr += sizeof(sSwathData->navigate.scAttPitch);
  memcpy(&sSwathData->navigate.scAttYaw, ptr, 
		sizeof(sSwathData->navigate.scAttYaw));
  ptr += sizeof(sSwathData->navigate.scAttYaw);
  memcpy(&sSwathData->navigate.att1, ptr, sizeof(sSwathData->navigate.att1));
  ptr += sizeof(sSwathData->navigate.att1);
  memcpy(&sSwathData->navigate.att2, ptr, sizeof(sSwathData->navigate.att2));
  ptr += sizeof(sSwathData->navigate.att2);
  memcpy(&sSwathData->navigate.att3, ptr, sizeof(sSwathData->navigate.att3));
  ptr += sizeof(sSwathData->navigate.att3);
  memcpy(&sSwathData->navigate.att4, ptr, sizeof(sSwathData->navigate.att4));
  ptr += sizeof(sSwathData->navigate.att4);
  memcpy(&sSwathData->navigate.att5, ptr, sizeof(sSwathData->navigate.att5));
  ptr += sizeof(sSwathData->navigate.att5);
  memcpy(&sSwathData->navigate.att6, ptr, sizeof(sSwathData->navigate.att6));
  ptr += sizeof(sSwathData->navigate.att6);
  memcpy(&sSwathData->navigate.att7, ptr, sizeof(sSwathData->navigate.att7));
  ptr += sizeof(sSwathData->navigate.att7);
  memcpy(&sSwathData->navigate.att8, ptr, sizeof(sSwathData->navigate.att8));
  ptr += sizeof(sSwathData->navigate.att8);
  memcpy(&sSwathData->navigate.att9, ptr, sizeof(sSwathData->navigate.att9));
  ptr += sizeof(sSwathData->navigate.att9);
  memcpy(&sSwathData->navigate.greenHourAng, ptr, 
		sizeof(sSwathData->navigate.greenHourAng));
  free(databuf);


  /* Read SDS from HDF format */
  /*
   * Scaling calculation.  For reading,
   *	Divided the data by L2A21_SIGMAZERO.
   */
  sdsRead(&tempSDS, &scaleFactor, &sSwathData->sigmaZero_scale);
  for (i = 0; i < 49; ++i)
        {
        if (sSwathData->sigmaZero_scale[i] != TK_MISS_I2)
	   sSwathData->sigmaZero[i] = 
		sSwathData->sigmaZero_scale[i] * 1.0 / scaleFactor;
        else
           sSwathData->sigmaZero[i] = TK_DEFAULT;
        }
  /*
   * Scaling calculation.  For reading,
   *    Divided the data by L2A21_PATHATTEN.
   */
  sdsRead(&tempSDS, &scaleFactor, &sSwathData->pathAtten_scale);
  for (i = 0; i < 49; ++i)
        {
        if (sSwathData->pathAtten_scale[i] != TK_MISS_I2)
           sSwathData->pathAtten[i] = 
		sSwathData->pathAtten_scale[i] * 1.0 / scaleFactor;
        else
           sSwathData->pathAtten[i] = TK_DEFAULT;
        }
  sdsRead(&tempSDS, &scaleFactor, &sSwathData->reliabFlag);
  sdsRead(&tempSDS, &scaleFactor, &sSwathData->reliabFactor);

  /*
   * Scaling calculation.  For reading,
   *    Divided the data by L2A21_INCANGLE.
   */
  sdsRead(&tempSDS, &scaleFactor, &sSwathData->incAngle_scale);
  for (i = 0; i < 49; ++i)
        {
        if (sSwathData->incAngle_scale[i] != TK_MISS_I2)
           sSwathData->incAngle[i] = 
		sSwathData->incAngle_scale[i] * 1.0 / scaleFactor;
        else
           sSwathData->incAngle[i] = TK_DEFAULT;
        }
  sdsRead(&tempSDS, &scaleFactor, &sSwathData->rainFlag);

  return(TK_SUCCESS);
}

/***********************************************************************************
*
*  FUNCTION STATEMENT:
*       writeScan_2A_23(IO_HANDLE *sGranuleHandle, L2A_23_SWATHDATA *sSwathData) 
*
*  DESCRIPTION:
*       This function is used to write the L2A-23 information in HDF format.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       sGranuleHandle          A structure containing information about the file.
*                               sGranuleHandle is returned by TKopen().
*       sSwathData              A structure containing the complete scanline data.
*                               This structure must be declared to correspond to 
*                               the data file being written.  The structure of 
*                               sSwathData is manipulated by TKwriteScan() and other 
*				toolkit routines.
*
*  OUTPUT PARAMETERS:           None
*
*  CALLED BY:                   TKwriteScan()
*
*  ROUTINES CALLED:     
*       Routine Name            Description
*       ------------            -----------
*       allocateVdataMemory     This routine is used to allocate memory to vdata.
*       vdataWrite              This routine is used to write the vdata in HDF 
*                               format.
*       sdsWrite                This routine is used to write the scientific data
*                               set in HDF format.      
*
***********************************************************************************/

int writeScan_2A_23(IO_HANDLE *sGranuleHandle, L2A_23_SWATHDATA *sSwathData)
{ 
  SDS_LINK *tempSDS = sGranuleHandle->cfgSTR->vGroup->subVgroup->sds;
  VDATA_LINK *tempVdata = sGranuleHandle->cfgSTR->vGroup->subVgroup->vdata;
  uint8 *databuf, *ptr;

  /* Allocate memory for packing the vdata */
  databuf = allocateVdataMemory(&tempVdata->vdataFields);
  ptr = databuf;
  memcpy(ptr, &sSwathData->scanTime, sizeof(sSwathData->scanTime));
  /* Writing packed vdata, scanTime, to an HDF file */
  vdataWrite(&tempVdata, databuf); 
  free(databuf);

  sdsWrite(&tempSDS, &sSwathData->geolocation);


  /* Allocate memory for packing the vdata */
  databuf = allocateVdataMemory(&tempVdata->vdataFields);
  ptr = databuf;
  memcpy(ptr, &sSwathData->scanStatus.missing,
                sizeof(sSwathData->scanStatus.missing));
  ptr += sizeof(sSwathData->scanStatus.missing);
  memcpy(ptr, &sSwathData->scanStatus.validity, 
		sizeof(sSwathData->scanStatus.validity));
  ptr += sizeof(sSwathData->scanStatus.validity);
  memcpy(ptr, &sSwathData->scanStatus.qac, 
		sizeof(sSwathData->scanStatus.qac));
  ptr += sizeof(sSwathData->scanStatus.qac);
  memcpy(ptr, &sSwathData->scanStatus.geoQuality, 
		sizeof(sSwathData->scanStatus.geoQuality));
  ptr += sizeof(sSwathData->scanStatus.geoQuality);
  memcpy(ptr, &sSwathData->scanStatus.dataQuality, 
		sizeof(sSwathData->scanStatus.dataQuality));
  ptr += sizeof(sSwathData->scanStatus.dataQuality);
  memcpy(ptr, &sSwathData->scanStatus.scOrient, 
		sizeof(sSwathData->scanStatus.scOrient));
  ptr += sizeof(sSwathData->scanStatus.scOrient);
  memcpy(ptr, &sSwathData->scanStatus.acsMode, 
		sizeof(sSwathData->scanStatus.acsMode));
  ptr += sizeof(sSwathData->scanStatus.acsMode);
  memcpy(ptr, &sSwathData->scanStatus.yawUpdateS, 
		sizeof(sSwathData->scanStatus.yawUpdateS));
  ptr += sizeof(sSwathData->scanStatus.yawUpdateS);
  memcpy(ptr, &sSwathData->scanStatus.prMode, 
		sizeof(sSwathData->scanStatus.prMode));
  ptr += sizeof(sSwathData->scanStatus.prMode);
  memcpy(ptr, &sSwathData->scanStatus.prStatus1,
                sizeof(sSwathData->scanStatus.prStatus1));
  ptr += sizeof(sSwathData->scanStatus.prStatus1);
  memcpy(ptr, &sSwathData->scanStatus.prStatus2,
                sizeof(sSwathData->scanStatus.prStatus2));
  ptr += sizeof(sSwathData->scanStatus.prStatus2);
  memcpy(ptr, &sSwathData->scanStatus.fractOrbitN, 
		sizeof(sSwathData->scanStatus.fractOrbitN));
  /* Writing packed vdata, scanStatus block, to an HDF file */
  vdataWrite(&tempVdata, databuf);
  free(databuf);

  /* Allocate memory for packing the vdata */
  databuf = allocateVdataMemory(&tempVdata->vdataFields);
  ptr = databuf;
  memcpy(ptr, &sSwathData->navigate.scPosX, sizeof(sSwathData->navigate.scPosX));
  ptr += sizeof(sSwathData->navigate.scPosX);
  memcpy(ptr, &sSwathData->navigate.scPosY, sizeof(sSwathData->navigate.scPosY));
  ptr += sizeof(sSwathData->navigate.scPosY);
  memcpy(ptr, &sSwathData->navigate.scPosZ, sizeof(sSwathData->navigate.scPosZ));
  ptr += sizeof(sSwathData->navigate.scPosZ);
  memcpy(ptr, &sSwathData->navigate.scVelX, sizeof(sSwathData->navigate.scVelX));
  ptr += sizeof(sSwathData->navigate.scVelX);
  memcpy(ptr, &sSwathData->navigate.scVelY, sizeof(sSwathData->navigate.scVelY));
  ptr += sizeof(sSwathData->navigate.scVelY);
  memcpy(ptr, &sSwathData->navigate.scVelZ, sizeof(sSwathData->navigate.scVelZ));
  ptr += sizeof(sSwathData->navigate.scVelZ);
  memcpy(ptr, &sSwathData->navigate.scLat, sizeof(sSwathData->navigate.scLat));
  ptr += sizeof(sSwathData->navigate.scLat);
  memcpy(ptr, &sSwathData->navigate.scLon, sizeof(sSwathData->navigate.scLon));
  ptr += sizeof(sSwathData->navigate.scLon);
  memcpy(ptr, &sSwathData->navigate.scAlt, sizeof(sSwathData->navigate.scAlt));
  ptr += sizeof(sSwathData->navigate.scAlt);
  memcpy(ptr, &sSwathData->navigate.scAttRoll, 
		sizeof(sSwathData->navigate.scAttRoll));
  ptr += sizeof(sSwathData->navigate.scAttRoll);
  memcpy(ptr, &sSwathData->navigate.scAttPitch, 
		sizeof(sSwathData->navigate.scAttPitch));
  ptr += sizeof(sSwathData->navigate.scAttPitch);
  memcpy(ptr, &sSwathData->navigate.scAttYaw, 
		sizeof(sSwathData->navigate.scAttYaw));
  ptr += sizeof(sSwathData->navigate.scAttYaw);
  memcpy(ptr, &sSwathData->navigate.att1, sizeof(sSwathData->navigate.att1));
  ptr += sizeof(sSwathData->navigate.att1);
  memcpy(ptr, &sSwathData->navigate.att2, sizeof(sSwathData->navigate.att2));
  ptr += sizeof(sSwathData->navigate.att2);
  memcpy(ptr, &sSwathData->navigate.att3, sizeof(sSwathData->navigate.att3));
  ptr += sizeof(sSwathData->navigate.att3);
  memcpy(ptr, &sSwathData->navigate.att4, sizeof(sSwathData->navigate.att4));
  ptr += sizeof(sSwathData->navigate.att4);
  memcpy(ptr, &sSwathData->navigate.att5, sizeof(sSwathData->navigate.att5));
  ptr += sizeof(sSwathData->navigate.att5);
  memcpy(ptr, &sSwathData->navigate.att6, sizeof(sSwathData->navigate.att6));
  ptr += sizeof(sSwathData->navigate.att6);
  memcpy(ptr, &sSwathData->navigate.att7, sizeof(sSwathData->navigate.att7));
  ptr += sizeof(sSwathData->navigate.att7);
  memcpy(ptr, &sSwathData->navigate.att8, sizeof(sSwathData->navigate.att8));
  ptr += sizeof(sSwathData->navigate.att8);
  memcpy(ptr, &sSwathData->navigate.att9, sizeof(sSwathData->navigate.att9));
  ptr += sizeof(sSwathData->navigate.att9);
  memcpy(ptr, &sSwathData->navigate.greenHourAng, 
		sizeof(sSwathData->navigate.greenHourAng));
  /* Writing packed vdata, navigation block, to an HDF file */
  vdataWrite(&tempVdata, databuf);
  free(databuf);
  


  /* Writing scientific data set into HDF format */
  sdsWrite(&tempSDS, &sSwathData->rainFlag);
  sdsWrite(&tempSDS, &sSwathData->rainType);
  sdsWrite(&tempSDS, &sSwathData->shallowRain);
  sdsWrite(&tempSDS, &sSwathData->status);
  sdsWrite(&tempSDS, &sSwathData->binBBpeak);
  sdsWrite(&tempSDS, &sSwathData->HBB);
  sdsWrite(&tempSDS, &sSwathData->BBintensity);
  sdsWrite(&tempSDS, &sSwathData->freezH);
  sdsWrite(&tempSDS, &sSwathData->stormH);
  sdsWrite(&tempSDS, &sSwathData->spare);
  sdsWrite(&tempSDS, &sSwathData->BBboundary);
  sdsWrite(&tempSDS, &sSwathData->BBwidth);
  sdsWrite(&tempSDS, &sSwathData->BBstatus);

  return(TK_SUCCESS);
}

int writeScan_2A_23_R1(IO_HANDLE *sGranuleHandle, L2A_23_R1_SWATHDATA *sSwathData)
{ 
  SDS_LINK *tempSDS = sGranuleHandle->cfgSTR->vGroup->subVgroup->sds;
  VDATA_LINK *tempVdata = sGranuleHandle->cfgSTR->vGroup->subVgroup->vdata;
  uint8 *databuf, *ptr;
  int i, j;

  /* Allocate memory for packing the vdata */
  databuf = allocateVdataMemory(&tempVdata->vdataFields);
  ptr = databuf;
  memcpy(ptr, &sSwathData->scanTime, sizeof(sSwathData->scanTime));
  /* Writing packed vdata, scanTime, to an HDF file */
  vdataWrite(&tempVdata, databuf); 
  free(databuf);


  for ( i = 0; i < 49; ++i )
    for ( j = 0; j < 2; ++j )
      {
	if ( sSwathData->geolocation[i][j] > TK_DEFAULT )
	  sSwathData->geolocation_scale[i][j] =
	    (int16)RND(sSwathData->geolocation[i][j] * GEOLOC_SCALE );
	else 
	  sSwathData->geolocation_scale[i][j] = TK_MISS_I2;
      }
  sdsWrite(&tempSDS, &sSwathData->geolocation_scale);

  /* Writing scientific data set into HDF format */
  sdsWrite(&tempSDS, &sSwathData->rainType);
  sdsWrite(&tempSDS, &sSwathData->freezH);
  sdsWrite(&tempSDS, &sSwathData->stormH);

  return(TK_SUCCESS);
}


/***********************************************************************************
*
*  FUNCTION STATEMENT:
*       readScan_2A_23(IO_HANDLE *sGranuleHandle, L2A_23_SWATHDATA *sSwathData) 
*
*  DESCRIPTION:
*       This function is used to read the L2A_23 information in HDF format.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       sGranuleHandle          A structure containing information about the file.
*                               sGranuleHandle is returned by TKopen().
*
*  OUTPUT PARAMETERS: 
*       Parameter Name          Description
*       --------------          -----------
*       sSwathData              A structure containing the complete scanline data.
*                               This structure must be declared to correspond to 
*                               the data file being read.  The structure of 
*                               sSwathData is manipulated by TKreadScan() and other 
*				toolkit routines.
*
*  CALLED BY:                   TKreadScan()
*
*  ROUTINES CALLED:     
*       Routine Name            Description
*       ------------            -----------
*       allocateVdataMemory     This routine is used to allocate memory to vdata.
*       vdataRead               This routine is used to read the vdata in HDF 
*                               format.
*       sdsRead                 This routine is used to read the scientific data
*                               set in HDF format.      
*
***********************************************************************************/
int readScan_2A_23(IO_HANDLE *sGranuleHandle, L2A_23_SWATHDATA *sSwathData)
{ 
  SDS_LINK *tempSDS = sGranuleHandle->cfgSTR->vGroup->subVgroup->sds;
  VDATA_LINK *tempVdata = sGranuleHandle->cfgSTR->vGroup->subVgroup->vdata;
  uint8 *databuf, *ptr;
  int scaleFactor;
  int i;
  static int typeChangeWarnFlag = 1;
  static int typeChangeWarnFlag2 = 1;

  /* Allocate memory for unpacking the vdata */
  databuf = allocateVdataMemory(&tempVdata->vdataFields);
  ptr = databuf;
  /* Unpack the content of databuf into scanTime */
  vdataRead(&tempVdata, ptr);
  memcpy(&sSwathData->scanTime, ptr, sizeof(float64));
  free(databuf);

  sdsRead(&tempSDS, &scaleFactor, &sSwathData->geolocation);


  /* Allocate memory for unpacking the vdata */
  databuf = allocateVdataMemory(&tempVdata->vdataFields);
  ptr = databuf;
  /* Unpack the content of databuf into scanStatus block */
  vdataRead(&tempVdata, ptr);
  memcpy(&sSwathData->scanStatus.missing, ptr,
                sizeof(sSwathData->scanStatus.missing));
  ptr += sizeof(sSwathData->scanStatus.missing);
  memcpy(&sSwathData->scanStatus.validity, ptr, 
		sizeof(sSwathData->scanStatus.validity));
  ptr += sizeof(sSwathData->scanStatus.validity);
  memcpy(&sSwathData->scanStatus.qac, ptr, 
		sizeof(sSwathData->scanStatus.qac));
  ptr += sizeof(sSwathData->scanStatus.qac);
  memcpy(&sSwathData->scanStatus.geoQuality, ptr, 
		sizeof(sSwathData->scanStatus.geoQuality));
  ptr += sizeof(sSwathData->scanStatus.geoQuality);
  memcpy(&sSwathData->scanStatus.dataQuality, ptr, 
		sizeof(sSwathData->scanStatus.dataQuality));
  ptr += sizeof(sSwathData->scanStatus.dataQuality);
  memcpy(&sSwathData->scanStatus.scOrient, ptr, 
		sizeof(sSwathData->scanStatus.scOrient));
  ptr += sizeof(sSwathData->scanStatus.scOrient);
  memcpy(&sSwathData->scanStatus.acsMode, ptr, 
		sizeof(sSwathData->scanStatus.acsMode));
  ptr += sizeof(sSwathData->scanStatus.acsMode);
  memcpy(&sSwathData->scanStatus.yawUpdateS, ptr, 
		sizeof(sSwathData->scanStatus.yawUpdateS));
  ptr += sizeof(sSwathData->scanStatus.yawUpdateS);
  memcpy(&sSwathData->scanStatus.prMode, ptr, 
		sizeof(sSwathData->scanStatus.prMode));
  ptr += sizeof(sSwathData->scanStatus.prMode);
  memcpy(&sSwathData->scanStatus.prStatus1, ptr,
                sizeof(sSwathData->scanStatus.prStatus1));
  ptr += sizeof(sSwathData->scanStatus.prStatus1);
  memcpy(&sSwathData->scanStatus.prStatus2, ptr,
                sizeof(sSwathData->scanStatus.prStatus2));
  ptr += sizeof(sSwathData->scanStatus.prStatus2);
  memcpy(&sSwathData->scanStatus.fractOrbitN, ptr, 
		sizeof(sSwathData->scanStatus.fractOrbitN));
  free(databuf);


  /* Allocate memory for unpacking the vdata */
  databuf = allocateVdataMemory(&tempVdata->vdataFields);
  ptr = databuf;
  /* Unpack the content of databuf into navigation block */
  vdataRead(&tempVdata, ptr);
  memcpy(&sSwathData->navigate.scPosX, ptr, sizeof(sSwathData->navigate.scPosX));
  ptr += sizeof(sSwathData->navigate.scPosX);
  memcpy(&sSwathData->navigate.scPosY, ptr, sizeof(sSwathData->navigate.scPosY));
  ptr += sizeof(sSwathData->navigate.scPosY);
  memcpy(&sSwathData->navigate.scPosZ, ptr, sizeof(sSwathData->navigate.scPosZ));
  ptr += sizeof(sSwathData->navigate.scPosZ);
  memcpy(&sSwathData->navigate.scVelX, ptr, sizeof(sSwathData->navigate.scVelX));
  ptr += sizeof(sSwathData->navigate.scVelX);
  memcpy(&sSwathData->navigate.scVelY, ptr, sizeof(sSwathData->navigate.scVelY));
  ptr += sizeof(sSwathData->navigate.scVelY);
  memcpy(&sSwathData->navigate.scVelZ, ptr, sizeof(sSwathData->navigate.scVelZ));
  ptr += sizeof(sSwathData->navigate.scVelZ);
  memcpy(&sSwathData->navigate.scLat, ptr, sizeof(sSwathData->navigate.scLat));
  ptr += sizeof(sSwathData->navigate.scLat);
  memcpy(&sSwathData->navigate.scLon, ptr, sizeof(sSwathData->navigate.scLon));
  ptr += sizeof(sSwathData->navigate.scLon);
  memcpy(&sSwathData->navigate.scAlt, ptr, sizeof(sSwathData->navigate.scAlt));
  ptr += sizeof(sSwathData->navigate.scAlt);
  memcpy(&sSwathData->navigate.scAttRoll, ptr, 
		sizeof(sSwathData->navigate.scAttRoll));
  ptr += sizeof(sSwathData->navigate.scAttRoll);
  memcpy(&sSwathData->navigate.scAttPitch, ptr, 
		sizeof(sSwathData->navigate.scAttPitch));
  ptr += sizeof(sSwathData->navigate.scAttPitch);
  memcpy(&sSwathData->navigate.scAttYaw, ptr, 
		sizeof(sSwathData->navigate.scAttYaw));
  ptr += sizeof(sSwathData->navigate.scAttYaw);
  memcpy(&sSwathData->navigate.att1, ptr, sizeof(sSwathData->navigate.att1));
  ptr += sizeof(sSwathData->navigate.att1);
  memcpy(&sSwathData->navigate.att2, ptr, sizeof(sSwathData->navigate.att2));
  ptr += sizeof(sSwathData->navigate.att2);
  memcpy(&sSwathData->navigate.att3, ptr, sizeof(sSwathData->navigate.att3));
  ptr += sizeof(sSwathData->navigate.att3);
  memcpy(&sSwathData->navigate.att4, ptr, sizeof(sSwathData->navigate.att4));
  ptr += sizeof(sSwathData->navigate.att4);
  memcpy(&sSwathData->navigate.att5, ptr, sizeof(sSwathData->navigate.att5));
  ptr += sizeof(sSwathData->navigate.att5);
  memcpy(&sSwathData->navigate.att6, ptr, sizeof(sSwathData->navigate.att6));
  ptr += sizeof(sSwathData->navigate.att6);
  memcpy(&sSwathData->navigate.att7, ptr, sizeof(sSwathData->navigate.att7));
  ptr += sizeof(sSwathData->navigate.att7);
  memcpy(&sSwathData->navigate.att8, ptr, sizeof(sSwathData->navigate.att8));
  ptr += sizeof(sSwathData->navigate.att8);
  memcpy(&sSwathData->navigate.att9, ptr, sizeof(sSwathData->navigate.att9));
  ptr += sizeof(sSwathData->navigate.att9);
  memcpy(&sSwathData->navigate.greenHourAng, ptr, 
		sizeof(sSwathData->navigate.greenHourAng));
  free(databuf);

  /* Read SDS from HDF format */
  sdsRead(&tempSDS, &scaleFactor, &sSwathData->rainFlag);

  if (versionCompare("5.8",sGranuleHandle->TKversion) >= 0)
  {
      if (typeChangeWarnFlag==1)
          TKreportWarningParm(W_TK_SDS_TYPE_CHANGE, tempSDS->name);
      typeChangeWarnFlag=0;
      sdsRead(&tempSDS, &scaleFactor, &sSwathData->old_type_rainType);
  }

  sdsRead(&tempSDS, &scaleFactor, &sSwathData->rainType);

  if (versionCompare("5.8",sGranuleHandle->TKversion) >= 0)
  {
      for (i = 0; i< 49; i++)
          sSwathData->rainType[i] = sSwathData->old_type_rainType[i];
  }

  if (versionCompare("5.8",sGranuleHandle->TKversion) >= 0)
  {
      sdsRead(&tempSDS, &scaleFactor, &sSwathData->warmRain);
  }
  else
  {
      for (i = 0; i< 49; i++)
          sSwathData->warmRain[i] = TK_DEFAULT;
  }

  sdsRead(&tempSDS, &scaleFactor, &sSwathData->shallowRain);
  sdsRead(&tempSDS, &scaleFactor, &sSwathData->status);

  if (versionCompare("5.8",sGranuleHandle->TKversion) >= 0)
  {
      sdsRead(&tempSDS, &scaleFactor, &sSwathData->rangeBinNum);
  }
  else
  {
      for (i = 0; i< 49; i++)
          sSwathData->rangeBinNum[i] = TK_DEFAULT;
  }

  sdsRead(&tempSDS, &scaleFactor, &sSwathData->binBBpeak);
  sdsRead(&tempSDS, &scaleFactor, &sSwathData->HBB);
  sdsRead(&tempSDS, &scaleFactor, &sSwathData->BBintensity);
  sdsRead(&tempSDS, &scaleFactor, &sSwathData->freezH);
  sdsRead(&tempSDS, &scaleFactor, &sSwathData->stormH);

  if (versionCompare("5.8",sGranuleHandle->TKversion) >= 0)
  {
      if (typeChangeWarnFlag2==1)
          TKreportWarningParm(W_TK_SDS_TYPE_CHANGE, tempSDS->name);
      typeChangeWarnFlag2=0;
      sdsRead(&tempSDS, &scaleFactor, &sSwathData->old_type_spare);
  }

  sdsRead(&tempSDS, &scaleFactor, &sSwathData->spare);

  if (versionCompare("5.8",sGranuleHandle->TKversion) >= 0)
  {
      for (i = 0; i< 49; i++)
          sSwathData->spare[i] = sSwathData->old_type_spare[i];
  }

  sdsRead(&tempSDS, &scaleFactor, &sSwathData->BBboundary);
  sdsRead(&tempSDS, &scaleFactor, &sSwathData->BBwidth);
  sdsRead(&tempSDS, &scaleFactor, &sSwathData->BBstatus);

  return(TK_SUCCESS);
}

int readScan_2A_23_R1(IO_HANDLE *sGranuleHandle, L2A_23_R1_SWATHDATA *sSwathData)
{ 
  SDS_LINK *tempSDS = sGranuleHandle->cfgSTR->vGroup->subVgroup->sds;
  VDATA_LINK *tempVdata = sGranuleHandle->cfgSTR->vGroup->subVgroup->vdata;
  uint8 *databuf, *ptr;
  int i,j;
  int scaleFactor;
  static int typeChangeWarnFlag = 1;

  /* Allocate memory for unpacking the vdata */
  databuf = allocateVdataMemory(&tempVdata->vdataFields);
  ptr = databuf;
  /* Unpack the content of databuf into scanTime */
  vdataRead(&tempVdata, ptr);
  memcpy(&sSwathData->scanTime, ptr, sizeof(float64));
  free(databuf);

  sdsRead(&tempSDS, &scaleFactor, &sSwathData->geolocation_scale);
  for ( i = 0; i < 49; ++i )
    for ( j = 0; j < 2; ++j )
      {
	/*if ( sSwathData->geolocation_scale[i][j] != TK_MISS_I2 ) tk.5.74 */
        if (( sSwathData->geolocation_scale[i][0] != TK_MISS_I2 )
           || ( sSwathData->geolocation_scale[i][1] != TK_MISS_I2 ))
	  sSwathData->geolocation[i][j] = sSwathData->geolocation_scale[i][j] * 1.0 / scaleFactor;
	else
	  sSwathData->geolocation[i][j] = TK_DEFAULT;
      }

  if (versionCompare("5.8",sGranuleHandle->TKversion) >= 0)
  {
      if (typeChangeWarnFlag==1)
          TKreportWarningParm(W_TK_SDS_TYPE_CHANGE, tempSDS->name);
      typeChangeWarnFlag=0;
      sdsRead(&tempSDS, &scaleFactor, &sSwathData->old_type_rainType);
  }

  sdsRead(&tempSDS, &scaleFactor, &sSwathData->rainType);

  if (versionCompare("5.8",sGranuleHandle->TKversion) >= 0)
  {
      for (i = 0; i< 49; i++)
          sSwathData->rainType[i] = sSwathData->old_type_rainType[i];
  }

  sdsRead(&tempSDS, &scaleFactor, &sSwathData->freezH);
  sdsRead(&tempSDS, &scaleFactor, &sSwathData->stormH);

  return(TK_SUCCESS);
}


/***********************************************************************************
*
*  FUNCTION STATEMENT:
*       writeScan_2A_25(IO_HANDLE *sGranuleHandle, L2A_25_SWATHDATA *sSwathData) 
*
*  DESCRIPTION:
*       This function is used to write the L2A-25 information in HDF format.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       sGranuleHandle          A structure containing information about the file.
*                               sGranuleHandle is returned by TKopen().
*
*       sSwathData              A structure containing the complete scanline data.
*                               This structure must be declared to correspond to 
*                               the data file being written.  The structure of 
*                               sSwathData is manipulated by TKwriteScan() and other 
*				toolkit routines.
*
*  OUTPUT PARAMETERS:           None
*
*  CALLED BY:                   TKwriteScan()
*
*  ROUTINES CALLED:     
*       Routine Name            Description
*       ------------            -----------
*       allocateVdataMemory     This routine is used to allocate memory to vdata.
*       vdataWrite              This routine is used to write the vdata in HDF 
*                               format.
*       sdsWrite                This routine is used to write the scientific data
*                               set in HDF format.      
*
***********************************************************************************/
int writeScan_2A_25(IO_HANDLE *sGranuleHandle, L2A_25_SWATHDATA *sSwathData)
{
  SDS_LINK *tempSDS = sGranuleHandle->cfgSTR->vGroup->subVgroup->sds;
  VDATA_LINK *tempVdata = sGranuleHandle->cfgSTR->vGroup->subVgroup->vdata;
  uint8 *databuf, *ptr;
  int	i, j;

  /* Allocate memory for packing the vdata */
  databuf = allocateVdataMemory(&tempVdata->vdataFields);
  ptr = databuf;
  memcpy(ptr, &sSwathData->scanTime, sizeof(sSwathData->scanTime));
  /* Writing packed vdata, scanTime, to an HDF file */
  vdataWrite(&tempVdata, databuf); 
  free(databuf);

  sdsWrite(&tempSDS, &sSwathData->geolocation);


  /* Allocate memory for packing the vdata */
  databuf = allocateVdataMemory(&tempVdata->vdataFields);
  ptr = databuf;
  memcpy(ptr, &sSwathData->scanStatus.missing,
                sizeof(sSwathData->scanStatus.missing));
  ptr += sizeof(sSwathData->scanStatus.missing);
  memcpy(ptr, &sSwathData->scanStatus.validity, 
		sizeof(sSwathData->scanStatus.validity));
  ptr += sizeof(sSwathData->scanStatus.validity);
  memcpy(ptr, &sSwathData->scanStatus.qac, 
		sizeof(sSwathData->scanStatus.qac));
  ptr += sizeof(sSwathData->scanStatus.qac);
  memcpy(ptr, &sSwathData->scanStatus.geoQuality, 
		sizeof(sSwathData->scanStatus.geoQuality));
  ptr += sizeof(sSwathData->scanStatus.geoQuality);
  memcpy(ptr, &sSwathData->scanStatus.dataQuality, 
		sizeof(sSwathData->scanStatus.dataQuality));
  ptr += sizeof(sSwathData->scanStatus.dataQuality);
  memcpy(ptr, &sSwathData->scanStatus.scOrient, 
		sizeof(sSwathData->scanStatus.scOrient));
  ptr += sizeof(sSwathData->scanStatus.scOrient);
  memcpy(ptr, &sSwathData->scanStatus.acsMode, 
		sizeof(sSwathData->scanStatus.acsMode));
  ptr += sizeof(sSwathData->scanStatus.acsMode);
  memcpy(ptr, &sSwathData->scanStatus.yawUpdateS, 
		sizeof(sSwathData->scanStatus.yawUpdateS));
  ptr += sizeof(sSwathData->scanStatus.yawUpdateS);
  memcpy(ptr, &sSwathData->scanStatus.prMode, 
		sizeof(sSwathData->scanStatus.prMode));
  ptr += sizeof(sSwathData->scanStatus.prMode);
  memcpy(ptr, &sSwathData->scanStatus.prStatus1,
                sizeof(sSwathData->scanStatus.prStatus1));
  ptr += sizeof(sSwathData->scanStatus.prStatus1);
  memcpy(ptr, &sSwathData->scanStatus.prStatus2,
                sizeof(sSwathData->scanStatus.prStatus2));
  ptr += sizeof(sSwathData->scanStatus.prStatus2);
  memcpy(ptr, &sSwathData->scanStatus.fractOrbitN, 
		sizeof(sSwathData->scanStatus.fractOrbitN));
  /* Writing packed vdata, scanStatus block, to an HDF file */
  vdataWrite(&tempVdata, databuf);
  free(databuf);


  /* Allocate memory for packing the vdata */
  databuf = allocateVdataMemory(&tempVdata->vdataFields);
  ptr = databuf;
  memcpy(ptr, &sSwathData->navigate.scPosX, sizeof(sSwathData->navigate.scPosX));
  ptr += sizeof(sSwathData->navigate.scPosX);
  memcpy(ptr, &sSwathData->navigate.scPosY, sizeof(sSwathData->navigate.scPosY));
  ptr += sizeof(sSwathData->navigate.scPosY);
  memcpy(ptr, &sSwathData->navigate.scPosZ, sizeof(sSwathData->navigate.scPosZ));
  ptr += sizeof(sSwathData->navigate.scPosZ);
  memcpy(ptr, &sSwathData->navigate.scVelX, sizeof(sSwathData->navigate.scVelX));
  ptr += sizeof(sSwathData->navigate.scVelX);
  memcpy(ptr, &sSwathData->navigate.scVelY, sizeof(sSwathData->navigate.scVelY));
  ptr += sizeof(sSwathData->navigate.scVelY);
  memcpy(ptr, &sSwathData->navigate.scVelZ, sizeof(sSwathData->navigate.scVelZ));
  ptr += sizeof(sSwathData->navigate.scVelZ);
  memcpy(ptr, &sSwathData->navigate.scLat, sizeof(sSwathData->navigate.scLat));
  ptr += sizeof(sSwathData->navigate.scLat);
  memcpy(ptr, &sSwathData->navigate.scLon, sizeof(sSwathData->navigate.scLon));
  ptr += sizeof(sSwathData->navigate.scLon);
  memcpy(ptr, &sSwathData->navigate.scAlt, sizeof(sSwathData->navigate.scAlt));
  ptr += sizeof(sSwathData->navigate.scAlt);
  memcpy(ptr, &sSwathData->navigate.scAttRoll, 
		sizeof(sSwathData->navigate.scAttRoll));
  ptr += sizeof(sSwathData->navigate.scAttRoll);
  memcpy(ptr, &sSwathData->navigate.scAttPitch, 
		sizeof(sSwathData->navigate.scAttPitch));
  ptr += sizeof(sSwathData->navigate.scAttPitch);
  memcpy(ptr, &sSwathData->navigate.scAttYaw, 
		sizeof(sSwathData->navigate.scAttYaw));
  ptr += sizeof(sSwathData->navigate.scAttYaw);
  memcpy(ptr, &sSwathData->navigate.att1, sizeof(sSwathData->navigate.att1));
  ptr += sizeof(sSwathData->navigate.att1);
  memcpy(ptr, &sSwathData->navigate.att2, sizeof(sSwathData->navigate.att2));
  ptr += sizeof(sSwathData->navigate.att2);
  memcpy(ptr, &sSwathData->navigate.att3, sizeof(sSwathData->navigate.att3));
  ptr += sizeof(sSwathData->navigate.att3);
  memcpy(ptr, &sSwathData->navigate.att4, sizeof(sSwathData->navigate.att4));
  ptr += sizeof(sSwathData->navigate.att4);
  memcpy(ptr, &sSwathData->navigate.att5, sizeof(sSwathData->navigate.att5));
  ptr += sizeof(sSwathData->navigate.att5);
  memcpy(ptr, &sSwathData->navigate.att6, sizeof(sSwathData->navigate.att6));
  ptr += sizeof(sSwathData->navigate.att6);
  memcpy(ptr, &sSwathData->navigate.att7, sizeof(sSwathData->navigate.att7));
  ptr += sizeof(sSwathData->navigate.att7);
  memcpy(ptr, &sSwathData->navigate.att8, sizeof(sSwathData->navigate.att8));
  ptr += sizeof(sSwathData->navigate.att8);
  memcpy(ptr, &sSwathData->navigate.att9, sizeof(sSwathData->navigate.att9));
  ptr += sizeof(sSwathData->navigate.att9);
  memcpy(ptr, &sSwathData->navigate.greenHourAng, 
		sizeof(sSwathData->navigate.greenHourAng));
  /* Writing packed vdata, navigation block, to an HDF file */
  vdataWrite(&tempVdata, databuf);
  free(databuf);
  
  sdsWrite(&tempSDS, &sSwathData->scLocalZenith);

  /* Write SDS into HDF format */
  /*
   * Scaling calculation.  For writing,
   *	Multiplied the data by L2A25_RAIN.
   */
  for (i = 0; i < 49; ++i)
    for (j = 0; j < 80; ++j)
    {
        /* The Following code was added to take care of the Missing or
           invalid data.  The reason to add this code was not to scale
           the invalid Data */

      if(sSwathData->rain[i][j] > TK_DEFAULT)
	  sSwathData->rain_scale[i][j] = 
		(int16)RND(sSwathData->rain[i][j] * L2A25_RAIN);
      else
	  sSwathData->rain_scale[i][j] = TK_MISS_I2;
    }

  sdsWrite(&tempSDS, &sSwathData->rain_scale);

  sdsWrite(&tempSDS, &sSwathData->reliab);

  /*
   * Scaling calculation.  For writing,
   *    Multiplied the data by L2A25_CORRECTZFACTOR.
   */
  for (i = 0; i < 49; ++i)
    for (j = 0; j < 80; ++j)
    {
        /* The Following code was added to take care of the Missing or
           invalid data.  The reason to add this code was not to scale
           the invalid Data */

      if(sSwathData->correctZFactor[i][j] > TK_DEFAULT)
          sSwathData->correctZFactor_scale[i][j] = 
	  (int16)RND(sSwathData->correctZFactor[i][j] * L2A25_CORRECTZFACTOR);
      else
          sSwathData->correctZFactor_scale[i][j] = TK_MISS_I2;
    }
  sdsWrite(&tempSDS, &sSwathData->correctZFactor_scale);

  /*
   * Scaling calculation.  For writing,
   *    Multiplied the data by L2A25_ATTENPARMALPHA.
   */
  for (i = 0; i < 49; ++i)
    for (j = 0; j < 5; ++j)
    {
        /* The Following code was added to take care of the Missing or
           invalid data.  The reason to add this code was not to scale
           the invalid Data */

      if(sSwathData->attenParmAlpha[i][j] > TK_DEFAULT)
          sSwathData->attenParmAlpha_scale[i][j] = 
	  (int16)RND(sSwathData->attenParmAlpha[i][j] * L2A25_ATTENPARMALPHA);
      else
          sSwathData->attenParmAlpha_scale[i][j] = TK_MISS_I2;
    }
  sdsWrite(&tempSDS, &sSwathData->attenParmAlpha_scale);

  /*
   * Scaling calculation.  For writing,
   *    Multiplied the data by L2A25_ATTENPARMBETA.
   */
  for (i = 0; i < 49; ++i)
  {
        /* The Following code was added to take care of the Missing or
           invalid data.  The reason to add this code was not to scale
           the invalid Data */

     if(sSwathData->attenParmBeta[i] > TK_DEFAULT)
         sSwathData->attenParmBeta_scale[i] =
          (int16)RND(sSwathData->attenParmBeta[i] * L2A25_ATTENPARMBETA);
     else
         sSwathData->attenParmBeta_scale[i] = TK_MISS_I2;
  }
  sdsWrite(&tempSDS, &sSwathData->attenParmBeta_scale);

  sdsWrite(&tempSDS, &sSwathData->parmNode);
  sdsWrite(&tempSDS, &sSwathData->precipWaterParmA);
  sdsWrite(&tempSDS, &sSwathData->precipWaterParmB);

  /*
   * Scaling calculation.  For writing,
   *    Multiplied the data by L2A25_ZRPARMA.
   */
  for (i = 0; i < 49; ++i)
    for (j = 0; j < 5; ++j)
    {
        /* The Following code was added to take care of the Missing or
           invalid data.  The reason to add this code was not to scale
           the invalid Data */

      if(sSwathData->ZRParmA[i][j] > TK_DEFAULT)
          sSwathData->ZRParmA_scale[i][j] = 
		(int16)RND(sSwathData->ZRParmA[i][j] * L2A25_ZRPARMA);
      else
          sSwathData->ZRParmA_scale[i][j] = TK_MISS_I2;
    }
  sdsWrite(&tempSDS, &sSwathData->ZRParmA_scale);

  /*
   * Scaling calculation.  For writing,
   *    Multiplied the data by L2A25_ZRPARMB.
   */
  for (i = 0; i < 49; ++i)
    for (j = 0; j < 5; ++j)
    {
        /* The Following code was added to take care of the Missing or
           invalid data.  The reason to add this code was not to scale
           the invalid Data */

      if(sSwathData->ZRParmB[i][j] > TK_DEFAULT)
          sSwathData->ZRParmB_scale[i][j] = 
		(int16)RND(sSwathData->ZRParmB[i][j] * L2A25_ZRPARMB);
      else
          sSwathData->ZRParmB_scale[i][j] = TK_MISS_I2;
    }
  sdsWrite(&tempSDS, &sSwathData->ZRParmB_scale);


  sdsWrite(&tempSDS, &sSwathData->zmmax);
  sdsWrite(&tempSDS, &sSwathData->rainFlag);
  sdsWrite(&tempSDS, &sSwathData->rangeBinNum);


  /*
   * Scaling calculation.  For writing,
   *    Multiplied the data by L2A25_RAINAVE.
   */
  for (i = 0; i < 49; ++i)
    for (j = 0; j < 2; ++j)
    {
        /* The Following code was added to take care of the Missing or
           invalid data.  The reason to add this code was not to scale
           the invalid Data */

      if(sSwathData->rainAve[i][j] > TK_DEFAULT)
	  sSwathData->rainAve_scale[i][j] = 
		(int16)RND(sSwathData->rainAve[i][j] * L2A25_RAINAVE);
      else
	  sSwathData->rainAve_scale[i][j] = TK_MISS_I2;
    }
  sdsWrite(&tempSDS, &sSwathData->rainAve_scale);


  /*
   * Scaling calculation.  For writing,
   *    Multiplied the data by L2A25_LIQWATERSUM.
   */
  for (i = 0; i < 49; ++i)
    {
        /* The Following code was added to take care of the Missing or
           invalid data.  The reason to add this code was not to scale
           the invalid Data */

      if(sSwathData->precipWaterSum[i] > TK_DEFAULT)
	  sSwathData->precipWaterSum_scale[i] = 
		(int16)RND(sSwathData->precipWaterSum[i] * L2A25_LIQWATERSUM);
      else
	  sSwathData->precipWaterSum_scale[i] = TK_MISS_I2;
    }
  sdsWrite(&tempSDS, &sSwathData->precipWaterSum_scale);

  sdsWrite(&tempSDS, &sSwathData->epsilon_0);

  sdsWrite(&tempSDS, &sSwathData->method);
  sdsWrite(&tempSDS, &sSwathData->epsilon);
  sdsWrite(&tempSDS, &sSwathData->zeta);
  sdsWrite(&tempSDS, &sSwathData->zeta_mn);
  sdsWrite(&tempSDS, &sSwathData->zeta_sd);

  /*sdsWrite(&tempSDS, &sSwathData->xi);*/
  sdsWrite(&tempSDS, &sSwathData->sigmaZero);
  sdsWrite(&tempSDS, &sSwathData->freezH);

  sdsWrite(&tempSDS, &sSwathData->nubfCorrectFactor);
  sdsWrite(&tempSDS, &sSwathData->qualityFlag);
  sdsWrite(&tempSDS, &sSwathData->nearSurfRain);
  sdsWrite(&tempSDS, &sSwathData->nearSurfZ);
  sdsWrite(&tempSDS, &sSwathData->e_SurfRain);
  sdsWrite(&tempSDS, &sSwathData->pia);
  sdsWrite(&tempSDS, &sSwathData->errorRain);
  sdsWrite(&tempSDS, &sSwathData->errorZ);
  sdsWrite(&tempSDS, &sSwathData->spare);
  sdsWrite(&tempSDS, &sSwathData->rainType);

  return(TK_SUCCESS);
}

/* 2A25 Nadir writeScan (2N25) Begin */

int writeScan_2N_25(IO_HANDLE *sGranuleHandle, L2N_25_SWATHDATA *sSwathData)
{
  SDS_LINK *tempSDS = sGranuleHandle->cfgSTR->vGroup->subVgroup->sds;
  VDATA_LINK *tempVdata = sGranuleHandle->cfgSTR->vGroup->subVgroup->vdata;
  uint8 *databuf, *ptr;
  int	i, j;

  /* Allocate memory for packing the vdata */
  databuf = allocateVdataMemory(&tempVdata->vdataFields);
  ptr = databuf;
  memcpy(ptr, &sSwathData->scanTime, sizeof(sSwathData->scanTime));
  /* Writing packed vdata, scanTime, to an HDF file */
  vdataWrite(&tempVdata, databuf); 
  free(databuf);

  sdsWrite(&tempSDS, &sSwathData->geolocation);


  /* Allocate memory for packing the vdata */
  databuf = allocateVdataMemory(&tempVdata->vdataFields);
  ptr = databuf;
  memcpy(ptr, &sSwathData->scanStatus.missing,
                sizeof(sSwathData->scanStatus.missing));
  ptr += sizeof(sSwathData->scanStatus.missing);
  memcpy(ptr, &sSwathData->scanStatus.validity, 
		sizeof(sSwathData->scanStatus.validity));
  ptr += sizeof(sSwathData->scanStatus.validity);
  memcpy(ptr, &sSwathData->scanStatus.qac, 
		sizeof(sSwathData->scanStatus.qac));
  ptr += sizeof(sSwathData->scanStatus.qac);
  memcpy(ptr, &sSwathData->scanStatus.geoQuality, 
		sizeof(sSwathData->scanStatus.geoQuality));
  ptr += sizeof(sSwathData->scanStatus.geoQuality);
  memcpy(ptr, &sSwathData->scanStatus.dataQuality, 
		sizeof(sSwathData->scanStatus.dataQuality));
  ptr += sizeof(sSwathData->scanStatus.dataQuality);
  memcpy(ptr, &sSwathData->scanStatus.scOrient, 
		sizeof(sSwathData->scanStatus.scOrient));
  ptr += sizeof(sSwathData->scanStatus.scOrient);
  memcpy(ptr, &sSwathData->scanStatus.acsMode, 
		sizeof(sSwathData->scanStatus.acsMode));
  ptr += sizeof(sSwathData->scanStatus.acsMode);
  memcpy(ptr, &sSwathData->scanStatus.yawUpdateS, 
		sizeof(sSwathData->scanStatus.yawUpdateS));
  ptr += sizeof(sSwathData->scanStatus.yawUpdateS);
  memcpy(ptr, &sSwathData->scanStatus.prMode, 
		sizeof(sSwathData->scanStatus.prMode));
  ptr += sizeof(sSwathData->scanStatus.prMode);
  memcpy(ptr, &sSwathData->scanStatus.prStatus1,
                sizeof(sSwathData->scanStatus.prStatus1));
  ptr += sizeof(sSwathData->scanStatus.prStatus1);
  memcpy(ptr, &sSwathData->scanStatus.prStatus2,
                sizeof(sSwathData->scanStatus.prStatus2));
  ptr += sizeof(sSwathData->scanStatus.prStatus2);
  memcpy(ptr, &sSwathData->scanStatus.fractOrbitN, 
		sizeof(sSwathData->scanStatus.fractOrbitN));
  /* Writing packed vdata, scanStatus block, to an HDF file */
  vdataWrite(&tempVdata, databuf);
  free(databuf);


  /* Allocate memory for packing the vdata */
  databuf = allocateVdataMemory(&tempVdata->vdataFields);
  ptr = databuf;
  memcpy(ptr, &sSwathData->navigate.scPosX, sizeof(sSwathData->navigate.scPosX));
  ptr += sizeof(sSwathData->navigate.scPosX);
  memcpy(ptr, &sSwathData->navigate.scPosY, sizeof(sSwathData->navigate.scPosY));
  ptr += sizeof(sSwathData->navigate.scPosY);
  memcpy(ptr, &sSwathData->navigate.scPosZ, sizeof(sSwathData->navigate.scPosZ));
  ptr += sizeof(sSwathData->navigate.scPosZ);
  memcpy(ptr, &sSwathData->navigate.scVelX, sizeof(sSwathData->navigate.scVelX));
  ptr += sizeof(sSwathData->navigate.scVelX);
  memcpy(ptr, &sSwathData->navigate.scVelY, sizeof(sSwathData->navigate.scVelY));
  ptr += sizeof(sSwathData->navigate.scVelY);
  memcpy(ptr, &sSwathData->navigate.scVelZ, sizeof(sSwathData->navigate.scVelZ));
  ptr += sizeof(sSwathData->navigate.scVelZ);
  memcpy(ptr, &sSwathData->navigate.scLat, sizeof(sSwathData->navigate.scLat));
  ptr += sizeof(sSwathData->navigate.scLat);
  memcpy(ptr, &sSwathData->navigate.scLon, sizeof(sSwathData->navigate.scLon));
  ptr += sizeof(sSwathData->navigate.scLon);
  memcpy(ptr, &sSwathData->navigate.scAlt, sizeof(sSwathData->navigate.scAlt));
  ptr += sizeof(sSwathData->navigate.scAlt);
  memcpy(ptr, &sSwathData->navigate.scAttRoll, 
		sizeof(sSwathData->navigate.scAttRoll));
  ptr += sizeof(sSwathData->navigate.scAttRoll);
  memcpy(ptr, &sSwathData->navigate.scAttPitch, 
		sizeof(sSwathData->navigate.scAttPitch));
  ptr += sizeof(sSwathData->navigate.scAttPitch);
  memcpy(ptr, &sSwathData->navigate.scAttYaw, 
		sizeof(sSwathData->navigate.scAttYaw));
  ptr += sizeof(sSwathData->navigate.scAttYaw);
  memcpy(ptr, &sSwathData->navigate.att1, sizeof(sSwathData->navigate.att1));
  ptr += sizeof(sSwathData->navigate.att1);
  memcpy(ptr, &sSwathData->navigate.att2, sizeof(sSwathData->navigate.att2));
  ptr += sizeof(sSwathData->navigate.att2);
  memcpy(ptr, &sSwathData->navigate.att3, sizeof(sSwathData->navigate.att3));
  ptr += sizeof(sSwathData->navigate.att3);
  memcpy(ptr, &sSwathData->navigate.att4, sizeof(sSwathData->navigate.att4));
  ptr += sizeof(sSwathData->navigate.att4);
  memcpy(ptr, &sSwathData->navigate.att5, sizeof(sSwathData->navigate.att5));
  ptr += sizeof(sSwathData->navigate.att5);
  memcpy(ptr, &sSwathData->navigate.att6, sizeof(sSwathData->navigate.att6));
  ptr += sizeof(sSwathData->navigate.att6);
  memcpy(ptr, &sSwathData->navigate.att7, sizeof(sSwathData->navigate.att7));
  ptr += sizeof(sSwathData->navigate.att7);
  memcpy(ptr, &sSwathData->navigate.att8, sizeof(sSwathData->navigate.att8));
  ptr += sizeof(sSwathData->navigate.att8);
  memcpy(ptr, &sSwathData->navigate.att9, sizeof(sSwathData->navigate.att9));
  ptr += sizeof(sSwathData->navigate.att9);
  memcpy(ptr, &sSwathData->navigate.greenHourAng, 
		sizeof(sSwathData->navigate.greenHourAng));
  /* Writing packed vdata, navigation block, to an HDF file */
  vdataWrite(&tempVdata, databuf);
  free(databuf);
  
  sdsWrite(&tempSDS, &sSwathData->scLocalZenith);

  /* Write SDS into HDF format */
  /*
   * Scaling calculation.  For writing,
   *	Multiplied the data by L2A25_RAIN.
   */
  for (i = 0; i < 1; ++i)
    for (j = 0; j < 80; ++j)
    {
        /* The Following code was added to take care of the Missing or
           invalid data.  The reason to add this code was not to scale
           the invalid Data */

      if(sSwathData->rain[i][j] > TK_DEFAULT)
	  sSwathData->rain_scale[i][j] = 
		(int16)RND(sSwathData->rain[i][j] * L2A25_RAIN);
      else
	  sSwathData->rain_scale[i][j] = TK_MISS_I2;
    }

  sdsWrite(&tempSDS, &sSwathData->rain_scale);

  sdsWrite(&tempSDS, &sSwathData->reliab);

  /*
   * Scaling calculation.  For writing,
   *    Multiplied the data by L2A25_CORRECTZFACTOR.
   */
  for (i = 0; i < 1; ++i)
    for (j = 0; j < 80; ++j)
    {
        /* The Following code was added to take care of the Missing or
           invalid data.  The reason to add this code was not to scale
           the invalid Data */

      if(sSwathData->correctZFactor[i][j] > TK_DEFAULT)
          sSwathData->correctZFactor_scale[i][j] = 
	  (int16)RND(sSwathData->correctZFactor[i][j] * L2A25_CORRECTZFACTOR);
      else
          sSwathData->correctZFactor_scale[i][j] = TK_MISS_I2;
    }
  sdsWrite(&tempSDS, &sSwathData->correctZFactor_scale);

  /*
   * Scaling calculation.  For writing,
   *    Multiplied the data by L2A25_ATTENPARMALPHA.
   */
  for (i = 0; i < 1; ++i)
    for (j = 0; j < 5; ++j)
    {
        /* The Following code was added to take care of the Missing or
           invalid data.  The reason to add this code was not to scale
           the invalid Data */

      if(sSwathData->attenParmAlpha[i][j] > TK_DEFAULT)
          sSwathData->attenParmAlpha_scale[i][j] = 
	  (int16)RND(sSwathData->attenParmAlpha[i][j] * L2A25_ATTENPARMALPHA);
      else
          sSwathData->attenParmAlpha_scale[i][j] = TK_MISS_I2;
    }
  sdsWrite(&tempSDS, &sSwathData->attenParmAlpha_scale);

  /*
   * Scaling calculation.  For writing,
   *    Multiplied the data by L2A25_ATTENPARMBETA.
   */
  for (i = 0; i < 1; ++i)
  {
        /* The Following code was added to take care of the Missing or
           invalid data.  The reason to add this code was not to scale
           the invalid Data */

     if(sSwathData->attenParmBeta[i] > TK_DEFAULT)
         sSwathData->attenParmBeta_scale[i] =
          (int16)RND(sSwathData->attenParmBeta[i] * L2A25_ATTENPARMBETA);
     else
         sSwathData->attenParmBeta_scale[i] = TK_MISS_I2;
  }
  sdsWrite(&tempSDS, &sSwathData->attenParmBeta_scale);

  sdsWrite(&tempSDS, &sSwathData->parmNode);
  sdsWrite(&tempSDS, &sSwathData->precipWaterParmA);
  sdsWrite(&tempSDS, &sSwathData->precipWaterParmB);

  /*
   * Scaling calculation.  For writing,
   *    Multiplied the data by L2A25_ZRPARMA.
   */
  for (i = 0; i < 1; ++i)
    for (j = 0; j < 5; ++j)
    {
        /* The Following code was added to take care of the Missing or
           invalid data.  The reason to add this code was not to scale
           the invalid Data */

      if(sSwathData->ZRParmA[i][j] > TK_DEFAULT)
          sSwathData->ZRParmA_scale[i][j] = 
		(int16)RND(sSwathData->ZRParmA[i][j] * L2A25_ZRPARMA);
      else
          sSwathData->ZRParmA_scale[i][j] = TK_MISS_I2;
    }
  sdsWrite(&tempSDS, &sSwathData->ZRParmA_scale);

  /*
   * Scaling calculation.  For writing,
   *    Multiplied the data by L2A25_ZRPARMB.
   */
  for (i = 0; i < 1; ++i)
    for (j = 0; j < 5; ++j)
    {
        /* The Following code was added to take care of the Missing or
           invalid data.  The reason to add this code was not to scale
           the invalid Data */

      if(sSwathData->ZRParmB[i][j] > TK_DEFAULT)
          sSwathData->ZRParmB_scale[i][j] = 
		(int16)RND(sSwathData->ZRParmB[i][j] * L2A25_ZRPARMB);
      else
          sSwathData->ZRParmB_scale[i][j] = TK_MISS_I2;
    }
  sdsWrite(&tempSDS, &sSwathData->ZRParmB_scale);


  sdsWrite(&tempSDS, &sSwathData->zmmax);
  sdsWrite(&tempSDS, &sSwathData->rainFlag);
  sdsWrite(&tempSDS, &sSwathData->rangeBinNum);


  /*
   * Scaling calculation.  For writing,
   *    Multiplied the data by L2A25_RAINAVE.
   */
  for (i = 0; i < 1; ++i)
    for (j = 0; j < 2; ++j)
    {
        /* The Following code was added to take care of the Missing or
           invalid data.  The reason to add this code was not to scale
           the invalid Data */

      if(sSwathData->rainAve[i][j] > TK_DEFAULT)
	  sSwathData->rainAve_scale[i][j] = 
		(int16)RND(sSwathData->rainAve[i][j] * L2A25_RAINAVE);
      else
	  sSwathData->rainAve_scale[i][j] = TK_MISS_I2;
    }
  sdsWrite(&tempSDS, &sSwathData->rainAve_scale);


  /*
   * Scaling calculation.  For writing,
   *    Multiplied the data by L2A25_LIQWATERSUM.
   */
  for (i = 0; i < 1; ++i)
    {
        /* The Following code was added to take care of the Missing or
           invalid data.  The reason to add this code was not to scale
           the invalid Data */

      if(sSwathData->precipWaterSum[i] > TK_DEFAULT)
	  sSwathData->precipWaterSum_scale[i] = 
		(int16)RND(sSwathData->precipWaterSum[i] * L2A25_LIQWATERSUM);
      else
	  sSwathData->precipWaterSum_scale[i] = TK_MISS_I2;
    }
  sdsWrite(&tempSDS, &sSwathData->precipWaterSum_scale);

  sdsWrite(&tempSDS, &sSwathData->epsilon_0);

  sdsWrite(&tempSDS, &sSwathData->method);
  sdsWrite(&tempSDS, &sSwathData->epsilon);
  sdsWrite(&tempSDS, &sSwathData->zeta);
  sdsWrite(&tempSDS, &sSwathData->zeta_mn);
  sdsWrite(&tempSDS, &sSwathData->zeta_sd);

  /*sdsWrite(&tempSDS, &sSwathData->xi);*/
  sdsWrite(&tempSDS, &sSwathData->sigmaZero);
  sdsWrite(&tempSDS, &sSwathData->freezH);

  sdsWrite(&tempSDS, &sSwathData->nubfCorrectFactor);
  sdsWrite(&tempSDS, &sSwathData->qualityFlag);
  sdsWrite(&tempSDS, &sSwathData->nearSurfRain);
  sdsWrite(&tempSDS, &sSwathData->nearSurfZ);
  sdsWrite(&tempSDS, &sSwathData->e_SurfRain);
  sdsWrite(&tempSDS, &sSwathData->pia);
  sdsWrite(&tempSDS, &sSwathData->errorRain);
  sdsWrite(&tempSDS, &sSwathData->errorZ);
  sdsWrite(&tempSDS, &sSwathData->spare);
  sdsWrite(&tempSDS, &sSwathData->rainType);

  return(TK_SUCCESS);
}


/* 2A25 Nadir writeScan (2N25) End */


/*  Subsetting Function */

int writeScan_2A_25_R1(IO_HANDLE *sGranuleHandle, L2A_25_R1_SWATHDATA *sSwathData)
{
  SDS_LINK *tempSDS = sGranuleHandle->cfgSTR->vGroup->subVgroup->sds;
  VDATA_LINK *tempVdata = sGranuleHandle->cfgSTR->vGroup->subVgroup->vdata;
  uint8 *databuf, *ptr;
  int	i, j;

  /* Allocate memory for packing the vdata */
  databuf = allocateVdataMemory(&tempVdata->vdataFields);
  ptr = databuf;
  memcpy(ptr, &sSwathData->scanTime, sizeof(sSwathData->scanTime));
  /* Writing packed vdata, scanTime, to an HDF file */
  vdataWrite(&tempVdata, databuf); 
  free(databuf);


  for ( i = 0; i < 49; ++i )
    for ( j = 0; j < 2; ++j )
      {
	if ( sSwathData->geolocation[i][j] > TK_DEFAULT )
	  sSwathData->geolocation_scale[i][j] =
	    (int16)RND(sSwathData->geolocation[i][j] * GEOLOC_SCALE );
	else 
	  sSwathData->geolocation_scale[i][j] = TK_MISS_I2;
      }
  sdsWrite(&tempSDS, &sSwathData->geolocation_scale);

  /* Write SDS into HDF format */
  /*
   * Scaling calculation.  For writing,
   *	Multiplied the data by L2A25_RAIN.
   */
  for (i = 0; i < 49; ++i)
    {
        /* The Following code was added to take care of the Missing or
           invalid data.  The reason to add this code was not to scale
           the invalid Data */

      if(sSwathData->e_SurfRain[i] > TK_DEFAULT)
	  sSwathData->e_SurfRain_scale[i] = 
		(int16)RND(sSwathData->e_SurfRain[i] * L2A25_E_SURFRAIN);
      else
	  sSwathData->e_SurfRain_scale[i] = TK_MISS_I2;
    }
  sdsWrite(&tempSDS, &sSwathData->e_SurfRain_scale);

  for (i = 0; i < 49; ++i)
    {
        /* The Following code was added to take care of the Missing or
           invalid data.  The reason to add this code was not to scale
           the invalid Data */

      if(sSwathData->nearSurfZ[i] > TK_DEFAULT)
	  sSwathData->nearSurfZ_scale[i] = 
		(int16)RND(sSwathData->nearSurfZ[i] * L2A25_NEARSURFZ);
      else
	  sSwathData->nearSurfZ_scale[i] = TK_MISS_I2;
    }
  sdsWrite(&tempSDS, &sSwathData->nearSurfZ_scale);

  sdsWrite(&tempSDS, &sSwathData->nearSurfBin);

  return(TK_SUCCESS);
}

int writeScan_2A_25_R2(IO_HANDLE *sGranuleHandle, L2A_25_R2_SWATHDATA *sSwathData)
{
  SDS_LINK *tempSDS = sGranuleHandle->cfgSTR->vGroup->subVgroup->sds;
  VDATA_LINK *tempVdata = sGranuleHandle->cfgSTR->vGroup->subVgroup->vdata;
  uint8 *databuf, *ptr;
  int	i, j;

  /* Allocate memory for packing the vdata */
  databuf = allocateVdataMemory(&tempVdata->vdataFields);
  ptr = databuf;
  memcpy(ptr, &sSwathData->scanTime, sizeof(sSwathData->scanTime));
  /* Writing packed vdata, scanTime, to an HDF file */
  vdataWrite(&tempVdata, databuf); 
  free(databuf);

  for ( i = 0; i < 49; ++i )
    for ( j = 0; j < 2; ++j )
      {
	if ( sSwathData->geolocation[i][j] > TK_DEFAULT )
	  sSwathData->geolocation_scale[i][j] =
	    (int16)RND(sSwathData->geolocation[i][j] * GEOLOC_SCALE );
	else 
	  sSwathData->geolocation_scale[i][j] = TK_MISS_I2;
      }
  sdsWrite(&tempSDS, &sSwathData->geolocation_scale);

  /* Write SDS into HDF format */
  /*
   * Scaling calculation.  For writing,
   *	Multiplied the data by L2A25_RAIN.
   */
  for (i = 0; i < 49; ++i)
    for (j = 0; j < 20; ++j)
    {
        /* The Following code was added to take care of the Missing or
           invalid data.  The reason to add this code was not to scale
           the invalid Data */

      if(sSwathData->rain[i][j] > TK_DEFAULT)
	  sSwathData->rain_scale[i][j] = 
		(int16)RND(sSwathData->rain[i][j] * L2A25_RAIN);
      else
	  sSwathData->rain_scale[i][j] = TK_MISS_I2;
    }

  sdsWrite(&tempSDS, &sSwathData->rain_scale);

  return(TK_SUCCESS);
}

int readScan_2A_25_R1(IO_HANDLE *sGranuleHandle, L2A_25_R1_SWATHDATA *sSwathData)
{ 
  SDS_LINK *tempSDS = sGranuleHandle->cfgSTR->vGroup->subVgroup->sds;
  VDATA_LINK *tempVdata = sGranuleHandle->cfgSTR->vGroup->subVgroup->vdata;
  uint8 *databuf, *ptr;
  int	i, j;
  int scaleFactor;


  /* Allocate memory for unpacking the vdata */
  databuf = allocateVdataMemory(&tempVdata->vdataFields);
  ptr = databuf;
  /* Unpack the content of databuf into scanTime */
  vdataRead(&tempVdata, ptr);
  memcpy(&sSwathData->scanTime, ptr, sizeof(float64));
  free(databuf);

  sdsRead(&tempSDS, &scaleFactor, &sSwathData->geolocation_scale);
  for ( i = 0; i < 49; ++i )
    for ( j = 0; j < 2; ++j )
      {
        /*if ( sSwathData->geolocation_scale[i][j] != TK_MISS_I2 ) tk5.74*/
        if (( sSwathData->geolocation_scale[i][0] != TK_MISS_I2 )
           || ( sSwathData->geolocation_scale[i][1] != TK_MISS_I2 ))
	  sSwathData->geolocation[i][j] = sSwathData->geolocation_scale[i][j] * 1.0 / scaleFactor;
	else
	  sSwathData->geolocation[i][j] = TK_DEFAULT;
      }

                                                                                                      
  /* Read SDS from HDF format */
  /*
   * Scaling calculation.  For reading,
   *    Divided the data by L2A25_RAIN.
   */


  if (versionCompare("6.38",sGranuleHandle->TKversion) >= 0)
  {
  sdsRead(&tempSDS, &scaleFactor, &sSwathData->nearSurfRain_scale);
  for (i = 0; i < 49; ++i)
    {
      if (sSwathData->nearSurfRain_scale[i] != TK_MISS_I2)
	sSwathData->nearSurfRain[i] = sSwathData->nearSurfRain_scale[i] * 1.0 / scaleFactor;
      else
	sSwathData->nearSurfRain[i] = TK_DEFAULT;
    }
  }

  sdsRead(&tempSDS, &scaleFactor, &sSwathData->e_SurfRain_scale);
  for (i = 0; i < 49; ++i)
    {
      if (sSwathData->e_SurfRain_scale[i] != TK_MISS_I2)
	sSwathData->e_SurfRain[i] = sSwathData->e_SurfRain_scale[i] * 1.0 / scaleFactor;
      else
	sSwathData->e_SurfRain[i] = TK_DEFAULT;
    }

  sdsRead(&tempSDS, &scaleFactor, &sSwathData->nearSurfZ_scale);
  for (i = 0; i < 49; ++i)
    {
      if (sSwathData->nearSurfZ_scale[i] != TK_MISS_I2)
	sSwathData->nearSurfZ[i] = sSwathData->nearSurfZ_scale[i] * 1.0 / scaleFactor;
      else
	sSwathData->nearSurfZ[j] = TK_DEFAULT;
    }

  sdsRead(&tempSDS, &scaleFactor, &sSwathData->nearSurfBin);  

  return(TK_SUCCESS);
}

int readScan_2A_25_R2 (IO_HANDLE *sGranuleHandle, L2A_25_R2_SWATHDATA *sSwathData)
{ 
  SDS_LINK *tempSDS = sGranuleHandle->cfgSTR->vGroup->subVgroup->sds;
  VDATA_LINK *tempVdata = sGranuleHandle->cfgSTR->vGroup->subVgroup->vdata;
  uint8 *databuf, *ptr;
  int	i, j;
  int scaleFactor;

  /* Allocate memory for unpacking the vdata */
  databuf = allocateVdataMemory(&tempVdata->vdataFields);
  ptr = databuf;
  /* Unpack the content of databuf into scanTime */
  vdataRead(&tempVdata, ptr);
  memcpy(&sSwathData->scanTime, ptr, sizeof(float64));
  free(databuf);

  sdsRead(&tempSDS, &scaleFactor, &sSwathData->geolocation_scale);
  for ( i = 0; i < 49; ++i )
    for ( j = 0; j < 2; ++j )
      {
        /*if ( sSwathData->geolocation_scale[i][j] != TK_MISS_I2 ) tk5.74*/
        if (( sSwathData->geolocation_scale[i][0] != TK_MISS_I2 )
           || ( sSwathData->geolocation_scale[i][1] != TK_MISS_I2 ))
	  sSwathData->geolocation[i][j] = sSwathData->geolocation_scale[i][j] * 1.0 / scaleFactor;
	else
	  sSwathData->geolocation[i][j] = TK_DEFAULT;
      }

  /* Read SDS from HDF format */
  /*
   * Scaling calculation.  For reading,
   *    Divided the data by L2A25_RAIN.
   */
  sdsRead(&tempSDS, &scaleFactor, &sSwathData->rain_scale);
  for (i = 0; i < 49; ++i)
    for (j = 0; j < 20; ++j)
        {
        if (sSwathData->rain_scale[i][j] != TK_MISS_I2)
            sSwathData->rain[i][j] = sSwathData->rain_scale[i][j] * 1.0 / scaleFactor;
        else
           sSwathData->rain[i][j] = TK_DEFAULT;
        }

  return(TK_SUCCESS);
}

/***********************************************************************************
*
*  FUNCTION STATEMENT:
*       readScan_2A_25(IO_HANDLE *sGranuleHandle, L2A_25_SWATHDATA *sSwathData) 
*
*  DESCRIPTION:
*       This function is used to read the L2A_25 information in HDF format.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       sGranuleHandle          A structure containing information about the file.
*                               sGranuleHandle is returned by TKopen().
*
*  OUTPUT PARAMETERS: 
*       Parameter Name          Description
*       --------------          -----------
*       sSwathData              A structure containing the complete scanline data.
*                               This structure must be declared to correspond to 
*                               the data file being read.  The structure of 
*                               sSwathData is manipulated by TKreadScan() and other 
*				toolkit routines.
*
*  CALLED BY:                   TKreadScan()
*
*  ROUTINES CALLED:     
*       Routine Name            Description
*       ------------            -----------
*       allocateVdataMemory     This routine is used to allocate memory to vdata.
*       vdataRead               This routine is used to read the vdata in HDF 
*                               format.
*       sdsRead                 This routine is used to read the scientific data
*                               set in HDF format.      
*
***********************************************************************************/
int readScan_2A_25(IO_HANDLE *sGranuleHandle, L2A_25_SWATHDATA *sSwathData)
{ 
  SDS_LINK *tempSDS = sGranuleHandle->cfgSTR->vGroup->subVgroup->sds;
  VDATA_LINK *tempVdata = sGranuleHandle->cfgSTR->vGroup->subVgroup->vdata;
  uint8 *databuf, *ptr;
  int	i, j;
  int scaleFactor;

  static int dimChangeWarnFlag = 1;
  static int dimChangeWarnFlag2 = 1;

  int16 old_dim_rangeBinNum[49][6];

  /* Allocate memory for unpacking the vdata */
  databuf = allocateVdataMemory(&tempVdata->vdataFields);
  ptr = databuf;
  /* Unpack the content of databuf into scanTime */
  vdataRead(&tempVdata, ptr);
  memcpy(&sSwathData->scanTime, ptr, sizeof(float64));
  free(databuf);

  sdsRead(&tempSDS, &scaleFactor, &sSwathData->geolocation);


  /* Allocate memory for unpacking the vdata */
  databuf = allocateVdataMemory(&tempVdata->vdataFields);
  ptr = databuf;
  /* Unpack the content of databuf into scanStatus block */
  vdataRead(&tempVdata, ptr);
  memcpy(&sSwathData->scanStatus.missing, ptr, 
		sizeof(sSwathData->scanStatus.missing));
  ptr += sizeof(sSwathData->scanStatus.missing);
  memcpy(&sSwathData->scanStatus.validity, ptr, 
		sizeof(sSwathData->scanStatus.validity));
  ptr += sizeof(sSwathData->scanStatus.validity);
  memcpy(&sSwathData->scanStatus.qac, ptr, 
		sizeof(sSwathData->scanStatus.qac));
  ptr += sizeof(sSwathData->scanStatus.qac);
  memcpy(&sSwathData->scanStatus.geoQuality, ptr, 
		sizeof(sSwathData->scanStatus.geoQuality));
  ptr += sizeof(sSwathData->scanStatus.geoQuality);
  memcpy(&sSwathData->scanStatus.dataQuality, ptr, 
		sizeof(sSwathData->scanStatus.dataQuality));
  ptr += sizeof(sSwathData->scanStatus.dataQuality);
  memcpy(&sSwathData->scanStatus.scOrient, ptr, 
		sizeof(sSwathData->scanStatus.scOrient));
  ptr += sizeof(sSwathData->scanStatus.scOrient);
  memcpy(&sSwathData->scanStatus.acsMode, ptr, 
		sizeof(sSwathData->scanStatus.acsMode));
  ptr += sizeof(sSwathData->scanStatus.acsMode);
  memcpy(&sSwathData->scanStatus.yawUpdateS, ptr, 
		sizeof(sSwathData->scanStatus.yawUpdateS));
  ptr += sizeof(sSwathData->scanStatus.yawUpdateS);
  memcpy(&sSwathData->scanStatus.prMode, ptr, 
		sizeof(sSwathData->scanStatus.prMode));
  ptr += sizeof(sSwathData->scanStatus.prMode);
  memcpy(&sSwathData->scanStatus.prStatus1, ptr,
                sizeof(sSwathData->scanStatus.prStatus1));
  ptr += sizeof(sSwathData->scanStatus.prStatus1);
  memcpy(&sSwathData->scanStatus.prStatus2, ptr,
                sizeof(sSwathData->scanStatus.prStatus2));
  ptr += sizeof(sSwathData->scanStatus.prStatus2);
  memcpy(&sSwathData->scanStatus.fractOrbitN, ptr, 
		sizeof(sSwathData->scanStatus.fractOrbitN));
  free(databuf);


  /* Allocate memory for unpacking the vdata */
  databuf = allocateVdataMemory(&tempVdata->vdataFields);
  ptr = databuf;
  /* Unpack the content of databuf into navigation block */
  vdataRead(&tempVdata, ptr);
  memcpy(&sSwathData->navigate.scPosX, ptr, sizeof(sSwathData->navigate.scPosX));
  ptr += sizeof(sSwathData->navigate.scPosX);
  memcpy(&sSwathData->navigate.scPosY, ptr, sizeof(sSwathData->navigate.scPosY));
  ptr += sizeof(sSwathData->navigate.scPosY);
  memcpy(&sSwathData->navigate.scPosZ, ptr, sizeof(sSwathData->navigate.scPosZ));
  ptr += sizeof(sSwathData->navigate.scPosZ);
  memcpy(&sSwathData->navigate.scVelX, ptr, sizeof(sSwathData->navigate.scVelX));
  ptr += sizeof(sSwathData->navigate.scVelX);
  memcpy(&sSwathData->navigate.scVelY, ptr, sizeof(sSwathData->navigate.scVelY));
  ptr += sizeof(sSwathData->navigate.scVelY);
  memcpy(&sSwathData->navigate.scVelZ, ptr, sizeof(sSwathData->navigate.scVelZ));
  ptr += sizeof(sSwathData->navigate.scVelZ);
  memcpy(&sSwathData->navigate.scLat, ptr, sizeof(sSwathData->navigate.scLat));
  ptr += sizeof(sSwathData->navigate.scLat);
  memcpy(&sSwathData->navigate.scLon, ptr, sizeof(sSwathData->navigate.scLon));
  ptr += sizeof(sSwathData->navigate.scLon);
  memcpy(&sSwathData->navigate.scAlt, ptr, sizeof(sSwathData->navigate.scAlt));
  ptr += sizeof(sSwathData->navigate.scAlt);
  memcpy(&sSwathData->navigate.scAttRoll, ptr, sizeof(sSwathData->navigate.scAttRoll));
  ptr += sizeof(sSwathData->navigate.scAttRoll);
  memcpy(&sSwathData->navigate.scAttPitch, ptr, 
	sizeof(sSwathData->navigate.scAttPitch));
  ptr += sizeof(sSwathData->navigate.scAttPitch);
  memcpy(&sSwathData->navigate.scAttYaw, ptr, sizeof(sSwathData->navigate.scAttYaw));
  ptr += sizeof(sSwathData->navigate.scAttYaw);
  memcpy(&sSwathData->navigate.att1, ptr, sizeof(sSwathData->navigate.att1));
  ptr += sizeof(sSwathData->navigate.att1);
  memcpy(&sSwathData->navigate.att2, ptr, sizeof(sSwathData->navigate.att2));
  ptr += sizeof(sSwathData->navigate.att2);
  memcpy(&sSwathData->navigate.att3, ptr, sizeof(sSwathData->navigate.att3));
  ptr += sizeof(sSwathData->navigate.att3);
  memcpy(&sSwathData->navigate.att4, ptr, sizeof(sSwathData->navigate.att4));
  ptr += sizeof(sSwathData->navigate.att4);
  memcpy(&sSwathData->navigate.att5, ptr, sizeof(sSwathData->navigate.att5));
  ptr += sizeof(sSwathData->navigate.att5);
  memcpy(&sSwathData->navigate.att6, ptr, sizeof(sSwathData->navigate.att6));
  ptr += sizeof(sSwathData->navigate.att6);
  memcpy(&sSwathData->navigate.att7, ptr, sizeof(sSwathData->navigate.att7));
  ptr += sizeof(sSwathData->navigate.att7);
  memcpy(&sSwathData->navigate.att8, ptr, sizeof(sSwathData->navigate.att8));
  ptr += sizeof(sSwathData->navigate.att8);
  memcpy(&sSwathData->navigate.att9, ptr, sizeof(sSwathData->navigate.att9));
  ptr += sizeof(sSwathData->navigate.att9);
  memcpy(&sSwathData->navigate.greenHourAng, ptr, 
	sizeof(sSwathData->navigate.greenHourAng));
  free(databuf);

  sdsRead(&tempSDS, &scaleFactor, &sSwathData->scLocalZenith);

  /* Read SDS from HDF format */
  /*
   * Scaling calculation.  For reading,
   *    Divided the data by L2A25_RAIN.
   */
  sdsRead(&tempSDS, &scaleFactor, &sSwathData->rain_scale);
  for (i = 0; i < 49; ++i)
    for (j = 0; j < 80; ++j)
        {
        if (sSwathData->rain_scale[i][j] != TK_MISS_I2)
            sSwathData->rain[i][j] = sSwathData->rain_scale[i][j] * 1.0 / scaleFactor;
        else
           sSwathData->rain[i][j] = TK_DEFAULT;
        }
  sdsRead(&tempSDS, &scaleFactor, &sSwathData->reliab);

  /*
   * Scaling calculation.  For reading,
   *    Divided the data by L2A25_CORRECTZFACTOR.
   */
  sdsRead(&tempSDS, &scaleFactor, &sSwathData->correctZFactor_scale);
  for (i = 0; i < 49; ++i)
    for (j = 0; j < 80; ++j)
        {
        if (sSwathData->correctZFactor_scale[i][j] != TK_MISS_I2)
           sSwathData->correctZFactor[i][j] = 
		(sSwathData->correctZFactor_scale[i][j] * 1.0 / scaleFactor);
        else
           sSwathData->correctZFactor[i][j] = TK_DEFAULT;
        }

  if (versionCompare("5.8",sGranuleHandle->TKversion) >= 0)
  {
      sdsRead(&tempSDS, &scaleFactor, &sSwathData->attenParmNode);
  }

  /*
   * Scaling calculation.  For reading,
   *    Divided the data by L2A25_ATTENPARMALPHA.
   */
  sdsRead(&tempSDS, &scaleFactor, &sSwathData->attenParmAlpha_scale);
  for (i = 0; i < 49; ++i)
    for (j = 0; j < 5; ++j)
        {
        if (sSwathData->attenParmAlpha_scale[i][j] != TK_MISS_I2)
           sSwathData->attenParmAlpha[i][j] = 
		sSwathData->attenParmAlpha_scale[i][j] * 1.0 / scaleFactor;
        else
           sSwathData->attenParmAlpha[i][j] = TK_DEFAULT;
        } 
/*
   * Scaling calculation.  For reading,
   *    Divided the data by L2A25_ATTENPARMBETA.
   */
  sdsRead(&tempSDS, &scaleFactor, &sSwathData->attenParmBeta_scale);
  for (i = 0; i < 49; ++i)
        {
        if (sSwathData->attenParmBeta_scale[i] != TK_MISS_I2)
           sSwathData->attenParmBeta[i] = 
		sSwathData->attenParmBeta_scale[i] * 1.0 / scaleFactor;
        else
           sSwathData->attenParmBeta[i] = TK_DEFAULT;
        }

  if (versionCompare("5.8",sGranuleHandle->TKversion) >= 0)
  {
      sdsRead(&tempSDS, &scaleFactor, &sSwathData->ZRParmNode);
  }

  sdsRead(&tempSDS, &scaleFactor, &sSwathData->parmNode);
  sdsRead(&tempSDS, &scaleFactor, &sSwathData->precipWaterParmA);
  sdsRead(&tempSDS, &scaleFactor, &sSwathData->precipWaterParmB);

  /*
   * Scaling calculation.  For reading,
   *    Divided the data by L2A25_ZRPARMA.
   */
  sdsRead(&tempSDS, &scaleFactor, &sSwathData->ZRParmA_scale);
  for (i = 0; i < 49; ++i)
    for (j = 0; j < 5; ++j)
        {
        if (sSwathData->ZRParmA_scale[i][j] != TK_MISS_I2)
           sSwathData->ZRParmA[i][j] = 
		sSwathData->ZRParmA_scale[i][j] * 1.0 / scaleFactor;
        else
           sSwathData->ZRParmA[i][j] = TK_DEFAULT;
        }
  /*
   * Scaling calculation.  For reading,
   *    Divided the data by L2A25_ZRPARMB.
   */
  sdsRead(&tempSDS, &scaleFactor, &sSwathData->ZRParmB_scale);
  for (i = 0; i < 49; ++i)
    for (j = 0; j < 5; ++j)
        {
        if (sSwathData->ZRParmB_scale[i][j] != TK_MISS_I2)
           sSwathData->ZRParmB[i][j] = 
		sSwathData->ZRParmB_scale[i][j] * 1.0 / scaleFactor;
        else
           sSwathData->ZRParmB[i][j] = TK_DEFAULT;
        }
  sdsRead(&tempSDS, &scaleFactor, &sSwathData->zmmax);
  sdsRead(&tempSDS, &scaleFactor, &sSwathData->rainFlag);

  if (versionCompare("5.8",sGranuleHandle->TKversion) >= 0)
  {
      if (dimChangeWarnFlag==1)
          TKreportWarningParm(W_TK_SDS_DIM_CHANGE, tempSDS->name);
      dimChangeWarnFlag=0;
      sdsRead(&tempSDS, &scaleFactor, old_dim_rangeBinNum);
      tempSDS=tempSDS->next;
  }
  else
  {
      sdsRead(&tempSDS, &scaleFactor, &sSwathData->rangeBinNum);
  }

  if (versionCompare("5.8",sGranuleHandle->TKversion) >= 0)
  {

  for (i = 0; i < 49; ++i)
    for (j = 0; j < 6; ++j)
        {
           sSwathData->rangeBinNum[i][j] = old_dim_rangeBinNum[i][j];
        }

  }

  sdsRead(&tempSDS, &scaleFactor, &sSwathData->rainAve_scale);
  for (i = 0; i < 49; ++i)
    for (j = 0; j < 2; ++j)
        {
        if (sSwathData->rainAve_scale[i][j] != TK_MISS_I2)
           sSwathData->rainAve[i][j]
                = sSwathData->rainAve_scale[i][j] * 1.0 / scaleFactor;
        else
           sSwathData->rainAve[i][j] = TK_DEFAULT;
        }

  sdsRead(&tempSDS, &scaleFactor, &sSwathData->precipWaterSum_scale);
  for (i = 0; i < 49; ++i)
        {
        if (sSwathData->precipWaterSum_scale[i] != TK_MISS_I2)
           sSwathData->precipWaterSum[i]
                = sSwathData->precipWaterSum_scale[i] * 1.0 / scaleFactor;
        else
           sSwathData->precipWaterSum[i] = TK_DEFAULT;
        }

  /*
   * Scaling calculation.  For reading,
   *    Divided the data by L2A25_WEIGHTW.
   */

  if (versionCompare("5.8",sGranuleHandle->TKversion) >= 0)
  {
  sdsRead(&tempSDS, &scaleFactor, &sSwathData->weightW_scale);
  for (i = 0; i < 49; ++i)
        {
        if (sSwathData->weightW_scale[i] != TK_MISS_I2)
           sSwathData->weightW[i] = sSwathData->weightW_scale[i] * 1.0 / scaleFactor;
        else
           sSwathData->weightW[i] = TK_DEFAULT;
        }
  }

  sdsRead(&tempSDS, &scaleFactor, &sSwathData->epsilon_0);

  sdsRead(&tempSDS, &scaleFactor, &sSwathData->method);
  sdsRead(&tempSDS, &scaleFactor, &sSwathData->epsilon);
  sdsRead(&tempSDS, &scaleFactor, &sSwathData->zeta);
  sdsRead(&tempSDS, &scaleFactor, &sSwathData->zeta_mn);
  sdsRead(&tempSDS, &scaleFactor, &sSwathData->zeta_sd);

  if (versionCompare("6.33",sGranuleHandle->TKversion) >= 0)
  {
      sdsRead(&tempSDS, &scaleFactor, &sSwathData->xi);
  }
  sdsRead(&tempSDS, &scaleFactor, &sSwathData->sigmaZero);
  sdsRead(&tempSDS, &scaleFactor, &sSwathData->freezH);

  if (versionCompare("5.8",sGranuleHandle->TKversion) >= 0)
  {
      sdsRead(&tempSDS, &scaleFactor, &sSwathData->thickThPIZ);
  }


  sdsRead(&tempSDS, &scaleFactor, &sSwathData->nubfCorrectFactor);
  sdsRead(&tempSDS, &scaleFactor, &sSwathData->qualityFlag);
  sdsRead(&tempSDS, &scaleFactor, &sSwathData->nearSurfRain);
  sdsRead(&tempSDS, &scaleFactor, &sSwathData->nearSurfZ);

  if (versionCompare("5.8",sGranuleHandle->TKversion) >= 0)
  {
      sdsRead(&tempSDS, &scaleFactor, &sSwathData->pia2a25);
  }

  sdsRead(&tempSDS, &scaleFactor, &sSwathData->e_SurfRain);

  sdsRead(&tempSDS, &scaleFactor, &sSwathData->pia);

  sdsRead(&tempSDS, &scaleFactor, &sSwathData->errorRain);
  sdsRead(&tempSDS, &scaleFactor, &sSwathData->errorZ);
  sdsRead(&tempSDS, &scaleFactor, &sSwathData->spare);

  sdsRead(&tempSDS, &scaleFactor, &sSwathData->rainType);

  return(TK_SUCCESS);
}


/* 2A25 Nadir readScan (2N25) Begin */

int readScan_2N_25(IO_HANDLE *sGranuleHandle, L2N_25_SWATHDATA *sSwathData)
{ 
  SDS_LINK *tempSDS = sGranuleHandle->cfgSTR->vGroup->subVgroup->sds;
  VDATA_LINK *tempVdata = sGranuleHandle->cfgSTR->vGroup->subVgroup->vdata;
  uint8 *databuf, *ptr;
  int	i, j;
  int scaleFactor;

  static int dimChangeWarnFlag = 1;
  static int dimChangeWarnFlag2 = 1;

  int16 old_dim_rangeBinNum[1][6];

  /* Allocate memory for unpacking the vdata */
  databuf = allocateVdataMemory(&tempVdata->vdataFields);
  ptr = databuf;
  /* Unpack the content of databuf into scanTime */
  vdataRead(&tempVdata, ptr);
  memcpy(&sSwathData->scanTime, ptr, sizeof(float64));
  free(databuf);

  sdsRead(&tempSDS, &scaleFactor, &sSwathData->geolocation);


  /* Allocate memory for unpacking the vdata */
  databuf = allocateVdataMemory(&tempVdata->vdataFields);
  ptr = databuf;
  /* Unpack the content of databuf into scanStatus block */
  vdataRead(&tempVdata, ptr);
  memcpy(&sSwathData->scanStatus.missing, ptr, 
		sizeof(sSwathData->scanStatus.missing));
  ptr += sizeof(sSwathData->scanStatus.missing);
  memcpy(&sSwathData->scanStatus.validity, ptr, 
		sizeof(sSwathData->scanStatus.validity));
  ptr += sizeof(sSwathData->scanStatus.validity);
  memcpy(&sSwathData->scanStatus.qac, ptr, 
		sizeof(sSwathData->scanStatus.qac));
  ptr += sizeof(sSwathData->scanStatus.qac);
  memcpy(&sSwathData->scanStatus.geoQuality, ptr, 
		sizeof(sSwathData->scanStatus.geoQuality));
  ptr += sizeof(sSwathData->scanStatus.geoQuality);
  memcpy(&sSwathData->scanStatus.dataQuality, ptr, 
		sizeof(sSwathData->scanStatus.dataQuality));
  ptr += sizeof(sSwathData->scanStatus.dataQuality);
  memcpy(&sSwathData->scanStatus.scOrient, ptr, 
		sizeof(sSwathData->scanStatus.scOrient));
  ptr += sizeof(sSwathData->scanStatus.scOrient);
  memcpy(&sSwathData->scanStatus.acsMode, ptr, 
		sizeof(sSwathData->scanStatus.acsMode));
  ptr += sizeof(sSwathData->scanStatus.acsMode);
  memcpy(&sSwathData->scanStatus.yawUpdateS, ptr, 
		sizeof(sSwathData->scanStatus.yawUpdateS));
  ptr += sizeof(sSwathData->scanStatus.yawUpdateS);
  memcpy(&sSwathData->scanStatus.prMode, ptr, 
		sizeof(sSwathData->scanStatus.prMode));
  ptr += sizeof(sSwathData->scanStatus.prMode);
  memcpy(&sSwathData->scanStatus.prStatus1, ptr,
                sizeof(sSwathData->scanStatus.prStatus1));
  ptr += sizeof(sSwathData->scanStatus.prStatus1);
  memcpy(&sSwathData->scanStatus.prStatus2, ptr,
                sizeof(sSwathData->scanStatus.prStatus2));
  ptr += sizeof(sSwathData->scanStatus.prStatus2);
  memcpy(&sSwathData->scanStatus.fractOrbitN, ptr, 
		sizeof(sSwathData->scanStatus.fractOrbitN));
  free(databuf);


  /* Allocate memory for unpacking the vdata */
  databuf = allocateVdataMemory(&tempVdata->vdataFields);
  ptr = databuf;
  /* Unpack the content of databuf into navigation block */
  vdataRead(&tempVdata, ptr);
  memcpy(&sSwathData->navigate.scPosX, ptr, sizeof(sSwathData->navigate.scPosX));
  ptr += sizeof(sSwathData->navigate.scPosX);
  memcpy(&sSwathData->navigate.scPosY, ptr, sizeof(sSwathData->navigate.scPosY));
  ptr += sizeof(sSwathData->navigate.scPosY);
  memcpy(&sSwathData->navigate.scPosZ, ptr, sizeof(sSwathData->navigate.scPosZ));
  ptr += sizeof(sSwathData->navigate.scPosZ);
  memcpy(&sSwathData->navigate.scVelX, ptr, sizeof(sSwathData->navigate.scVelX));
  ptr += sizeof(sSwathData->navigate.scVelX);
  memcpy(&sSwathData->navigate.scVelY, ptr, sizeof(sSwathData->navigate.scVelY));
  ptr += sizeof(sSwathData->navigate.scVelY);
  memcpy(&sSwathData->navigate.scVelZ, ptr, sizeof(sSwathData->navigate.scVelZ));
  ptr += sizeof(sSwathData->navigate.scVelZ);
  memcpy(&sSwathData->navigate.scLat, ptr, sizeof(sSwathData->navigate.scLat));
  ptr += sizeof(sSwathData->navigate.scLat);
  memcpy(&sSwathData->navigate.scLon, ptr, sizeof(sSwathData->navigate.scLon));
  ptr += sizeof(sSwathData->navigate.scLon);
  memcpy(&sSwathData->navigate.scAlt, ptr, sizeof(sSwathData->navigate.scAlt));
  ptr += sizeof(sSwathData->navigate.scAlt);
  memcpy(&sSwathData->navigate.scAttRoll, ptr, sizeof(sSwathData->navigate.scAttRoll));
  ptr += sizeof(sSwathData->navigate.scAttRoll);
  memcpy(&sSwathData->navigate.scAttPitch, ptr, 
	sizeof(sSwathData->navigate.scAttPitch));
  ptr += sizeof(sSwathData->navigate.scAttPitch);
  memcpy(&sSwathData->navigate.scAttYaw, ptr, sizeof(sSwathData->navigate.scAttYaw));
  ptr += sizeof(sSwathData->navigate.scAttYaw);
  memcpy(&sSwathData->navigate.att1, ptr, sizeof(sSwathData->navigate.att1));
  ptr += sizeof(sSwathData->navigate.att1);
  memcpy(&sSwathData->navigate.att2, ptr, sizeof(sSwathData->navigate.att2));
  ptr += sizeof(sSwathData->navigate.att2);
  memcpy(&sSwathData->navigate.att3, ptr, sizeof(sSwathData->navigate.att3));
  ptr += sizeof(sSwathData->navigate.att3);
  memcpy(&sSwathData->navigate.att4, ptr, sizeof(sSwathData->navigate.att4));
  ptr += sizeof(sSwathData->navigate.att4);
  memcpy(&sSwathData->navigate.att5, ptr, sizeof(sSwathData->navigate.att5));
  ptr += sizeof(sSwathData->navigate.att5);
  memcpy(&sSwathData->navigate.att6, ptr, sizeof(sSwathData->navigate.att6));
  ptr += sizeof(sSwathData->navigate.att6);
  memcpy(&sSwathData->navigate.att7, ptr, sizeof(sSwathData->navigate.att7));
  ptr += sizeof(sSwathData->navigate.att7);
  memcpy(&sSwathData->navigate.att8, ptr, sizeof(sSwathData->navigate.att8));
  ptr += sizeof(sSwathData->navigate.att8);
  memcpy(&sSwathData->navigate.att9, ptr, sizeof(sSwathData->navigate.att9));
  ptr += sizeof(sSwathData->navigate.att9);
  memcpy(&sSwathData->navigate.greenHourAng, ptr, 
	sizeof(sSwathData->navigate.greenHourAng));
  free(databuf);

  sdsRead(&tempSDS, &scaleFactor, &sSwathData->scLocalZenith);

  /* Read SDS from HDF format */
  /*
   * Scaling calculation.  For reading,
   *    Divided the data by L2A25_RAIN.
   */
  sdsRead(&tempSDS, &scaleFactor, &sSwathData->rain_scale);
  for (i = 0; i < 1; ++i)
    for (j = 0; j < 80; ++j)
        {
        if (sSwathData->rain_scale[i][j] != TK_MISS_I2)
            sSwathData->rain[i][j] = sSwathData->rain_scale[i][j] * 1.0 / scaleFactor;
        else
           sSwathData->rain[i][j] = TK_DEFAULT;
        }
  sdsRead(&tempSDS, &scaleFactor, &sSwathData->reliab);

  /*
   * Scaling calculation.  For reading,
   *    Divided the data by L2A25_CORRECTZFACTOR.
   */
  sdsRead(&tempSDS, &scaleFactor, &sSwathData->correctZFactor_scale);
  for (i = 0; i < 1; ++i)
    for (j = 0; j < 80; ++j)
        {
        if (sSwathData->correctZFactor_scale[i][j] != TK_MISS_I2)
           sSwathData->correctZFactor[i][j] = 
		(sSwathData->correctZFactor_scale[i][j] * 1.0 / scaleFactor);
        else
           sSwathData->correctZFactor[i][j] = TK_DEFAULT;
        }

  if (versionCompare("5.8",sGranuleHandle->TKversion) >= 0)
  {
      sdsRead(&tempSDS, &scaleFactor, &sSwathData->attenParmNode);
  }

  /*
   * Scaling calculation.  For reading,
   *    Divided the data by L2A25_ATTENPARMALPHA.
   */
  sdsRead(&tempSDS, &scaleFactor, &sSwathData->attenParmAlpha_scale);
  for (i = 0; i < 1; ++i)
    for (j = 0; j < 5; ++j)
        {
        if (sSwathData->attenParmAlpha_scale[i][j] != TK_MISS_I2)
           sSwathData->attenParmAlpha[i][j] = 
		sSwathData->attenParmAlpha_scale[i][j] * 1.0 / scaleFactor;
        else
           sSwathData->attenParmAlpha[i][j] = TK_DEFAULT;
        } 
/*
   * Scaling calculation.  For reading,
   *    Divided the data by L2A25_ATTENPARMBETA.
   */
  sdsRead(&tempSDS, &scaleFactor, &sSwathData->attenParmBeta_scale);
  for (i = 0; i < 1; ++i)
        {
        if (sSwathData->attenParmBeta_scale[i] != TK_MISS_I2)
           sSwathData->attenParmBeta[i] = 
		sSwathData->attenParmBeta_scale[i] * 1.0 / scaleFactor;
        else
           sSwathData->attenParmBeta[i] = TK_DEFAULT;
        }

  if (versionCompare("5.8",sGranuleHandle->TKversion) >= 0)
  {
      sdsRead(&tempSDS, &scaleFactor, &sSwathData->ZRParmNode);
  }

  sdsRead(&tempSDS, &scaleFactor, &sSwathData->parmNode);
  sdsRead(&tempSDS, &scaleFactor, &sSwathData->precipWaterParmA);
  sdsRead(&tempSDS, &scaleFactor, &sSwathData->precipWaterParmB);

  /*
   * Scaling calculation.  For reading,
   *    Divided the data by L2A25_ZRPARMA.
   */
  sdsRead(&tempSDS, &scaleFactor, &sSwathData->ZRParmA_scale);
  for (i = 0; i < 1; ++i)
    for (j = 0; j < 5; ++j)
        {
        if (sSwathData->ZRParmA_scale[i][j] != TK_MISS_I2)
           sSwathData->ZRParmA[i][j] = 
		sSwathData->ZRParmA_scale[i][j] * 1.0 / scaleFactor;
        else
           sSwathData->ZRParmA[i][j] = TK_DEFAULT;
        }
  /*
   * Scaling calculation.  For reading,
   *    Divided the data by L2A25_ZRPARMB.
   */
  sdsRead(&tempSDS, &scaleFactor, &sSwathData->ZRParmB_scale);
  for (i = 0; i < 1; ++i)
    for (j = 0; j < 5; ++j)
        {
        if (sSwathData->ZRParmB_scale[i][j] != TK_MISS_I2)
           sSwathData->ZRParmB[i][j] = 
		sSwathData->ZRParmB_scale[i][j] * 1.0 / scaleFactor;
        else
           sSwathData->ZRParmB[i][j] = TK_DEFAULT;
        }
  sdsRead(&tempSDS, &scaleFactor, &sSwathData->zmmax);
  sdsRead(&tempSDS, &scaleFactor, &sSwathData->rainFlag);

  if (versionCompare("5.8",sGranuleHandle->TKversion) >= 0)
  {
      if (dimChangeWarnFlag==1)
          TKreportWarningParm(W_TK_SDS_DIM_CHANGE, tempSDS->name);
      dimChangeWarnFlag=0;
      sdsRead(&tempSDS, &scaleFactor, old_dim_rangeBinNum);
      tempSDS=tempSDS->next;
  }
  else
  {
      sdsRead(&tempSDS, &scaleFactor, &sSwathData->rangeBinNum);
  }

  if (versionCompare("5.8",sGranuleHandle->TKversion) >= 0)
  {

  for (i = 0; i < 1; ++i)
    for (j = 0; j < 6; ++j)
        {
           sSwathData->rangeBinNum[i][j] = old_dim_rangeBinNum[i][j];
        }

  }

  sdsRead(&tempSDS, &scaleFactor, &sSwathData->rainAve_scale);
  for (i = 0; i < 1; ++i)
    for (j = 0; j < 2; ++j)
        {
        if (sSwathData->rainAve_scale[i][j] != TK_MISS_I2)
           sSwathData->rainAve[i][j]
                = sSwathData->rainAve_scale[i][j] * 1.0 / scaleFactor;
        else
           sSwathData->rainAve[i][j] = TK_DEFAULT;
        }

  sdsRead(&tempSDS, &scaleFactor, &sSwathData->precipWaterSum_scale);
  for (i = 0; i < 1; ++i)
        {
        if (sSwathData->precipWaterSum_scale[i] != TK_MISS_I2)
           sSwathData->precipWaterSum[i]
                = sSwathData->precipWaterSum_scale[i] * 1.0 / scaleFactor;
        else
           sSwathData->precipWaterSum[i] = TK_DEFAULT;
        }

  /*
   * Scaling calculation.  For reading,
   *    Divided the data by L2A25_WEIGHTW.
   */

  if (versionCompare("5.8",sGranuleHandle->TKversion) >= 0)
  {
  sdsRead(&tempSDS, &scaleFactor, &sSwathData->weightW_scale);
  for (i = 0; i < 1; ++i)
        {
        if (sSwathData->weightW_scale[i] != TK_MISS_I2)
           sSwathData->weightW[i] = sSwathData->weightW_scale[i] * 1.0 / scaleFactor;
        else
           sSwathData->weightW[i] = TK_DEFAULT;
        }
  }

  sdsRead(&tempSDS, &scaleFactor, &sSwathData->epsilon_0);

  sdsRead(&tempSDS, &scaleFactor, &sSwathData->method);
  sdsRead(&tempSDS, &scaleFactor, &sSwathData->epsilon);
  sdsRead(&tempSDS, &scaleFactor, &sSwathData->zeta);
  sdsRead(&tempSDS, &scaleFactor, &sSwathData->zeta_mn);
  sdsRead(&tempSDS, &scaleFactor, &sSwathData->zeta_sd);

  if (versionCompare("6.33",sGranuleHandle->TKversion) >= 0)
  {
      sdsRead(&tempSDS, &scaleFactor, &sSwathData->xi);
  }
  sdsRead(&tempSDS, &scaleFactor, &sSwathData->sigmaZero);
  sdsRead(&tempSDS, &scaleFactor, &sSwathData->freezH);

  if (versionCompare("5.8",sGranuleHandle->TKversion) >= 0)
  {
      sdsRead(&tempSDS, &scaleFactor, &sSwathData->thickThPIZ);
  }


  sdsRead(&tempSDS, &scaleFactor, &sSwathData->nubfCorrectFactor);
  sdsRead(&tempSDS, &scaleFactor, &sSwathData->qualityFlag);
  sdsRead(&tempSDS, &scaleFactor, &sSwathData->nearSurfRain);
  sdsRead(&tempSDS, &scaleFactor, &sSwathData->nearSurfZ);

  if (versionCompare("5.8",sGranuleHandle->TKversion) >= 0)
  {
      sdsRead(&tempSDS, &scaleFactor, &sSwathData->pia2a25);
  }

  sdsRead(&tempSDS, &scaleFactor, &sSwathData->e_SurfRain);

  sdsRead(&tempSDS, &scaleFactor, &sSwathData->pia);

  sdsRead(&tempSDS, &scaleFactor, &sSwathData->errorRain);
  sdsRead(&tempSDS, &scaleFactor, &sSwathData->errorZ);
  sdsRead(&tempSDS, &scaleFactor, &sSwathData->spare);

  sdsRead(&tempSDS, &scaleFactor, &sSwathData->rainType);

  return(TK_SUCCESS);
}


/* 2A25 Nadir readScan (2N25) End */


/***********************************************************************************
*
*  FUNCTION STATEMENT:
*       writeScan_2B_31(IO_HANDLE *sGranuleHandle, L2B_31_SWATHDATA *sSwathData) 
*
*  DESCRIPTION:
*       This function is used to write the L2B-31 information in HDF format.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       sGranuleHandle          A structure containing information about the file.
*                               sGranuleHandle is returned by TKopen().
*
*  OUTPUT PARAMETERS:
*       sSwathData              A structure containing the complete scanline data.
*                               This structure must be declared to correspond to 
*                               the data file being read.  The structures of both
*                               sScanHeader and sScanData are manipulated by
*                               TKreadScan() and other toolkit routines.
*
*
*  CALLED BY:                   TKwriteScan()
*
*  ROUTINES CALLED: 
*	allocateVdataMemory(), vdataWrite(), sdsWrite() 
*
***********************************************************************************/

int writeScan_2B_31(IO_HANDLE *sGranuleHandle, L2B_31_SWATHDATA *sSwathData)
{
  SDS_LINK *tempSDS = sGranuleHandle->cfgSTR->vGroup->subVgroup->sds;
  VDATA_LINK *tempVdata = sGranuleHandle->cfgSTR->vGroup->subVgroup->vdata;
  uint8 *databuf, *ptr;
  int   i, j;

  /* Allocate memory for packing the vdata */
  databuf = allocateVdataMemory(&tempVdata->vdataFields);
  ptr = databuf;
  memcpy(ptr, &sSwathData->scanTime, sizeof(sSwathData->scanTime));
  /* Writing packed vdata, scanTime, to an HDF file */
  vdataWrite(&tempVdata, databuf); 
  free(databuf);

  sdsWrite(&tempSDS, &sSwathData->geolocation);


  /* Allocate memory for packing the vdata */
  databuf = allocateVdataMemory(&tempVdata->vdataFields);
  ptr = databuf;
  memcpy(ptr, &sSwathData->scanStatus.missing,
                sizeof(sSwathData->scanStatus.missing));
  ptr += sizeof(sSwathData->scanStatus.missing);
  memcpy(ptr, &sSwathData->scanStatus.validity, 
                sizeof(sSwathData->scanStatus.validity));
  ptr += sizeof(sSwathData->scanStatus.validity);
  memcpy(ptr, &sSwathData->scanStatus.qac, 
		sizeof(sSwathData->scanStatus.qac));
  ptr += sizeof(sSwathData->scanStatus.qac);
  memcpy(ptr, &sSwathData->scanStatus.geoQuality, 
                sizeof(sSwathData->scanStatus.geoQuality));
  ptr += sizeof(sSwathData->scanStatus.geoQuality);
  memcpy(ptr, &sSwathData->scanStatus.dataQuality, 
                sizeof(sSwathData->scanStatus.dataQuality));
  ptr += sizeof(sSwathData->scanStatus.dataQuality);
  memcpy(ptr, &sSwathData->scanStatus.scOrient, 
                sizeof(sSwathData->scanStatus.scOrient));
  ptr += sizeof(sSwathData->scanStatus.scOrient);
  memcpy(ptr, &sSwathData->scanStatus.acsMode, 
                sizeof(sSwathData->scanStatus.acsMode));
  ptr += sizeof(sSwathData->scanStatus.acsMode);
  memcpy(ptr, &sSwathData->scanStatus.yawUpdateS, 
                sizeof(sSwathData->scanStatus.yawUpdateS));
  ptr += sizeof(sSwathData->scanStatus.yawUpdateS);
  memcpy(ptr, &sSwathData->scanStatus.prMode, 
                sizeof(sSwathData->scanStatus.prMode));
  ptr += sizeof(sSwathData->scanStatus.prMode);
  memcpy(ptr, &sSwathData->scanStatus.prStatus1,
                sizeof(sSwathData->scanStatus.prStatus1));
  ptr += sizeof(sSwathData->scanStatus.prStatus1);
  memcpy(ptr, &sSwathData->scanStatus.prStatus2,
                sizeof(sSwathData->scanStatus.prStatus2));
  ptr += sizeof(sSwathData->scanStatus.prStatus2);
  memcpy(ptr, &sSwathData->scanStatus.fractOrbitN, 
                sizeof(sSwathData->scanStatus.fractOrbitN));
  /* Writing packed vdata, scanStatus block, to an HDF file */
  vdataWrite(&tempVdata, databuf);
  free(databuf);


  /* Allocate memory for packing the vdata */
  databuf = allocateVdataMemory(&tempVdata->vdataFields);
  ptr = databuf;
  memcpy(ptr, &sSwathData->navigate.scPosX, sizeof(sSwathData->navigate.scPosX));
  ptr += sizeof(sSwathData->navigate.scPosX);
  memcpy(ptr, &sSwathData->navigate.scPosY, sizeof(sSwathData->navigate.scPosY));
  ptr += sizeof(sSwathData->navigate.scPosY);
  memcpy(ptr, &sSwathData->navigate.scPosZ, sizeof(sSwathData->navigate.scPosZ));
  ptr += sizeof(sSwathData->navigate.scPosZ);
  memcpy(ptr, &sSwathData->navigate.scVelX, sizeof(sSwathData->navigate.scVelX));
  ptr += sizeof(sSwathData->navigate.scVelX);
  memcpy(ptr, &sSwathData->navigate.scVelY, sizeof(sSwathData->navigate.scVelY));
  ptr += sizeof(sSwathData->navigate.scVelY);
  memcpy(ptr, &sSwathData->navigate.scVelZ, sizeof(sSwathData->navigate.scVelZ));
  ptr += sizeof(sSwathData->navigate.scVelZ);
  memcpy(ptr, &sSwathData->navigate.scLat, sizeof(sSwathData->navigate.scLat));
  ptr += sizeof(sSwathData->navigate.scLat);
  memcpy(ptr, &sSwathData->navigate.scLon, sizeof(sSwathData->navigate.scLon));
  ptr += sizeof(sSwathData->navigate.scLon);
  memcpy(ptr, &sSwathData->navigate.scAlt, sizeof(sSwathData->navigate.scAlt));
  ptr += sizeof(sSwathData->navigate.scAlt);
  memcpy(ptr, &sSwathData->navigate.scAttRoll, 
                sizeof(sSwathData->navigate.scAttRoll));
  ptr += sizeof(sSwathData->navigate.scAttRoll);
  memcpy(ptr, &sSwathData->navigate.scAttPitch, 
                sizeof(sSwathData->navigate.scAttPitch));
  ptr += sizeof(sSwathData->navigate.scAttPitch);
  memcpy(ptr, &sSwathData->navigate.scAttYaw, 
                sizeof(sSwathData->navigate.scAttYaw));
  ptr += sizeof(sSwathData->navigate.scAttYaw);
  memcpy(ptr, &sSwathData->navigate.att1, sizeof(sSwathData->navigate.att1));
  ptr += sizeof(sSwathData->navigate.att1);
  memcpy(ptr, &sSwathData->navigate.att2, sizeof(sSwathData->navigate.att2));
  ptr += sizeof(sSwathData->navigate.att2);
  memcpy(ptr, &sSwathData->navigate.att3, sizeof(sSwathData->navigate.att3));
  ptr += sizeof(sSwathData->navigate.att3);
  memcpy(ptr, &sSwathData->navigate.att4, sizeof(sSwathData->navigate.att4));
  ptr += sizeof(sSwathData->navigate.att4);
  memcpy(ptr, &sSwathData->navigate.att5, sizeof(sSwathData->navigate.att5));
  ptr += sizeof(sSwathData->navigate.att5);
  memcpy(ptr, &sSwathData->navigate.att6, sizeof(sSwathData->navigate.att6));
  ptr += sizeof(sSwathData->navigate.att6);
  memcpy(ptr, &sSwathData->navigate.att7, sizeof(sSwathData->navigate.att7));
  ptr += sizeof(sSwathData->navigate.att7);
  memcpy(ptr, &sSwathData->navigate.att8, sizeof(sSwathData->navigate.att8));
  ptr += sizeof(sSwathData->navigate.att8);
  memcpy(ptr, &sSwathData->navigate.att9, sizeof(sSwathData->navigate.att9));
  ptr += sizeof(sSwathData->navigate.att9);
  memcpy(ptr, &sSwathData->navigate.greenHourAng, 
                sizeof(sSwathData->navigate.greenHourAng));
  /* Writing packed vdata, navigation block, to an HDF file */
  vdataWrite(&tempVdata, databuf);
  free(databuf);

  /* Write SDS into HDF format */
  /*
   * Scaling calculation.  For writing,
   *    Multiplied the data by L2B31_DHAT
   */
  for (i = 0; i < 49; ++i)
  {
        /* The Following code was added to take care of the Missing or
           invalid data.  The reason to add this code was not to scale
           the invalid Data */

     if(sSwathData->dHat[i] > TK_DEFAULT)
         sSwathData->dHat_scale[i] =
                (int16)RND(sSwathData->dHat[i] * L2B31_DHAT);
     else
         sSwathData->dHat_scale[i] = TK_MISS_I2;
  }
  sdsWrite(&tempSDS, &sSwathData->dHat_scale);

  /*
   * Scaling calculation.  For writing,
   *    Multiplied the data by L2B31_SIGMADHAT
   */
  for (i = 0; i < 49; ++i)
  {
        /* The Following code was added to take care of the Missing or
           invalid data.  The reason to add this code was not to scale
           the invalid Data */

     if(sSwathData->sigmaDHat[i] > TK_DEFAULT)
         sSwathData->sigmaDHat_scale[i] =
                (int16)RND(sSwathData->sigmaDHat[i] * L2B31_SIGMADHAT);
     else
         sSwathData->sigmaDHat_scale[i] = TK_MISS_I2;
  }
  sdsWrite(&tempSDS, &sSwathData->sigmaDHat_scale);

  /* Write SDS into HDF format */
  /*
   * Scaling calculation.  For writing,
   *    Multiplied the data by L2B31_RHAT
   */
  for (i = 0; i < 49; ++i)
    for (j = 0; j < 80; ++j)
    {
        /* The Following code was added to take care of the Missing or
           invalid data.  The reason to add this code was not to scale
           the invalid Data */

      if(sSwathData->rHat[i][j] > TK_DEFAULT)
          sSwathData->rHat_scale[i][j] =
                (int16)RND(sSwathData->rHat[i][j] * L2B31_RHAT);
      else
          sSwathData->rHat_scale[i][j] = TK_MISS_I2;
    }
  sdsWrite(&tempSDS, &sSwathData->rHat_scale);

  /* Write SDS into HDF format */
  /*
   * Scaling calculation.  For writing,
   *    Multiplied the data by L2B31_SIGMARHAT
   */
  for (i = 0; i < 49; ++i)
    for(j = 0; j < 80; ++j)
    {
        /* The Following code was added to take care of the Missing or
           invalid data.  The reason to add this code was not to scale
           the invalid Data */

      if(sSwathData->sigmaRHat[i][j] > TK_DEFAULT)
          sSwathData->sigmaRHat_scale[i][j] =
             (int16)RND(sSwathData->sigmaRHat[i][j] * L2B31_SIGMARHAT);
      else
          sSwathData->sigmaRHat_scale[i][j] = TK_MISS_I2;
    } 
  sdsWrite(&tempSDS, &sSwathData->sigmaRHat_scale);

/*note: gone from here is pia sigmaPIA tmiPIA tk 6.25*/

  sdsWrite(&tempSDS, &sSwathData->rrSurf);


  /* Write SDS into HDF format */
  /*
   * Scaling calculation.  For writing,
   *    Multiplied the data by L2B31_SIGMARRSURF
   */
  for (i = 0; i < 49; ++i)
  {
        /* The Following code was added to take care of the Missing or
           invalid data.  The reason to add this code was not to scale
           the invalid Data */

     if(sSwathData->sigmaRRsurf[i] > TK_DEFAULT)
         sSwathData->sigmaRRsurf_scale[i] =
            (int16)RND(sSwathData->sigmaRRsurf[i] * L2B31_SIGMARRSURF);
     else
         sSwathData->sigmaRRsurf_scale[i] = TK_MISS_I2;
  }
  sdsWrite(&tempSDS, &sSwathData->sigmaRRsurf_scale);

  sdsWrite(&tempSDS, &sSwathData->latentHeatHH);

  sdsWrite(&tempSDS, &sSwathData->spare);

  return(TK_SUCCESS);
}


/***********************************************************************************
*
*  FUNCTION STATEMENT:
*	readScan_2B_31(IO_HANDLE *sGranuleHandle, L2B_31_SWATHDATA *sSwathData) 
*
*  DESCRIPTION:
*       This function is used to read the L2B-31 information in HDF format.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       sGranuleHandle          A structure containing information about the file.
*                               sGranuleHandle is returned by TKopen().
*
*  OUTPUT PARAMETERS: 
*       Parameter Name          Description
*       --------------          -----------
*       sSwathData              A structure containing the complete scanline data.
*                               This structure must be declared to correspond to 
*                               the data file being read.  
*
*  CALLED BY:                   TKreadScan()
*
*  ROUTINES CALLED:
*	allocateVdataMemory(), vdataRead(), sdsRead()
*
***********************************************************************************/

int readScan_2B_31(IO_HANDLE *sGranuleHandle, L2B_31_SWATHDATA *sSwathData)
{
  SDS_LINK *tempSDS = sGranuleHandle->cfgSTR->vGroup->subVgroup->sds;
  VDATA_LINK *tempVdata = sGranuleHandle->cfgSTR->vGroup->subVgroup->vdata;
  uint8 *databuf, *ptr;
  int   i, j;
  int scaleFactor;

  intn  status;
  char sds_name[500];
  int32 sds_id, rank, data_type, n_attrs;
  int32 dim_sizes[500];

  /* Allocate memory for unpacking the vdata */
  databuf = allocateVdataMemory(&tempVdata->vdataFields);
  ptr = databuf;
  /* Unpack the content of databuf into scanTime */
  vdataRead(&tempVdata, ptr);
  memcpy(&sSwathData->scanTime, ptr, sizeof(float64));
  free(databuf);

  sdsRead(&tempSDS, &scaleFactor, &sSwathData->geolocation);


  /* Allocate memory for unpacking the vdata */
  databuf = allocateVdataMemory(&tempVdata->vdataFields);
  ptr = databuf;
  /* Unpack the content of databuf into scanStatus block */
  vdataRead(&tempVdata, ptr);
  memcpy(&sSwathData->scanStatus.missing, ptr, 
		sizeof(sSwathData->scanStatus.missing));
  ptr += sizeof(sSwathData->scanStatus.missing);
  memcpy(&sSwathData->scanStatus.validity, ptr, 
		sizeof(sSwathData->scanStatus.validity));
  ptr += sizeof(sSwathData->scanStatus.validity);
  memcpy(&sSwathData->scanStatus.qac, ptr, 
		sizeof(sSwathData->scanStatus.qac));
  ptr += sizeof(sSwathData->scanStatus.qac);
  memcpy(&sSwathData->scanStatus.geoQuality, ptr, 
		sizeof(sSwathData->scanStatus.geoQuality));
  ptr += sizeof(sSwathData->scanStatus.geoQuality);
  memcpy(&sSwathData->scanStatus.dataQuality, ptr, 
		sizeof(sSwathData->scanStatus.dataQuality));
  ptr += sizeof(sSwathData->scanStatus.dataQuality);
  memcpy(&sSwathData->scanStatus.scOrient, ptr, 
		sizeof(sSwathData->scanStatus.scOrient));
  ptr += sizeof(sSwathData->scanStatus.scOrient);
  memcpy(&sSwathData->scanStatus.acsMode, ptr, 
		sizeof(sSwathData->scanStatus.acsMode));
  ptr += sizeof(sSwathData->scanStatus.acsMode);
  memcpy(&sSwathData->scanStatus.yawUpdateS, ptr, 
		sizeof(sSwathData->scanStatus.yawUpdateS));
  ptr += sizeof(sSwathData->scanStatus.yawUpdateS);
  memcpy(&sSwathData->scanStatus.prMode, ptr, 
		sizeof(sSwathData->scanStatus.prMode));
  ptr += sizeof(sSwathData->scanStatus.prMode);
  memcpy(&sSwathData->scanStatus.prStatus1, ptr,
                sizeof(sSwathData->scanStatus.prStatus1));
  ptr += sizeof(sSwathData->scanStatus.prStatus1);
  memcpy(&sSwathData->scanStatus.prStatus2, ptr,
                sizeof(sSwathData->scanStatus.prStatus2));
  ptr += sizeof(sSwathData->scanStatus.prStatus2);
  memcpy(&sSwathData->scanStatus.fractOrbitN, ptr, 
		sizeof(sSwathData->scanStatus.fractOrbitN));
  free(databuf);


  /* Allocate memory for unpacking the vdata */
  databuf = allocateVdataMemory(&tempVdata->vdataFields);
  ptr = databuf;
  /* Unpack the content of databuf into navigation block */
  vdataRead(&tempVdata, ptr);
  memcpy(&sSwathData->navigate.scPosX, ptr, sizeof(sSwathData->navigate.scPosX));
  ptr += sizeof(sSwathData->navigate.scPosX);
  memcpy(&sSwathData->navigate.scPosY, ptr, sizeof(sSwathData->navigate.scPosY));
  ptr += sizeof(sSwathData->navigate.scPosY);
  memcpy(&sSwathData->navigate.scPosZ, ptr, sizeof(sSwathData->navigate.scPosZ));
  ptr += sizeof(sSwathData->navigate.scPosZ);
  memcpy(&sSwathData->navigate.scVelX, ptr, sizeof(sSwathData->navigate.scVelX));
  ptr += sizeof(sSwathData->navigate.scVelX);
  memcpy(&sSwathData->navigate.scVelY, ptr, sizeof(sSwathData->navigate.scVelY));
  ptr += sizeof(sSwathData->navigate.scVelY);
  memcpy(&sSwathData->navigate.scVelZ, ptr, sizeof(sSwathData->navigate.scVelZ));
  ptr += sizeof(sSwathData->navigate.scVelZ);
  memcpy(&sSwathData->navigate.scLat, ptr, sizeof(sSwathData->navigate.scLat));
  ptr += sizeof(sSwathData->navigate.scLat);
  memcpy(&sSwathData->navigate.scLon, ptr, sizeof(sSwathData->navigate.scLon));
  ptr += sizeof(sSwathData->navigate.scLon);
  memcpy(&sSwathData->navigate.scAlt, ptr, sizeof(sSwathData->navigate.scAlt));
  ptr += sizeof(sSwathData->navigate.scAlt);
  memcpy(&sSwathData->navigate.scAttRoll, ptr, sizeof(sSwathData->navigate.scAttRoll));
  ptr += sizeof(sSwathData->navigate.scAttRoll);
  memcpy(&sSwathData->navigate.scAttPitch, ptr, sizeof(sSwathData->navigate.scAttPitch));  
  ptr += sizeof(sSwathData->navigate.scAttPitch);
  memcpy(&sSwathData->navigate.scAttYaw, ptr, sizeof(sSwathData->navigate.scAttYaw));
  ptr += sizeof(sSwathData->navigate.scAttYaw);
  memcpy(&sSwathData->navigate.att1, ptr, sizeof(sSwathData->navigate.att1));
  ptr += sizeof(sSwathData->navigate.att1);
  memcpy(&sSwathData->navigate.att2, ptr, sizeof(sSwathData->navigate.att2));
  ptr += sizeof(sSwathData->navigate.att2);
  memcpy(&sSwathData->navigate.att3, ptr, sizeof(sSwathData->navigate.att3));
  ptr += sizeof(sSwathData->navigate.att3);
  memcpy(&sSwathData->navigate.att4, ptr, sizeof(sSwathData->navigate.att4));
  ptr += sizeof(sSwathData->navigate.att4);
  memcpy(&sSwathData->navigate.att5, ptr, sizeof(sSwathData->navigate.att5));
  ptr += sizeof(sSwathData->navigate.att5);
  memcpy(&sSwathData->navigate.att6, ptr, sizeof(sSwathData->navigate.att6));
  ptr += sizeof(sSwathData->navigate.att6);
  memcpy(&sSwathData->navigate.att7, ptr, sizeof(sSwathData->navigate.att7));
  ptr += sizeof(sSwathData->navigate.att7);
  memcpy(&sSwathData->navigate.att8, ptr, sizeof(sSwathData->navigate.att8));
  ptr += sizeof(sSwathData->navigate.att8);
  memcpy(&sSwathData->navigate.att9, ptr, sizeof(sSwathData->navigate.att9));
  ptr += sizeof(sSwathData->navigate.att9);
  memcpy(&sSwathData->navigate.greenHourAng, ptr, 
		sizeof(sSwathData->navigate.greenHourAng));
  free(databuf);


  /* Read SDS from HDF format */
  /*
   * Scaling calculation.  For reading,
   *    Divided the data by L2B31_DHAT
   */
  sdsRead(&tempSDS, &scaleFactor, &sSwathData->dHat_scale);
  for (i = 0; i < 49; ++i)
        {
        if (sSwathData->dHat_scale[i] != TK_MISS_I2)
           sSwathData->dHat[i] = sSwathData->dHat_scale[i] * 1.0 / scaleFactor;
        else
           sSwathData->dHat[i] = TK_DEFAULT;
        }
  /*
   * Scaling calculation.  For reading,
   *    Divided the data by L2B31_SIGMADHAT 
   *     ....deleted toolkit 6.22.....
   *     ....restored toolkit 6.25.....
   */
   sdsRead(&tempSDS, &scaleFactor, &sSwathData->sigmaDHat_scale);
   for (i = 0; i < 49; ++i)
         {
         if (sSwathData->sigmaDHat_scale[i] != TK_MISS_I2)
            sSwathData->sigmaDHat[i] = 
                 (sSwathData->sigmaDHat_scale[i] * 1.0 / L2B31_SIGMADHAT);
         else
            sSwathData->sigmaDHat[i] = TK_DEFAULT;
         }

  /*
   * Scaling calculation.  For reading,
   *    Divided the data by L2B31_EPSILON
   *     ....deleted toolkit 6.22.....
   */
  if (versionCompare("6.21",sGranuleHandle->TKversion) >= 0)
  {
      sdsRead(&tempSDS, &scaleFactor, &sSwathData->epsilon_scale);
      for (i = 0; i < 49; ++i)
            {
            if (sSwathData->epsilon_scale[i] != TK_MISS_I2)
               sSwathData->epsilon[i] =
                    (sSwathData->epsilon_scale[i] * 1.0 / scaleFactor);
            else
               sSwathData->epsilon[i] = TK_DEFAULT;
            }
  }
  else
  {
      for (i = 0; i < 49; ++i)
        {
           sSwathData->epsilon[i] = TK_DEFAULT;
        }
  }

  /*
   * Scaling calculation.  For reading,
   *    Divided the data by L2B31_SIGMAEPSILON
   *     ....deleted toolkit 6.22.....
   */
  if (versionCompare("6.21",sGranuleHandle->TKversion) >= 0)
  {
      sdsRead(&tempSDS, &scaleFactor, &sSwathData->sigmaEpsilon_scale);
      for (i = 0; i < 49; ++i)
            {
            if (sSwathData->sigmaEpsilon_scale[i] != TK_MISS_I2)
               sSwathData->sigmaEpsilon[i] =
                    (sSwathData->sigmaEpsilon_scale[i] * 1.0 / scaleFactor);
            else
               sSwathData->sigmaEpsilon[i] = TK_DEFAULT;
            }
  }
  else
  {
      for (i = 0; i < 49; ++i)
        {
           sSwathData->sigmaEpsilon[i] = TK_DEFAULT;
        }
  }
  /*
   * Scaling calculation.  For reading,
   *    Divided the data by L2B31_RHAT 
   */ 
  sdsRead(&tempSDS, &scaleFactor, &sSwathData->rHat_scale);
  for (i = 0; i < 49; ++i)
    for (j = 0; j < 80; ++j)
        {
        if (sSwathData->rHat_scale[i][j] != TK_MISS_I2)
           sSwathData->rHat[i][j] = 
		sSwathData->rHat_scale[i][j] * 1.0 / scaleFactor;
        else
           sSwathData->rHat[i][j] = TK_DEFAULT;
        }
  /*
   * Scaling calculation.  For reading,
   *    Divided the data by L2B31_SIGMARHAT 
   */ 
  sdsRead(&tempSDS, &scaleFactor, &sSwathData->sigmaRHat_scale);
  for (i = 0; i < 49; ++i)
    for (j = 0; j < 80; ++j)
        {
        if (sSwathData->sigmaRHat_scale[i][j] != TK_MISS_I2)
           sSwathData->sigmaRHat[i][j] = 
                sSwathData->sigmaRHat_scale[i][j] * 1.0 / scaleFactor;
        else 
           sSwathData->sigmaRHat[i][j] = TK_DEFAULT;
        }
  /*
   * Scaling calculation.  For reading,
   *    Divided the data by L2B31_PIA
   */
  if (versionCompare("6.24",sGranuleHandle->TKversion) >= 0)
  {
      sdsRead(&tempSDS, &scaleFactor, &sSwathData->pia_scale);
      for (i = 0; i < 49; ++i)
            {
            if (sSwathData->pia_scale[i] != TK_MISS_I2)
               sSwathData->pia[i] = 
                    sSwathData->pia_scale[i] * 1.0 / scaleFactor;
            else
               sSwathData->pia[i] = TK_DEFAULT;
            }
  }
  else
  {
      for (i = 0; i < 49; ++i)
        {
           sSwathData->pia[i] = TK_DEFAULT;
        }
  }

  /*
   * Scaling calculation.  For reading,
   *    Divided the data by L2B31_SIGMAPIA
   */
  if (versionCompare("6.24",sGranuleHandle->TKversion) >= 0)
  {
      sdsRead(&tempSDS, &scaleFactor, &sSwathData->sigmaPIA_scale);
      for (i = 0; i < 49; ++i)
            {
            if (sSwathData->sigmaPIA_scale[i] != TK_MISS_I2)
               sSwathData->sigmaPIA[i] = 
                    sSwathData->sigmaPIA_scale[i] * 1.0 / scaleFactor;
            else
               sSwathData->sigmaPIA[i] = TK_DEFAULT;
            }
  }
  else
  {
      for (i = 0; i < 49; ++i)
        {
           sSwathData->sigmaPIA[i] = TK_DEFAULT;
        }
  }

  /*
   * Scaling calculation.  For reading,
   *    Divided the data by L2B31_TMIPIA 
   */ 
  if (versionCompare("6.24",sGranuleHandle->TKversion) >= 0)
  {
      sdsRead(&tempSDS, &scaleFactor, &sSwathData->tmiPIA_scale);
      for (i = 0; i < 49; ++i)
            {
            if (sSwathData->tmiPIA_scale[i] != TK_MISS_I2)
               sSwathData->tmiPIA[i] = 
                    sSwathData->tmiPIA_scale[i] * 1.0 / scaleFactor;
            else
               sSwathData->tmiPIA[i] = TK_DEFAULT;
            }
  }
  else
  {
      for (i = 0; i < 49; ++i)
        {
           sSwathData->tmiPIA[i] = TK_DEFAULT;
        }
  }

  /*
   * Scaling calculation.  For reading,
   *    Divided the data by L2B31_SIGMATMIPIA
   *   ... deleted toolkit 6.22.........
   */ 
  if (versionCompare("6.21",sGranuleHandle->TKversion) >= 0)
  {
      sdsRead(&tempSDS, &scaleFactor, &sSwathData->sigmaTMIpia_scale);
      for (i = 0; i < 49; ++i)
            {
            if (sSwathData->sigmaTMIpia_scale[i] != TK_MISS_I2)
               sSwathData->sigmaTMIpia[i] =
                    (sSwathData->sigmaTMIpia_scale[i] * 1.0 / scaleFactor);
            else
               sSwathData->sigmaTMIpia[i] = TK_DEFAULT;
            }
  }
  else
  {
      for (i = 0; i < 49; ++i)
        {
           sSwathData->sigmaTMIpia[i] = TK_DEFAULT;
        }
  }

  status = SDgetinfo(tempSDS->sdsID,sds_name,&rank,dim_sizes,&data_type,&n_attrs);
  if (data_type==DFNT_INT16)
  {
      sdsRead(&tempSDS, &scaleFactor, &sSwathData->rrSurf_scale);
      for (i = 0; i < 49; ++i)
            {
            if (sSwathData->rrSurf_scale[i] != TK_MISS_I2)
               sSwathData->rrSurf[i] =
                    sSwathData->rrSurf_scale[i] * 1.0 / scaleFactor;
            else
               sSwathData->rrSurf[i] = TK_DEFAULT;
            }
  }
  else
  {
/*
  if (data_type==DFNT_FLOAT32)
*/
      sdsRead(&tempSDS, &scaleFactor, &sSwathData->rrSurf);
   }
      

  /*
   * Scaling calculation.  For reading,
   *    Divided the data by L2B31_SIGMARRSURF
   */
  sdsRead(&tempSDS, &scaleFactor, &sSwathData->sigmaRRsurf_scale);
  for (i = 0; i < 49; ++i)
        {
        if (sSwathData->sigmaRRsurf_scale[i] != TK_MISS_I2)
           sSwathData->sigmaRRsurf[i] =
                sSwathData->sigmaRRsurf_scale[i] * 1.0 / scaleFactor;
        else
           sSwathData->sigmaRRsurf[i] = TK_DEFAULT;
        }

  sdsRead(&tempSDS, &scaleFactor, &sSwathData->latentHeatHH);

  sdsRead(&tempSDS, &scaleFactor, &sSwathData->spare);

  return(TK_SUCCESS);
}


/***********************************************************************************
*
*  FUNCTION STATEMENT:
*       writeGrid_3A_25(IO_HANDLE *sGranuleHandle, L3A_25_GRID *sGrid)
*
*  DESCRIPTION:
*       This function is used to write the L3A-25 information in HDF format.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       sGranuleHandle          A structure containing information about the file.
*                               sGranuleHandle is returned by TKopen().
*       sGrid			A structure contains the complete grid data.
*
*  OUTPUT PARAMETERS:           None
*
*  CALLED BY:                   TKwriteGrid()
*
*  ROUTINES CALLED:
*	allocateVdataMemory(), vdataWrite(), sdsWrite()
*
***********************************************************************************/

int writeGrid_3A_25(IO_HANDLE *sGranuleHandle, L3A_25_GRID *sGrid)
{
  SDS_LINK *tempSDS = sGranuleHandle->cfgSTR->vGroup->subVgroup->sds;
  VDATA_LINK *tempVdata = sGranuleHandle->cfgSTR->vGroup->subVgroup->vdata;
  uint8 *databuf, *ptr;


  sdsWrite(&tempSDS, &sGrid->grid1.rainMean1);
  sdsWrite(&tempSDS, &sGrid->grid1.rainDev1);
  sdsWrite(&tempSDS, &sGrid->grid1.convRainMean1);
  sdsWrite(&tempSDS, &sGrid->grid1.convRainDev1);
  sdsWrite(&tempSDS, &sGrid->grid1.stratRainMean1);
  sdsWrite(&tempSDS, &sGrid->grid1.stratRainDev1);

  sdsWrite(&tempSDS, &sGrid->grid1.shallowRainMean1);
  sdsWrite(&tempSDS, &sGrid->grid1.shallowRainDev1);
  sdsWrite(&tempSDS, &sGrid->grid1.shallowIsoRainMean1);
  sdsWrite(&tempSDS, &sGrid->grid1.shallowIsoRainDev1);

  sdsWrite(&tempSDS, &sGrid->grid1.zmMean1);
  sdsWrite(&tempSDS, &sGrid->grid1.zmDev1);
  sdsWrite(&tempSDS, &sGrid->grid1.convZmMean1);
  sdsWrite(&tempSDS, &sGrid->grid1.convZmDev1);
  sdsWrite(&tempSDS, &sGrid->grid1.stratZmMean1);
  sdsWrite(&tempSDS, &sGrid->grid1.stratZmDev1);
  sdsWrite(&tempSDS, &sGrid->grid1.ztMean1);
  sdsWrite(&tempSDS, &sGrid->grid1.ztDev1);
  sdsWrite(&tempSDS, &sGrid->grid1.convZtMean1);
  sdsWrite(&tempSDS, &sGrid->grid1.convZtDev1);
  sdsWrite(&tempSDS, &sGrid->grid1.stratZtMean1);
  sdsWrite(&tempSDS, &sGrid->grid1.stratZtDev1);

  sdsWrite(&tempSDS, &sGrid->grid1.piaSrtMean);
  sdsWrite(&tempSDS, &sGrid->grid1.piaSrtDev);
  sdsWrite(&tempSDS, &sGrid->grid1.piaHbMean);
  sdsWrite(&tempSDS, &sGrid->grid1.piaHbDev);
  sdsWrite(&tempSDS, &sGrid->grid1.pia0Mean);
  sdsWrite(&tempSDS, &sGrid->grid1.pia0Dev);
  sdsWrite(&tempSDS, &sGrid->grid1.pia2a25Mean);
  sdsWrite(&tempSDS, &sGrid->grid1.pia2a25Dev);

  sdsWrite(&tempSDS, &sGrid->grid1.piaSrtssMean);
  sdsWrite(&tempSDS, &sGrid->grid1.piaSrtssDev);
  sdsWrite(&tempSDS, &sGrid->grid1.piaHbssMean);
  sdsWrite(&tempSDS, &sGrid->grid1.piaHbssDev);
  sdsWrite(&tempSDS, &sGrid->grid1.pia0ssMean);
  sdsWrite(&tempSDS, &sGrid->grid1.pia0ssDev);
  sdsWrite(&tempSDS, &sGrid->grid1.pia2a25ssMean);
  sdsWrite(&tempSDS, &sGrid->grid1.pia2a25ssDev);

  sdsWrite(&tempSDS, &sGrid->grid1.stormHtMean);
  sdsWrite(&tempSDS, &sGrid->grid1.stormHtDev);

  sdsWrite(&tempSDS, &sGrid->grid1.xiMean);
  sdsWrite(&tempSDS, &sGrid->grid1.xiDev);
  sdsWrite(&tempSDS, &sGrid->grid1.nubfCorFacMean);
  sdsWrite(&tempSDS, &sGrid->grid1.nubfCorFacDev);
  sdsWrite(&tempSDS, &sGrid->grid1.bbHtMean);
  sdsWrite(&tempSDS, &sGrid->grid1.bbHtDev);

  sdsWrite(&tempSDS, &sGrid->grid1.epsilon0StratMean1);
  sdsWrite(&tempSDS, &sGrid->grid1.epsilon0StratDev1);
  sdsWrite(&tempSDS, &sGrid->grid1.epsilon0ConvMean1);
  sdsWrite(&tempSDS, &sGrid->grid1.epsilon0ConvDev1);
  sdsWrite(&tempSDS, &sGrid->grid1.epsilonStratMean1);
  sdsWrite(&tempSDS, &sGrid->grid1.epsilonStratDev1);
  sdsWrite(&tempSDS, &sGrid->grid1.epsilonConvMean1);
  sdsWrite(&tempSDS, &sGrid->grid1.epsilonConvDev1);

/*  sdsWrite(&tempSDS, &sGrid->grid1.epsilonMean1);*/
/*  sdsWrite(&tempSDS, &sGrid->grid1.epsilonDev1);*/
  sdsWrite(&tempSDS, &sGrid->grid1.surfRainMean1);
  sdsWrite(&tempSDS, &sGrid->grid1.surfRainDev1);
  sdsWrite(&tempSDS, &sGrid->grid1.e_surfRainMean1);
  sdsWrite(&tempSDS, &sGrid->grid1.e_surfRainDev1);
  sdsWrite(&tempSDS, &sGrid->grid1.bbZmaxMean1);
  sdsWrite(&tempSDS, &sGrid->grid1.bbZmaxDev1);

  sdsWrite(&tempSDS, &sGrid->grid1.bbNadirHtMean1);
  sdsWrite(&tempSDS, &sGrid->grid1.bbNadirHtDev1);
  sdsWrite(&tempSDS, &sGrid->grid1.bbNadirWidthMean1);
  sdsWrite(&tempSDS, &sGrid->grid1.bbNadirWidthDev1);
  sdsWrite(&tempSDS, &sGrid->grid1.bbNadirZmaxMean1);
  sdsWrite(&tempSDS, &sGrid->grid1.bbNadirZmaxDev1);

/*  sdsWrite(&tempSDS, &sGrid->grid1.bbwidthMean1);*/
/*  sdsWrite(&tempSDS, &sGrid->grid1.bbwidthDev1);*/
  sdsWrite(&tempSDS, &sGrid->grid1.sdepthMean1);
  sdsWrite(&tempSDS, &sGrid->grid1.sdepthDev1);
/*  sdsWrite(&tempSDS, &sGrid->grid1.surfRainAllMean1);*/
/*  sdsWrite(&tempSDS, &sGrid->grid1.surfRainAllDev1);*/

  sdsWrite(&tempSDS, &sGrid->grid1.surfRainConvMean1);
  sdsWrite(&tempSDS, &sGrid->grid1.surfRainConvDev1);
  sdsWrite(&tempSDS, &sGrid->grid1.surfRainStratMean1);
  sdsWrite(&tempSDS, &sGrid->grid1.surfRainStratDev1);

  sdsWrite(&tempSDS, &sGrid->grid1.e_surfRainConvMean1);
  sdsWrite(&tempSDS, &sGrid->grid1.e_surfRainConvDev1);
  sdsWrite(&tempSDS, &sGrid->grid1.e_surfRainStratMean1);
  sdsWrite(&tempSDS, &sGrid->grid1.e_surfRainStratDev1);

  sdsWrite(&tempSDS, &sGrid->grid1.rzStratA1);
  sdsWrite(&tempSDS, &sGrid->grid1.rzStratB1);
  sdsWrite(&tempSDS, &sGrid->grid1.rzConvA1);
  sdsWrite(&tempSDS, &sGrid->grid1.rzConvB1);
  sdsWrite(&tempSDS, &sGrid->grid1.rzA1);
  sdsWrite(&tempSDS, &sGrid->grid1.rzB1);

  sdsWrite(&tempSDS, &sGrid->grid1.ttlPix1);
  sdsWrite(&tempSDS, &sGrid->grid1.bbPixNum1);
  sdsWrite(&tempSDS, &sGrid->grid1.bbNadirPix1);
/*  sdsWrite(&tempSDS, &sGrid->grid1.wrainPix1);*/
  sdsWrite(&tempSDS, &sGrid->grid1.surfRainPix1);
  sdsWrite(&tempSDS, &sGrid->grid1.surfRainStratPix1);
  sdsWrite(&tempSDS, &sGrid->grid1.surfRainConvPix1);
  sdsWrite(&tempSDS, &sGrid->grid1.e_surfRainPix1);
  sdsWrite(&tempSDS, &sGrid->grid1.e_surfRainStratPix1);
  sdsWrite(&tempSDS, &sGrid->grid1.e_surfRainConvPix1);
  /* tk 6.20 sdsWrite(&tempSDS, &sGrid->grid1.epsilonPix1); */
  sdsWrite(&tempSDS, &sGrid->grid1.epsilon0StratPix1);
  sdsWrite(&tempSDS, &sGrid->grid1.epsilon0ConvPix1);
  sdsWrite(&tempSDS, &sGrid->grid1.epsilonStratPix1);
  sdsWrite(&tempSDS, &sGrid->grid1.epsilonConvPix1);
/*  sdsWrite(&tempSDS, &sGrid->grid1.surfRainAllPix1);*/
  sdsWrite(&tempSDS, &sGrid->grid1.rainPix1);
  sdsWrite(&tempSDS, &sGrid->grid1.convRainPix1);
  sdsWrite(&tempSDS, &sGrid->grid1.stratRainPix1);
  sdsWrite(&tempSDS, &sGrid->grid1.ttlAnglePix1);
  sdsWrite(&tempSDS, &sGrid->grid1.rainAnglePix1);
  sdsWrite(&tempSDS, &sGrid->grid1.rzStratPix1);
  sdsWrite(&tempSDS, &sGrid->grid1.rzConvPix1);
  sdsWrite(&tempSDS, &sGrid->grid1.rzPix1);
  sdsWrite(&tempSDS, &sGrid->grid1.shallowRainPix1);
  sdsWrite(&tempSDS, &sGrid->grid1.shallowIsoRainPix1);
  sdsWrite(&tempSDS, &sGrid->grid1.stormHH);
  sdsWrite(&tempSDS, &sGrid->grid1.convStormHH);
  sdsWrite(&tempSDS, &sGrid->grid1.stratStormHH);
  sdsWrite(&tempSDS, &sGrid->grid1.BBHH);

  sdsWrite(&tempSDS, &sGrid->grid1.bbNadirWidthH);
  sdsWrite(&tempSDS, &sGrid->grid1.bbNadirHH);
  sdsWrite(&tempSDS, &sGrid->grid1.bbNadirZmaxH);

  sdsWrite(&tempSDS, &sGrid->grid1.snowIceLH);
  sdsWrite(&tempSDS, &sGrid->grid1.bbZmaxH);
/*  sdsWrite(&tempSDS, &sGrid->grid1.epsilonH);*/

  sdsWrite(&tempSDS, &sGrid->grid1.epsilon0StratH);
  sdsWrite(&tempSDS, &sGrid->grid1.epsilon0ConvH);
  sdsWrite(&tempSDS, &sGrid->grid1.epsilonStratH);
  sdsWrite(&tempSDS, &sGrid->grid1.epsilonConvH);

  sdsWrite(&tempSDS, &sGrid->grid1.surfRainH);
  sdsWrite(&tempSDS, &sGrid->grid1.e_surfRainH);
/*  sdsWrite(&tempSDS, &sGrid->grid1.surfRainAllH);*/

  sdsWrite(&tempSDS, &sGrid->grid1.surfRainConvH);
  sdsWrite(&tempSDS, &sGrid->grid1.surfRainStratH);
  sdsWrite(&tempSDS, &sGrid->grid1.e_surfRainConvH);
  sdsWrite(&tempSDS, &sGrid->grid1.e_surfRainStratH);

  sdsWrite(&tempSDS, &sGrid->grid1.zmH);
  sdsWrite(&tempSDS, &sGrid->grid1.convZmH);
  sdsWrite(&tempSDS, &sGrid->grid1.stratZmH);
  sdsWrite(&tempSDS, &sGrid->grid1.ztH);
  sdsWrite(&tempSDS, &sGrid->grid1.convZtH);
  sdsWrite(&tempSDS, &sGrid->grid1.stratZtH);
  sdsWrite(&tempSDS, &sGrid->grid1.rainH);
  sdsWrite(&tempSDS, &sGrid->grid1.convRainH);
  sdsWrite(&tempSDS, &sGrid->grid1.stratRainH);

  sdsWrite(&tempSDS, &sGrid->grid1.shallowRainH);
  sdsWrite(&tempSDS, &sGrid->grid1.shallowIsoRainH);

  sdsWrite(&tempSDS, &sGrid->grid1.piaSrtH);
  sdsWrite(&tempSDS, &sGrid->grid1.piaHbH);
  sdsWrite(&tempSDS, &sGrid->grid1.pia0H);
  sdsWrite(&tempSDS, &sGrid->grid1.pia2a25H);

  sdsWrite(&tempSDS, &sGrid->grid1.piaSrtssH);
  sdsWrite(&tempSDS, &sGrid->grid1.piaHbssH);
  sdsWrite(&tempSDS, &sGrid->grid1.pia0ssH);
  sdsWrite(&tempSDS, &sGrid->grid1.pia2a25ssH);

/*  sdsWrite(&tempSDS, &sGrid->grid1.zmGradH);*/
  sdsWrite(&tempSDS, &sGrid->grid1.xiH);
  sdsWrite(&tempSDS, &sGrid->grid1.nubfH);
/*  sdsWrite(&tempSDS, &sGrid->grid1.zpzmH);*/
  sdsWrite(&tempSDS, &sGrid->grid1.rainCCoef);
  sdsWrite(&tempSDS, &sGrid->grid1.convRainCCoef);
  sdsWrite(&tempSDS, &sGrid->grid1.stratRainCCoef);
/*  sdsWrite(&tempSDS, &sGrid->grid1.stormHtZmCCoef);*/
  sdsWrite(&tempSDS, &sGrid->grid1.piaCCoef);
/*  sdsWrite(&tempSDS, &sGrid->grid1.xiZmCCoef);*/

  sdsWrite(&tempSDS, &sGrid->grid1.piaSrtPix);
  sdsWrite(&tempSDS, &sGrid->grid1.piaHbPix);
  sdsWrite(&tempSDS, &sGrid->grid1.pia0Pix);
  sdsWrite(&tempSDS, &sGrid->grid1.pia2a25Pix);

  sdsWrite(&tempSDS, &sGrid->grid1.piaSrtssPix);
  sdsWrite(&tempSDS, &sGrid->grid1.piaHbssPix);
  sdsWrite(&tempSDS, &sGrid->grid1.pia0ssPix);
  sdsWrite(&tempSDS, &sGrid->grid1.pia2a25ssPix);

  sdsWrite(&tempSDS, &sGrid->grid1.rainCCoefPix);
  sdsWrite(&tempSDS, &sGrid->grid1.stratCCoefPix);
  sdsWrite(&tempSDS, &sGrid->grid1.convCCoefPix);

  sdsWrite(&tempSDS, &sGrid->grid1.piaCCoefPix);


  tempSDS = sGranuleHandle->cfgSTR->vGroup->subVgroup->next->sds;
  tempVdata = sGranuleHandle->cfgSTR->vGroup->subVgroup->next->vdata;



  sdsWrite(&tempSDS, &sGrid->grid2.rainMean2);
  sdsWrite(&tempSDS, &sGrid->grid2.rainDev2);
  sdsWrite(&tempSDS, &sGrid->grid2.convRainMean2);
  sdsWrite(&tempSDS, &sGrid->grid2.convRainDev2);
  sdsWrite(&tempSDS, &sGrid->grid2.stratRainMean2);
  sdsWrite(&tempSDS, &sGrid->grid2.stratRainDev2);

  sdsWrite(&tempSDS, &sGrid->grid2.shallowRainMean2);
  sdsWrite(&tempSDS, &sGrid->grid2.shallowRainDev2);
  sdsWrite(&tempSDS, &sGrid->grid2.shallowIsoRainMean2);
  sdsWrite(&tempSDS, &sGrid->grid2.shallowIsoRainDev2);

  sdsWrite(&tempSDS, &sGrid->grid2.zmMean2);
  sdsWrite(&tempSDS, &sGrid->grid2.convZmMean2);
  sdsWrite(&tempSDS, &sGrid->grid2.stratZmMean2);
  sdsWrite(&tempSDS, &sGrid->grid2.ztMean2);
  sdsWrite(&tempSDS, &sGrid->grid2.convZtMean2);
  sdsWrite(&tempSDS, &sGrid->grid2.stratZtMean2);
  sdsWrite(&tempSDS, &sGrid->grid2.stormHeightMean);
  sdsWrite(&tempSDS, &sGrid->grid2.stormHeightDev2);
  sdsWrite(&tempSDS, &sGrid->grid2.bbHeightMean);

  sdsWrite(&tempSDS, &sGrid->grid2.epsilon0StratMean2);
  sdsWrite(&tempSDS, &sGrid->grid2.epsilon0StratDev2);
  sdsWrite(&tempSDS, &sGrid->grid2.epsilon0ConvMean2);
  sdsWrite(&tempSDS, &sGrid->grid2.epsilon0ConvDev2);
  sdsWrite(&tempSDS, &sGrid->grid2.epsilonStratMean2);
  sdsWrite(&tempSDS, &sGrid->grid2.epsilonStratDev2);
  sdsWrite(&tempSDS, &sGrid->grid2.epsilonConvMean2);
  sdsWrite(&tempSDS, &sGrid->grid2.epsilonConvDev2);

  sdsWrite(&tempSDS, &sGrid->grid2.surfRainMean2);
  sdsWrite(&tempSDS, &sGrid->grid2.surfRainDev2);
  sdsWrite(&tempSDS, &sGrid->grid2.e_surfRainMean2);
  sdsWrite(&tempSDS, &sGrid->grid2.e_surfRainDev2);
  sdsWrite(&tempSDS, &sGrid->grid2.bbZmaxMean2);
  sdsWrite(&tempSDS, &sGrid->grid2.bbZmaxDev2);
  sdsWrite(&tempSDS, &sGrid->grid2.sdepthMean2);
  sdsWrite(&tempSDS, &sGrid->grid2.sdepthDev2);
  sdsWrite(&tempSDS, &sGrid->grid2.bbHeightDev2);
/*  sdsWrite(&tempSDS, &sGrid->grid2.surfRainAllMean2);*/
/*  sdsWrite(&tempSDS, &sGrid->grid2.surfRainAllDev2);*/

  sdsWrite(&tempSDS, &sGrid->grid2.surfRainConvMean2);
  sdsWrite(&tempSDS, &sGrid->grid2.surfRainConvDev2);
  sdsWrite(&tempSDS, &sGrid->grid2.surfRainStratMean2);
  sdsWrite(&tempSDS, &sGrid->grid2.surfRainStratDev2);

  sdsWrite(&tempSDS, &sGrid->grid2.e_surfRainConvMean2);
  sdsWrite(&tempSDS, &sGrid->grid2.e_surfRainConvDev2);
  sdsWrite(&tempSDS, &sGrid->grid2.e_surfRainStratMean2);
  sdsWrite(&tempSDS, &sGrid->grid2.e_surfRainStratDev2);

  sdsWrite(&tempSDS, &sGrid->grid2.rzStratA2);
  sdsWrite(&tempSDS, &sGrid->grid2.rzStratB2);
  sdsWrite(&tempSDS, &sGrid->grid2.rzConvA2);
  sdsWrite(&tempSDS, &sGrid->grid2.rzConvB2);
  sdsWrite(&tempSDS, &sGrid->grid2.rzA2);
  sdsWrite(&tempSDS, &sGrid->grid2.rzB2);

  sdsWrite(&tempSDS, &sGrid->grid2.ttlPix2);
  sdsWrite(&tempSDS, &sGrid->grid2.bbPixNum2);
/*  sdsWrite(&tempSDS, &sGrid->grid2.wrainPix2);*/
  sdsWrite(&tempSDS, &sGrid->grid2.surfRainPix2);
  sdsWrite(&tempSDS, &sGrid->grid2.surfRainConvPix2);
  sdsWrite(&tempSDS, &sGrid->grid2.surfRainStratPix2);
  sdsWrite(&tempSDS, &sGrid->grid2.e_surfRainPix2);
  sdsWrite(&tempSDS, &sGrid->grid2.e_surfRainConvPix2);
  sdsWrite(&tempSDS, &sGrid->grid2.e_surfRainStratPix2);
/*  sdsWrite(&tempSDS, &sGrid->grid2.surfRainAllPix2);*/
  sdsWrite(&tempSDS, &sGrid->grid2.rainPix2);
  sdsWrite(&tempSDS, &sGrid->grid2.convRainPix2);
  sdsWrite(&tempSDS, &sGrid->grid2.stratRainPix2);
  sdsWrite(&tempSDS, &sGrid->grid2.shallowRainPix2);
  sdsWrite(&tempSDS, &sGrid->grid2.shallowIsoRainPix2);
  sdsWrite(&tempSDS, &sGrid->grid2.epsilon0StratPix2);
  sdsWrite(&tempSDS, &sGrid->grid2.epsilon0ConvPix2);
  sdsWrite(&tempSDS, &sGrid->grid2.epsilonStratPix2);
  sdsWrite(&tempSDS, &sGrid->grid2.epsilonConvPix2);
  sdsWrite(&tempSDS, &sGrid->grid2.rzStratPix2);
  sdsWrite(&tempSDS, &sGrid->grid2.rzConvPix2);
  sdsWrite(&tempSDS, &sGrid->grid2.rzPix2);


  return(TK_SUCCESS);
}


/***********************************************************************************
*
*  FUNCTION STATEMENT:
*       readGrid_3A_25(IO_HANDLE *sGranuleHandle, L3A_25_GRID *sGrid)
*
*  DESCRIPTION:
*       This function is used to read the L3A_25 information in HDF format.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       sGranuleHandle          A structure containing information about the file.
*                               sGranuleHandle is returned by TKopen().
*
*  OUTPUT PARAMETERS:    
*       Parameter Name          Description
*       --------------          -----------
*       sGrid			A structure contains the complete grid data.
*
*  CALLED BY:                   TKreadGrid()
*
*  ROUTINES CALLED: 
*	allocateVdataMemory(), vdataRead(), sdsRead()
*
***********************************************************************************/

int readGrid_3A_25(IO_HANDLE *sGranuleHandle, L3A_25_GRID *sGrid)
{
  SDS_LINK *tempSDS = sGranuleHandle->cfgSTR->vGroup->subVgroup->sds;
  VDATA_LINK *tempVdata = sGranuleHandle->cfgSTR->vGroup->subVgroup->vdata;
  uint8 *databuf, *ptr;
  int scaleFactor;

  /* Read SDS from a HDF file */

  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.rainMean1);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.rainDev1);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.convRainMean1);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.convRainDev1);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.stratRainMean1);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.stratRainDev1);

  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.shallowRainMean1);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.shallowRainDev1);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.shallowIsoRainMean1);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.shallowIsoRainDev1);

  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.zmMean1);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.zmDev1);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.convZmMean1);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.convZmDev1);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.stratZmMean1);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.stratZmDev1);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.ztMean1);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.ztDev1);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.convZtMean1);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.convZtDev1);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.stratZtMean1);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.stratZtDev1);

  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.piaSrtMean);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.piaSrtDev);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.piaHbMean);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.piaHbDev);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.pia0Mean);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.pia0Dev);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.pia2a25Mean);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.pia2a25Dev);

  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.piaSrtssMean);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.piaSrtssDev);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.piaHbssMean);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.piaHbssDev);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.pia0ssMean);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.pia0ssDev);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.pia2a25ssMean);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.pia2a25ssDev);

  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.stormHtMean);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.stormHtDev);

  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.xiMean);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.xiDev);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.nubfCorFacMean);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.nubfCorFacDev);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.bbHtMean);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.bbHtDev);

  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.epsilon0StratMean1);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.epsilon0StratDev1);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.epsilon0ConvMean1);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.epsilon0ConvDev1);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.epsilonStratMean1);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.epsilonStratDev1);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.epsilonConvMean1);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.epsilonConvDev1);

  if (versionCompare("5.8",sGranuleHandle->TKversion) >= 0) sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.epsilonMean1);
  if (versionCompare("5.8",sGranuleHandle->TKversion) >= 0) sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.epsilonDev1);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.surfRainMean1);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.surfRainDev1);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.e_surfRainMean1);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.e_surfRainDev1);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.bbZmaxMean1);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.bbZmaxDev1);

  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.bbNadirHtMean1);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.bbNadirHtDev1);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.bbNadirWidthMean1);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.bbNadirWidthDev1);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.bbNadirZmaxMean1);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.bbNadirZmaxDev1);

  if (versionCompare("5.8",sGranuleHandle->TKversion) >= 0) sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.bbwidthMean1);
  if (versionCompare("5.8",sGranuleHandle->TKversion) >= 0) sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.bbwidthDev1);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.sdepthMean1);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.sdepthDev1);
  if (versionCompare("5.8",sGranuleHandle->TKversion) >= 0) sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.surfRainAllMean1);
  if (versionCompare("5.8",sGranuleHandle->TKversion) >= 0) sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.surfRainAllDev1);

  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.surfRainConvMean1);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.surfRainConvDev1);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.surfRainStratMean1);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.surfRainStratDev1);

  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.e_surfRainConvMean1);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.e_surfRainConvDev1);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.e_surfRainStratMean1);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.e_surfRainStratDev1);

  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.rzStratA1);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.rzStratB1);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.rzConvA1);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.rzConvB1);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.rzA1);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.rzB1);

  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.ttlPix1);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.bbPixNum1);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.bbNadirPix1);
  if (versionCompare("5.8",sGranuleHandle->TKversion) >= 0) sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.wrainPix1);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.surfRainPix1);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.surfRainStratPix1);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.surfRainConvPix1);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.e_surfRainPix1);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.e_surfRainStratPix1);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.e_surfRainConvPix1);

  if (versionCompare("6.19",sGranuleHandle->TKversion) >= 0) sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.epsilonPix1);

  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.epsilon0StratPix1);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.epsilon0ConvPix1);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.epsilonStratPix1);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.epsilonConvPix1);
  if (versionCompare("5.8",sGranuleHandle->TKversion) >= 0) sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.surfRainAllPix1);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.rainPix1);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.convRainPix1);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.stratRainPix1);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.ttlAnglePix1);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.rainAnglePix1);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.rzStratPix1);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.rzConvPix1);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.rzPix1);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.shallowRainPix1);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.shallowIsoRainPix1);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.stormHH);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.convStormHH);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.stratStormHH);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.BBHH);

  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.bbNadirWidthH);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.bbNadirHH);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.bbNadirZmaxH);

  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.snowIceLH);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.bbZmaxH);
  if (versionCompare("5.8",sGranuleHandle->TKversion) >= 0) sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.epsilonH);

  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.epsilon0StratH);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.epsilon0ConvH);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.epsilonStratH);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.epsilonConvH);

  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.surfRainH);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.e_surfRainH);
  if (versionCompare("5.8",sGranuleHandle->TKversion) >= 0) sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.surfRainAllH);

  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.surfRainConvH);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.surfRainStratH);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.e_surfRainConvH);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.e_surfRainStratH);

  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.zmH);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.convZmH);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.stratZmH);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.ztH);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.convZtH);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.stratZtH);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.rainH);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.convRainH);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.stratRainH);

  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.shallowRainH);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.shallowIsoRainH);

  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.piaSrtH);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.piaHbH);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.pia0H);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.pia2a25H);

  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.piaSrtssH);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.piaHbssH);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.pia0ssH);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.pia2a25ssH);

  if (versionCompare("5.8",sGranuleHandle->TKversion) >= 0) sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.zmGradH);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.xiH);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.nubfH);
  if (versionCompare("5.8",sGranuleHandle->TKversion) >= 0) sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.zpzmH);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.rainCCoef);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.convRainCCoef);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.stratRainCCoef);
  if (versionCompare("5.8",sGranuleHandle->TKversion) >= 0) sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.stormHtZmCCoef);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.piaCCoef);
  if (versionCompare("5.8",sGranuleHandle->TKversion) >= 0) sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.xiZmCCoef);


  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.piaSrtPix);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.piaHbPix);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.pia0Pix);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.pia2a25Pix);

  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.piaSrtssPix);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.piaHbssPix);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.pia0ssPix);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.pia2a25ssPix);

  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.rainCCoefPix);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.stratCCoefPix);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.convCCoefPix);

  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid1.piaCCoefPix);


  tempSDS = sGranuleHandle->cfgSTR->vGroup->subVgroup->next->sds;
  tempVdata = sGranuleHandle->cfgSTR->vGroup->subVgroup->next->vdata;


  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid2.rainMean2);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid2.rainDev2);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid2.convRainMean2);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid2.convRainDev2);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid2.stratRainMean2);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid2.stratRainDev2);

  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid2.shallowRainMean2);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid2.shallowRainDev2);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid2.shallowIsoRainMean2);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid2.shallowIsoRainDev2);

  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid2.zmMean2);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid2.convZmMean2);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid2.stratZmMean2);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid2.ztMean2);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid2.convZtMean2);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid2.stratZtMean2);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid2.stormHeightMean);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid2.stormHeightDev2);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid2.bbHeightMean);

  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid2.epsilon0StratMean2);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid2.epsilon0StratDev2);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid2.epsilon0ConvMean2);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid2.epsilon0ConvDev2);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid2.epsilonStratMean2);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid2.epsilonStratDev2);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid2.epsilonConvMean2);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid2.epsilonConvDev2);

  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid2.surfRainMean2);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid2.surfRainDev2);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid2.e_surfRainMean2);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid2.e_surfRainDev2);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid2.bbZmaxMean2);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid2.bbZmaxDev2);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid2.sdepthMean2);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid2.sdepthDev2);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid2.bbHeightDev2);
  if (versionCompare("5.8",sGranuleHandle->TKversion) >= 0) sdsRead(&tempSDS, &scaleFactor, &sGrid->grid2.surfRainAllMean2);
  if (versionCompare("5.8",sGranuleHandle->TKversion) >= 0) sdsRead(&tempSDS, &scaleFactor, &sGrid->grid2.surfRainAllDev2);

  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid2.surfRainConvMean2);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid2.surfRainConvDev2);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid2.surfRainStratMean2);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid2.surfRainStratDev2);

  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid2.e_surfRainConvMean2);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid2.e_surfRainConvDev2);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid2.e_surfRainStratMean2);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid2.e_surfRainStratDev2);

  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid2.rzStratA2);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid2.rzStratB2);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid2.rzConvA2);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid2.rzConvB2);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid2.rzA2);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid2.rzB2);

  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid2.ttlPix2);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid2.bbPixNum2);
  if (versionCompare("5.8",sGranuleHandle->TKversion) >= 0) sdsRead(&tempSDS, &scaleFactor, &sGrid->grid2.wrainPix2);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid2.surfRainPix2);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid2.surfRainConvPix2);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid2.surfRainStratPix2);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid2.e_surfRainPix2);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid2.e_surfRainConvPix2);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid2.e_surfRainStratPix2);
  if (versionCompare("5.8",sGranuleHandle->TKversion) >= 0) sdsRead(&tempSDS, &scaleFactor, &sGrid->grid2.surfRainAllPix2);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid2.rainPix2);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid2.convRainPix2);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid2.stratRainPix2);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid2.shallowRainPix2);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid2.shallowIsoRainPix2);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid2.epsilon0StratPix2);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid2.epsilon0ConvPix2);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid2.epsilonStratPix2);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid2.epsilonConvPix2);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid2.rzStratPix2);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid2.rzConvPix2);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->grid2.rzPix2);


  return(TK_SUCCESS);
}


/***********************************************************************************
*
*  FUNCTION STATEMENT:
*       writeGrid_3A_26(IO_HANDLE *sGranuleHandle, L3A_26_GRID *sGrid)
*
*  DESCRIPTION:
*       This function is used to write the L3A-26 information in HDF format.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       sGranuleHandle          A structure containing information about the file.
*                               sGranuleHandle is returned by TKopen().
*       sGrid			A structure contains the complete grid data.
*
*  OUTPUT PARAMETERS:           None
*
*  CALLED BY:                   TKwriteGrid()
*
*  ROUTINES CALLED: 
*	allocateVdatMemory(), vdataWrite(), sdsWrite() 
*
***********************************************************************************/

int writeGrid_3A_26(IO_HANDLE *sGranuleHandle, L3A_26_GRID *sGrid)
{
  SDS_LINK *tempSDS = sGranuleHandle->cfgSTR->vGroup->subVgroup->sds;
  VDATA_LINK *tempVdata = sGranuleHandle->cfgSTR->vGroup->subVgroup->vdata;
  uint8 *databuf, *ptr;

  /* Write SDS to a HDF file */
  sdsWrite(&tempSDS, &sGrid->ttlCount);
  sdsWrite(&tempSDS, &sGrid->rainCount);
  sdsWrite(&tempSDS, &sGrid->zeroOrderpDf);
  sdsWrite(&tempSDS, &sGrid->hbpDf);
  sdsWrite(&tempSDS, &sGrid->pDf2A25);
  sdsWrite(&tempSDS, &sGrid->zeroOrderFit);
  sdsWrite(&tempSDS, &sGrid->hbFit);
  sdsWrite(&tempSDS, &sGrid->fit2A25);
  sdsWrite(&tempSDS, &sGrid->reliabOrderFit);
  sdsWrite(&tempSDS, &sGrid->reliabHBfit);
  sdsWrite(&tempSDS, &sGrid->reliab2A25fit);
  sdsWrite(&tempSDS, &sGrid->rainMeanTH);

  return(TK_SUCCESS);
}

int writeGrid_3A_27(IO_HANDLE *sGranuleHandle, L3A_27_GRID *sGrid)
{
  SDS_LINK *tempSDS = sGranuleHandle->cfgSTR->vGroup->subVgroup->sds;
  VDATA_LINK *tempVdata = sGranuleHandle->cfgSTR->vGroup->subVgroup->vdata;
  uint8 *databuf, *ptr;

  sdsWrite(&tempSDS, &sGrid->LatentHeating);

  return(TK_SUCCESS);
}

/***********************************************************************************
*
*  FUNCTION STATEMENT:
*       readGrid_3A_26(IO_HANDLE *sGranuleHandle, L3A_26_GRID *sGrid)
*
*  DESCRIPTION:
*       This function is used to read the L3A_26 information in HDF format.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       sGranuleHandle          A structure containing information about the file.
*                               sGranuleHandle is returned by TKopen().
*
*  OUTPUT PARAMETERS:   
*       Parameter Name          Description
*       --------------          -----------
*       sGrid			A structure contains the complete grid data.
*
*  CALLED BY:                   TKreadGrid()
*
*  ROUTINES CALLED:  
*	allocateVdataMemory(), vdataRead(), sdsRead() 
*
***********************************************************************************/

int readGrid_3A_26(IO_HANDLE *sGranuleHandle, L3A_26_GRID *sGrid)
{
  SDS_LINK *tempSDS = sGranuleHandle->cfgSTR->vGroup->subVgroup->sds;
  VDATA_LINK *tempVdata = sGranuleHandle->cfgSTR->vGroup->subVgroup->vdata;
  uint8 *databuf, *ptr;
  int scaleFactor;

  /* Read SDS from a HDF file */
  sdsRead(&tempSDS, &scaleFactor, &sGrid->ttlCount);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->rainCount);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->zeroOrderpDf);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->hbpDf);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->pDf2A25);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->zeroOrderFit);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->hbFit);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->fit2A25);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->reliabOrderFit);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->reliabHBfit);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->reliab2A25fit);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->rainMeanTH);

  return(TK_SUCCESS);
}

int readGrid_3A_27(IO_HANDLE *sGranuleHandle, L3A_27_GRID *sGrid)
{
  SDS_LINK *tempSDS = sGranuleHandle->cfgSTR->vGroup->subVgroup->sds;
  VDATA_LINK *tempVdata = sGranuleHandle->cfgSTR->vGroup->subVgroup->vdata;
  uint8 *databuf, *ptr;
  int scaleFactor;

  sdsRead(&tempSDS, &scaleFactor, &sGrid->LatentHeating);

  return(TK_SUCCESS);
}

/***********************************************************************************
*
*  FUNCTION STATEMENT:
*       writeGrid_3B_31(IO_HANDLE *sGranuleHandle, L3B_31_GRID *sGrid)
*
*  DESCRIPTION:
*       This function is used to write the L3B-31 information in HDF format.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       sGranuleHandle          A structure containing information about the file.
*                               sGranuleHandle is returned by TKopen().
*       sGrid			A structure contains the complete grid data.
*
*  OUTPUT PARAMETERS:           None
*
*  CALLED BY:                   TKwriteGrid()
*
*  ROUTINES CALLED:
*	allocateVdataMemory(), sdsWrite(), vdataWrite()
*
***********************************************************************************/

int writeGrid_3B_31(IO_HANDLE *sGranuleHandle, L3B_31_GRID *sGrid)
{
  SDS_LINK *tempSDS = sGranuleHandle->cfgSTR->vGroup->subVgroup->sds;
  VDATA_LINK *tempVdata = sGranuleHandle->cfgSTR->vGroup->subVgroup->vdata;
  uint8 *databuf, *ptr;

  /* Write SDS to a HDF file */

  sdsWrite(&tempSDS, &sGrid->sfcrainTMI);
  sdsWrite(&tempSDS, &sGrid->convectRain);
  sdsWrite(&tempSDS, &sGrid->cloudWater);
  sdsWrite(&tempSDS, &sGrid->rainWaterTMI);
  sdsWrite(&tempSDS, &sGrid->cloudIce);
  sdsWrite(&tempSDS, &sGrid->graupel);
  sdsWrite(&tempSDS, &sGrid->latentHeat);
  sdsWrite(&tempSDS, &sGrid->sfcrainCOMB);
  sdsWrite(&tempSDS, &sGrid->rainWaterCOMB);
  sdsWrite(&tempSDS, &sGrid->sfcrainTMIoverlap);
  sdsWrite(&tempSDS, &sGrid->convectRainoverlap);
  sdsWrite(&tempSDS, &sGrid->sfcrainCOMBoverlap);
  sdsWrite(&tempSDS, &sGrid->surfAdjRatio);
  sdsWrite(&tempSDS, &sGrid->surfAdjRatioverlap);

  return(TK_SUCCESS);
}


/***********************************************************************************
*
*  FUNCTION STATEMENT:
*       readGrid_3B_31(IO_HANDLE *sGranuleHandle, L3B_31_GRID *sGrid)
*
*  DESCRIPTION:
*       This function is used to read the L3B_31 information in HDF format.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       sGranuleHandle          A structure containing information about the file.
*                               sGranuleHandle is returned by TKopen().
*
*  OUTPUT PARAMETERS:  
*       Parameter Name          Description
*       --------------          -----------
*       sGrid			A structure contains the complete grid data.
*
*  CALLED BY:                   TKreadGrid()
*
*  ROUTINES CALLED: 
*	allocateVdataMemory(), vdataRead(), sdsRead() 
*
***********************************************************************************/

int readGrid_3B_31(IO_HANDLE *sGranuleHandle, L3B_31_GRID *sGrid)
{ 
  SDS_LINK *tempSDS = sGranuleHandle->cfgSTR->vGroup->subVgroup->sds;
  VDATA_LINK *tempVdata = sGranuleHandle->cfgSTR->vGroup->subVgroup->vdata;
  uint8 *databuf, *ptr;
  int scaleFactor;
  int i,j,k;

  /* Read SDS from a HDF file */
  sdsRead(&tempSDS, &scaleFactor, &sGrid->sfcrainTMI);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->convectRain);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->cloudWater);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->rainWaterTMI);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->cloudIce);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->graupel);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->latentHeat);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->sfcrainCOMB);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->rainWaterCOMB);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->sfcrainTMIoverlap);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->convectRainoverlap);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->sfcrainCOMBoverlap);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->surfAdjRatio);
  sdsRead(&tempSDS, &scaleFactor, &sGrid->surfAdjRatioverlap);

  if (versionCompare("4.9.1",sGranuleHandle->TKversion) >= 0)
  {
      sdsRead(&tempSDS, &scaleFactor, &sGrid->surfRainfall);
  }
  else
  {
    for (i = 0; i< 72; i++)
      for (j = 0; j< 16; j++)
        sGrid->surfRainfall[i][j] = TK_DEFAULT;
  }

  if (versionCompare("4.9.1",sGranuleHandle->TKversion) >= 0)
  {
      sdsRead(&tempSDS, &scaleFactor, &sGrid->rainWater);
  }
  else
  {
    for (i = 0; i< 14; i++)
      for (j = 0; j< 72; j++)
        for (k = 0; k< 16; k++)
          sGrid->rainWater[i][j][k] = TK_DEFAULT;
  }

  if (versionCompare("4.9.1",sGranuleHandle->TKversion) >= 0)
  {
      sdsRead(&tempSDS, &scaleFactor, &sGrid->profAdjRatio);
  }
  else
  {
    for (i = 0; i< 14; i++)
      for (j = 0; j< 72; j++)
        for (k = 0; k< 16; k++)
          sGrid->profAdjRatio[i][j][k] = TK_DEFAULT;
  }

  return(TK_SUCCESS);
}

/***********************************************************************************
*
*  FUNCTION STATEMENT:
*       writeL2A25Header(IO_HANDLE *sGranuleHandle, CLUTTER_FLAGS *sHeader) 
*
*  DESCRIPTION:
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       sGranuleHandle          A structure containing information about the file.
*                               sGranuleHandle is returned by TKopen().
*	sHeader			A structure contains the complete header data.
*
*  OUTPUT PARAMETERS:           None
*
*  CALLED BY:                   TKwriteHeader()
*
*  ROUTINES CALLED:     
*       Routine Name            Description
*       ------------            -----------
*       allocateVdataMemory     This routine is used to allocate memory to vdata.
*       vdataWrite              This routine is used to write the vdata in HDF 
*                               format.
*
***********************************************************************************/

int writeL2A25Header(IO_HANDLE *sGranuleHandle, CLUTTER_FLAGS *sHeader)
{
  VDATA_LINK	*tempVdata = sGranuleHandle->cfgSTR->vGroup->vdata;
  uint8		*databuf, *ptr;
  int		i,j;

  /* Allocate memory for packing the vdata */
  databuf = allocMem(&tempVdata->vdataFields, CLUTFLAG_TBL_SIZE);
  ptr = databuf;

  for (i = 0; i < CLUTFLAG_TBL_SIZE; i++)
  {
    memcpy(ptr, &sHeader->clutFlag[i].mainlobeEdge, sizeof(int8));
    ptr += sizeof(int8);

    for (j = 0; j < 3; j++)
    {
    	memcpy(ptr, &sHeader->clutFlag[i].sidelobeRange[j], sizeof(int8));
    	ptr += sizeof(int8);
    }
  }

  if (VSwrite(tempVdata->vdataID, databuf, CLUTFLAG_TBL_SIZE, FULL_INTERLACE) == TK_HDF_FAIL)
  {
	/* VSwrite failed while writing header. */
	TKreportError(E_TK_VWRITEFAIL);
  }

  free(databuf);

  return(TK_SUCCESS);
}


/***********************************************************************************
*
*  FUNCTION STATEMENT:
*       readL2A25Header(IO_HANDLE *sGranuleHandle, CLUTTER_FLAGS *sHeader)
*
*  DESCRIPTION:
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       sGranuleHandle          A structure containing information about the file.
*                               sGranuleHandle is returned by TKopen().
*
*  OUTPUT PARAMETERS: 
*       Parameter Name          Description
*       --------------          -----------
*	sHeader			A structure contains the complete header data.
*
*  CALLED BY:                   TKreadHeader()
*
*  ROUTINES CALLED:     
*       Routine Name            Description
*       ------------            -----------
*       allocateVdataMemory     This routine is used to allocate memory to vdata.
*       vdataRead               This routine is used to read the vdata in HDF 
*                               format.
*
***********************************************************************************/

int readL2A25Header(IO_HANDLE *sGranuleHandle, CLUTTER_FLAGS *sHeader)
{
  VDATA_LINK	*tempVdata = sGranuleHandle->cfgSTR->vGroup->vdata;
  uint8		*databuf, *ptr;
  int		i,j;

  /* Allocate memory for unpacking the vdata */
  databuf = allocMem(&tempVdata->vdataFields, CLUTFLAG_TBL_SIZE);
  ptr = databuf;

  if (VSread(tempVdata->vdataID, databuf, CLUTFLAG_TBL_SIZE, FULL_INTERLACE) == TK_HDF_FAIL)
  {
	/* VSread failed while reading header. */
        TKreportError(E_TK_VREADFAIL);
  }


  for (i = 0; i < CLUTFLAG_TBL_SIZE; i++)
  {
    memcpy(&sHeader->clutFlag[i].mainlobeEdge, ptr, sizeof(int8));
    ptr += sizeof(int8);

    for (j = 0; j < 3; j++)
    {
    	memcpy(&sHeader->clutFlag[i].sidelobeRange[j], ptr, sizeof(int8));
    	ptr += sizeof(int8);
    }
  }

  free(databuf);

  return(TK_SUCCESS);
}


/***********************************************************************************
*
*  FUNCTION STATEMENT:
*       writeL1B21_L1C21Header(IO_HANDLE *sGranuleHandle, L1B21_L1C21_HEADER *sHeader) 
*
*  DESCRIPTION:
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       sGranuleHandle          A structure containing information about the file.
*                               sGranuleHandle is returned by TKopen().
*       sHeader			A structure contains the complete header data.
*
*  OUTPUT PARAMETERS:           None
*
*  CALLED BY:                   TKwriteHeader()
*
*  ROUTINES CALLED:     
*       Routine Name            Description
*       ------------            -----------
*       allocateVdataMemory     This routine is used to allocate memory to vdata.
*       vdataWrite              This routine is used to write the vdata in HDF 
*                               format.
*
***********************************************************************************/

int writeL1B21_L1C21Header(IO_HANDLE *sGranuleHandle, L1B21_L1C21_HEADER *sHeader)
{
  VDATA_LINK    *tempVdata = sGranuleHandle->cfgSTR->vGroup->vdata;
  uint8         *databuf, *ptr;
  int		i,j;

  /* Allocate memory for packing the vdata */
  databuf = allocMem(&tempVdata->vdataFields, 1);
  ptr = databuf;
  memcpy(ptr, &sHeader->prCalCoef.transCoef, sizeof(float32));
  ptr += sizeof(float32);
  memcpy(ptr, &sHeader->prCalCoef.receptCoef, sizeof(float32));
  ptr += sizeof(float32);
  for (i = 0; i < 16; i++)
  {
    memcpy(ptr, &sHeader->prCalCoef.fcifIOchar[i], sizeof(float32));
    ptr += sizeof(float32);
  }
  
  vdataWrite(&tempVdata, databuf);
  free(databuf);

  /* Allocate memory for packing the vdata */
  databuf = allocMem(&tempVdata->vdataFields, RAYHDR_TBL_SIZE);
  ptr = databuf;
  for (i = 0; i < RAYHDR_TBL_SIZE; i++)
  {
    memcpy(ptr, &sHeader->rayHdr[i].rayStart, sizeof(int16));
    ptr += sizeof(int16);
    memcpy(ptr, &sHeader->rayHdr[i].raySize, sizeof(int16));
    ptr += sizeof(int16);
    memcpy(ptr, &sHeader->rayHdr[i].angle, sizeof(float32));
    ptr += sizeof(float32);
    memcpy(ptr, &sHeader->rayHdr[i].startBinDist, sizeof(float32));
    ptr += sizeof(float32);
    memcpy(ptr, &sHeader->rayHdr[i].rainThres1, sizeof(float32));
    ptr += sizeof(float32);
    memcpy(ptr, &sHeader->rayHdr[i].rainThres2, sizeof(float32));
    ptr += sizeof(float32);
    memcpy(ptr, &sHeader->rayHdr[i].transAntenna, sizeof(float32));
    ptr += sizeof(float32);
    memcpy(ptr, &sHeader->rayHdr[i].recvAntenna, sizeof(float32));
    ptr += sizeof(float32);
    memcpy(ptr, &sHeader->rayHdr[i].onewayAlongTrack, sizeof(float32));
    ptr += sizeof(float32);
    memcpy(ptr, &sHeader->rayHdr[i].onewayCrossTrack, sizeof(float32));
    ptr += sizeof(float32);
    memcpy(ptr, &sHeader->rayHdr[i].eqvWavelength, sizeof(float32));
    ptr += sizeof(float32);
    memcpy(ptr, &sHeader->rayHdr[i].radarConst, sizeof(float32));
    ptr += sizeof(float32);
    memcpy(ptr, &sHeader->rayHdr[i].prIntrDelay, sizeof(float32));
    ptr += sizeof(float32);
    memcpy(ptr, &sHeader->rayHdr[i].rangeBinSize, sizeof(float32));
    ptr += sizeof(float32);
    memcpy(ptr, &sHeader->rayHdr[i].logAveOffset, sizeof(float32));
    ptr += sizeof(float32);
    memcpy(ptr, &sHeader->rayHdr[i].mainlobeEdge, sizeof(int8));
    ptr += sizeof(int8);
 
    for (j = 0; j < 3; j++)
    {
  	memcpy(ptr, &sHeader->rayHdr[i].sidelobeRange[j], sizeof(int8));
  	ptr += sizeof(int8);
    }
  }

  if (VSwrite(tempVdata->vdataID, databuf, RAYHDR_TBL_SIZE, FULL_INTERLACE) == TK_HDF_FAIL)
  {
	/* VSwrite failed while writing header */
	TKreportError(E_TK_VWRITEFAIL);
  }

  free(databuf);

  return(TK_SUCCESS);
}

/***********************************************************************************
*
*  FUNCTION STATEMENT:
*       readLL1B21_L1C21Header(IO_HANDLE *sGranuleHandle, L1B21_L1C21_HEADER *sHeader)
*
*  DESCRIPTION:
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       sGranuleHandle          A structure containing information about the file.
*                               sGranuleHandle is returned by TKopen().
*
*  OUTPUT PARAMETERS: 
*       Parameter Name          Description
*       --------------          -----------
*       sHeader			A structure contains the complete header data.
*
*  CALLED BY:                   TKreadHeader()
*
*  ROUTINES CALLED:     
*       Routine Name            Description
*       ------------            -----------
*       allocateVdataMemory     This routine is used to allocate memory to vdata.
*       vdataRead               This routine is used to read the vdata in HDF 
*                               format.
*
***********************************************************************************/

int readL1B21_L1C21Header(IO_HANDLE *sGranuleHandle, L1B21_L1C21_HEADER *sHeader)
{
  VDATA_LINK    *tempVdata = sGranuleHandle->cfgSTR->vGroup->vdata;
  uint8         *databuf, *ptr;
  int		i,j;

  /* Allocate memory for unpacking the vdata */
  databuf = allocMem(&tempVdata->vdataFields, 1);
  ptr = databuf;

  /* Unpack the content of databuf */
  vdataRead(&tempVdata, ptr);
  memcpy(&sHeader->prCalCoef.transCoef, ptr, sizeof(float32));
  ptr += sizeof(float32);
  memcpy(&sHeader->prCalCoef.receptCoef, ptr, sizeof(float32));
  ptr += sizeof(float32);
  for (i = 0; i < 16; i++)
  { 
    memcpy(&sHeader->prCalCoef.fcifIOchar[i], ptr, sizeof(float32));
    ptr += sizeof(float32);
  }

  free(databuf);


  /* Allocate memory for unpacking the vdata */
  databuf = allocMem(&tempVdata->vdataFields, RAYHDR_TBL_SIZE);
  ptr = databuf;

  if (VSread(tempVdata->vdataID, databuf, RAYHDR_TBL_SIZE, FULL_INTERLACE) == TK_HDF_FAIL)
  {
	/* VSread failed while reading header */
	TKreportError(E_TK_VREADFAIL);
  }

  /* Unpack the content of databuf */
  for (i = 0; i < RAYHDR_TBL_SIZE; i++)
  {
    memcpy(&sHeader->rayHdr[i].rayStart, ptr, sizeof(int16));
    ptr += sizeof(int16);
    memcpy(&sHeader->rayHdr[i].raySize, ptr, sizeof(int16));
    ptr += sizeof(int16);
    memcpy(&sHeader->rayHdr[i].angle, ptr, sizeof(float32));
    ptr += sizeof(float32);
    memcpy(&sHeader->rayHdr[i].startBinDist, ptr, sizeof(float32));
    ptr += sizeof(float32);
    memcpy(&sHeader->rayHdr[i].rainThres1, ptr, sizeof(float32));
    ptr += sizeof(float32);
    memcpy(&sHeader->rayHdr[i].rainThres2, ptr, sizeof(float32));
    ptr += sizeof(float32);
    memcpy(&sHeader->rayHdr[i].transAntenna, ptr, sizeof(float32));
    ptr += sizeof(float32);
    memcpy(&sHeader->rayHdr[i].recvAntenna, ptr, sizeof(float32));
    ptr += sizeof(float32);
    memcpy(&sHeader->rayHdr[i].onewayAlongTrack, ptr, sizeof(float32));
    ptr += sizeof(float32);
    memcpy(&sHeader->rayHdr[i].onewayCrossTrack, ptr, sizeof(float32));
    ptr += sizeof(float32);
    memcpy(&sHeader->rayHdr[i].eqvWavelength, ptr, sizeof(float32));
    ptr += sizeof(float32);
    memcpy(&sHeader->rayHdr[i].radarConst, ptr, sizeof(float32));
    ptr += sizeof(float32);
    memcpy(&sHeader->rayHdr[i].prIntrDelay, ptr, sizeof(float32));
    ptr += sizeof(float32);
    memcpy(&sHeader->rayHdr[i].rangeBinSize, ptr, sizeof(float32));
    ptr += sizeof(float32);
    memcpy(&sHeader->rayHdr[i].logAveOffset, ptr, sizeof(float32));
    ptr += sizeof(float32);
    memcpy(&sHeader->rayHdr[i].mainlobeEdge, ptr, sizeof(int8));
    ptr += sizeof(int8);

    for (j = 0; j < 3; j++)
    {
      memcpy(&sHeader->rayHdr[i].sidelobeRange[j], ptr, sizeof(int8));
      ptr += sizeof(int8);
    }
  }

  free(databuf);

  return(TK_SUCCESS);
}

/***********************************************************************************
*
*  FUNCTION STATEMENT:
*       setScalingFactors_1B21(VGROUP_LINK *vgroup)
*
*  DESCRIPTION:
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       vgroup                  List of vgroups.  The list of SDSs are retrieve from
*                               this node.
*
*  OUTPUT PARAMETERS:  None.
*
*  CALLED BY:                   setSDSattributes().
*
*  ROUTINES CALLED:     
*       Routine Name            Description
*       ------------            -----------
*       SDsetcal                This routine adds the scaling factor and offset 
*                               to the SDS.
*       TKreportWarning         This routine is called for warnings and informational
*                               messages.  Calling this routine will send an error
*                               message to the central log file and return control to 
*                               the calling routine.
*
***********************************************************************************/
int setScalingFactors_1B21 ( VGROUP_LINK *vgroup )
{

  int status = TK_SUCCESS;
  SDS_LINK *sds;

  if ( vgroup != NULL )
    {
      sds = vgroup->sds;
      while ( sds != NULL )
	{
	  if ( !strcmp ( sds->name, "systemNoise" ) )
	    status = SDsetcal ( sds->sdsID, (float64) L1B21_SYSTEMNOISE, 
			       (float64) 0, (float64) 0, (float64) 0, 
			       DFNT_INT16 );
	  else if ( !strcmp ( sds->name, "normalSample" ) )
	    status = SDsetcal ( sds->sdsID, (float64) L1B21_NORMALSAMPLE, 
			       (float64) 0, (float64) 0, (float64) 0, 
			       DFNT_INT16 );
	  else if ( !strcmp ( sds->name, "osSurf" ) )
	    status = SDsetcal ( sds->sdsID, (float64) L1B21_OSSURF, 
			       (float64) 0, (float64) 0, (float64) 0, 
			       DFNT_INT16 );
	  else if ( !strcmp ( sds->name, "osRain" ) )
	    status = SDsetcal ( sds->sdsID, (float64) L1B21_OSRAIN, 
			       (float64) 0, (float64) 0, (float64) 0, 
			       DFNT_INT16 );

	  if ( status == TK_HDF_FAIL )
	    TKreportWarning ( E_TK_SETCALINFO );
	  sds = sds->next;
	}
      setScalingFactors_1B21 ( vgroup->subVgroup );
      setScalingFactors_1B21 ( vgroup->next );
    }
  return status;
}

/***********************************************************************************
*
*  FUNCTION STATEMENT:
*       setScalingFactors_2A21(VGROUP_LINK *vgroup)
*
*  DESCRIPTION:
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       vgroup                  List of vgroups.  The list of SDSs are retrieve from
*                               this node.
*
*  OUTPUT PARAMETERS:  None.
*
*  CALLED BY:                   setSDSattributes().
*
*  ROUTINES CALLED:     
*       Routine Name            Description
*       ------------            -----------
*       SDsetcal                This routine adds the scaling factor and offset 
*                               to the SDS.
*       TKreportWarning         This routine is called for warnings and informational
*                               messages.  Calling this routine will send an error
*                               message to the central log file and return control to 
*                               the calling routine.
*
***********************************************************************************/
int setScalingFactors_2A21 ( VGROUP_LINK *vgroup )
{

  int status = TK_SUCCESS;
  SDS_LINK *sds;

  if ( vgroup != NULL )
    {
      sds = vgroup->sds;
      while ( sds != NULL )
	{
	  if ( !strcmp ( sds->name, "sigmaZero" ) )
	    status = SDsetcal ( sds->sdsID, (float64) L2A21_SIGMAZERO, 
			       (float64) 0, (float64) 0, (float64) 0, 
			       DFNT_INT16 );
	  else if ( !strcmp ( sds->name, "pathAtten" ) )
	    status = SDsetcal ( sds->sdsID, (float64) L2A21_PATHATTEN, 
			       (float64) 0, (float64) 0, (float64) 0, 
			       DFNT_INT16 );
	  else if ( !strcmp ( sds->name, "incAngle" ) )
	    status = SDsetcal ( sds->sdsID, (float64) L2A21_INCANGLE, 
			       (float64) 0, (float64) 0, (float64) 0, 
			       DFNT_INT16 );

	  if ( status == TK_HDF_FAIL )
	    TKreportWarning ( E_TK_SETCALINFO );
	  sds = sds->next;
	}
      setScalingFactors_2A21 ( vgroup->subVgroup );
      setScalingFactors_2A21 ( vgroup->next );
    }
  return status;
}

/***********************************************************************************
*
*  FUNCTION STATEMENT:
*       setScalingFactors_2A23(VGROUP_LINK *vgroup)
*
*  DESCRIPTION:
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       vgroup                  List of vgroups.  The list of SDSs are retrieve from
*                               this node.
*
*  OUTPUT PARAMETERS:  None.
*
*  CALLED BY:                   setSDSattributes().
*
*  ROUTINES CALLED:     
*       Routine Name            Description
*       ------------            -----------
*       SDsetcal                This routine adds the scaling factor and offset 
*                               to the SDS.
*       TKreportWarning         This routine is called for warnings and informational
*                               messages.  Calling this routine will send an error
*                               message to the central log file and return control to 
*                               the calling routine.
*
***********************************************************************************/
int setScalingFactors_2A23 ( VGROUP_LINK *vgroup, int realTimeFlag )
{

  int status = TK_SUCCESS;
  SDS_LINK *sds;

  if ( vgroup != NULL )
    {
      sds = vgroup->sds;
      while ( sds != NULL )
	{
	  if ( ( !strcmp ( sds->name, "geolocation" ) ) && ( realTimeFlag ) )
	    status = SDsetcal ( sds->sdsID, (float64) GEOLOC_SCALE, (float64) 0,
			       (float64) 0, (float64) 0, DFNT_INT16 );

	  if ( status == TK_HDF_FAIL )
	    TKreportWarning ( E_TK_SETCALINFO );
	  sds = sds->next;
	}
      setScalingFactors_2A23 ( vgroup->subVgroup, realTimeFlag );
      setScalingFactors_2A23 ( vgroup->next, realTimeFlag );
    }
  return status;
}

/***********************************************************************************
*
*  FUNCTION STATEMENT:
*       setScalingFactors_2A25(VGROUP_LINK *vgroup)
*
*  DESCRIPTION:
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       vgroup                  List of vgroups.  The list of SDSs are retrieve from
*                               this node.
*
*  OUTPUT PARAMETERS:  None.
*
*  CALLED BY:                   setSDSattributes().
*
*  ROUTINES CALLED:     
*       Routine Name            Description
*       ------------            -----------
*       SDsetcal                This routine adds the scaling factor and offset 
*                               to the SDS.
*       TKreportWarning         This routine is called for warnings and informational
*                               messages.  Calling this routine will send an error
*                               message to the central log file and return control to 
*                               the calling routine.
*
***********************************************************************************/
int setScalingFactors_2A25 ( VGROUP_LINK *vgroup, int realTimeFlag )
{

  int status = TK_SUCCESS;
  SDS_LINK *sds;

  if ( vgroup != NULL )
    {
      sds = vgroup->sds;
      while ( sds != NULL )
	{
	  if ( ( !strcmp ( sds->name, "geolocation" ) ) && ( realTimeFlag ) )
	    status = SDsetcal ( sds->sdsID, (float64) GEOLOC_SCALE, (float64) 0,
			       (float64) 0, (float64) 0, DFNT_INT16 );
	  else if ( !strcmp ( sds->name, "rain" ) )
	    status = SDsetcal ( sds->sdsID, (float64) L2A25_RAIN, (float64) 0, 
			       (float64) 0, (float64) 0, DFNT_INT16 );
	  else if ( !strcmp ( sds->name, "correctZFactor" ) )
	    status = SDsetcal ( sds->sdsID, (float64) L2A25_CORRECTZFACTOR, 
			       (float64) 0, (float64) 0, (float64) 0, 
			       DFNT_INT16 );
	  else if ( !strcmp ( sds->name, "attenParmAlpha" ) )
	    status = SDsetcal ( sds->sdsID, (float64) L2A25_ATTENPARMALPHA, 
			       (float64) 0, (float64) 0, (float64) 0, 
			       DFNT_INT16 );
	  else if ( !strcmp ( sds->name, "ZRParmA" ) )
	    status = SDsetcal ( sds->sdsID, (float64) L2A25_ZRPARMA, 
			       (float64) 0, (float64) 0, (float64) 0, 
			       DFNT_INT16 );
	  else if ( !strcmp ( sds->name, "ZRParmB" ) )
	    status = SDsetcal ( sds->sdsID, (float64) L2A25_ZRPARMB, 
			       (float64) 0, (float64) 0, (float64) 0, 
			       DFNT_INT16 );
	  else if ( !strcmp ( sds->name, "rainAve" ) )
	    status = SDsetcal ( sds->sdsID, (float64) L2A25_RAINAVE, 
			       (float64) 0, (float64) 0, (float64) 0, 
			       DFNT_INT16 );
	  else if ( !strcmp ( sds->name, "precipWaterSum" ) )
	    status = SDsetcal ( sds->sdsID, (float64) L2A25_LIQWATERSUM,
			       (float64) 0, (float64) 0, (float64) 0, 
			       DFNT_INT16 );
	  else if ( !strcmp ( sds->name, "attenParmBeta" ) )
	    status = SDsetcal ( sds->sdsID, (float64) L2A25_ATTENPARMBETA, 
			       (float64) 0, (float64) 0, (float64) 0, 
			       DFNT_INT16 );
	  else if ( !strcmp ( sds->name, "weightW" ) )
	    status = SDsetcal ( sds->sdsID, (float64) L2A25_WEIGHTW, 
			       (float64) 0, (float64) 0, (float64) 0, 
			       DFNT_INT16 );
	  else if ( ( !strcmp ( sds->name, "nearSurfRain" ) ) && ( realTimeFlag ) )
	    status = SDsetcal ( sds->sdsID, (float64) L2A25_NEARSURFRAIN,
			       (float64) 0, (float64) 0, (float64) 0,
			       DFNT_INT16 );
	  else if ( ( !strcmp ( sds->name, "e_SurfRain" ) ) && ( realTimeFlag ) )
	    status = SDsetcal ( sds->sdsID, (float64) L2A25_E_SURFRAIN,
			       (float64) 0, (float64) 0, (float64) 0,
			       DFNT_INT16 );
	  else if ( ( !strcmp ( sds->name, "nearSurfZ" ) ) && ( realTimeFlag ) )
	    status = SDsetcal ( sds->sdsID, (float64) L2A25_NEARSURFZ,
			       (float64) 0, (float64) 0, (float64) 0,
			       DFNT_INT16 );
	  if ( status == TK_HDF_FAIL )
	    TKreportWarning ( E_TK_SETCALINFO );
	  sds = sds->next;
	}
      setScalingFactors_2A25 ( vgroup->subVgroup, realTimeFlag );
      setScalingFactors_2A25 ( vgroup->next, realTimeFlag );
    }
  return status;
}

/***********************************************************************************
*
*  FUNCTION STATEMENT:
*       setScalingFactors_2B31(VGROUP_LINK *vgroup)
*
*  DESCRIPTION:
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       vgroup                  List of vgroups.  The list of SDSs are retrieve from
*                               this node.
*
*  OUTPUT PARAMETERS:  None.
*
*  CALLED BY:                   setSDSattributes().
*
*  ROUTINES CALLED:     
*       Routine Name            Description
*       ------------            -----------
*       SDsetcal                This routine adds the scaling factor and offset 
*                               to the SDS.
*       TKreportWarning         This routine is called for warnings and informational
*                               messages.  Calling this routine will send an error
*                               message to the central log file and return control to 
*                               the calling routine.
*
***********************************************************************************/
int setScalingFactors_2B31 ( VGROUP_LINK *vgroup )
{

  int status = TK_SUCCESS;
  SDS_LINK *sds;

  if ( vgroup != NULL )
    {
      sds = vgroup->sds;
      while ( sds != NULL )
	{
	  if ( !strcmp ( sds->name, "dHat" ) )
	    status = SDsetcal ( sds->sdsID, (float64) L2B31_DHAT, (float64) 0, 
			       (float64) 0, (float64) 0, DFNT_INT16 );
	  else if ( !strcmp ( sds->name, "sigmaDHat" ) )
	    status = SDsetcal ( sds->sdsID, (float64) L2B31_SIGMADHAT, 
			       (float64) 0,  (float64) 0, (float64) 0, 
			       DFNT_INT16 );
	  else if ( !strcmp ( sds->name, "epsilon" ) )
	    status = SDsetcal ( sds->sdsID, (float64) L2B31_EPSILON, 
			       (float64) 0, (float64) 0, (float64) 0, 
			       DFNT_INT16 );
	  else if ( !strcmp ( sds->name, "sigmaEpsilon" ) )
	    status = SDsetcal ( sds->sdsID, (float64) L2B31_SIGMAEPSILON, 
			       (float64) 0, (float64) 0, (float64) 0, 
			       DFNT_INT16 );
	  else if ( !strcmp ( sds->name, "rHat" ) )
	    status = SDsetcal ( sds->sdsID, (float64) L2B31_RHAT, (float64) 0, 
			       (float64) 0, (float64) 0, DFNT_INT16 );
	  else if ( !strcmp ( sds->name, "sigmaRHat" ) )
	    status = SDsetcal ( sds->sdsID, (float64) L2B31_SIGMARHAT, 
			       (float64) 0, (float64) 0, (float64) 0, 
			       DFNT_INT16 );
	  else if ( !strcmp ( sds->name, "pia" ) )
	    status = SDsetcal ( sds->sdsID, (float64) L2B31_PIA, (float64) 0, 
			       (float64) 0, (float64) 0, DFNT_INT16 );
	  else if ( !strcmp ( sds->name, "sigmaPIA" ) )
	    status = SDsetcal ( sds->sdsID, (float64) L2B31_SIGMAPIA, 
			       (float64) 0, (float64) 0, (float64) 0, 
			       DFNT_INT16 );
	  else if ( !strcmp ( sds->name, "tmiPIA" ) )
	    status = SDsetcal ( sds->sdsID, (float64) L2B31_TMIPIA, 
			       (float64) 0, (float64) 0, (float64) 0, 
			       DFNT_INT16 );
	  else if ( !strcmp ( sds->name, "sigmaTMIpia" ) )
	    status = SDsetcal ( sds->sdsID, (float64) L2B31_SIGMATMIPIA, 
			       (float64) 0, (float64) 0, (float64) 0, 
			       DFNT_INT16 );
/*
	  else if ( !strcmp ( sds->name, "rrSurf" ) )
	    status = SDsetcal ( sds->sdsID, (float64) L2B31_RRSURF, 
			       (float64) 0, (float64) 0, (float64) 0, 
			       DFNT_INT16 );
*/
	  else if ( !strcmp ( sds->name, "sigmaRRsurf" ) )
	    status = SDsetcal ( sds->sdsID, (float64) L2B31_SIGMARRSURF, 
			       (float64) 0, (float64) 0, (float64) 0, 
			       DFNT_INT16 );
	  if ( status == TK_HDF_FAIL )
	    TKreportWarning ( E_TK_SETCALINFO );
	  sds = sds->next;
	}
      setScalingFactors_2B31 ( vgroup->subVgroup );
      setScalingFactors_2B31 ( vgroup->next );
    }
  return status;
}
