/* DEVELOPMENT HISTORY */
/* Revision 1.159  2001/08/09 17:18:20  pkolbe
 * yet another another altitude increase quote prod version change
 *
 * Revision 1.158  2001/08/09 17:01:12  pkolbe
 * yet another altitude increase change; namely the product version needs
 * quotes around it
 *
 * Revision 1.157  2001/07/18 21:05:53  pkolbe
 * altitude increase changes
 *
 * Revision 1.155  1999/12/01 16:38:07  pkolbe
 * added empty granule gran input query fix
 *
 * Revision 1.154  1999/11/10 19:56:51  pkolbe
 * changed correct null elements to allow gv to write Software Version to metadata
 *
 * Revision 1.153  1999/10/26 17:52:58  pkolbe
 * added better error reporting when a failure occurs in an sds or vdata
 * read/write (now, not just a failure is reported, but what specific
 * sds or vdata it was that failed )
 *
 * Revision 1.152  1999/10/25 14:07:55  pkolbe
 * changed TKreportWarning(W_TK_NORECORDSINDB) to TKreportError(W_TK_NORECORDSINDB)
 * because granules were being shipped to the DAAC in operations which had
 * -999 in the lon of max lat in metadata
 *
 * Revision 1.151  1999/10/25 13:47:39  pkolbe
 * added more descriptive error message when a null hdf file is opened
 *
 * Revision 1.150  1999/10/05 14:05:03  pkolbe
 * added writing of tk version into metadata for real time case
 *
 * Revision 1.149  1999/09/22 19:10:49  pkolbe
 * changed the setting of lonMaxLat and tmiRolloverCoef metadata
 * to copy up from level 1
 *
 * Revision 1.148  1999/09/13 18:12:42  pkolbe
 * finished backward compatability code changes
 *
 * Revision 1.147  1999/09/08 18:51:44  pkolbe
 * changed metadata element ssmiAdjust to tmiRolloverCoef
 *
 * Revision 1.146  1999/08/31 20:40:58  pkolbe
 * removed tk backward compatability/version checking code changes (for now,
 * anyway)
 *
 * Revision 1.145  1999/08/27 19:24:45  pkolbe
 * added tk backward compatability/version checking code changes
 *
 * Revision 1.144  1999/08/24 15:16:44  pkolbe
 * changed maxValChan and minValChan initialization for cases TMI and VIRS
 * in initialize metadata to the default -9999  (.9 for float) as these will
 * be initialized now from the 1b01 and 1b11 algorithms themselves, per
 * conversation with Yong Zhang, 8/15/99
 *
 * Revision 1.143  1999/08/13 17:50:59  pkolbe
 * added dynamic scaling code
 *
 * Revision 1.142  1999/08/10 17:40:28  slywczak
 * Modified setSDSdimensionNames.  Added parenthesis around two if statements
 * that were checking for the eof conditions.  The gcc compiler on Linux was
 * not able to compile the code with the parenthesis.
 *
 * Revision 1.141  1999/06/24 13:51:31  pkolbe
 * yet another change to maxValChan in case TK_VIRS of initMetadata
 *
 * Revision 1.140  1999/06/23 21:46:00  pkolbe
 * yet even further fix of DR647 needed; changed DateofGenerationOfInputFiles
 * to be derved from the input processing date not the processing date
 * (processing date is null causing a core dump)
 *
 * Revision 1.139  1999/04/29 18:23:23  pkolbe
 * created new 1b01 max val channel limits
 *
 * Revision 1.138  1999/04/29 17:36:56  pkolbe
 * further fix of DR647 needed; changed DateofGenerationOfInputFiles
 * to be derved from the processing date not the collection date
 *
 * Revision 1.137  1999/01/08 13:03:16  slywczak
 * Added real time datatypes to the routine findAlgorithmTKID.
 *
 * Revision 1.136  1998/12/11 00:31:12  pkolbe
 * added setSDSdimensionNames
 *
 * Revision 1.135  1998/09/17 17:14:36  pkolbe
 * fixed DR 647 where the Date of generation of input files was one month
 * previous because the month starts from 0 not 1; therefore a one line
 * code change is all that was needed (increment the month ++ )
 *
 * Revision 1.134  1998/09/15 15:43:19  slywczak
 * Modified routine findAlgorithmID.  Added the real-time data types.
 *
 * Revision 1.133  1998/07/17 13:21:13  slywczak
 * Modified function setSDSattributes.  Removed the call to the scaling function
 * setScalingFactors_3A54.
 *
 * Revision 1.132  1998/07/13 19:02:18  pkolbe
 * added N,S,E,W BoundingCoordinates to real time metadata;
 * removed lon of max lat and alg version from real time metadata
 * because they are values that come from the database and the real
 * time system will not be accessing the database
 *
 * Revision 1.131  1998/07/09 15:01:19  slywczak
 * Modified the routine setSDSattributes.  Changed the calling API for
 * setScalingFactors_1B11, setScalingFactors_2A12, setScalingFactors_2A23,
 * and setScalingFactors_2A25.  Added the realtime flag to the calling sequence.
 *
 * Revision 1.130  1998/07/08 22:07:43  pkolbe
 * added more real time metadata code changes
 *
 * Revision 1.129  1998/07/08 15:57:03  pkolbe
 * added real time subsetting of core and ps metadata in the initMetadata and
 * finalizeMetadata routines
 *
 * Revision 1.128  1998/06/26 17:28:16  slywczak
 * Modified routines findAlgorithmDataType, prodType, and instType.  Added the
 * new realtime algorithm ID's to fileAlgorithmDataType.  Added the new realtime
 * algorithm ID's to prodType and added the product IDs to instType (returns
 * the instrument).  Realtime algorithm IDs are 1B11, 2A12, 2A23, 2A25r1, and
 * 2A25r2.
 *
 * Revision 1.127  1998/05/21 17:52:59  slywczak
 * Modifed the following functions:  openInputVdata, openInputSdata, vdataRead,
 * sdsRead, detachObjects, correctNULLElements, and initializeMetadata.  For
 * backwards compatibility support, the first 5 function were modified.  Now, when
 * an SDS or Vdata does not exist in the file, the toolkit will not error out.
 * The last two function were modified to correct an error in the metadata
 * elements ssmiAdjust.
 *
 * Revision 1.126  1998/05/14 18:41:01  slywczak
 * Modified the function openHDF, startSD, and parse_pvl.  Change the references
 * to UTreportWarning to TKreportWarningParm in openHDF and startSD.  With this
 * change, there is no dependency on the UT library.  Changed the warning message
 * (W_TK_STRSTR) in parse_pvl to report a more accurate message - the metadata
 * field does not exist and the name of the metadata field that couldn't be found.
 *
 * Revision 1.125  1998/05/05 16:16:55  slywczak
 * Modified initMetadata and finalizeMetadata.  Fixed a reference to a NULL
 * pointer in determine whether the file is a temp file or not.
 *
 * Revision 1.124  1998/04/27 19:09:38  pkolbe
 * added ssmi adjust coefficient metadata element code
 *
 * Revision 1.123  1998/04/27 18:31:31  slywczak
 * Modified routine initMetadata.  Changed initMetadata so that the  metadata
 * can be read for temporary files.
 *
 * Revision 1.122  1998/04/27 18:15:08  slywczak
 * Modified routine initMetadata and finalizeMetadata.  If the ending of the
 * filename is not HDF, these two routine will not report errors.  This will
 * prohibit erroneous errors from being reported to OpenView when using the HDF
 * files as temporary files.
 *
 * Revision 1.121  1998/04/23 16:23:35  slywczak
 * Modified the routine initMetadata.  Changed the default values for
 * MaximumValidValueofChannel from -110 to -130 and MinimumValidValueofChannel
 * from -20 to -30.
 *
 * Revision 1.120  1998/04/23 16:15:58  slywczak
 * Added new routine createPlanetaryGrid2.  This routine create the planetary
 * grid structure for the 3A25 product.  Had to modify the generic routine, since
 * the planetary grid 2 uses different values than the standard.  The lat/lon
 * boundary is +/- 37 for the planetary grid 2.
 *
 * Revision 1.119  1998/04/16 17:23:07  slywczak
 * Modified setSDSAttributes routine.  The routines called for 1B21, 2A21, and
 * 2B31 where incorrect - all called 2A25.  Correct the routine names.  These
 * routines set the scale factor and the offset in the HDF file.
 *
 * Revision 1.118  1998/04/13 19:17:12  slywczak
 * Modified routine finalizeMetadata.  Change the error checking on the query from
 * IM.  If the IM query is not successful, the toolkit will call TKreportError
 * instead of TKreportWarning.  This will allow the algorithm to error out and not
 * send invalid granules to the DAAC.
 *
 * Revision 1.117  1998/02/11 19:37:23  slywczak
 * Modified the following routine initMetadata, correctNULLElements,
 * finalizeMetadata, metaFree, and createMetadataPVL.  Modified initMetadata
 * to initialize the Solar Gains elements with a string (4 values) of -9999.9.
 * Modified correctNULLElements to write out the default string, if the data
 * type is not TK_L1B_01.  Modified finalizeMetadata to write the Solar Gains
 * element to the file upon calling TKclose.  Modified metaFree to free that
 * metadata element when no longer needed.  And, modified createMetadataPVL to
 * create the PVL string for the Solar Gains metadata element.
 *
 * Revision 1.116  1998/01/14 14:27:52  slywczak
 * Added new routine findAlgorithmTKID.  Modified routine readpvl_from_hdffile.
 * Passed the sds id into the routine instead of the IO structure.  This should
 * make the routine more flexible.
 *
 * Revision 1.115  1997/11/19 14:39:13  mack
 * Changed TSDIS_TK_ENV to TSU_ENV
 *
 * Revision 1.114  1997/10/30 21:39:35  slywczak
 * Modified function correctNULLElements.  Changed the overwriting so that the
 * new PR elements could be written at Level 2 and 3.
 *
 * Revision 1.113  1997/10/29 17:40:05  slywczak
 * Modified routine correctNULLElements, finalizeMetadata, and createPVLMetadata.
 * (1)  Made the radar wavelength variable available at L2 and L3;  (2)  Changed
 * the order of the metadata to make the parsing more flexible; and (3)  Checked
 * the file on start/stop times/dates for temporary hdf files (ends in .hdf.tmp).
 *
 * Revision 1.112  1997/10/16 20:08:08  slywczak
 * Modified routine updateStartStopTimesInGranuleMetadata.  Initialized the status
 * flag to TK_SUCCESS.  When the L1B process was occurring, this flag was getting
 * initialized to random values.
 *
 * Revision 1.111  1997/10/14 19:29:56  slywczak
 * Modified finalizeMetadata and updateStartandStopTimesInGranuleMetadata.
 * Moved the call to updateStartandStopTimesInGranuleMetadata in finalizeMetadata
 * so that this call is made regardless whether there is an input exists.  Also,
 * increased the error handling in updateStartandStopTimesInGranuleMetadata.
 *
 * Revision 1.110  1997/10/06 15:54:49  slywczak
 * Modified the routine initMetadata.  Removed the building of the input file
 * name string, generation date, and input data center for Level 3 algorithms.
 *
 * Revision 1.109  1997/09/29 22:19:53  slywczak
 * Modified function initMetadata.  Changed the initialization of the following
 * metadata elements from TK_TRUE to TK_FALSE:  meanMotion, orbitAdjustFlag,
 * radarName, numVOS, anomalyFlag, radarSpacingX, radarSpacingY, radarSpacingZ,
 * dzCal, gvL1CScale, and alpha.
 *
 * Revision 1.108  1997/09/15 20:47:37  slywczak
 * Modified function initMetadata and finializeMetadata.  Added checks for CSI and
 * QL products so that the query is not performed in initMetadata and the database
 * is not updated in finializeMetadata.
 *
 * Revision 1.107  1997/09/08 20:09:37  slywczak
 * Added new function findAlgorithmDataType to determine the algorithm id from
 * the product ID instead of the filename.  Modified functions openHDF and SDstart
 * to return an error code instead of failing when a file does not exist.
 *
 * Revision 1.106  1997/08/27 19:17:55  slywczak
 * Modified the routine initMetadata.  Added default values for the ECS
 * Description Metadata field for the GV L1B and L1C algorithms.
 *
 * Revision 1.105  1997/08/01 12:55:36  slywczak
 * Modified the following routines:  initMetadata, correctNULLElements,
 * finalizeMetadata, metaFree, and createMetadataPVL.  Removed the definitions
 * for the following elements:  Temperature Telemetry Abnormal, Troubled SSPA
 * Units, and Troubled LNA Units.
 *
 * Revision 1.104  1997/07/15 13:01:22  slywczak
 * Modified routine initMetadata.  Updated the parameter Longitude of Maximum
 * Latitude from NULL to -9999.9.
 *
 * Revision 1.103  1997/06/27 14:11:18  slywczak
 * Modified the routine correctNULLElements.  Change the setting of date/time to
 * NULL for L3 algorithms.
 *
 * Revision 1.102  1997/06/26 19:27:51  slywczak
 * Updated code based on the results of Purify.  Initialized structures.
 *
 * Revision 1.101  1997/06/23 22:12:58  slywczak
 * Modified the function correctNULLElements.  Changed the code so that the
 * min/max values/units also applys to GV Level 2.
 *
 * Revision 1.100  1997/06/23 13:39:55  slywczak
 * Modified the function initMetadata.  Changed the defaults from integer to
 * floating point. (-9999 -> -9999.9)
 *
 * Revision 1.99  1997/06/20 19:58:18  slywczak
 * Modified metaFree and correctNULLElements.  Based on the results from Purify,
 * memory had to be freed.  Added the free statements to metaFree.  In addition,
 * modifed the default metadata values so that all Level 3 products will
 * put 9999/99/99 for begin/end date and 99:99:99 for begin/end time by default.
 *
 * Revision 1.98  1997/06/19 16:00:26  slywczak
 * Modified function correctNULLElements.  At the request of the TRMM Office, the
 * GV Runtime Option will apply to all Levels of GV products instead of just
 * Level 1.  The reset to NULL at Level 2 and 3 was removed.
 *
 * Revision 1.97  1997/06/18 14:46:29  slywczak
 * Modified routine InitMetadata.  Modified the initialization of the Epoch Date
 * and Time for the GV elements, and added the initialization for algorithm 2B-31.
 *
 * Revision 1.96  1997/06/11 20:04:26  slywczak
 * Modified function initMetadata.  The previous version attempted to create
 * browse names for CSI and QL products.  Added logic to test for the existance
 * of these products and not create browse names for them.
 *
 * Revision 1.95  1997/05/29 16:42:28  slywczak
 * Added the following new routines:  prodType, createSwathRadarVdata,
 * createRadar3A25Metadata, createStructurePVL, createSwathStructureMetadata,
 * createL3StructureMetadata, createSwathRadarMetadata, attachSwathRadarVdata,
 * and writeSwathRadarMetadata.  Added new metadata elements for the PR Level
 * 1 algorithms.
 *
 * Revision 1.94  1997/05/16 14:24:52  slywczak
 * Added the routine setSDSattributes.  This routine is the driver for determining
 * the product ID and calling the appropriate routine to insert the scaling
 * factors into the HDF  file.
 *
 * Revision 1.93  1997/05/14 19:33:08  slywczak
 * Modified the following routines:  initMetadata and finalizeMetadata.  Added
 * the following new  routines:  removeIMSpaces, correctNULLElements, and
 * buildInputFileStrings.  Changed the initialize metadata routine to set the
 * metadata with correct initial values and changed the processing of the
 * query from IM.  Change the finalize routine to set any elements that are not
 * required for that product to their "missing" values.
 *
 * Revision 1.92  1997/04/23 14:07:30  slywczak
 * Modified routine resetVgroup.  Removed the recursive call to vgroup->subVgroup,
 * since this should have been the initial group to reset.  This routine is
 * initially called at the subVgroup Level.
 *
 * Revision 1.91  1997/04/22 15:16:50  slywczak
 * Modified routine initMetadata.  Modified the error checking of the IM query
 * GET_INPUT_GRANULE.  Previously, initMetadata only checked if the return status
 * was success or failure, and didn't check for no rows returned from the
 * database.  The W_TK_NORECORDSINDB error mnemonic was added as a possible
 * return.
 *
 * Revision 1.90  1997/02/28 03:50:43  slywczak
 * Update the following routines:  initMetadata.  Found more instances where
 * colons (:) were used as list separators instead of commas (,).  EOSDIS
 * requires the commas be used to separate elements of a list.
 *
 * Revision 1.89  1997/02/27 21:00:33  slywczak
 * Update the following routines:  initMetadata and mdcpyLong.  Changed the
 * metadata parameter from TK_INPUT_ID to TK_INPUT_FILES.  In the previous
 * version of the toolkit, it was called TK_INPUT_FILES, and changed to
 * TK_INPUT_ID by mistake.  To maintain compatibility, it has been changed back
 * to the original name. (Refer to IO.h for more information).
 *
 * Revision 1.88  1997/02/27 20:44:01  slywczak
 * Updated routine initMetadata.  Replaced the colons by commas in the following
 * Metadata elements:  Bad/Missing Pixels, Data Accuracy, Max. Valid Channel,
 * Min. Valid Channel, Sensor Alignment, and Scan Path.  Using the comma as a
 * list separator is an EOSDIS requirement.
 *
 * Revision 1.87  1997/02/21 22:37:57  piper
 * Temporarily comment out DMmakeGranuleName until permanent solution found.
 *
 * Revision 1.86  1997/02/13 17:51:46  piper
 * Fixed problem with missing values not being written to output files.
 *
 * Revision 1.85  1997/02/04 16:45:39  amak
 * Modified initMetadata().  Skipped the access of TSDIS database when the
 * filemode is TK_READ_ONLY in TSDIS_TK_ENV.
 *
 * Revision 1.84  1997/01/10 20:23:40  amak
 * Modified initMetadata().  Added cdate as one of the criterias for DM to
 * generate browseName.
 *
 * Revision 1.83  1997/01/07 22:43:53  amak
 * Modified initMetadata().
 *
 * Revision 1.82  1997/01/03 22:37:13  amak
 * Called TKreportError instead of TKreportWarning in mdcpy() and mdcpyLong().
 *
 * Revision 1.81  1997/01/02  15:32:04  amak
 * Added prologue
 *
 * Revision 1.80  1996/12/24 14:35:45  amak
 * Incorporated walkthrough comments.
 * - Changed '<' and '>' to double quotes around the #include statements.
 * - Call TKreportWarning to handle failure to IMopen, IMquery and IMallocateRec
 *   in initMetadata().
 * - Cleaned up metadata initialization logic in initMetadata().
 * - Used a mnemonic in the call to TKreportError at the beginning of
 *   createMetadataPVL().
 *
 * Revision 1.79  1996/12/16  20:13:41  amak
 * Fixed merging problem that caused by CVS.
 *
 * Revision 1.78  1996/12/16  19:15:51  pshah
 * moved some of the fields of the structure related to IM and DM from gIDtokens
 * to a new structure IMDMquery to make the code more clean
 *
 * Revision 1.77  1996/12/16  15:58:05  pshah
 * Added code to routine initMetadata for IMinterface
 *
 * Revision 1.75  1996/12/13  16:39:05  amak
 * Modified initMetadata - Size of coreMetadata.dataID has changed from 50
 * to 100. Corresponding call to mdcpy has been replacedd by mdcpyLong.
 *
 * Revision 1.74  1996/12/12  14:22:37  amak
 * Added new routine mdcpyLong().  Certain size of the metadata elements have
 * been changed from 50 to 100 and 300.  This new function is created to
 * adopt the new specification.
 * Made various modification to initMetadata.  Since some of the metadata
 * elements (orbitModelName, qaParamName, browseName, contact, granuleID,
 * percentBadMissPix, maxValidChannel, maxValidChannel, minValidChannel,
 * inputID, dateGenInputFile, dataCenterSrc, sensorAlgn, scanPathModel, &
 * radarCity) have changed size (bytes), the calls to mdcpy() have been
 * replaced by calls to mdcpyLong().
 * Cancelled invalid metadata fields - qualityIndCmt, prRainThreshold,
 * valNetName and numValSensors, in function initMetadata, finalizeMetadata,
 * metaFree and createMetadataPVL.
 * Added IM calls to query TSDIS database in function initMetadata.  This
 * add is used to fill some of the metadata fields value.
 *
 * Revision 1.73  1996/12/03  17:01:25  amak
 * Avoided allocating excessive memory in routine finalizeMetadata().
 *
 * Revision 1.72  1996/11/07  18:58:00  amak
 * Added prologue.
 *
 * Revision 1.71  1996/11/07  17:33:08  amak
 * Fixed potential error in a recursive loop inside processObjects().
 *
 * Revision 1.70  1996/11/07  14:09:10  dbazell
 * added two new metadata elements
 * - lonMaxLat (float)
 * - epochMillisec (int)
 *
 * Revision 1.69  1996/11/06  15:28:18  amak
 * Added detailed description to the routine.
 *
 * Revision 1.68  1996/11/05  19:09:34  dbazell
 * - Changed archivemetadta.0 to ArchiveMetadata.0, and coremetadata.0 to
 * 	CoreMetadata.0
 * - Added "END;" to the end of the metadata groups, deleted it from the
 * 	end of each metadata element
 *
 * Revision 1.67  1996/11/05  15:12:06  dbazell
 * Added double quotes around string variables in metadata routines.
 * Also changed strcpy to sprintf to do this.
 *
 * Revision 1.66  1996/11/01  20:42:46  amak
 * Added routine initFlag()
 *  - initialize metadata booleanFlag as TK_FALSE.
 * Added routine createMetadataPVL()
 *  - this routine is used to call finalizeMetadata.
 * Modified initMetadata()
 *  - call initFlag before calling mdcpy to initialize metadata booleanFlag
 * Made tremendous change to finalizeMetadata()
 *  - call manageFiles() to find out the correct metadata handle for copy.
 *  - add if-else statement to check if there is any update of the metadata
 *    requested by the user.
 *
 * Revision 1.65  1996/10/23  18:00:51  pshah
 * switched the logic for the function processObjects(..) to handle errors
 *
 * Revision 1.64  1996/10/18  18:23:48  amak
 * Added one line to switch statement in processObjects() in order to add
 * FILE_APPEND capability to L1GV processing.
 *
 * Revision 1.63  1996/10/17  20:27:22  amak
 * Added new routines make2a56SDS and make2a57SDS.
 *
 * Revision 1.62  1996/10/11  13:52:17  dbazell
 * Changed the attribute names for metadata:
 *  - CoreMetadata.0 -> coremetadata.0
 *  - PSMetadata.0 -> archivemetadata.0
 *
 * Revision 1.61  1996/10/11  13:46:26  dbazell
 * Modified create_pvl by adding \nEND; to end of pvl string.  Changed
 * len to be initialized to 75 rather than 70.
 *
 * Revision 1.60  1996/08/26  19:06:56  amak
 * Made the following modifications to support the handling of empty granule
 * or mistaken granule.
 * - Added new routine detachVdata() to detach all the vdata in the input
 *   vgroup list.  This function is called by TKclose().
 * - Re-defined the function prototype processObjects().  This routine is
 *   now returning status code either TK_SUCCESS or W_TK_NOGRANULEDATA.
 * - Added a new hdf call VSelts() in function openInputVdata().  VSelts()
 *   is used to return the number of elements in the specified vdata.  This
 *   check is for the empty granule.
 *
 * Revision 1.59  1996/08/06  14:27:55  amak
 * Added new routines to handle Rain Gauge and Disdrometer products.  The
 * name of the routines are makeNetHdr(), makeGaugeDesc(), makeDisdroDesc(),
 * and processNetworkHdr().
 *
 * Modified processObjects() and openInputVdata().  The new modification is
 * used to handle the reading of empty granule data.  Now the error of reading
 * empty granule can bubble up all the way to TKopen().
 *
 * Revision 1.58  1996/07/29  19:58:17  pshah
 * Added comments to clarify some of the routines
 *
 * Revision 1.57  1996/07/29  19:51:43  pshah
 * hdf_routines.c
 *
 * Revision 1.56  1996/07/18  19:25:13  amak
 * Added prologue.
 *
 * Revision 1.55  1996/07/10  17:39:40  dbazell
 * Removed old metadata routines and added new metadata routines.
 *
 * Revision 1.54  1996/06/25  18:30:17  amak
 * Modified file for 3.0 release.
 *
 * Revision 1.42.2.3  1996/06/07  15:20:45  amak
 * Commented out compression setting.
 *
 * Revision 1.42.2.2  1996/05/31  19:41:58  amak
 * Took out TKerrorRou.h and TKwarnRou.h as preprocessor statements and
 * replaced those with TKerrHandle.h and TS_TK_15.h.  This is used for
 * the new error handling routine.
 *
 * Revision 1.50  1996/06/07  14:50:32  amak
 * Added prologue.
 *
 * Revision 1.49  1996/05/22  18:17:18  amak
 * Replaced status code checking from TK_FAIL (1) to TK_HDF_FAIL (-1).
 *
 * Revision 1.48  1996/05/21  19:32:59  amak
 * Added new routines processHeader(), makePRcalCoefHdr(), makeRayHdr(), and
 * makeClutterHdr() to process the L1B-21, L1C-21, and L2A-25 headers for
 * reading and writing.
 *
 * Revision 1.47  1996/05/17  12:33:28  dbazell
 * Deleted some medadata fields and added EpochDate
 *
 * Revision 1.46  1996/05/14 13:06:27  dbazell
 * Moved meanMotion, orbitAdjFlag, attitudeMode, beginSolarBeta,
 * endSolarBeta, scnrlgnCal, and ephemFilename to coreMetadata
 *
 * Revision 1.45  1996/05/13 17:41:36  dbazell
 * Extensively changed metadata
 *
 * Revision 1.44  1996/04/19 15:40:48  amak
 * Made modifications to the preprocessor #include statement.  Replaced
 * TKerrorRou.h and TKwarnRou.h to TKerrHandle.h and TS_TK_15.h for new
 * error handling calls.
 *
 * Revision 1.43  1996/03/21  20:41:44  amak
 * Added prologue.
 *
 * Revision 1.42.2.1  1996/05/31  19:17:38  amak
 * Used sds compression routines.
 *
 * Revision 1.42  1996/03/07  21:39:44  amak
 * Added sds compression routine for testing.
 *
 * Revision 1.41  1996/03/04  19:35:16  amak
 * Splitted function readMetadata() into readCoreMetadata() and 
 * readPSMetadata().
 * Splitted function writeMetadata() into writeCoreMetadata() and
 * writePSMetadata().  Splitted function makeMetadataFields() into
 * makeCoreMetadataFields() and makePSMetadataFields().  In addition, modified
 * processMetadata() and detachMetadata().  All the above modifications and
 * implementations are based on the new changes on the metadata data structure
 * and the new strategy to handle Core Metadata and Product Specific Metadata
 * separately.
 *
 * Revision 1.40  1996/01/25  15:08:00  amak
 * This is a test.
 *
 * Revision 1.39  1995/12/12  20:08:16  thyun
 * Modified some type errors raised during the code review meeting.
 *
 * Revision 1.38  1995/12/12  15:09:39  amak
 * Fixed typo errors in prologues.
 *
 * Revision 1.37  1995/12/05  18:38:46  thyun
 * Inserted comments and prolog for the following routines, resetVgroup,
 * detachMetadata, processMetadata, makeMetadataFields, writeMetadata,
 * readMetadata, detachObjects, attachObjects, processObjects, attachVgroup,
 * startSD, and openHDF.
 *
 * Revision 1.36  1995/12/01  14:33:34  thyun
 * Inserted prolog for new procedures and functions.
 *
 * Revision 1.35  1995/11/28  16:05:57  thyun
 * Inserted and implemented routines (readMetadata, writeMetadata, 
 * makeMetadataFields, processMetadata, and detachMetadata).
 *
 * Revision 1.34  1995/11/22  20:14:30  thyun
 * No changes.
 *
 * Revision 1.33  1995/11/15  23:39:53  thyun
 * Implemented the procedures, openHDF, startSD, attachVgroup, detachVgroup, 
 * processObjects, attachObjects, and detachObjects.
 *
 * Revision 1.31  1995/10/06  19:39:31  thyun
 * Checked if memory is properly allocated.
 *
 * Revision 1.30  1995/09/18  14:23:46  thyun
 * Deleted a include <malloc.h>
 *
 * Revision 1.29  1995/08/29  15:00:52  amak
 * Added more detailed descriptions in comment headers.
 *
 * Revision 1.28  1995/08/25  18:03:38  thyun
 * Removde the include part of IO_PR.h.
 *
 * Revision 1.27  1995/08/15  20:44:37  thyun
 * Modified comments in file header as well as routine headers.
 *
 * Revision 1.26  1995/08/14  13:23:29  gsain
 * made changes
 *
 * Revision 1.25  1995/08/11  11:38:00  amak
 * Added comments for the TKreportWarning and TKreportError routines.
 *
 * Revision 1.24  1995/08/10  17:51:51  thyun
 * Remodified allocateVdataMemory and openOutputVdataField to calculate correct
 * size of vdata fields using new parsing rules and.
 *
 * Revision 1.23  1995/08/10  14:06:36  thyun
 * Modified allocateVdataMemory and openOutputVdataField to calculate correct 
 * size of vdata fields.
 *
 * Revision 1.22  1995/08/03  17:45:40  amak
 * Re-modified the code because CVS couldn't do version control nicely.
 *
 * Revision 1.21  1995/08/03  17:44:33  thyun
 * Deleted the procedure sizeOfString.
 *
 * Revision 1.20  1995/08/02  19:12:11  gsain
 * added comments
 *
 * Revision 1.19  1995/08/02  18:40:51  gsain
 * removed return statements
 *
 * Revision 1.18  1995/08/01  20:40:16  thyun
 * Updated the structure names such as VDATA_LINK and SDS_LINK as well as
 * variable names.  These changes occurred becase of changes in parsing.
 *
 * Revision 1.17  1995/08/01  13:41:09  gsain
 * removed HEprint
 *
 * Revision 1.16  1995/07/20  18:35:09  amak
 * Testing CVS.
 *
 * Revision 1.15  1995/07/19  13:30:58  thyun
 * Modified tempText in strcatFields, deleted tempText from openInputVdata, and
 * changed vWrite to vdataWrite, sWrite to sdsWrite, sRead to sdsRead, and
 * vRead to vdataRead.
 *
 * Revision 1.13  1995/07/10  14:44:17  gsain
 * error Routines
 *
 * Revision 1.12  1995/07/10  13:57:46  gsain
 * error Routines
 *
 * Revision 1.11  1995/07/07  21:08:17  amak
 * Added more detailed comments
 *
 * Revision 1.10  1995/07/07  18:48:27  thyun
 * Modified occurrances of 0 to NULL and -1 to TK_FAIL
 *
 * Revision 1.9  1995/07/07  14:14:19  thyun
 * Modified and added comments
 *
 * Revision 1.8  1995/07/06  17:22:01  amak
 * Made minor changes in order to follow standard
 *
 * Revision 1.7  1995/06/30  18:40:08  thyun
 * Modified strcatFields
 *
 * Revision 1.4  1995/06/13  14:28:55  amak
 * Adding more comments
 *
 * Revision 1.3  1995/06/08  19:13:17  amak
 * Added more comments
 * */

/*********************************************************************************
*
*  FILE NAME:           hdf_routines.c
*
*  DESCRIPTION:
*       This file contains all the functions/routines that call the HDF 
*       functions.
*
*  ROUTINES CALLED:
*       Routine Name            Description
*       ------------            -----------
*	strcatFields		This routine concatenates a list of strings into
*				a single string.
*	openInputVdata		This routine is used to initialize vdata table
*				prior to read scientific data in HDF.
*	openInputSdata		This routine is used to initialize SDS prior to
*				read scientific data in HDF.
*	openOutputVdataField	This routine is used to initialize vdata table
*				prior to write scientific data into HDF.
*	openOutputSdata		This routine is used to initialize SDS prior to
*				write scientific data into HDF.
*	vdataWrite		This routine is used to write the vdata into
*				HDF.
*	sdsWrite		This routine is used to write the scientific data
*				set into HDF.
*	vdataRead		This routine is used to read the vdata from HDF.
*	sdsRead			This routine is used to read the scientific data
*				set from HDF.
*	getSizeOfType		This function returns the size of type.
*	allocateVdataMemory	This function allocates memory to pack data and
*				return it.
*	openHDF			This routines returns ID returned by Hopen.
*	startSD			This routines returns ID returned by SDstart.
*	attachVgroup		This routine recursively attaches vgroup into the HDF 
*				file.  OutputFile is the HDF file pointer.
*	detachVgroup		This routine recursively detaches vgroup from the HDF 
*				file.
*	processObjects		This routine recursively creates HDF objects for 
*				either writing or reading from HDF file.
*	attachObjects		This routine recursively attaches vdata and sds 
*				objects into their vgroup.
*	detachObjects		This routine recursively detaches vdata and sds 
*				objects from their vgroup.
*	initMetadata		This routine fills the toolkit internal metadata 
*				structures for core and product specific metadata 
*				with default values.  If filemode = TK_NEW_FILE, 
*				then the metadata structures are initialized here; 
*				if filemode = TK_READ_ONLY, the information is 
*				obtained from the existing file metadata.
*	instType		Takes a productID as input (e.g. TK_L1B_01), and 
*				returns an instrument type (e.g., TK_VIRS)
*	mdcpy			Allocate space for metadata element name and value.
*       			Copy the name and value into the element
*	writepvl_to_hdffile	This function writes pvl data to the hdf file.
*	readpvl_from_hdffile	This function reads pvl data from hdf file.
*	create_pvl		This creates a PVL object's for core and ps metadata 
*				elements that need to be written to browse output
*				file. The inputs are parameter and value which
*				contain the object name and object value respectively.
*				The output pvl object is appended to the existing
*				pvl objects in outputStr. After all parameters are
*				processed the outputStr contains the pvl for either
*				core or ps metadata.
*	parse_pvl		This function takes a PVL contained in inputStr and
*				parses the value for a given parameter name.
*				This function is used for core and ps pvl objects only.
*				due to the same pvl format for both of them.
*	finalizeMetadata	This routine writes metadata into the HDF file if the 
*				filemode is write.  It also deallocates memory.
*	metaFree		This routine deallocates the memory that was allocated 
*				for the individual metadata elements.
*	resetVgroup		This routine resets all the vdata and sds according to 
*			        the offsetPos.
*	makePRcalCoefHdr	This routine used to create header fields list in link 
*				list form.
*	makeRayHdr		This routine used to create header fields list in link 
*				list form.
*	makeClutterHdr		This routine used to create header fields list in link 
*				list form.
*	processHeader		This routine used to process L1B21, L1C21, and L2A25 
*				header for either reading from or writing to HDF.
*	allocMem		This function allocates memory to pack data and 
*				return it.
*	processNetworkHdr	This routine used to initialize L2A56 and L2A57 fields
*				before writing to hdf file.
*	makeNetHdr		This routine used to create L2A56 or L2A57 Network
*				Descriptor fields in linked list form.
*	makeGaugeDesc		This routine used to create L2A56 Gauge Descriptor
*				field in linked list form.
*	makeDisdroDesc		This routine used to create L2A57 Disdrometer 
*				Descriptor field in linked list form.
*	detachVdata		This routine recursively detaches vdata objects
*				from the vgroup.
*	make2a56SDS		This routine is used to create L2A56 SDS fields in
*				linked list form.
*	make2a57SDS		This routine is used to create L2A57 SDS fields in
*				linked list form.
*	setFlag		        This routine initializes boolean flag for metadata 
*				to TK_FALSE.  This flag is used to check the metadata 
*				has been updated or not.
*	createMetadataPVL	This routine writes metadata into the HDF file if 
*				the filemode is write.  It also deallocates memory.
*
**********************************************************************************/

#include <mfhdf.h>
#include <time.h>
#include "file_manager.h"
#include "hdf_routines.h"
#include "TKerrHandle.h"
#include "TS_TK_15.h"

#ifndef TSU_ENV
#include "IM.h"
#include "DM.h"
#include "UT.h"
#endif

/*****************************************************************************
*
*  FUNCTION STATEMENT:  
*       strcatFields(char **text, VDATA_FIELD_LINK *vdataFieldList)
*
*  DESCRIPTION:
*       This routine concatenates a list of strings into a single string.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       vdataFieldList		A link list containing field name information.
*
*  OUTPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       text			A string containing all field names.
*
*  CALLED BY:   		openInputVdata(), openOutputVdataField()
*
*  ROUTINES CALLED:
*	Routine Name		Description
*	------------		-----------
*       TKreportWarning         This routine reports warning errors to console 
*                               and continues processing.
*
******************************************************************************/

void strcatFields(char **text, VDATA_FIELD_LINK *vdataFieldList)
{ 
  char *tempText;
  VDATA_FIELD_LINK *tempList = vdataFieldList;
  int totalStringSize = 0;

	/****************************************************************/
	/* Calculate the total string size from the vdataFieldList.	*/
	/****************************************************************/
  while (tempList != NULL)
  {      
	 totalStringSize = totalStringSize + strlen(tempList->name) + 1;
	 tempList = tempList->next;
  }

  tempList = vdataFieldList;

	/****************************************************************/
	/* Allocate memory to make a string, tempText, containing all	*/
	/* the names of vdata fields.					*/
	/****************************************************************/
  tempText = (char *)malloc(totalStringSize + 1);

  if (tempText != NULL)
  {   
      tempText[0] = '\0';
      while(tempList != NULL)
      {     
	    strcat(tempText, tempList->name);
	    tempList = tempList->next;
	    if (tempList != NULL)
		strcat(tempText, ",");
      }
      *text = tempText;
  }
  else
  {
      /* Memory allocation failed in strcatFields. */ 
      TKreportError(E_TK_BADMEMALLCAT);  
  }
}



/*****************************************************************************
*
*  FUNCTION STATEMENT:  
*       openInputVdata(int32 hOpenID, char *vDataName, VDATA_FIELD_LINK *vdataList)
*
*  DESCRIPTION:
*       This routine is used to initialize vdata table prior to read scientific
*       data in HDF.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       hOpenID			The HDF file ID returned from Hopen.
*       vdataName	        The vdata table name.	
*       vdataList		The list of fields of the vdata.
*
*  OUTPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       vID			The ID returned from VSattach.
*
*  CALLED BY:   		processObjects()
*
*  ROUTINES CALLED:
*	Routine Name		Description
*	------------		-----------
*       TKreportError           This routine reports fatal errors to console 
*                               and terminates all processes.
*
******************************************************************************/

int32 openInputVdata(int32 hOpenID, char *vDataName, VDATA_FIELD_LINK *vdataList)
{
  int32 vID, vDataRef;
  char *text;
  int code, status = TK_SUCCESS;

	/****************************************************************/
	/* VSfind looks in the HDF file for a Vdata with a givin name	*/
	/* (vDataName), and returns the Vdata reference number if 	*/
	/* successful and FAIL(-1) otherwise.				*/
	/****************************************************************/
  vDataRef = VSfind(hOpenID, vDataName);
  if (vDataRef == 0)
  {
      if (strcmp(vDataName, "CORE_METADATA") == 0)
          return (TK_FAIL);
      else if (strcmp(vDataName, "PRODUCT_SPECIFIC_METADATA") == 0)
          return (TK_FAIL);
      else if (strcmp(vDataName, "PR_CAL_COEF") == 0)
          return(TK_FAIL);
      else if (strcmp(vDataName, "RAY_HEADER") == 0)
          return(TK_FAIL);
      else if (strcmp(vDataName, "CLUTTER_FLAG") == 0)
          return(TK_FAIL);
      else if (strcmp(vDataName, "NETWORK_DESCRIPTOR") == 0)
	  return(TK_FAIL);
      else if (strcmp(vDataName, "GAUGE_DESCRIPTOR") == 0)
	  return(TK_FAIL);
      else if (strcmp(vDataName, "DISDROMETER_DESCRIPTOR") == 0)
	  return(TK_FAIL);
      else
      {
      	/* VSfind failed in openInputVdata */
	TKreportWarningParm(W_TK_FAIL_OPEN_VDATA, vDataName);
	vID = TK_DO_NOT_READ_DATA;
	status = TK_FAIL;
      }
  }

  if ( status == TK_SUCCESS )
    {
	/****************************************************************/
	/* VSattach attaches to the Vdata with the specified reference	*/
	/* number vDataRef and returns an identifier to that Vdata.	*/
	/****************************************************************/
      /* Attach the vdata in read mode */
      vID = VSattach(hOpenID, vDataRef, "r");
      if (vID == TK_HDF_FAIL)
	{  
	  /* VSattach failed in openInputVdata */ 
	  TKreportError(E_TK_FAILVSATTIN);	
	}
      
      /*
       **	VSelts returns the number of elements in the specified vdata
       **	if successful and TK_HDF_FAIL otherwise.
       */
      code = VSelts(vID);
      if (code == TK_HDF_FAIL)
	/* Unable to determine the number of elements in a vdata. */ 
	TKreportError(E_TK_FAILVSELTS);
      if (code == 0)
	{
	  VSdetach(vID);
	  /* Empty HDF file cannot be read. */
	  return(W_TK_NOGRANULEDATA);
	}
      
      /****************************************************************/
      /* Concatenate all the field names of vdata to text.		*/
      /****************************************************************/
      strcatFields(&text, vdataList);
      
      
      /****************************************************************/
      /* VSsetfields establishes the fields to be accessed within the	*/
      /* Vdata by any subsequent read or write routines(VSread,	*/
      /* VSwrite).							*/
      /****************************************************************/
      if (VSsetfields(vID, text) == TK_HDF_FAIL)
	{   
	  /* VSsetfields failed in openInputVdata */
	  TKreportError(E_TK_FAILVSSETFLDIN);  
	}
      
      /*****************************************************************/
      /* Deallocate memory allocated for text.			 */
      /*****************************************************************/
      if (text != NULL)
	free(text);
    }
  return(vID);
}




/****************************************************************************n
*
*  FUNCTION STATEMENT:  
*       openInputSdata(int32 sdStartID, char *sdName)
*
*  DESCRIPTION:
*       This routine is used to initialize SDS prior to read scientific data 
*	from HDF.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       sdStartID		The HDF file ID returned from SDstart.
*       sdName			The SDS name.
*
*  OUTPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       sdID			The ID returned from SDselect.
*
*  CALLED BY:			TKopen()
*
*  ROUTINES CALLED:  
*	Routine Name		Description
*	------------		-----------
*       TKreportError           This routine reports fatal errors to console
*                               and terminates all processes.
*
******************************************************************************/

int32 openInputSdata(int32 sdStartID, char *sdName)
{
  int32 sdIndex;
  int32 sdID;
  int status = TK_SUCCESS;

	/****************************************************************/
	/* SDnametoindex uses the sdName, the name of a scientific data	*/
	/* set, to determine the index assigned to it and returns an	*/
	/* sdIndex if successful FAIL(-1) otherwise.			*/
	/****************************************************************/
  sdIndex = SDnametoindex(sdStartID, sdName);
  if (sdIndex == TK_HDF_FAIL)
  {   
      	/* SDnametoindex failed in openInputSdata. */
	TKreportWarningParm(W_TK_FAIL_OPEN_SDS, sdName);
	status = TK_FAIL;
	sdID = TK_DO_NOT_READ_DATA;
  }

	/****************************************************************/
	/* SDselect gets the sdID for a specific data set and returns 	*/
	/* sdID if successful and FAIL(-1) otherwise.			*/ 
	/****************************************************************/
  if ( status == TK_SUCCESS )
    {
      sdID = SDselect(sdStartID, sdIndex);
      if (sdID == TK_HDF_FAIL)
	{   
	  /* SDselect failed in openInputSdata. */
	  TKreportError(E_TK_FAILSDSELECTIN);
	}
    }
  return(sdID);
}






/*****************************************************************************
*
*  FUNCTION STATEMENT:  
*       openOutputVdataField(int32 hOpenID, char *vName, 
				VDATA_FIELD_LINK *vdataFieldLink)
*
*  DESCRIPTION:
*       This routine is used to initialize vdata table prior to writing
*       scientific data into HDF.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       hOpenID			The HDF file name returned from Hopen.
*       vName			The vdata table name.
*       vdataFieldLink		The list of fields of the vdata.
*
*  OUTPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       vID			ID returned from VSattach.
*
*  CALLED BY:   		processObjects()
*
*  ROUTINES CALLED:
*	Routine Name		Description
*	------------		-----------
*       TKreportError           This routine reports fatal errors to console 
*                               and terminates all processes.
*
******************************************************************************/

int32 openOutputVdataField(int32 hOpenID, char *vName, VDATA_FIELD_LINK *vdataFieldLink)
{ 
  VDATA_FIELD_LINK *tempSTR = vdataFieldLink;
  int32 vID;
  int nFields = 0;
  char *text = 0;
  char *tempText;

	/****************************************************************/
	/* VSattach attaches to the Vdata with the specified reference 	*/
	/* number vDataRef and returns an identifier to that Vdata(vID).*/
	/****************************************************************/
  vID = VSattach(hOpenID, -1, "w");
  if (vID == TK_HDF_FAIL)
  {   
     	/* VSattach failed in openOutputVdataField. */
	TKreportError(E_TK_FAILVSATTOUT);
  }

	/****************************************************************/
	/* VSsetname assignes a name to a Vdata.			*/ 
	/****************************************************************/
  VSsetname(vID, vName);

	/****************************************************************/
	/* If the type is character type, obtain the string size of its */
	/* name and perform VSfdefine.  Otherwise, perform VSfdefine.	*/
	/* VSfdefine defines a new field in a Vdata and returns 	*/
	/* SUCCEED(0) if successful and FAIL(-1) otherwise.		*/ 
	/****************************************************************/
  while (tempSTR != NULL)
  { 
        if (VSfdefine(vID, tempSTR->name, tempSTR->type, tempSTR->fieldSize) != TK_SUCCESS)
        {   
            /* VSfdefine failed in openOutputVdataField. */
	    TKreportError(E_TK_FAILDEFOUT);
        }
        tempSTR = tempSTR->next;
  }

	/****************************************************************/
	/* Concatenate all the field names of vdata to text.		*/
	/****************************************************************/
  strcatFields(&text, vdataFieldLink);

	/****************************************************************/
	/* VSsetfields establishes the fields to be accessed within the */
	/* Vdata by any subsequent read or write routines(VSread, 	*/
	/* VSwrite).							*/
	/****************************************************************/
  if (VSsetfields(vID, text) == TK_HDF_FAIL)
  {  
      /* VSsetfields failed in openOutputVdataField. */ 
      TKreportError(E_TK_FAILSETFLDOUT);
  } 

	/****************************************************************/
	/* Deallocate memory allocated for text.			*/
	/****************************************************************/
  free(text);
  return(vID);
}


/*****************************************************************************
*
*  FUNCTION STATEMENT:  
*       openOutputSdata(int32 sdStartID, char *sdName, 
*			int32 type, int32 rank, int32 *dims)
*
*  DESCRIPTION:
*       This routine is used to initialize SDS prior to writing scientific
*       data into HDF.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       sdStartID		The HDF file ID.
*       sdName			The SDS name.
*       type			The type of SDS.
*       rank			The rank of SDS.
*       dims			The size of each dimensions.
*
*  OUTPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       sdIndex			The ID returned from SDcreate.
*
*  CALLED BY:   TKopen()
*
*  ROUTINES CALLED:  
*	Routine Name		Description
*	------------		-----------
*       TKreportError           This routine reports fatal errors to console 
*                               and terminates all processes.
*
******************************************************************************/

int32 openOutputSdata(int32 sdStartID, char *sdName, int32 type, int32 rank, int32 *dims)
{
  int32 sdIndex;
/*
  comp_info c_info;
*/

	/****************************************************************/
	/* SDcreate creates a new data set and returns sdIndex if 	*/
	/* successful and FAIL(-1) otherwise.				*/
	/****************************************************************/
  sdIndex = SDcreate(sdStartID, sdName, type, rank, dims);
  if (sdIndex == TK_HDF_FAIL)
  {  
      /* SDcreate failed in openOutputSdata. */ 
      TKreportError(E_TK_FAILCRTOUT);
  }

/* sds compression */
/*
  c_info.skphuff.skp_size = sizeof(uint16);
  if (SDsetcompress(sdIndex, COMP_CODE_SKPHUFF, &c_info) != TK_SUCCESS)
  {
      TKreportError(E_TK_FAILOUTCOMP);
  }
*/

  return(sdIndex);
}


/*****************************************************************************
*
*  FUNCTION STATEMENT:
*       vdataWrite(VDATA_LINK **vdataLink, void *data)
*
*  DESCRIPTION:
*       This routine is used to write the vdata into HDF.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       vdataLink		The list containing information of each field of vdata.
*       data			The data to be written in HDF.
*
*  OUTPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       vdataLink		The next field of the vdata field list.
*
*  CALLED BY:   readScan_1B_01(), readScan_3B_42(), readScan_3B_43(),
*               readScan_1B_11(), readScan_3A_11(), readScan_2A_12(),
*               readScan_1B_21(), readScan_2A_21(), readScan_2A_25(),
*               readScan_2A_23(), readScan_1B_21(), readScan_2B_31(),
*               readScan_3B_31(), readScan_3A_25(), readScan_3A_26()
*
*  ROUTINES CALLED:  
*	Routine Name		Description
*	------------		-----------
*       TKreportError           This routine reports fatal errors to console
*                               and terminates all processes.
*       TKreportWarning         This routine reports warning errors to console 
*                               and continues processing.
*
******************************************************************************/

int vdataWrite(VDATA_LINK **vdataLink, void *data)
{ 
  VDATA_LINK *tempVdata = *vdataLink; 

	/****************************************************************/
	/* VSwrite returns writes data into a Vdata and returns the 	*/
	/* total number of records written if successful and FAIL(-1) 	*/
	/* otherwise.							*/
	/****************************************************************/
  if (tempVdata != NULL)
  { 
    if (VSwrite(tempVdata->vdataID, (char *)data, 1, FULL_INTERLACE) == TK_HDF_FAIL)
      {   
	  /* VSwrite failed in vdataWrite. */
/*
          TKreportError(E_TK_FAILWRTVWRT);
*/
          TKreportErrorParm(E_TK_FAILWRTVWRT,tempVdata->name);
      }
  }
  else
  {
      /* Empty SDS list in vdataWrite */
      TKreportWarning(W_TK_BADSDSVWRT);
      return(TK_FAIL);
  }
  *vdataLink = tempVdata->next;
  return(TK_SUCCESS);
}


/*****************************************************************************
*
*  FUNCTION STATEMENT:
*       sdsWrite(SDS_LINK **sdsLink, void *data)
*
*  DESCRIPTION:
*       This routine is used to write the scientific data set into HDF.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       sdsLink			The list of SDS.
*       data			Data to be written into HDF.
*
*  OUTPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       sdsLink			The next list of SDS.
*
*  CALLED BY:   readScan_1B_01(), readScan_1B_11(), readScan_2A_12(),
*               readScan_1B_21(), readScan_2A_21(), readScan_2A_25(),
*               readScan_2A_23(), readScan_1B_21(), readScan_2B_31(),
*
*  ROUTINES CALLED: 
*	Routine Name		Description
*	------------		-----------
*       TKreportError           This routine reports fatal errors to console 
*                               and terminates all processes.
*       TKreportWarning         This routine reports warning errors to 
*                               console and continues processing.
*
******************************************************************************/

int sdsWrite(SDS_LINK **sdsLink, void *data)
{  
   SDS_LINK *tempSDS = *sdsLink;

	/****************************************************************/
	/* SDwritedata writes a hyperslab of data for a data set and 	*/
	/* returns SUCCEED(0) if successful and FAIL(-1) otherwise.	*/ 
	/****************************************************************/

   if (tempSDS != NULL)
   { 
       if (SDwritedata(tempSDS->sdsID, tempSDS->start, NULL,
                       tempSDS->edge, (char *)data) == TK_HDF_FAIL)
       {   
	   /* SDwritedata failed in sdsWrite. */
/*
           TKreportError(E_TK_FAILWRTSWRT);
*/
           TKreportErrorParm(E_TK_FAILWRTSWRT,tempSDS->name);
       }
       tempSDS->start[0]++;
   }
   else
   {
       /* Empty SDS list in sdsWrite */
       TKreportWarning(W_TK_BADSDSSWRT);
       return(TK_FAIL);
   }
   *sdsLink = tempSDS->next;
   return(TK_SUCCESS);
}




/*****************************************************************************
*
*  FUNCTION STATEMENT:
*       vdataRead(VDATA_LINK **vdataLink, void *data)
*
*  DESCRIPTION:
*       This routine is used to read the vdata from HDF.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       vdataLink		The list of vdatas.
*
*  OUTPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       vdataLink	 	The next vdata list.
*       data			Data read from HDF.
*
*  CALLED BY:   readScan_1B_01(), readScan_1B_11(), readScan_2A_12(),
*               readScan_1B_21(), readScan_2A_21(), readScan_2A_25(),
*               readScan_2A_23(), readScan_1B_21(), readScan_2B_31(),
*
*  ROUTINES CALLED:  
*	Routine Name		Description
*	------------		-----------
*       TKreportError           This routine reports fatal errors to console 
*                               and terminates all processes.
*       TKreportWarning         This routine reports warning errors to console
*                               and continues processing.
*
******************************************************************************/

int vdataRead(VDATA_LINK **vdataLink, void *data)
{  
   VDATA_LINK *tempVdata = *vdataLink;
   int status ;

   if ( tempVdata->vdataID != TK_DO_NOT_READ_DATA )
     {
	/********************************************************/
	/* VSread retrives data from a Vdata using given input 	*/
	/* parameters and returns the data read from that Vdata */
	/* in the buffer.					*/
	/********************************************************/
       if (tempVdata != NULL)
	 {   
	   status = VSread(tempVdata->vdataID, (char *)data, 1, FULL_INTERLACE);
	   if (status == TK_HDF_FAIL)
	     {  
	       /* VSread failed in vdataRead. */ 
	       TKreportError(E_TK_FAILRDVRD);
	     }
	 }
       else
	 {	 
	   /* Empty Vdata list in vdataRead */ 
	   TKreportWarning(W_TK_BADVDATAVRD);
	   return(TK_FAIL);
	 }
     }
   *vdataLink = tempVdata->next;
   return(TK_SUCCESS);
 }
   
/*****************************************************************************
*
*  FUNCTION STATEMENT:
*       sdsRead(SDS_LINK **sLink, void *data)
*
*  DESCRIPTION:
*       This routine is used to read the scientific data set from HDF.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       sdsLink			A link list pointing to the list of SDS.
*
*  OUTPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       sdsLink			A link list pointing to the next list of SDS.
*       data			Data read from HDF.
*
*  CALLED BY:   readScan_1B_01(), readScan_3B_42(), readScan_3B_43(),
*               readScan_1B_11(), readScan_3A_11(), readScan_2A_12(),
*               readScan_1B_21(), readScan_2A_21(), readScan_2A_25(),
*               readScan_2A_23(), readScan_1B_21(), readScan_2B_31(),
*               readScan_3B_31(), readScan_3A_25(), readScan_3A_26(),
*
*  ROUTINES CALLED:   
*	Routine Name		Description
*	------------		-----------
*       TKreportError           This routine reports fatal errors to console
*                               and terminates all processes.
*       TKreportWarning         This routine reports warning errors to console
*                               and continues processing.
*
******************************************************************************/
int sdsRead(SDS_LINK **sdsLink, int* scaleFactor, void *data)
{  
   SDS_LINK *tempSDS = *sdsLink;
	/****************************************************************/
	/* SDreaddata reads hyperslab of data from a data set and 	*/
	/* returns SUCCEED(0) if successful and FAIL(-1) otherwise.	*/
	/****************************************************************/

   if (tempSDS == NULL) /*if check added to check for null 9/8/99*/
   { 	  
       /* Empty SDS list in sdsRead */ 
       TKreportWarning(W_TK_BADSDSSRD);
       return(TK_FAIL);
   }

   if ( tempSDS->sdsID != TK_DO_NOT_READ_DATA )
     {
       if (tempSDS != NULL)
	 { 
	   if (SDreaddata(tempSDS->sdsID, tempSDS->start, NULL,
			  tempSDS->edge, (char *)data) == TK_HDF_FAIL)
	     {  
	       /* SDreaddata failed in sdsRead */ 
/*
	       TKreportError(E_TK_FAILRDSRD);
*/
	       TKreportErrorParm(E_TK_FAILRDSRD,tempSDS->name);
	     }
	   tempSDS->start[0]++;
	 }
       else
	 { 	  
	   /* Empty SDS list in sdsRead */ 
	   TKreportWarning(W_TK_BADSDSSRD);
	   return(TK_FAIL);
	 }
     }
   *scaleFactor = tempSDS->scaling_factor;
   *sdsLink = tempSDS->next;
   return(TK_SUCCESS);
 }

/*****************************************************************************
*
*  FUNCTION STATEMENT:
*       getSizeOfType(int32 type)
*
*  DESCRIPTION:
*       This function returns the size of type.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       type			HDF type.
*
*  OUTPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       sizeof(type)		The integer size of type.
*
*  CALLED BY:           allocateVdataMemory()
*
*  ROUTINES CALLED:   
*	Routine Name		Description
*	------------		-----------
*       TKreportError           This routine reports fatal errors to console
*                               and terminates all processes.
*
******************************************************************************/

int getSizeofType(int32 type)
{
    switch (type)
    {
      case DFNT_CHAR8 :
	return(sizeof(char8));
	break;

      case DFNT_INT8 :
 	return(sizeof(int8));
        break;

      case DFNT_INT16 :
	return(sizeof(int16));
        break;

      case DFNT_INT32 :
	return(sizeof(int32));
        break;

      case DFNT_FLOAT32 :
	return(sizeof(float32));
	break;

      case DFNT_FLOAT64 :
 	return(sizeof(float64));
	break;

      default :
	/* Incorrect HDF type in getSizeofType */
        TKreportError(E_TK_BADHDFTYPE);
    }
}


/*********************************************************************************
*
*  FUNCTION STATEMENT:
*       *allocateVdataMemory(VDATA_FIELD_LINK **vdataFieldLink)
*
*  DESCRIPTION:
*       This function allocates memory to pack data and return it.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       vdataFieldLink		A link list pointing to the list of vdata fields.
*
*  OUTPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       databuf			The memory allocated to store vdata.
*
*  CALLED BY:   
*       writeScan_2A_21(), readScan_2A_21(), writeScan_2A_23(), readScan_2A_23(),
*       writeScan_2A_25(), readScan_2A_25(), writeScan_2A_12(), readScan_2A_12()
*
*  ROUTINES CALLED:    		 None
*
**********************************************************************************/

uint8 *allocateVdataMemory(VDATA_FIELD_LINK **vdataFieldLink)
{ 
  VDATA_FIELD_LINK *tempVdataFieldLink = *vdataFieldLink;
  int totalSize = 0;
  uint8 *databuf = NULL;

  while (tempVdataFieldLink != NULL)
  { 
         totalSize = totalSize + (tempVdataFieldLink->fieldSize * 
				  getSizeofType(tempVdataFieldLink->type));
	 tempVdataFieldLink = tempVdataFieldLink->next; 
  }

  databuf = (uint8 *)malloc(totalSize);

  if (databuf !=  NULL)
  {
      memset(databuf, '\0', totalSize);
  }
  else
  {
      /* Memory allocation failed in allocateVdataMemory. */
      TKreportError(E_TK_MEMALFAILAVM);
  }

  return(databuf);
}




/*****************************************************************************
*
*  FUNCTION STATEMENT:
*	openHDF(char *filename, int32 accessMode, int16 n_dds)
*
*  DESCRIPTION:
*       This routines returns ID returned by Hopen.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*	filename		HDF file name.
*	accessMode		File access mode.
*	n_dds			Number of data descriptors in a block if
*				a new file is to be created.
*
*  OUTPUT PARAMETERS:           None.
*
*  CALLED BY:                   TKopen().
*
*  RETURN VALUES:               ID returned by Hopen.
*
*  ROUTINES CALLED:
*       Routine Name            Description
*       ------------            -----------
*       TKreportError           This routine reports fatal errors to console 
*                               and terminates all processes.
*	Hopen			A HDF routine which provides an access path to an
*				HDF file.
*
******************************************************************************/

int32 openHDF(char *filename, int32 accessMode, int16 n_dds)
{
  int32 hopenID = Hopen(filename, accessMode, n_dds);

  if (hopenID == TK_HDF_FAIL)
  {
    if (filename==NULL)
    {
        TKreportErrorParm ( E_TK_FAILHDFOPEN, "No file name specified");
    }
    else
    {
        TKreportErrorParm ( E_TK_FAILHDFOPEN, filename );
    }
    hopenID = E_TK_FAILHDFOPEN;
  }

  return (hopenID);
}


/*****************************************************************************
*
*  FUNCTION STATEMENT:
*	startSD(char *fileName, int32 accessMode)
*
*  DESCRIPTION:
*       This routines returns ID returned by SDstart.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       filename                HDF file name.
*       accessMode              File access mode.
*
*  OUTPUT PARAMETERS:           None.
*
*  CALLED BY:                   TKopen().
*
*  RETURN VALUES:               ID returned by SDstart.
*
*  ROUTINES CALLED:
*       Routine Name            Description
*       ------------            -----------
*       TKreportError           This routine reports fatal errors to console
*                               and terminates all processes.
*	SDstart			A HDF routine which opens the HDF file and 
*				initializes the SD interface.
*
******************************************************************************/

int32 startSD(char *fileName, int32 accessMode)
{

  int32 sdStartID = SDstart(fileName, accessMode);

  if (sdStartID == TK_HDF_FAIL)
  {
#ifdef OPERATION_ENV
    TKreportWarningParm ( E_TK_FAILSDSTART, fileName );
#else
    TKreportError(E_TK_FAILSDSTART);
#endif
    sdStartID = E_TK_FAILSDSTART;
  }

/* sds compression */
/*
  SDsetfillmode(sdStartID, SD_NOFILL);
*/

  return (sdStartID); 
}




/*****************************************************************************
*
*  FUNCTION STATEMENT:
*	attachVgroup(int32 outputFile, VGROUP_LINK *vgroup)
*
*  DESCRIPTION:
*	This routine recursively attaches vgroup into the HDF file.
*	OutputFile is the HDF file pointer.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*	outputFile		ID returned by Hopen.
*	vgroup			List of vgroups.
*
*  OUTPUT PARAMETERS:           
*       Parameter Name          Description
*       --------------          -----------
*	vgroup			List of vgroups which are attached 
*				into the HDF file.
*
*  CALLED BY:                   TKopen().
*
*  RETURN VALUES:               None.
*
*  ROUTINES CALLED:
*       Routine Name            Description
*       ------------            -----------
*	Vattach			A HDF routine attaches to a Vgroup and
*				returns an indentifier for that group.
*	Vsetname		A HDF routine sets the name for a Vgroup.
*	Vsetclass		A HDF routine sets the class name for a
*				Vgroup.
*	attachVgroup		This routine recursively attaches vgroup into
*				the HDF file.
*
*
******************************************************************************/

void attachVgroup(int32 outputFile, VGROUP_LINK *vgroup)
{

  if (vgroup != NULL)
  {
      /* 
      ** Attach the vgroup into the HDF file.
      */
      vgroup->vGroupID->vGroupID = Vattach(outputFile, -1, "w");

      /*
      ** If the Vattach is successful, then set its class and name.
      */
      if (vgroup->vGroupID->vGroupID != TK_HDF_FAIL)
      {
          Vsetname(vgroup->vGroupID->vGroupID , vgroup->vGroupID->name); 
          Vsetclass(vgroup->vGroupID->vGroupID, vgroup->vGroupID->name);
      }
      else
      { 
          /* Trouble with defining Vgroup in attachVgroup. */
          TKreportError(E_TK_BADDEFVG);
      }

      /*
      ** Recursively attach the sub-vgroups and other vgroups into the HDF
      ** file.
      */
      attachVgroup(outputFile, vgroup->subVgroup);
      attachVgroup(outputFile, vgroup->next);
  }
}




/*****************************************************************************
*
*  FUNCTION STATEMENT:
*	detachVgroup(VGROUP_LINK *vgroup)
*
*  DESCRIPTION:
*       This routine recursively detaches vgroup from the HDF file.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       vgroup                  List of vgroups.
*
*  OUTPUT PARAMETERS:		None.
*
*  CALLED BY:                   TKclose().
*
*  RETURN VALUES:               None.
*
*  ROUTINES CALLED:
*       Routine Name            Description
*       ------------            -----------
*       Vdetach			A HDF routine detaches a Vgroup that is 
*				currently attached, and terminates
*				further access to the Vgroup.
*	detachVgroup		This routine recursively detaches vgroup from 
*				the HDF file.
*
******************************************************************************/

void detachVgroup(VGROUP_LINK *vgroup)
{

  if (vgroup != NULL)
  {
      /* 
      ** Detach the vgroup from the HDF file.
      */
      Vdetach(vgroup->vGroupID->vGroupID);

      /*
      ** Recursively detach the sub-vgroups and vgroups from the HDF file.
      */
      detachVgroup(vgroup->subVgroup);
      detachVgroup(vgroup->next);
  }
}




/*****************************************************************************
*
*  FUNCTION STATEMENT:
*	processObjects(VGROUP_LINK *vgroup, char accessMode, int32 hopenID,
*                      int32 sdstartID, int32 numOfScanlines)
*
*  DESCRIPTION:
*       This routine recursively creates HDF objects for either writing or
*	reading from HDF file.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       vgroup                  List of vgroups.
*	accessMode		File mode for either reading or wrting.
*	hopenID			ID returned by Hopen.
*	sdstartID		ID returned by SDstart.
*	numOfScanlines		The number of scanlines to be inserted into
*				the HDF file.
*
*  OUTPUT PARAMETERS:           None.
*       Parameter Name          Description
*       --------------          -----------
*	vgroup			List of vgroups.  Each vgroup's fields, vdataID 
*				sdsID are initialized with new IDs.
*
*  CALLED BY:                   TKopen().
*
*  RETURN VALUES:               TK_SUCCESS for successfully initialize the objects
*				W_TK_NOGRANULEDATA for unsuccessfully initialize
*				the objects from an empty granule file
*
*  ROUTINES CALLED:
*       Routine Name            Description
*       ------------            -----------
*	openInputVdata		This routine is used to initialize vdata table 
*				prior to read scientific data in HDF.
*	openInputSdata		This routine is used to initialize SDS prior 
*				to read scientific data from HDF.
*	openOutputVdataField	This routine is used to initialize vdata table 
*				prior to writing scientific data into HDF.
*	openOutputSdata		This routine is used to initialize SDS prior
*				to writing scientific data into HDF.
*	processObjects		This routine recursively creates HDF objects 
*				for either wrting or reading from HDF file
*
******************************************************************************/

int32 processObjects(VGROUP_LINK *vgroup, char accessMode, int32 hopenID, 
		    int32 sdstartID, int32 numOfScanlines, int *granFlg)
{
  VDATA_LINK *vdata;
  SDS_LINK *sds;

  int32	retcode, vopenid;

  int status;
  float64 my_cal, my_cal_err, my_offset, my_offset_err;
  int32 my_data_type;

  if ( *granFlg == W_TK_NOGRANULEDATA )
    return ( W_TK_NOGRANULEDATA );
  else if (vgroup != NULL)
    {
      vdata = vgroup->vdata;
      sds = vgroup->sds;
      
      switch(accessMode)
	{
        case TK_READ_ONLY  :
	  
	  /*
	   ** Obtain SDS object IDs for reading from HDF file.
	   */
          while (sds != NULL)
	    {
	      sds->sdsID = openInputSdata(sdstartID, sds->name);
              status = SDgetcal ( sds->sdsID, &my_cal,
                        &my_cal_err, &my_offset, &my_offset_err, &my_data_type);              if (status==0)
                  sds->scaling_factor=my_cal;
              else
                  sds->scaling_factor=1;
	      sds = sds->next;
	    }
	  
	  /*
	   ** Obtain Vdata object IDs for reading from HDF file.
	   */
          while (vdata != NULL)
	    {
	      vopenid = openInputVdata(hopenID,vdata->name,vdata->vdataFields);
	      if (vopenid != W_TK_NOGRANULEDATA)
		{
		  vdata->vdataID = vopenid;
		  vdata = vdata->next;
		  *granFlg = TK_SUCCESS;
		}
	      else
		{
		  *granFlg = W_TK_NOGRANULEDATA;
		    return(W_TK_NOGRANULEDATA);
		}
	    }
	  break;
	  
	  
	case TK_APPEND :
        case TK_NEW_FILE   :
	  /*
	   ** Create Vdata objects for writing into the HDF file.
	   */
          while (vdata != NULL)
	    {
	      vdata->vdataID=openOutputVdataField(hopenID, vdata->name, 
						  vdata->vdataFields);
	      vdata = vdata->next;
	    }
	  /*
	   ** Create SDS objects for writing into the HDF file.
	   */
          while (sds != NULL)
	    {
	      sds->dim[0] = numOfScanlines;
	      sds->sdsID = openOutputSdata(sdstartID,sds->name,sds->type,sds->rank,
					   sds->dim);
	      sds = sds->next;
	    }
          break;

	} 
      
      /*
       ** Recursively process vdata and sds objects for sub-vgroups and other vgroups.
       ** Check for the return status and find out whether the granule file is empty
       ** or not.
       */
      if (processObjects(vgroup->subVgroup, accessMode, hopenID, 
			 sdstartID, numOfScanlines, granFlg) != W_TK_NOGRANULEDATA)
	{}
      if (processObjects(vgroup->next, accessMode, hopenID, 
			 sdstartID, numOfScanlines, granFlg) != W_TK_NOGRANULEDATA)
	return(TK_SUCCESS);
      else
	return(W_TK_NOGRANULEDATA);
    }
  
}

/*****************************************************************************
*
*  FUNCTION STATEMENT:
*	attachObjects(VGROUP_LINK *vgroup)
*
*  DESCRIPTION:
*       This routine recursively attaches vdata and sds objects into their
*	vgroup.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       vgroup                  List of vgroups.
*
*  OUTPUT PARAMETERS:           None.
*
*  CALLED BY:                   TKclose().
*
*  RETURN VALUES:               None.
*
*  ROUTINES CALLED:
*       Routine Name            Description
*       ------------            -----------
*       TKreportError           This routine reports fatal errors to console
*                               and terminates all processes.
*	Vinsert			A HDF routine which establishes a link from a
*				Vgroup to either another Vgroup or to a Vdata
*				identifier.
*	SDidtoref		A HDF routine which uses sdsID to determine
*				the reference number assigned to the specific
*				data set.
*	Vaddtagref		A HDF routine which inserts the tag/ref pair
*				of an HDF object into the given Vgroup.
*	attachObjects		This routine recursively attaches vdata and sds
*				objects into their vgroup.
*
******************************************************************************/

void attachObjects(VGROUP_LINK *vgroup)
{
  VDATA_LINK *vdata;
  SDS_LINK *sds;
  int32 sdsRefNo;

  if (vgroup != NULL)
  {
      vdata = vgroup->vdata;
      sds = vgroup->sds;

      /*
      ** Insert the vdata objects into the vgroup.
      */
      while (vdata != NULL)
      {      if (Vinsert(vgroup->vGroupID->vGroupID, vdata->vdataID) == TK_HDF_FAIL)
             {
                 /* Inserting a vdata object failed in attachObjects. */
                 TKreportError(E_TK_FAILVINCLS);
             }
             vdata = vdata->next;
      }

      /*
      ** Insert the sds objects into the vgroup.
      */
      while (sds != NULL)
      {
             sdsRefNo = SDidtoref(sds->sdsID);
             if (sdsRefNo == TK_HDF_FAIL)
             {
                 /* Obtaining SDS reference # failed in attachObjects. */
                 TKreportError(E_TK_FAILSREFCLS);
             }
             if (Vaddtagref(vgroup->vGroupID->vGroupID, DFTAG_NDG, sdsRefNo) == TK_HDF_FAIL)
             {
                 /* Inserting a sds object failed in attachObjects. */
                 TKreportError(E_TK_FAILSINCLS);
             }
             sds = sds->next;
      }


      if (vgroup->prevVgroup != NULL)
      {
          Vinsert(vgroup->prevVgroup->vGroupID->vGroupID, vgroup->vGroupID->vGroupID);
      }
      attachObjects(vgroup->subVgroup);
      attachObjects(vgroup->next);
  }
}




/*****************************************************************************
*
*  FUNCTION STATEMENT:
*	detachObjects(VGROUP_LINK *vgroup)
*
*  DESCRIPTION:
*       This routine recursively detaches vdata and sds objects from their
*	vgroup.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       vgroup                  List of vgroups.
*
*  OUTPUT PARAMETERS:           None.
*
*  CALLED BY:                   TKclose().
*
*  RETURN VALUES:               None.
*
*  ROUTINES CALLED:
*       Routine Name            Description
*       ------------            -----------
*       TKreportError           This routine reports fatal errors to console
*                               and terminates all processes.
*	SDendaccess		A HDF routine which closes the HDF file and
*				cleans up memory.
*	VSdetach		A HDF routine which detaches from the current
*				Vdata thereby termination further access to
*				that vdata.
*	detachObjects		This routine recursively detaches vdata and sds
*				objects from their vgroup.
*
******************************************************************************/

void detachObjects(VGROUP_LINK *vgroup)
{
  VDATA_LINK *vdata;
  SDS_LINK *sds;

  if (vgroup != NULL)
  {
      vdata = vgroup->vdata;
      sds = vgroup->sds;

      /* 
      ** Detach sds objects from the HDF file. 
      */
      while (sds != NULL)
      { 
	if ( sds->sdsID != TK_DO_NOT_READ_DATA )
	  {
	    if (SDendaccess(sds->sdsID) == TK_HDF_FAIL)
	      {
		/* SDendaccess failed in detachObjects. */
		TKreportWarning(E_TK_FAILSDENDDO);
	      }
	  }
	sds = sds->next;
      }
      /*
      ** Detach vdata ojbects from the HDF file.
      */
      while (vdata != NULL)
      {     
	if ( vdata->vdataID != TK_DO_NOT_READ_DATA )
	  {
	    VSdetach(vdata->vdataID);
	  }
	    vdata = vdata->next;
      }

      /* 
      ** Recursively detach vdata and sds objects for sub-vgroups and other vgroups.
      */
      detachObjects(vgroup->subVgroup);
      detachObjects(vgroup->next);
  } 
}

#ifndef TSU_ENV

/*****************************************************************************
*
*  FUNCTION STATEMENT:
*       removeIMspaces( char *stringWithSpaces ) 
*
*  DESCRIPTION:
*       This routine removes spaces that are appended to character strings.
*       This is only used for strings that are returned from IM.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       stringWithSpaces        Character string that contains spaces.
*
*  OUTPUT PARAMETERS:           
*       Parameter Name          Description
*       --------------          -----------
*       stringWithSpaces        Character string with the spaces removed.
*
*  CALLED BY:                   buildInputFileStrings(), initMetadata().
*
*  RETURN VALUES:               None.
*
*  ROUTINES CALLED:             None.
*
******************************************************************************/
void removeIMSpaces ( char *stringWithSpaces )
{
  int string_length;
  int i;

  string_length = strlen ( stringWithSpaces );
  for ( i = --string_length; i > 0; i-- )
    if ( stringWithSpaces[i] != ' ' )
      {
	stringWithSpaces[++i] = '\0';
	break;
      }

  return;
}

/*****************************************************************************
*
*  FUNCTION STATEMENT:
*       buildInputFileStrings( char **genDateStr, char **inputFilesStr, 
*			    char **dataCenterStr, 
*			    IMinput_Granule_Return_Struct *queryResults )
*
*  DESCRIPTION:
*       This routine builds the strings for the names of the input files,
*       the generation dates for the input files, and the data centers
*       of the input files.  The information is obtained from the 
*       database query.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       queryResults            The results from the query of the input
*                               file information.  
*
*  OUTPUT PARAMETERS:           
*       Parameter Name          Description
*       --------------          -----------
*       genDateStr              A comma delimited string of generation dates 
*                               for the input files.
* 
*       inputFilesStr           A comma delimited string of the name of the 
*                               input files.
*
*       dataCenterStr           A comma delimited string of the data centers
*                               where the input files were obtained.
*
*  CALLED BY:                   initMetadata()
*
*  RETURN VALUES:               None.
*
*  ROUTINES CALLED:             
*
*       removeIMspaces          Removes any excess spaces appended to strings
*                               that are returned form IM.
*
*       TKreportError		This routine reports error to
*                               console and terminates processing.
*
******************************************************************************/
void buildInputFileStrings ( char **genDateStr, char **inputFilesStr, 
			    char **dataCenterStr, 
			    IMinput_Granule_Return_Struct *queryResults )
{
  IMinput_Granule_Return_Struct *tempNode = NULL;
  int totalSizeGenDate = 0, totalSizeInputFile = 0, totalSizeDataCenter = 0;
  char tempDate[11];  /*  YYYY/MM/DD  */
  char *genDate = NULL, *inputFiles = NULL, *dataCenter = NULL;
  int string_length;
  int i;

  genDate = *genDateStr;
  inputFiles = *inputFilesStr;
  dataCenter = *dataCenterStr;

  tempNode = queryResults;

  while ( tempNode != NULL )
    {
      totalSizeGenDate += 11; /*  Length of the TSDIS Formatted Date (YYYY/MM/DD) */
      totalSizeInputFile += strcspn ( tempNode->Input_Granule_ID, " ") + 1;
      removeIMSpaces ( tempNode->Input_Originating_System );
      totalSizeDataCenter += strlen ( tempNode->Input_Originating_System ) + 1;
      tempNode = tempNode->next;
    }

  tempNode = queryResults;

  /*  Add some room for the double quotes  */
  genDate = ( char * ) malloc ( totalSizeGenDate + 4 );
  inputFiles = ( char * ) malloc ( totalSizeInputFile + 4 );
  dataCenter = ( char * ) malloc ( totalSizeDataCenter + 4 );

  if ( ( genDate != NULL ) || ( inputFiles != NULL ) || ( dataCenter != NULL ) )
    {
      strcpy ( genDate, "\"" );
      strcpy ( inputFiles, "\"" );
      strcpy ( dataCenter, "\"" );
      while ( tempNode != NULL )
	{
          sprintf ( tempDate, "%04d/%02d/%02d",
                   tempNode->Input_Processing_Datim->tm_year + 1900,
                   tempNode->Input_Processing_Datim->tm_mon + 1,
                   tempNode->Input_Processing_Datim->tm_mday );
	  strcat ( genDate, tempDate );
	  strncat ( inputFiles, tempNode->Input_Granule_ID, 
		   strcspn ( tempNode->Input_Granule_ID, " " ) );
	  removeIMSpaces ( 
			     tempNode->Input_Originating_System ); 
	  strcat ( dataCenter, tempNode->Input_Originating_System );
	  if ( tempNode->next != NULL )
	    {
	      strcat ( genDate, "," );
	      strcat ( inputFiles, "," );
	      strcat ( dataCenter, "," );
	    }
	  tempNode = tempNode->next;
	}
      
      strcat ( genDate, "\"" );
      strcat ( inputFiles, "\"" );
      strcat ( dataCenter, "\"" );
      genDate[strlen(genDate)+1] = '\0';
      inputFiles[strlen(inputFiles)+1] = '\0';
      dataCenter[strlen(dataCenter)+1] = '\0';

      *genDateStr = genDate;
      *inputFilesStr = inputFiles;
      *dataCenterStr = dataCenter;
    }
  else
    TKreportError ( E_TK_BADMEMALLCAT );

  return;
}
#endif

/*****************************************************************************
*
*  FUNCTION STATEMENT:
*       createBrowseName(IO_HANDLE *sHandle, char *browseName)
*
*  DESCRIPTION:
*       This routine creates the browse name for the L1C GV products.
*       The browse name is of the form:  
*                    DataType.CollectionDate.RadarName.ProductVersion.BRO
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       sHandle                 A structure containing file information about 
*                               the current file being manipulated.  sGranuleHandle
*                               is returned by TKopen().  
*
*
*  OUTPUT PARAMETERS:           
*       Parameter Name          Description
*       --------------          -----------
*       browseName              The resultant browse name created by this routine.
*
*  CALLED BY:                   initMetadata()
*
*  RETURN VALUES:               None.
*
*  ROUTINES CALLED:             None.
*
******************************************************************************/

void createBrowseName ( IO_HANDLE *sHandle, char *browseName )
{
  char tempBuffer[256];

  strcpy ( browseName, "/0" );
  strcpy ( browseName, "\"" );
  strcat ( browseName, sHandle->gIDtokens.datatype );
  strcat ( browseName, "_BR." );
  strcpy ( tempBuffer, sHandle->gIDtokens.collectDate );
  TKremoveQuotes ( tempBuffer );
  strcat ( browseName, tempBuffer );
  strcat ( browseName, "." );
  strcpy ( tempBuffer, sHandle->gIDtokens.radar );
  TKremoveQuotes ( tempBuffer );
  strcat ( browseName, tempBuffer );
  strcat ( browseName, "." );
  strcpy ( tempBuffer, sHandle->gIDtokens.algVersion );
  TKremoveQuotes ( tempBuffer );
  strcat ( browseName, tempBuffer );
  strcat ( browseName, ".BRO\"" );
  return;
}

/*****************************************************************************
*
*  FUNCTION STATEMENT:
*       initMetadata(IO_HANDLE *sGranuleHandle, char filemode)
*
*  DESCRIPTION:
*       This routine fills the toolkit internal metadata structures
*       for core and product specific metadata with default values.
*       If filemode = TK_NEW_FILE, then the metadata structures are 
*       initialized here; if filemode = TK_READ_ONLY, the information 
*       is obtained from the existing file metadata.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       sGranuleHandle          A structure containing file information about 
*                               the current file being manipulated.  sGranuleHandle
*                               is returned by TKopen().  It contains the 
*                               metadata structures being filled.
*
*       filemode                TK_READ_ONLY or TK_NEW_FILE
*
*  OUTPUT PARAMETERS:           
*
*  CALLED BY:                   TKopen()
*
*  RETURN VALUES:               None.
*
*  ROUTINES CALLED:             mdcpy, readpvl_from_hdffile, parse_pvl, TKreportWarning
*
******************************************************************************/
void initMetadata (IO_HANDLE *sGranuleHandle, char filemode)
{
  char *metadataStr;
  char *genDateString = NULL, *inputFilesString = NULL, *dataCenterString = NULL;
  char *startOfHDF = NULL;
  int  goodHDFFile = 0;
  char dataAcc[20];
  char maxValChan[40];
  char minValChan[40];
  char sensAlign[40];
  char sensAlignOffset[40];
  char scanPath[40];
  char scanPathOffset[40];
  char latLonRes[15];
  char westCoord[40];
  char eastCoord[40];
  char northCoord[40];
  char southCoord[40];
  char qualityFlag[40];
  char gvNumVos[40];
  char gvRadSpaceX[40];
  char gvRadSpaceY[40];
  char gvRadSpaceZ[40];
  char gvRadSizeZ[40];
  char gvDzCal[40];
  char gvScale[40];
  char gvAlpha[40];
  char gvRadarName[40];
  char utcfFlag[40];
  char minReflThres[40];
  char radarWavelength[40];
  char minMaxUnits[40];
  char productVersion[40];
  char toolkitVersion[40];
  char meanMotion[40];
  char orbitAdjust[40];
  char coverageType[40];
  char tk_missing[10];
  char instrumentID[12];
  char solarGains[40];
  char tmiRolloverCoef[160];

  int i, string_length;

  STORE_IM_DM_QUERY queryRes;
  time_t timeh;
  struct tm *local_time;

#ifndef TSU_ENV
  IMsession	*sSession;
  IMget_Input_Granule_Struct	*queryData;
  IMinput_Granule_Return_Struct	*queryDataReturn;
  int	retCode;


  DMgranuleidSTRUCT *gidstruct = (DMgranuleidSTRUCT *) calloc (1, sizeof(DMgranuleidSTRUCT));
  div_t	tme_result;

  struct tm *cdate = (struct tm *)malloc(sizeof(struct tm));
  gidstruct->cdate = cdate;

#endif

  startOfHDF = strstr ( sGranuleHandle->gIDtokens.granule_ID, "HDF" );
  if ( ( startOfHDF != NULL ) && ( !strcmp ( startOfHDF, "HDF" ) ) )
      goodHDFFile = 1;

  strcpy ( queryRes.dataID, "\"NULL\"" );
  strcpy ( queryRes.contact, "\"NULL\"" );
  strcpy ( queryRes.browseName, "\"NULL\"" );
  strcpy ( queryRes.lonmaxlat, "-9999.9" );
  strcpy ( queryRes.datacentsrc, "\"NULL\"" );
  strcpy ( queryRes.inputFiles, "\"NULL\"" );
  strcpy ( queryRes.gendateinputfile, "\"NULL\"" );
  strcpy ( queryRes.leapSecsFlag, "0" );
  strcpy ( queryRes.algorithmVersion, "\"NULL\"" );
  strcpy ( queryRes.orbitNumber, "-9999" );
  strcpy ( queryRes.algorithmID, "\"NULL\"" );

  /*
   * Convert missing value, TK_DEFAULT, to charater.
   */
  sprintf(tk_missing,"%4.1f",TK_DEFAULT);

  /*
   * Some of the default metadata values depend on instrument type.
   * Set these values up here.
   */
  strcpy(dataAcc, "\"NULL\"");
  strcpy(maxValChan, "\"NULL\"");
  strcpy(minValChan, "\"NULL\"");
  strcpy(sensAlign, "\"NULL\"");
  strcpy(sensAlignOffset, "\"NULL\"" );
  strcpy(scanPath, "\"NULL\"");
  strcpy(scanPathOffset, "\"NULL\"" );
  strcpy(latLonRes, tk_missing);
  strcpy(westCoord, "-180");
  strcpy(eastCoord, "180");
  strcpy(northCoord, "40");
  strcpy(southCoord,"-40");
  strcpy ( qualityFlag, "\"NOT BEING INVESTIGATED\"" );
  strcpy ( gvNumVos, "-9999" );
  strcpy ( gvRadSpaceX, "-9999.9" );
  strcpy ( gvRadSpaceY, "-9999.9" );
  strcpy ( gvRadSpaceZ, "-9999.9" );
  strcpy ( gvRadSizeZ, "-9999" );
  strcpy ( gvDzCal, "-9999.9" );
  strcpy ( gvScale, "-9999.9" );
  strcpy ( gvAlpha, "-9999.9" );
  strcpy ( gvRadarName, "\"NULL\"" );
  strcpy ( utcfFlag, "0" );
  strcpy ( minReflThres, "-9999.9" );
  strcpy ( radarWavelength, "-9999.9" );
  strcpy ( minMaxUnits, "\"NULL\"" );
  strcpy ( productVersion, sGranuleHandle->gIDtokens.algVersion );
  strcpy ( meanMotion, "-9999.9" );
  strcpy ( orbitAdjust, "0" );
  strcpy ( coverageType, "\"Both\"" );
  sprintf ( toolkitVersion, "\"%s\"", TK_VERSION_NUMBER );
  TKremoveQuotes ( productVersion );
  strcpy ( solarGains, "\"(-9999.9,-9999.9,-9999.9,-9999.9)\"" );

  strcpy ( tmiRolloverCoef, "\"(-9999.9");
  for (i=1;i<18;i++) strcat ( tmiRolloverCoef, ",-9999.9");
  strcat ( tmiRolloverCoef, ")\"");

  switch (instType(sGranuleHandle->productID)) {
  case TK_TMI:
    strcpy(dataAcc, "\"0,0,0,0,0,0,0,0,0\"");
    strcpy(maxValChan, "\"-9999,-9999,-9999,-9999,-9999,-9999,-9999,-9999,-9999\"");
    strcpy(minValChan, "\"-9999,-9999,-9999,-9999,-9999,-9999,-9999,-9999,-9999\"");
    strcpy(sensAlign, "\"0.0,0.0,0.0,1,2,3\"");
    strcpy(sensAlignOffset, "\"0,0,0,0,0,0,0,0,0\"");
    strcpy(scanPath, "\"3,1,49.0,-65.0,130.0,0.68611111,0.0\"");
    strcpy(scanPathOffset, "\"0,0,0,0,0,0,0,0,0\"");
    strcpy(minMaxUnits, "\"Kelvin\"" );
    strcpy ( solarGains, "\"(-9999.9,-9999.9,-9999.9,-9999.9)\"" );
    switch ( sGranuleHandle->productID )
      {
      case TK_L1B_11:
	strcpy ( coverageType, "\"Horizontal\"" );
	break;
      case TK_L3A_11:
	strcpy(latLonRes,"\"5deg\"");
	strcpy ( coverageType, "\"Horizontal\"" );
	break;
      }
    break;
  case TK_VIRS: 
    strcpy(dataAcc, "\"10,10,2,2,2\"");
    strcpy(maxValChan, "\"-9999.9,-9999.9,-9999.9,-9999.9,-9999.9\"");
    strcpy(minValChan, "\"-9999,-9999,-9999,-9999,-9999\"");
    strcpy(sensAlign, "\"0.0,0.0,0.0,1,2,3\"");
    strcpy(sensAlignOffset, "\"0,0,0,0,0\"" );
    strcpy(scanPath, "\"1,3,90,-45.0,90.0,0.15,0.0\"");
    strcpy(scanPathOffset, "\"0,0,0,0,0\"" );
    strcpy(minMaxUnits, "\"(mW)(cm^-2)(m^-1 10^6)(sr^-1)\"");
    strcpy ( coverageType, "\"Horizontal\"" );
    strcpy ( solarGains, "\"(-9999.9,-9999.9,-9999.9,-9999.9)\"" );
    break;
  case TK_PR:
    strcpy(sensAlign, "\"0.0,0.0,0.0,1,2,3\"");
    strcpy(sensAlignOffset, "\"0\"" );
    strcpy(scanPath, "\"1,3,90.0,-17.0,34.0,0.3,0.0\"");
    strcpy(scanPathOffset, "\"0\"" );
    switch (sGranuleHandle->productID) {
    case TK_L1B_21:
      strcpy(dataAcc, "\"0.9dBm\"");
      strcpy(maxValChan, "\"-30\"");
      strcpy(minValChan, "\"-130\"");
      strcpy ( radarWavelength, "0.02173" );
      strcpy ( minMaxUnits, "\"dBm\"" );
      break;
    case TK_L1C_21:
      strcpy(dataAcc, "\"1.0dBZ\"");
      strcpy(maxValChan, "\"80\"");
      strcpy(minValChan, "\"20\"");
      strcpy ( radarWavelength, "0.02173" );
      strcpy ( minMaxUnits, "\"dBz\"" );
      break;
    case TK_L2A_21:
      strcpy ( coverageType, "\"Horizontal\"" );
      break;
    case TK_L2A_23:
      strcpy ( coverageType, "\"Horizontal\"" );
      break;
    case TK_L3A_25:
      strcpy(latLonRes, "\"5deg,0.5deg\"");
      break;
    case TK_L3A_26:
      strcpy(latLonRes, "\"5deg\"");
      break;
    case TK_L3A_27:
      strcpy(latLonRes, "\"5deg,0.5deg\"");
      break;
    }
    break;
  case TK_GV:
    strcpy(sensAlign, "\"NULL\"");
    strcpy(sensAlign, "\"NULL\"");
    strcpy(scanPath, "\"NULL\"");
    strcpy(scanPath, "\"NULL\"");
    strcpy ( gvRadSpaceX, "2" );
    strcpy ( gvRadSpaceY, "2" );
    strcpy ( gvRadSpaceZ, "1.5" );
    strcpy ( gvRadSizeZ, "13" );
    strcpy ( gvRadarName, sGranuleHandle->gIDtokens.radar );
    strcpy ( utcfFlag, "-9999" );
    strcpy ( meanMotion, "-9999.9" );
    strcpy ( orbitAdjust, "-9999" );
    switch (sGranuleHandle->productID) {
    case TK_L1B_GV:
      strcpy(maxValChan, "\"100dBZ,50m/s,20dB\"");
      strcpy(minValChan, "\"-40dBZ,-50m/s,-10dB\"");
      strcpy(latLonRes, tk_missing);
      strcpy ( gvNumVos, "2" );
      strcpy ( gvRadSpaceX, "-9999.9" );
      strcpy ( gvRadSpaceY, "-9999.9" );
      strcpy ( gvRadSpaceZ, "-9999.9" );
      strcpy ( gvRadSizeZ, "-9999" );
      strcpy ( gvAlpha, "0.0" );
      strcpy ( minMaxUnits, "\"dBZ,m/s,dB\"" );
      strcpy ( queryRes.dataID, "\"GV Reflectivity, Velocity, Differential Reflectivity\"" );
      break;
    case TK_L1C_GV:
      strcpy(maxValChan, "\"100dBZ,20dB\"");
      strcpy(minValChan, "\"-15dBZ,-10dB\"");
      strcpy(minMaxUnits, "\"dBZ,dB\"" );
      strcpy(latLonRes, tk_missing);
      strcpy ( gvNumVos, "-9999.9" );
      strcpy ( gvRadSpaceX, "-9999.9" );
      strcpy ( gvRadSpaceY, "-9999.9" );
      strcpy ( gvRadSpaceZ, "-9999.9" );
      strcpy ( gvRadSizeZ, "-9999" );
      strcpy ( gvDzCal, "0.0" );
      strcpy ( gvScale, "200.0" );
      strcpy ( gvAlpha, "0.0" );
      strcpy ( minReflThres, "-15" );
      strcpy ( queryRes.dataID, "\"GV Corrected Reflectivity and Differential Reflectivity, Quality Mask\"" );
      createBrowseName ( sGranuleHandle, queryRes.browseName );
      break;
    case TK_L2A_53S:
    case TK_L2A_53TX:
    case TK_L2A_53FL:
    case TK_L3A_53S:
    case TK_L3A_53TX:
    case TK_L3A_53FL:
    case TK_L3A_54S:
    case TK_L3A_54TX:
    case TK_L3A_54FL:
      strcpy ( qualityFlag, "\"BEING INVESTIGATED\"" );
    case TK_L2A_54S:
    case TK_L2A_54FL:
    case TK_L2A_54TX:
      strcpy ( coverageType, "\"Horizontal\"" );
      break;
    }
    break;
  case TK_VARIOUS:
    switch (sGranuleHandle->productID){
    case TK_L2B_31:
      strcpy(sensAlign, "\"0.0,0.0,0.0,1,2,3\"");
      strcpy(sensAlignOffset, "\"0\"" );
      strcpy(scanPath, "\"1,3,90.0,-17.0,34.0,0.3,0.0\"");
      strcpy(scanPathOffset, "\"0\"" );
      break;
    case TK_L3B_31:
      strcpy(latLonRes, "\"5deg\"");
      break;
    case TK_L3B_42:
      strcpy(latLonRes, "\".25deg\"");
      strcpy(northCoord, "50");
      strcpy(southCoord,"-50");
      strcpy ( coverageType, "\"Horizontal\"" );
      break;
    case TK_L3B_43:
      strcpy(latLonRes, "\".25deg\"");
      strcpy(northCoord, "50");
      strcpy(southCoord,"-50");
      strcpy ( coverageType,  "\"Horizontal\"" );
      break;
    case TK_L3A_46:
      strcpy(latLonRes, "\"1deg\"");
      strcpy(westCoord, "0");
      strcpy(eastCoord, "360");
      strcpy(northCoord, "90");
      strcpy(southCoord,"-90");
      strcpy ( coverageType, "\"Horizontal\"" );
      break;
    }
    break;
  }


  if (sGranuleHandle->mode != TK_READ_ONLY)
    {
#ifndef TSU_ENV
      /**
       ** This part is only used in TSDIS development environment.
       ** Toolkit queries the database for metadata information.
       **/
      retCode = IMopen(TK_IM_USER_ID, &sSession);
      if (retCode == IM_SUCCESS)
	{

/* IMPORTANT : Please note, this query got changed for the version 6 algorithms */
-----specifically 3b42--------in the August 2004 time frame-------- P.Kolbe -----
/usr/people/pkolbe/work/im/sql/query=> cvs diff -r 1.17 -r 1.16 proc_q_InputGran.sql
<           ** for the  Metadata field.
<           IF (@GranID like '3B42%')
<           BEGIN
<              select gmd.Granule_ID,
*/
 
	  queryData = IMallocateRec(GET_INPUT_GRANULE);
	  if (queryData != NULL)
	    {
	      strcpy(queryData->Granule_ID, sGranuleHandle->gIDtokens.granule_ID);
	      
	      retCode = IMquery(sSession, (IMdata *)queryData, 
				(IMreturn **)&queryDataReturn);
	      if (retCode != IM_SUCCESS)
		{
		  if ( retCode == W_IM_QUERYRESULTS_NOTFOUND )
		    {
		      /*  No records in the database.  */
		      if ( ( strstr ( sGranuleHandle->gIDtokens.granuleName, "_QL" ) == NULL ) &&
                          ( sGranuleHandle->granuleFlag!=TK_EGG_EMPTY ) &&
			  ( strstr ( sGranuleHandle->gIDtokens.granuleName, "_CSI" ) == NULL ) &&
			  ( goodHDFFile ) )
			TKreportError ( W_TK_NORECORDSINDB );
		      strcpy ( queryRes.contact, "\"NULL\"" );
		      strcpy ( queryRes.lonmaxlat, "-9999.9" );
		      strcpy ( queryRes.datacentsrc, "\"NULL\"" );
		      strcpy ( queryRes.inputFiles, "\"NULL\"" );
		      strcpy ( queryRes.gendateinputfile, "\"NULL\"" );
		      strcpy ( queryRes.leapSecsFlag, "0" );
		      
		      strcpy ( queryRes.algorithmVersion, "\"NULL\"" );
		      strcpy ( queryRes.orbitNumber, sGranuleHandle->gIDtokens.orbitNumber );
		      strcpy ( queryRes.algorithmID, sGranuleHandle->gIDtokens.algID );
		      
		      timeh = time ( &timeh );
		      local_time = localtime ( &timeh );
		      sprintf ( sGranuleHandle->genDate, "\"%s\"",
			       asctime( local_time ) );
		    }
		  else
		    /* Unable to query the TSDIS database. */
		    TKreportError(W_TK_FAILIMQUERY);

		  IMfreeRec(queryData);
		  IMclose(sSession);
		}
	      else
		{
		  /***
		   ****	copy info from IM queryDataReturn to the corresponding fields
		   ***/

		  removeIMSpaces ( 
				  queryDataReturn->ECS_Description );
		  sprintf (queryRes.dataID, "\"%s\"", 
			   queryDataReturn->ECS_Description);

		  removeIMSpaces ( 
				  queryDataReturn->Algorithm_ID );

		  sprintf (queryRes.algorithmID, "\"%s\"",
			   queryDataReturn->Algorithm_ID);
		  sprintf(queryRes.algorithmVersion, "\"%s\"", 
			  queryDataReturn->Algorithm_Version);
		  sprintf(queryRes.orbitNumber, "%d",
			 queryDataReturn->Orbit_Number );

/* altitude increase change */
/*
		  sprintf ( productVersion, "%d",
			   queryDataReturn->Product_Version ); 
*/
		  sprintf(productVersion, "\"%s\"", 
			  queryDataReturn->Product_Version);

		  removeIMSpaces (
				  queryDataReturn->Contact_Name );

		  sprintf(queryRes.contact, "\"%s\"", 
			  queryDataReturn->Contact_Name);     

		  if(queryDataReturn->Processing_Datim != NULL)
		    sprintf ( sGranuleHandle->genDate, "\"%s\"",
			     asctime(queryDataReturn->Processing_Datim ) );
		      
		  if ( prodType ( sGranuleHandle->productID ) != TK_L3 )
		    {
		      buildInputFileStrings ( &genDateString, &inputFilesString,
					     &dataCenterString, queryDataReturn );
		      strcpy(queryRes.gendateinputfile, genDateString);
		      strcpy(queryRes.inputFiles, inputFilesString );
		      strcpy(queryRes.datacentsrc, dataCenterString );
		    }
		  
		  /**
		   ** Format char 'queryDataReturn->Time_Event_Flag' to string
		   ** 'queryRes.leapSecsFlag'.
		   **/
		  sprintf(queryRes.leapSecsFlag, "%c", queryDataReturn->Time_Event_Flag);
		  
		  /**
		   ** Format floating point 'queryDataReturn.Long_Of_Max_Lat' to string
		   ** 'queryDataReturn->Long_Of_Max_Lat'. 
		   **/
		  sprintf(queryRes.lonmaxlat, "%f", queryDataReturn->Long_Of_Max_Lat);
		  
		  IMrelease((IMreturn **)&queryDataReturn);
		  IMfreeRec(queryData);
		  IMclose(sSession);
		}
	    }
	}
      else
    	/* Failed to open TSDIS database. */
    	TKreportWarning(W_TK_FAILIMOPEN);
      /**
       ** Update browse name to metadata (coreMetadata.browseName) via DM 
       ** To access DM, 2 fields of gidStruct have to be setted before
       ** calling DMmakeGranuleName.  The fields are:
       **	datatype, proc_stream(processing stream), cdate (collection date)
       **/
      
      if ( ( sGranuleHandle->productID != TK_L1B_21 ) &&
	  ( sGranuleHandle->productID != TK_L2A_56 ) &&
	  ( sGranuleHandle->productID != TK_L1B_GV ) &&
	  ( sGranuleHandle->productID != TK_L3A_46 ) &&
	  ( sGranuleHandle->productID != TK_L2A_57 ) &&
	  ( strstr ( sGranuleHandle->gIDtokens.granuleName, "_QL" ) == NULL ) &&
	  ( strstr ( sGranuleHandle->gIDtokens.granuleName, "_CSI" ) == NULL ) )
	{
	  /**
	   ** Query the DM
	   **/
	  strcpy(gidstruct->datatype, sGranuleHandle->gIDtokens.datatype); 
	  strcat ( gidstruct->datatype, "_BR" );
	  
	  /**
	   ** Set processing stream to 'IP' for initial processing type.
	   **/
	  strcpy(gidstruct->proc_stream, "IP");
	  
	  /**
	   ** Parse the collection date string from yymmdd format to 
	   ** tm time format.
	   **/
	  
	  tme_result = div(sGranuleHandle->gIDtokens.cdate, 100);
	  
	  gidstruct->cdate->tm_mday = tme_result.rem;
	  
	  tme_result = div(tme_result.quot, 100);
	  
	  gidstruct->cdate->tm_mon = tme_result.rem;
	  --(gidstruct->cdate->tm_mon);
	  gidstruct->cdate->tm_year = tme_result.quot;
	  
	  if ( instType ( sGranuleHandle->productID ) == TK_GV )
	    {
	      strcpy ( instrumentID, sGranuleHandle->gIDtokens.radar );
	      TKremoveQuotes ( instrumentID );
	      strcpy ( gidstruct->instrument, instrumentID );
	    }

	  retCode =  DMmakeGranuleName(gidstruct);
	  
	  if (retCode != DM_SUCCESS)
	    /* Unable to generate a browse name from DM. */
	    TKreportWarning(W_TK_FAILDMMAKENAME);
	  else
	    sprintf(queryRes.browseName, "\"%s\"", gidstruct->granuleID);
	  
	  free ( gidstruct );
	  free(cdate);
	}
#else 
      strcpy ( queryRes.contact, "\"NULL\"" );
      strcpy ( queryRes.lonmaxlat, "-9999.9" );
      strcpy ( queryRes.datacentsrc, "\"NULL\"" );
      strcpy ( queryRes.inputFiles, "\"NULL\"" );
      strcpy ( queryRes.gendateinputfile, "\"NULL\"" );
      strcpy ( queryRes.leapSecsFlag, "0" );
      strcpy ( queryRes.algorithmVersion, "\"NULL\"" );

      strcpy  ( queryRes.orbitNumber, sGranuleHandle->gIDtokens.orbitNumber );
      strcpy ( queryRes.algorithmID, sGranuleHandle->gIDtokens.algID );

      timeh = time ( &timeh );
      local_time = localtime ( &timeh );
      sprintf ( sGranuleHandle->genDate, "\"%s\"",
	       asctime( local_time ) );
#ifdef NASDA_ENV
      query_ascii_db ( sGranuleHandle, &queryRes );
#endif
#endif

    }

  /*
   * First initialize the Core and PS metadata structures so the element 
   * names are correct.  This is done if we are reading or writing the 
   * metadata.  If filemode = TK_NEW_FILE, then these default values are
   * not changed.  If filemode = TK_READ_ONLY, then these default values
   * are overwritten by the actual values in the file being read.
   */

  /* Initialize the CoreMetadata Structure */
  setFlag(&sGranuleHandle->metadata.coreMetadata.orbitNumber, TK_TRUE);
  mdcpy(&sGranuleHandle->metadata.coreMetadata.orbitNumber, 
	"OrbitNumber", queryRes.orbitNumber );

  setFlag(&sGranuleHandle->metadata.coreMetadata.beginDate, TK_FALSE);
  mdcpy(&sGranuleHandle->metadata.coreMetadata.beginDate, 
	"RangeBeginningDate", "9999/99/99");

  setFlag(&sGranuleHandle->metadata.coreMetadata.beginTime, TK_FALSE);
  mdcpy(&sGranuleHandle->metadata.coreMetadata.beginTime, 
	"RangeBeginningTime", "99:99:99");

  setFlag(&sGranuleHandle->metadata.coreMetadata.endDate, TK_FALSE);
  mdcpy(&sGranuleHandle->metadata.coreMetadata.endDate, 
	"RangeEndingDate", "9999/99/99");

  setFlag(&sGranuleHandle->metadata.coreMetadata.endTime, TK_FALSE);
  mdcpy(&sGranuleHandle->metadata.coreMetadata.endTime, 
	"RangeEndingTime", "99:99:99");

  setFlag(&sGranuleHandle->metadata.coreMetadata.granuleID, TK_TRUE);
  mdcpyLong(&sGranuleHandle->metadata.coreMetadata.granuleID, 
	"GranulePointer", sGranuleHandle->gIDtokens.granuleName, TK_GRANULE_ID);

  setFlag(&sGranuleHandle->metadata.coreMetadata.dataID, TK_TRUE);
  mdcpyLong(&sGranuleHandle->metadata.coreMetadata.dataID,
        "ShortName", queryRes.dataID, TK_DATA_ID);

  setFlag(&sGranuleHandle->metadata.coreMetadata.fileSize, TK_FALSE);
  mdcpy(&sGranuleHandle->metadata.coreMetadata.fileSize, 
	"SizeMBECSDataGranule", "-9999.9");

  setFlag(&sGranuleHandle->metadata.coreMetadata.lonMaxLat, TK_FALSE);
  mdcpy(&sGranuleHandle->metadata.coreMetadata.lonMaxLat, 
        "LongitudeOfMaximumLatitude", queryRes.lonmaxlat);

  setFlag(&sGranuleHandle->metadata.coreMetadata.spatCovType, TK_TRUE);
  mdcpy(&sGranuleHandle->metadata.coreMetadata.spatCovType, 
	"SpatialCoverageType", coverageType );

  setFlag(&sGranuleHandle->metadata.coreMetadata.ellipsoidName, TK_TRUE);
  mdcpy(&sGranuleHandle->metadata.coreMetadata.ellipsoidName,
	"EllipsoidName", "\"World Geodetic System (WGS) 84\"" );

  setFlag(&sGranuleHandle->metadata.coreMetadata.earthEqRad, TK_TRUE);
  mdcpy(&sGranuleHandle->metadata.coreMetadata.earthEqRad, 
	"EquatorialRadius", "6378.137");

  setFlag(&sGranuleHandle->metadata.coreMetadata.flatteningRatio, TK_TRUE);
  mdcpy(&sGranuleHandle->metadata.coreMetadata.flatteningRatio, 
	"DenominatorFlatteningRatio", "0.00335281");

  setFlag(&sGranuleHandle->metadata.coreMetadata.orbitModelName, TK_TRUE);
  mdcpyLong(&sGranuleHandle->metadata.coreMetadata.orbitModelName, 
	"OrbitalModelName", "\"Definitive FDF Ephemeris\"", TK_ORBIT_MODEL_NAME);

  setFlag(&sGranuleHandle->metadata.coreMetadata.kepSemiMajorAxis, TK_FALSE);
  mdcpy(&sGranuleHandle->metadata.coreMetadata.kepSemiMajorAxis, 
	"SemiMajorAxis", "-9999.9");

  setFlag(&sGranuleHandle->metadata.coreMetadata.kepMeanAnomaly, TK_FALSE);
  mdcpy(&sGranuleHandle->metadata.coreMetadata.kepMeanAnomaly, 
	"MeanAnomaly", "-9999.9");

  setFlag(&sGranuleHandle->metadata.coreMetadata.kepRightAscenNode, TK_FALSE);
  mdcpy(&sGranuleHandle->metadata.coreMetadata.kepRightAscenNode, 
	"RightAscensionNode", "-9999.9");

  setFlag(&sGranuleHandle->metadata.coreMetadata.kepArgOfPerigee, TK_FALSE);
  mdcpy(&sGranuleHandle->metadata.coreMetadata.kepArgOfPerigee, 
	"ArgumentOfPerigee", "-9999.9");

  setFlag(&sGranuleHandle->metadata.coreMetadata.kepEccentricity, TK_FALSE);
  mdcpy(&sGranuleHandle->metadata.coreMetadata.kepEccentricity, 
	"Eccentricity", "-9999.9");

  setFlag(&sGranuleHandle->metadata.coreMetadata.kepInclination, TK_FALSE);
  mdcpy(&sGranuleHandle->metadata.coreMetadata.kepInclination, 
	"Inclination", "-9999.9");

  setFlag(&sGranuleHandle->metadata.coreMetadata.kepEpochTime, TK_FALSE);
  mdcpy(&sGranuleHandle->metadata.coreMetadata.kepEpochTime, 
	"EpochTime", "99:99:99");

  setFlag(&sGranuleHandle->metadata.coreMetadata.kepEpochDate, TK_FALSE);
  mdcpy(&sGranuleHandle->metadata.coreMetadata.kepEpochDate, 
	"EpochDate", "9999/99/99");

  setFlag(&sGranuleHandle->metadata.coreMetadata.kepEpochMillisec, TK_FALSE);
  mdcpy(&sGranuleHandle->metadata.coreMetadata.kepEpochMillisec, 
	"EpochMillisec", "-9999");

  setFlag(&sGranuleHandle->metadata.coreMetadata.westBoundCoord, TK_TRUE);
  mdcpy(&sGranuleHandle->metadata.coreMetadata.westBoundCoord, 
	"WestBoundingCoordinate", westCoord);

  setFlag(&sGranuleHandle->metadata.coreMetadata.eastBoundCoord, TK_TRUE);
  mdcpy(&sGranuleHandle->metadata.coreMetadata.eastBoundCoord, 
	"EastBoundingCoordinate", eastCoord);

  setFlag(&sGranuleHandle->metadata.coreMetadata.northBoundCoord, TK_TRUE);
  mdcpy(&sGranuleHandle->metadata.coreMetadata.northBoundCoord, 
	"NorthBoundingCoordinate", northCoord);

  setFlag(&sGranuleHandle->metadata.coreMetadata.southBoundCoord, TK_TRUE);
  mdcpy(&sGranuleHandle->metadata.coreMetadata.southBoundCoord, 
	"SouthBoundingCoordinate", southCoord);

  setFlag(&sGranuleHandle->metadata.coreMetadata.centerPointLat, TK_FALSE);
  mdcpy(&sGranuleHandle->metadata.coreMetadata.centerPointLat, 
	"CenterLatitude", "-9999.9");

  setFlag(&sGranuleHandle->metadata.coreMetadata.centerPointLon, TK_FALSE);
  mdcpy(&sGranuleHandle->metadata.coreMetadata.centerPointLon, 
	"CenterLongitude", "-9999.9");

  setFlag(&sGranuleHandle->metadata.coreMetadata.radius, TK_FALSE);
  mdcpy(&sGranuleHandle->metadata.coreMetadata.radius, 
	"RadiusValue", "-9999.9");

  setFlag(&sGranuleHandle->metadata.coreMetadata.latitudeRes, TK_TRUE);
  mdcpy(&sGranuleHandle->metadata.coreMetadata.latitudeRes, 
	"LatitudeResolution", latLonRes);

  setFlag(&sGranuleHandle->metadata.coreMetadata.longitudeRes, TK_TRUE);
  mdcpy(&sGranuleHandle->metadata.coreMetadata.longitudeRes, 
	"LongitudeResolution", latLonRes);

  setFlag(&sGranuleHandle->metadata.coreMetadata.geoCoordUnits, TK_TRUE);
  mdcpy(&sGranuleHandle->metadata.coreMetadata.geoCoordUnits, 
	"GeographicCoordinateUnits", "\"Decimal Degrees\"");

  setFlag(&sGranuleHandle->metadata.coreMetadata.temporRngType, TK_TRUE);
  mdcpy(&sGranuleHandle->metadata.coreMetadata.temporRngType, 
	"TemporalRangeType", "\"Continuous Range\"");

  setFlag(&sGranuleHandle->metadata.coreMetadata.qaParamName, TK_TRUE);
  mdcpyLong(&sGranuleHandle->metadata.coreMetadata.qaParamName, 
	"QualityAssuranceParameterName", "\"ScienceQualityFlag\"", TK_QA_PARAM_NAME);

  setFlag(&sGranuleHandle->metadata.coreMetadata.qaParamValue, TK_TRUE);
  mdcpy(&sGranuleHandle->metadata.coreMetadata.qaParamValue, 
	"QualityAssuranceParameterValue", qualityFlag);

  setFlag(&sGranuleHandle->metadata.coreMetadata.reproStat, TK_FALSE);
  mdcpy(&sGranuleHandle->metadata.coreMetadata.reproStat, 
	"ReprocessingActual", "\"NULL\"");

  setFlag(&sGranuleHandle->metadata.coreMetadata.browseName, TK_TRUE);
  mdcpyLong(&sGranuleHandle->metadata.coreMetadata.browseName, 
	"BrowsePointer", queryRes.browseName, TK_BROWSE_NAME);

  setFlag(&sGranuleHandle->metadata.coreMetadata.contact, TK_TRUE);
  mdcpyLong(&sGranuleHandle->metadata.coreMetadata.contact, 
        "ScienceContact", queryRes.contact, TK_CONTACT);

  setFlag(&sGranuleHandle->metadata.coreMetadata.meanMotion, TK_FALSE);
  mdcpy(&sGranuleHandle->metadata.coreMetadata.meanMotion, 
	"MeanMotion", meanMotion);

  setFlag(&sGranuleHandle->metadata.coreMetadata.orbitAdjFlag, TK_FALSE);
  mdcpy(&sGranuleHandle->metadata.coreMetadata.orbitAdjFlag, 
	"OrbitAdjustFlag", orbitAdjust);

  setFlag(&sGranuleHandle->metadata.coreMetadata.attitudeMode, TK_FALSE);
  mdcpy(&sGranuleHandle->metadata.coreMetadata.attitudeMode, 
	"AttitudeModeFlag", "-99");

  setFlag(&sGranuleHandle->metadata.coreMetadata.beginSolarBeta, TK_FALSE);
  mdcpy(&sGranuleHandle->metadata.coreMetadata.beginSolarBeta, 
	"SolarBetaAngleAtBeginningOfGranule", "-9999.9");

  setFlag(&sGranuleHandle->metadata.coreMetadata.endSolarBeta, TK_FALSE);
  mdcpy(&sGranuleHandle->metadata.coreMetadata.endSolarBeta, 
	"SolarBetaAngleAtEndOfGranule", "-9999.9");

  setFlag(&sGranuleHandle->metadata.coreMetadata.sensorAlgn, TK_TRUE);
  mdcpyLong(&sGranuleHandle->metadata.coreMetadata.sensorAlgn, 
	"SensorAlignment", sensAlign, TK_SENSOR_ALGN);

  setFlag(&sGranuleHandle->metadata.coreMetadata.sensorAlgnChanOff, TK_TRUE);
  mdcpy(&sGranuleHandle->metadata.coreMetadata.sensorAlgnChanOff, 
	"SensorAlignmentChannelOffsets", sensAlignOffset);

  setFlag(&sGranuleHandle->metadata.coreMetadata.scanPathModel, TK_TRUE);
  mdcpyLong(&sGranuleHandle->metadata.coreMetadata.scanPathModel, 
	"ScanPathModel", scanPath, TK_SCAN_PATH_MODEL);

  setFlag(&sGranuleHandle->metadata.coreMetadata.scanPathParam, TK_TRUE);
  mdcpy(&sGranuleHandle->metadata.coreMetadata.scanPathParam, 
	"ScanPathModelParam", scanPathOffset);

  setFlag(&sGranuleHandle->metadata.coreMetadata.ephemFilename, TK_FALSE);
  mdcpy(&sGranuleHandle->metadata.coreMetadata.ephemFilename, 
	"EphemerisFileID", "\"NULL\"");
    
  /* Initialize PS metadata Structure */
  setFlag(&sGranuleHandle->metadata.psMetadata.numDataGap, TK_FALSE);
  mdcpy(&sGranuleHandle->metadata.psMetadata.numDataGap, 
	"NumberOfDataGaps", "-9999.9");

  setFlag(&sGranuleHandle->metadata.psMetadata.algorithmVersion, TK_TRUE);
  mdcpy(&sGranuleHandle->metadata.psMetadata.algorithmVersion, 
	"AlgorithmVersion", queryRes.algorithmVersion);

  setFlag(&sGranuleHandle->metadata.psMetadata.productVersion, TK_TRUE);
  mdcpy(&sGranuleHandle->metadata.psMetadata.productVersion,
	"ProductVersion", productVersion );

  setFlag(&sGranuleHandle->metadata.psMetadata.toolkitVersion, TK_TRUE);
  mdcpy(&sGranuleHandle->metadata.psMetadata.toolkitVersion,
	"ToolkitVersion", toolkitVersion );

  setFlag(&sGranuleHandle->metadata.psMetadata.calCoeffVersion, TK_FALSE);
  mdcpy(&sGranuleHandle->metadata.psMetadata.calCoeffVersion,
	"CalibrationCoefficientVersion", "-9999" );

  setFlag(&sGranuleHandle->metadata.psMetadata.missingData, TK_FALSE);
  mdcpy(&sGranuleHandle->metadata.psMetadata.missingData, 
	"MissingData", "-9999");

  setFlag(&sGranuleHandle->metadata.psMetadata.percentBadMissPix, TK_FALSE);
  mdcpyLong(&sGranuleHandle->metadata.psMetadata.percentBadMissPix, 
	"PercentOfBadOrMissingPixels", "\"NULL\"", TK_PERCENT_BAD_MISS_PIXEL);

  setFlag(&sGranuleHandle->metadata.psMetadata.maxValidChannel, TK_TRUE);
  mdcpyLong(&sGranuleHandle->metadata.psMetadata.maxValidChannel, 
	"MaximumValidValueofChannel", maxValChan, TK_MAX_VALID_CHANNEL);

  setFlag(&sGranuleHandle->metadata.psMetadata.minValidChannel, TK_TRUE);
  mdcpyLong(&sGranuleHandle->metadata.psMetadata.minValidChannel, 
	"MinimumValidValueofChannel", minValChan, TK_MIN_VALID_CHANNEL);

  setFlag(&sGranuleHandle->metadata.psMetadata.minMaxUnits, TK_TRUE);
  mdcpy(&sGranuleHandle->metadata.psMetadata.minMaxUnits,
	"MinMaxUnits", minMaxUnits );

  setFlag(&sGranuleHandle->metadata.psMetadata.orbitSize, TK_FALSE);
  mdcpy(&sGranuleHandle->metadata.psMetadata.orbitSize, 
	"OrbitSize", "-9999");

  setFlag(&sGranuleHandle->metadata.psMetadata.radarWavelength, TK_FALSE);
  mdcpy(&sGranuleHandle->metadata.psMetadata.radarWavelength, 
	"RadarWavelength", radarWavelength);

  setFlag(&sGranuleHandle->metadata.psMetadata.minRefThreshold, TK_FALSE);
  mdcpy(&sGranuleHandle->metadata.psMetadata.minRefThreshold, 
	"MinimumReflectivityThreshold", minReflThres);

  setFlag(&sGranuleHandle->metadata.psMetadata.algorithmID, TK_TRUE);
  mdcpy(&sGranuleHandle->metadata.psMetadata.algorithmID, 
	"AlgorithmID", queryRes.algorithmID);

  setFlag(&sGranuleHandle->metadata.psMetadata.dataAccuracy, TK_TRUE);
  mdcpy(&sGranuleHandle->metadata.psMetadata.dataAccuracy, 
	"DataAccuracy", dataAcc);

  setFlag(&sGranuleHandle->metadata.psMetadata.dataGap, TK_FALSE);
  mdcpy(&sGranuleHandle->metadata.psMetadata.dataGap, 
	"DataGaps", "-9999.9");

  setFlag(&sGranuleHandle->metadata.psMetadata.inputID, TK_TRUE);
  mdcpyLong(&sGranuleHandle->metadata.psMetadata.inputID, 
        "InputFiles", queryRes.inputFiles, TK_INPUT_FILES);

  setFlag(&sGranuleHandle->metadata.psMetadata.dateGenInputFile, TK_TRUE);
  mdcpyLong(&sGranuleHandle->metadata.psMetadata.dateGenInputFile, 
        "DateOfGenerationOfInputFiles", queryRes.gendateinputfile, 
		TK_GEN_DATE_INPUT_FILES);

  setFlag(&sGranuleHandle->metadata.psMetadata.dataCenterSrc, TK_TRUE);
  mdcpyLong(&sGranuleHandle->metadata.psMetadata.dataCenterSrc, 
        "DataCenterSourceOfInputFiles", queryRes.datacentsrc, TK_DATA_CENTER_SRC);

  setFlag(&sGranuleHandle->metadata.psMetadata.generationDate, TK_TRUE);
  mdcpy(&sGranuleHandle->metadata.psMetadata.generationDate, 
	"GenerationDate", sGranuleHandle->genDate);

  setFlag(&sGranuleHandle->metadata.psMetadata.dayNight, TK_FALSE);
  mdcpy(&sGranuleHandle->metadata.psMetadata.dayNight, 
	"DayNight", "-9999.9");

  setFlag(&sGranuleHandle->metadata.psMetadata.solarGains, TK_FALSE);
  mdcpy(&sGranuleHandle->metadata.psMetadata.solarGains, 
	"SolarChannelGains", "\"(-9999.9,-9999.9,-9999.9,-9999.9)\"");

  setFlag(&sGranuleHandle->metadata.psMetadata.tmiRolloverCoef, TK_FALSE);
  mdcpy(&sGranuleHandle->metadata.psMetadata.tmiRolloverCoef,
        "TMIRolloverCoef", tmiRolloverCoef);

  setFlag(&sGranuleHandle->metadata.psMetadata.firstScanUTCdate, TK_FALSE);
  mdcpy(&sGranuleHandle->metadata.psMetadata.firstScanUTCdate, 
	"OrbitFirstScanUTCDate", "9999/99/99");

  setFlag(&sGranuleHandle->metadata.psMetadata.firstScanUTCtime, TK_FALSE);
  mdcpy(&sGranuleHandle->metadata.psMetadata.firstScanUTCtime, 
	"OrbitFirstScanUTCTime", "99:99:99");

  setFlag(&sGranuleHandle->metadata.psMetadata.firstScanUTCmsec, TK_FALSE);
  mdcpy(&sGranuleHandle->metadata.psMetadata.firstScanUTCmsec, 
	"OrbitFirstScanUTCMilliseconds", "-9999");

  setFlag(&sGranuleHandle->metadata.psMetadata.firstScanSCsec, TK_FALSE);
  mdcpy(&sGranuleHandle->metadata.psMetadata.firstScanSCsec, 
	"OrbitFirstSCSecs", "-9999");

  setFlag(&sGranuleHandle->metadata.psMetadata.firstScanSCsubsec, TK_FALSE);
  mdcpy(&sGranuleHandle->metadata.psMetadata.firstScanSCsubsec, 
	"OrbitFirstSCSubsecs", "-9999");

  setFlag(&sGranuleHandle->metadata.psMetadata.lastScanUTCdate, TK_FALSE);
  mdcpy(&sGranuleHandle->metadata.psMetadata.lastScanUTCdate, 
	"OrbitLastScanUTCDate", "9999/99/99");

  setFlag(&sGranuleHandle->metadata.psMetadata.lastScanUTCtime, TK_FALSE);
  mdcpy(&sGranuleHandle->metadata.psMetadata.lastScanUTCtime, 
	"OrbitLastScanUTCTime", "99:99:99");

  setFlag(&sGranuleHandle->metadata.psMetadata.lastScanUTCmsec, TK_FALSE);
  mdcpy(&sGranuleHandle->metadata.psMetadata.lastScanUTCmsec, 
	"OrbitLastScanUTCmilliseconds", "-9999");

  setFlag(&sGranuleHandle->metadata.psMetadata.lastScanSCsec, TK_FALSE);
  mdcpy(&sGranuleHandle->metadata.psMetadata.lastScanSCsec, 
	"OrbitLastSCSecs", "-9999");

  setFlag(&sGranuleHandle->metadata.psMetadata.lastScanSCsubsec, TK_FALSE);
  mdcpy(&sGranuleHandle->metadata.psMetadata.lastScanSCsubsec, 
	"OrbitLastSCSubsecs", "-9999");

  setFlag(&sGranuleHandle->metadata.psMetadata.utcfSec, TK_FALSE);
  mdcpy(&sGranuleHandle->metadata.psMetadata.utcfSec, 
	"UTCFSeconds", "-9999");

  setFlag(&sGranuleHandle->metadata.psMetadata.utcfSubsec, TK_FALSE);
  mdcpy(&sGranuleHandle->metadata.psMetadata.utcfSubsec, 
	"UTCFSubseconds", "-9999");

  setFlag(&sGranuleHandle->metadata.psMetadata.utcfFlag, TK_FALSE);
  mdcpy(&sGranuleHandle->metadata.psMetadata.utcfFlag, 
	"UTCFflag", utcfFlag);

  setFlag(&sGranuleHandle->metadata.psMetadata.leapSecsFlag, TK_TRUE);
  mdcpy(&sGranuleHandle->metadata.psMetadata.leapSecsFlag, 
        "LeapSecondsFlag", queryRes.leapSecsFlag);

  setFlag(&sGranuleHandle->metadata.psMetadata.radarName, TK_FALSE);
  mdcpy(&sGranuleHandle->metadata.psMetadata.radarName, 
	"RadarSiteName", gvRadarName);

  setFlag(&sGranuleHandle->metadata.psMetadata.radarCity, TK_FALSE);
  mdcpyLong(&sGranuleHandle->metadata.psMetadata.radarCity, 
	"RadarCity", "\"NULL\"", TK_RADAR_CITY);

  setFlag(&sGranuleHandle->metadata.psMetadata.radarState, TK_FALSE);
  mdcpy(&sGranuleHandle->metadata.psMetadata.radarState, 
	"RadarState", "\"NULL\"");

  setFlag(&sGranuleHandle->metadata.psMetadata.radarCountry, TK_FALSE);
  mdcpy(&sGranuleHandle->metadata.psMetadata.radarCountry, 
	"RadarCountry", "\"NULL\"");

  setFlag(&sGranuleHandle->metadata.psMetadata.numVOS, TK_FALSE);
  mdcpy(&sGranuleHandle->metadata.psMetadata.numVOS, 
	"NumberOfVOS", gvNumVos);

  setFlag(&sGranuleHandle->metadata.psMetadata.radarOriginLat, TK_FALSE);
  mdcpy(&sGranuleHandle->metadata.psMetadata.radarOriginLat, 
	"RadarGridOriginLatitude", "-9999.9");

  setFlag(&sGranuleHandle->metadata.psMetadata.radarOriginLon, TK_FALSE);
  mdcpy(&sGranuleHandle->metadata.psMetadata.radarOriginLon, 
	"RadarGridOriginLongitude", "-9999.9");

  setFlag(&sGranuleHandle->metadata.psMetadata.radarOriginAlt, TK_FALSE);
  mdcpy(&sGranuleHandle->metadata.psMetadata.radarOriginAlt, 
	"RadarGridOriginAltitude", "-9999.9");

  setFlag(&sGranuleHandle->metadata.psMetadata.radarSpacingX, TK_FALSE);
  mdcpy(&sGranuleHandle->metadata.psMetadata.radarSpacingX, 
	"RadarGridSpacingX", gvRadSpaceX);

  setFlag(&sGranuleHandle->metadata.psMetadata.radarSpacingY, TK_FALSE);
  mdcpy(&sGranuleHandle->metadata.psMetadata.radarSpacingY, 
	"RadarGridSpacingY", gvRadSpaceY);

  setFlag(&sGranuleHandle->metadata.psMetadata.radarSpacingZ, TK_FALSE);
  mdcpy(&sGranuleHandle->metadata.psMetadata.radarSpacingZ, 
	"RadarGridSpacingZ", gvRadSpaceZ);

  setFlag(&sGranuleHandle->metadata.psMetadata.radarGridSizeX, TK_FALSE);
  mdcpy(&sGranuleHandle->metadata.psMetadata.radarGridSizeX, 
	"RadarGridSizeX", "-9999");

  setFlag(&sGranuleHandle->metadata.psMetadata.radarGridSizeY, TK_FALSE);
  mdcpy(&sGranuleHandle->metadata.psMetadata.radarGridSizeY, 
	"RadarGridSizeY", "-9999");

  setFlag(&sGranuleHandle->metadata.psMetadata.radarGridSizeZ, TK_TRUE);
  mdcpy(&sGranuleHandle->metadata.psMetadata.radarGridSizeZ, 
	"RadarGridSizeZ", gvRadSizeZ);

  setFlag(&sGranuleHandle->metadata.psMetadata.dzcal, TK_FALSE);
  mdcpy(&sGranuleHandle->metadata.psMetadata.dzcal,
        "DZCal", gvDzCal);

  setFlag(&sGranuleHandle->metadata.psMetadata.gvL1Cscale, TK_FALSE);
  mdcpy(&sGranuleHandle->metadata.psMetadata.gvL1Cscale,
        "GVL1C_Scale", gvScale);

  setFlag(&sGranuleHandle->metadata.psMetadata.alpha, TK_FALSE);
  mdcpy(&sGranuleHandle->metadata.psMetadata.alpha,
        "Alpha", gvAlpha);

  setFlag(&sGranuleHandle->metadata.psMetadata.runtimeOption, TK_TRUE);
  mdcpyLong(&sGranuleHandle->metadata.psMetadata.runtimeOption,
        "RuntimeOptions", "\"NULL\"", TK_GV_RUNTIME_OPT);

  setFlag(&sGranuleHandle->metadata.psMetadata.anomalyFlag, TK_FALSE);
  mdcpyLong(&sGranuleHandle->metadata.psMetadata.anomalyFlag,
        "AnomalyFlag", "\"NOT EMPTY\"", TK_ANOMALY_FLAG);

  setFlag(&sGranuleHandle->metadata.psMetadata.softwareVersion, TK_FALSE);
  mdcpyLong(&sGranuleHandle->metadata.psMetadata.softwareVersion,
        "SoftwareVersion", "-9999", TK_SOFTWARE_VERSION);

  setFlag(&sGranuleHandle->metadata.psMetadata.databaseVersion, TK_FALSE);
  mdcpyLong(&sGranuleHandle->metadata.psMetadata.databaseVersion,
        "DatabaseVersion", "-9999", TK_DATABASE_VERSION);

  setFlag(&sGranuleHandle->metadata.psMetadata.totalQualityCode, TK_FALSE);
  mdcpyLong(&sGranuleHandle->metadata.psMetadata.totalQualityCode,
        "TotalQualityCode", "\"NULL\"", TK_TOTAL_QUALITY_CODE);

  setFlag(&sGranuleHandle->metadata.psMetadata.lonOnEquator, TK_FALSE);
  mdcpyLong(&sGranuleHandle->metadata.psMetadata.lonOnEquator,
        "LongitudeOnEquator", "-9999.9", TK_LON_ON_EQUATOR);

  setFlag(&sGranuleHandle->metadata.psMetadata.UTCDateOnEquator, TK_FALSE);
  mdcpyLong(&sGranuleHandle->metadata.psMetadata.UTCDateOnEquator,
        "UTCDateOnEquator", "9999/99/99", TK_UTC_DATE_ON_EQUATOR);

  setFlag(&sGranuleHandle->metadata.psMetadata.UTCTimeOnEquator, TK_FALSE);
  mdcpyLong(&sGranuleHandle->metadata.psMetadata.UTCTimeOnEquator,
        "UTCTimeOnEquator", "99:99:99", TK_UTC_TIME_ON_EQUATOR);

  setFlag(&sGranuleHandle->metadata.psMetadata.millisecOnEquator, TK_FALSE);
  mdcpyLong(&sGranuleHandle->metadata.psMetadata.millisecOnEquator,
        "UTCMillisecsOnEquator", "-9999", TK_UTC_MILLISEC_ON_EQUATOR);

  setFlag(&sGranuleHandle->metadata.psMetadata.centerScanUTCDate, TK_FALSE);
  mdcpyLong(&sGranuleHandle->metadata.psMetadata.centerScanUTCDate,
        "CenterScanUTCDate", "9999/99/99", TK_CENTER_SCAN_UTC_DATE);

  setFlag(&sGranuleHandle->metadata.psMetadata.centerScanUTCTime, TK_FALSE);
  mdcpyLong(&sGranuleHandle->metadata.psMetadata.centerScanUTCTime,
        "CenterScanUTCTime", "99:99:99", TK_CENTER_SCAN_UTC_TIME);

  setFlag(&sGranuleHandle->metadata.psMetadata.centerScanUTCMillisec, TK_FALSE);
  mdcpyLong(&sGranuleHandle->metadata.psMetadata.centerScanUTCMillisec,
        "CenterScanUTCMillisec", "-9999", TK_CENTER_SCAN_UTC_MILLISEC);

  setFlag(&sGranuleHandle->metadata.psMetadata.firstScanLat, TK_FALSE);
  mdcpyLong(&sGranuleHandle->metadata.psMetadata.firstScanLat,
        "FirstScanLat", "-9999.9", TK_FIRST_SCAN_LAT);

  setFlag(&sGranuleHandle->metadata.psMetadata.firstScanLon, TK_FALSE);
  mdcpyLong(&sGranuleHandle->metadata.psMetadata.firstScanLon,
        "FirstScanLon", "-9999.9", TK_FIRST_SCAN_LON);

  setFlag(&sGranuleHandle->metadata.psMetadata.lastScanLat, TK_FALSE);
  mdcpyLong(&sGranuleHandle->metadata.psMetadata.lastScanLat,
        "LastScanLat", "-9999.9", TK_LAST_SCAN_LAT);

  setFlag(&sGranuleHandle->metadata.psMetadata.lastScanLon, TK_FALSE);
  mdcpyLong(&sGranuleHandle->metadata.psMetadata.lastScanLon,
        "LastScanLon", "-9999.9", TK_LAST_SCAN_LON);

  setFlag(&sGranuleHandle->metadata.psMetadata.numOfRainScans, TK_FALSE);
  mdcpyLong(&sGranuleHandle->metadata.psMetadata.numOfRainScans,
        "NumberOfRainScans", "-9999", TK_NUM_OF_RAIN_SCANS);

  /*
   * If filemode = TK_READ_ONLY then read the metadata from the file, parse
   * the individual elements, and store them in the metadata structures.
   */
  
  if (filemode == TK_READ_ONLY) {
    /*
     * Read the metadata from the HDF file.  Core metadata and searchable
     * ps metadata are in the attribute called "CoreMetadata.0".  
     * Non-searchable product specific metadata is in the attribute 
     * "ArchiveMetadata.0".  Go figure.
     */
    
    metadataStr = (char *) malloc (TK_MAX_SZ_ATTRIBUTE * sizeof(metadataStr));

    if (metadataStr == NULL)
	/* Memory allocation failed in initMetadata */
	TKreportError(E_TK_MEMERRINIT);

    /* Read metadata from hdf file */
    if(readpvl_from_hdffile(sGranuleHandle->cfgSTR->sdStartID,"CoreMetadata.0",
			    metadataStr) != TK_SUCCESS)
      TKreportWarning(W_TK_READPVL);
    

    /* Fill the individual metadata elements */

  if (!sGranuleHandle->realTimeFlag)
  {
  /*
   * BEGIN non-real time
   */
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.coreMetadata.orbitNumber.name,
              sGranuleHandle->metadata.coreMetadata.orbitNumber.value);
  /*
   * END non-real time
   */
  }
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.coreMetadata.beginDate.name,
              sGranuleHandle->metadata.coreMetadata.beginDate.value);
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.coreMetadata.beginTime.name,
              sGranuleHandle->metadata.coreMetadata.beginTime.value);
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.coreMetadata.endDate.name,
              sGranuleHandle->metadata.coreMetadata.endDate.value);
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.coreMetadata.endTime.name,
              sGranuleHandle->metadata.coreMetadata.endTime.value);
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.coreMetadata.granuleID.name,
              sGranuleHandle->metadata.coreMetadata.granuleID.value);
  if (!sGranuleHandle->realTimeFlag)
  {
  /*
   * BEGIN non-real time
   */
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.coreMetadata.dataID.name,
              sGranuleHandle->metadata.coreMetadata.dataID.value);
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.coreMetadata.fileSize.name,
              sGranuleHandle->metadata.coreMetadata.fileSize.value);
  /*
   * END non-real time
   */
  }
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.coreMetadata.lonMaxLat.name,
              sGranuleHandle->metadata.coreMetadata.lonMaxLat.value);
  if (!sGranuleHandle->realTimeFlag)
  {
  /*
   * BEGIN non-real time
   */
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.coreMetadata.spatCovType.name,
              sGranuleHandle->metadata.coreMetadata.spatCovType.value);
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.coreMetadata.ellipsoidName.name,
              sGranuleHandle->metadata.coreMetadata.ellipsoidName.value);
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.coreMetadata.earthEqRad.name,
              sGranuleHandle->metadata.coreMetadata.earthEqRad.value);
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.coreMetadata.flatteningRatio.name,
              sGranuleHandle->metadata.coreMetadata.flatteningRatio.value);
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.coreMetadata.orbitModelName.name,
              sGranuleHandle->metadata.coreMetadata.orbitModelName.value);
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.coreMetadata.kepSemiMajorAxis.name,
              sGranuleHandle->metadata.coreMetadata.kepSemiMajorAxis.value);
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.coreMetadata.kepMeanAnomaly.name,
              sGranuleHandle->metadata.coreMetadata.kepMeanAnomaly.value);
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.coreMetadata.kepRightAscenNode.name,
              sGranuleHandle->metadata.coreMetadata.kepRightAscenNode.value);
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.coreMetadata.kepArgOfPerigee.name,
              sGranuleHandle->metadata.coreMetadata.kepArgOfPerigee.value);
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.coreMetadata.kepEccentricity.name,
              sGranuleHandle->metadata.coreMetadata.kepEccentricity.value);
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.coreMetadata.kepInclination.name,
              sGranuleHandle->metadata.coreMetadata.kepInclination.value);
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.coreMetadata.kepEpochTime.name,
              sGranuleHandle->metadata.coreMetadata.kepEpochTime.value);
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.coreMetadata.kepEpochMillisec.name,
              sGranuleHandle->metadata.coreMetadata.kepEpochMillisec.value);
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.coreMetadata.kepEpochDate.name,
              sGranuleHandle->metadata.coreMetadata.kepEpochDate.value);
  /*
   * END non-real time
   */
  }
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.coreMetadata.westBoundCoord.name,
              sGranuleHandle->metadata.coreMetadata.westBoundCoord.value);
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.coreMetadata.eastBoundCoord.name,
              sGranuleHandle->metadata.coreMetadata.eastBoundCoord.value);
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.coreMetadata.northBoundCoord.name,
              sGranuleHandle->metadata.coreMetadata.northBoundCoord.value);
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.coreMetadata.southBoundCoord.name,
              sGranuleHandle->metadata.coreMetadata.southBoundCoord.value);
  if (!sGranuleHandle->realTimeFlag)
  {
  /*
   * BEGIN non-real time
   */
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.coreMetadata.centerPointLat.name,
              sGranuleHandle->metadata.coreMetadata.centerPointLat.value);
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.coreMetadata.centerPointLon.name,
              sGranuleHandle->metadata.coreMetadata.centerPointLon.value);
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.coreMetadata.radius.name,
              sGranuleHandle->metadata.coreMetadata.radius.value);
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.coreMetadata.latitudeRes.name,
              sGranuleHandle->metadata.coreMetadata.latitudeRes.value);
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.coreMetadata.longitudeRes.name,
              sGranuleHandle->metadata.coreMetadata.longitudeRes.value);
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.coreMetadata.geoCoordUnits.name,
              sGranuleHandle->metadata.coreMetadata.geoCoordUnits.value);
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.coreMetadata.qaParamName.name,
              sGranuleHandle->metadata.coreMetadata.qaParamName.value);
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.coreMetadata.qaParamValue.name,
              sGranuleHandle->metadata.coreMetadata.qaParamValue.value);
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.coreMetadata.temporRngType.name,
              sGranuleHandle->metadata.coreMetadata.temporRngType.value);
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.coreMetadata.reproStat.name,
              sGranuleHandle->metadata.coreMetadata.reproStat.value);
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.coreMetadata.browseName.name,
              sGranuleHandle->metadata.coreMetadata.browseName.value);
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.coreMetadata.contact.name,
              sGranuleHandle->metadata.coreMetadata.contact.value);
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.coreMetadata.meanMotion.name,
              sGranuleHandle->metadata.coreMetadata.meanMotion.value);
  /*
   * END non-real time
   */
  }

    parse_pvl(metadataStr,
              sGranuleHandle->metadata.coreMetadata.orbitAdjFlag.name,
              sGranuleHandle->metadata.coreMetadata.orbitAdjFlag.value);
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.coreMetadata.attitudeMode.name,
              sGranuleHandle->metadata.coreMetadata.attitudeMode.value);

  if (!sGranuleHandle->realTimeFlag)
  {
  /*
   * BEGIN non-real time
   */

    parse_pvl(metadataStr,
              sGranuleHandle->metadata.coreMetadata.beginSolarBeta.name,
              sGranuleHandle->metadata.coreMetadata.beginSolarBeta.value);
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.coreMetadata.endSolarBeta.name,
              sGranuleHandle->metadata.coreMetadata.endSolarBeta.value);
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.coreMetadata.sensorAlgn.name,
              sGranuleHandle->metadata.coreMetadata.sensorAlgn.value);
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.coreMetadata.sensorAlgnChanOff.name,
              sGranuleHandle->metadata.coreMetadata.sensorAlgnChanOff.value);
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.coreMetadata.scanPathModel.name,
              sGranuleHandle->metadata.coreMetadata.scanPathModel.value);
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.coreMetadata.scanPathParam.name,
              sGranuleHandle->metadata.coreMetadata.scanPathParam.value);
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.coreMetadata.ephemFilename.name,
              sGranuleHandle->metadata.coreMetadata.ephemFilename.value);
  /*
   * END non-real time
   */
  }


    /*
     * Get the Product Specific metadata
     */
    metadataStr[0] = '\0';
    if(readpvl_from_hdffile(sGranuleHandle->cfgSTR->sdStartID,"ArchiveMetadata.0",
                            metadataStr) != TK_SUCCESS)
      TKreportWarning(W_TK_READPVL);

  if (!sGranuleHandle->realTimeFlag)
  {
  /*
   * BEGIN non-real time
   */

    parse_pvl(metadataStr,
              sGranuleHandle->metadata.psMetadata.numDataGap.name,
              sGranuleHandle->metadata.psMetadata.numDataGap.value);
  /*
   * END non-real time
   */
  }
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.psMetadata.algorithmVersion.name,
              sGranuleHandle->metadata.psMetadata.algorithmVersion.value);
  if (!sGranuleHandle->realTimeFlag)
  {
  /*
   * BEGIN non-real time
   */
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.psMetadata.productVersion.name,
              sGranuleHandle->metadata.psMetadata.productVersion.value);
  /*
   * END non-real time
   */
   }

    parse_pvl(metadataStr,
              sGranuleHandle->metadata.psMetadata.toolkitVersion.name,
              sGranuleHandle->metadata.psMetadata.toolkitVersion.value);

  if (!sGranuleHandle->realTimeFlag)
  {
  /*
   * BEGIN non-real time
   */
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.psMetadata.calCoeffVersion.name,
              sGranuleHandle->metadata.psMetadata.calCoeffVersion.value);
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.psMetadata.missingData.name,
              sGranuleHandle->metadata.psMetadata.missingData.value);
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.psMetadata.percentBadMissPix.name,
              sGranuleHandle->metadata.psMetadata.percentBadMissPix.value);
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.psMetadata.maxValidChannel.name,
              sGranuleHandle->metadata.psMetadata.maxValidChannel.value);
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.psMetadata.minValidChannel.name,
              sGranuleHandle->metadata.psMetadata.minValidChannel.value);
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.psMetadata.minMaxUnits.name,
              sGranuleHandle->metadata.psMetadata.minMaxUnits.value);
  /*
   * END non-real time
   */
   }

    parse_pvl(metadataStr,
              sGranuleHandle->metadata.psMetadata.orbitSize.name,
              sGranuleHandle->metadata.psMetadata.orbitSize.value);

  if (!sGranuleHandle->realTimeFlag)
  {
  /*
   * BEGIN non-real time
   */

    parse_pvl(metadataStr,
              sGranuleHandle->metadata.psMetadata.radarWavelength.name,
              sGranuleHandle->metadata.psMetadata.radarWavelength.value);
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.psMetadata.minRefThreshold.name,
              sGranuleHandle->metadata.psMetadata.minRefThreshold.value);
  /*
   * END non-real time
   */
   }
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.psMetadata.algorithmID.name,
              sGranuleHandle->metadata.psMetadata.algorithmID.value);

  if (!sGranuleHandle->realTimeFlag)
  {
  /*
   * BEGIN non-real time
   */
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.psMetadata.dataAccuracy.name,
              sGranuleHandle->metadata.psMetadata.dataAccuracy.value);
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.psMetadata.dataGap.name,
              sGranuleHandle->metadata.psMetadata.dataGap.value);
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.psMetadata.inputID.name,
              sGranuleHandle->metadata.psMetadata.inputID.value);
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.psMetadata.dateGenInputFile.name,
              sGranuleHandle->metadata.psMetadata.dateGenInputFile.value);
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.psMetadata.dataCenterSrc.name,
              sGranuleHandle->metadata.psMetadata.dataCenterSrc.value);
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.psMetadata.generationDate.name,
              sGranuleHandle->metadata.psMetadata.generationDate.value);
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.psMetadata.dayNight.name,
              sGranuleHandle->metadata.psMetadata.dayNight.value);
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.psMetadata.solarGains.name,
              sGranuleHandle->metadata.psMetadata.solarGains.value);
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.psMetadata.tmiRolloverCoef.name,
              sGranuleHandle->metadata.psMetadata.tmiRolloverCoef.value);
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.psMetadata.firstScanUTCdate.name,
              sGranuleHandle->metadata.psMetadata.firstScanUTCdate.value);
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.psMetadata.firstScanUTCtime.name,
              sGranuleHandle->metadata.psMetadata.firstScanUTCtime.value);
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.psMetadata.firstScanUTCmsec.name,
              sGranuleHandle->metadata.psMetadata.firstScanUTCmsec.value);
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.psMetadata.firstScanSCsec.name,
              sGranuleHandle->metadata.psMetadata.firstScanSCsec.value);
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.psMetadata.firstScanSCsubsec.name,
              sGranuleHandle->metadata.psMetadata.firstScanSCsubsec.value);
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.psMetadata.lastScanUTCdate.name,
              sGranuleHandle->metadata.psMetadata.lastScanUTCdate.value);
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.psMetadata.lastScanUTCtime.name,
              sGranuleHandle->metadata.psMetadata.lastScanUTCtime.value);
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.psMetadata.lastScanUTCmsec.name,
              sGranuleHandle->metadata.psMetadata.lastScanUTCmsec.value);
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.psMetadata.lastScanSCsec.name,
              sGranuleHandle->metadata.psMetadata.lastScanSCsec.value);
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.psMetadata.lastScanSCsubsec.name,
              sGranuleHandle->metadata.psMetadata.lastScanSCsubsec.value);
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.psMetadata.utcfSec.name,
              sGranuleHandle->metadata.psMetadata.utcfSec.value);
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.psMetadata.utcfSubsec.name,
              sGranuleHandle->metadata.psMetadata.utcfSubsec.value);
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.psMetadata.utcfFlag.name,
              sGranuleHandle->metadata.psMetadata.utcfFlag.value);
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.psMetadata.leapSecsFlag.name,
              sGranuleHandle->metadata.psMetadata.leapSecsFlag.value);
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.psMetadata.radarName.name,
              sGranuleHandle->metadata.psMetadata.radarName.value);
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.psMetadata.radarCity.name,
              sGranuleHandle->metadata.psMetadata.radarCity.value);
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.psMetadata.radarState.name,
              sGranuleHandle->metadata.psMetadata.radarState.value);
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.psMetadata.radarCountry.name,
              sGranuleHandle->metadata.psMetadata.radarCountry.value);
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.psMetadata.numVOS.name,
              sGranuleHandle->metadata.psMetadata.numVOS.value);
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.psMetadata.radarOriginLat.name,
              sGranuleHandle->metadata.psMetadata.radarOriginLat.value);
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.psMetadata.radarOriginLon.name,
              sGranuleHandle->metadata.psMetadata.radarOriginLon.value);
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.psMetadata.radarOriginAlt.name,
              sGranuleHandle->metadata.psMetadata.radarOriginAlt.value);
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.psMetadata.radarSpacingX.name,
              sGranuleHandle->metadata.psMetadata.radarSpacingX.value);
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.psMetadata.radarSpacingY.name,
              sGranuleHandle->metadata.psMetadata.radarSpacingY.value);
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.psMetadata.radarSpacingZ.name,
              sGranuleHandle->metadata.psMetadata.radarSpacingZ.value);
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.psMetadata.radarGridSizeX.name,
              sGranuleHandle->metadata.psMetadata.radarGridSizeX.value);
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.psMetadata.radarGridSizeY.name,
              sGranuleHandle->metadata.psMetadata.radarGridSizeY.value);
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.psMetadata.radarGridSizeZ.name,
              sGranuleHandle->metadata.psMetadata.radarGridSizeZ.value);
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.psMetadata.dzcal.name,
              sGranuleHandle->metadata.psMetadata.dzcal.value);
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.psMetadata.gvL1Cscale.name,
              sGranuleHandle->metadata.psMetadata.gvL1Cscale.value);
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.psMetadata.alpha.name,
              sGranuleHandle->metadata.psMetadata.alpha.value);
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.psMetadata.runtimeOption.name,
              sGranuleHandle->metadata.psMetadata.runtimeOption.value);
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.psMetadata.anomalyFlag.name,
              sGranuleHandle->metadata.psMetadata.anomalyFlag.value);
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.psMetadata.softwareVersion.name,
              sGranuleHandle->metadata.psMetadata.softwareVersion.value);
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.psMetadata.databaseVersion.name,
              sGranuleHandle->metadata.psMetadata.databaseVersion.value);
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.psMetadata.totalQualityCode.name,
              sGranuleHandle->metadata.psMetadata.totalQualityCode.value);
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.psMetadata.lonOnEquator.name,
              sGranuleHandle->metadata.psMetadata.lonOnEquator.value);
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.psMetadata.UTCDateOnEquator.name,
              sGranuleHandle->metadata.psMetadata.UTCDateOnEquator.value);
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.psMetadata.UTCTimeOnEquator.name,
              sGranuleHandle->metadata.psMetadata.UTCTimeOnEquator.value);
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.psMetadata.millisecOnEquator.name,
              sGranuleHandle->metadata.psMetadata.millisecOnEquator.value);
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.psMetadata.centerScanUTCDate.name,
              sGranuleHandle->metadata.psMetadata.centerScanUTCDate.value);
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.psMetadata.centerScanUTCTime.name,
              sGranuleHandle->metadata.psMetadata.centerScanUTCTime.value);
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.psMetadata.centerScanUTCMillisec.name,
              sGranuleHandle->metadata.psMetadata.centerScanUTCMillisec.value);
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.psMetadata.firstScanLat.name,
              sGranuleHandle->metadata.psMetadata.firstScanLat.value);
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.psMetadata.firstScanLon.name,
              sGranuleHandle->metadata.psMetadata.firstScanLon.value);
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.psMetadata.lastScanLat.name,
              sGranuleHandle->metadata.psMetadata.lastScanLat.value);
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.psMetadata.lastScanLon.name,
              sGranuleHandle->metadata.psMetadata.lastScanLon.value);
    parse_pvl(metadataStr,
              sGranuleHandle->metadata.psMetadata.numOfRainScans.name,
              sGranuleHandle->metadata.psMetadata.numOfRainScans.value);

  /*
   * END non-real time
   */
  }

    free (metadataStr);
  } /* End of if TK_READ_ONLY */

}


/*****************************************************************************
*
*  FUNCTION STATEMENT:
*       instType(productID);
*
*  DESCRIPTION:
*       Takes a productID as input (e.g. TK_L1B_01), and returns an
*       instrument type (e.g., TK_VIRS)
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       productID               ProductID macro for a given algorithm. 
*                               Values are listed in IO.h
*
*  OUTPUT PARAMETERS:           
*       Parameter Name          Description
*       --------------          -----------
*       None
*
*  CALLED BY:                   initMetadata()
*
*  RETURN VALUES:               Returns a macro for instrument type
*                               TK_VIRS, TK_TMI, TK_PR, TK_GV, TK_VARIOUS
*                               The last one being a catch all.
*
*  ROUTINES CALLED:
*
*  None
******************************************************************************/

int instType(int productID)
{
  switch (productID) {
  case TK_L1B_01:
    return(TK_VIRS);
    
  case TK_L1B_11:
  case TK_L2A_12:
  case TK_L3A_11:
  case TK_L3A_12:
  case TK_L1B_11_R1:
  case TK_L2A_12_R1:
    return(TK_TMI);
    
  case TK_L1B_21:
  case TK_L1C_21:
  case TK_L2A_21:
  case TK_L2A_23:
  case TK_L2A_25:
  case TK_L2N_25:
  case TK_L3A_25:
  case TK_L3A_26:
  case TK_L3A_27:
  case TK_L2A_23_R1:
  case TK_L2A_25_R1:
  case TK_L2A_25_R2:
    return(TK_PR);

  case TK_L1B_GV:
  case TK_L1C_GV:
  case TK_L2A_53S:
  case TK_L2A_53TX:
  case TK_L2A_53FL:
  case TK_L2A_54S:
  case TK_L2A_54TX:
  case TK_L2A_54FL:
  case TK_L2A_55S:
  case TK_L2A_55TX:
  case TK_L2A_55FL:
  case TK_L2A_56:
  case TK_L2A_57:
  case TK_L3A_53S:
  case TK_L3A_53TX:
  case TK_L3A_53FL:
  case TK_L3A_54S:
  case TK_L3A_54TX:
  case TK_L3A_54FL:
  case TK_L3A_55:
    return(TK_GV);
    
  case TK_L2B_31:
  case TK_L3B_31:
  case TK_L3B_42:
  case TK_L3B_43:
  case TK_L3A_46:
    return(TK_VARIOUS);
  }
}

/*****************************************************************************
*
*  FUNCTION STATEMENT:
*       findAlgorithmDataType(int productID)
*
*  DESCRIPTION:
*       Determine the Algorithm ID of a product based on the product ID.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       productID               ProductID macro for a given algorithm. 
*                               Values are listed in IO.h
*
*  OUTPUT PARAMETERS:           
*       Parameter Name          Description
*       --------------          -----------
*       None
*
*  CALLED BY:                   initMetadata()
*
*  RETURN VALUES:               Returns the datatype for an algorithm.
*
*  ROUTINES CALLED:
*
*  None
******************************************************************************/

char *findAlgorithmDataType(int productID)
{
  switch (productID) {
  case TK_L1B_01:
    return( "1B01" );
    
  case TK_L1B_11:
    return ( "1B11" );
  case TK_L2A_12:
    return ( "2A12" );
  case TK_L3A_11:
    return( "3A11" );
  case TK_L3A_12:
    return( "3A12" );
    
  case TK_L1B_21:
    return ( "1B21" );
  case TK_L1C_21:
    return ( "1C21" );
  case TK_L2A_21:
    return ( "2A21" );
  case TK_L2A_23:
    return ( "2A23" );
  case TK_L2A_25:
    return ( "2A25" );
  case TK_L2N_25:
    return ( "2N25" );
  case TK_L3A_25:
    return ( "3A25" );
  case TK_L3A_26:
    return( "3A26" );
  case TK_L3A_27:
    return( "3A27" );

  case TK_L1B_GV:
    return ( "1B51" );
  case TK_L1C_GV:
    return ( "1C51" );
  case TK_L2A_53S:
  case TK_L2A_53TX:
  case TK_L2A_53FL:
    return ( "2A53" );
  case TK_L2A_54S:
  case TK_L2A_54TX:
  case TK_L2A_54FL:
    return ( "2A54" );
  case TK_L2A_55S:
  case TK_L2A_55TX:
  case TK_L2A_55FL:
    return ( "2A55" );
  case TK_L2A_56:
    return ( "2A56" );
  case TK_L2A_57:
    return ( "2A57" );
  case TK_L3A_53S:
  case TK_L3A_53TX:
  case TK_L3A_53FL:
    return ( "3A53" );
  case TK_L3A_54S:
  case TK_L3A_54TX:
  case TK_L3A_54FL:
    return ( "3A54" );
  case TK_L3A_55:
    return( "3A55" );
    
  case TK_L2B_31:
    return ( "2B31" );
  case TK_L3B_31:
    return ( "3B31" );
  case TK_L3B_42:
    return ( "3B42" );
  case TK_L3B_43:
    return ( "3B43" );
  case TK_L3A_46:
    return( "3A46" );

  case TK_L1B_11_R1:
    return ( "1B11R1" );
  case TK_L2A_12_R1:
    return ( "2A12R1" );
  case TK_L2A_23_R1:
    return ( "2A23R1" );
  case TK_L2A_25_R1:
    return ( "2A25R1" );
  case TK_L2A_25_R2:
    return ( "2A25R2" );
  }
}

/*****************************************************************************
*
*  FUNCTION STATEMENT:
*       findAlgorithmTKID (char * algorithmID)
*
*  DESCRIPTION:
*       Determine the toolkit ID of a product based on the algorithm ID.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       algorithmID             Datatype for the given algorithm (e.g., 1B11 ).
*
*  OUTPUT PARAMETERS:           
*       Parameter Name          Description
*       --------------          -----------
*       None
*
*  CALLED BY:                   user routine.
*
*  RETURN VALUES:               Returns the toolkit ID for an algorithm.
*
*  ROUTINES CALLED:
*
*  None
******************************************************************************/

int findAlgorithmTKID ( char *algorithmID, char *radarSite )
{
  if ( !strcmp ( algorithmID, "1B01" ) )
    return TK_L1B_01;
  else if ( !strcmp ( algorithmID, "1B11" ) )
    return TK_L1B_11;
  else if ( !strcmp ( algorithmID, "1B11R1" ) )
    return TK_L1B_11_R1;
  else if ( !strcmp ( algorithmID, "2A12" ) )
    return TK_L2A_12;
  else if ( !strcmp ( algorithmID, "2A12R1" ) )
    return TK_L2A_12_R1;  
  else if ( !strcmp ( algorithmID, "3A11" ) )
    return TK_L3A_11;
  else if ( !strcmp ( algorithmID, "3A12" ) )
    return TK_L3A_12;
  else if ( !strcmp ( algorithmID, "1B21" ) )
    return TK_L1B_21;
  else if ( !strcmp ( algorithmID, "1C21" ) )
    return TK_L1C_21;
  else if ( !strcmp ( algorithmID, "2A21" ) )
    return TK_L2A_21;
  else if ( !strcmp ( algorithmID, "2A23" ) )
    return TK_L2A_23;  
  else if ( !strcmp ( algorithmID, "2A23R1" ) )
    return TK_L2A_23_R1;
  else if ( !strcmp ( algorithmID, "2A25" ) )
    return TK_L2A_25;
  else if ( !strcmp ( algorithmID, "2N25" ) )
    return TK_L2N_25;
  else if ( !strcmp ( algorithmID, "2A25R1" ) )
    return TK_L2A_25_R1;
  else if ( !strcmp ( algorithmID, "2A25R2" ) )
    return TK_L2A_25_R2;
  else if ( !strcmp ( algorithmID, "3A25" ) )
    return TK_L3A_25;
  else if ( !strcmp ( algorithmID, "3A26" ) )
    return TK_L3A_26;
  else if ( !strcmp ( algorithmID, "3A27" ) )
    return TK_L3A_27;
  else if ( !strcmp ( algorithmID, "2B31" ) )
    return TK_L2B_31;
  else if ( !strcmp ( algorithmID, "3B31" ) )
    return TK_L3B_31;
  else if ( !strcmp ( algorithmID, "3B42" ) )
    return TK_L3B_42;
  else if ( !strcmp ( algorithmID, "3B43" ) )
    return TK_L3B_43;
  else if ( !strcmp ( algorithmID, "3A46" ) )
    return TK_L3A_46;
  else if ( !strcmp ( algorithmID, "1B51" ) ) 
    return TK_L1B_GV;
  else if ( !strcmp ( algorithmID, "1C51" ) )
    return TK_L1C_GV;
  else if ( !strcmp ( algorithmID, "2A53" ) )
    {
      if ( !strcmp ( radarSite, "TEXS" ) )
	return TK_L2A_53TX;
      else if ( !strcmp ( radarSite, "FLOR" ) )
	return TK_L2A_53FL;
      else
	return TK_L2A_53S;
    }
  else if ( !strcmp ( algorithmID, "2A54" ) )
    {
      if ( !strcmp ( radarSite, "TEXS" ) )
	return TK_L2A_54TX;
      else if ( !strcmp ( radarSite, "FLOR" ) )
	return TK_L2A_54FL;
      else
	return TK_L2A_54S;
    }
  else if ( !strcmp ( algorithmID, "2A55" ) )
    {
      if ( !strcmp ( radarSite, "TEXS" ) )
	return TK_L2A_55TX;
      else if ( !strcmp ( radarSite, "FLOR" ) )
	return TK_L2A_55FL;
      else
	return TK_L2A_55S;
    }
  else if ( !strcmp ( algorithmID, "2A56" ) )
    return TK_L2A_56;
  else if ( !strcmp ( algorithmID, "2A57" ) )
    return TK_L2A_57;
  else if ( !strcmp ( algorithmID, "3A53" ) )
    {
      if ( !strcmp ( radarSite, "TEXS" ) )
	return TK_L3A_53TX;
      else if ( !strcmp ( radarSite, "FLOR" ) )
	return TK_L3A_53FL;
      else
	return TK_L3A_53S;
    }
  else if ( !strcmp ( algorithmID, "3A54" ) )
    {
      if ( !strcmp ( radarSite, "TEXS" ) )
	return TK_L3A_54TX;
      else if ( !strcmp ( radarSite, "FLOR" ) )
	return TK_L3A_54FL;
      else
	return TK_L3A_54S;
    }
  else if ( !strcmp ( algorithmID, "3A55" ) )
    return TK_L3A_55;

  return TK_FAIL;

}

/*****************************************************************************
*
*  FUNCTION STATEMENT:
*       radarType(radarID);
*
*  DESCRIPTION:
*       Takes a radarID as input (e.g. TK_DARW), and returns the radar code
*       (e.g., DARW).  The code will be used to query the database.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       radarID                 radarID macro for a radar site.
*
*  OUTPUT PARAMETERS:           
*       Parameter Name          Description
*       --------------          -----------
*       None
*
*  CALLED BY:                   TKqueryGVCoincidence()
*
*  RETURN VALUES:               The radar code as defined by TSDIS IM
*
*  ROUTINES CALLED:
*
*  None
******************************************************************************/

char *radarType(int radarID)
{
  switch (radarID) {
  case TK_DARW:
    return("DARW");
  case TK_MELB:
    return ("MELB");
  case TK_KWAJ:
    return ("KWAJ");
  case TK_HSTN:
    return ("HSTN");
  case TK_CORC:
    return ("CORC");
  case TK_FLOR:
    return ("FLOR");
  case TK_GUAM:
    return ("GUAM");
  case TK_ISBN:
    return ("ISBN");
  case TK_KEYW:
    return ("KEYW");
  case TK_LKCH:
    return ("LKCH");
  case TK_MIAM:
    return ("MIAM");
  case TK_NWBR:
    return ("NWBR");
  case TK_SAOP:
    return ("SAOP");
  case TK_TAMP:
    return ("TAMP");
  case TK_TEXS:
    return ("TEXS");
  case TK_THOM:
    return ("THOM");
  case TK_TWWF:
    return ("TWWF");
  }
}

/*****************************************************************************
*
*  FUNCTION STATEMENT:
*       int prodType(int productID)
*
*  DESCRIPTION:
*       Determine the product level based on the input product id.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       productID               A valid product id (toolkit type).  For example,
*                               TK_L2A_25 is a Level 2 product.
*
*  OUTPUT PARAMETERS:           None,
*
*  CALLED BY:                   correctNULLElements(), createSwathRadarMetadata()
*
*  RETURN VALUES:               The level indicator for the product ID.
*
*  ROUTINES CALLED:             None.
*
*
******************************************************************************/
int prodType(int productID)
{
  switch (productID) {
  case TK_L1B_01:
  case TK_L1B_11:
  case TK_L1B_21:
  case TK_L1C_21:
  case TK_L1B_GV:
  case TK_L1C_GV:
  case TK_L1B_11_R1:  /*  Subsetted Product  */
    return(TK_L1);
    

  case TK_L2A_12:
  case TK_L2A_21:
  case TK_L2A_23:
  case TK_L2A_25:
  case TK_L2N_25:
  case TK_L2A_53S:
  case TK_L2A_53TX:
  case TK_L2A_53FL:
  case TK_L2A_54S:
  case TK_L2A_54TX:
  case TK_L2A_54FL:
  case TK_L2A_55S:
  case TK_L2A_55TX:
  case TK_L2A_55FL:
  case TK_L2A_56:
  case TK_L2A_57:
  case TK_L2B_31:
  case TK_L2A_12_R1:  /*  Subsetted Product  */
  case TK_L2A_23_R1:  /*  Subsetted Product  */
  case TK_L2A_25_R1:  /*  Subsetted Product  */
  case TK_L2A_25_R2:  /*  Subsetted Product  */
    return(TK_L2);
    
  case TK_L3A_11:
  case TK_L3A_12:
  case TK_L3A_25:
  case TK_L3A_26:
  case TK_L3A_27:
  case TK_L3A_53S:
  case TK_L3A_53TX:
  case TK_L3A_53FL:
  case TK_L3A_54S:
  case TK_L3A_54TX:
  case TK_L3A_54FL:
  case TK_L3A_55:
  case TK_L3B_31:
  case TK_L3B_42:
  case TK_L3B_43:
  case TK_L3A_46:
    return(TK_L3);
  }
}

/*****************************************************************************
*
*  FUNCTION STATEMENT:
*       mdcpy(ELEMENT *e, char *name, char *value)
*
*  DESCRIPTION:
*       Allocate space for metadata element name and value.
*       Copy the name and value into the element
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       name                    Name of the metadata field used by EOSDIS to key upon.
*       value                   The value of the metadata field.  All values are
*                               stored as characters internally.
*
*  OUTPUT PARAMETERS:           
*       Parameter Name          Description
*       --------------          -----------
*       e                       Metadata ELEMENT.  This is a structure that
*                               contains a name field and a value field.
*
*  CALLED BY:                   initMetadata()
*
*  RETURN VALUES:               None.
*
*  ROUTINES CALLED:
*
*       TKreportError		This routine reports error to
*                               console and terminates processing.
*
******************************************************************************/
void mdcpy(ELEMENT *e , char *name, char *value)
{

  /* Allocate space for the name and the value */
  if ( (e->name = (char *) malloc(strlen(name)*sizeof(e->name) + 1) ) == (char *)NULL)
    TKreportError(E_TK_MDCPY);

  if ( (e->value = (char *) malloc(TK_SZ_ELEMENT_51 * sizeof(e->value)) ) == (char *)NULL)
    TKreportError(E_TK_MDCPY);

    
  /* copy the name and the value into the metadata element */
  strcpy( e->name, name );
  strcpy( e->value, value );
}

/*****************************************************************************
*
*  FUNCTION STATEMENT:
*       mdcpyLong(ELEMENT *e, char *name, char *value)
*
*  DESCRIPTION:
*       Allocate space for metadata element name and value.
*       Copy the name and value into the element
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       name                    Name of the metadata field used by EOSDIS to key upon.
*       value                   The value of the metadata field.  All values are
*                               stored as characters internally.
*
*  OUTPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       e                       Metadata ELEMENT.  This is a structure that
*                               contains a name field and a value field.
*
*  CALLED BY:                   initMetadata()
*
*  RETURN VALUES:               None.
*
*  ROUTINES CALLED:
*
*       TKreportError		This routine reports error to console
*                               and terminates processing.
*
******************************************************************************/
void mdcpyLong(ELEMENT *e , char *name, char *value, int metadataID)
{

  /* Allocate space for the name and the value */
  if ( (e->name = (char *) malloc(strlen(name)*sizeof(e->name) + 1) ) == (char *)NULL)
    TKreportError(E_TK_MDCPY);

  if (metadataID == TK_GV_RUNTIME_OPT)
  {
    e->value = (char *) malloc(TK_SZ_ELEMENT_1001 * sizeof(e->value));
    if (e->value == (char *)NULL)
	TKreportError(E_TK_MDCPY);
  }
  else if (metadataID == TK_INPUT_FILES)
  {
    e->value = (char *) malloc(TK_SZ_ELEMENT_301 * sizeof(e->value));
    if (e->value == (char *)NULL)
        TKreportError(E_TK_MDCPY);
  }
  else
  {
    e->value = (char *) malloc(TK_SZ_ELEMENT_101 * sizeof(e->value));
    if (e->value == (char *)NULL)
    	TKreportError(E_TK_MDCPY);
  }

  /* copy the name and the value into the metadata element */
  strcpy( e->name, name );
  strcpy( e->value, value );
}


/*************************************************************************************
*
*  FUNCTION STATEMENT:
*	writepvl_to_hdffile(IO_HANDLE whandle,char *fatr1,char *str1)
*
*  DESCRIPTION:            
*	This function writes pvl data to the hdf file.
*
*  INPUT PARAMETER:
*	Parameter Name          Description
*	--------------		-----------
*       whandle         	A structure that contains information about the file.
*       fatr1                   Name of the attribute being written 
*				(e.g. "CoreMetadata").
*       str1                    The pvl string to be written.
*
*  OUTPUT PARAMETER:
*	Parameter Name		Description
*	--------------		-----------
*	whandle			A structure that contains information about the
*				current file.
*
*  RETURN VALUES:		TK_SUCCESS, W_TK_SETATTR, W_TK_MAXSIZE
*
*  ROUTINE CALLED:		TKreportWarning()
*
*	DATE			PROGRAMMER		DESCRIPTION
*	----			----------		-----------
*	04/20/96		Prashant Shah		Implemented Code
*
***********************************************************************************/
int writepvl_to_hdffile(IO_HANDLE *whandle, char *fatr1, char *str1)
{

        int j;
        int32 sdstartID;

	/* add an END to the pvl string */
	strcat(str1, "END;\n");

        sdstartID = whandle->cfgSTR->sdStartID;
        j = strlen(str1);

        /*
        * If the length of the string exceeds the defined length, print an 
        * error message and exit out of the program
        */

        if (j > TK_MAX_SZ_ATTRIBUTE) 
	{
          TKreportWarning(W_TK_MAXSIZE);
	  return(W_TK_MAXSIZE);
        }

        /* set an attribute that describes the file (CoreMedata.0) */
        if (SDsetattr(sdstartID, fatr1, DFNT_CHAR8, j, (VOIDP)str1) != TK_SUCCESS) 
	{
          TKreportWarning(W_TK_SETATTR);
	  return(W_TK_SETATTR);
        }

        return(TK_SUCCESS);
}

/*******************************************************************************************
*
*
*  FUNCTION STATEMENT:
*	readpvl_from_hdffile(IO_HANDLE *whandle, char *fatr1, char *str1)
*
*  DESCRIPTION:            This function reads pvl data from hdf file
*
*  PARAMETERS:
*	Parameter Name		Description
*       --------------          -----------
*       whandle                 A character string containg  the name of the hdf file 
*                               where data is read from
*       fatr1                   Name of the attribute from which it is read 
*                               (e.g. "CoreMetadata").
*       str1                    The pvl string to be read.
*
*  RETURN VALUES:		TK_SUCCESS, W_TK_ATTRINFO
*
*  ROUTINE CALLED:		TKreportWarning()
*
*	DATE			PROGRAMMER		DESCRIPTION
*	----			----------		-----------
*	04/20/96		Prashant Shah		Implemented Code
*
******************************************************************************************/

int readpvl_from_hdffile(int32 sdstartID, char *fatr1, char *str1)
{
        int32 attr_index, num_type, count, size_attr;
        char attr_name[TK_FNAME_LEN];

        /* Find file attribute name "CoreMetadat.0" */
        attr_index = SDfindattr(sdstartID, fatr1);

        /* Get information about the file attribute */
        if (SDattrinfo(sdstartID, attr_index, attr_name, &num_type, &count) != TK_SUCCESS) 
	{
/*
          TKreportWarning(W_TK_ATTRINFO);
*/
          TKreportWarningParm(W_TK_ATTRINFO,attr_name);
          return(W_TK_ATTRINFO) ;
        }

        /* read the attribute data */
        if (SDreadattr(sdstartID, attr_index, str1) != TK_SUCCESS) 
	{
          TKreportWarning(W_TK_MAXSIZE);
	  return(W_TK_MAXSIZE);
        }

        return(TK_SUCCESS);
}


/**********************************************************
@FUNCTION STATEMENT:
@  create_pvl(char *outputStr, char *parameter, char  *value)
@
@DESCRIPTION:
@  This creates a PVL object's for core and ps metadata 
@  elements that need to be written to browse output
@  file. The inputs are parameter and value which
@  contain the object name and object value respectively.
@  The output pvl object is appended to the existing
@  pvl objects in outputStr. After all parameters are
@  processed the outputStr contains the pvl for either
@  core or ps metadata.
@  
@ 
@INPUT PARAMETERS:
@  Parameter Name     Description
@   --------------     -----------
@   parameter          character string containing
@                      name of the object.
@   value              character string containing
@                      the value for the object. 
@
@OUTPUT PARAMETERS:
@   Parameter Name     Description
@   --------------     -----------
@   outputStr          character string containing PVL.  
@
@CALLED BY:   
@   BRwriteMetadataToImg()
@             
@RETURN VALUES:        
@  NONE
@  
@ROUTINES CALLED:
@  Routine Name        Description
@  ------------        -----------
@  NONE 
@
@  DATE            PROGRAMMER                  CHANGE DESCRIPTION
@  ----            -----------                 ------------------
@  04/05/96       Sudhakar Gududuri            Implemented code
@
@**********************************************************/
void create_pvl(char *outputStr, char *parameter, char *value)
{

        char *tmpStr;
        int  len;

	len = 70;
        len += 2 * strlen(parameter);
        len += strlen(value);

        tmpStr = (char *) malloc(len * sizeof(char) + 1);

	if (tmpStr == NULL)
		/* Memory allocation failed in create_pvl. */
		TKreportError(E_TK_MEMFAILCREATEPVL); 

	sprintf(tmpStr,"OBJECT=%s;\n\tValue=%s;\n\tData_Location=%s\
;\n\tMandatory=%s;\nEND_OBJECT=%s;\n\n", \
		parameter,value,"PGE","FALSE",parameter);

        if (outputStr == NULL)
                outputStr = tmpStr;
        else
                outputStr = strcat(outputStr,tmpStr);

        free (tmpStr);
}

/**********************************************************
@FUNCTION STATEMENT:
@  parse_pvl(char *inputStr, char *parameter, char *value)
@
@DESCRIPTION:
@  This function takes a PVL contained in inputStr and
@  parses the value for a given parameter name.
@  This function is used for core and ps pvl objects only.
@  due to the same pvl format for both of them.
@  
@INPUT PARAMETERS:
@  Parameter Name     Description
@   --------------     -----------
@   inputStr           character string containing the
@                      PVL objects for core or ps 
@                      metadata.
@   parameter          character string containing
@                      name of the object.
@
@OUTPUT PARAMETERS:
@   Parameter Name     Description
@   --------------     -----------
@   value              character string containing
@                      the value for the object. 
@
@CALLED BY:   
@             
@RETURN VALUES:        
@ NONE
@  
@ROUTINES CALLED:
@  Routine Name        Description
@  ------------        -----------
@  NONE 
@
@  DATE            PROGRAMMER                  CHANGE DESCRIPTION
@  ----            -----------                 ------------------
@  04/05/96       Sudhakar Gududuri            Implemented code
@
@**********************************************************/
void parse_pvl(char *inputStr, char *parameter, char *value)
{

  int len;
  char *tmpStr1, *tmpStr2;
  
  tmpStr1 = strstr(inputStr,parameter);
  if (tmpStr1 == NULL)
    TKreportWarningParm(W_TK_NO_METADATA_FIELD, parameter);
  else 
    {
      tmpStr2 = strstr(tmpStr1,"Value=");
      if (tmpStr2 == NULL)
        TKreportWarningParm(W_TK_NO_METADATA_FIELD, parameter);
      else {
        len = strcspn((tmpStr2+strlen("Value=")),
                      ";");
        strncpy(value,(tmpStr2+strlen("Value=")),
                len);
        value[len]='\0';
      }
      
    }
}

/*****************************************************************************
*
*  FUNCTION STATEMENT:
*	correctNULLElements(IO_HANDLE *sGranuleHandle)
*
*  DESCRIPTION:
*	This routine will reset all of the metadata elements that should have
*       the default values.  This is useful for higher level products, since
*       they have the potential of getting values that might not be applicable
*       for that level.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       sGranuleHandle          A structure containing information about the file
*                               from which data can be obtained.  It also
*				contains metadata information. 
*
*  OUTPUT PARAMETERS:		
*       Parameter Name          Description
*       --------------          -----------
*       sGranuleHandle          Elements are set to NULL, as defined by the file 
*                               specs.
*
*  CALLED BY:                   finalizeMetadata()
*
*  RETURN VALUES:               None.
*
*  ROUTINES CALLED:
*       instType                Determines the instrument based on the product ID.
*
*       prodType                Determines the Level of the product based on the 
*                               product ID.
*
******************************************************************************/
void correctNULLElements ( IO_HANDLE *sGranuleHandle )
{
  int i;
  char tmiRolloverCoef[160];

  /* Build TMI rollover coefficient default string */
  strcpy ( tmiRolloverCoef, "\"(-9999.9");
  for (i=1;i<18;i++) strcat ( tmiRolloverCoef, ",-9999.9");
  strcat ( tmiRolloverCoef, ")\"");

  switch ( instType ( sGranuleHandle->productID ) )
    {
    case TK_TMI:

      setFlag(&sGranuleHandle->metadata.coreMetadata.centerPointLat, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.coreMetadata.centerPointLat.value, "-9999.9" );

      setFlag(&sGranuleHandle->metadata.coreMetadata.centerPointLon, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.coreMetadata.centerPointLon.value, "-9999.9" );

      setFlag(&sGranuleHandle->metadata.coreMetadata.radius, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.coreMetadata.radius.value, "-9999.9" );

      setFlag(&sGranuleHandle->metadata.psMetadata.calCoeffVersion, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.calCoeffVersion.value, "-9999" );

      setFlag(&sGranuleHandle->metadata.psMetadata.minRefThreshold, TK_TRUE); 
      strcpy ( sGranuleHandle->metadata.psMetadata.minRefThreshold.value, "-9999.9" );

      setFlag(&sGranuleHandle->metadata.psMetadata.radarName, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.radarName.value, "\"NULL\"" );

      setFlag(&sGranuleHandle->metadata.psMetadata.radarCity, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.radarCity.value, "\"NULL\"" );

      setFlag(&sGranuleHandle->metadata.psMetadata.radarState, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.radarState.value, "\"NULL\"" );

      setFlag(&sGranuleHandle->metadata.psMetadata.radarCountry, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.radarCountry.value, "\"NULL\"" );

      setFlag(&sGranuleHandle->metadata.psMetadata.numVOS, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.numVOS.value, "-9999" );

      setFlag(&sGranuleHandle->metadata.psMetadata.radarOriginLat, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.radarOriginLat.value, "-9999.9" );

      setFlag(&sGranuleHandle->metadata.psMetadata.radarOriginLon, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.radarOriginLon.value, "-9999.9" );

      setFlag(&sGranuleHandle->metadata.psMetadata.radarOriginAlt, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.radarOriginAlt.value, "-9999.9" );

      setFlag(&sGranuleHandle->metadata.psMetadata.radarSpacingX, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.radarSpacingX.value, "-9999.9" );

      setFlag(&sGranuleHandle->metadata.psMetadata.radarSpacingY, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.radarSpacingY.value, "-9999.9" );

      setFlag(&sGranuleHandle->metadata.psMetadata.radarSpacingZ, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.radarSpacingZ.value, "-9999.9" );

      setFlag(&sGranuleHandle->metadata.psMetadata.radarGridSizeX, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.radarGridSizeX.value, "-9999" );

      setFlag(&sGranuleHandle->metadata.psMetadata.radarGridSizeY, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.radarGridSizeY.value, "-9999" );

      setFlag(&sGranuleHandle->metadata.psMetadata.radarGridSizeZ, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.radarGridSizeZ.value, "-9999" );

      setFlag(&sGranuleHandle->metadata.psMetadata.dzcal, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.dzcal.value, "-9999.9" );

      setFlag(&sGranuleHandle->metadata.psMetadata.gvL1Cscale, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.gvL1Cscale.value, "-9999.9" );

      setFlag(&sGranuleHandle->metadata.psMetadata.alpha, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.alpha.value, "-9999.9" );

      setFlag(&sGranuleHandle->metadata.psMetadata.runtimeOption, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.runtimeOption.value, "\"NULL\"" );

      setFlag(&sGranuleHandle->metadata.psMetadata.softwareVersion, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.softwareVersion.value, "-9999" );
      
      setFlag(&sGranuleHandle->metadata.psMetadata.databaseVersion, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.databaseVersion.value, "-9999" );
      
      setFlag(&sGranuleHandle->metadata.psMetadata.totalQualityCode, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.totalQualityCode.value, "\"NULL\"" );
      
      setFlag(&sGranuleHandle->metadata.psMetadata.lonOnEquator, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.lonOnEquator.value, "-9999.9" );
      
      setFlag(&sGranuleHandle->metadata.psMetadata.UTCDateOnEquator, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.UTCDateOnEquator.value, "9999/99/99" );
      
      setFlag(&sGranuleHandle->metadata.psMetadata.UTCTimeOnEquator, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.UTCTimeOnEquator.value, "99:99:99" );
      
      setFlag(&sGranuleHandle->metadata.psMetadata.millisecOnEquator, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.millisecOnEquator.value, "-9999" );
      
      setFlag(&sGranuleHandle->metadata.psMetadata.centerScanUTCDate, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.centerScanUTCDate.value, "9999/99/99" );
      
      setFlag(&sGranuleHandle->metadata.psMetadata.centerScanUTCTime, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.centerScanUTCTime.value, "99:99:99" );
      
      setFlag(&sGranuleHandle->metadata.psMetadata.centerScanUTCMillisec, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.centerScanUTCMillisec.value, "-9999" );
      
      setFlag(&sGranuleHandle->metadata.psMetadata.firstScanLat, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.firstScanLat.value, "-9999.9" );
      
      setFlag(&sGranuleHandle->metadata.psMetadata.firstScanLon, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.firstScanLon.value, "-9999.9" );
      
      setFlag(&sGranuleHandle->metadata.psMetadata.lastScanLat, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.lastScanLat.value, "-9999.9" );
      
      setFlag(&sGranuleHandle->metadata.psMetadata.lastScanLon, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.lastScanLon.value, "-9999.9" );
      
      setFlag(&sGranuleHandle->metadata.psMetadata.numOfRainScans, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.numOfRainScans.value, "-9999" );

      break;
    case TK_VIRS:
      setFlag(&sGranuleHandle->metadata.coreMetadata.centerPointLat, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.coreMetadata.centerPointLat.value, "-9999.9" );

      setFlag(&sGranuleHandle->metadata.coreMetadata.centerPointLon, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.coreMetadata.centerPointLon.value, "-9999.9" );

      setFlag(&sGranuleHandle->metadata.coreMetadata.radius, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.coreMetadata.radius.value, "-9999.9" );

      setFlag(&sGranuleHandle->metadata.psMetadata.calCoeffVersion, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.calCoeffVersion.value, "-9999" ); 

      setFlag(&sGranuleHandle->metadata.psMetadata.minRefThreshold, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.minRefThreshold.value, "-9999.9" );

      setFlag(&sGranuleHandle->metadata.psMetadata.radarName, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.radarName.value, "\"NULL\"" );

      setFlag(&sGranuleHandle->metadata.psMetadata.radarCity, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.radarCity.value, "\"NULL\"" );

      setFlag(&sGranuleHandle->metadata.psMetadata.radarState, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.radarState.value, "\"NULL\"" );

      setFlag(&sGranuleHandle->metadata.psMetadata.radarCountry, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.radarCountry.value, "\"NULL\"" );

      setFlag(&sGranuleHandle->metadata.psMetadata.numVOS, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.numVOS.value, "-9999" );

      setFlag(&sGranuleHandle->metadata.psMetadata.radarOriginLat, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.radarOriginLat.value, "-9999.9" );

      setFlag(&sGranuleHandle->metadata.psMetadata.radarOriginLon, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.radarOriginLon.value, "-9999.9" );

      setFlag(&sGranuleHandle->metadata.psMetadata.radarOriginAlt, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.radarOriginAlt.value, "-9999.9" );

      setFlag(&sGranuleHandle->metadata.psMetadata.radarSpacingX, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.radarSpacingX.value, "-9999.9" );

      setFlag(&sGranuleHandle->metadata.psMetadata.radarSpacingY, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.radarSpacingY.value, "-9999.9" );

      setFlag(&sGranuleHandle->metadata.psMetadata.radarSpacingZ, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.radarSpacingZ.value, "-9999.9" );

      setFlag(&sGranuleHandle->metadata.psMetadata.radarGridSizeX, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.radarGridSizeX.value, "-9999" );

      setFlag(&sGranuleHandle->metadata.psMetadata.radarGridSizeY, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.radarGridSizeY.value, "-9999" );

      setFlag(&sGranuleHandle->metadata.psMetadata.radarGridSizeZ, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.radarGridSizeZ.value, "-9999" );

      setFlag(&sGranuleHandle->metadata.psMetadata.dzcal, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.dzcal.value, "-9999.9" );

      setFlag(&sGranuleHandle->metadata.psMetadata.gvL1Cscale, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.gvL1Cscale.value, "-9999.9" );

      setFlag(&sGranuleHandle->metadata.psMetadata.alpha, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.alpha.value, "-9999.9" );

      setFlag(&sGranuleHandle->metadata.psMetadata.runtimeOption, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.runtimeOption.value, "\"NULL\"" );

      setFlag(&sGranuleHandle->metadata.psMetadata.softwareVersion, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.softwareVersion.value, "-9999" );
      
      setFlag(&sGranuleHandle->metadata.psMetadata.databaseVersion, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.databaseVersion.value, "-9999" );
      
      setFlag(&sGranuleHandle->metadata.psMetadata.totalQualityCode, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.totalQualityCode.value, "\"NULL\"" );
      
      setFlag(&sGranuleHandle->metadata.psMetadata.lonOnEquator, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.lonOnEquator.value, "-9999.9" );
      
      setFlag(&sGranuleHandle->metadata.psMetadata.UTCDateOnEquator, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.UTCDateOnEquator.value, "9999/99/99" );
      
      setFlag(&sGranuleHandle->metadata.psMetadata.UTCTimeOnEquator, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.UTCTimeOnEquator.value, "99:99:99" );
      
      setFlag(&sGranuleHandle->metadata.psMetadata.millisecOnEquator, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.millisecOnEquator.value, "-9999" );
      
      setFlag(&sGranuleHandle->metadata.psMetadata.centerScanUTCDate, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.centerScanUTCDate.value, "9999/99/99" );
      
      setFlag(&sGranuleHandle->metadata.psMetadata.centerScanUTCTime, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.centerScanUTCTime.value, "99:99:99" );
      
      setFlag(&sGranuleHandle->metadata.psMetadata.centerScanUTCMillisec, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.centerScanUTCMillisec.value, "-9999" );
      
      setFlag(&sGranuleHandle->metadata.psMetadata.firstScanLat, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.firstScanLat.value, "-9999.9" );
      
      setFlag(&sGranuleHandle->metadata.psMetadata.firstScanLon, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.firstScanLon.value, "-9999.9" );
      
      setFlag(&sGranuleHandle->metadata.psMetadata.lastScanLat, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.lastScanLat.value, "-9999.9" );
      
      setFlag(&sGranuleHandle->metadata.psMetadata.lastScanLon, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.lastScanLon.value, "-9999.9" );
      
      setFlag(&sGranuleHandle->metadata.psMetadata.numOfRainScans, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.numOfRainScans.value, "-9999" );
      break;

    case TK_PR:
      setFlag(&sGranuleHandle->metadata.psMetadata.dayNight, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.dayNight.value, "-9999.9" ); 

      setFlag(&sGranuleHandle->metadata.psMetadata.solarGains, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.solarGains.value, 
	      "\"(-9999.9,-9999.9,-9999.9,-9999.9)\"" ); 

      setFlag(&sGranuleHandle->metadata.psMetadata.tmiRolloverCoef, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.tmiRolloverCoef.value,
              tmiRolloverCoef);

      setFlag(&sGranuleHandle->metadata.coreMetadata.centerPointLat, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.coreMetadata.centerPointLat.value, "-9999.9" );

      setFlag(&sGranuleHandle->metadata.coreMetadata.centerPointLon, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.coreMetadata.centerPointLon.value, "-9999.9" );

      setFlag(&sGranuleHandle->metadata.coreMetadata.radius, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.coreMetadata.radius.value, "-9999.9" );

      setFlag(&sGranuleHandle->metadata.psMetadata.minRefThreshold, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.minRefThreshold.value, "-9999.9" );

      setFlag(&sGranuleHandle->metadata.psMetadata.radarName, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.radarName.value, "\"NULL\"" );

      setFlag(&sGranuleHandle->metadata.psMetadata.radarCity, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.radarCity.value, "\"NULL\"" );

      setFlag(&sGranuleHandle->metadata.psMetadata.radarState, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.radarState.value, "\"NULL\"" );

      setFlag(&sGranuleHandle->metadata.psMetadata.radarCountry, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.radarCountry.value, "\"NULL\"" );

      setFlag(&sGranuleHandle->metadata.psMetadata.numVOS, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.numVOS.value, "-9999" );

      setFlag(&sGranuleHandle->metadata.psMetadata.radarOriginLat, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.radarOriginLat.value, "-9999.9" );

      setFlag(&sGranuleHandle->metadata.psMetadata.radarOriginLon, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.radarOriginLon.value, "-9999.9" );

      setFlag(&sGranuleHandle->metadata.psMetadata.radarOriginAlt, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.radarOriginAlt.value, "-9999.9" );

      setFlag(&sGranuleHandle->metadata.psMetadata.radarSpacingX, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.radarSpacingX.value, "-9999.9" );

      setFlag(&sGranuleHandle->metadata.psMetadata.radarSpacingY, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.radarSpacingY.value, "-9999.9" );

      setFlag(&sGranuleHandle->metadata.psMetadata.radarSpacingZ, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.radarSpacingZ.value, "-9999.9" );

      setFlag(&sGranuleHandle->metadata.psMetadata.radarGridSizeX, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.radarGridSizeX.value, "-9999" );

      setFlag(&sGranuleHandle->metadata.psMetadata.radarGridSizeY, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.radarGridSizeY.value, "-9999" );

      setFlag(&sGranuleHandle->metadata.psMetadata.radarGridSizeZ, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.radarGridSizeZ.value, "-9999" );

      setFlag(&sGranuleHandle->metadata.psMetadata.dzcal, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.dzcal.value, "-9999.9" );

      setFlag(&sGranuleHandle->metadata.psMetadata.gvL1Cscale, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.gvL1Cscale.value, "-9999.9" );

      setFlag(&sGranuleHandle->metadata.psMetadata.alpha, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.alpha.value, "-9999.9" );

      setFlag(&sGranuleHandle->metadata.psMetadata.runtimeOption, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.runtimeOption.value, "\"NULL\"" );

      break;

    case TK_GV:
      setFlag(&sGranuleHandle->metadata.coreMetadata.ellipsoidName, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.coreMetadata.ellipsoidName.value, "\"NULL\"" );

      setFlag(&sGranuleHandle->metadata.coreMetadata.earthEqRad, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.coreMetadata.earthEqRad.value, "-9999.9" );

      setFlag(&sGranuleHandle->metadata.coreMetadata.flatteningRatio, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.coreMetadata.flatteningRatio.value, "-9999.9" );

      setFlag(&sGranuleHandle->metadata.coreMetadata.orbitModelName, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.coreMetadata.orbitModelName.value, "\"NULL\"" );

      setFlag(&sGranuleHandle->metadata.coreMetadata.orbitNumber, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.coreMetadata.orbitNumber.value, "-9999" );

      setFlag(&sGranuleHandle->metadata.coreMetadata.kepSemiMajorAxis, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.coreMetadata.kepSemiMajorAxis.value, "-9999.9" );

      setFlag(&sGranuleHandle->metadata.coreMetadata.kepMeanAnomaly, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.coreMetadata.kepMeanAnomaly.value, "-9999.9" );

      setFlag(&sGranuleHandle->metadata.coreMetadata.kepRightAscenNode, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.coreMetadata.kepRightAscenNode.value, "-9999.9" );

      setFlag(&sGranuleHandle->metadata.coreMetadata.kepArgOfPerigee, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.coreMetadata.kepArgOfPerigee.value, "-9999.9" );

      setFlag(&sGranuleHandle->metadata.coreMetadata.kepEccentricity, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.coreMetadata.kepEccentricity.value, "-9999.9" );

      setFlag(&sGranuleHandle->metadata.coreMetadata.kepInclination, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.coreMetadata.kepInclination.value, "-9999.9" );

      setFlag(&sGranuleHandle->metadata.coreMetadata.kepEpochDate, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.coreMetadata.kepEpochDate.value, "9999/99/99" );

      setFlag(&sGranuleHandle->metadata.coreMetadata.kepEpochTime, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.coreMetadata.kepEpochTime.value, "99:99:99" );

      setFlag(&sGranuleHandle->metadata.coreMetadata.kepEpochMillisec, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.coreMetadata.kepEpochMillisec.value, "-9999" );

      setFlag(&sGranuleHandle->metadata.coreMetadata.lonMaxLat, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.coreMetadata.lonMaxLat.value, "-9999.9" );

      setFlag(&sGranuleHandle->metadata.coreMetadata.meanMotion, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.coreMetadata.meanMotion.value, "-9999.9" );

      setFlag(&sGranuleHandle->metadata.coreMetadata.orbitAdjFlag, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.coreMetadata.orbitAdjFlag.value, "-9999" );

      setFlag(&sGranuleHandle->metadata.coreMetadata.attitudeMode, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.coreMetadata.attitudeMode.value, "-9999" );

      setFlag(&sGranuleHandle->metadata.coreMetadata.beginSolarBeta, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.coreMetadata.beginSolarBeta.value, "-9999.9" );

      setFlag(&sGranuleHandle->metadata.coreMetadata.endSolarBeta, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.coreMetadata.endSolarBeta.value, "-9999.9" );

      setFlag(&sGranuleHandle->metadata.coreMetadata.sensorAlgn, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.coreMetadata.sensorAlgn.value, "\"NULL\"" );

      setFlag(&sGranuleHandle->metadata.coreMetadata.sensorAlgnChanOff, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.coreMetadata.sensorAlgnChanOff.value, "\"NULL\"" );

      setFlag(&sGranuleHandle->metadata.coreMetadata.scanPathModel, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.coreMetadata.scanPathModel.value, "\"NULL\"" );

      setFlag(&sGranuleHandle->metadata.coreMetadata.scanPathParam, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.coreMetadata.scanPathParam.value, "\"NULL\"" );

      setFlag(&sGranuleHandle->metadata.coreMetadata.ephemFilename, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.coreMetadata.ephemFilename.value, "\"NULL\"" );

      setFlag(&sGranuleHandle->metadata.psMetadata.orbitSize, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.orbitSize.value, "-9999" );

      setFlag(&sGranuleHandle->metadata.psMetadata.firstScanUTCdate, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.firstScanUTCdate.value, "9999/99/99" );

      setFlag(&sGranuleHandle->metadata.psMetadata.firstScanUTCtime, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.firstScanUTCtime.value, "99:99:99" );

      setFlag(&sGranuleHandle->metadata.psMetadata.firstScanUTCmsec, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.firstScanUTCmsec.value, "-9999" );

      setFlag(&sGranuleHandle->metadata.psMetadata.firstScanSCsec, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.firstScanSCsec.value, "-9999" );

      setFlag(&sGranuleHandle->metadata.psMetadata.firstScanSCsubsec, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.firstScanSCsubsec.value, "-9999" );

      setFlag(&sGranuleHandle->metadata.psMetadata.lastScanUTCdate, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.lastScanUTCdate.value, "9999/99/99" );

      setFlag(&sGranuleHandle->metadata.psMetadata.lastScanUTCtime, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.lastScanUTCtime.value, "99:99:99" );

      setFlag(&sGranuleHandle->metadata.psMetadata.lastScanUTCmsec, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.lastScanUTCmsec.value, "-9999" );

      setFlag(&sGranuleHandle->metadata.psMetadata.lastScanSCsec, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.lastScanSCsec.value, "-9999" );

      setFlag(&sGranuleHandle->metadata.psMetadata.lastScanSCsubsec, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.lastScanSCsubsec.value, "-9999" );

      setFlag(&sGranuleHandle->metadata.psMetadata.utcfSec, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.utcfSec.value, "-9999" );

      setFlag(&sGranuleHandle->metadata.psMetadata.utcfSubsec, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.utcfSubsec.value, "-9999" );

      setFlag(&sGranuleHandle->metadata.psMetadata.utcfFlag, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.utcfFlag.value, "-9999" );

/*
      setFlag(&sGranuleHandle->metadata.psMetadata.softwareVersion, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.softwareVersion.value, "-9999" );
*/
      
      setFlag(&sGranuleHandle->metadata.psMetadata.databaseVersion, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.databaseVersion.value, "-9999" );
      
      setFlag(&sGranuleHandle->metadata.psMetadata.totalQualityCode, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.totalQualityCode.value, "\"NULL\"" );
      
      setFlag(&sGranuleHandle->metadata.psMetadata.lonOnEquator, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.lonOnEquator.value, "-9999.9" );
      
      setFlag(&sGranuleHandle->metadata.psMetadata.UTCDateOnEquator, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.UTCDateOnEquator.value, "9999/99/99" );
      
      setFlag(&sGranuleHandle->metadata.psMetadata.UTCTimeOnEquator, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.UTCTimeOnEquator.value, "99:99:99" );
      
      setFlag(&sGranuleHandle->metadata.psMetadata.millisecOnEquator, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.millisecOnEquator.value, "-9999" );
      
      setFlag(&sGranuleHandle->metadata.psMetadata.centerScanUTCDate, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.centerScanUTCDate.value, "9999/99/99" );
      
      setFlag(&sGranuleHandle->metadata.psMetadata.centerScanUTCTime, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.centerScanUTCTime.value, "99:99:99" );
      
      setFlag(&sGranuleHandle->metadata.psMetadata.centerScanUTCMillisec, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.centerScanUTCMillisec.value, "-9999" );
      
      setFlag(&sGranuleHandle->metadata.psMetadata.firstScanLat, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.firstScanLat.value, "-9999.9" );
      
      setFlag(&sGranuleHandle->metadata.psMetadata.firstScanLon, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.firstScanLon.value, "-9999.9" );
      
      setFlag(&sGranuleHandle->metadata.psMetadata.lastScanLat, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.lastScanLat.value, "-9999.9" );
      
      setFlag(&sGranuleHandle->metadata.psMetadata.lastScanLon, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.lastScanLon.value, "-9999.9" );
      
      setFlag(&sGranuleHandle->metadata.psMetadata.numOfRainScans, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.numOfRainScans.value, "-9999" );
      
      switch ( sGranuleHandle->productID )
	{
	case TK_L3A_53S:
	case TK_L3A_53TX:
	case TK_L3A_53FL:
	case TK_L3A_54S:
	case TK_L3A_54TX:
	case TK_L3A_54FL:
	case TK_L3A_55:
	  setFlag(&sGranuleHandle->metadata.psMetadata.minRefThreshold, TK_TRUE);
	  strcpy ( sGranuleHandle->metadata.psMetadata.minRefThreshold.value, "-9999.9" );

	  setFlag(&sGranuleHandle->metadata.psMetadata.numVOS, TK_TRUE);
	  strcpy ( sGranuleHandle->metadata.psMetadata.numVOS.value, "-9999" );
	case TK_L2A_53S:
	case TK_L2A_53TX:
	case TK_L2A_53FL:
	case TK_L2A_54S:
	case TK_L2A_54TX:
	case TK_L2A_54FL:
	case TK_L2A_55S:
	case TK_L2A_55TX:
	case TK_L2A_55FL:
	case TK_L2A_56:
	case TK_L2A_57:
	  setFlag(&sGranuleHandle->metadata.coreMetadata.centerPointLat, TK_TRUE);
	  strcpy ( sGranuleHandle->metadata.coreMetadata.centerPointLat.value, "-9999.9" );

	  setFlag(&sGranuleHandle->metadata.coreMetadata.centerPointLon, TK_TRUE);
	  strcpy ( sGranuleHandle->metadata.coreMetadata.centerPointLon.value, "-9999.9" );

	  setFlag(&sGranuleHandle->metadata.coreMetadata.radius, TK_TRUE);
	  strcpy ( sGranuleHandle->metadata.coreMetadata.radius.value, "-9999.9" );
	  break;
	}
      break;
    case TK_VARIOUS:
      setFlag(&sGranuleHandle->metadata.coreMetadata.centerPointLat, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.coreMetadata.centerPointLat.value, "-9999.9" );

      setFlag(&sGranuleHandle->metadata.coreMetadata.centerPointLon, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.coreMetadata.centerPointLon.value, "-9999.9" );

      setFlag(&sGranuleHandle->metadata.coreMetadata.radius, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.coreMetadata.radius.value, "-9999.9" );

      setFlag(&sGranuleHandle->metadata.psMetadata.runtimeOption, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.runtimeOption.value, "\"NULL\"" );

      setFlag(&sGranuleHandle->metadata.psMetadata.alpha, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.alpha.value, "-9999.9" );

      setFlag(&sGranuleHandle->metadata.psMetadata.gvL1Cscale, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.gvL1Cscale.value, "-9999.9" );

      setFlag(&sGranuleHandle->metadata.psMetadata.dzcal, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.dzcal.value, "-9999.9" );
      break;
    }

  switch ( prodType ( sGranuleHandle->productID ) )
    {
    case TK_L1:
      setFlag(&sGranuleHandle->metadata.coreMetadata.latitudeRes, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.coreMetadata.latitudeRes.value, "-9999.9" );

      setFlag(&sGranuleHandle->metadata.coreMetadata.longitudeRes, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.coreMetadata.longitudeRes.value, "-9999.9" );

      setFlag(&sGranuleHandle->metadata.coreMetadata.geoCoordUnits, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.coreMetadata.geoCoordUnits.value, "\"NULL\"" );
      switch ( sGranuleHandle->productID )
	{
	case TK_L1B_11:
	  setFlag(&sGranuleHandle->metadata.psMetadata.dayNight, TK_TRUE);
	  strcpy ( sGranuleHandle->metadata.psMetadata.dayNight.value, "-9999.9" ); 

	  setFlag ( &sGranuleHandle->metadata.psMetadata.solarGains, TK_TRUE );
	  strcpy ( sGranuleHandle->metadata.psMetadata.solarGains.value, 
		  "\"(-9999.9,-9999.9,-9999.9,-9999.9)\"" );

	  setFlag(&sGranuleHandle->metadata.psMetadata.calCoeffVersion, TK_TRUE);
	  strcpy ( sGranuleHandle->metadata.psMetadata.calCoeffVersion.value, "-9999" ); 

	  setFlag(&sGranuleHandle->metadata.psMetadata.radarWavelength, TK_TRUE);
	  strcpy ( sGranuleHandle->metadata.psMetadata.radarWavelength.value, "-9999.9" );

	  break;

	case TK_L1B_GV:

	  setFlag(&sGranuleHandle->metadata.psMetadata.dzcal, TK_TRUE);
	  strcpy ( sGranuleHandle->metadata.psMetadata.dzcal.value, "-9999.9" );

	  setFlag(&sGranuleHandle->metadata.psMetadata.gvL1Cscale, TK_TRUE);
	  strcpy ( sGranuleHandle->metadata.psMetadata.gvL1Cscale.value, "-9999.9" );

	  setFlag(&sGranuleHandle->metadata.psMetadata.alpha, TK_TRUE);
	  strcpy ( sGranuleHandle->metadata.psMetadata.alpha.value, "-9999.9" );

	case TK_L1C_GV:

	  setFlag(&sGranuleHandle->metadata.psMetadata.dayNight, TK_TRUE);
	  strcpy ( sGranuleHandle->metadata.psMetadata.dayNight.value, "-9999.9" ); 

	  setFlag ( &sGranuleHandle->metadata.psMetadata.solarGains, TK_TRUE );
	  strcpy ( sGranuleHandle->metadata.psMetadata.solarGains.value, 
		  "\"(-9999.9,-9999.9,-9999.9,-9999.9)\"" );

          setFlag(&sGranuleHandle->metadata.psMetadata.tmiRolloverCoef, TK_TRUE);
          strcpy ( sGranuleHandle->metadata.psMetadata.tmiRolloverCoef.value,
                  tmiRolloverCoef);

	  setFlag(&sGranuleHandle->metadata.psMetadata.radarOriginLat, TK_TRUE);
	  strcpy ( sGranuleHandle->metadata.psMetadata.radarOriginLat.value, "-9999.9" );

	  setFlag(&sGranuleHandle->metadata.psMetadata.radarOriginLon, TK_TRUE);
	  strcpy ( sGranuleHandle->metadata.psMetadata.radarOriginLon.value, "-9999.9" );

	  setFlag(&sGranuleHandle->metadata.psMetadata.radarOriginAlt, TK_TRUE);
	  strcpy ( sGranuleHandle->metadata.psMetadata.radarOriginAlt.value, "-9999.9" );

	  setFlag(&sGranuleHandle->metadata.psMetadata.radarSpacingX, TK_TRUE);
	  strcpy ( sGranuleHandle->metadata.psMetadata.radarSpacingX.value, "-9999.9" );

	  setFlag(&sGranuleHandle->metadata.psMetadata.radarSpacingY, TK_TRUE);
	  strcpy ( sGranuleHandle->metadata.psMetadata.radarSpacingY.value, "-9999.9" );

	  setFlag(&sGranuleHandle->metadata.psMetadata.radarSpacingZ, TK_TRUE);
	  strcpy ( sGranuleHandle->metadata.psMetadata.radarSpacingZ.value, "-9999.9" );

	  setFlag(&sGranuleHandle->metadata.psMetadata.radarGridSizeX, TK_TRUE);
	  strcpy ( sGranuleHandle->metadata.psMetadata.radarGridSizeX.value, "-9999" );

	  setFlag(&sGranuleHandle->metadata.psMetadata.radarGridSizeY, TK_TRUE);
	  strcpy ( sGranuleHandle->metadata.psMetadata.radarGridSizeY.value, "-9999" );

	  setFlag(&sGranuleHandle->metadata.psMetadata.radarGridSizeZ, TK_TRUE);
	  strcpy ( sGranuleHandle->metadata.psMetadata.radarGridSizeZ.value, "-9999" );

	  break;

	case TK_L1B_01:
	  setFlag(&sGranuleHandle->metadata.psMetadata.calCoeffVersion, TK_TRUE);
	  strcpy ( sGranuleHandle->metadata.psMetadata.calCoeffVersion.value, "-9999" ); 

	  setFlag(&sGranuleHandle->metadata.psMetadata.radarWavelength, TK_TRUE);
	  strcpy ( sGranuleHandle->metadata.psMetadata.radarWavelength.value, "-9999.9" );

          setFlag(&sGranuleHandle->metadata.psMetadata.tmiRolloverCoef, TK_TRUE);
          strcpy ( sGranuleHandle->metadata.psMetadata.tmiRolloverCoef.value,
                  tmiRolloverCoef);

	  break;
	}
      break;
    case TK_L2:
/* Toolkit version 5.7.2 correction - see notes -
      setFlag(&sGranuleHandle->metadata.psMetadata.calCoeffVersion, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.calCoeffVersion.value, "-9999" ); 
*/

      setFlag(&sGranuleHandle->metadata.psMetadata.dataAccuracy, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.dataAccuracy.value, "\"NULL\"" );

      setFlag(&sGranuleHandle->metadata.psMetadata.maxValidChannel, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.maxValidChannel.value, "\"NULL\"" );

      setFlag(&sGranuleHandle->metadata.psMetadata.minValidChannel, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.minValidChannel.value, "\"NULL\"" );

      setFlag(&sGranuleHandle->metadata.psMetadata.dayNight, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.dayNight.value, "-9999.9" ); 

      setFlag ( &sGranuleHandle->metadata.psMetadata.solarGains, TK_TRUE );
      strcpy ( sGranuleHandle->metadata.psMetadata.solarGains.value, 
	      "\"(-9999.9,-9999.9,-9999.9,-9999.9)\"" );

      if ( sGranuleHandle->productID != TK_L2A_12)
      {
          setFlag(&sGranuleHandle->metadata.psMetadata.tmiRolloverCoef, TK_TRUE);
          strcpy ( sGranuleHandle->metadata.psMetadata.tmiRolloverCoef.value,tmiRolloverCoef);
      }

      setFlag(&sGranuleHandle->metadata.psMetadata.dzcal, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.dzcal.value, "-9999.9" );

      setFlag(&sGranuleHandle->metadata.psMetadata.gvL1Cscale, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.gvL1Cscale.value, "-9999.9" );

      setFlag(&sGranuleHandle->metadata.psMetadata.alpha, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.alpha.value, "-9999.9" );

      switch ( sGranuleHandle->productID )
	{
	case TK_L2A_12:
	case TK_L2A_21:
	case TK_L2A_23:
	case TK_L2A_25:
	case TK_L2B_31:
	  setFlag(&sGranuleHandle->metadata.coreMetadata.latitudeRes, TK_TRUE);
	  strcpy ( sGranuleHandle->metadata.coreMetadata.latitudeRes.value, "-9999.9" );

	  setFlag(&sGranuleHandle->metadata.coreMetadata.longitudeRes, TK_TRUE);
	  strcpy ( sGranuleHandle->metadata.coreMetadata.longitudeRes.value, "-9999.9" );

	  setFlag(&sGranuleHandle->metadata.coreMetadata.geoCoordUnits, TK_TRUE);
	  strcpy ( sGranuleHandle->metadata.coreMetadata.geoCoordUnits.value, "\"NULL\"" ); 

	  setFlag(&sGranuleHandle->metadata.psMetadata.minMaxUnits, TK_TRUE);
	  strcpy ( sGranuleHandle->metadata.psMetadata.minMaxUnits.value, "\"NULL\"" );

	  setFlag(&sGranuleHandle->metadata.psMetadata.radarWavelength, TK_TRUE);
	  strcpy ( sGranuleHandle->metadata.psMetadata.radarWavelength.value, "-9999.9" );

	  break;
	}
      break;
    case TK_L3:
      setFlag(&sGranuleHandle->metadata.coreMetadata.ellipsoidName, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.coreMetadata.ellipsoidName.value, "\"NULL\"" );

      setFlag(&sGranuleHandle->metadata.coreMetadata.earthEqRad, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.coreMetadata.earthEqRad.value, "-9999.9" );

      setFlag(&sGranuleHandle->metadata.coreMetadata.flatteningRatio, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.coreMetadata.flatteningRatio.value, "-9999.9" );

      setFlag(&sGranuleHandle->metadata.coreMetadata.orbitModelName, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.coreMetadata.orbitModelName.value, "\"NULL\"" );

      setFlag(&sGranuleHandle->metadata.coreMetadata.orbitNumber, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.coreMetadata.orbitNumber.value, "-9999" );

      setFlag(&sGranuleHandle->metadata.coreMetadata.kepSemiMajorAxis, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.coreMetadata.kepSemiMajorAxis.value, "-9999.9" );

      setFlag(&sGranuleHandle->metadata.coreMetadata.kepMeanAnomaly, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.coreMetadata.kepMeanAnomaly.value, "-9999.9" );

      setFlag(&sGranuleHandle->metadata.coreMetadata.kepRightAscenNode, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.coreMetadata.kepRightAscenNode.value, "-9999.9" );

      setFlag(&sGranuleHandle->metadata.coreMetadata.kepArgOfPerigee, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.coreMetadata.kepArgOfPerigee.value, "-9999.9" );

      setFlag(&sGranuleHandle->metadata.coreMetadata.kepEccentricity, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.coreMetadata.kepEccentricity.value, "-9999.9" );

      setFlag(&sGranuleHandle->metadata.coreMetadata.kepInclination, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.coreMetadata.kepInclination.value, "-9999.9" );

      setFlag(&sGranuleHandle->metadata.coreMetadata.kepEpochDate, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.coreMetadata.kepEpochDate.value, "9999/99/99" );

      setFlag(&sGranuleHandle->metadata.coreMetadata.kepEpochTime, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.coreMetadata.kepEpochTime.value, "99:99:99" );

      setFlag(&sGranuleHandle->metadata.coreMetadata.kepEpochMillisec, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.coreMetadata.kepEpochMillisec.value, "-9999.9" );

      setFlag(&sGranuleHandle->metadata.psMetadata.missingData, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.missingData.value, "-9999" );

      setFlag(&sGranuleHandle->metadata.psMetadata.percentBadMissPix, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.percentBadMissPix.value, "\"NULL\"" );

      setFlag(&sGranuleHandle->metadata.psMetadata.numDataGap, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.numDataGap.value, "-9999.9" );

      setFlag(&sGranuleHandle->metadata.psMetadata.dataGap, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.dataGap.value, "-9999.9" );

      setFlag(&sGranuleHandle->metadata.psMetadata.dataAccuracy, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.dataAccuracy.value, "\"NULL\"" );

      setFlag(&sGranuleHandle->metadata.psMetadata.maxValidChannel, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.maxValidChannel.value, "\"NULL\"" );

      setFlag(&sGranuleHandle->metadata.psMetadata.minValidChannel, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.minValidChannel.value, "\"NULL\"" );

      setFlag(&sGranuleHandle->metadata.psMetadata.minMaxUnits, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.minMaxUnits.value, "\"NULL\"" );

      setFlag(&sGranuleHandle->metadata.psMetadata.inputID, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.inputID.value, "\"NULL\"" );

      setFlag(&sGranuleHandle->metadata.psMetadata.dateGenInputFile, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.dateGenInputFile.value, "\"NULL\"" );

      setFlag(&sGranuleHandle->metadata.psMetadata.dataCenterSrc, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.dataCenterSrc.value, "\"NULL\"" );

      setFlag(&sGranuleHandle->metadata.psMetadata.dayNight, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.dayNight.value, "-9999.9" ); 

      setFlag ( &sGranuleHandle->metadata.psMetadata.solarGains, TK_TRUE );
      strcpy ( sGranuleHandle->metadata.psMetadata.solarGains.value, 
	      "\"(-9999.9,-9999.9,-9999.9,-9999.9)\"" );

      if ( sGranuleHandle->productID != TK_L3A_11)
      {

          setFlag(&sGranuleHandle->metadata.psMetadata.tmiRolloverCoef, TK_TRUE);
          strcpy ( sGranuleHandle->metadata.psMetadata.tmiRolloverCoef.value,
              tmiRolloverCoef);
      }

      setFlag(&sGranuleHandle->metadata.coreMetadata.lonMaxLat, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.coreMetadata.lonMaxLat.value, "-9999.9" );

      setFlag(&sGranuleHandle->metadata.coreMetadata.meanMotion, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.coreMetadata.meanMotion.value, "-9999.9" );

      setFlag(&sGranuleHandle->metadata.coreMetadata.orbitAdjFlag, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.coreMetadata.orbitAdjFlag.value, "-9999" );

      setFlag(&sGranuleHandle->metadata.coreMetadata.attitudeMode, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.coreMetadata.attitudeMode.value, "-9999" );

      setFlag(&sGranuleHandle->metadata.coreMetadata.beginSolarBeta, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.coreMetadata.beginSolarBeta.value, "-9999.9" );

      setFlag(&sGranuleHandle->metadata.coreMetadata.endSolarBeta, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.coreMetadata.endSolarBeta.value, "-9999.9" );

      setFlag(&sGranuleHandle->metadata.coreMetadata.sensorAlgn, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.coreMetadata.sensorAlgn.value, "\"NULL\"" );

      setFlag(&sGranuleHandle->metadata.coreMetadata.sensorAlgnChanOff, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.coreMetadata.sensorAlgnChanOff.value, "\"NULL\"" );

      setFlag(&sGranuleHandle->metadata.coreMetadata.scanPathModel, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.coreMetadata.scanPathModel.value, "\"NULL\"" );

      setFlag(&sGranuleHandle->metadata.coreMetadata.scanPathParam, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.coreMetadata.scanPathParam.value, "\"NULL\"" );

      setFlag(&sGranuleHandle->metadata.coreMetadata.ephemFilename, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.coreMetadata.ephemFilename.value, "\"NULL\"" );

      setFlag(&sGranuleHandle->metadata.psMetadata.orbitSize, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.orbitSize.value, "-9999" );

      setFlag(&sGranuleHandle->metadata.psMetadata.firstScanUTCdate, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.firstScanUTCdate.value, "9999/99/99" );

      setFlag(&sGranuleHandle->metadata.psMetadata.firstScanUTCtime, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.firstScanUTCtime.value, "99:99:99" );

      setFlag(&sGranuleHandle->metadata.psMetadata.firstScanUTCmsec, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.firstScanUTCmsec.value, "-9999" );

      setFlag(&sGranuleHandle->metadata.psMetadata.firstScanSCsec, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.firstScanSCsec.value, "-9999" );

      setFlag(&sGranuleHandle->metadata.psMetadata.firstScanSCsubsec, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.firstScanSCsubsec.value, "-9999" );

      setFlag(&sGranuleHandle->metadata.psMetadata.lastScanUTCdate, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.lastScanUTCdate.value, "9999/99/99" );

      setFlag(&sGranuleHandle->metadata.psMetadata.lastScanUTCtime, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.lastScanUTCtime.value, "99:99:99" );

      setFlag(&sGranuleHandle->metadata.psMetadata.lastScanUTCmsec, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.lastScanUTCmsec.value, "-9999" );

      setFlag(&sGranuleHandle->metadata.psMetadata.lastScanSCsec, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.lastScanSCsec.value, "-9999" );

      setFlag(&sGranuleHandle->metadata.psMetadata.lastScanSCsubsec, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.lastScanSCsubsec.value, "-9999" );

      setFlag(&sGranuleHandle->metadata.psMetadata.utcfSec, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.utcfSec.value, "-9999" );

      setFlag(&sGranuleHandle->metadata.psMetadata.utcfSubsec, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.utcfSubsec.value, "-9999" );

      setFlag(&sGranuleHandle->metadata.psMetadata.utcfFlag, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.utcfFlag.value, "-9999" );

      setFlag(&sGranuleHandle->metadata.psMetadata.dzcal, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.dzcal.value, "-9999.9" );

      setFlag(&sGranuleHandle->metadata.psMetadata.gvL1Cscale, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.gvL1Cscale.value, "-9999.9" );

      setFlag(&sGranuleHandle->metadata.psMetadata.alpha, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.alpha.value, "-9999.9" );

      setFlag(&sGranuleHandle->metadata.psMetadata.lonOnEquator, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.lonOnEquator.value, "-9999.9" );
      
      setFlag(&sGranuleHandle->metadata.psMetadata.UTCDateOnEquator, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.UTCDateOnEquator.value, "9999/99/99" );
      
      setFlag(&sGranuleHandle->metadata.psMetadata.UTCTimeOnEquator, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.UTCTimeOnEquator.value, "99:99:99" );
      
      setFlag(&sGranuleHandle->metadata.psMetadata.millisecOnEquator, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.millisecOnEquator.value, "-9999" );
      
      setFlag(&sGranuleHandle->metadata.psMetadata.centerScanUTCDate, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.centerScanUTCDate.value, "9999/99/99" );
      
      setFlag(&sGranuleHandle->metadata.psMetadata.centerScanUTCTime, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.centerScanUTCTime.value, "99:99:99" );
      
      setFlag(&sGranuleHandle->metadata.psMetadata.centerScanUTCMillisec, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.centerScanUTCMillisec.value, "-9999" );
      
      setFlag(&sGranuleHandle->metadata.psMetadata.firstScanLat, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.firstScanLat.value, "-9999.9" );
      
      setFlag(&sGranuleHandle->metadata.psMetadata.firstScanLon, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.firstScanLon.value, "-9999.9" );
      
      setFlag(&sGranuleHandle->metadata.psMetadata.lastScanLat, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.lastScanLat.value, "-9999.9" );
      
      setFlag(&sGranuleHandle->metadata.psMetadata.lastScanLon, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.lastScanLon.value, "-9999.9" );
      
      setFlag(&sGranuleHandle->metadata.psMetadata.numOfRainScans, TK_TRUE);
      strcpy ( sGranuleHandle->metadata.psMetadata.numOfRainScans.value, "-9999" );

      switch ( sGranuleHandle->productID )
	{
	case TK_L3A_11:
	case TK_L3A_12:
	case TK_L3A_25:
	case TK_L3A_26:
	case TK_L3A_27:
	case TK_L3B_31:
	case TK_L3B_42:
	case TK_L3B_43:

	  setFlag(&sGranuleHandle->metadata.psMetadata.radarWavelength, TK_TRUE);
	  strcpy ( sGranuleHandle->metadata.psMetadata.radarWavelength.value, "-9999.9" );

	  break;
	}

      break;
    }
  return;
}

#ifndef TSU_ENV 
/*****************************************************************************
*
*  FUNCTION STATEMENT:
*	updateStartStopTimesinGranuleMetadata (IO_HANDLE *sOutputGranuleHandle,
*                                              IO_HANDLE *sInputGranuleHandle)
*
*  DESCRIPTION:
*	This routine will update the start and stop times in the tsdis
*       database for all applicable higher level products.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       sOutputGranuleHandle    A structure containing information about the 
*                               output file from which data can be obtained.  It 
*                               also contains metadata information. 
*
*       sInputGranuleHandle     A structure containing information about the 
*                               input file from which data can be obtained.  It 
*                               also contains metadata information. 
*
*  OUTPUT PARAMETERS:		None.
*
*  CALLED BY:                   finalizeMetadata()
*
*  RETURN VALUES:               TK_SUCCESS or appropriate error value.
*
*  ROUTINES CALLED:
*
*       IMopen                  Opens a database session.
*
*       IMallocateRec           Allocates memory for the database record.
*
*       IMupdate                Updates the start and stop times in the database.
*
*       IMfreeRec               Frees memory allocated by IMallocateRec
*
*       IMclose                 Closes the database session.
*
*	TKreportWarning         This routine reports warning errors to console
*                               and continues processing.
*
******************************************************************************/
int updateStartStopTimesinGranuleMetadata ( IO_HANDLE *sOutputGranuleHandle, 
					    IO_HANDLE *sInputGranuleHandle )
{
  IMsession *sSession;
  IMgranule_MetaData_Update_Struct *sOutputGranuleUpdate;
  int status = TK_SUCCESS;

  if ( ( sOutputGranuleHandle->productID != TK_L1B_01 ) &&
      ( sOutputGranuleHandle->productID != TK_L1B_11 ) &&
      ( sOutputGranuleHandle->productID != TK_L1B_21 ) &&
      ( sOutputGranuleHandle->productID != TK_L1B_GV ) &&
      ( sOutputGranuleHandle->productID != TK_L3A_46 ) &&
      ( sOutputGranuleHandle->productID != TK_L2A_56 ) &&
      ( sOutputGranuleHandle->productID != TK_L2A_57 ) )
    {
      status = IMopen(TK_IM_USER_ID, &sSession); 
      if ( status != IM_SUCCESS )
	{
	  TKreportWarning ( W_TK_FAILIMOPEN );
	  return ( W_TK_FAILIMOPEN );
	}
      
      sOutputGranuleUpdate = IMallocateRec ( GRANULE_METADATA_UPDATE );
      if ( sOutputGranuleUpdate == NULL )
	{
	  TKreportWarning ( W_TK_FAILALLOCREC );
	  IMclose ( sSession );
	  return ( W_TK_FAILALLOCREC );
	}
      
      
      strcpy ( sOutputGranuleUpdate->Granule_ID, 
	      sOutputGranuleHandle->gIDtokens.granule_ID );
      
      sOutputGranuleUpdate->Start_Datim = ( struct tm * ) calloc ( 1,
						     sizeof ( struct tm ) );
      sOutputGranuleUpdate->Stop_Datim = ( struct tm * ) calloc ( 1,
						     sizeof ( struct tm ) );
      if ( ( sOutputGranuleUpdate->Start_Datim == NULL ) || 
	  ( sOutputGranuleUpdate->Stop_Datim == NULL ) )
	{
	  TKreportWarning ( E_TK_BADMEMALLCAT );
	  IMfreeRec ( sOutputGranuleUpdate );
	  IMclose ( sSession );
	  return ( E_TK_BADMEMALLCAT );
	}
      
      /*
       *  Extract the beginning date.
       */
      if ( sOutputGranuleHandle->metadata.coreMetadata.beginDate.booleanFlag ==
	  TK_TRUE ) /*  Get the beginning date from the output handle  */
	sscanf ( sOutputGranuleHandle->metadata.coreMetadata.beginDate.value,
		"%4d/%2d/%2d",
		&sOutputGranuleUpdate->Start_Datim->tm_year,
		&sOutputGranuleUpdate->Start_Datim->tm_mon,
		&sOutputGranuleUpdate->Start_Datim->tm_mday );
      else if ( sInputGranuleHandle != NULL ) /*  Get the beginning date from the input handle  */
	sscanf ( sInputGranuleHandle->metadata.coreMetadata.beginDate.value,
		"%4d/%2d/%2d",
		&sOutputGranuleUpdate->Start_Datim->tm_year,
		&sOutputGranuleUpdate->Start_Datim->tm_mon,
		&sOutputGranuleUpdate->Start_Datim->tm_mday );
      else 
	{
	  TKreportWarning ( W_TK_NOSTARTSTOPTIMES );
	  IMfreeRec ( sOutputGranuleUpdate );
	  IMclose ( sSession );
	  return ( W_TK_NOSTARTSTOPTIMES );
	}

      (sOutputGranuleUpdate->Start_Datim->tm_mon)--;
      sOutputGranuleUpdate->Start_Datim->tm_year -= 1900;
      
      /*
       *  Extract the beginning time.
       */
      if ( sOutputGranuleHandle->metadata.coreMetadata.beginTime.booleanFlag ==
	  TK_TRUE ) /*  Get the beginning time from the output handle  */
	sscanf ( sOutputGranuleHandle->metadata.coreMetadata.beginTime.value,
		"%2d:%2d:%2d",
		&sOutputGranuleUpdate->Start_Datim->tm_hour,
		&sOutputGranuleUpdate->Start_Datim->tm_min,
		&sOutputGranuleUpdate->Start_Datim->tm_sec );
      else if ( sInputGranuleHandle != NULL )  /*  Get the beginning date from the input handle  */
	sscanf ( sInputGranuleHandle->metadata.coreMetadata.beginTime.value,
		"%2d:%2d:%2d",
		&sOutputGranuleUpdate->Start_Datim->tm_hour,
		&sOutputGranuleUpdate->Start_Datim->tm_min,
		&sOutputGranuleUpdate->Start_Datim->tm_sec );
      else 
	{
	  TKreportWarning ( W_TK_NOSTARTSTOPTIMES );
	  IMfreeRec ( sOutputGranuleUpdate );
	  IMclose ( sSession );
	  return ( W_TK_NOSTARTSTOPTIMES );
	}
      
      /*
       *  Extract the stop date.
       */
      if ( sOutputGranuleHandle->metadata.coreMetadata.endDate.booleanFlag ==
	  TK_TRUE ) /*  Get the stop date from the output handle  */
	sscanf ( sOutputGranuleHandle->metadata.coreMetadata.endDate.value,
		"%4d/%2d/%2d",
		&sOutputGranuleUpdate->Stop_Datim->tm_year,
		&sOutputGranuleUpdate->Stop_Datim->tm_mon,
		&sOutputGranuleUpdate->Stop_Datim->tm_mday );
      else if ( sInputGranuleHandle != NULL ) /*  Get the stop date from the input handle  */
	sscanf ( sInputGranuleHandle->metadata.coreMetadata.endDate.value,
		"%4d/%2d/%2d",
		&sOutputGranuleUpdate->Stop_Datim->tm_year,
		&sOutputGranuleUpdate->Stop_Datim->tm_mon,
		&sOutputGranuleUpdate->Stop_Datim->tm_mday );
      else 
	{
	  TKreportWarning ( W_TK_NOSTARTSTOPTIMES );
	  IMfreeRec ( sOutputGranuleUpdate );
	  IMclose ( sSession );
	  return ( W_TK_NOSTARTSTOPTIMES );
	}
      
      (sOutputGranuleUpdate->Stop_Datim->tm_mon)--;
      sOutputGranuleUpdate->Stop_Datim->tm_year -= 1900;
      
      /*
       *  Extract the stop time
       */
      if ( sOutputGranuleHandle->metadata.coreMetadata.endTime.booleanFlag ==
	  TK_TRUE ) /*  Get the stop time from the output handle  */
	sscanf ( sOutputGranuleHandle->metadata.coreMetadata.endTime.value,
		"%2d:%2d:%2d",
		&sOutputGranuleUpdate->Stop_Datim->tm_hour,
		&sOutputGranuleUpdate->Stop_Datim->tm_min,
		&sOutputGranuleUpdate->Stop_Datim->tm_sec );
      else if ( sInputGranuleHandle != NULL ) /*  Get the stop date from the input handle  */
	sscanf ( sInputGranuleHandle->metadata.coreMetadata.endTime.value,
		"%2d:%2d:%2d",
		&sOutputGranuleUpdate->Stop_Datim->tm_hour,
		&sOutputGranuleUpdate->Stop_Datim->tm_min,
		&sOutputGranuleUpdate->Stop_Datim->tm_sec );
      else 
	{
	  TKreportWarning ( W_TK_NOSTARTSTOPTIMES );
	  IMfreeRec ( sOutputGranuleUpdate );
	  IMclose ( sSession );
	  return ( W_TK_NOSTARTSTOPTIMES );
	}
      
      sOutputGranuleUpdate->Start_Datim_Chg_Flag = 1;
      sOutputGranuleUpdate->Stop_Datim_Chg_Flag = 1;
      status = IMupdate ( sSession, ( IMdata * ) sOutputGranuleUpdate );
      if ( status != IM_SUCCESS )
	{
	  IMfreeRec ( sOutputGranuleUpdate );
	  IMclose ( sSession );
	  if ( status == W_IM_NOROWS_UPDATED )
	    {
	      TKreportWarning ( W_TK_NORECSUPDATE );
	      return ( W_TK_NORECSUPDATE );
	    }
	  else
	    {
	      TKreportWarning ( W_TK_FAILIMUPDATE );
	      return ( W_TK_FAILIMUPDATE );
	    }
	}
      
      IMclose ( sSession );
    }
  return ( status );
}
#endif
 
/*****************************************************************************
*
*  FUNCTION STATEMENT:
*	finalizeMetadata(IO_HANDLE *sGranuleHandle)
*
*  DESCRIPTION:
*	This routine writes metadata into the HDF file if the filemode is
*	write.  It also deallocates memory.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       sGranuleHandle          A structure containing information about the file
*                               from which data can be obtained.  It also
*				contains metadata information. 
*
*  OUTPUT PARAMETERS:		
*       Parameter Name          Description
*       --------------          -----------
*       sGranuleHandle          Now the memory allocated for its field metadata
*				is deallocated.
*
*  CALLED BY:                   TKclose()
*
*  RETURN VALUES:               None.
*
*  ROUTINES CALLED:
*       create_pvl              Creates a pvl string
*
*	TKreportWarning         This routine reports warning errors to console
*                               and continues processing.
*
*       write_pvl_to_hdffile    Stores the pvl string as a global attribute in 
*                               an HDF file
*
*       metaFree                Frees memory allocated by mdcpy
*
******************************************************************************/
int finalizeMetadata(IO_HANDLE *sGranuleHandle)
{

  char	*metadataStr;
  char  *startOfHDF = NULL;
  int	status = TK_SUCCESS;
  int   updateGranule;
  IO_HANDLE	*tempHandle = NULL; 
  
  updateGranule = manageFiles(sGranuleHandle, &tempHandle, CHECK_FILE_STATUS);
  if (updateGranule == TK_NOUPDATE)
    {
      correctNULLElements ( sGranuleHandle );
      createMetadataPVL(sGranuleHandle);
    }
  else
    {
      correctNULLElements ( sGranuleHandle );
      
      metadataStr = (char *) malloc (TK_MAX_SZ_ATTRIBUTE * sizeof(metadataStr));
      
      if (metadataStr == NULL)
	{
	  /* Memory allocation failed in finalizeMetadata. */
	  TKreportError(E_TK_MEMFAILMETA);
	}
      
      /* 
       ** Write core metadata to attribute string 
       */
      metadataStr[0] = '\0';
      
      if (!sGranuleHandle->realTimeFlag)
      {  
      /*
       * BEGIN non-real time
       */
      if (sGranuleHandle->metadata.coreMetadata.orbitNumber.booleanFlag != TK_TRUE)

       	create_pvl(metadataStr,
		   tempHandle->metadata.coreMetadata.orbitNumber.name,
		   tempHandle->metadata.coreMetadata.orbitNumber.value);
      else
    	create_pvl(metadataStr, 
		   sGranuleHandle->metadata.coreMetadata.orbitNumber.name,
		   sGranuleHandle->metadata.coreMetadata.orbitNumber.value);
      /*
       * END non-real time
       */
      }
      
      if (sGranuleHandle->metadata.coreMetadata.beginDate.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.coreMetadata.beginDate.name,
		   tempHandle->metadata.coreMetadata.beginDate.value);
      else
	create_pvl(metadataStr, 
		   sGranuleHandle->metadata.coreMetadata.beginDate.name, 
		   sGranuleHandle->metadata.coreMetadata.beginDate.value);
      
      
      if (sGranuleHandle->metadata.coreMetadata.beginTime.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.coreMetadata.beginTime.name,
		   tempHandle->metadata.coreMetadata.beginTime.value);
      else
  	create_pvl(metadataStr, 
		   sGranuleHandle->metadata.coreMetadata.beginTime.name, 
		   sGranuleHandle->metadata.coreMetadata.beginTime.value);
      
      if (sGranuleHandle->metadata.coreMetadata.endDate.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.coreMetadata.endDate.name,
		   tempHandle->metadata.coreMetadata.endDate.value);
      else
    	create_pvl(metadataStr, 
		   sGranuleHandle->metadata.coreMetadata.endDate.name, 
		   sGranuleHandle->metadata.coreMetadata.endDate.value);
      
      if (sGranuleHandle->metadata.coreMetadata.endTime.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.coreMetadata.endTime.name,
		   tempHandle->metadata.coreMetadata.endTime.value);
      else
    	create_pvl(metadataStr, 
		   sGranuleHandle->metadata.coreMetadata.endTime.name, 
		   sGranuleHandle->metadata.coreMetadata.endTime.value);
      
      if (sGranuleHandle->metadata.coreMetadata.granuleID.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.coreMetadata.granuleID.name,
		   tempHandle->metadata.coreMetadata.granuleID.value);
      else
	create_pvl(metadataStr, 
		   sGranuleHandle->metadata.coreMetadata.granuleID.name, 
		   sGranuleHandle->metadata.coreMetadata.granuleID.value);
      
      if (!sGranuleHandle->realTimeFlag)
      {
      /*
       * BEGIN non-real time
       */

      if (sGranuleHandle->metadata.coreMetadata.dataID.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.coreMetadata.dataID.name,
		   tempHandle->metadata.coreMetadata.dataID.value);
      else
  	create_pvl(metadataStr, 
		   sGranuleHandle->metadata.coreMetadata.dataID.name, 
		   sGranuleHandle->metadata.coreMetadata.dataID.value);
      
      /* use the file size obtained from the system call to size() */
      create_pvl(metadataStr, 
		 sGranuleHandle->metadata.coreMetadata.fileSize.name, 
		 sGranuleHandle->fsize);

      /*
       * END non-real time
       */
      }
      if (sGranuleHandle->metadata.coreMetadata.lonMaxLat.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.coreMetadata.lonMaxLat.name,
		   tempHandle->metadata.coreMetadata.lonMaxLat.value);
      else
  	create_pvl(metadataStr, 
		   sGranuleHandle->metadata.coreMetadata.lonMaxLat.name, 
		   sGranuleHandle->metadata.coreMetadata.lonMaxLat.value);

      if (!sGranuleHandle->realTimeFlag)
      {
      /*
       * BEGIN non-real time
       */
      if (sGranuleHandle->metadata.coreMetadata.spatCovType.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.coreMetadata.spatCovType.name,
		   tempHandle->metadata.coreMetadata.spatCovType.value);
      else
  	create_pvl(metadataStr, 
		   sGranuleHandle->metadata.coreMetadata.spatCovType.name, 
		   sGranuleHandle->metadata.coreMetadata.spatCovType.value);
      
      if (sGranuleHandle->metadata.coreMetadata.earthEqRad.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.coreMetadata.ellipsoidName.name,
		   tempHandle->metadata.coreMetadata.ellipsoidName.value);
      else
  	create_pvl(metadataStr, 
		   sGranuleHandle->metadata.coreMetadata.ellipsoidName.name, 
		   sGranuleHandle->metadata.coreMetadata.ellipsoidName.value);
      
      if (sGranuleHandle->metadata.coreMetadata.earthEqRad.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.coreMetadata.earthEqRad.name,
		   tempHandle->metadata.coreMetadata.earthEqRad.value);
      else
  	create_pvl(metadataStr, 
		   sGranuleHandle->metadata.coreMetadata.earthEqRad.name, 
		   sGranuleHandle->metadata.coreMetadata.earthEqRad.value);
      
      if (sGranuleHandle->metadata.coreMetadata.flatteningRatio.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.coreMetadata.flatteningRatio.name,
		   tempHandle->metadata.coreMetadata.flatteningRatio.value);
      else
  	create_pvl(metadataStr, 
		   sGranuleHandle->metadata.coreMetadata.flatteningRatio.name, 
		   sGranuleHandle->metadata.coreMetadata.flatteningRatio.value);
      
      if (sGranuleHandle->metadata.coreMetadata.orbitModelName.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.coreMetadata.orbitModelName.name,
		   tempHandle->metadata.coreMetadata.orbitModelName.value);
      else
  	create_pvl(metadataStr, 
		   sGranuleHandle->metadata.coreMetadata.orbitModelName.name, 
		   sGranuleHandle->metadata.coreMetadata.orbitModelName.value);
      
      
      if (sGranuleHandle->metadata.coreMetadata.kepSemiMajorAxis.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.coreMetadata.kepSemiMajorAxis.name,
		   tempHandle->metadata.coreMetadata.kepSemiMajorAxis.value);
      else
  	create_pvl(metadataStr, 
		   sGranuleHandle->metadata.coreMetadata.kepSemiMajorAxis.name, 
		   sGranuleHandle->metadata.coreMetadata.kepSemiMajorAxis.value);
      
      
      if (sGranuleHandle->metadata.coreMetadata.kepMeanAnomaly.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.coreMetadata.kepMeanAnomaly.name,
		   tempHandle->metadata.coreMetadata.kepMeanAnomaly.value);
      else
  	create_pvl(metadataStr, 
		   sGranuleHandle->metadata.coreMetadata.kepMeanAnomaly.name, 
		   sGranuleHandle->metadata.coreMetadata.kepMeanAnomaly.value);
      
      
      if (sGranuleHandle->metadata.coreMetadata.kepRightAscenNode.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.coreMetadata.kepRightAscenNode.name,
		   tempHandle->metadata.coreMetadata.kepRightAscenNode.value);
      else
  	create_pvl(metadataStr, 
		   sGranuleHandle->metadata.coreMetadata.kepRightAscenNode.name, 
		   sGranuleHandle->metadata.coreMetadata.kepRightAscenNode.value);
      
      
      if (sGranuleHandle->metadata.coreMetadata.kepArgOfPerigee.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.coreMetadata.kepArgOfPerigee.name,
		   tempHandle->metadata.coreMetadata.kepArgOfPerigee.value);
      else
  	create_pvl(metadataStr, 
		   sGranuleHandle->metadata.coreMetadata.kepArgOfPerigee.name, 
		   sGranuleHandle->metadata.coreMetadata.kepArgOfPerigee.value);
      
      
      if (sGranuleHandle->metadata.coreMetadata.kepEccentricity.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.coreMetadata.kepEccentricity.name,
		   tempHandle->metadata.coreMetadata.kepEccentricity.value);
      else
  	create_pvl(metadataStr, 
		   sGranuleHandle->metadata.coreMetadata.kepEccentricity.name, 
		   sGranuleHandle->metadata.coreMetadata.kepEccentricity.value);
      
      
      if (sGranuleHandle->metadata.coreMetadata.kepInclination.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.coreMetadata.kepInclination.name,
		   tempHandle->metadata.coreMetadata.kepInclination.value);
      else
  	create_pvl(metadataStr, 
		   sGranuleHandle->metadata.coreMetadata.kepInclination.name, 
		   sGranuleHandle->metadata.coreMetadata.kepInclination.value);
      
      if (sGranuleHandle->metadata.coreMetadata.kepEpochTime.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.coreMetadata.kepEpochTime.name,
		   tempHandle->metadata.coreMetadata.kepEpochTime.value);
      else
  	create_pvl(metadataStr, 
		   sGranuleHandle->metadata.coreMetadata.kepEpochTime.name, 
		   sGranuleHandle->metadata.coreMetadata.kepEpochTime.value);
      
      if (sGranuleHandle->metadata.coreMetadata.kepEpochDate.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.coreMetadata.kepEpochDate.name,
		   tempHandle->metadata.coreMetadata.kepEpochDate.value);
      else
  	create_pvl(metadataStr, 
		   sGranuleHandle->metadata.coreMetadata.kepEpochDate.name, 
		   sGranuleHandle->metadata.coreMetadata.kepEpochDate.value);
      
      if (sGranuleHandle->metadata.coreMetadata.kepEpochMillisec.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.coreMetadata.kepEpochMillisec.name,
		   tempHandle->metadata.coreMetadata.kepEpochMillisec.value);
      else
  	create_pvl(metadataStr, 
		   sGranuleHandle->metadata.coreMetadata.kepEpochMillisec.name, 
		   sGranuleHandle->metadata.coreMetadata.kepEpochMillisec.value);
      
      /*
       * END non-real time
       */
      }

      if (sGranuleHandle->metadata.coreMetadata.westBoundCoord.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.coreMetadata.westBoundCoord.name,
		   tempHandle->metadata.coreMetadata.westBoundCoord.value);
      else
  	create_pvl(metadataStr, 
		   sGranuleHandle->metadata.coreMetadata.westBoundCoord.name, 
		   sGranuleHandle->metadata.coreMetadata.westBoundCoord.value);
      
      if (sGranuleHandle->metadata.coreMetadata.eastBoundCoord.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.coreMetadata.eastBoundCoord.name,
		   tempHandle->metadata.coreMetadata.eastBoundCoord.value);
      else
  	create_pvl(metadataStr, 
		   sGranuleHandle->metadata.coreMetadata.eastBoundCoord.name, 
		   sGranuleHandle->metadata.coreMetadata.eastBoundCoord.value);
      
      if (sGranuleHandle->metadata.coreMetadata.northBoundCoord.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.coreMetadata.northBoundCoord.name,
		   tempHandle->metadata.coreMetadata.northBoundCoord.value);
      else
  	create_pvl(metadataStr, 
		   sGranuleHandle->metadata.coreMetadata.northBoundCoord.name, 
		   sGranuleHandle->metadata.coreMetadata.northBoundCoord.value);
      
      if (sGranuleHandle->metadata.coreMetadata.southBoundCoord.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.coreMetadata.southBoundCoord.name,
		   tempHandle->metadata.coreMetadata.southBoundCoord.value);
      else
  	create_pvl(metadataStr, 
		   sGranuleHandle->metadata.coreMetadata.southBoundCoord.name, 
		   sGranuleHandle->metadata.coreMetadata.southBoundCoord.value);

      if (!sGranuleHandle->realTimeFlag)
      {
      /*
       * BEGIN non-real time
       */

      if (sGranuleHandle->metadata.coreMetadata.centerPointLat.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.coreMetadata.centerPointLat.name,
		   tempHandle->metadata.coreMetadata.centerPointLat.value);
      else
  	create_pvl(metadataStr, 
		   sGranuleHandle->metadata.coreMetadata.centerPointLat.name, 
		   sGranuleHandle->metadata.coreMetadata.centerPointLat.value);
      
      if (sGranuleHandle->metadata.coreMetadata.centerPointLon.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.coreMetadata.centerPointLon.name,
		   tempHandle->metadata.coreMetadata.centerPointLon.value);
      else
  	create_pvl(metadataStr, 
		   sGranuleHandle->metadata.coreMetadata.centerPointLon.name, 
		   sGranuleHandle->metadata.coreMetadata.centerPointLon.value);
      
      if (sGranuleHandle->metadata.coreMetadata.radius.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.coreMetadata.radius.name,
		   tempHandle->metadata.coreMetadata.radius.value);
      else
  	create_pvl(metadataStr, 
		   sGranuleHandle->metadata.coreMetadata.radius.name, 
		   sGranuleHandle->metadata.coreMetadata.radius.value);
      
      if (sGranuleHandle->metadata.coreMetadata.latitudeRes.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.coreMetadata.latitudeRes.name,
		   tempHandle->metadata.coreMetadata.latitudeRes.value);
      else
  	create_pvl(metadataStr, 
		   sGranuleHandle->metadata.coreMetadata.latitudeRes.name, 
		   sGranuleHandle->metadata.coreMetadata.latitudeRes.value);
      
      if (sGranuleHandle->metadata.coreMetadata.longitudeRes.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.coreMetadata.longitudeRes.name,
		   tempHandle->metadata.coreMetadata.longitudeRes.value);
      else
  	create_pvl(metadataStr, 
		   sGranuleHandle->metadata.coreMetadata.longitudeRes.name, 
		   sGranuleHandle->metadata.coreMetadata.longitudeRes.value);
      
      if (sGranuleHandle->metadata.coreMetadata.geoCoordUnits.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.coreMetadata.geoCoordUnits.name,
		   tempHandle->metadata.coreMetadata.geoCoordUnits.value);
      else
  	create_pvl(metadataStr, 
		   sGranuleHandle->metadata.coreMetadata.geoCoordUnits.name, 
		   sGranuleHandle->metadata.coreMetadata.geoCoordUnits.value);
      
      if (sGranuleHandle->metadata.coreMetadata.temporRngType.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.coreMetadata.temporRngType.name,
		   tempHandle->metadata.coreMetadata.temporRngType.value);
      else
  	create_pvl(metadataStr, 
		   sGranuleHandle->metadata.coreMetadata.temporRngType.name, 
		   sGranuleHandle->metadata.coreMetadata.temporRngType.value);
      
      if (sGranuleHandle->metadata.coreMetadata.qaParamName.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.coreMetadata.qaParamName.name,
		   tempHandle->metadata.coreMetadata.qaParamName.value);
      else
  	create_pvl(metadataStr, 
		   sGranuleHandle->metadata.coreMetadata.qaParamName.name, 
		   sGranuleHandle->metadata.coreMetadata.qaParamName.value);
      
      if (sGranuleHandle->metadata.coreMetadata.qaParamValue.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.coreMetadata.qaParamValue.name,
		   tempHandle->metadata.coreMetadata.qaParamValue.value);
      else
  	create_pvl(metadataStr, 
		   sGranuleHandle->metadata.coreMetadata.qaParamValue.name, 
		   sGranuleHandle->metadata.coreMetadata.qaParamValue.value);
      
      if (sGranuleHandle->metadata.coreMetadata.reproStat.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.coreMetadata.reproStat.name,
		   tempHandle->metadata.coreMetadata.reproStat.value);
      else
  	create_pvl(metadataStr, 
		   sGranuleHandle->metadata.coreMetadata.reproStat.name, 
		   sGranuleHandle->metadata.coreMetadata.reproStat.value);
      
      if (sGranuleHandle->metadata.coreMetadata.browseName.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.coreMetadata.browseName.name,
		   tempHandle->metadata.coreMetadata.browseName.value);
      else
  	create_pvl(metadataStr, 
		   sGranuleHandle->metadata.coreMetadata.browseName.name, 
		   sGranuleHandle->metadata.coreMetadata.browseName.value);
      
      if (sGranuleHandle->metadata.coreMetadata.contact.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.coreMetadata.contact.name,
		   tempHandle->metadata.coreMetadata.contact.value);
      else
  	create_pvl(metadataStr, 
		   sGranuleHandle->metadata.coreMetadata.contact.name, 
		   sGranuleHandle->metadata.coreMetadata.contact.value);
      
      if (sGranuleHandle->metadata.coreMetadata.meanMotion.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.coreMetadata.meanMotion.name,
		   tempHandle->metadata.coreMetadata.meanMotion.value);
      else
  	create_pvl(metadataStr, 
		   sGranuleHandle->metadata.coreMetadata.meanMotion.name, 
		   sGranuleHandle->metadata.coreMetadata.meanMotion.value);
      
      /*
       * END non-real time
       */
      }

      if (sGranuleHandle->metadata.coreMetadata.orbitAdjFlag.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.coreMetadata.orbitAdjFlag.name,
		   tempHandle->metadata.coreMetadata.orbitAdjFlag.value);
      else
  	create_pvl(metadataStr, 
		   sGranuleHandle->metadata.coreMetadata.orbitAdjFlag.name, 
		   sGranuleHandle->metadata.coreMetadata.orbitAdjFlag.value);
      
      if (sGranuleHandle->metadata.coreMetadata.attitudeMode.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.coreMetadata.attitudeMode.name,
		   tempHandle->metadata.coreMetadata.attitudeMode.value);
      else
  	create_pvl(metadataStr, 
		   sGranuleHandle->metadata.coreMetadata.attitudeMode.name, 
		   sGranuleHandle->metadata.coreMetadata.attitudeMode.value);
      
      if (!sGranuleHandle->realTimeFlag)
      {
      /*
       * BEGIN non-real time
       */

      if (sGranuleHandle->metadata.coreMetadata.beginSolarBeta.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.coreMetadata.beginSolarBeta.name,
		   tempHandle->metadata.coreMetadata.beginSolarBeta.value);
      else
  	create_pvl(metadataStr, 
		   sGranuleHandle->metadata.coreMetadata.beginSolarBeta.name, 
		   sGranuleHandle->metadata.coreMetadata.beginSolarBeta.value);
      
      if (sGranuleHandle->metadata.coreMetadata.endSolarBeta.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.coreMetadata.endSolarBeta.name,
		   tempHandle->metadata.coreMetadata.endSolarBeta.value);
      else
  	create_pvl(metadataStr, 
		   sGranuleHandle->metadata.coreMetadata.endSolarBeta.name, 
		   sGranuleHandle->metadata.coreMetadata.endSolarBeta.value);
      
      if (sGranuleHandle->metadata.coreMetadata.sensorAlgn.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.coreMetadata.sensorAlgn.name,
		   tempHandle->metadata.coreMetadata.sensorAlgn.value);
      else
  	create_pvl(metadataStr, 
		   sGranuleHandle->metadata.coreMetadata.sensorAlgn.name, 
		   sGranuleHandle->metadata.coreMetadata.sensorAlgn.value);
      
      if (sGranuleHandle->metadata.coreMetadata.sensorAlgnChanOff.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.coreMetadata.sensorAlgnChanOff.name,
		   tempHandle->metadata.coreMetadata.sensorAlgnChanOff.value);
      else
  	create_pvl(metadataStr, 
		   sGranuleHandle->metadata.coreMetadata.sensorAlgnChanOff.name, 
		   sGranuleHandle->metadata.coreMetadata.sensorAlgnChanOff.value);
      
      if (sGranuleHandle->metadata.coreMetadata.scanPathModel.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.coreMetadata.scanPathModel.name,
		   tempHandle->metadata.coreMetadata.scanPathModel.value);
      else
  	create_pvl(metadataStr, 
		   sGranuleHandle->metadata.coreMetadata.scanPathModel.name, 
		   sGranuleHandle->metadata.coreMetadata.scanPathModel.value);
      
      if (sGranuleHandle->metadata.coreMetadata.scanPathParam.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.coreMetadata.scanPathParam.name,
		   tempHandle->metadata.coreMetadata.scanPathParam.value);
      else
  	create_pvl(metadataStr, 
		   sGranuleHandle->metadata.coreMetadata.scanPathParam.name, 
		   sGranuleHandle->metadata.coreMetadata.scanPathParam.value);
      
      if (sGranuleHandle->metadata.coreMetadata.ephemFilename.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.coreMetadata.ephemFilename.name,
		   tempHandle->metadata.coreMetadata.ephemFilename.value);
      else
  	create_pvl(metadataStr, 
		   sGranuleHandle->metadata.coreMetadata.ephemFilename.name, 
		   sGranuleHandle->metadata.coreMetadata.ephemFilename.value);
      
      /*
       * END non-real time
       */
      }

      /* 
       ** Write the attribute string containing Core metadata to the HDF File 
       */
      if (writepvl_to_hdffile(sGranuleHandle,"CoreMetadata.0",metadataStr) != TK_SUCCESS)
	TKreportWarning(W_TK_WRITEPVL);
      
      
      
      /*
       * Write Product Specific metadata to attribute string
       */
      metadataStr[0] = '\0';
      
      if (!sGranuleHandle->realTimeFlag)
      {
      /*
       * BEGIN non-real time
       */

      if (sGranuleHandle->metadata.psMetadata.dataGap.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.psMetadata.dataGap.name,
		   tempHandle->metadata.psMetadata.dataGap.value);
      else
  	create_pvl(metadataStr, 
		   sGranuleHandle->metadata.psMetadata.dataGap.name, 
		   sGranuleHandle->metadata.psMetadata.dataGap.value);

      if (sGranuleHandle->metadata.psMetadata.numDataGap.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.psMetadata.numDataGap.name,
		   tempHandle->metadata.psMetadata.numDataGap.value);
      else
  	create_pvl(metadataStr, 
		   sGranuleHandle->metadata.psMetadata.numDataGap.name, 
		   sGranuleHandle->metadata.psMetadata.numDataGap.value);
      /*
       * END non-real time
       */
      }

      
/*special case below, realtime alg version needs to ripple up., but we dont 
want to risk breaking the non-real time case  - we want "temp" 
not "sGranule" p.kolbe jan 2004 */


      if (sGranuleHandle->realTimeFlag) /*start special*/
      {
        create_pvl(metadataStr,
		   tempHandle->metadata.psMetadata.algorithmVersion.name,
		   tempHandle->metadata.psMetadata.algorithmVersion.value);
      }
      else
      {

      if (sGranuleHandle->metadata.psMetadata.algorithmVersion.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.psMetadata.algorithmVersion.name,
		   tempHandle->metadata.psMetadata.algorithmVersion.value);
      else
  	create_pvl(metadataStr, 
		   sGranuleHandle->metadata.psMetadata.algorithmVersion.name, 
		   sGranuleHandle->metadata.psMetadata.algorithmVersion.value);
      } /*end special*/
      


      if (!sGranuleHandle->realTimeFlag)
      {
      /*
       * BEGIN non-real time
       */
      if (sGranuleHandle->metadata.psMetadata.productVersion.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.psMetadata.productVersion.name,
		   tempHandle->metadata.psMetadata.productVersion.value);
      else
  	create_pvl(metadataStr, 
		   sGranuleHandle->metadata.psMetadata.productVersion.name, 
		   sGranuleHandle->metadata.psMetadata.productVersion.value);
      /*
       * END non-real time
       */
      }

      if (sGranuleHandle->metadata.psMetadata.toolkitVersion.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.psMetadata.toolkitVersion.name,
		   tempHandle->metadata.psMetadata.toolkitVersion.value);
      else
  	create_pvl(metadataStr, 
		   sGranuleHandle->metadata.psMetadata.toolkitVersion.name, 
		   sGranuleHandle->metadata.psMetadata.toolkitVersion.value);
      
      if (!sGranuleHandle->realTimeFlag)
      {
      /*
       * BEGIN non-real time
       */
      if (sGranuleHandle->metadata.psMetadata.calCoeffVersion.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.psMetadata.calCoeffVersion.name,
		   tempHandle->metadata.psMetadata.calCoeffVersion.value);
      else
  	create_pvl(metadataStr, 
		   sGranuleHandle->metadata.psMetadata.calCoeffVersion.name, 
		   sGranuleHandle->metadata.psMetadata.calCoeffVersion.value);
      
      if (sGranuleHandle->metadata.psMetadata.missingData.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.psMetadata.missingData.name,
		   tempHandle->metadata.psMetadata.missingData.value);
      else
  	create_pvl(metadataStr, 
		   sGranuleHandle->metadata.psMetadata.missingData.name, 
		   sGranuleHandle->metadata.psMetadata.missingData.value);
      
      if (sGranuleHandle->metadata.psMetadata.percentBadMissPix.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.psMetadata.percentBadMissPix.name,
		   tempHandle->metadata.psMetadata.percentBadMissPix.value);
      else
  	create_pvl(metadataStr, 
		   sGranuleHandle->metadata.psMetadata.percentBadMissPix.name, 
		   sGranuleHandle->metadata.psMetadata.percentBadMissPix.value);
      
      if (sGranuleHandle->metadata.psMetadata.maxValidChannel.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.psMetadata.maxValidChannel.name,
		   tempHandle->metadata.psMetadata.maxValidChannel.value);
      else
  	create_pvl(metadataStr, 
		   sGranuleHandle->metadata.psMetadata.maxValidChannel.name, 
		   sGranuleHandle->metadata.psMetadata.maxValidChannel.value);
      
      if (sGranuleHandle->metadata.psMetadata.minValidChannel.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.psMetadata.minValidChannel.name,
		   tempHandle->metadata.psMetadata.minValidChannel.value);
      else
  	create_pvl(metadataStr, 
		   sGranuleHandle->metadata.psMetadata.minValidChannel.name, 
		   sGranuleHandle->metadata.psMetadata.minValidChannel.value);
      
      if (sGranuleHandle->metadata.psMetadata.minMaxUnits.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.psMetadata.minMaxUnits.name,
		   tempHandle->metadata.psMetadata.minMaxUnits.value);
      else
  	create_pvl(metadataStr, 
		   sGranuleHandle->metadata.psMetadata.minMaxUnits.name, 
		   sGranuleHandle->metadata.psMetadata.minMaxUnits.value);
      /*
       * END non-real time
       */
      }
      
      if (sGranuleHandle->metadata.psMetadata.orbitSize.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.psMetadata.orbitSize.name,
		   tempHandle->metadata.psMetadata.orbitSize.value);
      else
  	create_pvl(metadataStr, 
		   sGranuleHandle->metadata.psMetadata.orbitSize.name, 
		   sGranuleHandle->metadata.psMetadata.orbitSize.value);
      
      if (!sGranuleHandle->realTimeFlag)
      {
      /*
       * BEGIN non-real time
       */

      if (sGranuleHandle->metadata.psMetadata.radarWavelength.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.psMetadata.radarWavelength.name,
		   tempHandle->metadata.psMetadata.radarWavelength.value);
      else
  	create_pvl(metadataStr, 
		   sGranuleHandle->metadata.psMetadata.radarWavelength.name, 
		   sGranuleHandle->metadata.psMetadata.radarWavelength.value);
      
      if (sGranuleHandle->metadata.psMetadata.minRefThreshold.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.psMetadata.minRefThreshold.name,
		   tempHandle->metadata.psMetadata.minRefThreshold.value);
      else
  	create_pvl(metadataStr, 
		   sGranuleHandle->metadata.psMetadata.minRefThreshold.name, 
		   sGranuleHandle->metadata.psMetadata.minRefThreshold.value);
      /*
       * END non-real time
       */
      }

      if (sGranuleHandle->metadata.psMetadata.algorithmID.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.psMetadata.algorithmID.name,
		   tempHandle->metadata.psMetadata.algorithmID.value);
      else
  	create_pvl(metadataStr, 
		   sGranuleHandle->metadata.psMetadata.algorithmID.name, 
		   sGranuleHandle->metadata.psMetadata.algorithmID.value);
      

      if (!sGranuleHandle->realTimeFlag)
      {
      /*
       * BEGIN non-real time
       */
      if (sGranuleHandle->metadata.psMetadata.dataAccuracy.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.psMetadata.dataAccuracy.name,
		   tempHandle->metadata.psMetadata.dataAccuracy.value);
      else
  	create_pvl(metadataStr, 
		   sGranuleHandle->metadata.psMetadata.dataAccuracy.name, 
		   sGranuleHandle->metadata.psMetadata.dataAccuracy.value);
      
      if (sGranuleHandle->metadata.psMetadata.inputID.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.psMetadata.inputID.name,
		   tempHandle->metadata.psMetadata.inputID.value);
      else
  	create_pvl(metadataStr, 
		   sGranuleHandle->metadata.psMetadata.inputID.name, 
		   sGranuleHandle->metadata.psMetadata.inputID.value);
      
      if (sGranuleHandle->metadata.psMetadata.dateGenInputFile.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.psMetadata.dateGenInputFile.name,
		   tempHandle->metadata.psMetadata.dateGenInputFile.value);
      else
  	create_pvl(metadataStr, 
		   sGranuleHandle->metadata.psMetadata.dateGenInputFile.name, 
		   sGranuleHandle->metadata.psMetadata.dateGenInputFile.value);
      
      if (sGranuleHandle->metadata.psMetadata.dataCenterSrc.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.psMetadata.dataCenterSrc.name,
		   tempHandle->metadata.psMetadata.dataCenterSrc.value);
      else
  	create_pvl(metadataStr, 
		   sGranuleHandle->metadata.psMetadata.dataCenterSrc.name, 
		   sGranuleHandle->metadata.psMetadata.dataCenterSrc.value);
      
      /* Use the generation date obtained from the system call to time() */
      create_pvl(metadataStr, 
		 sGranuleHandle->metadata.psMetadata.generationDate.name, 
		 sGranuleHandle->genDate);
      
      if (sGranuleHandle->metadata.psMetadata.dayNight.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.psMetadata.dayNight.name,
		   tempHandle->metadata.psMetadata.dayNight.value);
      else
  	create_pvl(metadataStr, 
		   sGranuleHandle->metadata.psMetadata.dayNight.name, 
		   sGranuleHandle->metadata.psMetadata.dayNight.value);

      if (sGranuleHandle->metadata.psMetadata.solarGains.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.psMetadata.solarGains.name,
		   tempHandle->metadata.psMetadata.solarGains.value);
      else
  	create_pvl(metadataStr, 
		   sGranuleHandle->metadata.psMetadata.solarGains.name, 
		   sGranuleHandle->metadata.psMetadata.solarGains.value);
      
      if (sGranuleHandle->metadata.psMetadata.tmiRolloverCoef.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
                   tempHandle->metadata.psMetadata.tmiRolloverCoef.name,
                   tempHandle->metadata.psMetadata.tmiRolloverCoef.value);
      else
        create_pvl(metadataStr,
                   sGranuleHandle->metadata.psMetadata.tmiRolloverCoef.name,
                   sGranuleHandle->metadata.psMetadata.tmiRolloverCoef.value);

      if (sGranuleHandle->metadata.psMetadata.firstScanUTCdate.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.psMetadata.firstScanUTCdate.name,
		   tempHandle->metadata.psMetadata.firstScanUTCdate.value);
      else
  	create_pvl(metadataStr, 
		   sGranuleHandle->metadata.psMetadata.firstScanUTCdate.name, 
		   sGranuleHandle->metadata.psMetadata.firstScanUTCdate.value);
      
      if (sGranuleHandle->metadata.psMetadata.firstScanUTCtime.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.psMetadata.firstScanUTCtime.name,
		   tempHandle->metadata.psMetadata.firstScanUTCtime.value);
      else
  	create_pvl(metadataStr, 
		   sGranuleHandle->metadata.psMetadata.firstScanUTCtime.name, 
		   sGranuleHandle->metadata.psMetadata.firstScanUTCtime.value);
      
      if (sGranuleHandle->metadata.psMetadata.firstScanUTCmsec.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.psMetadata.firstScanUTCmsec.name,
		   tempHandle->metadata.psMetadata.firstScanUTCmsec.value);
      else
  	create_pvl(metadataStr, 
		   sGranuleHandle->metadata.psMetadata.firstScanUTCmsec.name, 
		   sGranuleHandle->metadata.psMetadata.firstScanUTCmsec.value);
      
      if (sGranuleHandle->metadata.psMetadata.firstScanSCsec.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.psMetadata.firstScanSCsec.name,
		   tempHandle->metadata.psMetadata.firstScanSCsec.value);
      else
  	create_pvl(metadataStr, 
		   sGranuleHandle->metadata.psMetadata.firstScanSCsec.name, 
		   sGranuleHandle->metadata.psMetadata.firstScanSCsec.value);
      
      if (sGranuleHandle->metadata.psMetadata.firstScanSCsubsec.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.psMetadata.firstScanSCsubsec.name,
		   tempHandle->metadata.psMetadata.firstScanSCsubsec.value);
      else
  	create_pvl(metadataStr, 
		   sGranuleHandle->metadata.psMetadata.firstScanSCsubsec.name, 
		   sGranuleHandle->metadata.psMetadata.firstScanSCsubsec.value);
      
      if (sGranuleHandle->metadata.psMetadata.lastScanUTCdate.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.psMetadata.lastScanUTCdate.name,
		   tempHandle->metadata.psMetadata.lastScanUTCdate.value);
      else
  	create_pvl(metadataStr, 
		   sGranuleHandle->metadata.psMetadata.lastScanUTCdate.name, 
		   sGranuleHandle->metadata.psMetadata.lastScanUTCdate.value);
      
      if (sGranuleHandle->metadata.psMetadata.lastScanUTCtime.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.psMetadata.lastScanUTCtime.name,
		   tempHandle->metadata.psMetadata.lastScanUTCtime.value);
      else
  	create_pvl(metadataStr, 
		   sGranuleHandle->metadata.psMetadata.lastScanUTCtime.name, 
		   sGranuleHandle->metadata.psMetadata.lastScanUTCtime.value);
      
      if (sGranuleHandle->metadata.psMetadata.lastScanUTCmsec.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.psMetadata.lastScanUTCmsec.name,
		   tempHandle->metadata.psMetadata.lastScanUTCmsec.value);
      else
  	create_pvl(metadataStr, 
		   sGranuleHandle->metadata.psMetadata.lastScanUTCmsec.name, 
		   sGranuleHandle->metadata.psMetadata.lastScanUTCmsec.value);
      
      if (sGranuleHandle->metadata.psMetadata.lastScanSCsec.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.psMetadata.lastScanSCsec.name,
		   tempHandle->metadata.psMetadata.lastScanSCsec.value);
      else
  	create_pvl(metadataStr, 
		   sGranuleHandle->metadata.psMetadata.lastScanSCsec.name, 
		   sGranuleHandle->metadata.psMetadata.lastScanSCsec.value);
      
      if (sGranuleHandle->metadata.psMetadata.lastScanSCsubsec.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.psMetadata.lastScanSCsubsec.name,
		   tempHandle->metadata.psMetadata.lastScanSCsubsec.value);
      else
  	create_pvl(metadataStr, 
		   sGranuleHandle->metadata.psMetadata.lastScanSCsubsec.name, 
		   sGranuleHandle->metadata.psMetadata.lastScanSCsubsec.value);
      
      if (sGranuleHandle->metadata.psMetadata.utcfSec.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.psMetadata.utcfSec.name,
		   tempHandle->metadata.psMetadata.utcfSec.value);
      else
  	create_pvl(metadataStr, 
		   sGranuleHandle->metadata.psMetadata.utcfSec.name, 
		   sGranuleHandle->metadata.psMetadata.utcfSec.value);
      
      if (sGranuleHandle->metadata.psMetadata.utcfSubsec.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.psMetadata.utcfSubsec.name,
		   tempHandle->metadata.psMetadata.utcfSubsec.value);
      else
  	create_pvl(metadataStr, 
		   sGranuleHandle->metadata.psMetadata.utcfSubsec.name, 
		   sGranuleHandle->metadata.psMetadata.utcfSubsec.value);
      
      if (sGranuleHandle->metadata.psMetadata.utcfFlag.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.psMetadata.utcfFlag.name,
		   tempHandle->metadata.psMetadata.utcfFlag.value);
      else
  	create_pvl(metadataStr, 
		   sGranuleHandle->metadata.psMetadata.utcfFlag.name, 
		   sGranuleHandle->metadata.psMetadata.utcfFlag.value);
      
      if (sGranuleHandle->metadata.psMetadata.leapSecsFlag.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.psMetadata.leapSecsFlag.name,
		   tempHandle->metadata.psMetadata.leapSecsFlag.value);
      else
  	create_pvl(metadataStr, 
		   sGranuleHandle->metadata.psMetadata.leapSecsFlag.name, 
		   sGranuleHandle->metadata.psMetadata.leapSecsFlag.value);
      
      if (sGranuleHandle->metadata.psMetadata.radarName.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.psMetadata.radarName.name,
		   tempHandle->metadata.psMetadata.radarName.value);
      else
  	create_pvl(metadataStr, 
		   sGranuleHandle->metadata.psMetadata.radarName.name, 
		   sGranuleHandle->metadata.psMetadata.radarName.value);
      
      if (sGranuleHandle->metadata.psMetadata.radarCity.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.psMetadata.radarCity.name,
		   tempHandle->metadata.psMetadata.radarCity.value);
      else
  	create_pvl(metadataStr, 
		   sGranuleHandle->metadata.psMetadata.radarCity.name, 
		   sGranuleHandle->metadata.psMetadata.radarCity.value);
      
      if (sGranuleHandle->metadata.psMetadata.radarState.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.psMetadata.radarState.name,
		   tempHandle->metadata.psMetadata.radarState.value);
      else
  	create_pvl(metadataStr, 
		   sGranuleHandle->metadata.psMetadata.radarState.name, 
		   sGranuleHandle->metadata.psMetadata.radarState.value);
      
      if (sGranuleHandle->metadata.psMetadata.radarCountry.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.psMetadata.radarCountry.name,
		   tempHandle->metadata.psMetadata.radarCountry.value);
      else
  	create_pvl(metadataStr, 
		   sGranuleHandle->metadata.psMetadata.radarCountry.name, 
		   sGranuleHandle->metadata.psMetadata.radarCountry.value);
      
      if (sGranuleHandle->metadata.psMetadata.numVOS.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.psMetadata.numVOS.name,
		   tempHandle->metadata.psMetadata.numVOS.value);
      else
  	create_pvl(metadataStr, 
		   sGranuleHandle->metadata.psMetadata.numVOS.name, 
		   sGranuleHandle->metadata.psMetadata.numVOS.value);
      
      if (sGranuleHandle->metadata.psMetadata.radarOriginLat.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.psMetadata.radarOriginLat.name,
		   tempHandle->metadata.psMetadata.radarOriginLat.value);
      else
  	create_pvl(metadataStr, 
		   sGranuleHandle->metadata.psMetadata.radarOriginLat.name, 
		   sGranuleHandle->metadata.psMetadata.radarOriginLat.value);
      
      if (sGranuleHandle->metadata.psMetadata.radarOriginLon.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.psMetadata.radarOriginLon.name,
		   tempHandle->metadata.psMetadata.radarOriginLon.value);
      else
  	create_pvl(metadataStr, 
		   sGranuleHandle->metadata.psMetadata.radarOriginLon.name, 
		   sGranuleHandle->metadata.psMetadata.radarOriginLon.value);
      
      if (sGranuleHandle->metadata.psMetadata.radarOriginAlt.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.psMetadata.radarOriginAlt.name,
		   tempHandle->metadata.psMetadata.radarOriginAlt.value);
      else
  	create_pvl(metadataStr, 
		   sGranuleHandle->metadata.psMetadata.radarOriginAlt.name, 
		   sGranuleHandle->metadata.psMetadata.radarOriginAlt.value);
      
      if (sGranuleHandle->metadata.psMetadata.radarSpacingX.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.psMetadata.radarSpacingX.name,
		   tempHandle->metadata.psMetadata.radarSpacingX.value);
      else
  	create_pvl(metadataStr, 
		   sGranuleHandle->metadata.psMetadata.radarSpacingX.name, 
		   sGranuleHandle->metadata.psMetadata.radarSpacingX.value);
      
      if (sGranuleHandle->metadata.psMetadata.radarSpacingY.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.psMetadata.radarSpacingY.name,
		   tempHandle->metadata.psMetadata.radarSpacingY.value);
      else
  	create_pvl(metadataStr, 
		   sGranuleHandle->metadata.psMetadata.radarSpacingY.name, 
		   sGranuleHandle->metadata.psMetadata.radarSpacingY.value);
      
      if (sGranuleHandle->metadata.psMetadata.radarSpacingZ.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.psMetadata.radarSpacingZ.name,
		   tempHandle->metadata.psMetadata.radarSpacingZ.value);
      else
  	create_pvl(metadataStr, 
		   sGranuleHandle->metadata.psMetadata.radarSpacingZ.name, 
		   sGranuleHandle->metadata.psMetadata.radarSpacingZ.value);
      
      if (sGranuleHandle->metadata.psMetadata.radarGridSizeX.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.psMetadata.radarGridSizeX.name,
		   tempHandle->metadata.psMetadata.radarGridSizeX.value);
      else
  	create_pvl(metadataStr, 
		   sGranuleHandle->metadata.psMetadata.radarGridSizeX.name, 
		   sGranuleHandle->metadata.psMetadata.radarGridSizeX.value);
      
      if (sGranuleHandle->metadata.psMetadata.radarGridSizeY.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.psMetadata.radarGridSizeY.name,
		   tempHandle->metadata.psMetadata.radarGridSizeY.value);
      else
  	create_pvl(metadataStr, 
		   sGranuleHandle->metadata.psMetadata.radarGridSizeY.name, 
		   sGranuleHandle->metadata.psMetadata.radarGridSizeY.value);
      
      if (sGranuleHandle->metadata.psMetadata.radarGridSizeZ.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.psMetadata.radarGridSizeZ.name,
		   tempHandle->metadata.psMetadata.radarGridSizeZ.value);
      else
  	create_pvl(metadataStr, 
		   sGranuleHandle->metadata.psMetadata.radarGridSizeZ.name, 
		   sGranuleHandle->metadata.psMetadata.radarGridSizeZ.value);
      
      if (sGranuleHandle->metadata.psMetadata.dzcal.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.psMetadata.dzcal.name,
		   tempHandle->metadata.psMetadata.dzcal.value);
      else
        create_pvl(metadataStr,
		   sGranuleHandle->metadata.psMetadata.dzcal.name,
		   sGranuleHandle->metadata.psMetadata.dzcal.value);
      
      if (sGranuleHandle->metadata.psMetadata.gvL1Cscale.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.psMetadata.gvL1Cscale.name,
		   tempHandle->metadata.psMetadata.gvL1Cscale.value);
      else
        create_pvl(metadataStr,
		   sGranuleHandle->metadata.psMetadata.gvL1Cscale.name,
		   sGranuleHandle->metadata.psMetadata.gvL1Cscale.value);

      if (sGranuleHandle->metadata.psMetadata.alpha.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.psMetadata.alpha.name,
		   tempHandle->metadata.psMetadata.alpha.value);
      else
        create_pvl(metadataStr,
		   sGranuleHandle->metadata.psMetadata.alpha.name,
		   sGranuleHandle->metadata.psMetadata.alpha.value);
      
      if (sGranuleHandle->metadata.psMetadata.runtimeOption.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.psMetadata.runtimeOption.name,
		   tempHandle->metadata.psMetadata.runtimeOption.value);
      else
        create_pvl(metadataStr,
		   sGranuleHandle->metadata.psMetadata.runtimeOption.name,
		   sGranuleHandle->metadata.psMetadata.runtimeOption.value);
      
      if (sGranuleHandle->metadata.psMetadata.anomalyFlag.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.psMetadata.anomalyFlag.name,
		   tempHandle->metadata.psMetadata.anomalyFlag.value);
      else
        create_pvl(metadataStr,
		   sGranuleHandle->metadata.psMetadata.anomalyFlag.name,
		   sGranuleHandle->metadata.psMetadata.anomalyFlag.value);
      
      if (sGranuleHandle->metadata.psMetadata.softwareVersion.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.psMetadata.softwareVersion.name,
		   tempHandle->metadata.psMetadata.softwareVersion.value);
      else
        create_pvl(metadataStr,
		   sGranuleHandle->metadata.psMetadata.softwareVersion.name,
		   sGranuleHandle->metadata.psMetadata.softwareVersion.value);
      
      if (sGranuleHandle->metadata.psMetadata.databaseVersion.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.psMetadata.databaseVersion.name,
		   tempHandle->metadata.psMetadata.databaseVersion.value);
      else
        create_pvl(metadataStr,
		   sGranuleHandle->metadata.psMetadata.databaseVersion.name,
		   sGranuleHandle->metadata.psMetadata.databaseVersion.value);
      
      if (sGranuleHandle->metadata.psMetadata.totalQualityCode.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.psMetadata.totalQualityCode.name,
		   tempHandle->metadata.psMetadata.totalQualityCode.value);
      else
        create_pvl(metadataStr,
		   sGranuleHandle->metadata.psMetadata.totalQualityCode.name,
		   sGranuleHandle->metadata.psMetadata.totalQualityCode.value);
      
      if (sGranuleHandle->metadata.psMetadata.lonOnEquator.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.psMetadata.lonOnEquator.name,
		   tempHandle->metadata.psMetadata.lonOnEquator.value);
      else
        create_pvl(metadataStr,
		   sGranuleHandle->metadata.psMetadata.lonOnEquator.name,
		   sGranuleHandle->metadata.psMetadata.lonOnEquator.value);
      
      if (sGranuleHandle->metadata.psMetadata.UTCDateOnEquator.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.psMetadata.UTCDateOnEquator.name,
		   tempHandle->metadata.psMetadata.UTCDateOnEquator.value);
      else
        create_pvl(metadataStr,
		   sGranuleHandle->metadata.psMetadata.UTCDateOnEquator.name,
		   sGranuleHandle->metadata.psMetadata.UTCDateOnEquator.value);
      
      if (sGranuleHandle->metadata.psMetadata.UTCTimeOnEquator.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.psMetadata.UTCTimeOnEquator.name,
		   tempHandle->metadata.psMetadata.UTCTimeOnEquator.value);
      else
        create_pvl(metadataStr,
		   sGranuleHandle->metadata.psMetadata.UTCTimeOnEquator.name,
		   sGranuleHandle->metadata.psMetadata.UTCTimeOnEquator.value);
      
      if (sGranuleHandle->metadata.psMetadata.millisecOnEquator.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.psMetadata.millisecOnEquator.name,
		   tempHandle->metadata.psMetadata.millisecOnEquator.value);
      else
        create_pvl(metadataStr,
		   sGranuleHandle->metadata.psMetadata.millisecOnEquator.name,
		   sGranuleHandle->metadata.psMetadata.millisecOnEquator.value);
      
      if (sGranuleHandle->metadata.psMetadata.centerScanUTCDate.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.psMetadata.centerScanUTCDate.name,
		   tempHandle->metadata.psMetadata.centerScanUTCDate.value);
      else
        create_pvl(metadataStr,
		   sGranuleHandle->metadata.psMetadata.centerScanUTCDate.name,
		   sGranuleHandle->metadata.psMetadata.centerScanUTCDate.value);
      
      if (sGranuleHandle->metadata.psMetadata.centerScanUTCTime.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.psMetadata.centerScanUTCTime.name,
		   tempHandle->metadata.psMetadata.centerScanUTCTime.value);
      else
        create_pvl(metadataStr,
		   sGranuleHandle->metadata.psMetadata.centerScanUTCTime.name,
		   sGranuleHandle->metadata.psMetadata.centerScanUTCTime.value);
      
      if (sGranuleHandle->metadata.psMetadata.centerScanUTCMillisec.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.psMetadata.centerScanUTCMillisec.name,
		   tempHandle->metadata.psMetadata.centerScanUTCMillisec.value);
      else
        create_pvl(metadataStr,
		   sGranuleHandle->metadata.psMetadata.centerScanUTCMillisec.name,
		   sGranuleHandle->metadata.psMetadata.centerScanUTCMillisec.value);
      
      if (sGranuleHandle->metadata.psMetadata.firstScanLat.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.psMetadata.firstScanLat.name,
		   tempHandle->metadata.psMetadata.firstScanLat.value);
      else
        create_pvl(metadataStr,
		   sGranuleHandle->metadata.psMetadata.firstScanLat.name,
		   sGranuleHandle->metadata.psMetadata.firstScanLat.value);
      
      if (sGranuleHandle->metadata.psMetadata.firstScanLon.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.psMetadata.firstScanLon.name,
		   tempHandle->metadata.psMetadata.firstScanLon.value);
      else
        create_pvl(metadataStr,
		   sGranuleHandle->metadata.psMetadata.firstScanLon.name,
		   sGranuleHandle->metadata.psMetadata.firstScanLon.value);
      
      if (sGranuleHandle->metadata.psMetadata.lastScanLat.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.psMetadata.lastScanLat.name,
		   tempHandle->metadata.psMetadata.lastScanLat.value);
      else
        create_pvl(metadataStr,
		   sGranuleHandle->metadata.psMetadata.lastScanLat.name,
		   sGranuleHandle->metadata.psMetadata.lastScanLat.value);
      
      if (sGranuleHandle->metadata.psMetadata.lastScanLon.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.psMetadata.lastScanLon.name,
		   tempHandle->metadata.psMetadata.lastScanLon.value);
      else
        create_pvl(metadataStr,
		   sGranuleHandle->metadata.psMetadata.lastScanLon.name,
		   sGranuleHandle->metadata.psMetadata.lastScanLon.value);
      
      if (sGranuleHandle->metadata.psMetadata.numOfRainScans.booleanFlag != TK_TRUE)
        create_pvl(metadataStr,
		   tempHandle->metadata.psMetadata.numOfRainScans.name,
		   tempHandle->metadata.psMetadata.numOfRainScans.value);
      else
        create_pvl(metadataStr,
		   sGranuleHandle->metadata.psMetadata.numOfRainScans.name,
		   sGranuleHandle->metadata.psMetadata.numOfRainScans.value);
      /*
       * END non-real time
       */
      }
      
      /* 
       ** Write the PS metadata attribute to the HDF file 
       */
      if( writepvl_to_hdffile(sGranuleHandle, "ArchiveMetadata.0", metadataStr) != TK_SUCCESS)
	TKreportWarning(W_TK_WRITEPVL);
      
      /* free memory */
      free(metadataStr);
    }

  /*
   *  Write the start/stop times to the TSDIS database (only in the TSDIS
   *  environment).
   */

#ifndef TSU_ENV
  startOfHDF = strstr ( sGranuleHandle->gIDtokens.granule_ID, "HDF" );
  if ( ( strstr ( sGranuleHandle->gIDtokens.granuleName, "_QL" ) == NULL ) &&
      ( strstr ( sGranuleHandle->gIDtokens.granuleName, "_CSI" ) == NULL ) &&
      ( ( startOfHDF != NULL ) && ( !strcmp ( startOfHDF, "HDF" ) ) ) )
    status = updateStartStopTimesinGranuleMetadata ( sGranuleHandle, tempHandle );
#endif

  metaFree(sGranuleHandle);
  return (status);
}

/*****************************************************************************
*
*  FUNCTION STATEMENT:
*	metaFree(IO_HANDLE *sGranuleHandle)
*
*  DESCRIPTION:
*	This routine deallocates the memory that was allocated for the
*       individual metadata elements
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       sGranuleHandle          A structure containing information about the file
*                               from which data can be obtained.  It also
*				contains metadata information. 
*
*  OUTPUT PARAMETERS:		
*       Parameter Name          Description
*       --------------          -----------
*       sGranuleHandle          Now the memory allocated for the metadata
*				is deallocated.
*
*  CALLED BY:                   finalizeMetadata()
*
*  RETURN VALUES:               None.
*
*  ROUTINES CALLED:
*
******************************************************************************/
void metaFree(IO_HANDLE *sGranuleHandle)
{

  /*
   * Free the core metadata elements
   */
  if ( sGranuleHandle->metadata.coreMetadata.orbitNumber.name != NULL )
    free( sGranuleHandle->metadata.coreMetadata.orbitNumber.name);
  if ( sGranuleHandle->metadata.coreMetadata.orbitNumber.value != NULL )
    free( sGranuleHandle->metadata.coreMetadata.orbitNumber.value);

  if ( sGranuleHandle->metadata.coreMetadata.beginDate.name != NULL )
    free( sGranuleHandle->metadata.coreMetadata.beginDate.name);
  if ( sGranuleHandle->metadata.coreMetadata.beginDate.value != NULL )
    free( sGranuleHandle->metadata.coreMetadata.beginDate.value);

  if ( sGranuleHandle->metadata.coreMetadata.beginTime.name != NULL )
    free( sGranuleHandle->metadata.coreMetadata.beginTime.name);
  if ( sGranuleHandle->metadata.coreMetadata.beginTime.value != NULL )
    free( sGranuleHandle->metadata.coreMetadata.beginTime.value);

  if ( sGranuleHandle->metadata.coreMetadata.endDate.name != NULL )
    free( sGranuleHandle->metadata.coreMetadata.endDate.name);
  if ( sGranuleHandle->metadata.coreMetadata.endDate.value != NULL )
    free( sGranuleHandle->metadata.coreMetadata.endDate.value);

  if ( sGranuleHandle->metadata.coreMetadata.endTime.name != NULL )
    free( sGranuleHandle->metadata.coreMetadata.endTime.name);
  if ( sGranuleHandle->metadata.coreMetadata.endTime.value != NULL )
    free( sGranuleHandle->metadata.coreMetadata.endTime.value);

  if ( sGranuleHandle->metadata.coreMetadata.granuleID.name != NULL )
    free( sGranuleHandle->metadata.coreMetadata.granuleID.name);
  if ( sGranuleHandle->metadata.coreMetadata.granuleID.value != NULL )
    free( sGranuleHandle->metadata.coreMetadata.granuleID.value);

  if ( sGranuleHandle->metadata.coreMetadata.dataID.name != NULL )
    free( sGranuleHandle->metadata.coreMetadata.dataID.name);
  if ( sGranuleHandle->metadata.coreMetadata.dataID.value != NULL )
    free( sGranuleHandle->metadata.coreMetadata.dataID.value);

  if ( sGranuleHandle->metadata.coreMetadata.fileSize.name != NULL )
    free( sGranuleHandle->metadata.coreMetadata.fileSize.name);
  if ( sGranuleHandle->metadata.coreMetadata.fileSize.value != NULL )
    free( sGranuleHandle->metadata.coreMetadata.fileSize.value);

  if ( sGranuleHandle->metadata.coreMetadata.lonMaxLat.name != NULL )
    free( sGranuleHandle->metadata.coreMetadata.lonMaxLat.name);
  if ( sGranuleHandle->metadata.coreMetadata.lonMaxLat.value != NULL ) 
    free( sGranuleHandle->metadata.coreMetadata.lonMaxLat.value);

  if ( sGranuleHandle->metadata.coreMetadata.spatCovType.name != NULL )
    free( sGranuleHandle->metadata.coreMetadata.spatCovType.name);
  if ( sGranuleHandle->metadata.coreMetadata.spatCovType.value != NULL )
    free( sGranuleHandle->metadata.coreMetadata.spatCovType.value);

  if ( sGranuleHandle->metadata.coreMetadata.ellipsoidName.name != NULL )
    free( sGranuleHandle->metadata.coreMetadata.ellipsoidName.name);
  if ( sGranuleHandle->metadata.coreMetadata.ellipsoidName.value != NULL )
    free( sGranuleHandle->metadata.coreMetadata.ellipsoidName.value);

  if ( sGranuleHandle->metadata.coreMetadata.earthEqRad.name != NULL ) 
    free( sGranuleHandle->metadata.coreMetadata.earthEqRad.name);
  if ( sGranuleHandle->metadata.coreMetadata.earthEqRad.value != NULL )
    free( sGranuleHandle->metadata.coreMetadata.earthEqRad.value);

  if ( sGranuleHandle->metadata.coreMetadata.flatteningRatio.name != NULL )
    free( sGranuleHandle->metadata.coreMetadata.flatteningRatio.name);
  if ( sGranuleHandle->metadata.coreMetadata.flatteningRatio.value != NULL )
    free( sGranuleHandle->metadata.coreMetadata.flatteningRatio.value);

  if ( sGranuleHandle->metadata.coreMetadata.orbitModelName.name != NULL )
    free( sGranuleHandle->metadata.coreMetadata.orbitModelName.name);
  if ( sGranuleHandle->metadata.coreMetadata.orbitModelName.value != NULL )
    free( sGranuleHandle->metadata.coreMetadata.orbitModelName.value);

  if ( sGranuleHandle->metadata.coreMetadata.kepSemiMajorAxis.name != NULL )
    free( sGranuleHandle->metadata.coreMetadata.kepSemiMajorAxis.name);
  if ( sGranuleHandle->metadata.coreMetadata.kepSemiMajorAxis.value != NULL )
    free( sGranuleHandle->metadata.coreMetadata.kepSemiMajorAxis.value);

  if ( sGranuleHandle->metadata.coreMetadata.kepMeanAnomaly.name != NULL )
    free( sGranuleHandle->metadata.coreMetadata.kepMeanAnomaly.name);
  if ( sGranuleHandle->metadata.coreMetadata.kepMeanAnomaly.value != NULL )
    free( sGranuleHandle->metadata.coreMetadata.kepMeanAnomaly.value);

  if ( sGranuleHandle->metadata.coreMetadata.kepRightAscenNode.name != NULL )
    free( sGranuleHandle->metadata.coreMetadata.kepRightAscenNode.name);
  if ( sGranuleHandle->metadata.coreMetadata.kepRightAscenNode.value != NULL )
    free( sGranuleHandle->metadata.coreMetadata.kepRightAscenNode.value);

  if ( sGranuleHandle->metadata.coreMetadata.kepArgOfPerigee.name != NULL )
    free( sGranuleHandle->metadata.coreMetadata.kepArgOfPerigee.name);
  if ( sGranuleHandle->metadata.coreMetadata.kepArgOfPerigee.value != NULL )
    free( sGranuleHandle->metadata.coreMetadata.kepArgOfPerigee.value);

  if ( sGranuleHandle->metadata.coreMetadata.kepEccentricity.name != NULL ) 
    free( sGranuleHandle->metadata.coreMetadata.kepEccentricity.name);
  if ( sGranuleHandle->metadata.coreMetadata.kepEccentricity.value != NULL )
    free( sGranuleHandle->metadata.coreMetadata.kepEccentricity.value);

  if ( sGranuleHandle->metadata.coreMetadata.kepInclination.name != NULL )
    free( sGranuleHandle->metadata.coreMetadata.kepInclination.name);
  if ( sGranuleHandle->metadata.coreMetadata.kepInclination.value != NULL )
    free( sGranuleHandle->metadata.coreMetadata.kepInclination.value);

  if ( sGranuleHandle->metadata.coreMetadata.kepEpochTime.name != NULL ) 
    free( sGranuleHandle->metadata.coreMetadata.kepEpochTime.name);
  if ( sGranuleHandle->metadata.coreMetadata.kepEpochTime.value != NULL )
    free( sGranuleHandle->metadata.coreMetadata.kepEpochTime.value);

  if ( sGranuleHandle->metadata.coreMetadata.kepEpochDate.name != NULL )
    free( sGranuleHandle->metadata.coreMetadata.kepEpochDate.name);
  if ( sGranuleHandle->metadata.coreMetadata.kepEpochDate.value != NULL )
    free( sGranuleHandle->metadata.coreMetadata.kepEpochDate.value);

  if ( sGranuleHandle->metadata.coreMetadata.kepEpochMillisec.name != NULL )
    free( sGranuleHandle->metadata.coreMetadata.kepEpochMillisec.name);
  if ( sGranuleHandle->metadata.coreMetadata.kepEpochMillisec.value != NULL )
    free( sGranuleHandle->metadata.coreMetadata.kepEpochMillisec.value);

  if ( sGranuleHandle->metadata.coreMetadata.westBoundCoord.name != NULL )
    free( sGranuleHandle->metadata.coreMetadata.westBoundCoord.name);
  if ( sGranuleHandle->metadata.coreMetadata.westBoundCoord.value != NULL )
    free( sGranuleHandle->metadata.coreMetadata.westBoundCoord.value);

  if ( sGranuleHandle->metadata.coreMetadata.eastBoundCoord.name != NULL )
    free( sGranuleHandle->metadata.coreMetadata.eastBoundCoord.name);
  if ( sGranuleHandle->metadata.coreMetadata.eastBoundCoord.value != NULL )
    free( sGranuleHandle->metadata.coreMetadata.eastBoundCoord.value);

  if ( sGranuleHandle->metadata.coreMetadata.northBoundCoord.name != NULL )
    free( sGranuleHandle->metadata.coreMetadata.northBoundCoord.name);
  if ( sGranuleHandle->metadata.coreMetadata.northBoundCoord.value != NULL )
    free( sGranuleHandle->metadata.coreMetadata.northBoundCoord.value);

  if ( sGranuleHandle->metadata.coreMetadata.southBoundCoord.name != NULL )
    free( sGranuleHandle->metadata.coreMetadata.southBoundCoord.name);
  if ( sGranuleHandle->metadata.coreMetadata.southBoundCoord.value != NULL )
    free( sGranuleHandle->metadata.coreMetadata.southBoundCoord.value);

  if ( sGranuleHandle->metadata.coreMetadata.centerPointLat.name != NULL )
    free( sGranuleHandle->metadata.coreMetadata.centerPointLat.name);
  if ( sGranuleHandle->metadata.coreMetadata.centerPointLat.value != NULL )
    free( sGranuleHandle->metadata.coreMetadata.centerPointLat.value);

  if ( sGranuleHandle->metadata.coreMetadata.centerPointLon.name != NULL )
    free( sGranuleHandle->metadata.coreMetadata.centerPointLon.name);
  if ( sGranuleHandle->metadata.coreMetadata.centerPointLon.value != NULL )
    free( sGranuleHandle->metadata.coreMetadata.centerPointLon.value);

  if ( sGranuleHandle->metadata.coreMetadata.radius.name != NULL )
    free( sGranuleHandle->metadata.coreMetadata.radius.name);
  if ( sGranuleHandle->metadata.coreMetadata.radius.value != NULL )
    free( sGranuleHandle->metadata.coreMetadata.radius.value);

  if ( sGranuleHandle->metadata.coreMetadata.latitudeRes.name != NULL )
    free( sGranuleHandle->metadata.coreMetadata.latitudeRes.name);
  if ( sGranuleHandle->metadata.coreMetadata.latitudeRes.value != NULL )
    free( sGranuleHandle->metadata.coreMetadata.latitudeRes.value);

  if ( sGranuleHandle->metadata.coreMetadata.longitudeRes.name != NULL )
    free( sGranuleHandle->metadata.coreMetadata.longitudeRes.name);
  if ( sGranuleHandle->metadata.coreMetadata.longitudeRes.value != NULL )
    free( sGranuleHandle->metadata.coreMetadata.longitudeRes.value);

  if ( sGranuleHandle->metadata.coreMetadata.geoCoordUnits.name != NULL )
    free( sGranuleHandle->metadata.coreMetadata.geoCoordUnits.name);
  if ( sGranuleHandle->metadata.coreMetadata.geoCoordUnits.value != NULL )
    free( sGranuleHandle->metadata.coreMetadata.geoCoordUnits.value);

  if ( sGranuleHandle->metadata.coreMetadata.temporRngType.name != NULL )
    free( sGranuleHandle->metadata.coreMetadata.temporRngType.name);
  if ( sGranuleHandle->metadata.coreMetadata.temporRngType.value != NULL )
    free( sGranuleHandle->metadata.coreMetadata.temporRngType.value);

  if ( sGranuleHandle->metadata.coreMetadata.qaParamName.name != NULL )
    free( sGranuleHandle->metadata.coreMetadata.qaParamName.name);
  if ( sGranuleHandle->metadata.coreMetadata.qaParamName.value != NULL )
    free( sGranuleHandle->metadata.coreMetadata.qaParamName.value);

  if ( sGranuleHandle->metadata.coreMetadata.qaParamValue.name != NULL )
    free( sGranuleHandle->metadata.coreMetadata.qaParamValue.name);
  if ( sGranuleHandle->metadata.coreMetadata.qaParamValue.value != NULL )
    free( sGranuleHandle->metadata.coreMetadata.qaParamValue.value);

  if ( sGranuleHandle->metadata.coreMetadata.reproStat.name != NULL )
    free( sGranuleHandle->metadata.coreMetadata.reproStat.name);
  if ( sGranuleHandle->metadata.coreMetadata.reproStat.value != NULL )
    free( sGranuleHandle->metadata.coreMetadata.reproStat.value);

  if ( sGranuleHandle->metadata.coreMetadata.browseName.name != NULL )
    free( sGranuleHandle->metadata.coreMetadata.browseName.name);
  if ( sGranuleHandle->metadata.coreMetadata.browseName.value != NULL )
    free( sGranuleHandle->metadata.coreMetadata.browseName.value);

  if ( sGranuleHandle->metadata.coreMetadata.contact.name != NULL )
    free( sGranuleHandle->metadata.coreMetadata.contact.name);
  if ( sGranuleHandle->metadata.coreMetadata.contact.value != NULL )
    free( sGranuleHandle->metadata.coreMetadata.contact.value);

  if ( sGranuleHandle->metadata.coreMetadata.meanMotion.name != NULL )
      free( sGranuleHandle->metadata.coreMetadata.meanMotion.name);
  if ( sGranuleHandle->metadata.coreMetadata.meanMotion.value != NULL )
    free( sGranuleHandle->metadata.coreMetadata.meanMotion.value);

  if ( sGranuleHandle->metadata.coreMetadata.orbitAdjFlag.name != NULL )
    free( sGranuleHandle->metadata.coreMetadata.orbitAdjFlag.name);
  if ( sGranuleHandle->metadata.coreMetadata.orbitAdjFlag.value != NULL )
    free( sGranuleHandle->metadata.coreMetadata.orbitAdjFlag.value);

  if ( sGranuleHandle->metadata.coreMetadata.attitudeMode.name != NULL )
    free( sGranuleHandle->metadata.coreMetadata.attitudeMode.name);
  if ( sGranuleHandle->metadata.coreMetadata.attitudeMode.value != NULL )
    free( sGranuleHandle->metadata.coreMetadata.attitudeMode.value);

  if ( sGranuleHandle->metadata.coreMetadata.beginSolarBeta.name != NULL )
    free( sGranuleHandle->metadata.coreMetadata.beginSolarBeta.name);
  if ( sGranuleHandle->metadata.coreMetadata.beginSolarBeta.value != NULL )
    free( sGranuleHandle->metadata.coreMetadata.beginSolarBeta.value);

  if ( sGranuleHandle->metadata.coreMetadata.endSolarBeta.name != NULL )
    free( sGranuleHandle->metadata.coreMetadata.endSolarBeta.name);
  if ( sGranuleHandle->metadata.coreMetadata.endSolarBeta.value != NULL )
    free( sGranuleHandle->metadata.coreMetadata.endSolarBeta.value);

  if ( sGranuleHandle->metadata.coreMetadata.sensorAlgn.name != NULL )
    free ( sGranuleHandle->metadata.coreMetadata.sensorAlgn.name);
  if( sGranuleHandle->metadata.coreMetadata.sensorAlgn.value != NULL )
    free( sGranuleHandle->metadata.coreMetadata.sensorAlgn.value);

  if ( sGranuleHandle->metadata.coreMetadata.sensorAlgnChanOff.name != NULL )
    free( sGranuleHandle->metadata.coreMetadata.sensorAlgnChanOff.name);
  if ( sGranuleHandle->metadata.coreMetadata.sensorAlgnChanOff.value != NULL )
    free( sGranuleHandle->metadata.coreMetadata.sensorAlgnChanOff.value);

  if ( sGranuleHandle->metadata.coreMetadata.scanPathModel.name != NULL )
    free( sGranuleHandle->metadata.coreMetadata.scanPathModel.name);
  if ( sGranuleHandle->metadata.coreMetadata.scanPathModel.value != NULL )
    free( sGranuleHandle->metadata.coreMetadata.scanPathModel.value);

  if ( sGranuleHandle->metadata.coreMetadata.scanPathParam.name != NULL )
    free( sGranuleHandle->metadata.coreMetadata.scanPathParam.name);
  if ( sGranuleHandle->metadata.coreMetadata.scanPathParam.value != NULL )
    free( sGranuleHandle->metadata.coreMetadata.scanPathParam.value);

  if ( sGranuleHandle->metadata.coreMetadata.ephemFilename.name != NULL )
    free( sGranuleHandle->metadata.coreMetadata.ephemFilename.name);
  if ( sGranuleHandle->metadata.coreMetadata.ephemFilename.value != NULL ) 
    free( sGranuleHandle->metadata.coreMetadata.ephemFilename.value);

  /*
   * Free the Product Specific metadata elements
   */
  if ( sGranuleHandle->metadata.psMetadata.numDataGap.name != NULL )
    free( sGranuleHandle->metadata.psMetadata.numDataGap.name);
  if ( sGranuleHandle->metadata.psMetadata.numDataGap.value != NULL )
    free( sGranuleHandle->metadata.psMetadata.numDataGap.value);

  if ( sGranuleHandle->metadata.psMetadata.algorithmVersion.name != NULL )
    free( sGranuleHandle->metadata.psMetadata.algorithmVersion.name);
  if ( sGranuleHandle->metadata.psMetadata.algorithmVersion.value != NULL )
    free( sGranuleHandle->metadata.psMetadata.algorithmVersion.value);

  if ( sGranuleHandle->metadata.psMetadata.productVersion.name != NULL )
    free( sGranuleHandle->metadata.psMetadata.productVersion.name);
  if ( sGranuleHandle->metadata.psMetadata.productVersion.value != NULL )
    free( sGranuleHandle->metadata.psMetadata.productVersion.value);

  if ( sGranuleHandle->metadata.psMetadata.toolkitVersion.name != NULL )
    free( sGranuleHandle->metadata.psMetadata.toolkitVersion.name);
  if ( sGranuleHandle->metadata.psMetadata.toolkitVersion.value != NULL )
    free( sGranuleHandle->metadata.psMetadata.toolkitVersion.value);

  if ( sGranuleHandle->metadata.psMetadata.calCoeffVersion.name != NULL )
    free( sGranuleHandle->metadata.psMetadata.calCoeffVersion.name);
  if ( sGranuleHandle->metadata.psMetadata.calCoeffVersion.value != NULL )
    free( sGranuleHandle->metadata.psMetadata.calCoeffVersion.value);

  if ( sGranuleHandle->metadata.psMetadata.missingData.name != NULL )
    free( sGranuleHandle->metadata.psMetadata.missingData.name);
  if ( sGranuleHandle->metadata.psMetadata.missingData.value != NULL )
    free( sGranuleHandle->metadata.psMetadata.missingData.value);

  if ( sGranuleHandle->metadata.psMetadata.percentBadMissPix.name != NULL )
    free( sGranuleHandle->metadata.psMetadata.percentBadMissPix.name);
  if ( sGranuleHandle->metadata.psMetadata.percentBadMissPix.value != NULL )
    free( sGranuleHandle->metadata.psMetadata.percentBadMissPix.value);

  if ( sGranuleHandle->metadata.psMetadata.maxValidChannel.name != NULL )
    free( sGranuleHandle->metadata.psMetadata.maxValidChannel.name);
  if ( sGranuleHandle->metadata.psMetadata.maxValidChannel.value != NULL )
    free( sGranuleHandle->metadata.psMetadata.maxValidChannel.value);

  if ( sGranuleHandle->metadata.psMetadata.minValidChannel.name != NULL )
    free( sGranuleHandle->metadata.psMetadata.minValidChannel.name);
  if ( sGranuleHandle->metadata.psMetadata.minValidChannel.value != NULL )
    free( sGranuleHandle->metadata.psMetadata.minValidChannel.value);

  if ( sGranuleHandle->metadata.psMetadata.minMaxUnits.name != NULL )
    free( sGranuleHandle->metadata.psMetadata.minMaxUnits.name);
  if ( sGranuleHandle->metadata.psMetadata.minMaxUnits.value != NULL )
    free( sGranuleHandle->metadata.psMetadata.minMaxUnits.value);

  if ( sGranuleHandle->metadata.psMetadata.orbitSize.name != NULL )
    free( sGranuleHandle->metadata.psMetadata.orbitSize.name);
  if ( sGranuleHandle->metadata.psMetadata.orbitSize.value != NULL )
    free( sGranuleHandle->metadata.psMetadata.orbitSize.value);

  if ( sGranuleHandle->metadata.psMetadata.radarWavelength.name != NULL )
    free( sGranuleHandle->metadata.psMetadata.radarWavelength.name);
  if ( sGranuleHandle->metadata.psMetadata.radarWavelength.value != NULL )
    free( sGranuleHandle->metadata.psMetadata.radarWavelength.value);

  if ( sGranuleHandle->metadata.psMetadata.minRefThreshold.name != NULL )
    free( sGranuleHandle->metadata.psMetadata.minRefThreshold.name);
  if ( sGranuleHandle->metadata.psMetadata.minRefThreshold.value != NULL )
    free( sGranuleHandle->metadata.psMetadata.minRefThreshold.value);

  if ( sGranuleHandle->metadata.psMetadata.algorithmID.name != NULL )
    free( sGranuleHandle->metadata.psMetadata.algorithmID.name);
  if ( sGranuleHandle->metadata.psMetadata.algorithmID.value != NULL )
    free( sGranuleHandle->metadata.psMetadata.algorithmID.value);

  if ( sGranuleHandle->metadata.psMetadata.dataAccuracy.name != NULL )
    free( sGranuleHandle->metadata.psMetadata.dataAccuracy.name);
  if ( sGranuleHandle->metadata.psMetadata.dataAccuracy.value != NULL )
    free( sGranuleHandle->metadata.psMetadata.dataAccuracy.value);

  if ( sGranuleHandle->metadata.psMetadata.dataGap.name != NULL )
    free( sGranuleHandle->metadata.psMetadata.dataGap.name);
  if ( sGranuleHandle->metadata.psMetadata.dataGap.value != NULL )
    free( sGranuleHandle->metadata.psMetadata.dataGap.value);

  if ( sGranuleHandle->metadata.psMetadata.inputID.name != NULL ) 
    free( sGranuleHandle->metadata.psMetadata.inputID.name);
  if ( sGranuleHandle->metadata.psMetadata.inputID.value != NULL )
    free( sGranuleHandle->metadata.psMetadata.inputID.value);

  if ( sGranuleHandle->metadata.psMetadata.dateGenInputFile.name != NULL )
    free( sGranuleHandle->metadata.psMetadata.dateGenInputFile.name);
  if ( sGranuleHandle->metadata.psMetadata.dateGenInputFile.value != NULL ) 
    free( sGranuleHandle->metadata.psMetadata.dateGenInputFile.value);

  if ( sGranuleHandle->metadata.psMetadata.dataCenterSrc.name != NULL )
    free( sGranuleHandle->metadata.psMetadata.dataCenterSrc.name);
  if ( sGranuleHandle->metadata.psMetadata.dataCenterSrc.value != NULL )
    free( sGranuleHandle->metadata.psMetadata.dataCenterSrc.value);

  if ( sGranuleHandle->metadata.psMetadata.generationDate.name != NULL )
    free( sGranuleHandle->metadata.psMetadata.generationDate.name);
  if ( sGranuleHandle->metadata.psMetadata.generationDate.value != NULL )
    free( sGranuleHandle->metadata.psMetadata.generationDate.value);

  if ( sGranuleHandle->metadata.psMetadata.dayNight.name != NULL )
    free( sGranuleHandle->metadata.psMetadata.dayNight.name);
  if ( sGranuleHandle->metadata.psMetadata.dayNight.value != NULL )
    free( sGranuleHandle->metadata.psMetadata.dayNight.value);

  if ( sGranuleHandle->metadata.psMetadata.solarGains.name != NULL )
    free( sGranuleHandle->metadata.psMetadata.solarGains.name);
  if ( sGranuleHandle->metadata.psMetadata.solarGains.value != NULL )
    free( sGranuleHandle->metadata.psMetadata.solarGains.value);

  if ( sGranuleHandle->metadata.psMetadata.tmiRolloverCoef.name != NULL )
    free( sGranuleHandle->metadata.psMetadata.tmiRolloverCoef.name);
  if ( sGranuleHandle->metadata.psMetadata.tmiRolloverCoef.value != NULL )
    free( sGranuleHandle->metadata.psMetadata.tmiRolloverCoef.value);

  if ( sGranuleHandle->metadata.psMetadata.firstScanUTCdate.name != NULL )
    free( sGranuleHandle->metadata.psMetadata.firstScanUTCdate.name);
  if ( sGranuleHandle->metadata.psMetadata.firstScanUTCdate.value != NULL )
    free( sGranuleHandle->metadata.psMetadata.firstScanUTCdate.value);

  if ( sGranuleHandle->metadata.psMetadata.firstScanUTCtime.name != NULL )
    free( sGranuleHandle->metadata.psMetadata.firstScanUTCtime.name);
  if ( sGranuleHandle->metadata.psMetadata.firstScanUTCtime.value != NULL )
    free( sGranuleHandle->metadata.psMetadata.firstScanUTCtime.value);

  if ( sGranuleHandle->metadata.psMetadata.firstScanUTCmsec.name != NULL )
    free( sGranuleHandle->metadata.psMetadata.firstScanUTCmsec.name);
  if ( sGranuleHandle->metadata.psMetadata.firstScanUTCmsec.value != NULL )
    free( sGranuleHandle->metadata.psMetadata.firstScanUTCmsec.value);

  if ( sGranuleHandle->metadata.psMetadata.firstScanSCsec.name != NULL )
    free( sGranuleHandle->metadata.psMetadata.firstScanSCsec.name);
  if ( sGranuleHandle->metadata.psMetadata.firstScanSCsec.value != NULL )
    free( sGranuleHandle->metadata.psMetadata.firstScanSCsec.value);

  if ( sGranuleHandle->metadata.psMetadata.firstScanSCsubsec.name != NULL )
    free( sGranuleHandle->metadata.psMetadata.firstScanSCsubsec.name);
  if ( sGranuleHandle->metadata.psMetadata.firstScanSCsubsec.value != NULL )
    free( sGranuleHandle->metadata.psMetadata.firstScanSCsubsec.value);

  if ( sGranuleHandle->metadata.psMetadata.lastScanUTCdate.name != NULL )
    free( sGranuleHandle->metadata.psMetadata.lastScanUTCdate.name);
  if ( sGranuleHandle->metadata.psMetadata.lastScanUTCdate.value != NULL )
    free( sGranuleHandle->metadata.psMetadata.lastScanUTCdate.value);

  if ( sGranuleHandle->metadata.psMetadata.lastScanUTCtime.name != NULL )
    free( sGranuleHandle->metadata.psMetadata.lastScanUTCtime.name);
  if ( sGranuleHandle->metadata.psMetadata.lastScanUTCtime.value != NULL )
    free( sGranuleHandle->metadata.psMetadata.lastScanUTCtime.value);

  if ( sGranuleHandle->metadata.psMetadata.lastScanUTCmsec.name != NULL )
    free( sGranuleHandle->metadata.psMetadata.lastScanUTCmsec.name);
  if ( sGranuleHandle->metadata.psMetadata.lastScanUTCmsec.value != NULL )
    free( sGranuleHandle->metadata.psMetadata.lastScanUTCmsec.value);

  if ( sGranuleHandle->metadata.psMetadata.lastScanSCsec.name != NULL )
    free( sGranuleHandle->metadata.psMetadata.lastScanSCsec.name);
  if ( sGranuleHandle->metadata.psMetadata.lastScanSCsec.value != NULL )
    free( sGranuleHandle->metadata.psMetadata.lastScanSCsec.value);

  if ( sGranuleHandle->metadata.psMetadata.lastScanSCsubsec.name != NULL )
    free( sGranuleHandle->metadata.psMetadata.lastScanSCsubsec.name);
  if ( sGranuleHandle->metadata.psMetadata.lastScanSCsubsec.value != NULL )
    free( sGranuleHandle->metadata.psMetadata.lastScanSCsubsec.value);

  if ( sGranuleHandle->metadata.psMetadata.utcfSec.name != NULL )
    free( sGranuleHandle->metadata.psMetadata.utcfSec.name);
  if ( sGranuleHandle->metadata.psMetadata.utcfSec.value != NULL )
    free( sGranuleHandle->metadata.psMetadata.utcfSec.value);

  if ( sGranuleHandle->metadata.psMetadata.utcfSubsec.name != NULL )
    free( sGranuleHandle->metadata.psMetadata.utcfSubsec.name);
  if ( sGranuleHandle->metadata.psMetadata.utcfSubsec.value != NULL )
    free( sGranuleHandle->metadata.psMetadata.utcfSubsec.value);

  if ( sGranuleHandle->metadata.psMetadata.utcfFlag.name != NULL )
    free( sGranuleHandle->metadata.psMetadata.utcfFlag.name);
  if ( sGranuleHandle->metadata.psMetadata.utcfFlag.value != NULL )
    free( sGranuleHandle->metadata.psMetadata.utcfFlag.value);

  if ( sGranuleHandle->metadata.psMetadata.leapSecsFlag.name != NULL )
    free( sGranuleHandle->metadata.psMetadata.leapSecsFlag.name);
  if ( sGranuleHandle->metadata.psMetadata.leapSecsFlag.value != NULL ) 
    free( sGranuleHandle->metadata.psMetadata.leapSecsFlag.value);

  if ( sGranuleHandle->metadata.psMetadata.radarName.name != NULL )
    free( sGranuleHandle->metadata.psMetadata.radarName.name);
  if ( sGranuleHandle->metadata.psMetadata.radarName.value != NULL )
    free( sGranuleHandle->metadata.psMetadata.radarName.value);

  if ( sGranuleHandle->metadata.psMetadata.radarCity.name != NULL )
    free( sGranuleHandle->metadata.psMetadata.radarCity.name);
  if ( sGranuleHandle->metadata.psMetadata.radarCity.value != NULL )
    free( sGranuleHandle->metadata.psMetadata.radarCity.value);

  if ( sGranuleHandle->metadata.psMetadata.radarState.name != NULL )
    free( sGranuleHandle->metadata.psMetadata.radarState.name);
  if ( sGranuleHandle->metadata.psMetadata.radarState.value != NULL )
    free( sGranuleHandle->metadata.psMetadata.radarState.value);

  if ( sGranuleHandle->metadata.psMetadata.radarCountry.name != NULL )
    free( sGranuleHandle->metadata.psMetadata.radarCountry.name);
  if ( sGranuleHandle->metadata.psMetadata.radarCountry.value != NULL )
    free( sGranuleHandle->metadata.psMetadata.radarCountry.value);

  if ( sGranuleHandle->metadata.psMetadata.numVOS.name != NULL )
    free( sGranuleHandle->metadata.psMetadata.numVOS.name);
  if ( sGranuleHandle->metadata.psMetadata.numVOS.value != NULL )
    free( sGranuleHandle->metadata.psMetadata.numVOS.value);

  if ( sGranuleHandle->metadata.psMetadata.radarOriginLat.name != NULL )
    free( sGranuleHandle->metadata.psMetadata.radarOriginLat.name);
  if ( sGranuleHandle->metadata.psMetadata.radarOriginLat.value != NULL )
    free( sGranuleHandle->metadata.psMetadata.radarOriginLat.value);

  if ( sGranuleHandle->metadata.psMetadata.radarOriginLon.name != NULL )
    free( sGranuleHandle->metadata.psMetadata.radarOriginLon.name);
  if ( sGranuleHandle->metadata.psMetadata.radarOriginLon.value != NULL )
    free( sGranuleHandle->metadata.psMetadata.radarOriginLon.value);

  if ( sGranuleHandle->metadata.psMetadata.radarOriginAlt.name != NULL )
    free( sGranuleHandle->metadata.psMetadata.radarOriginAlt.name);
  if ( sGranuleHandle->metadata.psMetadata.radarOriginAlt.value != NULL ) 
    free( sGranuleHandle->metadata.psMetadata.radarOriginAlt.value);

  if ( sGranuleHandle->metadata.psMetadata.radarSpacingX.name != NULL )
    free( sGranuleHandle->metadata.psMetadata.radarSpacingX.name);
  if ( sGranuleHandle->metadata.psMetadata.radarSpacingX.value != NULL )
    free( sGranuleHandle->metadata.psMetadata.radarSpacingX.value);

  if ( sGranuleHandle->metadata.psMetadata.radarSpacingY.name != NULL )
    free( sGranuleHandle->metadata.psMetadata.radarSpacingY.name);
  if ( sGranuleHandle->metadata.psMetadata.radarSpacingY.value != NULL ) 
    free( sGranuleHandle->metadata.psMetadata.radarSpacingY.value);

  if ( sGranuleHandle->metadata.psMetadata.radarSpacingZ.name != NULL )
    free( sGranuleHandle->metadata.psMetadata.radarSpacingZ.name);
  if ( sGranuleHandle->metadata.psMetadata.radarSpacingZ.value != NULL )
    free( sGranuleHandle->metadata.psMetadata.radarSpacingZ.value);

  if ( sGranuleHandle->metadata.psMetadata.radarGridSizeX.name != NULL )
    free( sGranuleHandle->metadata.psMetadata.radarGridSizeX.name);
  if ( sGranuleHandle->metadata.psMetadata.radarGridSizeX.value != NULL )
    free( sGranuleHandle->metadata.psMetadata.radarGridSizeX.value);

  if ( sGranuleHandle->metadata.psMetadata.radarGridSizeY.name != NULL )
    free( sGranuleHandle->metadata.psMetadata.radarGridSizeY.name);
  if ( sGranuleHandle->metadata.psMetadata.radarGridSizeY.value != NULL )
    free( sGranuleHandle->metadata.psMetadata.radarGridSizeY.value);

  if ( sGranuleHandle->metadata.psMetadata.radarGridSizeZ.name != NULL )
    free( sGranuleHandle->metadata.psMetadata.radarGridSizeZ.name);
  if ( sGranuleHandle->metadata.psMetadata.radarGridSizeZ.value != NULL )
    free( sGranuleHandle->metadata.psMetadata.radarGridSizeZ.value);

  if ( sGranuleHandle->metadata.psMetadata.dzcal.name != NULL )
    free( sGranuleHandle->metadata.psMetadata.dzcal.name);
  if ( sGranuleHandle->metadata.psMetadata.dzcal.value != NULL )
    free( sGranuleHandle->metadata.psMetadata.dzcal.value);

  if ( sGranuleHandle->metadata.psMetadata.gvL1Cscale.name != NULL ) 
    free( sGranuleHandle->metadata.psMetadata.gvL1Cscale.name);
  if ( sGranuleHandle->metadata.psMetadata.gvL1Cscale.value != NULL )
    free( sGranuleHandle->metadata.psMetadata.gvL1Cscale.value);

  if ( sGranuleHandle->metadata.psMetadata.alpha.name != NULL )
    free( sGranuleHandle->metadata.psMetadata.alpha.name);
  if ( sGranuleHandle->metadata.psMetadata.alpha.value != NULL )
    free( sGranuleHandle->metadata.psMetadata.alpha.value);

  if ( sGranuleHandle->metadata.psMetadata.runtimeOption.name != NULL )
    free( sGranuleHandle->metadata.psMetadata.runtimeOption.name);
  if ( sGranuleHandle->metadata.psMetadata.runtimeOption.value != NULL )
    free( sGranuleHandle->metadata.psMetadata.runtimeOption.value );

  if ( sGranuleHandle->metadata.psMetadata.anomalyFlag.name != NULL ) 
    free( sGranuleHandle->metadata.psMetadata.anomalyFlag.name);
  if ( sGranuleHandle->metadata.psMetadata.anomalyFlag.value != NULL )
    free( sGranuleHandle->metadata.psMetadata.anomalyFlag.value);

  if ( sGranuleHandle->metadata.psMetadata.softwareVersion.name != NULL )
    free( sGranuleHandle->metadata.psMetadata.softwareVersion.name);
  if ( sGranuleHandle->metadata.psMetadata.softwareVersion.value != NULL )
    free( sGranuleHandle->metadata.psMetadata.softwareVersion.value);

  if ( sGranuleHandle->metadata.psMetadata.databaseVersion.name != NULL )
    free( sGranuleHandle->metadata.psMetadata.databaseVersion.name);
  if ( sGranuleHandle->metadata.psMetadata.databaseVersion.value != NULL )
    free( sGranuleHandle->metadata.psMetadata.databaseVersion.value);

  if ( sGranuleHandle->metadata.psMetadata.totalQualityCode.name != NULL )
    free( sGranuleHandle->metadata.psMetadata.totalQualityCode.name);
  if ( sGranuleHandle->metadata.psMetadata.totalQualityCode.value != NULL ) 
    free( sGranuleHandle->metadata.psMetadata.totalQualityCode.value);

  if ( sGranuleHandle->metadata.psMetadata.lonOnEquator.name != NULL )
    free( sGranuleHandle->metadata.psMetadata.lonOnEquator.name);
  if ( sGranuleHandle->metadata.psMetadata.lonOnEquator.value != NULL )
    free( sGranuleHandle->metadata.psMetadata.lonOnEquator.value);

  if ( sGranuleHandle->metadata.psMetadata.UTCDateOnEquator.name != NULL )
    free( sGranuleHandle->metadata.psMetadata.UTCDateOnEquator.name);
  if ( sGranuleHandle->metadata.psMetadata.UTCDateOnEquator.value != NULL )
    free( sGranuleHandle->metadata.psMetadata.UTCDateOnEquator.value);

  if ( sGranuleHandle->metadata.psMetadata.UTCTimeOnEquator.name != NULL )
    free( sGranuleHandle->metadata.psMetadata.UTCTimeOnEquator.name);
  if ( sGranuleHandle->metadata.psMetadata.UTCTimeOnEquator.value != NULL )
    free( sGranuleHandle->metadata.psMetadata.UTCTimeOnEquator.value);

  if ( sGranuleHandle->metadata.psMetadata.millisecOnEquator.name != NULL )
    free( sGranuleHandle->metadata.psMetadata.millisecOnEquator.name);
  if ( sGranuleHandle->metadata.psMetadata.millisecOnEquator.value != NULL )
    free( sGranuleHandle->metadata.psMetadata.millisecOnEquator.value);

  if ( sGranuleHandle->metadata.psMetadata.centerScanUTCDate.name != NULL )
    free( sGranuleHandle->metadata.psMetadata.centerScanUTCDate.name);
  if ( sGranuleHandle->metadata.psMetadata.centerScanUTCDate.value != NULL ) 
    free( sGranuleHandle->metadata.psMetadata.centerScanUTCDate.value);

  if ( sGranuleHandle->metadata.psMetadata.centerScanUTCTime.name != NULL) 
    free( sGranuleHandle->metadata.psMetadata.centerScanUTCTime.name);
  if ( sGranuleHandle->metadata.psMetadata.centerScanUTCTime.value != NULL)
    free( sGranuleHandle->metadata.psMetadata.centerScanUTCTime.value);

  if ( sGranuleHandle->metadata.psMetadata.centerScanUTCMillisec.name != NULL )
    free( sGranuleHandle->metadata.psMetadata.centerScanUTCMillisec.name);
  if ( sGranuleHandle->metadata.psMetadata.centerScanUTCMillisec.value != NULL )
    free( sGranuleHandle->metadata.psMetadata.centerScanUTCMillisec.value);

  if ( sGranuleHandle->metadata.psMetadata.firstScanLat.name != NULL )
    free( sGranuleHandle->metadata.psMetadata.firstScanLat.name);
  if ( sGranuleHandle->metadata.psMetadata.firstScanLat.value != NULL )
    free( sGranuleHandle->metadata.psMetadata.firstScanLat.value);

  if ( sGranuleHandle->metadata.psMetadata.firstScanLon.name != NULL )  
    free( sGranuleHandle->metadata.psMetadata.firstScanLon.name);
  if ( sGranuleHandle->metadata.psMetadata.firstScanLon.value != NULL )
    free( sGranuleHandle->metadata.psMetadata.firstScanLon.value);

  if ( sGranuleHandle->metadata.psMetadata.lastScanLat.name != NULL )
    free( sGranuleHandle->metadata.psMetadata.lastScanLat.name);
  if ( sGranuleHandle->metadata.psMetadata.lastScanLat.value != NULL )
    free( sGranuleHandle->metadata.psMetadata.lastScanLat.value);

  if ( sGranuleHandle->metadata.psMetadata.lastScanLon.name != NULL )
    free( sGranuleHandle->metadata.psMetadata.lastScanLon.name);
  if ( sGranuleHandle->metadata.psMetadata.lastScanLon.value != NULL )
    free( sGranuleHandle->metadata.psMetadata.lastScanLon.value);

  if ( sGranuleHandle->metadata.psMetadata.numOfRainScans.name != NULL )
    free( sGranuleHandle->metadata.psMetadata.numOfRainScans.name);
  if ( sGranuleHandle->metadata.psMetadata.numOfRainScans.value != NULL)
    free( sGranuleHandle->metadata.psMetadata.numOfRainScans.value);

}


/*****************************************************************************
*
*  FUNCTION STATEMENT:
*	resetVgroup(VGROUP_LINK *vgroup, int offsetPos)
*
*  DESCRIPTION:
*	This routine resets all the vdata and sds according to 
*	the offsetPos.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*	vgroup			The list of vgroups.
*	offsetPos		The position of offset.
*
*  OUTPUT PARAMETERS:           
*       Parameter Name          Description
*       --------------          -----------
*	vgroup			The list of vgroups which now contains
*				repositioned vdata and sds.
*
*  CALLED BY:                   TKseek()
*
*  RETURN VALUES:               None.
*
*  ROUTINES CALLED:
*       Routine Name            Description
*       ------------            -----------
*	VSseek			A HDF routine which provides a mechanism for
*				random-access I/O within a Vdata.
*	resetVgroup		This routine resets all the vdata and sds 
*				according to the offsetPos.
*
******************************************************************************/

void resetVgroup(VGROUP_LINK *vgroup, int offsetPos)
{
  VDATA_LINK *tempVdata;
  SDS_LINK  *tempSDS;

  if (vgroup != NULL)
  {
      tempVdata = vgroup->vdata;
      tempSDS = vgroup->sds;

      /*
      ** Relocating the HDF file positions.
      ** VSseek provides a mechanism for random-access I/O within a
      ** Vdata and returns the record position (Zero or a positive
      ** integer) if success; otherwise FAIL (-1).
      */
      while (tempVdata != NULL)
      {      if (VSseek(tempVdata->vdataID, offsetPos) == TK_HDF_FAIL)
             {
                 /* VSseek failed in resetVgroup. */
                 TKreportError(E_TK_FAILVSSK);
             }
             tempVdata = tempVdata->next;
      }

      /*
      ** Resetting start provides a mechanism for random-access I/O within
      ** SDS.
      */
      while(tempSDS != NULL)
      {
            tempSDS->start[0] = offsetPos;
            tempSDS = tempSDS->next;
      }

      /*
      ** Recursively reset the sub-vgroups and other vgroups.
      */
      resetVgroup(vgroup->next, offsetPos);
  }
}



/*****************************************************************************
*
*  FUNCTION STATEMENT:
*       makePRcalCoefHdr(VDATA_FIELD_LINK **vdataFields)
*
*  DESCRIPTION:
*       This routine used to initialize 1B21 or 1C21 Calibration Coefficient 
*	header fields list in link list form.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       vdataFields             Fields of list of vdata.
*
*  OUTPUT PARAMETERS:           None
*
*  CALLED BY:                   processHeader()
*
*  RETURN VALUES:               None
*
*  ROUTINES CALLED:             None
*
******************************************************************************/

void makePRcalCoefHdr(VDATA_FIELD_LINK **vdataFields)
{
  char	l1b_l1cPRfields[3][MAX_SIZE];
  int	l1b_l1cPRsize[3];
  int32	prfieldtype[3];
  int32 prdim[3];
  VDATA_FIELD_LINK	*tempField  = NULL;
  VDATA_FIELD_LINK	*tempFields = NULL;
  int	i;


  strcpy(l1b_l1cPRfields[0], "transCoef");
  l1b_l1cPRsize[0] = 1;
  prdim[0] = 1;
  prfieldtype[0] = DFNT_FLOAT32;

  strcpy(l1b_l1cPRfields[1], "receptCoef");
  l1b_l1cPRsize[1] = 1;
  prdim[1] = 1;
  prfieldtype[1] = DFNT_FLOAT32;

  strcpy(l1b_l1cPRfields[2], "fcifIOchar");
  l1b_l1cPRsize[2] = 16;
  prdim[2] = 16;
  prfieldtype[2] = DFNT_FLOAT32;

  /*
  ** Create a vdata field list. 
  */
  for (i = 0; i < 3; ++i)
  {
    tempField = (VDATA_FIELD_LINK *) malloc(sizeof(VDATA_FIELD_LINK));
    if (tempField == NULL)
	/* Memory allocation failed in makePRcalCoefHdr(). */
	TKreportError(E_TK_MEMFAILPRCALHDR);

    memset(tempField, '\0', sizeof(VDATA_FIELD_LINK));
    strcpy(tempField->name, l1b_l1cPRfields[i]);
    tempField->type = prfieldtype[i];
    tempField->fieldSize = l1b_l1cPRsize[i];
    tempField->dim = prdim[i];
    tempField->next = NULL;
    tempField->rear = NULL;
    if (tempFields == NULL)
    {
           tempFields = tempField;
    }
    else
    {
           tempFields->rear->next = tempField;
    }
    tempFields->rear = tempField;
  }

  /*
  ** Return the vdata field list by assigning vdataFields with tempFields.
  */
  *vdataFields = tempFields;

}


/*****************************************************************************
*
*  FUNCTION STATEMENT:
*       makeRayHdr(VDATA_FIELD_LINK **vdataFields)
*
*  DESCRIPTION:
*       This routine used to initialize 1B21 or 1C21 Ray Header fields 
*	list in link list form.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       vdataFields             Fields of list of vdata.
*
*  OUTPUT PARAMETERS:           None
*
*  CALLED BY:                   processHeader()
*
*  RETURN VALUES:               None
*
*  ROUTINES CALLED:             None
*
******************************************************************************/

void makeRayHdr(VDATA_FIELD_LINK **vdataFields)
{
  char  l1b_l1cRayfields[17][MAX_SIZE];
  int   l1b_l1cRaysize[17];
  int32 rayfieldtype[17];
  int32 raydim[17];
  VDATA_FIELD_LINK      *tempField  = NULL;
  VDATA_FIELD_LINK      *tempFields = NULL;
  int   i;

  strcpy(l1b_l1cRayfields[0], "rayStart");
  l1b_l1cRaysize[0] = 1;
  raydim[0] = 1;
  rayfieldtype[0] = DFNT_INT16;

  strcpy(l1b_l1cRayfields[1], "raySize");
  l1b_l1cRaysize[1] = 1;
  raydim[1] = 1;
  rayfieldtype[1] = DFNT_INT16;

  strcpy(l1b_l1cRayfields[2], "angle");
  l1b_l1cRaysize[2] = 1;
  raydim[2] = 1;
  rayfieldtype[2] = DFNT_FLOAT32;

  strcpy(l1b_l1cRayfields[3], "startBinDist");
  l1b_l1cRaysize[3] = 1;
  raydim[3] = 1;
  rayfieldtype[3] = DFNT_FLOAT32;

  strcpy(l1b_l1cRayfields[4], "rainThres1");
  l1b_l1cRaysize[4] = 1;
  raydim[4] = 1;
  rayfieldtype[4] = DFNT_FLOAT32;

  strcpy(l1b_l1cRayfields[5], "rainThres2");
  l1b_l1cRaysize[5] = 1;
  raydim[5] = 1;
  rayfieldtype[5] = DFNT_FLOAT32;

  strcpy(l1b_l1cRayfields[6], "transAntenna");
  l1b_l1cRaysize[6] = 1;
  raydim[6] = 1;
  rayfieldtype[6] = DFNT_FLOAT32;

  strcpy(l1b_l1cRayfields[7], "recvAntenna");
  l1b_l1cRaysize[7] = 1;
  raydim[7] = 1;
  rayfieldtype[7] = DFNT_FLOAT32;

  strcpy(l1b_l1cRayfields[8], "onewayAlongTrack");
  l1b_l1cRaysize[8] = 1;
  raydim[8] = 1;
  rayfieldtype[8] = DFNT_FLOAT32;

  strcpy(l1b_l1cRayfields[9], "onewayCrossTrack");
  l1b_l1cRaysize[9] = 1;
  raydim[9] = 1;
  rayfieldtype[9] = DFNT_FLOAT32;

  strcpy(l1b_l1cRayfields[10], "eqvWavelength");
  l1b_l1cRaysize[10] = 1;
  raydim[10] = 1;
  rayfieldtype[10] = DFNT_FLOAT32;

  strcpy(l1b_l1cRayfields[11], "radarConst");
  l1b_l1cRaysize[11] = 1;
  raydim[11] = 1;
  rayfieldtype[11] = DFNT_FLOAT32;

  strcpy(l1b_l1cRayfields[12], "prIntrDelay");
  l1b_l1cRaysize[12] = 1;
  raydim[12] = 1;
  rayfieldtype[12] = DFNT_FLOAT32;

  strcpy(l1b_l1cRayfields[13], "rangeBinSize");
  l1b_l1cRaysize[13] = 1;
  raydim[13] = 1;
  rayfieldtype[13] = DFNT_FLOAT32;

  strcpy(l1b_l1cRayfields[14], "logAveOffset");
  l1b_l1cRaysize[14] = 1;
  raydim[14] = 1;
  rayfieldtype[14] = DFNT_FLOAT32;

  strcpy(l1b_l1cRayfields[15], "mainlobeEdge");
  l1b_l1cRaysize[15] = 1;
  raydim[15] = 1;
  rayfieldtype[15] = DFNT_INT8;

  strcpy(l1b_l1cRayfields[16], "sidelobeRange");
  l1b_l1cRaysize[16] = 3;
  raydim[16] = 3;
  rayfieldtype[16] = DFNT_INT8;


  /*
  ** Create a vdata field list.
  */
  for (i = 0; i < 17; ++i)
  {
    tempField = (VDATA_FIELD_LINK *) malloc(sizeof(VDATA_FIELD_LINK));
    if (tempField == NULL)
	/* Memory allocation failed in makeRayHdr(). */
	TKreportError(E_TK_MEMFAILRAYHDR);

    memset(tempField, '\0', sizeof(VDATA_FIELD_LINK));
    strcpy(tempField->name, l1b_l1cRayfields[i]);
    tempField->type = rayfieldtype[i];
    tempField->fieldSize = l1b_l1cRaysize[i];
    tempField->dim = raydim[i];
    tempField->next = NULL;
    tempField->rear = NULL;
    if (tempFields == NULL)
    {
    	tempFields = tempField;
    }
    else
    {
    	tempFields->rear->next = tempField;
    }
    tempFields->rear = tempField;
  }

  /*
  ** Return the vdata field list by assigning vdataFields with tempFields.
  */
  *vdataFields = tempFields;

}


/*****************************************************************************
*
*  FUNCTION STATEMENT:
*       makeClutterHdr(VDATA_FIELD_LINK **vdataFields)
*
*  DESCRIPTION:
*       This routine used to create 2A25 Clutter Header fields list in link 
*	list form.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       vdataFields		Fields of list of vdata
*
*  OUTPUT PARAMETERS:           None
*
*  CALLED BY:                   processHeader()
*
*  RETURN VALUES:               None
*
*  ROUTINES CALLED:             None
*
******************************************************************************/
void makeClutterHdr(VDATA_FIELD_LINK **vdataFields)
{
  char  l2afields[2][MAX_SIZE];
  int32 l2asize[2];
  int32	l2adim[2];
  int32 l2afieldtype[2];
  VDATA_FIELD_LINK      *tempField  = NULL;
  VDATA_FIELD_LINK      *tempFields = NULL;
  int   i;

  strcpy(l2afields[0], "mainlobeEdge");
  l2asize[0] = 1;
  l2adim[0] = 1;
  l2afieldtype[0] = DFNT_INT8;

  strcpy(l2afields[1], "sidelobeRange");
  l2asize[1] = 3;
  l2adim[1] = 3;
  l2afieldtype[1] = DFNT_INT8;

  /*
  ** Create a vdata field list.
  */
  for (i = 0; i < 2; ++i)
  {
    tempField = (VDATA_FIELD_LINK *) malloc(sizeof(VDATA_FIELD_LINK));
    if (tempField == NULL)
	/* Memory allocation failed in makeClutterHdr(). */
	TKreportError(E_TK_MEMFAILCLUTHDR);


    memset(tempField, '\0', sizeof(VDATA_FIELD_LINK));
    strcpy(tempField->name, l2afields[i]);
    tempField->type = l2afieldtype[i];
    tempField->fieldSize = l2asize[i];
    tempField->dim = l2adim[i];
    tempField->next = NULL;
    tempField->rear = NULL;
    if (tempFields == NULL)
    {
           tempFields = tempField;
    }
    else
    {
           tempFields->rear->next = tempField;
    }
    tempFields->rear = tempField;
  }

  *vdataFields = tempFields;

}


/*****************************************************************************
*
*  FUNCTION STATEMENT:
*       processHeader(VGROUP_LINK *vgroup, int productID)
*
*  DESCRIPTION:
*       This routine used to process L1B21, L1C21, and L2A25 header for
*	either reading from or writing to HDF.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       vgroup			A structure that stores the vgroup's info.
*	productID		Product identifier
*
*  OUTPUT PARAMETERS:		None
*
*  CALLED BY:                   TKopen()
*
*  RETURN VALUES:		None
*
*  ROUTINES CALLED:
*	makePRcalCoefHdr(), makeRayHdr(), makeClutterHdr()
*
******************************************************************************/
void processHeader(VGROUP_LINK *vgroup, int productID)
{

  switch(productID)
  {
	case TK_L1B_21:
	case TK_L1C_21:
	  /*
	  ** Initialize L1B21 or L1C21 PR Calibration Coefficient Header for 
	  ** reading or writing.
	  */
  	  vgroup->vdata = (VDATA_LINK *)malloc(sizeof(VDATA_LINK));
	  if (vgroup->vdata == NULL)
		/* Memory allocation failed in processHeader(). */
		TKreportError(E_TK_MEMFAILPROHDR);

	  makePRcalCoefHdr(&vgroup->vdata->vdataFields); 
	  strcpy(vgroup->vdata->name, "PR_CAL_COEF");

	  /*
	  ** Initialize L1B21 or L1C21 Ray Header for reading or writing.
	  */
  	  vgroup->vdata->next = (VDATA_LINK *)malloc(sizeof(VDATA_LINK));
	  if (vgroup->vdata->next == NULL)
		/* Memory allocation failed in processHeader(). */
                TKreportError(E_TK_MEMFAILPROHDR);

	  makeRayHdr(&vgroup->vdata->next->vdataFields);
	  strcpy(vgroup->vdata->next->name, "RAY_HEADER");
	  vgroup->vdata->next->next = NULL;
	  break;

	
	case TK_L2A_25:
	case TK_L2N_25:
	  /*
	  ** Initialize L2A25 Clutter Flag Header for reading or writing.
	  **/
  	  vgroup->vdata = (VDATA_LINK *)malloc(sizeof(VDATA_LINK));
          if (vgroup->vdata == NULL)
                /* Memory allocation failed in processHeader(). */
                TKreportError(E_TK_MEMFAILPROHDR);

 	  makeClutterHdr(&vgroup->vdata->vdataFields);
	  strcpy(vgroup->vdata->name, "CLUTTER_FLAG");
	  vgroup->vdata->next = NULL;
	  break;


	default:
	  break;
  }


}




/*********************************************************************************
*
*  FUNCTION STATEMENT:
*       allocMem(VDATA_FIELD_LINK **vdataFieldLink, int nRecords)
*
*  DESCRIPTION:
*       This function allocates memory to pack data and return it.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       vdataFieldLink          A link list pointing to the list of vdata fields.
*	nRecords		Number of records of vdata
*
*  OUTPUT PARAMETERS:		None
*
*  CALLED BY:
*	writeL1B21_L1C21Header(), readL1B21_L1C21Header(),
*       writeL2A25Header(), readL2A25Header()
*
*  ROUTINES CALLED:            
*	TKreportError() 
*
**********************************************************************************/

uint8 *allocMem(VDATA_FIELD_LINK **vdataFieldLink, int nRecords)
{
  VDATA_FIELD_LINK *tempVdataFieldLink = *vdataFieldLink;
  int totalSize = 0;
  int ttl;
  uint8 *databuf = NULL;

  while (tempVdataFieldLink != 0)
  {
         totalSize = totalSize + (tempVdataFieldLink->fieldSize *
                     (getSizeofType(tempVdataFieldLink->type) * tempVdataFieldLink->dim));
         tempVdataFieldLink = tempVdataFieldLink->next;
  }

  ttl = totalSize * nRecords;
  databuf = (uint8 *)malloc(ttl);

  if (databuf !=  NULL)
  {
      memset(databuf, '\0', ttl);
  }
  else
  {
      /* Memory allocation failed in allocMem */
      TKreportError(E_TK_MEMALLOCFAIL);
  }

  return(databuf);
}


/*****************************************************************************
*
*  FUNCTION STATEMENT:
*       makeNetHdr(VDATA_FIELD_LINK **vdataFields)
*
*  DESCRIPTION:
*       This routine used to initialize Network Descriptor fields list in link 
*	list form.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       vdataFields		Fields of list of vdata.
*
*  OUTPUT PARAMETERS:           None
*
*  CALLED BY:                   processNetworkHdr()
*
*  RETURN VALUES:               None
*
*  ROUTINES CALLED:             None
*
******************************************************************************/

void makeNetHdr(VDATA_FIELD_LINK **vdataFields)
{
  char  l2afields[3][MAX_SIZE];
  int32 l2asize[3];
  int32 l2adim[3];
  int32 l2afieldtype[3];
  VDATA_FIELD_LINK      *tempField  = NULL;
  VDATA_FIELD_LINK      *tempFields = NULL;
  int   i;

  strcpy(l2afields[0], "networkName");
  l2asize[0] = 21;
  l2adim[0] = 21;
  l2afieldtype[0] = DFNT_CHAR8;

  strcpy(l2afields[1], "nValidSensor");
  l2asize[1] = 1;
  l2adim[1] = 1;
  l2afieldtype[1] = DFNT_INT16;

  strcpy(l2afields[2], "networkID");
  l2asize[2] = 1;
  l2adim[2] = 1;
  l2afieldtype[2] = DFNT_INT8;

  /*
  ** Create a vdata field list.
  */
  for (i = 0; i < 3; ++i)
  {
    tempField = (VDATA_FIELD_LINK *) malloc(sizeof(VDATA_FIELD_LINK));
    if (tempField == NULL)
	/* Memory allocation failed in makeNetHdr(). */
	TKreportError(E_TK_MEMFAILNETHDR);

    memset(tempField, '\0', sizeof(VDATA_FIELD_LINK));
    strcpy(tempField->name, l2afields[i]);
    tempField->type = l2afieldtype[i];
    tempField->fieldSize = l2asize[i];
    tempField->dim = l2adim[i];
    tempField->next = NULL;
    tempField->rear = NULL;
    if (tempFields == NULL)
    {
           tempFields = tempField;
    }
    else
    {
           tempFields->rear->next = tempField;
    }
    tempFields->rear = tempField;
  }

  *vdataFields = tempFields;

}


/*****************************************************************************
*
*  FUNCTION STATEMENT:
*       makeGaugeDesc(VDATA_FIELD_LINK **vdataFields)
*
*  DESCRIPTION:
*       This routine used to initialize Gauge Descriptor fields list in link 
*       list form.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       vdataFields             Fields of list of vdata.
*
*  OUTPUT PARAMETERS:           None
*
*  CALLED BY:                   processNetworkHdr()
*
*  RETURN VALUES:               None
*
*  ROUTINES CALLED:             None
*
******************************************************************************/

void makeGaugeDesc(VDATA_FIELD_LINK **vdataFields)
{
  char  l2a56Gaugefields[13][MAX_SIZE];
  int   l2a56Gaugesize[13];
  int32 rayfieldtype[13];
  int32 raydim[13];
  VDATA_FIELD_LINK      *tempField  = NULL;
  VDATA_FIELD_LINK      *tempFields = NULL;
  int   i;

  strcpy(l2a56Gaugefields[0], "networkID");
  l2a56Gaugesize[0] = 1;
  raydim[0] = 1;
  rayfieldtype[0] = DFNT_INT8;

  strcpy(l2a56Gaugefields[1], "gaugeType");
  l2a56Gaugesize[1] = 21;
  raydim[1] = 21;
  rayfieldtype[1] = DFNT_CHAR8;

  strcpy(l2a56Gaugefields[2], "gaugeName");
  l2a56Gaugesize[2] = 21;
  raydim[2] = 21;
  rayfieldtype[2] = DFNT_CHAR8;

  strcpy(l2a56Gaugefields[3], "gaugeMakeModel");
  l2a56Gaugesize[3] = 21;
  raydim[3] = 21;
  rayfieldtype[3] = DFNT_CHAR8;

  strcpy(l2a56Gaugefields[4], "gaugeNumber");
  l2a56Gaugesize[4] = 1;
  raydim[4] = 1;
  rayfieldtype[4] = DFNT_INT32;

  strcpy(l2a56Gaugefields[5], "gaugeLat");
  l2a56Gaugesize[5] = 1;
  raydim[5] = 1;
  rayfieldtype[5] = DFNT_FLOAT32;

  strcpy(l2a56Gaugefields[6], "gaugeLong");
  l2a56Gaugesize[6] = 1;
  raydim[6] = 1;
  rayfieldtype[6] = DFNT_FLOAT32;

  strcpy(l2a56Gaugefields[7], "range");
  l2a56Gaugesize[7] = 1;
  raydim[7] = 1;
  rayfieldtype[7] = DFNT_FLOAT32;

  strcpy(l2a56Gaugefields[8], "azimuth");
  l2a56Gaugesize[8] = 1;
  raydim[8] = 1;
  rayfieldtype[8] = DFNT_FLOAT32;

  strcpy(l2a56Gaugefields[9], "xCoordinate");
  l2a56Gaugesize[9] = 1;
  raydim[9] = 1;
  rayfieldtype[9] = DFNT_FLOAT32;

  strcpy(l2a56Gaugefields[10], "yCoordinate");
  l2a56Gaugesize[10] = 1;
  raydim[10] = 1;
  rayfieldtype[10] = DFNT_FLOAT32;

  strcpy(l2a56Gaugefields[11], "resolution");
  l2a56Gaugesize[11] = 1;
  raydim[11] = 1;
  rayfieldtype[11] = DFNT_FLOAT32;

  strcpy(l2a56Gaugefields[12], "elevation");
  l2a56Gaugesize[12] = 1;
  raydim[12] = 1;
  rayfieldtype[12] = DFNT_FLOAT32;

  /*
  ** Create a vdata field list.
  */
  for (i = 0; i < 13; ++i)
  {
    tempField = (VDATA_FIELD_LINK *) malloc(sizeof(VDATA_FIELD_LINK));
    if (tempField == NULL)
	/* Memory allocation failed in makeGaugeDesc(). */
	TKreportError(E_TK_MEMFAILGAUGE);

    memset(tempField, '\0', sizeof(VDATA_FIELD_LINK));
    strcpy(tempField->name, l2a56Gaugefields[i]);
    tempField->type = rayfieldtype[i];
    tempField->fieldSize = l2a56Gaugesize[i];
    tempField->dim = raydim[i];
    tempField->next = NULL;
    tempField->rear = NULL;
    if (tempFields == NULL)
    {
        tempFields = tempField;
    }
    else
    {
        tempFields->rear->next = tempField;
    }
    tempFields->rear = tempField;
  }

  /*
  ** Return the vdata field list by assigning vdataFields with tempFields.
  */
  *vdataFields = tempFields;

}


/*****************************************************************************
*
*  FUNCTION STATEMENT:
*       makeDisdroDesc(VDATA_FIELD_LINK **vdataFields)
*
*  DESCRIPTION:
*       This routine used to initialize Disdrometer Descriptor fields list in 
*	link list form.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       vdataFields             Fields of list of vdata.
*
*  OUTPUT PARAMETERS:           None
*
*  CALLED BY:                   processNetworkHdr()
*
*  RETURN VALUES:               None
*
*  ROUTINES CALLED:             None
*
******************************************************************************/

void makeDisdroDesc(VDATA_FIELD_LINK **vdataFields)
{
  char  l2a57Disdrofields[10][MAX_SIZE];
  int   l2a57Disdrosize[10];
  int32 rayfieldtype[10];
  int32 raydim[10];
  VDATA_FIELD_LINK      *tempField  = NULL;
  VDATA_FIELD_LINK      *tempFields = NULL;
  int   i;

  strcpy(l2a57Disdrofields[0], "networkID");
  l2a57Disdrosize[0] = 1;
  raydim[0] = 1;
  rayfieldtype[0] = DFNT_INT8;

  strcpy(l2a57Disdrofields[1], "disdroName");
  l2a57Disdrosize[1] = 21;
  raydim[1] = 21;
  rayfieldtype[1] = DFNT_CHAR8;

  strcpy(l2a57Disdrofields[2], "disdroMakeModel");
  l2a57Disdrosize[2] = 21;
  raydim[2] = 21;
  rayfieldtype[2] = DFNT_CHAR8;

  strcpy(l2a57Disdrofields[3], "disdroNumber");
  l2a57Disdrosize[3] = 1;
  raydim[3] = 1;
  rayfieldtype[3] = DFNT_INT32;

  strcpy(l2a57Disdrofields[4], "disdroLat");
  l2a57Disdrosize[4] = 1;
  raydim[4] = 1;
  rayfieldtype[4] = DFNT_FLOAT32;

  strcpy(l2a57Disdrofields[5], "disdroLong");
  l2a57Disdrosize[5] = 1;
  raydim[5] = 1;
  rayfieldtype[5] = DFNT_FLOAT32;

  strcpy(l2a57Disdrofields[6], "range");
  l2a57Disdrosize[6] = 1;
  raydim[6] = 1;
  rayfieldtype[6] = DFNT_FLOAT32;

  strcpy(l2a57Disdrofields[7], "azimuth");
  l2a57Disdrosize[7] = 1;
  raydim[7] = 1;
  rayfieldtype[7] = DFNT_FLOAT32;

  strcpy(l2a57Disdrofields[8], "resolution");
  l2a57Disdrosize[8] = 1;
  raydim[8] = 1;
  rayfieldtype[8] = DFNT_FLOAT32;

  strcpy(l2a57Disdrofields[9], "elevation");
  l2a57Disdrosize[9] = 1;
  raydim[9] = 1;
  rayfieldtype[9] = DFNT_FLOAT32;


  /*
  ** Create a vdata field list.
  */
  for (i = 0; i < 10; ++i)
  {
    tempField = (VDATA_FIELD_LINK *) malloc(sizeof(VDATA_FIELD_LINK));
    if (tempField == NULL)
	/* Memory allocation failed in makeDisdroDesc(). */
	TKreportError(E_TK_MEMFAILDISDRO); 

    memset(tempField, '\0', sizeof(VDATA_FIELD_LINK));
    strcpy(tempField->name, l2a57Disdrofields[i]);
    tempField->type = rayfieldtype[i];
    tempField->fieldSize = l2a57Disdrosize[i];
    tempField->dim = raydim[i];
    tempField->next = NULL;
    tempField->rear = NULL;
    if (tempFields == NULL)
    {
        tempFields = tempField;
    }
    else
    {
        tempFields->rear->next = tempField;
    }
    tempFields->rear = tempField;
  }

  /*
  ** Return the vdata field list by assigning vdataFields with tempFields.
  */
  *vdataFields = tempFields;


}



/*****************************************************************************
*
*  FUNCTION STATEMENT:
*       processNetworkHdr(VGROUP_LINK *vgroup, int productID)
*
*  DESCRIPTION:
*       This routine used to process L2A56 and L2A57 Network Descriptor header 
*	for writing to HDF.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       vgroup                  A structure that stores the vgroup's info.
*       productID               Product identifier
*
*  OUTPUT PARAMETERS:           None
*
*  CALLED BY:                   TKopen()
*
*  RETURN VALUES:               None
*
*  ROUTINES CALLED:
*       makeNetHdr(), makeGaugeDesc(), makeDisdroDesc(), make2a56SDS(),
*	make2a57SDS()
*
******************************************************************************/

void processNetworkHdr(VGROUP_LINK *vgroup, int productID)
{
  vgroup->vdata = (VDATA_LINK *)malloc(sizeof(VDATA_LINK));
  if (vgroup->vdata == NULL)
	/* Memory allocation failed in processNetworkHdr(). */
	TKreportError(E_TK_MEMFAILNETWORK);

  makeNetHdr(&vgroup->vdata->vdataFields);
  strcpy(vgroup->vdata->name, "NETWORK_DESCRIPTOR");

  vgroup->vdata->next = (VDATA_LINK *)malloc(sizeof(VDATA_LINK));
  if (vgroup->vdata->next == NULL)
        /* Memory allocation failed in processNetworkHdr(). */
        TKreportError(E_TK_MEMFAILNETWORK);


  switch (productID)
  {
    case TK_L2A_56:
	makeGaugeDesc(&vgroup->vdata->next->vdataFields);
	strcpy(vgroup->vdata->next->name, "GAUGE_DESCRIPTOR");
	vgroup->vdata->next->next = NULL;

  	vgroup->sds = (SDS_LINK *)malloc(sizeof(SDS_LINK));
	if (vgroup->sds == NULL)
	    /* Memory allocation failed in processNetworkHdr(). */
            TKreportError(E_TK_MEMFAILNETWORK);

	make2a56SDS(&vgroup->sds);

	break;

    case TK_L2A_57:
	makeDisdroDesc(&vgroup->vdata->next->vdataFields);
        strcpy(vgroup->vdata->next->name, "DISDROMETER_DESCRIPTOR");
        vgroup->vdata->next->next = NULL;

  	vgroup->sds = (SDS_LINK *)malloc(sizeof(SDS_LINK));
        if (vgroup->sds == NULL)
            /* Memory allocation failed in processNetworkHdr(). */
            TKreportError(E_TK_MEMFAILNETWORK);

	make2a57SDS(&vgroup->sds);

	break;

  }
}




/*********************************************************************************
*
*  FUNCTION STATEMENT:
*       detachVdata(VGROUP_LINK *vgroup)
*
*  DESCRIPTION:
*       This routine recursively detaches vdata objects from the vgroup.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*	vgroup			List of vgroups.
*
*  OUTPUT PARAMETERS:		None
*
*  CALLED BY:			TKclose()
*
*  RETURN VALUES:		None
*
*  ROUTINES CALLED:		detachVdata()
*
**********************************************************************************/

void detachVdata(VGROUP_LINK *vgroup)
{
  VDATA_LINK	*vdata;


  if (vgroup != NULL)
  {
    vdata = vgroup->vdata;

    while (vdata != NULL)
    {
	VSdetach(vdata->vdataID);
	vdata = vdata->next;
    }

    /*
    **	Recursively detach vdata objects for sub-vgroups and other vgroups.
    */
    detachVdata(vgroup->subVgroup);
    detachVdata(vgroup->next);
  }
}



/*****************************************************************************
*
*  FUNCTION STATEMENT:
*       make2a56SDS(SDS_LINK **stack)
*
*  DESCRIPTION:
*       This routine used to initialize L2A56 sds fields in linked list form.
*
*  INPUT PARAMETERS:		None
*
*  OUTPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       stack			Fields of list of sds.
*
*  OUTPUT PARAMETERS:          
*	Parameter Name		Description
*	--------------		-----------
*	stack			Fields of list of sds. 
*
*  CALLED BY:                   processNetworkHdr()
*
*  RETURN VALUES:               None
*
*  ROUTINES CALLED:             None
*
******************************************************************************/

void make2a56SDS(SDS_LINK **stack)
{
  int	i,j;
  int32	dataType[3];
  char	name[3][MAX_SIZE];
  int32	range[3];
  int32	dim[3][MAX_RANK];

  SDS_LINK *tempStack = NULL;

  strcpy(name[0], "hour");
  dataType[0] = DFNT_INT8;
  range[0] = 1;
  dim[0][0] = 1440;

  strcpy(name[1], "minute");
  dataType[1] = DFNT_INT8;
  range[1] = 1;
  dim[1][0] = 1440;

  strcpy(name[2], "meanRainRate");
  dataType[2] = DFNT_INT16;
  range[2] = 1;
  dim[2][0] = 1440;

  for (i = 0; i < 3; ++i)
  {
      SDS_LINK *tempSNode = (SDS_LINK *)malloc(sizeof(SDS_LINK));
      if (tempSNode == NULL)
	/* Memory allocation failed in make2a56SDS(). */
	TKreportError(E_TK_MEMFAIL2A56SDS);

      memset(tempSNode, '\0', sizeof(SDS_LINK));
      strcpy(tempSNode->name, name[i]);
      tempSNode->type = dataType[i];

      tempSNode->rank = range[i] + 1;
      for (j = 1; j < (range[i] + 1); ++j)
      {
           tempSNode->dim[j] = dim[i][j-1];
           tempSNode->start[j] = 0;
           tempSNode->edge[j] = dim[i][j-1];
      }

      /*
      ** edge, dim, and start are initialized here for HDF routine
      ** functions such as SDnametoindex, SDcreate, SDwritedata, and
      ** SDreaddata.
      */
      tempSNode->edge[0] = 1;
      tempSNode->dim[0] = 1;
      tempSNode->start[0] = 0;
      tempSNode->next = NULL;
      tempSNode->rear = NULL;

      /*
      ** Insert the sds node at the end of stack.
      */
      if (tempStack == NULL)
      {
          tempStack = tempSNode;
      }
      else
      {
          tempStack->rear->next = tempSNode;
      }
      tempStack->rear = tempSNode;
  }
  *stack = tempStack;

}


/*****************************************************************************
*
*  FUNCTION STATEMENT:
*       make2a57SDS(SDS_LINK **stack)
*
*  DESCRIPTION:
*       This routine used to initialize L2A57 sds fields in linked list form.
*
*  INPUT PARAMETERS:
*	Parameter Name		Description
*	--------------		-----------
*	stack			Fields of list of sds.
*
*  OUTPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       stack                   Fields of list of sds.
*
*  OUTPUT PARAMETERS:           None
*
*  CALLED BY:                   processNetworkHdr()
*
*  RETURN VALUES:               None
*
*  ROUTINES CALLED:             None
*
******************************************************************************/
void make2a57SDS(SDS_LINK **stack)
{
  int   i,j;
  int32 dataType[3];
  char  name[3][MAX_SIZE];
  int32 range[3];
  int32 dim[3][MAX_RANK];

  SDS_LINK *tempStack = NULL;

  strcpy(name[0], "hour");
  dataType[0] = DFNT_INT8;
  range[0] = 1;
  dim[0][0] = 1440;

  strcpy(name[1], "minute");
  dataType[1] = DFNT_INT8;
  range[1] = 1;
  dim[1][0] = 1440;

  strcpy(name[2], "nConcentration");
  dataType[2] = DFNT_INT16;
  range[2] = 2;
  dim[2][0] = 1440;
  dim[2][1] = 20;

  for (i = 0; i < 3; ++i)
  {
      SDS_LINK *tempSNode = (SDS_LINK *)malloc(sizeof(SDS_LINK));
      if (tempSNode == NULL)
	/* Memory allocation failed in make2a57SDS(). */
	TKreportError(E_TK_MEMFAIL2A57SDS);

      memset(tempSNode, '\0', sizeof(SDS_LINK));
      strcpy(tempSNode->name, name[i]);
      tempSNode->type = dataType[i];

      tempSNode->rank = range[i] + 1;
      for (j = 1; j < (range[i] + 1); ++j)
      {
           tempSNode->dim[j] = dim[i][j-1];
           tempSNode->start[j] = 0;
           tempSNode->edge[j] = dim[i][j-1];
      }

      /*
      ** edge, dim, and start are initialized here for HDF routine
      ** functions such as SDnametoindex, SDcreate, SDwritedata, and
      ** SDreaddata.
      */
      tempSNode->edge[0] = 1;
      tempSNode->dim[0] = 1;
      tempSNode->start[0] = 0;
      tempSNode->next = NULL;
      tempSNode->rear = NULL;

      /*
      ** Insert the sds node at the end of stack.
      */
      if (tempStack == NULL)
      {
          tempStack = tempSNode;
      }
      else
      {
          tempStack->rear->next = tempSNode;
      }
      tempStack->rear = tempSNode;
  }
  *stack = tempStack;

}


/*****************************************************************************
*
*  FUNCTION STATEMENT:
*       setFlag(ELEMENT *metadata)
*
*  DESCRIPTION:
*       This routine initializes boolean flag for metadata to TK_FALSE.  This flag
*	is used to check whether the metadata has been updated or not.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*	metadata		ELEMENT structure to store metadata name, value, 
*				and booleanFlag.
*
*  OUTPUT PARAMETERS:	
*	Parameter Name		Description
*	--------------		-----------
*	metadata		ELEMENT structure to store metadata name, value,
*				and booleanFlag.	
*
*  CALLED BY:                   initMetadata()
*
*  RETURN VALUES:               None
*
*  ROUTINES CALLED:		None
*
******************************************************************************/

void setFlag (ELEMENT *metadata, int value)
{
  metadata->booleanFlag = value;

}



/*****************************************************************************
*
*  FUNCTION STATEMENT:
*       createMetadataPVL(IO_HANDLE *sGranuleHandle)
*
*  DESCRIPTION:
*       This routine writes metadata into the HDF file if the filemode is
*       write.  It also deallocates memory.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       sGranuleHandle          A structure containing information about the 
*				file from which data can be obtained.  It also
*                               contains metadata information.
*
*  OUTPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       sGranuleHandle          Now the memory allocated for its field metadata
*                               is deallocated.
*
*  CALLED BY:                   finalizeMetadata()
*
*  RETURN VALUES:               None.
*
*  ROUTINES CALLED:
*       create_pvl              Creates a pvl string
*
*       TKreportWarning         This routine reports warning errors to console
*                               and continues processing.
*
*       write_pvl_to_hdffile    Stores the pvl string as a global attribute in
*                               an HDF file
*
*       metaFree                Frees memory allocated by mdcpy
*
******************************************************************************/

void createMetadataPVL(IO_HANDLE *sGranuleHandle)
{

  char	*metadataStr;

  metadataStr = (char *) malloc (TK_MAX_SZ_ATTRIBUTE * sizeof(metadataStr));

  if (metadataStr == NULL)
  {
    /* Memory allocation failed in createMetadataPVL. */
    TKreportError(E_TK_MEMFAILMETAPVL);
  }

  strcpy ( metadataStr, "" );
  /* 
  ** Write core metadata to attribute string 
  */

  if (!sGranuleHandle->realTimeFlag)
  {
  /*
   * BEGIN non-real time
   */

#ifdef REAL_TIME_ENV
         sGranuleHandle->metadata.coreMetadata.orbitNumber.value="99999";
#endif

  create_pvl(metadataStr, 
	     sGranuleHandle->metadata.coreMetadata.orbitNumber.name,
	     sGranuleHandle->metadata.coreMetadata.orbitNumber.value);
  /*
   * END non-real time
   */
  }

  create_pvl(metadataStr, 
	     sGranuleHandle->metadata.coreMetadata.beginDate.name, 
	     sGranuleHandle->metadata.coreMetadata.beginDate.value);


  create_pvl(metadataStr, 
	     sGranuleHandle->metadata.coreMetadata.beginTime.name, 
	     sGranuleHandle->metadata.coreMetadata.beginTime.value);

  create_pvl(metadataStr, 
	     sGranuleHandle->metadata.coreMetadata.endDate.name, 
	     sGranuleHandle->metadata.coreMetadata.endDate.value);

  create_pvl(metadataStr, 
	     sGranuleHandle->metadata.coreMetadata.endTime.name, 
	     sGranuleHandle->metadata.coreMetadata.endTime.value);

  create_pvl(metadataStr, 
	     sGranuleHandle->metadata.coreMetadata.granuleID.name, 
	     sGranuleHandle->metadata.coreMetadata.granuleID.value);

  if (!sGranuleHandle->realTimeFlag)
  {
  /*
   * BEGIN non-real time
   */

  create_pvl(metadataStr, 
	     sGranuleHandle->metadata.coreMetadata.dataID.name, 
	     sGranuleHandle->metadata.coreMetadata.dataID.value);

  /* use the file size obtained from the system call to size() */
  create_pvl(metadataStr, 
	     sGranuleHandle->metadata.coreMetadata.fileSize.name, 
	     sGranuleHandle->fsize);

  /*
   * END non-real time
   */
  }
  create_pvl(metadataStr, 
	     sGranuleHandle->metadata.coreMetadata.lonMaxLat.name, 
	     sGranuleHandle->metadata.coreMetadata.lonMaxLat.value);

  if (!sGranuleHandle->realTimeFlag)
  {
  /*
   * BEGIN non-real time
   */
  create_pvl(metadataStr, 
	     sGranuleHandle->metadata.coreMetadata.spatCovType.name, 
	     sGranuleHandle->metadata.coreMetadata.spatCovType.value);

  create_pvl(metadataStr, 
	     sGranuleHandle->metadata.coreMetadata.ellipsoidName.name, 
	     sGranuleHandle->metadata.coreMetadata.ellipsoidName.value);

  create_pvl(metadataStr, 
	     sGranuleHandle->metadata.coreMetadata.earthEqRad.name, 
	     sGranuleHandle->metadata.coreMetadata.earthEqRad.value);

  create_pvl(metadataStr, 
	     sGranuleHandle->metadata.coreMetadata.flatteningRatio.name, 
	     sGranuleHandle->metadata.coreMetadata.flatteningRatio.value);

  create_pvl(metadataStr, 
	     sGranuleHandle->metadata.coreMetadata.orbitModelName.name, 
	     sGranuleHandle->metadata.coreMetadata.orbitModelName.value);

  create_pvl(metadataStr, 
	     sGranuleHandle->metadata.coreMetadata.kepSemiMajorAxis.name, 
	     sGranuleHandle->metadata.coreMetadata.kepSemiMajorAxis.value);


  create_pvl(metadataStr, 
	     sGranuleHandle->metadata.coreMetadata.kepMeanAnomaly.name, 
	     sGranuleHandle->metadata.coreMetadata.kepMeanAnomaly.value);


  create_pvl(metadataStr, 
	     sGranuleHandle->metadata.coreMetadata.kepRightAscenNode.name, 
	     sGranuleHandle->metadata.coreMetadata.kepRightAscenNode.value);


  create_pvl(metadataStr, 
	     sGranuleHandle->metadata.coreMetadata.kepArgOfPerigee.name, 
	     sGranuleHandle->metadata.coreMetadata.kepArgOfPerigee.value);


  create_pvl(metadataStr, 
	     sGranuleHandle->metadata.coreMetadata.kepEccentricity.name, 
	     sGranuleHandle->metadata.coreMetadata.kepEccentricity.value);


  create_pvl(metadataStr, 
	     sGranuleHandle->metadata.coreMetadata.kepInclination.name, 
	     sGranuleHandle->metadata.coreMetadata.kepInclination.value);

  create_pvl(metadataStr, 
	     sGranuleHandle->metadata.coreMetadata.kepEpochTime.name, 
	     sGranuleHandle->metadata.coreMetadata.kepEpochTime.value);

  create_pvl(metadataStr, 
	     sGranuleHandle->metadata.coreMetadata.kepEpochDate.name, 
	     sGranuleHandle->metadata.coreMetadata.kepEpochDate.value);

  create_pvl(metadataStr, 
	     sGranuleHandle->metadata.coreMetadata.kepEpochMillisec.name, 
	     sGranuleHandle->metadata.coreMetadata.kepEpochMillisec.value);

  create_pvl(metadataStr, 
	     sGranuleHandle->metadata.coreMetadata.westBoundCoord.name, 
	     sGranuleHandle->metadata.coreMetadata.westBoundCoord.value);

  create_pvl(metadataStr, 
	     sGranuleHandle->metadata.coreMetadata.eastBoundCoord.name, 
	     sGranuleHandle->metadata.coreMetadata.eastBoundCoord.value);

  create_pvl(metadataStr, 
	     sGranuleHandle->metadata.coreMetadata.northBoundCoord.name, 
	     sGranuleHandle->metadata.coreMetadata.northBoundCoord.value);

  create_pvl(metadataStr, 
	     sGranuleHandle->metadata.coreMetadata.southBoundCoord.name, 
	     sGranuleHandle->metadata.coreMetadata.southBoundCoord.value);

  create_pvl(metadataStr, 
	     sGranuleHandle->metadata.coreMetadata.centerPointLat.name, 
	     sGranuleHandle->metadata.coreMetadata.centerPointLat.value);

  create_pvl(metadataStr, 
	     sGranuleHandle->metadata.coreMetadata.centerPointLon.name, 
	     sGranuleHandle->metadata.coreMetadata.centerPointLon.value);

  create_pvl(metadataStr, 
	     sGranuleHandle->metadata.coreMetadata.radius.name, 
	     sGranuleHandle->metadata.coreMetadata.radius.value);

  create_pvl(metadataStr, 
	     sGranuleHandle->metadata.coreMetadata.latitudeRes.name, 
	     sGranuleHandle->metadata.coreMetadata.latitudeRes.value);

  create_pvl(metadataStr, 
	     sGranuleHandle->metadata.coreMetadata.longitudeRes.name, 
	     sGranuleHandle->metadata.coreMetadata.longitudeRes.value);

  create_pvl(metadataStr, 
	     sGranuleHandle->metadata.coreMetadata.geoCoordUnits.name, 
	     sGranuleHandle->metadata.coreMetadata.geoCoordUnits.value);

  create_pvl(metadataStr, 
	     sGranuleHandle->metadata.coreMetadata.temporRngType.name, 
	     sGranuleHandle->metadata.coreMetadata.temporRngType.value);

  create_pvl(metadataStr, 
	     sGranuleHandle->metadata.coreMetadata.qaParamName.name, 
	     sGranuleHandle->metadata.coreMetadata.qaParamName.value);

  create_pvl(metadataStr, 
	     sGranuleHandle->metadata.coreMetadata.qaParamValue.name, 
	     sGranuleHandle->metadata.coreMetadata.qaParamValue.value);

  create_pvl(metadataStr, 
	     sGranuleHandle->metadata.coreMetadata.reproStat.name, 
	     sGranuleHandle->metadata.coreMetadata.reproStat.value);

  create_pvl(metadataStr, 
	     sGranuleHandle->metadata.coreMetadata.browseName.name, 
	     sGranuleHandle->metadata.coreMetadata.browseName.value);

  create_pvl(metadataStr, 
	     sGranuleHandle->metadata.coreMetadata.contact.name, 
	     sGranuleHandle->metadata.coreMetadata.contact.value);

  create_pvl(metadataStr, 
	     sGranuleHandle->metadata.coreMetadata.meanMotion.name, 
	     sGranuleHandle->metadata.coreMetadata.meanMotion.value);

  /*
   * END non-real time
   */
  }

  create_pvl(metadataStr, 
	     sGranuleHandle->metadata.coreMetadata.orbitAdjFlag.name, 
	     sGranuleHandle->metadata.coreMetadata.orbitAdjFlag.value);

  create_pvl(metadataStr, 
	     sGranuleHandle->metadata.coreMetadata.attitudeMode.name, 
	     sGranuleHandle->metadata.coreMetadata.attitudeMode.value);

  if (!sGranuleHandle->realTimeFlag)
  {
  /*
   * BEGIN non-real time
   */

  create_pvl(metadataStr, 
	     sGranuleHandle->metadata.coreMetadata.beginSolarBeta.name, 
	     sGranuleHandle->metadata.coreMetadata.beginSolarBeta.value);

  create_pvl(metadataStr, 
	     sGranuleHandle->metadata.coreMetadata.endSolarBeta.name, 
	     sGranuleHandle->metadata.coreMetadata.endSolarBeta.value);

  create_pvl(metadataStr, 
	     sGranuleHandle->metadata.coreMetadata.sensorAlgn.name, 
	     sGranuleHandle->metadata.coreMetadata.sensorAlgn.value);

  create_pvl(metadataStr, 
	     sGranuleHandle->metadata.coreMetadata.sensorAlgnChanOff.name, 
	     sGranuleHandle->metadata.coreMetadata.sensorAlgnChanOff.value);

  create_pvl(metadataStr, 
	     sGranuleHandle->metadata.coreMetadata.scanPathModel.name, 
	     sGranuleHandle->metadata.coreMetadata.scanPathModel.value);

  create_pvl(metadataStr, 
	     sGranuleHandle->metadata.coreMetadata.scanPathParam.name, 
	     sGranuleHandle->metadata.coreMetadata.scanPathParam.value);

  create_pvl(metadataStr, 
	     sGranuleHandle->metadata.coreMetadata.ephemFilename.name, 
	     sGranuleHandle->metadata.coreMetadata.ephemFilename.value);

  /*
   * END non-real time
   */
  }

  /* 
  ** Write the attribute string containing Core metadata to the HDF File 
  */
  if (writepvl_to_hdffile(sGranuleHandle,"CoreMetadata.0",metadataStr) != TK_SUCCESS)
    TKreportWarning(W_TK_WRITEPVL);



  /*
   * Write Product Specific metadata to attribute string
   */
  metadataStr[0] = '\0';


  if (!sGranuleHandle->realTimeFlag)
  {
  /*
   * BEGIN non-real time
   */

  create_pvl(metadataStr, 
	     sGranuleHandle->metadata.psMetadata.dataGap.name, 
	     sGranuleHandle->metadata.psMetadata.dataGap.value);

  create_pvl(metadataStr, 
	     sGranuleHandle->metadata.psMetadata.numDataGap.name, 
	     sGranuleHandle->metadata.psMetadata.numDataGap.value);

   }
  /*
   * END non-real time
   */
  create_pvl(metadataStr, 
	     sGranuleHandle->metadata.psMetadata.algorithmVersion.name, 
	     sGranuleHandle->metadata.psMetadata.algorithmVersion.value);

  if (!sGranuleHandle->realTimeFlag)
  {
  /*
   * BEGIN non-real time
   */
  create_pvl(metadataStr, 
	     sGranuleHandle->metadata.psMetadata.productVersion.name, 
	     sGranuleHandle->metadata.psMetadata.productVersion.value);
   }
  /*
   * END non-real time
   */

   create_pvl(metadataStr, 
	     sGranuleHandle->metadata.psMetadata.toolkitVersion.name, 
	     sGranuleHandle->metadata.psMetadata.toolkitVersion.value);

  if (!sGranuleHandle->realTimeFlag)
  {
  /*
   * BEGIN non-real time
   */
  create_pvl(metadataStr, 
	     sGranuleHandle->metadata.psMetadata.calCoeffVersion.name, 
	     sGranuleHandle->metadata.psMetadata.calCoeffVersion.value);

  create_pvl(metadataStr, 
	     sGranuleHandle->metadata.psMetadata.missingData.name, 
	     sGranuleHandle->metadata.psMetadata.missingData.value);

  create_pvl(metadataStr, 
	     sGranuleHandle->metadata.psMetadata.percentBadMissPix.name, 
	     sGranuleHandle->metadata.psMetadata.percentBadMissPix.value);

  create_pvl(metadataStr, 
	     sGranuleHandle->metadata.psMetadata.maxValidChannel.name, 
	     sGranuleHandle->metadata.psMetadata.maxValidChannel.value);

  create_pvl(metadataStr, 
	     sGranuleHandle->metadata.psMetadata.minValidChannel.name, 
	     sGranuleHandle->metadata.psMetadata.minValidChannel.value);

  create_pvl(metadataStr, 
	     sGranuleHandle->metadata.psMetadata.minMaxUnits.name, 
	     sGranuleHandle->metadata.psMetadata.minMaxUnits.value);

   }
  /*
   * END non-real time
   */

  create_pvl(metadataStr, 
	     sGranuleHandle->metadata.psMetadata.orbitSize.name, 
	     sGranuleHandle->metadata.psMetadata.orbitSize.value);


  if (!sGranuleHandle->realTimeFlag)
  {
  /*
   * BEGIN non-real time
   */

  create_pvl(metadataStr, 
	     sGranuleHandle->metadata.psMetadata.radarWavelength.name, 
	     sGranuleHandle->metadata.psMetadata.radarWavelength.value);

  create_pvl(metadataStr, 
	     sGranuleHandle->metadata.psMetadata.minRefThreshold.name, 
	     sGranuleHandle->metadata.psMetadata.minRefThreshold.value);
  /*
   * END non-real time
   */
  }

  create_pvl(metadataStr, 
	     sGranuleHandle->metadata.psMetadata.algorithmID.name, 
	     sGranuleHandle->metadata.psMetadata.algorithmID.value);

  if (!sGranuleHandle->realTimeFlag)
  {
  /*
   * BEGIN non-real time
   */

  create_pvl(metadataStr, 
	     sGranuleHandle->metadata.psMetadata.dataAccuracy.name, 
	     sGranuleHandle->metadata.psMetadata.dataAccuracy.value);

  create_pvl(metadataStr, 
	     sGranuleHandle->metadata.psMetadata.inputID.name, 
	     sGranuleHandle->metadata.psMetadata.inputID.value);

  create_pvl(metadataStr, 
	     sGranuleHandle->metadata.psMetadata.dateGenInputFile.name, 
	     sGranuleHandle->metadata.psMetadata.dateGenInputFile.value);

  create_pvl(metadataStr, 
	     sGranuleHandle->metadata.psMetadata.dataCenterSrc.name, 
	     sGranuleHandle->metadata.psMetadata.dataCenterSrc.value);

  /* Use the generation date obtained from the system call to time() */
  create_pvl(metadataStr, 
	     sGranuleHandle->metadata.psMetadata.generationDate.name, 
	     sGranuleHandle->genDate);

  create_pvl(metadataStr, 
	     sGranuleHandle->metadata.psMetadata.dayNight.name, 
	     sGranuleHandle->metadata.psMetadata.dayNight.value);

  create_pvl(metadataStr, 
	     sGranuleHandle->metadata.psMetadata.solarGains.name, 
	     sGranuleHandle->metadata.psMetadata.solarGains.value);

  create_pvl(metadataStr,
             sGranuleHandle->metadata.psMetadata.tmiRolloverCoef.name,
             sGranuleHandle->metadata.psMetadata.tmiRolloverCoef.value);

  create_pvl(metadataStr, 
	     sGranuleHandle->metadata.psMetadata.firstScanUTCdate.name, 
	     sGranuleHandle->metadata.psMetadata.firstScanUTCdate.value);

  create_pvl(metadataStr, 
	     sGranuleHandle->metadata.psMetadata.firstScanUTCtime.name, 
	     sGranuleHandle->metadata.psMetadata.firstScanUTCtime.value);

  create_pvl(metadataStr, 
	     sGranuleHandle->metadata.psMetadata.firstScanUTCmsec.name, 
	     sGranuleHandle->metadata.psMetadata.firstScanUTCmsec.value);

  create_pvl(metadataStr, 
	     sGranuleHandle->metadata.psMetadata.firstScanSCsec.name, 
	     sGranuleHandle->metadata.psMetadata.firstScanSCsec.value);

  create_pvl(metadataStr, 
	     sGranuleHandle->metadata.psMetadata.firstScanSCsubsec.name, 
	     sGranuleHandle->metadata.psMetadata.firstScanSCsubsec.value);

  create_pvl(metadataStr, 
	     sGranuleHandle->metadata.psMetadata.lastScanUTCdate.name, 
	     sGranuleHandle->metadata.psMetadata.lastScanUTCdate.value);

  create_pvl(metadataStr, 
	     sGranuleHandle->metadata.psMetadata.lastScanUTCtime.name, 
	     sGranuleHandle->metadata.psMetadata.lastScanUTCtime.value);

  create_pvl(metadataStr, 
	     sGranuleHandle->metadata.psMetadata.lastScanUTCmsec.name, 
	     sGranuleHandle->metadata.psMetadata.lastScanUTCmsec.value);

  create_pvl(metadataStr, 
	     sGranuleHandle->metadata.psMetadata.lastScanSCsec.name, 
	     sGranuleHandle->metadata.psMetadata.lastScanSCsec.value);

  create_pvl(metadataStr, 
	     sGranuleHandle->metadata.psMetadata.lastScanSCsubsec.name, 
	     sGranuleHandle->metadata.psMetadata.lastScanSCsubsec.value);

  create_pvl(metadataStr, 
	     sGranuleHandle->metadata.psMetadata.utcfSec.name, 
	     sGranuleHandle->metadata.psMetadata.utcfSec.value);

  create_pvl(metadataStr, 
	     sGranuleHandle->metadata.psMetadata.utcfSubsec.name, 
	     sGranuleHandle->metadata.psMetadata.utcfSubsec.value);

  create_pvl(metadataStr, 
	     sGranuleHandle->metadata.psMetadata.utcfFlag.name, 
	     sGranuleHandle->metadata.psMetadata.utcfFlag.value);

  create_pvl(metadataStr, 
	     sGranuleHandle->metadata.psMetadata.leapSecsFlag.name, 
	     sGranuleHandle->metadata.psMetadata.leapSecsFlag.value);

  create_pvl(metadataStr, 
	     sGranuleHandle->metadata.psMetadata.radarName.name, 
	     sGranuleHandle->metadata.psMetadata.radarName.value);

  create_pvl(metadataStr, 
	     sGranuleHandle->metadata.psMetadata.radarCity.name, 
	     sGranuleHandle->metadata.psMetadata.radarCity.value);

  create_pvl(metadataStr, 
	     sGranuleHandle->metadata.psMetadata.radarState.name, 
	     sGranuleHandle->metadata.psMetadata.radarState.value);

  create_pvl(metadataStr, 
	     sGranuleHandle->metadata.psMetadata.radarCountry.name, 
	     sGranuleHandle->metadata.psMetadata.radarCountry.value);

  create_pvl(metadataStr, 
	     sGranuleHandle->metadata.psMetadata.numVOS.name, 
	     sGranuleHandle->metadata.psMetadata.numVOS.value);

  create_pvl(metadataStr, 
	     sGranuleHandle->metadata.psMetadata.radarOriginLat.name, 
	     sGranuleHandle->metadata.psMetadata.radarOriginLat.value);

  create_pvl(metadataStr, 
	     sGranuleHandle->metadata.psMetadata.radarOriginLon.name, 
	     sGranuleHandle->metadata.psMetadata.radarOriginLon.value);

  create_pvl(metadataStr, 
	     sGranuleHandle->metadata.psMetadata.radarOriginAlt.name, 
	     sGranuleHandle->metadata.psMetadata.radarOriginAlt.value);

  create_pvl(metadataStr, 
	     sGranuleHandle->metadata.psMetadata.radarSpacingX.name, 
	     sGranuleHandle->metadata.psMetadata.radarSpacingX.value);

  create_pvl(metadataStr, 
	     sGranuleHandle->metadata.psMetadata.radarSpacingY.name, 
	     sGranuleHandle->metadata.psMetadata.radarSpacingY.value);

  create_pvl(metadataStr, 
	     sGranuleHandle->metadata.psMetadata.radarSpacingZ.name, 
	     sGranuleHandle->metadata.psMetadata.radarSpacingZ.value);

  create_pvl(metadataStr, 
	     sGranuleHandle->metadata.psMetadata.radarGridSizeX.name, 
	     sGranuleHandle->metadata.psMetadata.radarGridSizeX.value);

  create_pvl(metadataStr, 
	     sGranuleHandle->metadata.psMetadata.radarGridSizeY.name, 
	     sGranuleHandle->metadata.psMetadata.radarGridSizeY.value);

  create_pvl(metadataStr, 
	     sGranuleHandle->metadata.psMetadata.radarGridSizeZ.name, 
	     sGranuleHandle->metadata.psMetadata.radarGridSizeZ.value);

  create_pvl(metadataStr,
             sGranuleHandle->metadata.psMetadata.dzcal.name,
             sGranuleHandle->metadata.psMetadata.dzcal.value);

  create_pvl(metadataStr,
             sGranuleHandle->metadata.psMetadata.gvL1Cscale.name,
             sGranuleHandle->metadata.psMetadata.gvL1Cscale.value);

  create_pvl(metadataStr,
             sGranuleHandle->metadata.psMetadata.alpha.name,
             sGranuleHandle->metadata.psMetadata.alpha.value);

  create_pvl(metadataStr,
             sGranuleHandle->metadata.psMetadata.runtimeOption.name,
             sGranuleHandle->metadata.psMetadata.runtimeOption.value);

  create_pvl(metadataStr,
             sGranuleHandle->metadata.psMetadata.anomalyFlag.name,
             sGranuleHandle->metadata.psMetadata.anomalyFlag.value);

  create_pvl(metadataStr,
	     sGranuleHandle->metadata.psMetadata.softwareVersion.name,
	     sGranuleHandle->metadata.psMetadata.softwareVersion.value);
  
  create_pvl(metadataStr,
	     sGranuleHandle->metadata.psMetadata.databaseVersion.name,
	     sGranuleHandle->metadata.psMetadata.databaseVersion.value);
  
  create_pvl(metadataStr,
	     sGranuleHandle->metadata.psMetadata.totalQualityCode.name,
	     sGranuleHandle->metadata.psMetadata.totalQualityCode.value);
  
  create_pvl(metadataStr,
	     sGranuleHandle->metadata.psMetadata.lonOnEquator.name,
	     sGranuleHandle->metadata.psMetadata.lonOnEquator.value);
  
  create_pvl(metadataStr,
	     sGranuleHandle->metadata.psMetadata.UTCDateOnEquator.name,
	     sGranuleHandle->metadata.psMetadata.UTCDateOnEquator.value);
  
  create_pvl(metadataStr,
	     sGranuleHandle->metadata.psMetadata.UTCTimeOnEquator.name,
	     sGranuleHandle->metadata.psMetadata.UTCTimeOnEquator.value);
  
  create_pvl(metadataStr,
	     sGranuleHandle->metadata.psMetadata.millisecOnEquator.name,
	     sGranuleHandle->metadata.psMetadata.millisecOnEquator.value);
  
  create_pvl(metadataStr,
	     sGranuleHandle->metadata.psMetadata.centerScanUTCDate.name,
	     sGranuleHandle->metadata.psMetadata.centerScanUTCDate.value);
  
  create_pvl(metadataStr,
	     sGranuleHandle->metadata.psMetadata.centerScanUTCTime.name,
	     sGranuleHandle->metadata.psMetadata.centerScanUTCTime.value);
  
  create_pvl(metadataStr,
	     sGranuleHandle->metadata.psMetadata.centerScanUTCMillisec.name,
	     sGranuleHandle->metadata.psMetadata.centerScanUTCMillisec.value);
  
  create_pvl(metadataStr,
	     sGranuleHandle->metadata.psMetadata.firstScanLat.name,
	     sGranuleHandle->metadata.psMetadata.firstScanLat.value);
  
  create_pvl(metadataStr,
	     sGranuleHandle->metadata.psMetadata.firstScanLon.name,
	     sGranuleHandle->metadata.psMetadata.firstScanLon.value);
  
  create_pvl(metadataStr,
	     sGranuleHandle->metadata.psMetadata.lastScanLat.name,
	     sGranuleHandle->metadata.psMetadata.lastScanLat.value);
  
  create_pvl(metadataStr,
	     sGranuleHandle->metadata.psMetadata.lastScanLon.name,
	     sGranuleHandle->metadata.psMetadata.lastScanLon.value);
  
  create_pvl(metadataStr,
	     sGranuleHandle->metadata.psMetadata.numOfRainScans.name,
	     sGranuleHandle->metadata.psMetadata.numOfRainScans.value);

  /*
   * END non-real time
   */
  }
  
  /* 
  ** Write the PS metadata attribute to the HDF file 
  */
  if( writepvl_to_hdffile(sGranuleHandle, "ArchiveMetadata.0", metadataStr) != TK_SUCCESS)
    TKreportWarning(W_TK_WRITEPVL);


  /* free memory */
  free (metadataStr);
}

/*****************************************************************************
*
*  FUNCTION STATEMENT:
*       setSDSdimensionNames( VGROUP_LINK *vgroup, FILE* in_file , char* datatype)
*
*  DESCRIPTION:
*       This routine will name the appropriate sds dimensions by reading 
*       the SDS_DIMENSION_NAMES file (in the $TSDISTK/config/ directory). 
*       It will always return TK_SUCCESS because the sds dimension naming 
*       is only a recent feature which is not critical to data integrity.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       sGranuleHandle          the hdf v group
*       in_file                 the SDS_DIMENSION_NAMES file handle 
*       datype                  the data type 
*
*  OUTPUT PARAMETERS:  None
*
*  CALLED BY: setSDSattributes                    
*
*  RETURN VALUES:               TK_SUCCESS only
*
*  ROUTINES CALLED: None
*
******************************************************************************/
int setSDSdimensionNames ( VGROUP_LINK *vgroup, FILE* in_file , char* datatype)
{

  int status = TK_SUCCESS;
  SDS_LINK *sds;
  FILE *curr_fptr;
  char line[256];
  char* ch;
  char* ch2;
  int i, dim_id ;
  char dim_name[99];

  if ( feof(in_file) )
      return TK_SUCCESS;

  if ( vgroup != NULL )
  {
      sds = vgroup->sds;
      while ( sds != NULL )
      {
          curr_fptr=in_file;

          do { 
              fgets(line, sizeof(line), curr_fptr);
          }
          while ((!feof(curr_fptr))&&(!strstr(line,datatype)));

          while ((!feof(curr_fptr))&&(!strstr(line,sds->name)))
              fgets(line, sizeof(line), curr_fptr);

          if (!feof(curr_fptr))
              fgets(line, sizeof(line), curr_fptr);

          if ( feof(curr_fptr) ) 
              return TK_SUCCESS;

          ch=line;
          for (i = 0; i < sds->rank; i++) 
          {
              ch2=dim_name;
              dim_id = SDgetdimid(sds->sdsID,i);
              if (i==0)
              { 
                  sprintf ( dim_name, "scan");
              }
              else
              {
                  while ((isspace(*ch))&&(*ch!='\0'))
                      ch++; 
                  while ((!isspace(*ch))&&(*ch!='\0'))
                      *ch2++=*ch++; 
                  *ch2='\0';
              }
              if (strlen(dim_name)>0)
                  status = SDsetdimname(dim_id, dim_name );
          }
          sds = sds->next;
      }
      setSDSdimensionNames ( vgroup->subVgroup, in_file, datatype);
      setSDSdimensionNames ( vgroup->next, in_file, datatype );
    }
  return TK_SUCCESS;
}

/*****************************************************************************
*
*  FUNCTION STATEMENT:
*       setSDSattributes(IO_HANDLE *sGranuleHandle)
*
*  DESCRIPTION:
*       This routine will call the appropriate routine to write the scaling
*       factors based on the product id.  If an error status is returned,
*       it will report it to the calling routine.
*       ***NOTE*** Now it also calls setSDSdimensionNames -P. Kolbe 12/98 !!!!
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       sGranuleHandle          A structure containing information about the
*                               file from which data can be obtained.  It also
*                               contains metadata information.
*
*  OUTPUT PARAMETERS:  None
*
*  CALLED BY:                   TKopen()
*
*  RETURN VALUES:               TK_SUCCESS or the appropriate error code.
*
*  ROUTINES CALLED:
*       setSDSdimensionNames    Set the sds dimension names for all data types.
*       setScalingFactors_1B01  Write the scaling factors for VIRS 1B01.
*       setScalingFactors_1B11  Write the scaling factors for TMI 1B11.
*       setScalingFactors_2A12  Write the scaling factors for TMI 2A12.
*       setScalingFactors_3A11  Write the scaling factors for TMI 3A11.
*       setScalingFactors_1B21  Write the scaling factors for PR 1B21.
*       setScalingFactors_2A21  Write the scaling factors for PR 2A21.
*       setScalingFactors_2A25  Write the scaling factors for PR 2A25.
*       setScalingFactors_2B31  Write the scaling factors for PR 2B31.
*       setScalingFactors_2A53  Write the scaling factors for GV 2A53.
*       setScalingFactors_2A55  Write the scaling factors for GV 2A55.
*       setScalingFactors_2A56  Write the scaling factors for GV 2A56.
*       setScalingFactors_3A53  Write the scaling factors for GV 3A53.
*       setScalingFactors_3A55  Write the scaling factors for GV 3A55.
*
******************************************************************************/

int setSDSattributes ( IO_HANDLE *sGranuleHandle )
{
  FILE *in_file;
  int status = TK_SUCCESS;
  char* path_info;
  char dimension_file[256];
  char* eof_ptr;

  path_info = getenv( "TSDISTK" );
  strcpy(dimension_file,path_info);
  strcat(dimension_file,"/config/SDS_DIMENSION_NAMES");
  in_file = fopen(dimension_file, "r");
  if (in_file != NULL)
  {
          status = setSDSdimensionNames(sGranuleHandle->cfgSTR->vGroup, 
                    in_file, sGranuleHandle->gIDtokens.datatype );
          fclose(in_file);
  }

  switch ( sGranuleHandle->productID )
    {
    case TK_L1B_01:
      status = setScalingFactors_1B01 ( sGranuleHandle->cfgSTR->vGroup );
      break;
    case TK_L1B_11:
    case TK_L1B_11_R1:
      status = setScalingFactors_1B11 ( sGranuleHandle->cfgSTR->vGroup, 
				       sGranuleHandle->realTimeFlag );
      break;
    case TK_L1B_21:
    case TK_L1C_21:
      status = setScalingFactors_1B21 ( sGranuleHandle->cfgSTR->vGroup );
      break;
    case TK_L2A_12:   
    case TK_L2A_12_R1:
      status = setScalingFactors_2A12 ( sGranuleHandle->cfgSTR->vGroup,
				       sGranuleHandle->realTimeFlag );
      break;
    case TK_L2A_21:
      status = setScalingFactors_2A21 ( sGranuleHandle->cfgSTR->vGroup );
      break;
    case TK_L2A_23_R1:
      status = setScalingFactors_2A23 ( sGranuleHandle->cfgSTR->vGroup,
				       sGranuleHandle->realTimeFlag );
      break;
    case TK_L2A_25:
    case TK_L2N_25:
    case TK_L2A_25_R1:
    case TK_L2A_25_R2:
      status = setScalingFactors_2A25 ( sGranuleHandle->cfgSTR->vGroup,
				       sGranuleHandle->realTimeFlag );
      break;
    case TK_L2A_53S:
    case TK_L2A_53TX:
    case TK_L2A_53FL:
      status = setScalingFactors_2A53 ( sGranuleHandle->cfgSTR->vGroup );
      break;
    case TK_L2A_55S:
    case TK_L2A_55TX:
    case TK_L2A_55FL:
      status = setScalingFactors_2A55 ( sGranuleHandle->cfgSTR->vGroup );
      break;
    case TK_L2A_56:
      status = setScalingFactors_2A56 ( sGranuleHandle->cfgSTR->vGroup );
      break;
    case TK_L2B_31:
      status = setScalingFactors_2B31 ( sGranuleHandle->cfgSTR->vGroup );
      break;
    case TK_L3A_11:
      status = setScalingFactors_3A11 ( sGranuleHandle->cfgSTR->vGroup );
      break;
    case TK_L3A_53S:
    case TK_L3A_53TX:
    case TK_L3A_53FL:
      status = setScalingFactors_3A53 ( sGranuleHandle->cfgSTR->vGroup );
      break;
    case TK_L3A_55:
      status = setScalingFactors_3A55 ( sGranuleHandle->cfgSTR->vGroup );
      break;

    }
  return status;
}

/*****************************************************************************
*
*  FUNCTION STATEMENT:
*       createSwathRadarVdata(IO_HANDLE *sGranuleHandle, int32 *vID,
*                             char *vdata_name, int32 metadata_length )
*
*  DESCRIPTION:
*       This routine will create the Vdata structure that will contains 
*       either the SwathStructure Metadata or the Planetary Grid Metadata.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       sGranuleHandle          A structure containing information about the 
*				file from which data can be obtained.  It also
*                               contains metadata information.
*
*       vdata_name              The name of the vdata.
*
*       metadata_length         The length of the metadata string.
*
*  OUTPUT PARAMETERS:           
*       Parameter Name          Description
*       --------------          -----------
*       vID                     The id of the newly created Vdata.
*
*  CALLED BY:                   createRadar3A25Metadata, createSwathRadarMetadata
*
*  RETURN VALUES:               None.
*
*  ROUTINES CALLED:
*          TKreportError
*          VSattach
*          VSfdefine
*          VSsetfields
*          VSsetname
*          VSsetclass
*
******************************************************************************/
void createSwathRadarVdata ( IO_HANDLE *sGranuleHandle, int32 *vID,
			    char *vdata_name, int32 metadata_length )
{

  *vID = VSattach ( sGranuleHandle->cfgSTR->hOpenID, -1, "w" );
  if ( *vID == TK_HDF_FAIL )
    TKreportError ( E_TK_FAILVSATTOUT );

  VSfdefine ( *vID, "VALUES", DFNT_CHAR8, metadata_length );
  VSsetfields ( *vID, "VALUES" );

  VSsetname ( *vID, vdata_name );
  VSsetclass ( *vID, "Attr0.0" );

  return;
}

/*****************************************************************************
*
*  FUNCTION STATEMENT:
*              void attachSwathRadarVdata ( VGROUP_LINK *vgroup, int32 vID, 
*			                    char *vgroup_name )
*
*  DESCRIPTION:
*       This routine will attach the Structure Metadata Vdata to
*       the appropriate VGroup.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       VGROUP_LINK             A linked-list of vgroups, SDSs and Vdatas that 
*                               are in the HDF file.
*
*       vID                     The id of the newly created Vdata.
*
*       vgroup_name             The name of the vgroup to which the vdata should
*                               be attached.
*
*  OUTPUT PARAMETERS:           None.
*
*  CALLED BY:                   createRadar3A25Metadata, createSwathRadarMetadata
*
*  RETURN VALUES:               None.
*
*  ROUTINES CALLED:
*          TKreportError
*          Vinsert
*          attachSwathRadarVdata
*
******************************************************************************/
void attachSwathRadarVdata ( VGROUP_LINK *vgroup, int32 vID, 
			    char *vgroup_name )
{
  int status;
  VGROUP_LINK *tvgroup = vgroup;

  if ( tvgroup == NULL )
    return;

  while ( tvgroup != NULL )
    {
      if ( !strcmp ( tvgroup->vGroupID->name, vgroup_name ) )
	{
	  status = Vinsert ( tvgroup->vGroupID->vGroupID, vID );
	  if ( status == TK_HDF_FAIL )
	    TKreportError ( E_TK_FAILVINCLS );
	}
      tvgroup = tvgroup->next;
    }
  attachSwathRadarVdata ( vgroup->subVgroup, vID, vgroup_name );
  
  return;
}

/*****************************************************************************
*
*  FUNCTION STATEMENT:
*       void createRadar3A25Metadata ( IO_HANDLE *sGranuleHandle )
*
*  DESCRIPTION:
*       This routine will create the Vdata structure that will contains 
*       either the Planetary Grid Metadata for the algorithm 3A25.
*       It will also attach the Vdata to the appropriate place in the
*       HDF file.  
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       sGranuleHandle          A structure containing information about the 
*				file from which data can be obtained.  It also
*                               contains metadata information.
*
*  OUTPUT PARAMETERS:           None
*
*  CALLED BY:                   createRadar3A25Metadata, createSwathRadarMetadata
*
*  RETURN VALUES:               None.
*
*  ROUTINES CALLED:
*          TKreportError
*          createL3StructureMetadata
*          createSwathRadarVdata
*          attachSwathRadarVdata
*          VSwrite
*
******************************************************************************/
void createRadar3A25Metadata ( IO_HANDLE *sGranuleHandle )
{
  int32 vID, metadata_length, status;
  char vdata_name[40], vgroup_name[40];
  char *StructureMetadata;

  strcpy ( vdata_name, "GridStructure" );
  strcpy ( vgroup_name, "PlanetaryGrid1" );
  StructureMetadata = createL3StructureMetadata ( sGranuleHandle );
  metadata_length = strlen ( StructureMetadata );
  createSwathRadarVdata ( sGranuleHandle, &vID, vdata_name, 
			 metadata_length );

  status = VSwrite ( vID, ( char * ) StructureMetadata, 1, FULL_INTERLACE );
  if ( status == TK_HDF_FAIL )
/*
    TKreportError ( E_TK_FAILWRTVWRT );
*/
    TKreportErrorParm ( E_TK_FAILWRTVWRT,vgroup_name );
  attachSwathRadarVdata ( sGranuleHandle->cfgSTR->vGroup, vID, 
			 vgroup_name );
  free ( StructureMetadata );
  VSdetach ( vID );

  strcpy ( vgroup_name, "PlanetaryGrid2" );
  StructureMetadata = createPlanetaryGrid2Metadata ( sGranuleHandle );
  metadata_length = strlen ( StructureMetadata );
  createSwathRadarVdata ( sGranuleHandle, &vID, vdata_name,
			 metadata_length);
  status = VSwrite ( vID, ( char * ) StructureMetadata, 1, FULL_INTERLACE );
  if ( status == TK_HDF_FAIL )
/*
    TKreportError ( E_TK_FAILWRTVWRT );
*/
    TKreportErrorParm ( E_TK_FAILWRTVWRT,vgroup_name );
  attachSwathRadarVdata ( sGranuleHandle->cfgSTR->vGroup, vID, 
			 vgroup_name );
  free ( StructureMetadata );
  VSdetach ( vID );
  return;
}

/*****************************************************************************
*
*  FUNCTION STATEMENT:
*       void createStructurePVL ( char *outStr, char *parameter, char *value )
*
*  DESCRIPTION:
*       This routine will build the metadata string that must be written
*       to the Structure metadata Vdata.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       parameter               The metadata name.
*
*       value                   The metadata value for the corresponding name.
*
*  OUTPUT PARAMETERS:           
*       Parameter Name          Description
*       --------------          -----------
*       outStr                  The concatenated metadata string.
*
*  CALLED BY:                   createL3StructureMetadata
*
*  RETURN VALUES:               None.
*
*  ROUTINES CALLED:
*          TKreportError
*
******************************************************************************/
void createStructurePVL ( char *outStr, char *parameter, char *value )
{
  char *tempStr = NULL;
  int len;
  
  len = 70 + ( 2*strlen(parameter) ) + strlen ( value );

  tempStr = ( char * ) calloc ( len, sizeof ( char ) );
  if ( tempStr == NULL )
    TKreportError ( E_TK_MEMFAILCREATEPVL );

  sprintf ( tempStr, "OBJECT=%s;\n\tValue=%s;\nEND_OBJECT=%s;\n\n",
	   parameter, value, parameter );
  if ( outStr == NULL )
    outStr = tempStr;
  else
    outStr = strcat ( outStr, tempStr );
  free ( tempStr );

  return;
}

/*****************************************************************************
*
*  FUNCTION STATEMENT:
*       char *createSwathStructureMetadata ( IO_HANDLE *sGranuleHandle )
*
*  DESCRIPTION:
*       This routine will build the swath structure metadata string.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       sGranuleHandle          A structure containing information about the 
*				file from which data can be obtained.  It also
*                               contains metadata information.
*
*  OUTPUT PARAMETERS:           None.
*
*  CALLED BY:                   createSwathRadarMetadata
*
*  RETURN VALUES:               The completed structure metadata string.
*
*  ROUTINES CALLED:
*          TKreportError
*
******************************************************************************/
char *createSwathStructureMetadata ( IO_HANDLE *sGranuleHandle )
{
  char *StrMet;
  char *tempStr;

  StrMet = ( char * ) calloc ( TK_MAX_SZ_ATTRIBUTE, sizeof ( char ) );
  if ( StrMet == NULL )
    TKreportError ( E_TK_BADMEMALLCAT );

  tempStr = ( char * ) calloc ( TK_MAX_SZ_ATTRIBUTE, sizeof ( char ) );
  if ( tempStr == NULL )
    TKreportError ( E_TK_BADMEMALLCAT );
  
  sprintf ( StrMet, "OBJECT=%s\n\tName=%s\n\tSize=%s\nEND_OBJECT\n\n",
	   "Dimension", "nscan", 
	   sGranuleHandle->metadata.psMetadata.orbitSize.value );
  
  switch ( instType ( sGranuleHandle->productID ) )
    {
    case TK_TMI:
      sprintf ( tempStr, "OBJECT=%s;\n\tName=%s;\n\tSize=%s;\nEND_OBJECT;\n\n",
	   "Dimension", "npixel_low", "104" );
      strcat ( StrMet, tempStr );
      sprintf ( tempStr, "OBJECT=%s;\n\tName=%s;\n\tSize=%s;\nEND_OBJECT;\n\n",
	   "Dimension", "npixel_high", "208" );
      strcat ( StrMet, tempStr );
      sprintf ( tempStr, "OBJECT=%s;\n\tName=%s;\n\tSize=%s;\nEND_OBJECT;\n\n",
	       "Dimension", "nchannel_low", "7" );
      strcat ( StrMet, tempStr );
      sprintf ( tempStr, "OBJECT=%s;\n\tName=%s;\n\tSize=%s;\nEND_OBJECT;\n\n",
	   "Dimension", "nchannel_high", "2" );
      strcat ( StrMet, tempStr );
      sprintf ( tempStr, "OBJECT=%s;\n\tName=%s;\n\tDataType=%s;\n\t\
Dimension=%s;\n\tDimension=%s;\nEND_OBJECT;\n\n",
	   "GeoParameter", "Latitude", "float32", "nscan", "npixel_high" );
      strcat ( StrMet, tempStr );
      sprintf ( tempStr, "OBJECT=%s;\n\tName=%s;\n\tDataType=%s;\n\t\
Dimension=%s;\n\tDimension=%s;\nEND_OBJECT;\n\n",
	   "GeoParameter", "Latitude", "float32", "nscan", "npixel_high" );
      strcat ( StrMet, tempStr );
      sprintf ( tempStr, "OBJECT=%s;\n\tName=%s;\n\tDataType=%s;\n\t\
Dimension=%s;\nEND_OBJECT;\n\n",
	   "GeoParameter", "Time", "float64", "nscan" );
      strcat ( StrMet, tempStr );
      sprintf ( tempStr, "OBJECT=%s;\n\tName=%s;\n\tDataType=%s;\n\t\
Dimension=%s;\n\tDimension=%s;\n\tDimension=%s;\nEND_OBJECT;\n\n",
	       "DataParameter", "Low Resolution Channel", "int16", "nscan", 
	       "npixel_low", "nchannel_low" );
      strcat ( StrMet, tempStr );
      sprintf ( tempStr, "OBJECT=%s;\n\tName=%s;\n\tDataType=%s;\n\t\
Dimension=%s;\n\tDimension=%s;\n\tDimension=%s;\nEND_OBJECT;\n\n",
	       "DataParameter", "High Resolution Channel", "int16", "nscan", 
	       "npixel_high", "nchannel_high" );
      strcat ( StrMet, tempStr );
      sprintf ( tempStr, "OBJECT=%s;\n\tName=%s;\n\tDataDimension=%s;\n\t\
GeoDimension=%s;\n\tOffset=%s;\n\tIncrement=%s;\nEND_OBJECT;\n\n",
	       "DimensionMap", "npixel_low", "npixel_high", "0", "-2" );
      strcat ( StrMet, tempStr );
      break;
    case TK_VIRS:
      sprintf ( tempStr, "OBJECT=%s;\n\tName=%s;\n\tSize=%s;\nEND_OBJECT;\n\n",
	   "Dimension", "npixel", "261" );
      strcat ( StrMet, tempStr );
      sprintf ( tempStr, "OBJECT=%s;\n\tName=%s;\n\tSize=%s;\nEND_OBJECT;\n\n",
	       "Dimension", "nchannel", "5" );
      strcat ( StrMet, tempStr );
      sprintf ( tempStr, "OBJECT=%s;\n\tName=%s;\n\tDataType=%s;\n\t\
Dimension=%s;\n\tDimension=%s;\nEND_OBJECT;\n\n",
	   "GeoParameter", "Latitude", "float32", "nscan", "npixel" );
      strcat ( StrMet, tempStr );
      sprintf ( tempStr, "OBJECT=%s;\n\tName=%s;\n\tDataType=%s;\n\t\
Dimension=%s;\nEND_OBJECT;\n\n",
	   "GeoParameter", "Time", "float64", "nscan" );
      strcat ( StrMet, tempStr );
      sprintf ( tempStr, "OBJECT=%s;\n\tName=%s;\n\tDataType=%s;\n\t\
Dimension=%s;\n\tDimension=%s;\n\tDimension=%s;\nEND_OBJECT;\n\n",
	       "DataParameter", "Channel", "int16", "nscan", 
	       "npixel", "nchannel" );
      strcat ( StrMet, tempStr );
      break;
    case TK_PR:
      sprintf ( tempStr, "OBJECT=%s;\n\tName=%s;\n\tSize=%s;\nEND_OBJECT;\n\n",
	   "Dimension", "npixel", "49" );
      strcat ( StrMet, tempStr );
      sprintf ( tempStr, "OBJECT=%s;\n\tName=%s;\n\tSize=%s;\nEND_OBJECT;\n\n",
	       "Dimension", "nchannel", "1" );
      strcat ( StrMet, tempStr );
      sprintf ( tempStr, "OBJECT=%s;\n\tName=%s;\n\tDataType=%s;\n\t\
Dimension=%s;\n\tDimension=%s;\nEND_OBJECT;\n\n",
	   "GeoParameter", "Latitude", "float32", "nscan", "npixel" );
      strcat ( StrMet, tempStr );
      sprintf ( tempStr, "OBJECT=%s;\n\tName=%s;\n\tDataType=%s;\n\t\
Dimension=%s;\nEND_OBJECT;\n\n",
	   "GeoParameter", "Time", "float64", "nscan" );
      strcat ( StrMet, tempStr );
      sprintf ( tempStr, "OBJECT=%s;\n\tName=%s;\n\tDataType=%s;\n\t\
Dimension=%s;\n\tDimension=%s;\n\tDimension=%s;\nEND_OBJECT;\n\n",
	       "DataParameter", "Channel", "int16", "nscan", 
	       "npixel", "nchannel" );
      strcat ( StrMet, tempStr );
      break;
    }

  strcat ( StrMet, "END;\n" );
  free ( tempStr );
  return StrMet;
}

/*****************************************************************************
*
*  FUNCTION STATEMENT:
*       char *createL3StructureMetadata ( IO_HANDLE *sGranuleHandle )
*
*  DESCRIPTION:
*       This routine will create the planetary metadata structure string.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       sGranuleHandle          A structure containing information about the 
*				file from which data can be obtained.  It also
*                               contains metadata information.
*
*  OUTPUT PARAMETERS:           None.
*
*  CALLED BY:                   createRadar3A25Metadata, createSwathRadarMetadata
*
*  RETURN VALUES:               The completed planetary metadata string.
*
*  ROUTINES CALLED:
*          TKreportError
*
******************************************************************************/
char *createL3StructureMetadata ( IO_HANDLE *sGranuleHandle )
{
  char *StrMet = NULL;

  StrMet = ( char * ) calloc ( TK_MAX_SZ_ATTRIBUTE, sizeof ( char ) );
  if ( StrMet == NULL )
    TKreportError ( E_TK_BADMEMALLCAT );
  
  createStructurePVL ( StrMet, "bin_meth", "ARITHMEAN" );
  createStructurePVL ( StrMet, "registration", "CENTER" );
  createStructurePVL ( StrMet, sGranuleHandle->metadata.coreMetadata.latitudeRes.name,
	      sGranuleHandle->metadata.coreMetadata.latitudeRes.value );
  createStructurePVL ( StrMet, sGranuleHandle->metadata.coreMetadata.longitudeRes.name,
	      sGranuleHandle->metadata.coreMetadata.longitudeRes.value );
  createStructurePVL ( StrMet, sGranuleHandle->metadata.coreMetadata.northBoundCoord.name,
	      sGranuleHandle->metadata.coreMetadata.northBoundCoord.value );
  createStructurePVL ( StrMet, sGranuleHandle->metadata.coreMetadata.southBoundCoord.name,
	      sGranuleHandle->metadata.coreMetadata.southBoundCoord.value );
  createStructurePVL ( StrMet, sGranuleHandle->metadata.coreMetadata.eastBoundCoord.name,
	      sGranuleHandle->metadata.coreMetadata.eastBoundCoord.value );
  createStructurePVL ( StrMet, sGranuleHandle->metadata.coreMetadata.westBoundCoord.name,
	      sGranuleHandle->metadata.coreMetadata.westBoundCoord.value );
  switch ( sGranuleHandle->productID )
    {
    case TK_L3A_46:
      createStructurePVL ( StrMet, "Origin", "Northwest" );
      break;
    default:
      createStructurePVL ( StrMet, "Origin", "Southwest" );
      break;
    }
      
  strcat ( StrMet, "END;\n" );
  return StrMet;
}

/*****************************************************************************
*
*  FUNCTION STATEMENT:
*       char *createPlanetaryGrid2Metadata ( IO_HANDLE *sGranuleHandle )
*
*  DESCRIPTION:
*       This routine will create the planetary metadata structure string.
*       NOTE:  This routine is different that createL3StructureMetadata since
*       the values for the high-resolution grid (lat/lon boundary values) 
*       had to be hard coded and not pulled from metadata.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       sGranuleHandle          A structure containing information about the 
*				file from which data can be obtained.  It also
*                               contains metadata information.
*
*  OUTPUT PARAMETERS:           None.
*
*  CALLED BY:                   createRadar3A25Metadata
*
*  RETURN VALUES:               The completed planetary metadata string.
*
*  ROUTINES CALLED:
*          TKreportError
*
******************************************************************************/
char *createPlanetaryGrid2Metadata ( IO_HANDLE *sGranuleHandle )
{
  char *StrMet = NULL;

  StrMet = ( char * ) calloc ( TK_MAX_SZ_ATTRIBUTE, sizeof ( char ) );
  if ( StrMet == NULL )
    TKreportError ( E_TK_BADMEMALLCAT );
  
  createStructurePVL ( StrMet, "bin_meth", "ARITHMEAN" );
  createStructurePVL ( StrMet, "registration", "CENTER" );
  createStructurePVL ( StrMet, sGranuleHandle->metadata.coreMetadata.latitudeRes.name,
	      sGranuleHandle->metadata.coreMetadata.latitudeRes.value );
  createStructurePVL ( StrMet, sGranuleHandle->metadata.coreMetadata.longitudeRes.name,
	      sGranuleHandle->metadata.coreMetadata.longitudeRes.value );
  createStructurePVL ( StrMet, sGranuleHandle->metadata.coreMetadata.northBoundCoord.name,
	      "37" );
  createStructurePVL ( StrMet, sGranuleHandle->metadata.coreMetadata.southBoundCoord.name,
	      "-37" );
  createStructurePVL ( StrMet, sGranuleHandle->metadata.coreMetadata.eastBoundCoord.name,
	      sGranuleHandle->metadata.coreMetadata.eastBoundCoord.value );
  createStructurePVL ( StrMet, sGranuleHandle->metadata.coreMetadata.westBoundCoord.name,
	      sGranuleHandle->metadata.coreMetadata.westBoundCoord.value );
  createStructurePVL ( StrMet, "Origin", "Southwest" );
  
  strcat ( StrMet, "END;\n" );
  return StrMet;
}

/*****************************************************************************
*
*  FUNCTION STATEMENT:
*       void createSwathRadarMetadata ( IO_HANDLE *sGranuleHandle )
*
*  DESCRIPTION:
*       This routine will create the Vdata structure that will contains 
*       either the SwathStructure Metadata or the Planetary Grid Metadata.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       sGranuleHandle          A structure containing information about the 
*				file from which data can be obtained.  It also
*                               contains metadata information.
*
*  OUTPUT PARAMETERS:           None.
*
*  CALLED BY:                   writeSwathRadarMetadata
*
*  RETURN VALUES:               None.
*
*  ROUTINES CALLED:
*          TKreportError
*          createSwathStructureMetadata
*          createL3StructureMetadata
*          createSwathRadarVdata
*          attachSwathRadarVdata
*
******************************************************************************/
void createSwathRadarMetadata ( IO_HANDLE *sGranuleHandle )
{
  int32 vID, metadata_length, status;
  char vdata_name[40], vgroup_name[40];
  char *StructureMetadata;

  switch ( prodType ( sGranuleHandle->productID ) )
    {
    case TK_L1:
    case TK_L2:
      strcpy ( vdata_name, "SwathStructure" );
      strcpy ( vgroup_name, "SwathData" );
      StructureMetadata = createSwathStructureMetadata ( sGranuleHandle );
      metadata_length = strlen ( StructureMetadata );
      break;
    case TK_L3:
      strcpy ( vdata_name, "GridStructure" );
      strcpy ( vgroup_name, "PlanetaryGrid" );
      StructureMetadata = createL3StructureMetadata ( sGranuleHandle );
      metadata_length = strlen ( StructureMetadata );
      break;
    }

  createSwathRadarVdata ( sGranuleHandle, &vID, vdata_name, 
			 metadata_length );
  status = VSwrite ( vID, ( char * ) StructureMetadata, 1, FULL_INTERLACE );
  if ( status == TK_HDF_FAIL )
/*
    TKreportError ( E_TK_FAILWRTVWRT );
*/
    TKreportErrorParm ( E_TK_FAILWRTVWRT,vdata_name );

  attachSwathRadarVdata ( sGranuleHandle->cfgSTR->vGroup, vID, 
			 vgroup_name );
  free ( StructureMetadata );
  VSdetach ( vID );
  return;
}

/*****************************************************************************
*
*  FUNCTION STATEMENT:
*       void writeSwathRadarMetadata ( IO_HANDLE *sGranuleHandle )
*
*  DESCRIPTION:
*       This routine is the main interface routine for writing the swath
*       and planetary structure metadata.
*
*  INPUT PARAMETERS:
*       Parameter Name          Description
*       --------------          -----------
*       sGranuleHandle          A structure containing information about the 
*				file from which data can be obtained.  It also
*                               contains metadata information.
*
*  OUTPUT PARAMETERS:           None.
*
*  CALLED BY:                   TKclose
*
*  RETURN VALUES:               None.
*
*  ROUTINES CALLED:
*          instType
*          createRadar3A25Metadata
*          createSwathRadarMetadata
*
******************************************************************************/
void writeSwathRadarMetadata ( IO_HANDLE *sGranuleHandle )
{

  switch ( instType ( sGranuleHandle->productID ) )
    {
    case TK_GV:
      break;
    default:
      switch ( sGranuleHandle->productID )
	{
	case TK_L3A_25:
	  createRadar3A25Metadata ( sGranuleHandle ); 
	  break;
	default:
	  createSwathRadarMetadata ( sGranuleHandle );
	  break;
	}
      break;
    }
  return;
}


void checkTKenvironment ( )
{
  FILE *in_file;
  char* path_info;
  char tk_env_check_file[256];
  char tk_version[50];
  int i;
  static number_of_calls=0;

  number_of_calls++;

  path_info = getenv( "TSDISTK" );         /*let's help the TSU user know when his
                                             toolkit environment variable is set wrong */
  strcpy(tk_env_check_file,path_info);
  strcat(tk_env_check_file,"/config/TSDISTK_CHECK");
  if (number_of_calls<2)
  {
      in_file = fopen(tk_env_check_file, "r");
      if (in_file != NULL)
      {
          memset(tk_version, '\0', 50);
          for (i = 0; i< 10; i++)
              tk_version[i] = getc(in_file);
          if (versionCompare(tk_version,TK_VERSION_NUMBER) != 0)
              printf("\nGeneral warning: Please check your TSDISTK environment variable. It might be set incorrectly. The verson of the Toolkit you are compiling/linking to is %s and the full path of your TSDISTK environment variable is %s \n\n", TK_VERSION_NUMBER, path_info);
          fclose(in_file);
      }
      else
      {
          printf("\nGeneral warning: Please check your TSDISTK environment variable. It might be set incorrectly. The verson of the Toolkit you are compiling/linking to is %s and the full path of your TSDISTK environment variable is %s \n\n", TK_VERSION_NUMBER, path_info);
      }
  }
}




